#
#  Build the test model for the Engineering Features plug-in.
#
from abaqus import *
from abaqusConstants import *
from caeModules import *
#
# Create native2Native model.
#
Mdb()
mdb.models.changeKey(fromName='Model-1', toName='native2Native')
mdl1 = mdb.models['native2Native']
s = mdl1.ConstrainedSketch(name='__profile__', sheetSize=50.0)
g = s.geometry
s.setPrimaryObject(option=STANDALONE)
s.ConstructionLine(point1=(0.0, -25.0), point2=(0.0, 25.0))
s.FixedConstraint(entity=g.findAt((0.0, 0.0)))
s.Line(point1=(4.0, 0.0), point2=(4.0, 2.0))
s.VerticalConstraint(entity=g.findAt((4.0, 1.0)), addUndoState=False)
p = mdl1.Part(name='innerShell', dimensionality=THREE_D, type=DEFORMABLE_BODY)
p.BaseShellRevolve(sketch=s, angle=45.0, flipRevolveDirection=OFF)
s.unsetPrimaryObject()
del s
#
s1 = mdl1.ConstrainedSketch(name='__profile__', sheetSize=50.0)
g = s1.geometry
s1.setPrimaryObject(option=STANDALONE)
s1.ConstructionLine(point1=(0.0, -25.0), point2=(0.0, 25.0))
s1.FixedConstraint(entity=g.findAt((0.0, 0.0)))
s1.Line(point1=(8.0, 0.0), point2=(8.0, 2.0))
s1.VerticalConstraint(entity=g.findAt((8.0, 1.0)), addUndoState=False)
p = mdl1.Part(name='outerShell', dimensionality=THREE_D, type=DEFORMABLE_BODY)
p.BaseShellRevolve(sketch=s1, angle=45.0, flipRevolveDirection=OFF)
s1.unsetPrimaryObject()
del s1
#
a = mdl1.rootAssembly
a.DatumCsysByDefault(CARTESIAN)
p = mdl1.parts['innerShell']
a.Instance(name='innerShell-1', part=p, dependent=ON)
p = mdl1.parts['outerShell']
a.Instance(name='outerShell-1', part=p, dependent=ON)
#
f1 = a.instances['innerShell-1'].faces
faces1 = f1.findAt(((2.93216, 0.666667, 2.720742), ))
a.Set(faces=faces1, name='inner')
f1 = a.instances['outerShell-1'].faces
faces1 = f1.findAt(((5.86432, 0.666667, 5.441484), ))
a.Set(faces=faces1, name='outer')
#
f = p.faces
pickedRegions = f.findAt(((5.86432, 0.666667, 5.441484), ))
p.setMeshControls(regions=pickedRegions, elemShape=QUAD, technique=STRUCTURED)
elemType1 = mesh.ElemType(elemCode=S4R, elemLibrary=STANDARD, 
                          secondOrderAccuracy=OFF, hourglassControl=DEFAULT)
elemType2 = mesh.ElemType(elemCode=S3, elemLibrary=STANDARD)
faces = f.findAt(((5.86432, 0.666667, 5.441484), ))
pickedRegions =(faces, )
p.setElementType(regions=pickedRegions, elemTypes=(elemType1, elemType2))
e = p.edges
pickedEdges = e.findAt(((5.656854, 1.5, 5.656854), ), ((8.0, 0.5, 0.0), ))
p.seedEdgeByNumber(edges=pickedEdges, number=3, constraint=FINER)
pickedEdges = e.findAt(((7.846282, 0.0, 1.560723), ),
                       ((7.846282, 2.0, 1.560723), ))
p.seedEdgeByNumber(edges=pickedEdges, number=6, constraint=FINER)
p.generateMesh()
#
p = mdl1.parts['innerShell']
f = p.faces
pickedRegions = f.findAt(((2.93216, 0.666667, 2.720742), ))
p.setMeshControls(regions=pickedRegions, elemShape=QUAD, technique=STRUCTURED)
faces = f.findAt(((2.93216, 0.666667, 2.720742), ))
pickedRegions =(faces, )
p.setElementType(regions=pickedRegions, elemTypes=(elemType1, elemType2))
e = p.edges
pickedEdges = e.findAt(((2.828427, 1.5, 2.828427), ), ((4.0, 0.5, 0.0), ))
p.seedEdgeByNumber(edges=pickedEdges, number=3, constraint=FINER)
pickedEdges = e.findAt(((3.923141, 0.0, 0.780361), ),
                       ((3.923141, 2.0, 0.780361), ))
p.seedEdgeByNumber(edges=pickedEdges, number=6, constraint=FINER)
p.generateMesh()
a.regenerate()
#
# Create orphan2SameOrphan model.
#
mdl2 = mdb.Model(name='orphan2SameOrphan', objectToCopy=mdl1)
a = mdl2.rootAssembly
session.viewports['Viewport: 1'].setValues(displayedObject=a)
a1 = mdl2.rootAssembly
a1.InstanceFromBooleanMerge(name='Part-1', instances=(
    a1.instances['innerShell-1'], a1.instances['outerShell-1'], ), 
    mergeNodes=NONE, domain=MESH, originalInstances=SUPPRESS)
#
# Create native2SameNative model.
#
mdl3 = mdb.Model(name='native2SameNative', modelType=STANDARD_EXPLICIT)
a = mdl3.rootAssembly
s = mdl3.ConstrainedSketch(name='__profile__', sheetSize=50.0)
g = s.geometry
s.setPrimaryObject(option=STANDALONE)
s.Line(point1=(0.0, 0.0), point2=(4.0, 0.0))
s.HorizontalConstraint(entity=g.findAt((2.0, 0.0)), addUndoState=False)
s.Line(point1=(4.0, 2.0), point2=(0.0, 2.0))
s.HorizontalConstraint(entity=g.findAt((2.0, 2.0)), addUndoState=False)
s.ArcByCenterEnds(center=(4.0, 1.0), point1=(4.0, 0.0), point2=(4.0, 2.0), 
                  direction=COUNTERCLOCKWISE)
p = mdl3.Part(name='sideU', dimensionality=THREE_D, type=DEFORMABLE_BODY)
p = mdl3.parts['sideU']
p.BaseShellExtrude(sketch=s, depth=2.0)
s.unsetPrimaryObject()
del s
#
a.DatumCsysByDefault(CARTESIAN)
a.Instance(name='sideU-1', part=p, dependent=ON)
#
f = p.faces
pickedRegions = f.findAt(((1.333333, 2.0, 1.333333), ),
                         ((4.094768, 1.995499, 1.333333), ),
                         ((2.666667, 0.0, 1.333333), ))
p.setMeshControls(regions=pickedRegions, elemShape=QUAD, technique=STRUCTURED)
faces = f.findAt(((1.333333, 2.0, 1.333333), ),
                 ((4.094768, 1.995499, 1.333333), ),
                 ((2.666667, 0.0, 1.333333), ))
pickedRegions =(faces, )
p.setElementType(regions=pickedRegions, elemTypes=(elemType1, elemType2))
e = p.edges
pickedEdges = e.findAt(((4.0, 2.0, 0.5), ), ((0.0, 2.0, 0.5), ),
                       ((4.0, 0.0, 0.5), ), ((0.0, 0.0, 0.5), ))
p.seedEdgeByNumber(edges=pickedEdges, number=2, constraint=FINER)
pickedEdges = e.findAt(((1.0, 2.0, 2.0), ), ((3.0, 2.0, 0.0), ),
                       ((3.0, 0.0, 2.0), ), ((1.0, 0.0, 0.0), ))
p.seedEdgeByNumber(edges=pickedEdges, number=3, constraint=FINER)
pickedEdges = e.findAt(((4.707107, 1.707107, 2.0), ),
                       ((4.707107, 0.292893, 0.0), ))
p.seedEdgeByNumber(edges=pickedEdges, number=6, constraint=FINER)
p.generateMesh()
p.regenerate()
a = mdl3.rootAssembly
a.regenerate()
#
n1 = a.instances['sideU-1'].nodes
nodes1 = n1[4:8]+n1[19:20]+n1[25:30]+n1[37:39]
a.Set(nodes=nodes1, name='bottomNodes')
e1 = a.instances['sideU-1'].elements
elements1 = e1[0:6]
a.Set(elements=elements1, name='topElements')
n1 = a.instances['sideU-1'].nodes
nodes1 = n1[0:4]+n1[8:14]+n1[30:32]
a.Set(nodes=nodes1, name='topNodes')
a.regenerate()
#
# Create native2Orphan model.
#
mdl4 = mdb.Model(name='native2Orphan', objectToCopy=mdl1)
p = mdl4.parts['innerShell']
a1 = mdl4.rootAssembly
a1.makeIndependent(instances=(a1.instances['innerShell-1'], ))
a1.makeIndependent(instances=(a1.instances['outerShell-1'], ))
instOS4 = a1.instances['outerShell-1']
a1.InstanceFromBooleanMerge(name='outerShell-mesh-1', instances=(instOS4, ),
                            mergeNodes=NONE, domain=MESH, 
                            originalInstances=SUPPRESS)
#
a = mdl4.rootAssembly
instOSM4 = a.instances['outerShell-mesh-1-1']
e1 = instOSM4.elements
elements1 = e1[0:18]
a.Set(elements=elements1, name='outerElements')
#
# Create native2NativeIndep model.
#
mdl5 = mdb.Model(name='native2NativeIndep', objectToCopy=mdl1)
a1 = mdl5.rootAssembly
a1.makeIndependent(instances=(a1.instances['innerShell-1'], ))
a1.makeIndependent(instances=(a1.instances['outerShell-1'], ))
a1.regenerate()
#
#  Save DB, and exit.
#
mdb.saveAs(pathName='rtestModel')
sys.exit()
