"""
Sets node and element start number offsets for flat input files.
"""
from abaqusGui import *
from abaqusConstants import *
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
# Class definition
#
class NEOffsetDB(AFXDataDialog):
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def __init__(self, form):
        AFXDataDialog.__init__(self, form, 'Offset Nodes\Elements', \
            self.OK|self.CANCEL, DIALOG_ACTIONS_SEPARATOR,0,0,200,0)
        va = AFXVerticalAligner(self)
        self.od = AFXTextField(va, 5, 'Node number offset:',
                               form.nodeOffsetKw, 0, AFXTEXTFIELD_INTEGER)
        self.id = AFXTextField(va, 5, 'Element number offset:',
                               form.elementOffsetKw, 0, AFXTEXTFIELD_INTEGER)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
# Class definition
#
class NEOffsetForm(AFXForm):
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def __init__(self, owner):
        # Construct the base class.
        AFXForm.__init__(self, owner)
        #Command to NEoffset
        self.cmd = AFXGuiCommand(self, 'offset', 'neOffset')
        self.nodeOffsetKw = AFXIntKeyword(self.cmd, 'nodeOffset', TRUE, 0)
        self.elementOffsetKw = AFXIntKeyword(self.cmd, 'elementOffset',
                                             TRUE, 0)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def getFirstDialog(self):
        return NEOffsetDB(self)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def verifyKeywordValues(self):        
        """
        Method to check for negative inputs.
        """
        db = self.getCurrentDialog()
        errMsg = 'Offset value must be a positive integer.'
        if self.nodeOffsetKw.getValue() < 0:
            showAFXErrorDialog(db, errMsg)
            db.od.setFocusToTextField()
            return FALSE
        if self.elementOffsetKw.getValue() < 0:
            showAFXErrorDialog(db, errMsg)
            db.id.setFocusToTextField()
            return FALSE
        
        return AFXForm.verifyKeywordValues(self)
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
from symbolicConstants import SymbolicConstant

toolset = getAFXApp().getAFXMainWindow().getPluginToolset()

kernelInitString = 'import neOffset'
version = '1.1'
author = 'ABAQUS, Inc.'
offsetDescription = "Offsets node and element start numbers for "\
                    "flattened input files."
toolset.registerGuiMenuButton(buttonText='Tools|Offset Nodes/Elements...',
                              object=NEOffsetForm(toolset),
                              kernelInitString=kernelInitString,
                              applicableModules=('Part', 'Property',
                                                 'Assembly', 'Step',
                                                 'Interaction', 'Load',
                                                 'Mesh', 'Job'),
                              version=version,
                              author=author,
                              description=offsetDescription)

