"""This module is called by the GUI plugin of Czone to execute the kernel commands
This is not a stand-alone kernel script."""

from abaqus import *
from customKernel import *
from abaqusConstants import NONE
from symbolicConstants import *
from types import *
from abq_Czone.czoneConstants import *
import methodCallback

ValueError = '\nArgumentError'

#===============================
# Class to create czone type Tsai-Wu Damage Definition
#===============================
class TsaiWuDamage(CommandRegister):
    """The class creates a TsaiWu Damage in czone model
    Usage: TsaiWuDamage(name)"""
    def __init__(self, name):
        CommandRegister.__init__(self)
        self.name = name

#        print 'cZone plug-in TsaiWuDamage "%s" has been created.' % self.name
        


#================================================       
# Class to create Czone Definition.
#================================================
#
class CzoneDefinition(CommandRegister):
    """The class creates a CZone Definition in CZone model
    Usage: CzoneDefinition( name)"""
    def __init__(self,name):


        CommandRegister.__init__(self)
        self.name = name
 
#        print 'cZone plug-in Czone Definition "%s" has been created.' % self.name

    
#================================================       
# Class to create czone type Surface Interaction  Condition Definition.
#================================================
#
class SurfaceIntDefinition(CommandRegister):
    """The class creates a SurfaceDefinition in CZone model
    Usage: CzoneDefinition(setname)"""
    def __init__(self, name):


        CommandRegister.__init__(self)
        self.name = name
 
#        print 'cZone plug-in SURFACE "%s" has been created.' % self.name
            
#================================================       
# Class to create czone type Czone Initial Condition Definition.
#================================================
#
class InitialCondDefinition(CommandRegister):
    """The class creates a Initial Condition in CZone model
    Usage: CzoneDefinition(setname)"""
    def __init__(self, name):


        CommandRegister.__init__(self)
        self.name = name
 
#        print 'cZone plug-in initial Condition "%s" has been created.' % self.name

##================================================       
# Class to create czone type Surface Interaction Definition.
#================================================
#
class SurfIntDefinition(CommandRegister):
    """The class creates a Surface Interaction Def Prop in CZone model
    Usage: CzoneDefinition( interactionname, frictioncoefficient)"""
    def __init__(self,name):

        CommandRegister.__init__(self)
        self.name = name
 
#        print 'cZone plug-in Surface Interaction "%s" has been created.' % self.name

#==================================
# Class to create models repository
#==================================


class Model(RepositorySupport):
    """This class creates a czone model with the current model name as its key"""
    def __init__(self,name):
        RepositorySupport.__init__(self)
        if not hasattr(self, 'tsaiwuDamage'):
            self.Repository('tsaiwuDamage',TsaiWuDamage)
        if not hasattr(self, 'czoneDefinition'):
            self.Repository('czoneDefinition',CzoneDefinition)
        if not hasattr(self, 'initialCondDefinition'):
            self.Repository('initialCondDefinition',InitialCondDefinition)
        if not hasattr(self, 'surfaceIntDefinition'):
            self.Repository('surfaceIntDefinition',SurfaceIntDefinition)

        self.name = name
        
#==================================================================
# Class to create abstract class czone to hold the czone repositories
#==================================================================
#
class Czone(RepositorySupport):
    """This abstract class serves as a name space container for
    czone specific models data"""
    def __init__(self):

       RepositorySupport.__init__(self)
       self.Repository('models', Model)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~       
def onModelChange(callingMethod,posArgs,keywordArgs,userData):
    

    callingObject = str(callingMethod)
    start = callingObject.find(' ')
    end = callingObject.find('copyAuxMdbModel')
    callingObject = callingObject[start+2:end-1]
    if callingObject == 'mdb':

        # Interactively, keywordArgs will be populated
        # Note, this will fail ugly if arguments change
        if keywordArgs.keys():
            oldName = keywordArgs['fromName']
            newName = keywordArgs['toName']
        # Through API, use posArgs
        #
        else:
            oldName = posArgs[0]
            newName = posArgs[1]
        if oldName in mdb.models.keys() and callingObject == 'mdb':
            mdb.customData.czone.Model(oldName)

            
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def onModelImport(mdbModel,posArgs,editModelData,userData):
    """This method creates/copies a model object in the czone.models repository
    when a new model in mdb is created"""
    try:

        toModelName = editModelData['name']
        mdb.customData.czone.Model(toModelName) # failing here
        fromModel = editModelData['objectToCopy']
        fromModelName = fromModel.name
        oldModel = mdb.customData.czone.models[fromModelName]

        # Interfaces
        #
        for obj in [oldModel.tsaiwuDamage,
                    oldModel.czoneDefinition,
                    oldModel.surfIntDefinition,
                    oldModel.initialCondDefinition]:
            for key in obj.keys:
                obj[key].copy(key,toModelName)
    except:
        pass


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def onModelCreate(mdbModel,posArgs,editModelData,userData):
    """This method creates/copies a model object in the czone.models repository
    when a new model in mdb is created"""
    try:

        toModelName = editModelData['name']
        mdb.customData.czone.Model(toModelName) # failing here
        fromModel = editModelData['objectToCopy']
        fromModelName = fromModel.name
        oldModel = mdb.customData.czone.models[fromModelName]

        # Interfaces
        #
        for obj in [oldModel.tsaiwuDamage,
                    oldModel.czoneDefinition,
                    oldModel.surfIntDefinition,
                    oldModel.initialCondDefinition]:
            for key in obj.keys:
                obj[key].copy(key,toModelName)
    except:
        pass

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def onModelRename(callingMethod,posArgs,keywordArgs,userData):
    """This method renames the current model name in czone"""
    callingObject = str(callingMethod)
    start = callingObject.find(' ')
    end = callingObject.find('changeKey')
    callingObject = callingObject[start+2:end-1]
    if callingObject == 'mdb.models':

        # Interactively, keywordArgs will be populated
        # Note, this will fail ugly if arguments change
        if keywordArgs.keys():
            oldName = keywordArgs['fromName']
            newName = keywordArgs['toName']

        # Through API, use posArgs
        #
        else:
            oldName = posArgs[0]
            newName = posArgs[1]
        verifyMdbForApp(None)
        if oldName in mdb.models.keys() and callingObject == 'mdb.models':
            mdb.customData.czone.models.changeKey(fromName=oldName,toName=newName)

# The following callbacks are called for creating, renaming, copying a mdb model
# and make the corresponding changes to the czone model.
#
methodCallback.addCallback(mdb, 'Model', onModelCreate,callAfter=True)
methodCallback.addCallback(mdb.models, 'changeKey', onModelRename)
methodCallback.addCallback(mdb, 'ModelFromInputFile', onModelImport, callAfter=True)
methodCallback.addCallback(mdb, 'copyAuxMdbModel', onModelChange, callAfter=True)
methodCallback.addCallback(mdb, 'ModelFromOdbFile', onModelImport, callAfter=True)

