from abaqusGui import *
from abq_Czone.wizardDB import WizardDB
from abaqusConstants import PRESS, DISPLACEMENT
import os
from kernelAccess import mdb,session
from abq_Czone.tsaiwuForm import TsaiwuForm
from abq_Czone.surfIntDefForm import SurfIntDefForm
from abq_Czone.czoneParametersForm import CzoneParametersForm
from abq_Czone.initialConditionForm import InitialConditionForm
###########################################################################
# Class definition
###########################################################################

class CzoneDB(WizardDB):

    [
        STEP_MATERIAL,
        STEP_CZONE_PAR,
        STEP_SURF_INT,
        STEP_INIT_COND,
    ] = range(4)
    #numInstance = 0    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

	#CzoneDB.numInstance +=1
        WizardDB.__init__(self, form)
        #getAFXAliasMap().setPrefix(self, 'wizardDB2-%d' %CzoneDB.numInstance)
        getAFXAliasMap().setPrefix(self,  'CzoneDB')
        path = __file__
        absPath = os.path.abspath(path)
        absDir = os.path.dirname(absPath)
        fileName = os.path.join(absDir, 'notes_delete.png')
        deleteIcon = afxCreatePNGIcon(fileName)
        deleteIcon.scale(24,24)
        fileName = os.path.join(absDir, 'notes_add.png')
        addIcon = afxCreatePNGIcon(fileName)
        addIcon.scale(24,24)
        fileName = os.path.join(absDir, 'materialIcon.gif')
        materialIcon = afxCreateGIFIcon(fileName)
        fileName = os.path.join(absDir, 'notes_edit.png')
        editIcon = afxCreatePNGIcon(fileName)
        editIcon.scale(24,24)
        fileName = os.path.join(absDir, 'treeIcon.gif')
        treeIcon = afxCreateGIFIcon(fileName)
        treeIcon.scale(1,27)
        #------------------------------------------------------------------
        # Czone parameter definition
        #

        step = self.Step(name='Crush Stress', stepId=self.STEP_CZONE_PAR, icon = treeIcon )
        gb = FXGroupBox(step.parent, 'Crush Stress Definition', FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)

        gbVf = FXVerticalFrame(p=gb, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        HFrame_11 = FXHorizontalFrame(p=gbVf, opts=LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        
        self.czoneParBtn = FXButton(HFrame_11, None,addIcon, CzoneParametersForm(form.getOwner(),'CREATE',''), AFXForm.ID_ACTIVATE, BUTTON_TOOLBAR|BUTTON_NORMAL)
        getAFXAliasMap().setName(self.czoneParBtn, 'czoneparbtn') 
        self.czoneParBtn.setTipText("Add Crush Stress to existing material...")
        self.editCzone = FXButton(p=HFrame_11, text=None, ic=editIcon,tgt=self, sel=self.ID_EDITCZONE , opts=BUTTON_TOOLBAR|BUTTON_NORMAL)
        getAFXAliasMap().setName(self.editCzone, 'editczonebtn') 
        self.editCzone.setTipText("Edit...")
        FXMAPFUNC(self, SEL_COMMAND, self.ID_EDITCZONE, CzoneDB.onCmdEditCzone)
        self.delczoneBtn = FXButton(p=HFrame_11, text=None, ic=deleteIcon,tgt=self, sel=self.ID_DELCZONE, opts=BUTTON_TOOLBAR|BUTTON_NORMAL)  
        getAFXAliasMap().setName(self.delczoneBtn, 'delczonebtn') 
        self.delczoneBtn.setTipText("Delete...")
        FXMAPFUNC(self, SEL_COMMAND, self.ID_DELCZONE, CzoneDB.onCmdDeleteCzoneDefinition)
                  
        self.form = form
        listVf = FXVerticalFrame(p=gbVf, opts=FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X, x=50, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        listVf.setSelector(99)
        self.List_4 = AFXList(p=listVf, nvis=7, tgt=form.czoneDefListKw, sel=0, opts=HSCROLLING_OFF|LIST_SINGLESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        getAFXAliasMap().setName(self.List_4, 'list_4') 
        #------------------------------------------------------------------
        # Surface Interaction definition 
        #
        step = self.Step(name='Surface Interaction', stepId=self.STEP_SURF_INT , icon = treeIcon)        

        gb = FXGroupBox(step.parent, 'Surface Interaction Definition', FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        gbVf = FXVerticalFrame(p=gb, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        HFrame_11 = FXHorizontalFrame(p=gbVf, opts=LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        self.surfintBtn = FXButton(HFrame_11, None, addIcon, SurfIntDefForm(form.getOwner(),'CREATE',''), AFXForm.ID_ACTIVATE, BUTTON_TOOLBAR|BUTTON_NORMAL)
        getAFXAliasMap().setName(self.surfintBtn, 'surfintBtn') 
        self.surfintBtn.setTipText("Create Surface Interaction for CZone Property")
        self.editSurf = FXButton(p=HFrame_11, text=None, ic=editIcon,tgt=self, sel=self.ID_EDITSURF, opts=BUTTON_TOOLBAR|BUTTON_NORMAL)
        getAFXAliasMap().setName(self.editSurf, 'editsurfBtn') 
        self.editSurf.setTipText("Edit...")
        FXMAPFUNC(self, SEL_COMMAND, self.ID_EDITSURF, CzoneDB.onCmdEditSurf)
        self.delsurfBtn = FXButton(p=HFrame_11, text=None, ic=deleteIcon,tgt=self, sel=self.ID_DELSURF, opts=BUTTON_TOOLBAR|BUTTON_NORMAL)
        getAFXAliasMap().setName(self.delsurfBtn, 'delsurfBtn')
        self.delsurfBtn.setTipText("Delete...")
        FXMAPFUNC(self, SEL_COMMAND, self.ID_DELSURF, CzoneDB.onCmdDeleteSurfaceInteraction)
        self.form = form
        listVf = FXVerticalFrame(p=gbVf, opts=FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X, x=50, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        listVf.setSelector(99)
        self.List_5 = AFXList(p=listVf, nvis=7, tgt=form.surfIntDefListKw, sel=0, opts=HSCROLLING_OFF|LIST_SINGLESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        getAFXAliasMap().setName(self.List_5, 'list5')

        '''
        #------------------------------------------------------------------
        # Initial Condition definition.
        #
        step = self.Step(name='Predefined Field', stepId=self.STEP_INIT_COND , icon = treeIcon)
        gb = FXGroupBox(step.parent, 'Predefined Field Definition', FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        gbVf = FXVerticalFrame(p=gb, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        HFrame_11 = FXHorizontalFrame(p=gbVf, opts=LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        self.initCondBtn = FXButton(HFrame_11, None, addIcon, InitialConditionForm(form.getOwner()), AFXForm.ID_ACTIVATE,BUTTON_TOOLBAR|BUTTON_NORMAL)
        getAFXAliasMap().setName(self.initCondBtn, 'initcondbtn')
        self.initCondBtn.setTipText("Add predefined field for CZone Property...")
        self.delinitBtn = FXButton(p=HFrame_11, text=None, ic=deleteIcon,tgt=self, sel=self.ID_DELINIT, opts=BUTTON_TOOLBAR|BUTTON_NORMAL)
        getAFXAliasMap().setName(self.delinitBtn, 'delinitbtn')
        self.delinitBtn.setTipText("Delete...")
        FXMAPFUNC(self, SEL_COMMAND, self.ID_DELINIT, CzoneDB.onCmdDeleteInitialCondition)
        self.form = form
        listVf = FXVerticalFrame(p=gbVf, opts=FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X, x=50, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        listVf.setSelector(99)
        self.List_6 = AFXList(p=listVf, nvis=7, tgt=form.initCondDefListKw, sel=0, opts=HSCROLLING_OFF|LIST_SINGLESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        getAFXAliasMap().setName(self.List_6, 'list6')
	'''

        #------------------------------------------------------------------
        # MATERIAL
        #

        step = self.Step2('Tsai-Wu Damage', self.STEP_MATERIAL , treeIcon)

        gb = FXGroupBox(step.parent, 'Tsai-Wu Damage Definition', FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        gbVf = FXVerticalFrame(p=gb, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)

        self.form = form
        self.tsaiwuForm = TsaiwuForm(form.getOwner(),'CREATE','')
        HFbtn = FXHorizontalFrame(p=gbVf, opts=LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        getAFXAliasMap().setName(HFbtn, 'hfbtn')     
        self.tsaiwuBtn = FXButton(HFbtn, None, addIcon, self.tsaiwuForm , AFXForm.ID_ACTIVATE,BUTTON_TOOLBAR|BUTTON_NORMAL)
        getAFXAliasMap().setName(self.tsaiwuBtn, 'tsaiwubtn')   
        self.tsaiwuBtn.setTipText("Add Tsai-Wu damage to existing material...")
        self.editbtn = FXButton(p=HFbtn, text=None, ic=editIcon,tgt=self, sel=self.ID_EDITSAI, opts=BUTTON_TOOLBAR|BUTTON_NORMAL)  
        getAFXAliasMap().setName(self.editbtn, 'editbtn') 
        self.editbtn.setTipText("Edit...")
        FXMAPFUNC(self, SEL_COMMAND, self.ID_EDITSAI, CzoneDB.onCmdEditTsaiwu)
        self.delmatBtn = FXButton(p=HFbtn, text=None, ic=deleteIcon,tgt=self, sel=self.ID_DELTSAI , opts=BUTTON_TOOLBAR|BUTTON_NORMAL) 
        getAFXAliasMap().setName(self.delmatBtn, 'delmatbtn') 
        self.delmatBtn.setTipText("Delete...")
        FXMAPFUNC(self, SEL_COMMAND, self.ID_DELTSAI, CzoneDB.onCmdDeleteTsaiwuMaterial)
        listVf = FXVerticalFrame(p=gbVf, opts=FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        listVf.setSelector(99)
        self.List_3 = AFXList(p=listVf, nvis=7, tgt=form.tsaiwuListKw, sel=0, opts=HSCROLLING_OFF|LIST_SINGLESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        getAFXAliasMap().setName(self.List_3, 'list3') 
        modelName = getCurrentContext()['modelName']
        if mdb.customData.czone.models.has_key(modelName):
            if not len(mdb.customData.czone.models[modelName].tsaiwuDamage) == 0:
                materials= mdb.customData.czone.models[modelName].tsaiwuDamage.keys()
                for material in materials:
                    if self.List_3.findItem(material,-1,SEARCH_FORWARD| SEARCH_WRAP) == -1:
                        self.List_3.appendItem(text=material)
                    if not material in mdb.models[modelName].materials.keys():
                        numItems = self.List_3.getNumItems()
                        for index in range(numItems):
                            if self.List_3.getItemText(index) == material:
                                self.List_3.removeItem(index)
                                del mdb.customData.czone.models[modelName].tsaiwuDamage[material]
                                break

        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onSwitchStep(self, currentStep, newStep):
        # Error checking
        #
        if currentStep.stepId == self.STEP_MATERIAL:
            # Nothing needs to be done
            pass
        elif currentStep.stepId == self.STEP_CZONE_PAR:
            pass

        elif currentStep.stepId == self.STEP_SURF_INT:
            pass

        if newStep.stepId == self.STEP_MATERIAL:
            modelName = getCurrentContext()['modelName']
            if mdb.customData.czone.models.has_key(modelName):
                if not len(mdb.customData.czone.models[modelName].tsaiwuDamage) == 0:
                    materials= mdb.customData.czone.models[modelName].tsaiwuDamage.keys()
                    for material in materials:
                        if self.List_3.findItem(material,-1,SEARCH_FORWARD| SEARCH_WRAP) == -1:
                            self.List_3.appendItem(text=material)
                        if not material in mdb.models[modelName].materials.keys():
                            numItems = self.List_3.getNumItems()
                            for index in range(numItems):
                                if self.List_3.getItemText(index) == material:
                                    self.List_3.removeItem(index)
                                    del mdb.customData.czone.models[modelName].tsaiwuDamage[material]
                                    break
        elif newStep.stepId == self.STEP_CZONE_PAR:
            modelName = getCurrentContext()['modelName']
            if mdb.customData.czone.models.has_key(modelName):
                if not len(mdb.customData.czone.models[modelName].czoneDefinition) == 0:
                    intproperties= mdb.customData.czone.models[modelName].czoneDefinition.keys()
                    for property in intproperties:
                        if self.List_4.findItem(property,-1,SEARCH_FORWARD| SEARCH_WRAP) == -1:
                            self.List_4.appendItem(text=property)
                        if not property in mdb.models[modelName].materials.keys():
                            numItems = self.List_4.getNumItems()
                            for index in range(numItems):
                                if self.List_4.getItemText(index) == property:
                                    self.List_4.removeItem(index)
                                    del mdb.customData.czone.models[modelName].czoneDefinition[property]
                                    break
        elif newStep.stepId == self.STEP_SURF_INT:
            modelName = getCurrentContext()['modelName'] 
            if mdb.customData.czone.models.has_key(modelName):                        
                if not len(mdb.customData.czone.models[modelName].surfaceIntDefinition) == 0:
                    intproperties= mdb.customData.czone.models[modelName].surfaceIntDefinition.keys()
                    for property in intproperties:
                        if self.List_5.findItem(property,-1,SEARCH_FORWARD| SEARCH_WRAP) == -1:
                            self.List_5.appendItem(text=property)
                        if not property in mdb.models[modelName].interactionProperties.keys():
                            numItems = self.List_5.getNumItems()
                            for index in range(numItems):
                                if self.List_5.getItemText(index) == property:
                                    self.List_5.removeItem(index)
                                    del mdb.customData.czone.models[modelName].surfaceIntDefinition[property]
                                    break
	'''
        elif newStep.stepId == self.STEP_INIT_COND:
            modelName = getCurrentContext()['modelName']
            if mdb.customData.czone.models.has_key(modelName):
                if not len(mdb.customData.czone.models[modelName].initialCondDefinition) == 0:
                    definedsets= mdb.customData.czone.models[modelName].initialCondDefinition.keys()
                    for set in definedsets:
                        if self.List_6.findItem(set,-1,SEARCH_FORWARD| SEARCH_WRAP) == -1:
                            self.List_6.appendItem(text=set)
                        if not set in mdb.models[modelName].rootAssembly.sets.keys():
                            numItems = self.List_6.getNumItems()
                            for index in range(numItems):
                                if self.List_6.getItemText(index) == set:
                                    self.List_6.removeItem(index)
                                    del mdb.customData.czone.models[modelName].initialCondDefinition[set]
                                    break
	'''
        return True
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
        


        # Register a query on CustomData
        #
        self.currentModelName = getCurrentContext()['modelName']
        mdb.models[self.currentModelName].materials.registerQuery(self.updateManager_Material)
        mdb.customData.czone.models[self.currentModelName].initialCondDefinition.registerQuery(self.updateManager_InitCondDef)
        mdb.customData.czone.models[self.currentModelName].surfaceIntDefinition.registerQuery(self.updateManager_SurfIntDef)
        mdb.customData.czone.models[self.currentModelName].czoneDefinition.registerQuery(self.updateManager_CzoneDef)
        mdb.customData.czone.models[self.currentModelName].tsaiwuDamage.registerQuery(self.updateManager_TsaiwuDef)
        AFXDataDialog.show(self)


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def hide(self):
        
        # Register a query on CustomData
        #
        self.currentModelName = getCurrentContext()['modelName']
        mdb.models[self.currentModelName].materials.unregisterQuery(self.updateManager_Material)        
        mdb.customData.czone.models[self.currentModelName].initialCondDefinition.unregisterQuery(self.updateManager_InitCondDef)
        mdb.customData.czone.models[self.currentModelName].surfaceIntDefinition.unregisterQuery(self.updateManager_SurfIntDef)
        mdb.customData.czone.models[self.currentModelName].czoneDefinition.unregisterQuery(self.updateManager_CzoneDef)
        mdb.customData.czone.models[self.currentModelName].tsaiwuDamage.unregisterQuery(self.updateManager_TsaiwuDef)
        if self.tsaiwuForm.getCurrentDialog():
            self.tsaiwuForm.getCurrentDialog().hide()
        #self.form.czdeactivateKw.setValue(self.form.czdeactivateKw.getValue())
        AFXDataDialog.hide(self)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def updateManager_Material(self):
        modelName = getCurrentContext()['modelName']
        if mdb.customData.czone.models.has_key(modelName):
            if not len(mdb.customData.czone.models[modelName].tsaiwuDamage) == 0:
                tsaiWumaterials= mdb.customData.czone.models[modelName].tsaiwuDamage.keys()
                for material in tsaiWumaterials:
                    if not material in mdb.models[modelName].materials.keys():
                        numItems = self.List_3.getNumItems()
                        for index in range(numItems):
                            if self.List_3.getItemText(index) == material:
                                self.List_3.removeItem(index)
                                del mdb.customData.czone.models[modelName].tsaiwuDamage[material]
                                break
            if not len(mdb.customData.czone.models[modelName].czoneDefinition) == 0:	
                cZonematerials= mdb.customData.czone.models[modelName].czoneDefinition.keys()	
                for material in cZonematerials:
                    if not material in mdb.models[modelName].materials.keys():
                        numItems = self.List_4.getNumItems()
                        for index in range(numItems):
                            if self.List_4.getItemText(index) == material:
                                self.List_4.removeItem(index)
                                del mdb.customData.czone.models[modelName].czoneDefinition[material]
                                break
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                    
    def updateManager_InitCondDef(self):
        modelName = getCurrentContext()['modelName']
        if mdb.customData.czone.models.has_key(modelName):
            if not len(mdb.customData.czone.models[modelName].initialCondDefinition) == 0:
                definedsets= mdb.customData.czone.models[modelName].initialCondDefinition.keys()
                for set in definedsets:
                    if self.List_6.findItem(set,-1,SEARCH_FORWARD| SEARCH_WRAP) == -1:
                        self.List_6.appendItem(text=set)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                        
    def updateManager_SurfIntDef(self):
        modelName = getCurrentContext()['modelName']
        if mdb.customData.czone.models.has_key(modelName):
            if not len(mdb.customData.czone.models[modelName].surfaceIntDefinition) == 0:
                intproperties= mdb.customData.czone.models[modelName].surfaceIntDefinition.keys()
                for property in intproperties:
                    if self.List_5.findItem(property,-1,SEARCH_FORWARD| SEARCH_WRAP) == -1:
                        self.List_5.appendItem(text=property)
                    if not property in mdb.models[modelName].interactionProperties.keys():
                        mdb.models[modelName].ContactProperty(name=property)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                        
    def updateManager_CzoneDef(self):
        modelName = getCurrentContext()['modelName']
        if mdb.customData.czone.models.has_key(modelName):
            if not len(mdb.customData.czone.models[modelName].czoneDefinition) == 0:
                materials= mdb.customData.czone.models[modelName].czoneDefinition.keys()
                for material in materials:
                    if self.List_4.findItem(material,-1,SEARCH_FORWARD| SEARCH_WRAP) == -1:
                        self.List_4.appendItem(text=material)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                        
    def updateManager_TsaiwuDef(self):
        modelName = getCurrentContext()['modelName']
        if not len(mdb.customData.czone.models[modelName].tsaiwuDamage) == 0:
            materials= mdb.customData.czone.models[modelName].tsaiwuDamage.keys()
            for material in materials:
                if self.List_3.findItem(material,-1,SEARCH_FORWARD| SEARCH_WRAP) == -1:
                    self.List_3.appendItem(text=material)
                                        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def onCmdEditTsaiwu(self, sender, sel, ptr):
        
        from tsaiwuForm import TsaiwuForm
        index = self.List_3.getSingleSelection()
        if index == -1 :
            showAFXErrorDialog(self, 'Select a Material to Edit.')
            return False
        materialKw = self.List_3.getItemText(index)

        self.tsaiForm= TsaiwuForm(self.form.getOwner(), 'EDIT', materialKw)
        self.tsaiForm.setModal(TRUE)
        self.tsaiForm.activate()


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdEditCzone(self, sender, sel, ptr):
        
        from czoneParametersForm import CzoneParametersForm
        index = self.List_4.getSingleSelection()
        if index == -1 :
            showAFXErrorDialog(self, 'Select a Material to Edit.')
            return False
        materialKw = self.List_4.getItemText(index)
        self.czoneDefForm= CzoneParametersForm(self.form.getOwner(), 'EDIT', materialKw)
        self.czoneDefForm.setModal(TRUE)
        self.czoneDefForm.activate()

        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdEditSurf(self, sender, sel, ptr):
        
        from surfIntDefForm import SurfIntDefForm
        index = self.List_5.getSingleSelection()
        if index == -1 :
            showAFXErrorDialog(self, 'Select a Surface Interaction to Edit.')
            return False
        intNameKw = self.List_5.getItemText(index)
        self.surfForm= SurfIntDefForm(self.form.getOwner(),'EDIT',intNameKw)
        self.surfForm.setModal(TRUE)
        self.surfForm.activate()
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def onCmdDeleteTsaiwuMaterial(self,sender,sel,ptr):
        index = self.List_3.getSingleSelection()
        if index == -1 :
            showAFXErrorDialog(self, 'Select a Material to delete.')
            return False
        else:
            modelName = getCurrentContext()['modelName']
            materialName = self.List_3.getItemText(index)
            self.List_3.removeItem(index)
            del mdb.customData.czone.models[modelName].tsaiwuDamage[materialName]
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def onCmdDeleteCzoneDefinition(self,sender,sel,ptr):

        index = self.List_4.getSingleSelection()
        if index == -1 :
            showAFXErrorDialog(self, 'Select a CZone definition to delete.')
            return False
        else:
            modelName = getCurrentContext()['modelName']
            materialName = self.List_4.getItemText(index)
            self.List_4.removeItem(index)
            del mdb.customData.czone.models[modelName].czoneDefinition[materialName]
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def onCmdDeleteSurfaceInteraction(self,sender,sel,ptr):

        index = self.List_5.getSingleSelection()
        if index == -1 :
            showAFXErrorDialog(self, 'Select a Surface Interaction to delete.')
            return False
        else:
            modelName = getCurrentContext()['modelName']
            interactionName = self.List_5.getItemText(index)
            self.List_5.removeItem(index)
            del mdb.customData.czone.models[modelName].surfaceIntDefinition[interactionName]
            del mdb.models[modelName].interactionProperties[interactionName]

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def onCmdDeleteInitialCondition(self,sender,sel,ptr):

        index = self.List_6.getSingleSelection()
        if index == -1 :
            showAFXErrorDialog(self, 'Select a Set to delete.')
            return False
        else:
            modelName = getCurrentContext()['modelName']
            setName = self.List_6.getItemText(index)
            self.List_6.removeItem(index)
            del mdb.customData.czone.models[modelName].initialCondDefinition[setName]


        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def processUpdates(self):
        """
        """

        #

        if not self.List_3.getNumItems() > 0:
            self.editbtn.disable()
            self.delmatBtn.disable()
        else:
            self.editbtn.enable()
            self.delmatBtn.enable()
        if not self.List_4.getNumItems() > 0:
            self.editCzone.disable()
            self.delczoneBtn.disable()
        else:
            self.editCzone.enable()
            self.delczoneBtn.enable()
        if not self.List_5.getNumItems() > 0:
            self.editSurf.disable()
            self.delsurfBtn.disable()
        else:
            self.editSurf.enable()
            self.delsurfBtn.enable()
        #if not self.List_6.getNumItems() > 0:
        #    self.delinitBtn.disable()
        #else:
        #    self.delinitBtn.enable()
        #print "self.form.czdeactivateKw===",self.form.czdeactivateKw.getValue()
        #print "self.form.czdeactivateKw getValueAsString",self.form.czdeactivateKw.getValueAsString()
        if self.form.czdeactivateKw.getValue():
            self.stepactiveIndex = 0
            self.switcher.setCurrent(self.stepactiveIndex)
            self.stepNames.selectItem(self.treeItems[self.stepactiveIndex])
            if not self.stepNames.isItemSelected(self.stepNames.getFirstItem()):
                self.forceRefresh()
            self.stepNames.disable()
            self.stepNames2.disable()
            self.stepNames.deselectItem(self.stepNames.getLastItem())
            self.stepNames2.deselectItem(self.stepNames2.getFirstItem())
            self.stepNames.disableItem(self.treeItems[1])
            self.stepNames2.disableItem(self.treeItems2[0])
            #self.stepNames.disableItem(self.treeItems[3])
        else:
            self.stepNames.enable()
            self.stepNames2.enable()
            self.stepNames.enableItem(self.treeItems[1])
            self.stepNames2.enableItem(self.treeItems2[0])
            #self.stepNames.enableItem(self.treeItems[3])                                     

