from abaqus import mdb, session
import customKernel, re
import abq_Czone.czoneClassModule as czoneClassModule

# Create an appData object for this application.
#
pgData = customKernel.AbaqusAppData()
from abq_Czone.czoneConstants import pgVersion, pgName
splitter = re.compile('\.*-*')

# Add custom attributes to the appData object.
# This will not work if updateVersion (increment) is EF
#
pgData.majorVersion, pgData.minorVersion, pgData.updateVersion = map(int,splitter.split(pgVersion))

# Install the appData for this session.
# setAppData will create session.customData.appData
# setAppData may be called only once per session.
#
customKernel.setAppData(pgName, pgData)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def verifyMdb(mdbAppData):
    # If no appData, initialize the MDB.
    #
    if mdbAppData == None:
        initializeMdb()
        return

    # Check for our appData.
    #
    if not mdbAppData.has_key(pgName):
        initializeMdb()
        return

    # Check if czone and models repositories exist
    #
    if not hasattr(mdb.customData, 'czone'):
        mdb.customData.czone = czoneClassModule.Czone()

    vModelNameKeys = mdb.customData.czone.models.keys()

    for modelName in mdb.models.keys():
        if modelName not in vModelNameKeys:
            mdb.customData.czone.Model(modelName)

    # Check some application attributes on the MDB.
    #
    try:
        mdbMajorVersion = mdbAppData[pgName].majorVersion
        mdbMinorVersion = mdbAppData[pgName].minorVersion
        mdbUpdateVersion = mdbAppData[pgName].updateVersion

        # Write to rpy file the app version number
        #
        databaseVersion = '%s.%s-%s'%(mdbMajorVersion, mdbMinorVersion,mdbUpdateVersion)
        pluginVersion = '%s.%s-%s'%(pgData.majorVersion,pgData.minorVersion,pgData.updateVersion)
        backwardCompatibMsg = 'ERROR: The plug-in version being loaded is older than the one '
        backwardCompatibMsg += 'that created custom data in the database. Please upgrade your plug-in.\n'
        if pgData.majorVersion < mdbMajorVersion:
            print backwardCompatibMsg
        elif pgData.majorVersion > mdbMajorVersion:
            upgradeToMajorVersion()
        elif pgData.minorVersion < mdbMinorVersion:
            print backwardCompatibMsg
        elif pgData.minorVersion > mdbMinorVersion:
            upgradeToMinorVersion(pgData.majorVersion, pgData.minorVersion,  pgData.updateVersion, mdbMinorVersion)
#        print '-------------------------------------------------------------\n\n'
    except:
        raise ValueError, 'Could not access application data attribute.'

# Set the verifyMdb method for the application.
# setVerifyMdb may be called only once per application name.
#

customKernel.setVerifyMdb(pgName, verifyMdb)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def upgradeToMajorVersion():
    
    # No upgrades yet
    #
    pass

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def upgradeToMinorVersion(toMajor, toMinor, toUpdate, fromMinor ):

    # No upgrades yet
    #


    # From 2.1-6 to 2.2-1 or higher need to add:
    #   Contact Clearance object
    #
    #   and update interface objects.
    #
    printMsg = 0
    if toMinor >= 2 and fromMinor <= 1:

        printMsg = 1

    # From 2.2-x to 2.3-x or higher, No changes. Just upgrade to 6.10
    #
    if toMinor >= 3 and fromMinor <= 2:
        printMsg = 1

    if printMsg:
        print 'CZONE database upgraded to version %s.%s-%s' % (toMajor, toMinor, toUpdate)    

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def initializeMdb():
    if 'versions' not in dir(mdb.customData):
        mdb.customData.versions = {}
    mdb.customData.versions[pgName] = '%d.%d-%d' % \
        (pgData.majorVersion,
         pgData.minorVersion,
         pgData.updateVersion)

    vp = session.currentViewportName
    modelName = session.sessionState[vp]['modelName']
    if not hasattr(mdb.customData, 'czone'):
        mdb.customData.czone = czoneClassModule.Czone()
    if mdb.customData.czone.models.has_key(modelName):
        m = mdb.customData.czone.models[modelName]
    else:
        mdb.customData.czone.Model(modelName)


customKernel.setInitializeMdb(pgName, initializeMdb)
# This file is executed after Abaqus/CAE has started, so we need to 
# process the initial MDB (either a new, empty MDB created by Abaqus/CAE,
# or a database opened via the -database command line argument).
#

customKernel.processInitialMdb(pgName)
