from abaqusConstants import *
from abaqusGui import *
from kernelAccess import mdb, session
import os

thisPath = os.path.abspath(__file__)
thisDir = os.path.dirname(thisPath)


###########################################################################
# Class definition
###########################################################################

class CzoneParametersDB(AFXDataDialog):
    #numInstance = 0  	
    [
    	ID_FIELD_DEP
    ] = range(5100,5101)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form, actionKw, materialKw):

        # Construct the base class.
        #
        #CzoneParametersDB.numInstance +=1
        #AFXDataDialog.__init__(self, form, 'CZone Property',
        #    0, DIALOG_ACTIONS_SEPARATOR, self.OK|self.CANCEL)
        AFXDataDialog.__init__(self, form, 'CZone Property', self.OK|self.CANCEL) 

        #getAFXAliasMap().setPrefix(self, 'czoneparametersDB-%d' %CzoneParametersDB.numInstance)
        getAFXAliasMap().setPrefix(self, 'CzoneParametersDB')
        #self.appendActionButton(text='OK',tgt=self,
        #                        sel=self.ID_CLICKED_OK)
        #self.appendActionButton(text='Cancel',tgt=self,
        #                        sel=self.ID_CLICKED_CANCEL)       

        frame = AFXVerticalAligner(self, 0, 0,0,0,0, 0,0,0,0)

        # Model combo
        # Since all forms will be canceled if the  model changes,
        # we do not need to register a query on the model.
        #
        self.currentModelName = getCurrentContext()['modelName']
        self.ComboBox_1 = AFXComboBox(p=frame, ncols=0, nvis=1, text='CZone Material:', tgt=form.materialfromCzoneDefKw, sel=0)
        getAFXAliasMap().setName(self.ComboBox_1, 'combobox1')
        self.ComboBox_1.setMaxVisible(10)
        if actionKw == 'EDIT':
            names = mdb.models[self.currentModelName].materials.keys()        
            for name in names:
                if name == materialKw:
                    self.ComboBox_1.appendItem(name)
                    form.materialfromCzoneDefKw.setValue(name)
            if self.ComboBox_1.getNumItems()== 0:
                    showAFXErrorDialog(self,'Material edition not possible.\nPlease check if the material exists in the model.')
        else:
            names = mdb.models[self.currentModelName].materials.keys()        
            names.sort()
            if mdb.customData.czone.models.has_key(self.currentModelName):
                if not len(mdb.customData.czone.models[self.currentModelName].czoneDefinition) == 0:
                    for index in range(len(names), -1, -1):
                        if len(names) > index:
                            if names[index] in mdb.customData.czone.models[self.currentModelName].czoneDefinition.keys():
                                del names[index]
                for name in names:
                    self.ComboBox_1.appendItem(name)

            if not form.materialfromCzoneDefKw.getValue() in names:
                if not len(names) == 0:
                    form.materialfromCzoneDefKw.setValue( names[0] )
                else :
                    form.materialfromCzoneDefKw.setValue( '')
                

        # Materials combo
        #
        self.form = form
        self.actionKwTemp = actionKw
        self.materialKw = materialKw
        HFrame_1 = FXHorizontalFrame(p=self, opts=LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        l = FXLabel(p=HFrame_1, text='Crush Stress:', opts=JUSTIFY_LEFT)
        rado1=FXRadioButton(p=HFrame_1, text='Tabular', tgt=form.GroupBoxKw1, sel=TABULAR.getId())
        getAFXAliasMap().setName(rado1, 'rado1btn')
        rado2=FXRadioButton(p=HFrame_1, text='Constant', tgt=form.GroupBoxKw1, sel=CONSTANT.getId())
        getAFXAliasMap().setName(rado2, 'rado2btn')
        self.textField = AFXTextField(p=HFrame_1, ncols=12, labelText='', tgt=form.crushStressConstValueKw, sel=0)
        getAFXAliasMap().setName(self.textField, 'txtfield1')

        HFrame_2 = FXHorizontalFrame(p=self, opts=LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
	self.spinner = AFXSpinner(HFrame_2, 2, 'Field Variables:', self.form.FieldVariableDepKw, 0)
	self.spinner.setRange(0, 3)
	self.spinner.setValue(0)
        getAFXAliasMap().setName(self.spinner, 'FieldVariableDep')
        self.fieldVarValue=0
        #FXMAPFUNC(self, SEL_COMMAND, self.ID_FIELD_DEP, CzoneParametersDB.insertCol)
        
        self.tempTable = FXCheckButton(p=HFrame_2, text='Temperature', tgt=self.form.tempTableKw, sel=0)
        getAFXAliasMap().setName(self.tempTable, 'TemperatureTable')
        self.form.tempTableKw.setTarget(self)
        self.form.tempTableKw.setSelector(self.ID_FIELD_DEP)
        self.form.tempTableKw.setValue(False)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_FIELD_DEP, CzoneParametersDB.inclTemperature)
        self.nColTable=3
        

        vf = FXVerticalFrame(self, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X,
            0,0,0,0, 0,0,0,0)
        # Note: Set the selector to indicate that this widget should not be
        #       colored differently from its parent when the 'Color layout managers'
        #       button is checked in the RSG Dialog Builder dialog.
        vf.setSelector(99)
        
        self.table1 = AFXTable(vf, 7, 3, 7, 3, form.czCrushStressTabValueKw, 0, AFXTABLE_EDITABLE|AFXTABLE_COLUMN_RESIZABLE|LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.table1, 'tbl1')
	
        self.table1.setPopupOptions(AFXTable.POPUP_CUT|AFXTable.POPUP_COPY|AFXTable.POPUP_PASTE|AFXTable.POPUP_INSERT_ROW|AFXTable.POPUP_DELETE_ROW|AFXTable.POPUP_CLEAR_CONTENTS)
        self.table1.setLeadingRows(1)
        self.table1.setLeadingColumns(1)
        self.table1.setColumnWidth(1, 100)
        self.table1.setColumnType(1, AFXTable.FLOAT)
        self.table1.setColumnWidth(2, 75)
        self.table1.setColumnType(2, AFXTable.INT)
        self.table1.setLeadingRowLabels('Crush Stress\tMaterial Angle(deg)')
        self.table1.setStretchableColumn( self.table1.getNumColumns()-1 )
        self.table1.showHorizontalGrid(True)
        self.table1.showVerticalGrid(True)
        self.velCheckBtn = FXCheckButton(p=self, text='Crush Stress Velocity Factor', tgt=form.czVelocityKw, sel=0)
        getAFXAliasMap().setName(self.velCheckBtn, 'chkbtn')

        vf = FXVerticalFrame(self, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X,
            0,0,0,0, 0,0,0,0)
        # Note: Set the selector to indicate that this widget should not be
        #       colored differently from its parent when the 'Color layout managers'
        #       button is checked in the RSG Dialog Builder dialog.
        vf.setSelector(99)
        self.table2 = AFXTable(vf, 7, 3, 7, 3, form.czVelocityTabKw, 0, AFXTABLE_EDITABLE|LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.table2, 'tbl2')

        self.table2.setPopupOptions(AFXTable.POPUP_CUT|AFXTable.POPUP_COPY|AFXTable.POPUP_PASTE|AFXTable.POPUP_INSERT_ROW|AFXTable.POPUP_DELETE_ROW|AFXTable.POPUP_CLEAR_CONTENTS)
        self.table2.setLeadingRows(1)
        self.table2.setLeadingColumns(1)
        self.table2.setColumnWidth(1, 140)
        self.table2.setColumnType(1, AFXTable.INT)
        self.table2.setColumnWidth(2, 100)
        self.table2.setColumnType(2, AFXTable.FLOAT)
        self.table2.setLeadingRowLabels('Scaling factor\tRelative velocity')
        self.table2.setStretchableColumn( self.table2.getNumColumns()-1 )
        self.table2.showHorizontalGrid(True)
        self.table2.showVerticalGrid(True)
        
        self.totalFields=0
        if actionKw == 'EDIT':
            self.ComboBox_1.disable()
            if mdb.customData.czone.models.has_key(self.currentModelName):

                m=mdb.customData.czone.models[self.currentModelName].czoneDefinition[materialKw]
                if m.CrushStressDefType == CONSTANT:
                    
                    form.GroupBoxKw1.setValue(CONSTANT.getId())
                    form.crushStressConstValueKw.setValue(str(m.CrushStressConstVal))

                    for row in range(3):
                        for column in range(2):
                            self.table1.setItemValue(row+1,column+1,'')
                else:
                    form.GroupBoxKw1.setValue(TABULAR.getId())
                    #mdb.customData.czone.models['Model-1'].czoneDefinition['Material-1'].CrushStressTableVal
                    val=m.CrushStressTableVal
                    try:
                       self.totalFields=m.FieldVariableDep
                       self.temperatureBol=m.tempTable
                    except:
                       #mdb.customData.czone.models[self.currentModelName].czoneDefinition[materialKw].FieldVariableDep = '0'
                       self.totalFields=0
                       self.form.FieldVariableDepKw.setValue(0)
                       #mdb.customData.czone.models[self.currentModelName].czoneDefinition[materialKw].tempTable = 0
                       self.temperatureBol=0
                       self.form.tempTableKw.setValue(0)
                    
                    self.form.tempTableKw.setValue(self.temperatureBol)
                    self.nColTable=self.nColTable+self.temperatureBol
                    if self.temperatureBol==1:
                    	   CzoneParametersDB.editTable_Temperature(self)
                    
                    #nCols=len(val[0])
                    for row in range(len(val)):
                        for column in range(self.totalFields+2+self.temperatureBol):
                            form.czCrushStressTabValueKw.setValue(row,column,str(m.CrushStressTableVal[row][column]))
                    self.spinner.setValue(self.totalFields)
                    self.form.FieldVariableDepKw.setValue(self.totalFields)

                if m.IsCzVelocityDefined :

                    form.czVelocityKw.setValue(TRUE)
                    self.velCheckBtn.setCheck(TRUE)

                    for row in range(len(m.VelocityTableVal)):
                        for column in range(2):
                            form.czVelocityTabKw.setValue(row,column,str(m.VelocityTableVal[row][column]))
                else:
                    self.velCheckBtn.setCheck(FALSE)
                    form.czVelocityKw.setValue(FALSE)

                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):

        AFXDataDialog.show(self)

        # Register a query on materials
        #
        self.currentModelName = getCurrentContext()['modelName']
        mdb.models[self.currentModelName].materials.registerQuery(self.updateComboBox_1Materials)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        AFXDataDialog.hide(self)
        self.currentModelName = getCurrentContext()['modelName']
        mdb.models[self.currentModelName].materials.unregisterQuery(self.updateComboBox_1Materials)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onComboBox_1MaterialsChanged(self, sender, sel, ptr):

        self.updateComboBox_1Materials()
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateComboBox_1Materials(self):


        self.currentmodelName = getCurrentContext()['modelName']
        # Update the names in the Materials combo
        #
        self.ComboBox_1.clearItems()
        names = mdb.models[self.currentmodelName].materials.keys()
        names.sort()
        if mdb.customData.czone.models.has_key(self.currentModelName):
            if not len(mdb.customData.czone.models[self.currentModelName].czoneDefinition) == 0:
                for index in range(len(names),-1,-1):
                    if len(names) > index :
                        if names[index] in mdb.customData.czone.models[self.currentModelName].czoneDefinition.keys():
                            del names[index]
        for name in names:
            self.ComboBox_1.appendItem(name)
        if names:
            if not self.form.materialfromCzoneDefKw.getValue() in names:
                self.form.materialfromCzoneDefKw.setValue( names[0] )
        else:
            self.form.materialfromCzoneDefKw.setValue('')

        self.resize( self.getDefaultWidth(), self.getDefaultHeight() )

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def processUpdates(self):
        """
        """
        # Check the radio button and grey out the alternate option.
        #
        czoneStressDefinition = self.form.GroupBoxKw1.getValue()
        czoneVelocityDefinition = self.form.czVelocityKw.getValue()
        n_FieldVariables = self.form.FieldVariableDepKw.getValue()
        nCols=self.table1.getNumColumns() 
        if n_FieldVariables+self.nColTable != nCols: #Including leading col
            if n_FieldVariables+self.nColTable > nCols:	
        	for i in range(n_FieldVariables+self.nColTable-nCols):
        	   self.table1.insertColumns(nCols+i,1)
        	   self.table1.setItemText(0, nCols+i,'Field '+ str(nCols-self.nColTable+i+1))
            else:
                 for i in range(-n_FieldVariables-self.nColTable+nCols):
                   self.table1.deleteColumns(nCols-1,1)
                   self.form.czCrushStressTabValueKw.removeColumns(nCols-i-2, 1)
        '''
        if n_FieldVariables!=self.fieldVarValue and n_FieldVariables >= 0:
           nCols=self.table1.getNumColumns()
           if n_FieldVariables > self.fieldVarValue:
	        self.table1.insertColumns(nCols,1)
	        self.fieldVarValue=n_FieldVariables
	        self.table1.setItemText(0, nCols,'Field '+ str(n_FieldVariables))
           else:
	        self.table1.deleteColumns(nCols-1,1)
	        self.fieldVarValue=n_FieldVariables
                
        '''   
        if czoneStressDefinition == CONSTANT.getId():
            self.disableTabDef()
            
        else:
            self.disableConstantDef()
        if czoneVelocityDefinition == FALSE:
            self.table2.disable()

        else:
            self.table2.enable()
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def inclTemperature(self, sender, sel, data):
        CzoneParametersDB.editTable_Temperature(self)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def editTable_Temperature(self):    
        if self.form.tempTableKw.getValue() == 0:
	   self.nColTable=3
           self.table1.deleteColumns(3,1)
           self.form.czCrushStressTabValueKw.removeColumns(2, 1)
	else:
           self.nColTable=4
	   self.table1.insertColumns(3,1)
           self.table1.setItemText(0, 3,'Temperature')
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def disableTabDef(self):
        """
        """
        self.table1.disable()
        self.spinner.disable()
        self.tempTable.disable()
        self.textField.enable()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def disableConstantDef(self):
        """
        """
        self.table1.enable()
        self.spinner.enable()
        self.tempTable.enable()
        self.textField.disable()

