from abaqusGui import *
from abaqusConstants import ALL
import osutils, os
from abaqusConstants import *
import types

###########################################################################
# Class definition
###########################################################################

class CzoneParametersForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, actionKw, materialKw):
        
        # Construct the base class.
        #

        AFXForm.__init__(self, owner)
        self.actionKw = actionKw
        self.materialKw = materialKw
        self.radioButtonGroups = {}
        self.cmd = AFXGuiCommand(mode=self, method='writeCzoneParameters',objectName='abq_Czone.czoneUtils', registerQuery=False)
        #(modelName, materialName, crushStressDefType=CONSTANT, crushStressConstVal = 0,crushStressTableVal=(), isCzVelocityDefined=False, velocityTableVal=()):
        pickedDefault = ''
        self.modelNameKw = AFXStringKeyword(self.cmd, 'modelName', True, getCurrentContext()['modelName'])
        self.materialfromCzoneDefKw = AFXStringKeyword(self.cmd, 'materialName', True)
        if actionKw == 'CREATE':
            self.GroupBoxKw1 = AFXSymConstKeyword(self.cmd, 'crushStressDefType',True, CONSTANT.getId())
        else:
            self.GroupBoxKw1 = AFXSymConstKeyword(self.cmd, 'crushStressDefType',True)
        self.FieldVariableDepKw = AFXIntKeyword(self.cmd, 'FieldVariableDep', False, 0)
        self.tempTableKw = AFXBoolKeyword(self.cmd, 'tempTable', AFXBoolKeyword.TRUE_FALSE, False, False)
        self.crushStressConstValueKw = AFXFloatKeyword(self.cmd, 'crushStressConstVal', False)
        self.czCrushStressTabValueKw = AFXTableKeyword(self.cmd, 'crushStressTableVal', False )
        self.czCrushStressTabValueKw.setColumnType(0, AFXTABLE_TYPE_FLOAT)
        self.czCrushStressTabValueKw.setColumnType(1, AFXTABLE_TYPE_FLOAT)
        self.czVelocityKw = AFXBoolKeyword(self.cmd, 'isCzVelocityDefined', AFXBoolKeyword.TRUE_FALSE, False, False)
        self.czVelocityTabKw = AFXTableKeyword(self.cmd, 'velocityTableVal', False)
        self.czVelocityTabKw.setColumnType(0, AFXTABLE_TYPE_FLOAT)
        self.czVelocityTabKw.setColumnType(1, AFXTABLE_TYPE_FLOAT)
        self.setModal(TRUE)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        import czoneParametersDB
        self.cmd.setKeywordValuesToDefaults()
        return czoneParametersDB.CzoneParametersDB(self, self.actionKw, self.materialKw)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        # Try to set the appropriate radio button on. If the user did
        # not specify any buttons to be on, do nothing.
        #
        for kw1,kw2,d in self.radioButtonGroups.values():
            
            try:
                value = d[ kw1.getValue() ]
                kw2.setValue(value)
            except:
                pass
        return True
       

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        

        if self.materialfromCzoneDefKw.getValue() == '':
            showAFXErrorDialog(self.getCurrentDialog(), 'Material must be selected.')
            return FALSE
        if self.GroupBoxKw1.getValue() == CONSTANT.getId():
            if not self.crushStressConstValueKw.getValue() >= 0:
                showAFXErrorDialog(self.getCurrentDialog(), 'CZone Crush Stress value is incorrect.\nPlease re-enter the value.')
                return FALSE
        if self.GroupBoxKw1.getValue() == TABULAR.getId():
            if self.czCrushStressTabValueKw.getValues() == '':
                showAFXErrorDialog(self.getCurrentDialog(), 'CZone Crush Stress is not defined.')
                return FALSE
            else:
                i=1
		tableCols=2+self.FieldVariableDepKw.getValue()
		if self.tempTableKw.getValue()==True:
		   tableCols+=1

                for row in range(self.czCrushStressTabValueKw.getNumRows()):
                    #for column in range( self.czCrushStressTabValueKw.getNumColumns(row)):
                    for column in range(tableCols):
                        try:
                            val=float(self.czCrushStressTabValueKw.getValue(row,column))
                            if val < 0.0:
                                 showAFXErrorDialog(self.getCurrentDialog(),'CZone Crush Stress definition of Row %d, Column %d is negative.\n' %(row+1, column+1))
                                 return FALSE
                        except:
                            if self.czCrushStressTabValueKw.getValue(row,column)=='':
                               self.czCrushStressTabValueKw.setValue(row, column, '0')
                            else:
                                showAFXErrorDialog(self.getCurrentDialog(),'CZone Crush Stress definition of Row %d, Column %d is incomplete or incorrect.\n' %(row+1, column+1))
                                return FALSE
                            '''	
                              if value < 0.0 :
                                if column == 0:
                                    showAFXErrorDialog(self.getCurrentDialog(),'CZone Crush Stress definition is incomplete or incorrect.\n "%s" in "Crush Stress" column should be corrected.' % value)
                                    return FALSE
                                elif column == 1:
                                    showAFXErrorDialog(self.getCurrentDialog(),'CZone Crush Stress definition is incomplete or incorrect.\n "%s" in "Material Angle" column should be corrected.' % value)
                                    return FALSE
                                else:
                                    showAFXErrorDialog(self.getCurrentDialog(),'CZone Crush Stress definition is incomplete or incorrect.%d \n' %value)
                                    return FALSE
                           except:
                            exceptionvalue = self.czCrushStressTabValueKw.getValue(row,column)
                            print 'value==',exceptionvalue

                            if column == 0:
                                showAFXErrorDialog(self.getCurrentDialog(),'CZone Crush Stress definition is of incorrect type.\n "%s" in "Crush Stress" column should be corrected.' % exceptionvalue)
                                return FALSE
                            elif column == 1:
                                showAFXErrorDialog(self.getCurrentDialog(),'CZone Crush Stress definition is of incorrect type.\n "%s" in "Material Angle" column should be corrected.' % exceptionvalue)
                                return FALSE        
                            else:
                                showAFXErrorDialog(self.getCurrentDialog(),'CZone Crush Stress definition is incomplete or incorrect.%s ***\n' %exceptionvalue)
                                return FALSE   
                            '''
        if self.czVelocityKw.getValue():
            if self.czVelocityTabKw.getValues() == '':
                showAFXErrorDialog(self.getCurrentDialog(),'CZone Velocity is not defined.')
                return FALSE
            else:
                for row in range(self.czVelocityTabKw.getNumRows()):
                    for column in range (self.czVelocityTabKw.getNumColumns(row)):
                        try:
                            value = float(self.czVelocityTabKw.getValue(row,column))

                            if value < 0.0 :
                                if column == 0:
                                    showAFXErrorDialog(self.getCurrentDialog(),'CZone Velocity definition is incomplete or incorrect.\n "%s" in "Scaling factor" column should be corrected.' % value)
                                    return FALSE
                                if column == 1:
                                    showAFXErrorDialog(self.getCurrentDialog(),'CZone Velocity definition is incomplete or incorrect.\n "%s" in "Relative velocity" column should be corrected.' % value)
                                    return FALSE
                        except:
                            exceptionvalue = self.czVelocityTabKw.getValue(row,column)
                            if column == 0:
                                showAFXErrorDialog(self.getCurrentDialog(),'CZone Velocity definition is of incorrect type.\n "%s" in "Scaling factor" column should be corrected.' % exceptionvalue)
                                return FALSE
                            if column == 1:
                                showAFXErrorDialog(self.getCurrentDialog(),'CZone Velocity definition is of incorrect type.\n "%s" in "Relative velocity" column should be corrected.' % exceptionvalue)
                                return FALSE


        return True


