"""
Kernel module for the CAX-CCL Plugin
"""
from abaqus import *
from abaqusConstants import *
from customKernel import *
from customKernel import CommandRegister, RepositorySupport
import testUtils,osutils
from symbolicConstants import *
from types import *
from job import JobType
import methodCallback

###~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


def findKeywordPosUsing2Strings(modelName, searchString1, searchString2):
    """
    
    Method to find the index of a keyword search string in the
    sieBlocks object.
    Attributes:
        modelName - Name of the model.
        searchString - Keyword whose occurence is to be found.
    
    """
    searchIndex = -1
    keyIndex = -1
    keywordBlock = mdb.models[modelName].keywordBlock
    sieBlocksObj= keywordBlock.sieBlocks
    for keyBlocks in sieBlocksObj:
        keyIndex = keyIndex + 1
        blocks = keyBlocks.split('\n')
        for block in blocks:
            if block.lower().startswith(searchString1):
                if not block.lower().find(searchString2.lower()) == -1:
                    searchIndex = keyIndex
                    break
    return searchIndex
###~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


def findKeywordPosforSurfInt(modelName, searchString1, searchString2):
    """
    
    Method to find the index of a keyword search string in the
    sieBlocks object.
    Attributes:
        modelName - Name of the model.
        searchString - Keyword whose occurence is to be found.
    
    """
    searchIndex = -1
    keyIndex = -1
    keywordBlock = mdb.models[modelName].keywordBlock
    sieBlocksObj= keywordBlock.sieBlocks
    for keyBlocks in sieBlocksObj:
        keyIndex = keyIndex + 1
        blocks = keyBlocks.split('\n')
        for block in blocks:
            if block.lower().startswith(searchString1):
                block = block.replace(' ','')
                keyworditem,nameitem = block.split('name=')
                if not nameitem.lower().find(searchString2.lower()) == -1:
                    beforename, name, aftername = nameitem.lower().partition(searchString2.lower())
                    if aftername.startswith(',') or aftername == '':
                        searchIndex = keyIndex
                        break
    return searchIndex
###~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    

    
def findKeywordMultiplePos(modelName, searchString1, searchString2):
    """
    
    Method to find the index of a keyword search string in the
    sieBlocks object.
    Attributes:
        modelName - Name of the model.
        searchString - Keyword whose occurence is to be found.
    
    """
    searchIndex = []
    keyIndex = -1
    keywordBlock = mdb.models[modelName].keywordBlock
    sieBlocksObj= keywordBlock.sieBlocks
    for keyBlocks in sieBlocksObj:
        keyIndex = keyIndex + 1
        blocks = keyBlocks.split('\n')
        for block in blocks:
            if block.lower().startswith(searchString1):
                if not block.lower().find(searchString2.lower()) == -1:
                    searchIndex.append(keyIndex)
    return searchIndex

###~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    

def removeCzoneSpecificKeywords(keywordBlock, modelName):

    """    
    Method to check & remove for CZone specific keywords. If found replace with ''        
    """
    i=0
    for sb in keywordBlock.sieBlocks:
        
        if not sb.lower().find('*user defined field') == -1:
            if not sb.find(',CZONE') == -1:
                keywordBlock.replace(i,'')
        elif sb.lower().startswith('*depvar,'):
            if not sb.find('DELETE=1') == -1:
                keywordBlock.replace(i,'')
        elif sb.lower().startswith('*surface behavior, pressure-overclosure=crush stress'):
            #if not sb.find('CZONE') == -1:
            #    if not sb.find('depvar=127') == -1:
            #        existCustomProp = 0
            #        for intprop in mdb.customData.czone.models[modelName].surfaceIntDefinition.keys():
            #            if not sb.find(intprop) == -1:
            #                existCustomProp = 1
            #        if existCustomProp == 0:
            #            keywordBlock.replace(i,'')       
	    keywordBlock.replace(i+1,'')     
	    keywordBlock.replace(i,'')
        elif sb.lower().startswith('*surface interaction'):
            existIntProp = 0
            for intprop in mdb.models[modelName].interactionProperties.keys():
                if not sb.find(intprop) == -1:
                    existIntProp = 1
            if existIntProp == 0:
                keywordBlock.replace(i,'')
        elif sb.lower().startswith('*crush stress'):
            keywordBlock.replace(i,'')
        elif sb.lower().startswith('*crush stress velocity factor'):
            keywordBlock.replace(i,'')
        elif sb.lower().startswith('** define'):
            if not sb.find('CZONE') == -1:
                keywordBlock.replace(i,'')
        elif not sb.find('** CZONE') == -1:
            if not sb.find('DEFINITIONS') == -1:
                keywordBlock.replace(i,'')
            if not sb.find('PREDEFINED') == -1:
                if not sb.find('FIELDS') == -1:
                    keywordBlock.replace(i,'')
        elif sb.lower().startswith('*end czone'):
            keywordBlock.replace(i,'')
        elif sb.startswith('*INITIAL CONDITIONS,TYPE=FIELD,VARIABLE'):
            keywordBlock.replace(i,'')
        elif sb.lower().startswith('*contact output\nczastate'):
                keywordBlock.replace(i,'')
        elif sb.lower().startswith('*contact output'):
             if not sb.lower().find('czastate') == -1:
                if keywordBlock.sieBlocks[i-1]!= '*Output, field, variable=PRESELECT':
                   #keywordBlock.replace(i,sb[:len(sb)-9])
                   keywordBlock.replace(i,sb.lower().replace(', czastate',''))
                else:
                   keywordBlock.replace(i,'')
        #elif sb.lower().startswith('*conflicts'):
        #   keywordBlock.replace(i,'')
        
        i +=1        

    return 1
###~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    

def removeConflicts(index,  modelName):
    
    """    
    Method to check & remove *Conflicts that are generated due to Surf Int. If found replace with ''        
    """
    keywordBlock = mdb.models[modelName].keywordBlock
    sieB = keywordBlock.sieBlocks
    ### Check if the input file has keywords below the given index. If not reduce the range
    rangeMax = 0
    try: 
        while rangeMax <10:
            Checkexists = sieB[index+rangeMax]
            rangeMax += 1
    except:
        pass
    
    for i in range (index-10, index + rangeMax,1):
        if sieB[i].lower().startswith('*conflicts'):
            keywordBlock.replace(i,'')
         
    return 1
###~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    


def writeCzoneToKeywordBlock(callingMethod,args,keywordArgs,userData):
    
##    """
##    Method checks if the model has any unhandled features.

##    """
    
    if not hasattr(callingMethod.caller, 'model'):
        return
    try:
        modelName = callingMethod.caller.model
        jobName   = callingMethod.caller.name

    except:
        errorMessageFunc(
            'Error: The callback procedure did not return a model name.\n')

    if hasattr(mdb.customData, 'czone'):

        
        if not modelName in sorted(mdb.models.keys()):
            errorMessage =  ':Model Name "%s" sent to callback method ' \
                            'is not available in the mdb.\n' %(modelName)
            errorMessageFunc(errorMessage)
        
## Check if New Materials, Surface Interactions or Sets were Deleted in the model and give error if so.
    if hasattr(mdb.customData, 'czone'):
        if mdb.customData.czone.models.has_key(modelName):
            for material in mdb.customData.czone.models[modelName].tsaiwuDamage.keys():
                if not material in mdb.models[modelName].materials.keys():
                    error1 = "The material " + '"' + material + '"' + " with Tsai-Wu damage definition is not available in the model " + '"' + modelName + '".\n'
                    error2 = "Please remove the material's damage definition from CZone before continuing."
                    error = error1 + error2
                    raise ValueError, error
            for material in mdb.customData.czone.models[modelName].czoneDefinition.keys():
                if not material in mdb.models[modelName].materials.keys():
                    error1 = "The material " + '"' + material + '"' + " with CZone Property definition is not available in the model " + '"' + modelName + '".\n'
                    error2 = "Please remove the definition from CZone before continuing."
                    error = error1 + error2
                    raise ValueError, error
            for surfInt in mdb.customData.czone.models[modelName].surfaceIntDefinition.keys():
                if not surfInt in mdb.models[modelName].interactionProperties.keys():
                    error1 = "The Surface Interaction " + '"' + surfInt + '"' + " definition is not available in the model " + '"' + modelName + '".\n'
                    error2 = "Please remove the Surface Interaction definition from CZone before continuing."
                    error = error1 + error2
                    raise ValueError, error
            for set in mdb.customData.czone.models[modelName].initialCondDefinition.keys():
                if not set in mdb.models[modelName].rootAssembly.sets.keys():
                    error1 = "The Assembly set " + '"' + set + '"' + " is not available in the model " + '"' + modelName + '".\n'
                    error2 = "Please remove the Set definition from CZone before continuing."
                    error = error1 + error2
                    raise ValueError, error
            if 'IsCzDeactivated' in mdb.customData.czone.models[modelName]._getAttributeNames():
                deactivate = mdb.customData.czone.models[modelName].IsCzDeactivated
            else:
                deactivate = FALSE
            if deactivate:
                print 'CZone Plug-in is deactivated. Only Tsai-Wu damage definitions will be written to the input file.'
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# START WRITING THE Custom Data to the Keyword Block

            model = mdb.models[modelName]
            keywordBlock = model.keywordBlock
            keywordBlock.synchVersions(storeNodesAndElements=FALSE)
            removeCzoneSpecificKeywords(keywordBlock, modelName)
            sieB = keywordBlock.sieBlocks
            
            keywordIndex = -1
            tsaiwuMatToProcess = mdb.customData.czone.models[modelName].tsaiwuDamage.keys()
            for material in mdb.customData.czone.models[modelName].tsaiwuDamage.keys():
                members = mdb.models[modelName].materials[material].__members__
                numItems = len(members) - 3 # Number of keyword items in the *Material - example *density, etc. The "3" there are 3 members like "Description, etc that are not keywords.
                for keyword in sieB:
                    Block = keyword
                    if Block.lower().startswith('*material'):
                        if not Block.lower().find(material.lower()) == -1:
                            index = findKeywordPosUsing2Strings(modelName,'*material',material)
                            tsaiwuCustomdata=mdb.customData.czone.models[modelName].tsaiwuDamage[material]
                            text = '*DEPVAR, DELETE=1\n'
                            text += '3\n'
                            text += '1, STATUS, "Status of the integration point"\n'
                            text += '2, TSAIWUCRT, "TSAI-WU Damage initiation criteria"\n'
                            text += '3, SDEG, "Scalar stiffness degradation"\n'
                            text +=  '*USER DEFINED FIELD, properties=16,CZONE\n'
                            text += '**TSAI-WU FAILURE STRESSES\n'
                            text += str(tsaiwuCustomdata.TensileStressFiberDir) + ',' + str(tsaiwuCustomdata.CompStressFiberDir) + ',' + str(tsaiwuCustomdata.TensileStressTransDir) + ',' + str(tsaiwuCustomdata.CompStressTransDir) + ',' + str(tsaiwuCustomdata.ShearStrengthXYplane) + ',' + str(tsaiwuCustomdata.InteractionTerm) + '\n'
                            #text += str(tsaiwuCustomdata.YoungsModin1Dir) + ',' + str(tsaiwuCustomdata.YoungsModin2Dir) + ',' + str(tsaiwuCustomdata.PoissonRatio) + ',' + str(tsaiwuCustomdata.ShearModulus) + ','  + str(tsaiwuCustomdata.EnergyPerUnitArea) + '\n'
                            
                            try:
                            	e=mdb.models[modelName].materials[material].elastic
                            except:
                            	raise AttributeError, 'Material-' + material + ' is not elastic'
                            if e.type==ISOTROPIC:
                            	val=e.table[0]
                            	YoungsModin1Dir=val[0]
                            	YoungsModin2Dir=val[0]
                            	PoissonRatio=val[1]
                            	ShearModulus=YoungsModin1Dir*0.5/(1+PoissonRatio)
                            elif e.type==LAMINA:
                            	val=e.table[0]
                            	YoungsModin1Dir=val[0]
                            	YoungsModin2Dir=val[1]
                            	PoissonRatio=val[2]
                            	ShearModulus=val[3]
                            else:
                            	raise ValueError, 'Material-' + material + ' should be either ISOTROPIC OR LAMINA'
                            
                            text += str(YoungsModin1Dir) + ',' + str(YoungsModin2Dir) + ',' + str(PoissonRatio) + ',' + str(ShearModulus) + ','  + str(tsaiwuCustomdata.EnergyPerUnitArea) + '\n'
                            #for material in mdb.customData.czone.models[modelName].czoneDefinition.keys():
                            for tempRemove in [1]:
                            	czoneCustomData=mdb.customData.czone.models[modelName].czoneDefinition[material]
                                text += '** DEFINE CZONE MATERIAL PROPERTIES\n'
                                
                                
                                if czoneCustomData.CrushStressDefType == CONSTANT:
                                    text += '*CRUSH STRESS\n'
                                    crushStressConst = str(czoneCustomData.CrushStressConstVal)
                                    for row in [0,90,180,270,360]:
                                         text += str(crushStressConst) + ',' + str(row) + '\n'
                                else:
                                   #for row in czoneCustomData.CrushStressTableVal:
                                   #        text += str(row[0]) + ',' + str(row[1]) + '\n'
				   
				   try: #Check if FieldVariableDep keyword is available (older plugin versions don't)
				      if czoneCustomData.FieldVariableDep > 0:   
				   	  text += '*CRUSH STRESS, DEPENDENCIES='+cstr(czoneCustomData.FieldVariableDep)+'\n'
				      else:
				          text += '*CRUSH STRESS\n'
				   except AttributeError:
				      czoneCustomData.FieldVariableDep = 0
				      czoneCustomData.tempTable = False
				   
				   #czoneCustomData.tempTable = False   
				   if czoneCustomData.tempTable==True:
				     for row in czoneCustomData.CrushStressTableVal:
					for column in row:
					   text += str(column) + ','
					text = text[:len(text)-1] +'\n' # Remove the last comma
				   else:
				     for row in czoneCustomData.CrushStressTableVal:
					for i in range(len(row)):
					   if i == 2:
					      text += ',' #For temperature definition
					   text += str(row[i]) + ','                                 
					text = text[:len(text)-1] +'\n'
                                
                                if czoneCustomData.IsCzVelocityDefined:
                                	text += '*CRUSH STRESS VELOCITY FACTOR\n'
                                	for row in czoneCustomData.VelocityTableVal:
                                	   text += str(row[0]) + ',' + str(row[1]) + '\n'
                                
                                #keywordBlock.insert(insertIndex, text)
                            
                           
                            if not index == -1:
                                keywordBlock.insert(position = index + numItems, text = text)
                            else:
                                error1 = "The Plug-in is not able to find the material " + '"' + material + '"' + " in the input file.\n"
                                error2 = "Please report this error."
                                error = error1 + error2
                                raise ValueError, error
                            tsaiwuMatToProcess.remove(material)
                            break
            
            if not deactivate:
                for material in mdb.customData.czone.models[modelName].czoneDefinition.keys():
                    if not material in mdb.customData.czone.models[modelName].tsaiwuDamage.keys():
                        if material.lower().startswith('abq_ply_fabric'):

                            members = mdb.models[modelName].materials[material].__members__
                            if 'userMaterial' in members:
                                numItems = len(members) - 4 # Number of keyword items in the *Material - example *density, etc, The 2 there is specific to non keyword items for usermaterial
                                for keyword in sieB:
                                    Block = keyword
                                    if Block.lower().startswith('*material'):
                                        if not Block.lower().find(material.lower()) == -1:
                                            index = findKeywordPosUsing2Strings(modelName,'*material',material)
                                            
                                            text =  '*USER DEFINED FIELD, properties=1,CZONE\n'
                                            text += '16\n'
                                            if not index == -1:
                                                keywordBlock.insert(position = index + numItems, text = text)
                                            else:
                                                error1 = "The Plug-in is not able to find the material " + '"' + material + '"' + " in the input file.\n"
                                                error2 = "Please report this error."
                                                error = error1 + error2
                                                raise ValueError, error
                        else:
                            
                            members = mdb.models[modelName].materials[material].__members__
                            numItems = len(members) - 3 # Number of keyword items in the *Material - example *density, etc. The 3 there is specific to non keyword items
                            
                            
                            if not 'hashinDamageInitiation' in mdb.models[modelName].materials[material].__members__:
                            	#Loop through starting from: *material, name= <'material'>
                            	#till: *damage is found or *material ends. If *damage is found, no warning is issued, else a warning message is issued
                            	for i in range(len(sieB)):
                            	   keyword=sieB[i]
                            	   damageKeywordFlag=0
                            	   if keyword.lower().startswith('*material, name='+material.lower()):
                            	     #If material='Material-1', make sure the check is done on only 'material-1' and not on 'material-1_xxx' etc.
                            	     #',' is checked because there can be optional parameters defined for *material keyword
                            	     if keyword.lower()=='*material, name='+material.lower() or keyword.lower().startswith('*material, name='+material.lower() +','):
                            	       for j in range(i+1,len(sieB)):
                            	          keyword_temp=sieB[j].lower()
                            	          if keyword_temp.startswith('*damage'):
                            	          	damageKeywordFlag=1
                            	          	break
                            	          elif keyword_temp.startswith('*material') or keyword_temp.startswith('*surface') or keyword_temp.startswith('*section') \
                            	          or keyword_temp.startswith('*step'):
                            	          	damageKeywordFlag=0
                            	          	break
                            	       if damageKeywordFlag == 0:
                            	          reply = getWarningReply(message='Warning! The CZone material "'+ material+'" has no damage defined.\n Proceed?', buttons=(YES,NO))
				          if reply == NO:
   					  	raise ValueError, 'Input file not written' 
   					  break
                            	       else:
   					  break
                            #if not 'hashinDamageInitiation' in mdb.models[modelName].materials[material].__members__:
                            #    error1 = "The CZone material " + '"' + material + '"' + " has no damage defined.\n"
                            #    error2 = "Please define damage for the material."
                            #    error = error1 + error2
                            #    raise ValueError, error
                            for keyword in sieB:
                                Block = keyword
                                if Block.lower().startswith('*material'):
                                    if not Block.lower().find(material.lower()) == -1:
                                        index = findKeywordPosUsing2Strings(modelName,'*material',material)
                                        #text = '*DEPVAR, DELETE=1\n'
                                        #text += '1\n'
                                        #text +=  '*USER DEFINED FIELD, properties=1,CZONE\n'
                                        #text += '1\n'
                                        czoneCustomData=mdb.customData.czone.models[modelName].czoneDefinition[material]
				        text = '** DEFINE CZONE MATERIAL PROPERTIES\n'

					if czoneCustomData.CrushStressDefType == CONSTANT:
					    text += '*CRUSH STRESS\n'
					    crushStressConst = str(czoneCustomData.CrushStressConstVal)
					    for row in [0,90,180,270,360]:
						 text += str(crushStressConst) + ',' + str(row) + '\n'
					else:
					   #for row in czoneCustomData.CrushStressTableVal:
					   #        text += str(row[0]) + ',' + str(row[1]) + '\n'

					   try: #Check if FieldVariableDep keyword is available (older plugin versions don't)
					     if czoneCustomData.FieldVariableDep > 0:   
				   	  	text += '*CRUSH STRESS, DEPENDENCIES='+str(czoneCustomData.FieldVariableDep)+'\n'
					     else:
				          	text += '*CRUSH STRESS\n'
					   except AttributeError:
				      		czoneCustomData.FieldVariableDep = 0
				      		czoneCustomData.tempTable = False
				      		text += '*CRUSH STRESS\n'
				      
					   if czoneCustomData.tempTable==True:
					     for row in czoneCustomData.CrushStressTableVal:
						for column in row:
						   text += str(column) + ','
						text = text[:len(text)-1] +'\n' # Remove the last comma
					   else:
					     for row in czoneCustomData.CrushStressTableVal:
						for i in range(len(row)):
						   if i == 2:
						      text += ',' #For temperature definition
						   text += str(row[i]) + ','                                 
						text = text[:len(text)-1] +'\n'

					if czoneCustomData.IsCzVelocityDefined:
						text += '*CRUSH STRESS VELOCITY FACTOR\n'
						for row in czoneCustomData.VelocityTableVal:
						   text += str(row[0]) + ',' + str(row[1]) + '\n'

                                        if not index == -1:
                                            keywordBlock.insert(position = index, text = text)
                                        else:
                                            error1 = "The Plug-in is not able to find the material " + '"' + material + '"' + " in the input file.\n"
                                            error2 = "Please report this error."
                                            error = error1 + error2
                                            raise ValueError, error
            
# Surf Int is handled slightly differently. If deactivate = TRUE, a dummy Surf Int is written. Else CZone Surf Int is written.                
            
            if len(mdb.customData.czone.models[modelName].surfaceIntDefinition.keys()) > 0 :
                for intProp in mdb.customData.czone.models[modelName].surfaceIntDefinition.keys():
                    index = findKeywordPosforSurfInt(modelName,'*surface interaction', intProp)
                    insertIndex = index 

                    if index == -1:
                        if not len(mdb.models[modelName].steps.keys()) <= 1:
                            index = findKeywordPosUsing2Strings(modelName,'** step:',(mdb.models[modelName].steps.keys())[1])
                            insertIndex = index  - 1   # Inserting one index above the *Step definitions
                        else:
                            index = len(keywordBlock.sieBlocks)
                            insertIndex = index - 1   # Inserting at the end of the input file.
                        if index == -1:
                            raise ValueError, 'Unknown error while writing Surface Interaction.\n Please report the error.'
                    
                    if not deactivate and len(mdb.customData.czone.models[modelName].czoneDefinition.keys()) > 0:
                        surfIntCustomData=mdb.customData.czone.models[modelName].surfaceIntDefinition[intProp]
                        text =  '*Surface Interaction,name=' + intProp + '\n'
                        text += '*Surface Behavior, pressure-overclosure=Crush Stress\n'
                        try:
                          if surfIntCustomData.CBAngle!=60:
                            text += str(surfIntCustomData.CBAngle) +'\n'
                        except AttributeError:
                          surfIntCustomData.CBAngle=60
                        text +=  '*Friction\n'+str(surfIntCustomData.coefficientOfFriction) + '\n'
                        keywordBlock.replace(insertIndex, text)
                    else:
                        text =  '*Surface Interaction,name789=' + intProp + '\n'
                        keywordBlock.replace(insertIndex, text)
                    if not index == -1:
                        removeConflicts(insertIndex, modelName)
            

            if not deactivate:
                if len(mdb.customData.czone.models[modelName].czoneDefinition.keys()) > 0:
                    if not len(mdb.models[modelName].steps.keys()) <= 1:
                        index = findKeywordPosUsing2Strings(modelName,'** step:',(mdb.models[modelName].steps.keys())[1]) 
                        insertIndex = index  - 1   # Inserting one index above the *Step definitions
                    else:
                        index = len(keywordBlock.sieBlocks)
                        insertIndex = index - 1   # Inserting at the end of the input file.

                    if index == -1:
                        raise ValueError, 'Unknown Error. Please report the error'
                    
                    
                    
                    if not len(mdb.models[modelName].steps.keys()) <= 1:
			    index = findKeywordMultiplePos(modelName,'*output,','field')
			    indexTemp = findKeywordMultiplePos(modelName,'*contact',' output')
			    #print '\n\nindex==',index
			    #print 'indexTemp==',indexTemp
			    index.sort()
			    indexTemp.sort()
			    kyBlock=mdb.models[modelName].keywordBlock
			    sieBl=kyBlock.sieBlocks
			    if index == []:
				index = findKeywordMultiplePos(modelName,'*end','step')
				index=[ind-1 for ind in index]
				#print 'insertIndex==',insertIndex
				if index == []:
				    raise ValueError, 'Unknown error while writing Field Outputs.\n Please report the error.'
				else:
					text = '*contact output\n'
					text += 'CZASTATE\n'
					i=0
					for insertIndex in index:
						#insertIndex = index[0]
						keywordBlock.insert(insertIndex+i, text)
						i+=1 
			    elif indexTemp!=[]:
				#find if '*contact output' was for field or history output
				#Assume there can be a maximum of 3 elements bt '*output, field' and '*contact output'
				indexHis = findKeywordMultiplePos(modelName,'*output,','history')
				indexEndStep = findKeywordMultiplePos(modelName,'*end','step')
				indexStep = findKeywordMultiplePos(modelName,'*step,','name')
				#print 'indexStep,indexHis,indexEndStep,==',indexStep,indexHis,indexEndStep
				if len(indexStep) != len(indexEndStep):
				  raise ValueError, 'Unknown error while writing Field Outputs.\n Please report the error.'
				check=0
				indexWithPreselect=[]
				for nstep in range(len(indexStep)):
				
				    kyBlock=mdb.models[modelName].keywordBlock
			    	    sieBl=kyBlock.sieBlocks
				    if indexHis[nstep] > indexStep[nstep] and indexHis[nstep] < indexEndStep[nstep]:
				       check=indexHis[nstep]
				    else:
				       # There is no *output history in this step
				       check=indexEndStep[nstep]
				    
				    #print 'check==',check
				    count=0
				    for nindexTemp in indexTemp:
				        if nindexTemp > indexEndStep[nstep]:
				           break
				        if nindexTemp > indexStep[nstep] and nindexTemp < check:
				           #print 'Appending here=='+ sieBl[nindexTemp] 
				           #print 'see here==', sieBl 
				           #print 'nindexTemp,indexStep[nstep],check==',nindexTemp,indexStep[nstep],check
				           #if mdb.models[modelName].steps.values()[nstep+1].fieldOutputRequestStates['F-Output-1'].variables != 'PRESELECT':
				           #   text=sieBl[nindexTemp] +', CZASTATE'
				           #   kyBlock.replace(nindexTemp, text)
				           #   count +=1
				           #else:
				           #   count = 0
				           indexWithPreselect.append(nindexTemp)
				           count +=1
				    	elif nindexTemp > check and nindexTemp < indexEndStep[nstep]:
				    	   #*contact output corresponds to history output
				    	   pass
				    	elif nindexTemp > indexEndStep[nstep]:
				    	   #*contact output is for next step
				           count=1
				           break
				    if count == 0:
				    	#print 'Add new CZASTATE o/p here '+ sieBl[indexStep[nstep]]
				    	if index[nstep] < indexEndStep[nstep]:
				    	   indexWithPreselect.append(index[nstep])
				    	else:
				    	   indexWithPreselect.append(indexEndStep[nstep]-1)
				    	   
				
				text = '*contact output\n'
				text += 'CZASTATE\n'				    
				i=0
				#print 'indexWithPreselect==',indexWithPreselect
				for indP in indexWithPreselect:
					kyBlock.insert(indP+i, text)
					i+=1 
								
				'''
				i=0
				j=0
				sieB=model.keywordBlock.sieBlocks
				indexWithPreselect=[]
				for ind2 in indexTemp:
				   ind=index[j]
				   if ind2 <= ind+3:
					j+=1
					s=sieB[ind2]
					text=s +',CZASTATE'
					keywordBlock.replace(ind2, text)
				   else:
					#This step doesn't have '*contact output' defined or has \
					#'*Output, field, variable=PRESELECT'
					indexWithPreselect.append(ind)
				text = '*contact output\n'
				text += 'CZASTATE\n'
				i=0
				print 'indexWithPreselect==',indexWithPreselect
				for indP in indexWithPreselect:
					keywordBlock.insert(indP+i, text)
					i+=1 
				
				sieBl=mdb.models[modelName].keywordBlock.sieBlocks
				i=0
				indexWithPreselect=[]
				while i < len(sieBl):
				     keyword=sieB[i]
				     if keyword.lower().startswith('*step'):
				        i=i+1
				        while sieB[i].lower().startswith('*output, field'):
				        
				        
				        
				        
				        stepNameTemp=keyword.split('name=')[-1]
				        stepName=stepNameTemp.split(',')[0]
				        print 'Stepname==',stepName
				        for var in m.steps[stepName].fieldOutputRequestStates.values()
				           if var.variables=='PRESELECT':
				           	indexWithPreselect.append(i)
				     else:
				        i+=1
                            	     '''
			    else:
				text = '*contact output\n'
				text += 'CZASTATE\n'
				i=0
				for insertIndex in index:
					#insertIndex = index[0]
					keywordBlock.insert(insertIndex+i, text)
					i+=1 
				#print 'insertIndex here==',insertIndex
				    


				                        
                    
                    #text = '*END CZONE\n'
                    
                    #keywordBlock.insert(insertIndex, text)
                    #for material in mdb.customData.czone.models[modelName].czoneDefinition.keys():
                    #    czoneCustomData=mdb.customData.czone.models[modelName].czoneDefinition[material]
                        #text = '*CZONE MATERIAL,MATERIAL=' + str(material) + '\n'
                        
                    #    text = '*CRUSH STRESS\n'
                    #    if czoneCustomData.CrushStressDefType == CONSTANT:
                    #        crushStressConst = str(czoneCustomData.CrushStressConstVal)
                    #        for row in [0,90,180,270,360]:
                    #             text += str(crushStressConst) + ',' + str(row) + '\n'
                    #    else:
			#     for row in czoneCustomData.CrushStressTableVal:
                    #             text += str(row[0]) + ',' + str(row[1]) + '\n'
                        
                    #    if czoneCustomData.IsCzVelocityDefined:
                    #    	text += '*CRUSH STRESS VELOCITY FACTOR'
                    #    	for row in czoneCustomData.VelocityTableVal:
                    #    	   text += str(row[0]) + ',' + str(row[1]) + '\n'
                        
                        #if czoneCustomData.CrushStressDefType == CONSTANT:
                        #    text += '*CZONE CRUSH STRESS, TYPE=CONSTANT\n'
                        #    text += str(czoneCustomData.CrushStressConstVal) + '\n'

                        #else:
                        #    text += '*CZONE CRUSH STRESS, TYPE=TABULAR\n'
                        #    for row in czoneCustomData.CrushStressTableVal:
                        #        text += str(row[0]) + ',' + str(row[1]) + '\n'
                        #if czoneCustomData.IsCzVelocityDefined:
                        #    text += '*CZONE VELOCITY\n'
                        #    for row in czoneCustomData.VelocityTableVal:
                        #        text += str(row[0]) + ',' + str(row[1]) + '\n'
                     #   keywordBlock.insert(insertIndex, text)
                        
                    #text =  "** ----------------------------------------------------------------\n"
                    #text += "**\n"
                    #text += "** CZONE DEFINITIONS\n"
                    #text += "**\n"
                    #text += "*CZONE\n"
                    #keywordBlock.insert(insertIndex ,text)
            
                    
            #if not deactivate and len(mdb.customData.czone.models[modelName].czoneDefinition.keys()) > 0:

                #if len(mdb.customData.czone.models[modelName].initialCondDefinition.keys()) >0:
                #    index = findKeywordPosUsing2Strings(modelName,'** czone','definitions')

                #    if index == -1:
                #        raise ValueError, 'Unknown error while writing Predefined Field.\nPlease report this error.'
                #    insertIndex = index - 1    # Inserting one index above the *Step definitions
                #    for set in mdb.customData.czone.models[modelName].initialCondDefinition.keys():
                #        czoneCustomData=mdb.customData.czone.models[modelName].initialCondDefinition[set]
                #        text = '*INITIAL CONDITIONS,TYPE=FIELD,VARIABLE=1\n'
                #        text +=  set + ',0.0\n'
                #        text +=  '*INITIAL CONDITIONS,TYPE=FIELD,VARIABLE=2\n'
                #        text +=  set + ',1.0\n'
                #        text +=  '*INITIAL CONDITIONS,TYPE=FIELD,VARIABLE=3\n'
                #        text +=  set + ',1.0\n'
                #        text +=  '*INITIAL CONDITIONS,TYPE=FIELD,VARIABLE=4\n'
                #        text +=  set + ',0.0\n'
                #        keywordBlock.insert(insertIndex ,text)
                #    text =  "** ----------------------------------------------------------------\n"
                #    text += "**\n"
                #    text += "** CZONE PREDEFINED FIELDS\n"
                #    text += "**\n"
                    
                    #keywordBlock.insert(insertIndex, text)
                    #if not len(mdb.models[modelName].steps.keys()) <= 1:
                    #    for set in mdb.customData.czone.models[modelName].initialCondDefinition.keys():
                    #        index = findKeywordMultiplePos(modelName,'** output','requests')
                    #        index.sort()
                    #        if index == []:
                    #            index = findKeywordMultiplePos(modelName,'*end','step')
                    #            if index == []:
                    #                raise ValueError, 'Unknown error while writing Predefined Field.\n Please report the error.'
                    #        text = '*FIELD, VARIABLE=4\n'
                    #        text += set + ', 2.0\n'
                    #        insertIndex = index[0] - 1
                            
                    #        keywordBlock.insert(insertIndex, text)
                            
                            
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def removeCzoneKeywordBlock(callingMethod,args,keywordArgs,userData):
### This method removes the keywords from keyword block after input file is written. This is to prevent potential conflicts in the keyword block.    
    
    if not hasattr(callingMethod.caller, 'model'):
        return
    try:
        modelName = callingMethod.caller.model
        jobName   = callingMethod.caller.name
    except:
        errorMessageFunc(
            'Error: The callback procedure did not return a model name.\n')

    if hasattr(mdb.customData, 'czone'):

        if not modelName in sorted(mdb.models.keys()):
            errorMessage =  ':Model Name "%s" sent to callback method ' \
                            'is not available in the mdb.\n' %(modelName)
            errorMessageFunc(errorMessage)
        
    
    if hasattr(mdb.customData, 'czone'):
        if mdb.customData.czone.models.has_key(modelName):
            model = mdb.models[modelName]
            keywordBlock = model.keywordBlock
            keywordBlock.synchVersions(storeNodesAndElements=FALSE)
            removeCzoneSpecificKeywords(keywordBlock, modelName)
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

def writeTsaiWuParameters(modelName, materialName,tensileStressFiberDir,compStressFiberDir,tensileStressTransDir,compStressTransDir,
                          shearStrengthXYplane,interactionTerm,energyPerUnitArea ):



##    """
##    Method stores the dialog data in session custom data and
##    converts the model into Restart model.
##    Arguements
##        modelName - Current Model Name
##        materialName - material Name 
##        tensileStressFiberDir - Tensile Stress in Fiber Direction
##        compStressFiberDir - Compressive Stress in Fiber Direction 
##        tensileStressTransDir - Tensile Stress in Transverse Direction
##        compStressTransDir - Compressive Stress in Transverse Direction
##        shearStrengthXYplane - Shear Strength in XY plane 
##        interactionTerm - Interaction Term
##        youngsModin1Dir - Young's Modulus in 1 dir
##        youngsModin2Dir - Youngs's Modulus in 2 dir
##        poissonRatio  - Poisson's Ratio
##        shearModulus - Shear Modulus
##        energyPerUnitArea - Energy Per Unit Area

##    """


    if materialName == '':
        raise ValueError, 'Material must be defined.'
### The following checks were commented out at customer's (internal)request. Having this is a good idea. 

##    if tensileStressFiberDir <=0.0:
##        raise ValueError, 'Tensile Stress in Fiber Dir value is incorrect.'
##    if not (type(tensileStressFiberDir) == int or type(tensileStressFiberDir) == float):
##        raise ValueError, 'Tensile Stress in Fiber Dir value type is incorrect.'
##    if compStressFiberDir <= 0.0:
##        raise ValueError,'Compressive Stress in Fiber Direction value is empty or incorrect.'
##    if not (type(compStressFiberDir) == int or type(compStressFiberDir) == float):
##        raise ValueError,'Compressive Stress in Fiber Direction value type is incorrect.'
##    if tensileStressTransDir <= 0.0:
##        raise ValueError,'Tensile Stress in Transverse Direction value is empty or incorrect.'
##    if not (type(tensileStressTransDir) == int or type(tensileStressTransDir) == float):
##        raise ValueError,'Tensile Stress in Transverse Direction value type is incorrect.'
##    if compStressTransDir <= 0.0:
##        raise ValueError,'Compressive Stress in Transverse Direction value is empty or incorrect.'
##    if not (type(compStressTransDir) == int or type(compStressTransDir) == float):
##        raise ValueError,'Compressive Stress in Transverse Direction value type is incorrect.'        
##    if shearStrengthXYplane <= 0.0:
##        raise ValueError,'Shear Strength in XY plane value is empty or incorrect.'
##    if not (type(shearStrengthXYplane) == int or type(shearStrengthXYplane) == float):
##        raise ValueError,'Shear Strength in XY plane value type is incorrect.'
##    if interactionTerm <= 0:
##        raise ValueError,'Interaction Term value is empty or incorrect.'
##    if not (type(interactionTerm) == int or type(interactionTerm) == float):
##        raise ValueError,'Interaction Term value type is incorrect.'
##    if youngsModin1Dir <= 0:
##        raise ValueError,"Young's Modulus in 1 direction value is empty or incorrect."
##    if not (type(youngsModin1Dir) == int or type(youngsModin1Dir) == float):
##        raise ValueError,"Young's Modulus in 1 direction value type is incorrect."
##    if youngsModin2Dir <= 0:
##        raise ValueError,"Young's Modulus in 2 direction value is empty or incorrect."
##    if not (type(youngsModin2Dir) == int or type(youngsModin2Dir) == float):
##        raise ValueError,"Young's Modulus in 2 direction value type is incorrect."
##    if poissonRatio < 0.0 or poissonRatio > 0.5:
##        raise ValueError,"Poisson's Ratio value is empty or incorrect."
##    if not (type(poissonRatio) == int or type(poissonRatio) == float) :
##        raise ValueError,"Poisson's Ratio value type is incorrect."
##    if shearModulus <= 0:
##        raise ValueError,'Shear Modulus value is empty or incorrect.'
##    if not (type(shearModulus) == int or type(shearModulus) == float):
##        raise ValueError,'Shear Modulus value type is incorrect.'
##    if energyPerUnitArea <= 0:
##        raise ValueError,'Energy per unit area value is empty or incorrect.'
##    if not (type(energyPerUnitArea) == int or type (energyPerUnitArea) == float):
##        raise ValueError,'Energy per unit area value type is incorrect.'
    
    if not mdb.customData.czone.models[modelName].tsaiwuDamage.has_key(materialName):
        mdb.customData.czone.models[modelName].TsaiWuDamage(materialName)
    
    if compStressFiberDir >0:
    	compStressFiberDir = compStressFiberDir * -1
    if compStressTransDir >0:
    	compStressTransDir = compStressTransDir * -1
    mdb.customData.czone.models[modelName].tsaiwuDamage[materialName].TensileStressFiberDir=tensileStressFiberDir
    mdb.customData.czone.models[modelName].tsaiwuDamage[materialName].CompStressFiberDir=compStressFiberDir
    mdb.customData.czone.models[modelName].tsaiwuDamage[materialName].TensileStressTransDir=tensileStressTransDir
    mdb.customData.czone.models[modelName].tsaiwuDamage[materialName].CompStressTransDir= compStressTransDir
    mdb.customData.czone.models[modelName].tsaiwuDamage[materialName].ShearStrengthXYplane=shearStrengthXYplane
    mdb.customData.czone.models[modelName].tsaiwuDamage[materialName].InteractionTerm=interactionTerm
    #mdb.customData.czone.models[modelName].tsaiwuDamage[materialName].YoungsModin1Dir=youngsModin1Dir
    #mdb.customData.czone.models[modelName].tsaiwuDamage[materialName].YoungsModin2Dir=youngsModin2Dir
    #mdb.customData.czone.models[modelName].tsaiwuDamage[materialName].PoissonRatio=poissonRatio
    #mdb.customData.czone.models[modelName].tsaiwuDamage[materialName].ShearModulus=shearModulus
    mdb.customData.czone.models[modelName].tsaiwuDamage[materialName].EnergyPerUnitArea=energyPerUnitArea

#    print '                                              '
#    print 'Tsaiwu Damage parameters %s were written.' % materialName

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

def writeCzoneParameters(modelName, materialName, crushStressDefType=CONSTANT, FieldVariableDep=0, tempTable=False, crushStressConstVal = 0,crushStressTableVal=(), isCzVelocityDefined=False, velocityTableVal=()):
    
    
##    """
##    Method stores the dialog data in session custom data and
##    converts the model into Restart model.
##    Arguements
##        modelName - Current Model Name
##        materialName - Material name on which Czone property is defined.
##        crushStressDefType - CZone Crush Stress type - CONSTANT or TABULAR.
##        crushStressConstVal  - CZone Crush Stress Constant Value if crushStressDefType=CONSTANT
##        crushStressTableVal  - CZone Crush Stress Tabular Value if crushStressDefType=TABULAR
##        isCzVelocityDefined - Boolean for CZone Velocity. TRUE if defined.
##        velocityTableVal - CZone Velocity Tabular values.

##    """


    #print 'FieldVariableDep==',FieldVariableDep,crushStressTableVal
    if not mdb.customData.czone.models[modelName].czoneDefinition.has_key(materialName):
        mdb.customData.czone.models[modelName].CzoneDefinition(materialName)

    if materialName == '':
        raise ValueError, 'Material must be defined.'

    if isCzVelocityDefined:
        if velocityTableVal == ():
            raise ValueError, 'CZone Velocity is not defined.'
        else:
            i=1
            for row in velocityTableVal:
                j=1
                for item in row:
                    try:
                        value = float(item)
                        if value < 0.0:
                            if j == 1:
                                raise ValueError,'CZone Velocity definition is incomplete or incorrect.\n "%s" in "Scaling factor" column should be corrected.' % value
                            if j == 2:
                                raise ValueError,'CZone Velocity definition is incomplete or incorrect.\n "%s" in "Relative velocity" column should be corrected.' % value
                    except:
                        if j == 1:
                            raise TypeError,'CZone Velocity definition is of incorrect type.\n "%s" in "Scaling factor" column should be corrected.' % value
                        if j == 2:
                            raise TypeError,'CZone Velocity definition is of incorrect type.\n "%s" in "Relative velocity" column should be corrected.' % value

                    else:
                        try:
                            value = int(item)
                            if value < 0:
                                if j == 1:
                                    raise ValueError,'CZone Velocity definition is incomplete or incorrect.\n "%s" in "Scaling factor" column should be corrected.' % value
                                if j == 2:
                                    raise ValueError,'CZone Velocity definition is incomplete or incorrect.\n "%s" in "Relative velocity" column should be corrected.' % value
                        except:
                            if j == 1:
                                raise TypeError,'CZone Velocity definition is of incorrect type.\n "%s" in "Scaling factor" column should be corrected.' % value
                            if j == 2:
                                raise TypeError,'CZone Velocity definition is of incorrect type.\n "%s" in "Relative velocity" column should be corrected.' % value
                j+=1        
            i+=1
            
    
    if crushStressDefType == 'CONSTANT':
        if crushStessConstVal > 0:
            raise ValueError, 'CZone Crush Stress value should be a positive integer.'
    if crushStressDefType == 'TABULAR':
        if crushStressTableVal == ():
            raise ValueError, 'CZone Crush Stress is not defined.'
        else:
            i=1
            for row in crushStressTableVal:
                j=1
                for item in row:
                    try:
                        value = float(item)
                        if value < 0.0:
                            if j == 1:
                                raise ValueError,'CZone Crush Stress definition is incomplete or incorrect.\n "%s" in "Crush Stress" column should be corrected.' % value
                            if j == 2:
                                raise ValueError,'CZone Crush Stress definition is incomplete or incorrect.\n "%s" in "Material Angle" column should be corrected.' % value
                            if j > 2:
                                raise ValueError,'CZone Crush Stress definition is incomplete or incorrect.\n "%s" in "Field Variable" column should be corrected.' % value
                    except:
                        if j == 1:
                            raise TypeError,'CZone Crush Stress definition is of incorrect type.\n "%s" in "Crush Stress" column should be corrected.' % value
                        if j == 2:
                            raise TypeError,'CZone Crush Stress definition is of incorrect type.\n "%s" in "Material Angle" column should be corrected.' % value
                        if j > 2:
                            raise ValueError,'CZone Crush Stress definition is incomplete or incorrect.\n "%s" in "Field Variable" column should be corrected.' % value

                    else:
                        try:
                            value = int(item)
                            if value < 0:
                                if j == 1:
                                    raise ValueError,'CZone Crush Stress definition is incomplete or incorrect.\n "%s" in "Crush Stress" column should be corrected.' % value
                                if j == 2:
                                    raise ValueError,'CZone Crush Stress definition is incomplete or incorrect.\n "%s" in "Material Angle" column should be corrected.' % value
                                if j > 2:
                                    raise ValueError,'CZone Crush Stress definition is incomplete or incorrect.\n "%s" in "Field Variable" column should be corrected.' % value
                        except:
                            if j == 1:
                                raise TypeError,'CZone Crush Stress definition is of incorrect type.\n "%s" in "Crush Stress" column should be corrected.' % value
                            if j == 2:
                                raise TypeError,'CZone Crush Stress definition is of incorrect type.\n "%s" in "Material Angle" column should be corrected.' % value
                            if j > 2:
                                raise ValueError,'CZone Crush Stress definition is incomplete or incorrect.\n "%s" in "Field Variable" column should be corrected.' % value
                j+=1        
            i+=1
    mdb.customData.czone.models[modelName].czoneDefinition[materialName].CrushStressDefType=crushStressDefType
    mdb.customData.czone.models[modelName].czoneDefinition[materialName].CrushStressConstVal=crushStressConstVal
    mdb.customData.czone.models[modelName].czoneDefinition[materialName].CrushStressTableVal=crushStressTableVal
    mdb.customData.czone.models[modelName].czoneDefinition[materialName].FieldVariableDep=FieldVariableDep
    mdb.customData.czone.models[modelName].czoneDefinition[materialName].tempTable=tempTable
    mdb.customData.czone.models[modelName].czoneDefinition[materialName].IsCzVelocityDefined=isCzVelocityDefined
    mdb.customData.czone.models[modelName].czoneDefinition[materialName].VelocityTableVal=velocityTableVal

#    print '                                              '
#    print 'Czone Defintion parameters %s were written.' % materialName
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

def writeSurfIntParameters(interactionPropName,coefficientOfFriction,advancedOptions=False,CBAngle=60):

    #print 'interactionPropName,coefficientOfFriction,advancedOptions,CBAngle==',interactionPropName,coefficientOfFriction,advancedOptions,CBAngle
##    """
##    Method stores the dialog data in session custom data and
##    converts the model keyword during input file writing.
##    Arguements
##        modelName - Current model Name
##        interactionPropName - the interaction property name 
##        coefficientOfFriction = coefficient of Friction value.    
##    """
    modelName = session.sessionState[session.currentViewportName]['modelName']
    if interactionPropName == "":
        raise ValueError, 'Please define a Surface Interaction.'
    if not coefficientOfFriction:
        raise ValueError, 'Please define Coefficient of Friction.'
    if not mdb.customData.czone.models[modelName].surfaceIntDefinition.has_key(interactionPropName):
        mdb.customData.czone.models[modelName].SurfaceIntDefinition(interactionPropName)
    mdb.customData.czone.models[modelName].surfaceIntDefinition[interactionPropName].coefficientOfFriction=coefficientOfFriction
    mdb.customData.czone.models[modelName].surfaceIntDefinition[interactionPropName].CBAngle=CBAngle
    mdb.customData.czone.models[modelName].surfaceIntDefinition[interactionPropName].advancedOptions=advancedOptions
#    print '                                              '
#    print 'Surface Interaction parameters %s were written.' % interactionPropName
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
def writeInitialCondParameters(modelName, assemblySetName):


##    """
##    Method stores the dialog data in session custom data and
##    converts the model keyword during input file writing.
##    Arguements
##        modelName - Current model Name
##        assemblySetName - the assembly set name on which *Initial Condition will be written.
##    """

    if assemblySetName == "":
        raise ValueError, 'Please define an Assembly Set.'

    if not mdb.customData.czone.models[modelName].initialCondDefinition.has_key(assemblySetName):
        mdb.customData.czone.models[modelName].InitialCondDefinition(assemblySetName)
    
#    print '                                              '
#    print 'Initial Condition parameters %s were written.' % assemblySetName

###~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def writeDeactivate(tsaiWuDefList=(), cZoneDefList=(), surfInteractionDefList=(), initialCondDefList=(), isCzDeactivatedStatus=FALSE):
## All values except isCzDeactivated are not used here.
    modelName = session.sessionState[session.currentViewportName]['modelName']
    prevStatus = FALSE
    if not mdb.customData.czone.models.has_key(modelName):
        raise ValueError, 'Unknown error. Please report to Support.'
    else:
        if 'IsCzDeactivated' in mdb.customData.czone.models[modelName]._getAttributeNames():
            prevStatus = mdb.customData.czone.models[modelName].IsCzDeactivated
        mdb.customData.czone.models[modelName].IsCzDeactivated=isCzDeactivatedStatus
        print '                                              '
    if isCzDeactivatedStatus:
        print 'CZone Plug-in Deactivated.'
    else:       
        if prevStatus:
            print 'CZone Plug-in Activated.'

methodCallback.addCallback(JobType, 'writeInput', writeCzoneToKeywordBlock, callAfter=FALSE)
methodCallback.addCallback(JobType, 'writeInput', removeCzoneKeywordBlock, callAfter=TRUE)

