from abaqusConstants import *
from abaqusGui import *
from kernelAccess import mdb, session
import os


###########################################################################
# Class definition
###########################################################################

class SurfIntDefDB(AFXDataDialog):
    [
        ID_INTPROP,
        ID_CHECKBUTTON
        ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+2)
    #numInstance = 0
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form, actionKw, intNameKw):

        # Construct the base class.
        #
        #SurfIntDefDB.numInstance +=1
        AFXDataDialog.__init__(self, form, 'Surface Interation Definition',
            self.OK|self.CANCEL)
        #getAFXAliasMap().setPrefix(self, 'surfIntdefDB-%d' %SurfIntDefDB.numInstance)    
        getAFXAliasMap().setPrefix(self, 'SurfIntDefDB')   
        #self.appendActionButton(text='OK',tgt=self,
        #                        sel=self.ID_CLICKED_OK)
        #self.appendActionButton(text='Cancel',tgt=self,
        #                        sel=self.ID_CLICKED_CANCEL)       
        path = __file__
        absPath = os.path.abspath(path)
        absDir = os.path.dirname(absPath)
        fileName1= os.path.join(absDir, 'icoR_interactionProperty.png')
        interProppngIcon = afxCreatePNGIcon(fileName1)
            
        frame = FXHorizontalFrame(self, 0, 0,0,0,0, 0,0,0,0)

        # Model combo
        # Since all forms will be canceled if the  model changes,
        # we do not need to register a query on the model.
        #
        VFrame_1 = FXVerticalFrame(p=self, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        HFrame_1 = FXHorizontalFrame(VFrame_1, 0, 0,0,0,0, 0,0,0,0)
        self.SurfIntTxtField=AFXTextField(p=HFrame_1, ncols=15, labelText='Surface Interaction', tgt=form.interactionPropNameKw, sel=0)
        getAFXAliasMap().setName(self.SurfIntTxtField, 'surfinttxt1')

        self.currentModelName = getCurrentContext()['modelName']
        self.form = form
        self.spinner = AFXFloatSpinner(VFrame_1, 4, 'Friction coefficent', form.coefFrictionKw, 0)
        getAFXAliasMap().setName(self.spinner, 'spinner')
        self.spinner.setRange(0, 100)
        self.spinner.setIncrement(0.1)
        
        self.advBtn = FXCheckButton(p=VFrame_1, text='Advanced Options', tgt=self.form.advancedOptionsKw, sel=0)
        getAFXAliasMap().setName(self.advBtn, 'advancedOptChkbtn')
        self.form.advancedOptionsKw.setTarget(self)
        self.form.advancedOptionsKw.setSelector(self.ID_CHECKBUTTON)
        self.form.advancedOptionsKw.setValue(False)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CHECKBUTTON, SurfIntDefDB.advOptions)
        #self.searchBtn.hide()
        
        
        gb1 = FXGroupBox(VFrame_1, '', FRAME_GROOVE|LAYOUT_SIDE_LEFT|LAYOUT_FILL_X)
        self.CBAngle=AFXTextField(p=gb1, ncols=7, labelText='Crush-Bounce Angle (deg)', tgt=form.CBAngleKw, sel=0)
        self.CBAngle.disable()
        getAFXAliasMap().setName(self.CBAngle, 'CBAngle')
        
        #self.extraTextF1=AFXTextField(p=gb1, ncols=7, labelText='Extra ', tgt=form.extraKw, sel=0)
        #self.extraTextF1.disable()
        #getAFXAliasMap().setName(self.extraTextF1, 'Extra')
        
        if actionKw == 'EDIT':
            form.interactionPropNameKw.setValue(intNameKw)

            self.SurfIntTxtField.disable()

            if mdb.customData.czone.models.has_key(self.currentModelName):
                if mdb.customData.czone.models[self.currentModelName].surfaceIntDefinition.has_key(intNameKw):
                    surfInt=mdb.customData.czone.models[self.currentModelName].surfaceIntDefinition[intNameKw]
                    form.coefFrictionKw.setValue(surfInt.coefficientOfFriction)
                    try:
                       advOptTemp=surfInt.advancedOptions
                       form.CBAngleKw.setValue(surfInt.CBAngle)
                       form.advancedOptionsKw.setValue(advOptTemp)
                       if advOptTemp == True:
                          self.CBAngle.enable()
                    except:
                       surfInt.advancedOptions=False
                       surfInt.CBAngle=60
                       form.CBAngleKw.setValue(60)
                       form.advancedOptionsKw.setValue(False)
                       self.CBAngle.disable()
                       
        else:
            names = mdb.customData.czone.models[self.currentModelName].surfaceIntDefinition.keys()        
            names.sort()
            index = 0
            suffix = 0
            for name in names:
                if not name.find('CZAsurfInt-') == -1:
                    prefix,suffix = name.split('-')
                    try:
                        if prefix == 'CZAsurfInt' and int(suffix) > index:
                            index = int(suffix)
                    except:
                        pass
            if not index == 0:
                index = index + 1
                surfIntName = 'CZAsurfInt-' + str(index)
            else:
                surfIntName = 'CZAsurfInt-' + '1'
            
            form.interactionPropNameKw.setValue(surfIntName)
            self.CBAngle.setText('')
            #self.extraTextF1.setText('')
                    

    def advOptions(self, sender, sel, data):
        if self.form.advancedOptionsKw.getValue() == 0:
           self.CBAngle.disable()
           #self.extraTextF1.disable()
        else:
           self.CBAngle.enable()
           #self.extraTextF1.enable()
