from abaqusGui import *
from abq_Czone.tsaiwuDB import TsaiwuDB


###########################################################################
# Class definition
###########################################################################

class WizardDB(AFXDataDialog):

    [
        ID_TREE,
        ID_TREE2,
        ID_MATRL,
        ID_TSAIWU,
        ID_DEL,
        ID_INTPROP,
        ID_DELTSAI,
        ID_DELCZONE,
        ID_DELSURF,
        ID_DELINIT,
        ID_EDITSAI,
        ID_EDITCZONE,
        ID_EDITSURF,
    ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+13)

    #numInstance = 0
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):
        #WizardDB.numInstance +=1
        
        self.form = form
        self.STEP_FINISH = -1
        
        AFXDataDialog.__init__(self, form, 'CZone for Abaqus Plug-in',self.OK|self.CANCEL)
        #getAFXAliasMap().setPrefix(self, 'wizardDB-%d' %WizardDB.numInstance)
        #getAFXAliasMap().setPrefix(self, 'WizardDB')
        #okBtn = self.getActionButton(self.ID_CLICKED_OK)
        #okBtn.setText('                OK                ')

        FXMAPFUNC(self, SEL_COMMAND, self.ID_TREE, WizardDB.onCmdTree)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_TREE2, WizardDB.onCmdTree2)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_MATRL, WizardDB.onCmdMatrl)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_TSAIWU, WizardDB.onCmdTsaiwu)

        topHf = FXHorizontalFrame(self, LAYOUT_FILL_X )
        
        # Definition container
        #
        hf1 = FXVerticalFrame(topHf,LAYOUT_FILL_Y,0,0,0,0,0,0,0,0)
        gb = FXGroupBox(hf1, 'Definitions', FRAME_GROOVE|LAYOUT_FILL_Y|LAYOUT_SIDE_LEFT)
        hf = FXVerticalFrame(gb,LAYOUT_FILL_Y,0,0,0,0,0,0,0,0)
        vf = FXVerticalFrame(hf,FRAME_SUNKEN|FRAME_THICK,0,0,0,0,0,0,0,0)
        self.stepNames = FXTreeList(vf, 4, self, self.ID_TREE,LAYOUT_CENTER_Y|
            TREELIST_SHOWS_BOXES|TREELIST_ROOT_BOXES|TREELIST_SINGLESELECT|HSCROLLING_OFF,0,0,0,0)
        getAFXAliasMap().setName(self.stepNames, 'stepnames')
        
        gb2 = FXGroupBox(hf1, 'Damage Models', FRAME_GROOVE|LAYOUT_FILL_Y|LAYOUT_SIDE_LEFT)
	hf2 = FXVerticalFrame(gb2,LAYOUT_FILL_Y,0,0,0,0,0,0,0,0)
	vf2 = FXVerticalFrame(hf2,FRAME_SUNKEN|FRAME_THICK,0,0,0,0,0,0,0,0)
	self.stepNames2 = FXTreeList(vf2, 2, self, self.ID_TREE2, LAYOUT_CENTER_Y|
	            TREELIST_SHOWS_BOXES|TREELIST_ROOT_BOXES|TREELIST_SINGLESELECT|HSCROLLING_OFF,0,0,0,0)
        getAFXAliasMap().setName(self.stepNames2, 'stepnames2')
        
        gb1 = FXGroupBox(hf1, '', FRAME_GROOVE|LAYOUT_SIDE_LEFT|LAYOUT_FILL_X)
        self.deactivateCZone = FXCheckButton(p=gb1, text='Deactivate CZone', tgt=form.czdeactivateKw, sel=0)
        getAFXAliasMap().setName(self.deactivateCZone, 'deactivatecheck')
        self.deactivateCZone.setTipText("NOTE: CZone Property will not be written if this check button is toggled on.\n\
        Only Tsai-Wu damage definition will be written to the input file.")

        if getCurrentContext()['modelName'] in mdb.customData.czone.models.keys():
            if 'IsCzDeactivated' in mdb.customData.czone.models[getCurrentContext()['modelName']].__members__:
                form.czdeactivateKw.setValue(mdb.customData.czone.models[getCurrentContext()['modelName']].IsCzDeactivated)
            else:
                form.czdeactivateKw.setValue(FALSE)
        else:

            form.czdeactivateKw.setValue(FALSE)
        
        # Page content
        #
        vf = FXVerticalFrame(topHf, LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0)
        self.switcher = FXSwitcher(vf, LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0)
        getAFXAliasMap().setName(self.switcher, 'switcher')
        # Action area
        #
        self.steps = []
        self.steps2 = []
        self.names = []
        self.names2 = []
        self.treeItems = []
        self.treeItems2 = []
        self.stepIndex = 0
        self.stepIndex2 = 0
        self.oldStepIndex = 0

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def Step(self, name, stepId, icon=None):
        self.names.append( name )
        self.treeItems.append( self.stepNames.addItemLast(None, name, oi=icon, ci=icon) )
        item = Step(name, stepId, FXPacker(self.switcher, 0, 0,0,0,0, 0,0,0,0), icon)
        self.stepNames.selectItem(self.stepNames.getFirstItem())
        self.steps.append( item )
        return self.steps[-1]
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def Step2(self, name, stepId, icon=None):
        self.names2.append( name )
        self.treeItems2.append( self.stepNames2.addItemLast(None, name, oi=icon, ci=icon) )
        item = Step2(name, stepId, FXPacker(self.switcher, 0, 0,0,0,0, 0,0,0,0), icon)
        self.steps2.append( item )
        self.stepNames2.deselectItem(self.stepNames2.getFirstItem())
        return self.steps2[-1]
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdMatrl(self, sender, sel, ptr):
        sel = getAFXApp().getAFXMainWindow().getSelectorFromFunction('Material->Manager')
        tgt = getAFXApp().getAFXMainWindow().getTargetFromFunction('Material->Manager')
        tgt.handle(self, MKUINT(sel, SEL_COMMAND), None)
        

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdTsaiwu(self, sender, sel, ptr):
        from tsaiwuDB import TsaiwuDB
        materialKw = ''
        self.tsaiDB= TsaiwuDB(self.form,'CREATE' ,materialKw)
        self.tsaiDB.create()
        self.tsaiDB.show()
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdCzonedefine(self, sender, sel, ptr):

        from czoneParametersDB import CzoneParametersDB
        materialKw = ''
        self.czoneDefDB= CzoneParametersDB(self.form,'CREATE', materialKw)
        self.czoneDefDB.create()
        self.czoneDefDB.showModal()


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdSurfIntdefine(self,sender,sel,ptr):

        from surfIntDefDB import SurfIntDefDB
        materialKw = ''
        self.surfDB= SurfIntDefDB(self.form, 'CREATE', materialKw)
        self.surfDB.create()
        self.surfDB.show()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdIntConddefine(self,sender,sel,ptr):

        from initialConditionDB import InitialConditionDB
        self.initcondDB= InitialConditionDB(self.form)
        self.initcondDB.create()
        self.initcondDB.showModal()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdIntpropdefine(self,sender,sel,ptr):

        sel = getAFXApp().getAFXMainWindow().getSelectorFromFunction('Material->Manager')
        tgt = getAFXApp().getAFXMainWindow().getTargetFromFunction('Material->Manager')
        tgt.handle(self, MKUINT(sel, SEL_COMMAND), None)



    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def switchStep(self):
        currentStep = self.steps[self.oldStepIndex]
        newStep = self.steps[self.stepIndex]

        ok = self.onSwitchStep(currentStep, newStep)

        if ok:

            self.showStep()
        else:

            self.stepIndex = self.oldStepIndex
            self.stepNames.selectItem(self.treeItems[self.stepIndex])
            self.stepNames.setCurrentItem(self.treeItems[self.stepIndex])

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onSwitchStep(self, currentStep, newStep):
    
       # Derive classes should overwrite this method to perform
       # inter-step processing as well as finishing the wizard.
       
       # If this method returns True, then the next step will be shown.
       # If this method returns False, then the current step will remain
       # (presumably because the user must correct inputs in the the
       # current step)
       
       return True
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def showStep(self):
        self.switcher.setCurrent(self.stepIndex)
        self.stepNames.selectItem(self.treeItems[self.stepIndex])

        self.forceRefresh()
        self.getInitialFocusWidget().setFocus()
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdTree(self, sender, sel, ptr):
        # Find the selected tree item, 
        # then display its associated switcher pane.
        #
        i = -1
        w = self.stepNames.getFirstItem()
        self.stepNames2.deselectItem(self.stepNames2.getFirstItem())
	
        while(w):
        
            i += 1
            if self.stepNames.isItemSelected(w):

                self.oldStepIndex = self.stepIndex
                self.stepIndex = i
                self.switchStep()
                break
                
            if w.getFirst():
                w=w.getFirst()

                continue
                
            while  not w.getNext() and w.getParent():
                w=w.getParent()

            w=w.getNext()
    
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdTree2(self, sender, sel, ptr):
        # Find the selected tree item, 
        # then display its associated switcher pane.
        #
        i = -1
        self.stepNames.deselectItem(self.stepNames.getFirstItem())
        self.stepNames.deselectItem(self.stepNames.getLastItem())
	self.stepIndex2 = 2
	self.switcher.setCurrent(self.stepIndex2)
        self.stepNames2.selectItem(self.stepNames2.getFirstItem())
	self.forceRefresh()
        self.getInitialFocusWidget().setFocus()
        
               
###########################################################################
# Class definition
###########################################################################

class Step:

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, name, stepId, parent, icon=None):
        self.name = name
        self.stepId = stepId
        self.icon = icon
        self.parent = parent

###########################################################################
# Class definition
###########################################################################

class Step2:

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, name, stepId, parent, icon=None):
        self.name = name
        self.stepId = stepId
        self.icon = icon
        self.parent = parent



