from abaqusConstants import *
from abaqus import *
try:
    from caeModules import *
except:
    pass
#isAnalyticalRigid = True

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getMaxMin(points, maxX, minX, maxY, minY, code):
    for pt in points:
        if code == 1:
            if pt.pointOn[0][0] > maxX:
                maxX = pt.pointOn[0][0]
            if pt.pointOn[0][0] < minX:
                minX = pt.pointOn[0][0]
            if pt.pointOn[0][1] > maxY:
                maxY = pt.pointOn[0][1]
            if pt.pointOn[0][1] < minY:
                minY = pt.pointOn[0][1]
        else:
            if float(pt.pointOn.strip('[), ( '']').split(',')[0]) > maxX:
                maxX = float(pt.pointOn.strip('[), ( '']').split(',')[0])
            if float(pt.pointOn.strip('[), ( '']').split(',')[0]) < minX:
                minX = float(pt.pointOn.strip('[), ( '']').split(',')[0])
            if float(pt.pointOn.strip('[), ( '']').split(',')[1]) > maxY:
                maxY = float(pt.pointOn.strip('[), ( '']').split(',')[1])
            if float(pt.pointOn.strip('[), ( '']').split(',')[1]) < minY:
                minY = float(pt.pointOn.strip('[), ( '']').split(',')[1])
    return (maxX, minX, maxY, minY)


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def deletePartOnException(vp, vpModelName, copyPartName, part):

    try:
        del mdb.models[vpModelName].parts[copyPartName]
        vp.setValues(displayedObject=part)
    except:
        print 'Unable to set viewport to 3D part.'

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def deleteSketchLeftBelow(vpModelName, copyPart, datum, cutPlane, cutAxis,
                          maxY, minY, sheetSize, point1, point2):
    """Delete sketch to the left of a given plane."""

    t = copyPart.MakeSketchTransform(sketchPlane=datum[cutPlane.id], sketchUpEdge=datum[cutAxis.id],
                    sketchPlaneSide=SIDE1, sketchOrientation=RIGHT, origin=(0,0,0))
    s = mdb.models[vpModelName].ConstrainedSketch(name='__profile__', sheetSize = sheetSize, transform=t)
    s.setPrimaryObject(option=SUPERIMPOSE)
    copyPart.projectReferencesOntoSketch(sketch=s, filter=COPLANAR_EDGES)
    s.rectangle(point1=point1, point2=point2)
    copyPart.CutExtrude(sketchPlane=datum[cutPlane.id], sketchUpEdge=datum[cutAxis.id], sketchPlaneSide=SIDE1,
        sketchOrientation=RIGHT, sketch=s, flipExtrudeDirection=OFF)
    s.unsetPrimaryObject()
    del mdb.models[vpModelName].sketches['__profile__']


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createAxiSymmFrom3D(isAnalyticalRigid):
    """Create Axisymmetric part from a 3D part.
    Part must have been sketched in XY plane and revolved about Y-axis
    Function has commented out lines if user wants to revolve about X-axis"""

    vp = session.viewports[session.currentViewportName]
    vpModelName = vp.displayedObject.modelName
    vpPartName = vp.displayedObject.name
    part = mdb.models[vpModelName].parts[vpPartName]

    if part.queryGeometry(printResults=False)['category'] == 'Geometry':
        i = 1

        # Give unique name and create new part
        #
        while vpPartName + "_axi_"+str(i) in mdb.models[vpModelName].parts.keys():
            i += 1
        copyPartName = vpPartName + "_axi_"+str(i)
        copyPart = mdb.models[vpModelName].Part(name=copyPartName, objectToCopy=part)
        vp.setValues(displayedObject=copyPart)

        # Get part objects
        #
        face = copyPart.faces
        datumPlane = copyPart.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=0.0)
        cell = copyPart.cells
        datum = copyPart.datums

        # Partition cell using XY plane
        #
        try:
            copyPart.PartitionCellByDatumPlane(datumPlane=datum[datumPlane.id], cells=cell[:])
        except:
            print 'Unable to partition. Utility will attempt to remove faces.\n'\
                  'Please double-check created part.'

        removeFaceList = []
        flagXyPlaneFace = False
        for f in face:
            if f.getNormal() == (0,0,1) or f.getNormal() == (0,0,-1):
                flagXyPlaneFace = True
                continue
            else:
                removeFaceList.append(f)

        # If faces found in XY plane, retain that face and remove remaining ones
        #
        if flagXyPlaneFace:

            # Remove all remaining faces
            #
            copyPart.RemoveFaces(faceList = removeFaceList, deleteCells=True)

            # Uncomment following 2 lines (and comment out following 2 lines) to
            # revolve about X-axis.
            #
            # About X-axis
            #cutPlane = copyPart.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=0.0)
            #cutAxis = copyPart.DatumAxisByPrincipalAxis(principalAxis=XAXIS)

            # About Y-axis
            #
            cutPlane = copyPart.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=0.0)
            cutAxis = copyPart.DatumAxisByPrincipalAxis(principalAxis=YAXIS)

            points = copyPart.vertices
            iPoints = copyPart.ips

            # Grab all bounding boxes vertices and interesting points
            # Find max and min coordinates
            #
            try:
                maxX = minX = points[0].pointOn[0][0]
                maxY = minY = points[0].pointOn[0][1]
                (maxX, minX, maxY, minY) = getMaxMin(points, maxX, minX, maxY, minY, 1)
                (maxX, minX, maxY, minY) = getMaxMin(iPoints, maxX, minX, maxY, minY, 2)
            except:
                deletePartOnException(vp, vpModelName, copyPartName, part)
                raise AbaqusException, 'Unable to find any points on the 3D part.'

            # Part is revolved about Y.
            # Create arbitrary sketch with arbitrary thickness to remove the left side of the part.
            # Draw a big rectangle in YZ plane and cut through all. We do this
            # by adding 1e6 to max and min values
            #
            point1 = (10, (maxY + 1e7))
            point2 = (-10,(minY - 1e7))
            sheetSize = 1e7 + 2 * (abs(maxY)+abs(minY))
            try:
                deleteSketchLeftBelow(vpModelName, copyPart, datum, cutPlane, cutAxis,maxY, minY, sheetSize, point1, point2)
            except:
                deletePartOnException(vp, vpModelName, copyPartName, part)
                raise AbaqusException, 'Unable to delete faces.'

            # If part is revolved about X, uncomment out the following lines:
            #
            # Sketch to remove the lower side of the part,
            # Draw a big rectangle in XZ plane and cut through all.
            #
            #point1 = (10, (maxX +abs(maxX)))
            #point2 = (-10, (minX-abs(minX)))
            #sheetSize = 2 * (abs(maxX)+abs(minX))
            #deleteSketchLeftBelow(vpModelName, copyPart, datum, cutPlane, cutAxis,maxX, minX, sheetSize, point1, point2)"""

            # Set part type
            #
            try:
                copyPart.setValues(space=AXISYMMETRIC, type=part.type)
                print "Axi-symmetric part '"+copyPartName+ "' has been successfully created from 3D part '"+part.name+"'."
                vp.setValues(displayedObject=copyPart)
            except:
                deletePartOnException(vp, vpModelName, copyPartName, part)
                raise AbaqusException, 'Cannot change modeling space to axisymmetric.\n\n'\
                                       'Only parts which are planar in XY plane and\n'\
                                       'completely in positive X can be axisymmetric.'
            if isAnalyticalRigid:
                try:
                    e = copyPart.edges
                    allEdges = []
                    for i in e:
                        allEdges.append(i)
                    allEdges = tuple(allEdges)
                    s1 = mdb.models[vpModelName].ConstrainedSketch(name='__profile__',sheetSize=1e6)
                    copyPart.projectEdgesOntoSketch(sketch=s1, edges=allEdges)
                    s1.ConstructionLine(point1=(0.0, -1e6), point2=(0.0, 1e6))
                    mdb.models[vpModelName].ConstrainedSketch(name='internalSktchForAnaRgdPrt', objectToCopy=s1)
                    del mdb.models[vpModelName].sketches['__profile__']
                    i = 1
                    while vpPartName + "_AnaRgd_"+str(i) in mdb.models[vpModelName].parts.keys():
                        i += 1
                    copyNewPartName = vpPartName + "_AnaRgd_"+str(i)
                    newAnaRgdPrt = mdb.models[vpModelName].Part(name=copyNewPartName , dimensionality=AXISYMMETRIC,
                        type=ANALYTIC_RIGID_SURFACE)
                    newAnaRgdPrt.AnalyticRigidSurf2DPlanar(sketch=mdb.models[vpModelName].sketches['internalSktchForAnaRgdPrt'])
                    del mdb.models[vpModelName].sketches['internalSktchForAnaRgdPrt']
                    print "\nAxi-symmetric analytical rigid part '"+copyNewPartName+ "' has been successfully created from 3D part '"+part.name+"'."
                    vp.setValues(displayedObject=newAnaRgdPrt)
                except:
                    raise AbaqusException,'Unable to create analytical rigid part.'
        else:
            deletePartOnException(vp, vpModelName, copyPartName, part)
            raise AbaqusException, 'Unable to set part type to Axisymmetric.'
    else:
        raise AbaqusException, 'This functionality is not supported for orphan mesh parts.'
