from abaqusGui import *

thisPath = os.path.abspath(__file__)
thisDir = os.path.dirname(thisPath)
sendCommand("import sys")
sendCommand("sys.path.append(r'%s')" % thisDir)
from kernelAccess import *
from abaqusConstants import *

from sessionGui import FileToolsetGui

###########################################################################
# Class definition
###########################################################################

class mytoolsetGui(AFXToolsetGui):
    [ID_WARNING,ID_CALL]=range(AFXModuleGui.ID_LAST, AFXModuleGui.ID_LAST+2)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self):

        # Construct the CAE file toolset.
        #
        AFXToolsetGui.__init__(self,'AxiFrom3D')
        FXMAPFUNC(self,SEL_COMMAND,self.ID_CALL,mytoolsetGui.callKernelScript)
        self.group = AFXToolbarGroup(owner=self,title="Create Axisymmetric From 3D")
        (head, tail) = os.path.split(__file__)
        fileLoadPath1 = os.path.join(head, "abq_createAxiSymmFrom3D", "3d_to_axisym.png")
        createIcon = afxCreateIcon(fileLoadPath1)
        createIcon.scale(24,24)
        self.save = AFXToolButton(self.group,'\tCreate\nAxisymmetric From 3D',createIcon,tgt=self, sel=self.ID_CALL)
        
        #self.isAnalyticalRigidKw = AFXBoolKeyword(self.cmd, 'isAnalyticalRigid', AFXBoolKeyword.TRUE_FALSE, True, True)
        
        # set the prefix of the menu pane
        getAFXAliasMap().setPrefix(self.group, '3D To Axi Group')
        
        # name the component test button
        getAFXAliasMap().setName(self.save, '3D To Axi')
        
        FXMAPFUNC(self, SEL_COMMAND, self.ID_WARNING,mytoolsetGui.onCmdWarning)


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getKernelInitializationCommand(self):
        if not getAFXApp().isProductViewer():
            try:
                return 'import abq_createAxiSymmFrom3D.createAxiSymmFrom3DUtils'
            except:
                return ''
        else:
            return ''

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def callKernelScript(self,sender,sel,ptr):
        mainWindow = getAFXApp().getAFXMainWindow()

        if getDisplayedObjectType() == PART:
            self.vp = session.viewports[session.currentViewportName]
            self.vpModelName = self.vp.displayedObject.modelName
            self.vpPartName = self.vp.displayedObject.name
            self.part = mdb.models[self.vpModelName].parts[self.vpPartName]
            if self.part.queryGeometry(printResults=False)['category'] == 'Geometry':
                if self.part.space == THREE_D:
                    showAFXWarningDialog( mainWindow,'Do you want to convert to an axisymmetric analytical rigid part as well?',AFXDialog.YES | AFXDialog.NO,self, self.ID_WARNING)
                    # sendCommand("import abq_createAxiSymmFrom3D.createAxiSymmFrom3DUtils")
                    # try:
                        # #showAFXWarningDialog( self.getCurrentDialog(),'Do you like to create Analytical Rigid part for the same?',AFXDialog.YES | AFXDialog.NO,self, self.ID_WARNING)
                                
                        # sendCommand("abq_createAxiSymmFrom3D.createAxiSymmFrom3DUtils.createAxiSymmFrom3D()")
                    # except Exception, inst:
                        # showAFXErrorDialog(getAFXApp().getAFXMainWindow(),inst.args[0])
                    # except:
                        # showAFXErrorDialog(getAFXApp().getAFXMainWindow(),
                            # 'Error occured while converting to axisymmetric part.\n\n'\
                            # 'Please contact Simulia support for assistance.')
                else:
                    showAFXErrorDialog(getAFXApp().getAFXMainWindow(),
                        'The plug-in is supported only for 3D parts.')
            else:
                showAFXErrorDialog(getAFXApp().getAFXMainWindow(),
                    'The plug-in is supported only for native parts.')
        else:
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(),
                'A part must be displayed in the current viewport.')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdWarning(self, sender, sel, ptr):

        if sender.getPressedButtonId() == AFXDialog.ID_CLICKED_YES:
            #self.isAnalyticalRigidKw.setValue(True)
            isAnalyticalRigid=True
            sendCommand("import abq_createAxiSymmFrom3D.createAxiSymmFrom3DUtils")
            try:
                sendCommand("abq_createAxiSymmFrom3D.createAxiSymmFrom3DUtils.createAxiSymmFrom3D(True)")
            except Exception, inst:
                showAFXErrorDialog(getAFXApp().getAFXMainWindow(),inst.args[0])
            except:
                showAFXErrorDialog(getAFXApp().getAFXMainWindow(),
                    'Error occured while converting to axisymmetric part.\n\n'\
                    'Please contact Simulia support for assistance.')
        elif sender.getPressedButtonId() == AFXDialog.ID_CLICKED_NO:
            #self.isAnalyticalRigidKw.setValue(False)
            sendCommand("import abq_createAxiSymmFrom3D.createAxiSymmFrom3DUtils")
            try:
                sendCommand("abq_createAxiSymmFrom3D.createAxiSymmFrom3DUtils.createAxiSymmFrom3D(False)")
            except Exception, inst:
                showAFXErrorDialog(getAFXApp().getAFXMainWindow(),inst.args[0])
            except:
                showAFXErrorDialog(getAFXApp().getAFXMainWindow(),
                    'Error occured while converting to axisymmetric part.\n\n'\
                    'Please contact Simulia support for assistance.')
        

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def onModuleSwitch(moduleName):
    if not moduleName == 'Part':
        ins.group.hide()
    else:
        ins.group.show()

setSwitchModuleHook(onModuleSwitch)
# Register commands
version = '1.1-1'
mainWindow = getAFXApp().getAFXMainWindow()
ins = mytoolsetGui()
try:
    mainWindow.registerToolset(ins, GUI_IN_TOOLBAR)
except:
    pass


