from abaqus import *
from abaqusConstants import *
import customKernel
import sys, os
from caeModules import *
import uti, math

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def externalHelical(ModelName, PartName, p2_x, p2_y, Rr, Rc, Ra, Rp, delta, WSC, GTN, Width, MeshType, meshsize, 
		App, AngX, AngY, AngZ, GearSubIdent, Recovery_flag, number_sector, JobName, NumEig, hangle):

	#### ---------------------------------------------------------------
	####	Remove Faces	
	p = mdb.models[ModelName].parts[PartName]
	f = p.faces
	fface = []
	for i in range(len(f)):
		if f[i].pointOn[0][2]!=0.0:
			fface.append(f[i])
	p.RemoveFaces(faceList = fface, deleteCells=False)

	p = mdb.models[ModelName].parts[PartName]
	f, e, d = p.faces, p.edges, p.datums
	face = f.findAt((0,p2_y*0.99,0))
	edge = e.findAt((-p2_x*0.99,p2_y*0.99,0),)
	t = p.MakeSketchTransform(sketchPlane=face, sketchUpEdge=edge, sketchPlaneSide=SIDE1, origin=(0.0, 0.0, 0.0))
	s = mdb.models[ModelName].ConstrainedSketch(name='__profile__', sheetSize=100.00, gridSpacing=5, transform=t)
	g, v, d1, c = s.geometry, s.vertices, s.dimensions, s.constraints
	s.setPrimaryObject(option=SUPERIMPOSE)
	p.projectReferencesOntoSketch(sketch=s, filter=COPLANAR_EDGES)
	s.CircleByCenterPerimeter(center=(0.0, 0.0), point1=(0.0, Rr))
	p.PartitionFaceBySketch(sketchUpEdge=edge, faces=face, sketch=s)
	s.unsetPrimaryObject()
	del mdb.models[ModelName].sketches['__profile__']

	#### ---------------------------------------------------------------
	#### 	Copy Part
	p = mdb.models[ModelName].Part(name=PartName+'-Copy', 
			objectToCopy=mdb.models[ModelName].parts[PartName])

	#### ---------------------------------------------------------------
	#### 	Create Angle for Helix Angle 
	#rotangle = float(Width)*tan(float(hangle))/float(Ra)
	rotangle = float(Width)*tan(float(hangle))/float(Rp)

	print ' '
	print 'Rotation Angle for '+str(hangle*180./math.pi)+' Helix Angle: '+str(rotangle*180./math.pi)+' Degree'
	print ' '
	
	if WSC!=1:
		rotangle = rotangle/WSC
		Width = Width/WSC

	print ' '
	print 'Divided by '+str(WSC)+' Section in width direction '
	print ' '

	##raise ValueError, "CHECK 001 !!"
	#### ---------------------------------------------------------------
	#### 	Create Sector with Helical Teeth 
	a = mdb.models[ModelName].rootAssembly
	p = mdb.models[ModelName].parts[PartName]
	a.Instance(name=PartName+'-1', part=p, dependent=ON)
	p = mdb.models[ModelName].parts[PartName+'-Copy']
	a.Instance(name=PartName+'-Copy'+'-1', part=p, dependent=ON)
	a.translate(instanceList=(PartName+'-Copy'+'-1', ), vector=(0.0, 0.0, Width))
	a.rotate(instanceList=(PartName+'-Copy'+'-1', ), axisPoint=(0.0, 0.0, 0.0), axisDirection=(
		0.0, 0.0, 1.0), angle=rotangle*180./math.pi)
	a.InstanceFromBooleanMerge(name=PartName+'01', instances=(a.instances[PartName+'-1'], 
	    a.instances[PartName+'-Copy'+'-1'], ), originalInstances=DELETE, domain=GEOMETRY)

	##raise ValueError, "CHECK 002 !!"
	#### ---------------------------------------------------------------
	#### 	Create Wire for Helical reference line
	p = mdb.models[ModelName].parts[PartName+'01']
	numPoints = WSC*2
	Helix = []
	Helix_angle = []
	####

	for ip in range(numPoints+1):
		theta = (float(ip)/float(numPoints))*float(-1.*rotangle)

		if App==1:
			fx = float(Rr)*math.sin(theta)
			fy = float(Rr)*math.cos(theta)
		elif App==2:
			fx = float(Rp)*math.sin(theta)
			fy = float(Rp)*math.cos(theta)
		else:
			raise ValueError, "Approach should be 1 or 2 !!"
		fz = (float(ip)/float(numPoints))*float(Width)
		
		if ip%2==0.0:
			Helix.append((fx,fy,fz))
			Helix_angle.append(theta)

	d = p.datums
	l = len(p.datums)

	##raise ValueError, "CHECK 003 !!"
	#### ---------------------------------------------------------------
	#### 	Create Loft Sector with Helical Teeth 
	p = mdb.models[ModelName].parts[PartName+'01']
	e = p.edges
	edges01 = e.getByBoundingBox(-1e20,-1e20,-1,1e20,1e20,Width/2.)

	if len(edges01)!=10:
		print len(edges01)
		raise ValueError, "Edge length should be 10 !! - Track01"

	ed01 = []
	for nume01 in range(len(edges01)):
		ed01.append(edges01[nume01])

	for nume01 in range(len(ed01)):
		for nume02 in range(len(ed01)-1):
			a = ed01[nume01].pointOn[0][1]
			b = ed01[nume02].pointOn[0][1]
			if a > b:
				temp = ed01[nume01]
				ed01[nume01] = ed01[nume02]
				ed01[nume02] = temp 

	edges001 = []
	edges001.append(ed01[4])
	edges001.append(ed01[5])
	edges001.append(ed01[6])

	for nume01 in range(len(edges001)):
		for nume02 in range(len(edges001)-1):
			a = edges001[nume01].pointOn[0][0]
			b = edges001[nume02].pointOn[0][0]
			if a > b:
				temp = edges001[nume01]
				edges001[nume01] = edges001[nume02]
				edges001[nume02] = temp 

	edges01 = []
	for nume01 in range(4):
		edges01.append(ed01[nume01])
	#edges01.append(edges001[1])
	for nume01 in range(3):
		edges01.append(ed01[nume01+7])
	edges01.append(edges001[0])
	edges01.append(edges001[2])

	edges02 = e.getByBoundingBox(-1e20,-1e20,Width/2.,1e20,1e20,Width+1.)

	if len(edges02)!=10:
		print len(edges02)
		raise ValueError, "Edge length should be 10 !! - Track01"

	ed02 = []
	for nume01 in range(len(edges02)):
		ed02.append(edges02[nume01])

	for nume01 in range(len(ed02)):
		for nume02 in range(len(ed02)-1):
			a = ed02[nume01].pointOn[0][1]
			b = ed02[nume02].pointOn[0][1]
			if a > b:
				temp = ed02[nume01]
				ed02[nume01] = ed02[nume02]
				ed02[nume02] = temp 
	edges002 = []
	edges002.append(ed02[4])
	edges002.append(ed02[5])
	edges002.append(ed02[6])

	for nume01 in range(len(edges002)):
		for nume02 in range(len(edges002)-1):
			a = edges002[nume01].pointOn[0][0]
			b = edges002[nume02].pointOn[0][0]
			if a > b:
				temp = edges002[nume01]
				edges002[nume01] = edges002[nume02]
				edges002[nume02] = temp 

	edges02 = []
	for nume01 in range(4):
		edges02.append(ed02[nume01])
	#edges02.append(edges002[1])
	for nume01 in range(3):
		edges02.append(ed02[nume01+7])
	edges02.append(edges002[0])
	edges02.append(edges002[2])

	###

	#e1 = p.edges
	#for i in range(len(e1)):
	#	if 0<e1[i].pointOn[0][2] and e1[i].pointOn[0][2]<Width:
	#		pathEdges = e1[i]
	#fx = pathEdges.pointOn[0][0]
	#fy = pathEdges.pointOn[0][1]
	#fz = pathEdges.pointOn[0][2]

	p.SolidLoft(loftsections=((edges01),(edges02)), globalSmoothing=ON, keepInternalBoundaries=True)

	#raise ValueError, "CHECK 004 !!"
	#### ---------------------------------------------------------------
	####	Selecting internal 4 edges
	p = mdb.models[ModelName].parts[PartName+'01']
	session.viewports['Viewport: 1'].setValues(displayedObject=p)
	e = p.edges

	edges01 = e.getByBoundingBox(-1e20,p2_y*0.999,-1,1e20,Rr*1.001,Width/2.)
	temp_y = Rr*math.cos((2.*delta+abs(rotangle)))
	edges02 = e.getByBoundingBox(-1e20,temp_y*0.999,Width/2.,1e20,Rr*1.001,Width+1.)
	edges03 = e.getByBoundingBox(-1e20,temp_y*0.999,-1,1e20,Rr*1.001,Width+1.)

	if len(edges01)!=3:
		print len(edges01)
		raise ValueError, "Edge length should be 3 !! - Track01"
	if len(edges02)!=3:
		print len(edges02)
		raise ValueError, "Edge length should be 3 !! - Track02"
	if len(edges03)!=10:
		print len(edges03)
		raise ValueError, "Edge length should be 10 !! - Track03"

	final4edges = []
	ed01 = []
	for nume01 in range(len(edges01)):
		ed01.append(edges01[nume01])
	for nume01 in range(len(ed01)):
		for nume02 in range(len(ed01)-1):
			a = ed01[nume01].pointOn[0][0]
	 		b = ed01[nume02].pointOn[0][0]
	 		if a > b:
	 			temp = ed01[nume01]
	 			ed01[nume01] = ed01[nume02]
	 			ed01[nume02] = temp 
	final4edges.append(ed01[1])
	ed02 = []
	for nume01 in range(len(edges02)):
	 	ed02.append(edges02[nume01])
	for nume01 in range(len(ed02)):
	 	for nume02 in range(len(ed02)-1):
	 		a = ed02[nume01].pointOn[0][0]
	 		b = ed02[nume02].pointOn[0][0]
	 		if a > b:
	 			temp = ed02[nume01]
	 			ed02[nume01] = ed02[nume02]
	 			ed02[nume02] = temp 
	final4edges.append(ed02[1])
	ed03 = []
	for nume01 in range(len(edges03)):
		if 0.<edges03[nume01].pointOn[0][2] and edges03[nume01].pointOn[0][2]<Width*0.999:
			ed03.append(edges03[nume01])
	for nume01 in range(len(ed03)):
	 	for nume02 in range(len(ed03)-1):
	 		a = ed03[nume01].pointOn[0][0]
	 		b = ed03[nume02].pointOn[0][0]
	 		if a > b:
	 			temp = ed03[nume01]
	 			ed03[nume01] = ed03[nume02]
	 			ed03[nume02] = temp 

	if len(ed03)!=4:
		raise ValueError, "Edge length should be 4 !! - Track04"

	final4edges.append(ed03[1])
	final4edges.append(ed03[2])
		
	p.CoverEdges(edgeList=(final4edges), tryAnalytical=False)

	#raise ValueError, "CHECK 005 !!"
	#### ---------------------------------------------------------------
	a1 = mdb.models[ModelName].rootAssembly
	p = mdb.models[ModelName].parts[PartName+'01']
	for iflag in range(WSC):
		a1.Instance(name=PartName+'01'+'-'+str(iflag+1), part=p, dependent=ON)
		if iflag>0:
			a1.translate(instanceList=(PartName+'01'+'-'+str(iflag+1), ), vector=(0.0, 0.0, Width*iflag))
			a1.rotate(instanceList=(PartName+'01'+'-'+str(iflag+1), ), axisPoint=(0.0, 0.0, 0.0), 
			    axisDirection=(0.0, 0.0,1.0), angle=(rotangle*float(iflag)*180./math.pi))

	if WSC==1:
		p = mdb.models[ModelName].Part(name=PartName+'01_FINAL', 
				objectToCopy=mdb.models[ModelName].parts[PartName+'01'])
		pass
	elif WSC==3:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==5:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==7:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==9:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], a1.instances[PartName+'01'+'-8'], a1.instances[PartName+'01'+'-9'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==11:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], a1.instances[PartName+'01'+'-8'], a1.instances[PartName+'01'+'-9'], 
	            a1.instances[PartName+'01'+'-10'], a1.instances[PartName+'01'+'-11'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==13:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], a1.instances[PartName+'01'+'-8'], a1.instances[PartName+'01'+'-9'], 
	            a1.instances[PartName+'01'+'-10'], a1.instances[PartName+'01'+'-11'], a1.instances[PartName+'01'+'-12'], 
	            a1.instances[PartName+'01'+'-13'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==15:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], a1.instances[PartName+'01'+'-8'], a1.instances[PartName+'01'+'-9'], 
	            a1.instances[PartName+'01'+'-10'], a1.instances[PartName+'01'+'-11'], a1.instances[PartName+'01'+'-12'], 
	            a1.instances[PartName+'01'+'-13'], a1.instances[PartName+'01'+'-14'], a1.instances[PartName+'01'+'-15'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==17:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], a1.instances[PartName+'01'+'-8'], a1.instances[PartName+'01'+'-9'], 
	            a1.instances[PartName+'01'+'-10'], a1.instances[PartName+'01'+'-11'], a1.instances[PartName+'01'+'-12'], 
	            a1.instances[PartName+'01'+'-13'], a1.instances[PartName+'01'+'-14'], a1.instances[PartName+'01'+'-15'], 
	            a1.instances[PartName+'01'+'-16'], a1.instances[PartName+'01'+'-17'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==19:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], a1.instances[PartName+'01'+'-8'], a1.instances[PartName+'01'+'-9'], 
	            a1.instances[PartName+'01'+'-10'], a1.instances[PartName+'01'+'-11'], a1.instances[PartName+'01'+'-12'], 
	            a1.instances[PartName+'01'+'-13'], a1.instances[PartName+'01'+'-14'], a1.instances[PartName+'01'+'-15'], 
	            a1.instances[PartName+'01'+'-16'], a1.instances[PartName+'01'+'-17'], a1.instances[PartName+'01'+'-18'], 
	            a1.instances[PartName+'01'+'-19'], ),  
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)

	#raise ValueError, "CHECK 006 !!"
	#### ---------------------------------------------------------------

	if WSC!=1:
		rotangle = rotangle*WSC
		Width = Width*WSC

	Helix = []
	Helix_angle = []
	####

	for ip in range(numPoints+1):
		theta = (float(ip)/float(numPoints))*float(-1.*rotangle)

		fx = float(Rp)*math.sin(theta)
		fy = float(Rp)*math.cos(theta)
		fz = (float(ip)/float(numPoints))*float(Width)
		
		if ip%2==0.0:
			Helix.append((fx,fy,fz))
			Helix_angle.append(theta)

	#### ---------------------------------------------------------------
	####	Face Set-up
	p = mdb.models[ModelName].parts[PartName+'01_FINAL']
	f = p.faces
	#face = f.getByBoundingBox(-Ra,abs(float(Rr)*math.cos(2*delta+rotangle))*0.999,-1., Ra, 1.e20,Width+1.)
	face = f.getByBoundingBox(-1e20,-1e20,-1e20, 1e20,1e20,1e20)
	p.Set(faces=face, name='TEST')

	for i in range(len(Helix)):
		for j in range(len(Helix)-1):
			a = Helix[i][2]
			b = Helix[j][2]
			if a<b:
				tmp = Helix[i]
				Helix[i] = Helix[j]
				Helix[j] = tmp 	

	#print ' '
	#print ' ------ Helix Coordinate Values at Width Section '
	#print ' '
	#for i in range(len(Helix)):
	# 	print Helix[i]
	# 	print Helix_angle[i]
	#print ' '

	test = []
	for i in range(len(p.sets['TEST'].faces)):
		test.append(p.sets['TEST'].faces[i])

	for i in range(len(test)):
		for j in range(len(test)-1):
			a = test[i].pointOn[0][2]
			b = test[j].pointOn[0][2]
			if a<b:
				tmp = test[i]
				test[i] = test[j]
				test[j] = tmp 	

	#for i in range(len(test)):
	# 	print test[i]

	CN0F = []
	CN1F = []
	for i in range(len(Helix)-1):
		z01 = Helix[i][2]
		z02 = Helix[i+1][2]
		face_tmp = []
		for j in range(len(test)):
			if z02>test[j].pointOn[0][2] and test[j].pointOn[0][2]>z01:

				tmp1 = abs(z01/float(test[j].pointOn[0][2]))
				tmp2 = abs(z02/float(test[j].pointOn[0][2]))
				flag = 0
				if (0.9999<tmp1 and tmp1<1.00001) or (0.9999<tmp2 and tmp2<1.00001):
					flag=1
				if flag==0:
					face_tmp.append(test[j])
		if len(face_tmp)!=10:
			raise ValueError, "Face should be 10 !!"

		for ii in range(len(face_tmp)):
			for jj in range(len(face_tmp)-ii):
				a = face_tmp[ii].pointOn[0][1]
				b = face_tmp[ii+jj].pointOn[0][1]

				a_x = face_tmp[ii].pointOn[0][0]
				a_y = face_tmp[ii].pointOn[0][1]
				b_x = face_tmp[ii+jj].pointOn[0][0]
				b_y = face_tmp[ii+jj].pointOn[0][1]

				tmp_angle = Helix_angle[i+1]
				#tmp_angle = (Helix_angle[i]+Helix_angle[i+1])/2.

				a1_x = math.cos(tmp_angle)*a_x     + math.sin(tmp_angle*-1.)*a_y
				a1_y =-math.sin(tmp_angle*-1.)*a_x + math.cos(tmp_angle)*a_y

				b1_x = math.cos(tmp_angle)*b_x     + math.sin(tmp_angle*-1.)*b_y
				b1_y =-math.sin(tmp_angle*-1.)*b_x + math.cos(tmp_angle)*b_y

				if a1_y>b1_y:
					tmp = face_tmp[ii]
					face_tmp[ii] = face_tmp[ii+jj]
					face_tmp[ii+jj] = tmp 	

		CN0F.append(face_tmp[6])
		CN0F.append(face_tmp[7])

		face_tmp01 = []
		face_tmp01.append(face_tmp[3])
		face_tmp01.append(face_tmp[4])
		face_tmp01.append(face_tmp[5])

		for ii in range(len(face_tmp01)):
		 	for jj in range(len(face_tmp01)-ii):
		  		a = face_tmp01[ii].pointOn[0][0]
		  		b = face_tmp01[ii+jj].pointOn[0][0]
 
 		 		a_x = face_tmp01[ii].pointOn[0][0]
 		 		a_y = face_tmp01[ii].pointOn[0][1]
 		 		b_x = face_tmp01[ii+jj].pointOn[0][0]
 		 		b_y = face_tmp01[ii+jj].pointOn[0][1]
 
 		 		tmp_angle = Helix_angle[i+1]
 		 		#tmp_angle = (Helix_angle[i]+Helix_angle[i+1])/2.
 
 		 		a1_x = math.cos(tmp_angle)*a_x     + math.sin(tmp_angle*-1.)*a_y
 		 		a1_y =-math.sin(tmp_angle*-1.)*a_x + math.cos(tmp_angle)*a_y
 
 		 		b1_x = math.cos(tmp_angle)*b_x     + math.sin(tmp_angle*-1.)*b_y
 		 		b1_y =-math.sin(tmp_angle*-1.)*b_x + math.cos(tmp_angle)*b_y
 
 		 		if a1_x>b1_x:
 		 			tmp = face_tmp01[ii]
 		 			face_tmp01[ii] = face_tmp01[ii+jj]
 		 			face_tmp01[ii+jj] = tmp 	

		CN1F.append(face_tmp01[1])
			
	iflag = 0
	for i in range(WSC):
		face = f.findAt((CN0F[iflag].pointOn[0], ))
		p.Set(faces=face, name='TF01Set0'+str(i+1))
		face = f.findAt((CN0F[iflag+1].pointOn[0],))
		p.Set(faces=face, name='TF02Set0'+str(i+1))
		p.SetByBoolean(name='TFSet0'+str(i+1), sets=(p.sets['TF01Set0'+str(i+1)], p.sets['TF02Set0'+str(i+1)],))
		iflag = iflag+2
		del mdb.models[ModelName].parts[PartName+'01_FINAL'].sets['TF01Set0'+str(i+1)]
		del mdb.models[ModelName].parts[PartName+'01_FINAL'].sets['TF02Set0'+str(i+1)]
	#del mdb.models[ModelName].parts[PartName+'02'].sets['TEST']

	for i in range(WSC):
		face = f.findAt((CN1F[i].pointOn[0], ))
		p.Set(faces=face, name='TFBSet0'+str(i+1))
	#del mdb.models[ModelName].parts[PartName+'02'].sets['TEST']

	#raise ValueError, "CHECK 007 !!"
	#### ---------------------------------------------------------------
	####	Material
	mdb.models[ModelName].Material(name='STEEL')
	mdb.models[ModelName].materials['STEEL'].Density(table=((7.8e-09, ), ))
	mdb.models[ModelName].materials['STEEL'].Elastic(table=((200000.0, 0.3), ))
	mdb.models[ModelName].HomogeneousSolidSection(name='SOLID', material='STEEL', thickness=None)

	#### ---------------------------------------------------------------
	####	Arranging PART
	del mdb.models[ModelName].parts[PartName]
	del mdb.models[ModelName].parts[PartName+'-Copy']
	del mdb.models[ModelName].parts[PartName+'01']
	PartName = PartName+'01_FINAL'

	#raise ValueError, "CHECK 008 !!"
	#### ---------------------------------------------------------------
	####	Meshing
	p = mdb.models[ModelName].parts[PartName]
	c = p.cells
	cells = c.getByBoundingBox(-1e20,-1e20,-1e20,1e20,1e20,1e20)
	region = p.Set(cells=cells, name='ALL')
	p = mdb.models[ModelName].parts[PartName]
	p.SectionAssignment(region=region, sectionName='SOLID', offset=0.0, 
			offsetType=MIDDLE_SURFACE, offsetField='', 
			thicknessAssignment=FROM_SECTION)
	c = p.cells
	###p.setMeshControls(regions=c, technique=SWEEP, algorithm=ADVANCING_FRONT)
	##p.setMeshControls(regions=c, elemShape=TET, technique=FREE)
	p = mdb.models[ModelName].parts[PartName]
	e = p.edges
	side_edge = []
	for i in range(len(e)):
		for j in range(WSC):
			a = float(j)*float(Width)/float(WSC)
			b = float(j+1)*float(Width)/float(WSC)
			if ((a+.1/WSC)<e[i].pointOn[0][2] and e[i].pointOn[0][2]<(b-(.1/WSC))): 
				side_edge.append(e[i])

	c = p.cells
	if MeshType=='Hex-dominant':
		p.setMeshControls(regions=c, elemShape=HEX_DOMINATED)
	elif MeshType=='Tetra':
		p.setMeshControls(regions=c, elemShape=TET, technique=FREE)

	p = mdb.models[ModelName].parts[PartName]
	e = p.edges
	if meshsize==0.0:
		tmp_meshsize = (Ra-Rr)/8.
		print 'Meshsize: '+str(tmp_meshsize)
		org_meshsize = meshsize
		meshsize = tmp_meshsize

		#pickedEdge10 = e.getByBoundingBox(-1e20,-1e20,0, 1e20,1e20,Width)
		#p.seedEdgeByNumber(edges=pickedEdge10, number=1, constraint=FINER)

		p.seedEdgeByNumber(edges=side_edge, number=1, constraint=FINER)
		p.seedPart(size=meshsize, minSizeFactor=0.1)

	else:
		p.seedEdgeByNumber(edges=side_edge, number=1, constraint=FINER)
		p.seedPart(size=meshsize, minSizeFactor=0.1)

	p.generateMesh()				
	print ' '
	print ' --------------------------------------------------------------- '
	print ' '

	#raise ValueError, "CHECK 009 !!"
	#### -----------------------------------------------------------------------------------------
	####	Reference Node for DCOUP Generation - Approach A and B
	print ' '
	print ' -------------  DCOUP Reference Nodes'
	print ' '
	iflag = 0
	for i in range(GTN):
		fx = (Helix[iflag][0]+Helix[iflag+1][0])*0.5
		fy = (Helix[iflag][1]+Helix[iflag+1][1])*0.5
		fz = (Helix[iflag][2]+Helix[iflag+1][2])*0.5
		if App==1:
			#p.Node(coordinates=(fx, fy-(Ra-Rr)+meshsize*0.5, fz))
			#print 'DCOUP Reference Node: '+str(fx)+', '+str(fy-(Ra-Rr)+meshsize*0.5)+', '+str(fz)
			p.Node(coordinates=(fx, fy, fz))
			print 'DCOUP Reference Node: '+str(fx)+', '+str(fy)+', '+str(fz)
			nodes = p.nodes[-1:]
			p.Set(nodes=nodes, name='Ref-CN0'+str(i+1))
		elif App==2:
			#p.Node(coordinates=(fx, fy-(Ra-Rp)+meshsize*0.5, fz))
			#print 'DCOUP Reference Node: '+str(fx)+', '+str(fy-(Ra-Rp)+meshsize*0.5)+', '+str(fz)
			p.Node(coordinates=(fx, fy, fz))
			print 'DCOUP Reference Node: '+str(fx)+', '+str(fy)+', '+str(fz)
			nodes = p.nodes[-1:]
			p.Set(nodes=nodes, name='Ref-CN0'+str(i+1))
		else:
			raise ValueError, "Approach should be 1 or 2 !!"
		iflag += 2

	pass

	#raise ValueError, "Check Point #14 !!"
	#### ---------------------------------------------------------------
	####	Constraint for DCOUP
	Set_tmp = []
	for i in range(WSC):
		if float(i)%2.==0:
			if App==1:
				Set_tmp.append('TFBSet0'+str(i+1))
			elif App==2:
				Set_tmp.append('TFSet0'+str(i+1))
			else:
				raise ValueError, "Approach should be 1 or 2 !!"
	a = mdb.models[ModelName].rootAssembly
	for i in range(GTN):
		region1=a.instances[PartName+'-1'].sets['Ref-CN0'+str(i+1)]
		region2=a.instances[PartName+'-1'].sets[Set_tmp[i]]
		mdb.models[ModelName].Coupling(name='Constraint-'+str(i+1), controlPoint=region1, 
		    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=DISTRIBUTING, 
		    weightingMethod=UNIFORM, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
		    ur2=ON, ur3=ON)	

	#raise ValueError, "CHECK 010 !!"
	#### ---------------------------------------------------------------
	#### 	Assembly Radial Sweep 360 degree	
	a.RadialInstancePattern(instanceList=(PartName+'-1', ), point=(0.0, 0.0, 0.0), 
			axis=(0.0, 0.0, 1.0), number=number_sector, totalAngle=360.0)

	#### ---------------------------------------------------------------
	####  	Constraint Creation for Radial Instance	
	## instanace number in circumferential direction
	tot_rad_num = int(number_sector)
	#rad_num = 2
	##	constraint ID
	c_num = int(GTN)+1
	#c_num = 6
	##	reference node number for teeth
	tot_p_num = int(GTN)
	#p_num = 1
	##
	a = mdb.models[ModelName].rootAssembly
	for r in range(tot_rad_num-1): 
		rad_num = r + 2
		for p in range(tot_p_num): 
			p_num = p + 1

			region1=a.instances[PartName+'-1-rad-'+str(rad_num)].sets['Ref-CN0'+str(p_num)]
			region2=a.instances[PartName+'-1-rad-'+str(rad_num)].sets[Set_tmp[p]]
			mdb.models[ModelName].Coupling(name='Constraint-'+str(c_num), controlPoint=region1, 
			    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=DISTRIBUTING, 
			    weightingMethod=UNIFORM, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
			    ur2=ON, ur3=ON)
			c_num += 1	

	#raise ValueError, "CHECK 011 !!"
	#### ---------------------------------------------------------------
	####   	RETAINED NSET	
	a.regenerate()
	reftmp = []
	inst = a.allInstances
	for i in range(len(inst.keys())):
		for j in range(len(inst[inst.keys()[i]].sets.keys())):
			if ('CN0' in inst[inst.keys()[i]].sets.keys()[j])==TRUE:
				reftmp.append(inst[inst.keys()[i]].sets[inst[inst.keys()[i]].sets.keys()[j]])
	#print reftmp

	if int(GTN)*int(number_sector)!=int(len(reftmp)):
		raise ValueError, ' Reference Search ERROR !!'

	a.SetByBoolean(name='Set-RETAINED01', sets=((reftmp)))

	pass

	#raise ValueError, "CHECK 012 !!"
	#### ---------------------------------------------------------------
	####   	Dummy Input file Creation 	
	mdb.models[ModelName].setValues(noPartsInputFile=ON)
	mdb.Job(name=JobName+'-TMP', model=ModelName, description='', type=ANALYSIS, 
			atTime=None, waitMinutes=0, waitHours=0, queue=None, memory=90, 
			memoryUnits=PERCENTAGE, getMemoryFromAnalysis=True, 
			explicitPrecision=SINGLE, nodalOutputPrecision=SINGLE, echoPrint=OFF, 
			modelPrint=OFF, contactPrint=OFF, historyPrint=OFF, userSubroutine='', 
			scratch='', resultsFormat=ODB, parallelizationMethodExplicit=DOMAIN, 
			numDomains=1, activateLoadBalancing=False, multiprocessingMode=DEFAULT, 
			numCpus=1, numGPUs=0)
	mdb.jobs[JobName+'-TMP'].writeInput(consistencyChecking=OFF)

	#### ---------------------------------------------------------------
	####   	Read Dummy Input file and Merge Coincidence Nodes 
	ModelName = JobName+'-TMP'
	PartName2 = 'PART-1'
	mdb.ModelFromInputFile(name=ModelName, inputFileName=JobName+'-TMP.inp')	
	a = mdb.models[ModelName].rootAssembly
	p = mdb.models[ModelName].parts[PartName2]
	mdb.meshEditOptions.setValues(enableUndo=True, maxUndoCacheElements=0.5)
	n = p.nodes
	nodes = n[0:len(n)]
	p.mergeNodes(nodes=nodes, tolerance=0.01)
	#### ---------------------------------------------------------------
	####    Move Model to match origin (0,0,0)	
	a=mdb.models[ModelName].rootAssembly
	a.translate(instanceList=(PartName2+'-1', ), vector=(0.0, 0.0, -Width/2.))
	#### ---------------------------------------------------------------
	####    RETAINED NODE SET-UP 
	a.DatumPointByCoordinate(coords=(0.0, 0.0, 0.0))
	d1 = a.datums
	a.ReferencePoint(point=d1[d1.keys()[len(d1.keys())-1]].pointOn)
	r1 = a.referencePoints
	refPoints1=(r1[r1.keys()[0]], )
	region1=a.Set(referencePoints=refPoints1, name='SET-RETAINED10')
	a.SetByBoolean(name='RETAINED', sets=(a.sets['SET-RETAINED01'], a.sets['SET-RETAINED10'],))

	#### ---------------------------------------------------------------
	####    Center Constraint Build 
	nodes1 = a.instances[a.instances.keys()[0]].nodes.getByBoundingCylinder((0,0,-Width),(0,0,Width),float(Rc+meshsize*0.1))
	region2 = a.Set(nodes=nodes1, name='BoreSurface')

	mdb.models[ModelName].Coupling(name='Constraint-Center', controlPoint=region1, 
	    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=DISTRIBUTING, 
	    weightingMethod=UNIFORM, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
	    ur2=ON, ur3=ON)	

	#### ---------------------------------------------------------------
	####    Rotation Assembly at + Z axis 
	a = mdb.models[ModelName].rootAssembly
	a.rotate(instanceList=(PartName2+'-1', ), axisPoint=(0.0, 0.0, 0.0), 
	    axisDirection=(1.0, 0.0, 0.0), angle=AngX)
	a.rotate(instanceList=(PartName2+'-1', ), axisPoint=(0.0, 0.0, 0.0), 
	    axisDirection=(0.0, 1.0, 0.0), angle=AngY)
	a.rotate(instanceList=(PartName2+'-1', ), axisPoint=(0.0, 0.0, 0.0), 
	    axisDirection=(0.0, 0.0, 1.0), angle=AngZ)

	#### ---------------------------------------------------------------
	####	FREQ and SUBSTR GENERATION STEP Build 
	a = mdb.models[ModelName].rootAssembly
	mdb.models[ModelName].FrequencyStep(name='FREQ', previous='Initial', 
			simLinearDynamics=ON, normalization=MASS, numEigen=NumEig)
	region = a.sets['RETAINED']
	mdb.models[ModelName].EncastreBC(name='RETAINED', createStepName='FREQ', 
			region=region, localCsys=None)
	mdb.models[ModelName].SubstructureGenerateStep(name='SUBSTR_GEN', 
			previous='FREQ', substructureIdentifier=GearSubIdent, computeReducedMassMatrix=ON, 
			computeReducedViscousDampingMatrix=ON, computeReducedStructuralDampingMatrix=ON, 
			modeRange=((1, NumEig, 1), ), retainedEigenmodesMethod=MODE_RANGE)
	region = a.sets['RETAINED']
	mdb.models[ModelName].RetainedNodalDofsBC(name='RETAINDOF', createStepName='SUBSTR_GEN', 
			region=region, u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, ur3=ON)
	mdb.models[ModelName].steps['SUBSTR_GEN'].setValues(computeReducedViscousDampingMatrix=OFF)
	mdb.models[ModelName].steps['SUBSTR_GEN'].setValues( computeReducedStructuralDampingMatrix=OFF)
	if Recovery_flag==0:
		mdb.models[ModelName].steps['SUBSTR_GEN'].setValues(recoveryMatrix=NONE)
	mdb.models[ModelName].setValues(noPartsInputFile=ON)
	mdb.Job(name=JobName, model=ModelName, description='', type=ANALYSIS, atTime=None, 
			waitMinutes=0, waitHours=0, queue=None, memory=90, memoryUnits=PERCENTAGE, 
			getMemoryFromAnalysis=True, explicitPrecision=SINGLE, nodalOutputPrecision=SINGLE, 
			echoPrint=OFF, modelPrint=OFF, contactPrint=OFF, historyPrint=OFF, 
			userSubroutine='', scratch='', resultsFormat=ODB, multiprocessingMode=DEFAULT, 
			numCpus=1, numGPUs=0)
	mdb.jobs[JobName].writeInput(consistencyChecking=OFF)	

	#raise ValueError, "CHECK 013 !!"
	#### ---------------------------------------------------------------
	#### 	FLEXIBLE BODY Keyword Add.	
	if Recovery_flag==0:
		readfile = open(JobName+'.inp', 'r')
		writefile = open(JobName+'-SimpackFlex.inp', 'w')

		read_line = readfile.readlines()
		num_line = 1
		for line in read_line:
			if num_line==len(read_line): 
				writefile.write('*FLEXIBLE BODY, TYPE=SIMPACK\n')
				writefile.write(line)
			else:
				writefile.write(line)
			num_line += 1
		
		readfile.close()
		writefile.close()	
	elif Recovery_flag==1:
		readfile = open(JobName+'.inp', 'r')
		writefile = open(JobName+'-SimpackFlex.inp', 'w')

		read_line = readfile.readlines()
		num_line = 1
		for line in read_line:
			if num_line==len(read_line): 
				writefile.write('*FLEXIBLE BODY, TYPE=SIMPACK\n')
				writefile.write('*ELEMENT RECOVERY MATRIX, POSITION=AVERAGED AT NODES\n')
				writefile.write('S, E\n')
				writefile.write(line)
			else:
				writefile.write(line)
			num_line += 1
		
		readfile.close()
		writefile.close()
	#### ---------------------------------------------------------------
	####  	Final Input File for Abaqus to Simpack: JobName+'-SimpackFlex.inp'	
	os.system('del '+JobName+'-TMP.inp')

	p = mdb.models[ModelName].parts[PartName2]
        session.viewports['Viewport: 1'].setValues(displayedObject=p)

	###################################################################################

	print ' '
	print JobName+'.inp is created                 -- Substructure Generation'
	print JobName+'-SimpackFlex.inp is created     -- Substructure Generation for Simpack Flex Body'
	print ' '

	return 1
