from abaqusGui import *


###########################################################################
# Class definition
###########################################################################

class gearForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        
        # Construct the base class.
        #
        AFXForm.__init__(self, owner)
        self.radioButtonGroups = {}

        self.cmd = AFXGuiCommand(mode=self, method='gearUtility',
            objectName='abq_gear.gearUtils', registerQuery=False)
        pickedDefault = ''

	self.GEARPARAM_TableKw 	= AFXTableKeyword(self.cmd, 'GEARPARAM_Table', True, 0, -1, AFXTUPLE_TYPE_STRING)
        self.gearTypeKw 	= AFXStringKeyword(None, 'gearType', True, 'External')
        self.moduleKw 		= AFXStringKeyword(None, 'module', True, '3.')
        self.pressureAngleKw 	= AFXStringKeyword(None, 'pressureAngle', True, '20.')
        self.shiftFactorKw 	= AFXStringKeyword(None, 'shiftFactor', True, '0.')
        self.helixAngleKw 	= AFXStringKeyword(None, 'helixAngle', True, '0.')
        self.bevelAngleKw 	= AFXStringKeyword(None, 'bevelAngle', True, '0.')
        self.adddedTypeKw 	= AFXStringKeyword(None, 'adddedType', True, 'Coefficient')
        self.addendumValueKw 	= AFXStringKeyword(None, 'addendumValue', True, '1.')
        self.dedendumValueKw 	= AFXStringKeyword(None, 'dedendumValue', True, '1.')
        self.numberTeethKw 	= AFXStringKeyword(None, 'numberTeeth', True, '23')
        self.flankWidthKw 	= AFXStringKeyword(None, 'flankWidwth', True, '30.')
        self.boreDKw 		= AFXStringKeyword(None, 'boreD', True, '10.')
        self.ringDKw 		= AFXStringKeyword(None, 'ringD', True, '90.')
        self.approachTypeKw 	= AFXStringKeyword(None, 'approachType', True, 'A Approach')
        self.numDcoupNodeKw 	= AFXStringKeyword(None, 'numDcoupNode', True, '5')
        self.meshSizeKw 	= AFXStringKeyword(None, 'meshSize', True, '0.')
        self.gearRotXKw 	= AFXStringKeyword(None, 'gearRotX', True, '0.')
        self.gearRotYKw 	= AFXStringKeyword(None, 'gearRotY', True, '0.')
        self.gearRotZKw 	= AFXStringKeyword(None, 'gearRotZ', True, '-90')
        self.numEigKw 		= AFXStringKeyword(None, 'numEig', True, '10')
        self.subIdentNumKw	= AFXStringKeyword(None, 'subIdentNum', True, '101')
        self.recoveryKw		= AFXStringKeyword(None, 'recovery', True, 'No Recovery')
        self.meshTypeKw		= AFXStringKeyword(None, 'meshType', True, 'Hex-dominant')
        self.unitKw		= AFXStringKeyword(None, 'unit', True, 'millimeter')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        from abq_gear.gearDB import gearDB
        self.db = gearDB(self)
        return self.db

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        # Try to set the appropriate radio button on. If the user did
        # not specify any buttons to be on, do nothing.
        #
        for kw1,kw2,d in self.radioButtonGroups.values():
            try:
                value = d[ kw1.getValue() ]
                kw2.setValue(value)
            except:
                pass
        return True

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def okToCancel(self):

        # No need to close the dialog when a file operation (such
        # as New or Open) or model change is executed.
        #
        return False


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

	#---------------------------------------------------------------------------------------
	#	Gear Parameter	
        self.GEARPARAM_TableKw.setValue(0,0,  str(self.gearTypeKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,1,  str(self.moduleKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,2,  str(self.pressureAngleKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,3,  str(self.shiftFactorKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,4,  str(self.helixAngleKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,5,  str(self.bevelAngleKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,6,  str(self.adddedTypeKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,7,  str(self.addendumValueKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,8,  str(self.dedendumValueKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,9,  str(self.numberTeethKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,10, str(self.flankWidthKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,11, str(self.boreDKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,12, str(self.ringDKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,13, str(self.approachTypeKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,14, str(self.numDcoupNodeKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,15, str(self.meshSizeKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,16, str(self.gearRotXKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,17, str(self.gearRotYKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,18, str(self.gearRotZKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,19, str(self.numEigKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,20, str(self.subIdentNumKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,21, str(self.recoveryKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,22, str(self.meshTypeKw.getValue()))
        self.GEARPARAM_TableKw.setValue(0,23, str(self.unitKw.getValue()))

	return 1
