from abaqus import *
from abaqusConstants import *
import customKernel
from caeModules import *
import os
import math

###	Gear SubCode
from abq_gear import externalSpur
from abq_gear import externalHelical

from abq_gear import internalSpur
from abq_gear import internalHelical

def gearUtility(GEARPARAM_Table=None):

    print ' '
    print '------------------------------------------------------------------------------------------ '
    print 'Gear Type				: %s'%GEARPARAM_Table[0][0]
    print 'Gear Normal Module			: %s'%GEARPARAM_Table[0][1]
    print 'Gear Pressue Angle			: %s'%GEARPARAM_Table[0][2]
    print 'Gear Shift Factor			: %s'%GEARPARAM_Table[0][3]
    print 'Gear Helix Angle			: %s'%GEARPARAM_Table[0][4]
    print 'Gear Bevel Angle			: %s'%GEARPARAM_Table[0][5]
    print 'Gear Number of Teeth			: %s'%GEARPARAM_Table[0][9]
    print 'Gear Flank Width			: %s'%GEARPARAM_Table[0][10]
    if GEARPARAM_Table[0][0]=='External':
    	print 'Gear Bore Diameter			: %s'%GEARPARAM_Table[0][11]
    elif GEARPARAM_Table[0][0]=='Internal':
	print 'Gear Ring Diameter			: %s'%GEARPARAM_Table[0][12]
    else:
	raise ValueError, "Gear Type is not specified!!"
    print 'Gear Coupling Approach Type		: %s'%GEARPARAM_Table[0][13]
    print 'Gear Number of Coupling Nodes		: %s'%GEARPARAM_Table[0][14]
    print ' '
    print 'Gear Addendum/Dedendum Type		: %s'%GEARPARAM_Table[0][6]
    print 'Gear Addendum Type			: %s'%GEARPARAM_Table[0][7]
    print 'Gear Dedendum Type			: %s'%GEARPARAM_Table[0][8]
    print ' '
    print 'Mesh Size				: %s'%GEARPARAM_Table[0][15]
    print 'Gear Rotation Angle at Global X		: %s'%GEARPARAM_Table[0][16]
    print 'Gear Rotation Angle at Global Y		: %s'%GEARPARAM_Table[0][17]
    print 'Gear Rotation Angle at Global Z		: %s'%GEARPARAM_Table[0][18]
    print 'Number of Eigenvalues in Frequency Step	: %s'%GEARPARAM_Table[0][19]
    print 'Substructure Identification Number	: %s'%GEARPARAM_Table[0][20]
    print 'Recovery Flag				: %s'%GEARPARAM_Table[0][21]
    print 'Mesh Type				: %s'%GEARPARAM_Table[0][22]
    print 'Model Unit				: %s'%GEARPARAM_Table[0][23]
    print '------------------------------------------------------------------------------------------ '

    # ------------------------------------------------------------------------------------------------------

    GearType 	= GEARPARAM_Table[0][0]
    GearModule 	= float(GEARPARAM_Table[0][1])
    GearPressureAngle = float(GEARPARAM_Table[0][2])
    GearShiftFactor = float(GEARPARAM_Table[0][3])
    GearHelixAngle = float(GEARPARAM_Table[0][4])
    GearBevelAngle = float(GEARPARAM_Table[0][5])
    GearNumTeeth = int(GEARPARAM_Table[0][9])
    GearFlankWidth = float(GEARPARAM_Table[0][10])
    if GEARPARAM_Table[0][0]=='External':
    	GearBoreDiam = float(GEARPARAM_Table[0][11])
    elif GEARPARAM_Table[0][0]=='Internal':
    	GearRingDiam = float(GEARPARAM_Table[0][12])
    GearCouplingAppType = GEARPARAM_Table[0][13]
    if GearCouplingAppType=='A Approach': 
	    App = 1
    elif GearCouplingAppType=='B Approach': 
	    App = 2
    GearNumCouplingNode = int(GEARPARAM_Table[0][14])
    if GearNumCouplingNode==1:
	    GearWidthSectorNum = 1
    elif GearNumCouplingNode==2:
	    GearWidthSectorNum = 3
    elif GearNumCouplingNode==3:
	    GearWidthSectorNum = 5
    elif GearNumCouplingNode==4:
	    GearWidthSectorNum = 7
    elif GearNumCouplingNode==5:
	    GearWidthSectorNum = 9
    elif GearNumCouplingNode==6:
	    GearWidthSectorNum = 11
    elif GearNumCouplingNode==7:
	    GearWidthSectorNum = 13
    elif GearNumCouplingNode==8:
	    GearWidthSectorNum = 15
    elif GearNumCouplingNode==9:
	    GearWidthSectorNum = 17
    elif GearNumCouplingNode==10:
	    GearWidthSectorNum = 19
    GearAddDedType = GEARPARAM_Table[0][6]
    GearAddValue = float(GEARPARAM_Table[0][7])
    GearDedValue = float(GEARPARAM_Table[0][8])
    GearMeshSize = float(GEARPARAM_Table[0][15])
    GearRotX = float(GEARPARAM_Table[0][16])
    GearRotY = float(GEARPARAM_Table[0][17])
    GearRotZ = float(GEARPARAM_Table[0][18])
    GearNumEig = int(GEARPARAM_Table[0][19])
    GearSubIdent = int(GEARPARAM_Table[0][20])
    GearRecovery = GEARPARAM_Table[0][21]
    if GearRecovery=='No Recovery':
	    Recovery_flag = 0
    elif GearRecovery=='Recovery':
	    Recovery_flag = 1
    GearMeshType = GEARPARAM_Table[0][22]
    GearUnit = GEARPARAM_Table[0][23]

    # ------------------------------------------------------------------------------------------------------

    ModelName = 'GEAR_FE'	# Model Name
    JobName   = 'GEAR01SUBSTR'	# Job Name
    PartName  = 'GEAR01P'	# Part Name 

    pangle = GearPressureAngle * math.pi/180. 
    hangle = GearHelixAngle * math.pi/180. 
    SHFT = GearShiftFactor
    N1 = GearNumTeeth 
    module = GearModule 
    ADC = GearAddValue 
    DDC = GearDedValue 
    Width = GearFlankWidth
    GTN = GearNumCouplingNode
    WSC = GearWidthSectorNum
    MeshType = GearMeshType
    meshsize = GearMeshSize
    if GEARPARAM_Table[0][0]=='External':
	    Dc = GearBoreDiam
    elif GEARPARAM_Table[0][0]=='Internal':
	    D_ring = GearRingDiam

    if hangle==0.0:
	    HelixType = 0
    else:
	    HelixType = 1
	    pangle = math.atan(math.tan(pangle)/math.cos(hangle))
    if GearType==2: 
	    SHFT = -1.*SHFT
    
    # Check Point
    # print HelixType

    if GearAddDedType=='Coefficient':
	    Da = N1*module/math.cos(hangle) + (ADC+SHFT)*module*2.  ## Addendum Diameter ( User can change it ) 
	    Dd = N1*module/math.cos(hangle) - (DDC-SHFT)*module*2.  ## Dedendum Diameter ( User can change it ) 
    elif GearAddDedType=='Diameter':
	    Da = ADC  ## Addendum Diameter ( User can change it ) 
	    Dd = DDC  ## Dedendum Diameter ( User can change it ) 
    else:
	raise ValueError, "Gear Addendum/Dedendum Type is not specified!!"

    ##	---------------------------------------------------------------
    ## Matching Simpack Gear Wheel Definition
    Ra = Da/2.
    Rd = Dd/2.
    ####
    Rp = ((N1*module)/math.cos(hangle)+2.*SHFT*module)/2.
    Rb = ((N1*module)/math.cos(hangle))*math.cos(pangle)/2.
    ####
    Rr = Rd
    Dr = Rr*2.
    ####
    if GEARPARAM_Table[0][0]=='External':
	    Rc = Dc/2.
    elif GEARPARAM_Table[0][0]=='Internal':
	    R_ring = D_ring/2.
	    Rc = Rb*0.001
    ##	---------------------------------------------------------------
    ##	---------------------------------------------------------------
    print ' '
    print '------------------------------------------------------------------------------------------ '
    print 'Addendum Radius (Ra): ' + str(Ra)
    print 'Pitch Radius    (Rp): ' + str(Rp)
    print 'Dedendum Radius (Rd): ' + str(Rd)
    print 'Root Radius     (Rr): ' + str(Rr)
    if GEARPARAM_Table[0][0]=='External':
	    print 'Bore Radius     (Rc): ' + str(Rc)
    elif GEARPARAM_Table[0][0]=='Internal':
	    print 'Ring Radius     (R_ring): ' + str(D_ring/2.)
    print 'Base Radius     (Rb): ' + str(Rb)
    print '------------------------------------------------------------------------------------------ '
    print ' '
    ##	---------------------------------------------------------------
    if GEARPARAM_Table[0][0]=='External':
	    if (Ra>Rp) and (Rp>Rd) and (Rd>Rc):
		    print ' ' 
	            print ' Satisfied: Ra > Rp > Rd > Rc'
	            print ' Assuming Rd = Rr'
        	    print ' Satisfied: Addendum Radius > Pitch Radius > Dedendum Radius > Bore Radius'
        	    print ' '
        	    pass
            else: 
	            raise ValueError, 'Ra > Rp > Rd > Rc is not satisfied!!!'

            if Rc>0: 
        	    pass
            else: 
        	    raise ValueError, 'Bore Radius should be greater than zero!!!'
    elif GEARPARAM_Table[0][0]=='Internal':
	    if (D_ring>Ra) and (Ra>Rp) and (Rp>Rd) :
		    print ' ' 
	            print ' Satisfied: R_ring > Ra > Rp > Rd '
	            print ' Assuming Rd = Rr'
        	    print ' Satisfied: Ring Radius > Addendum Radius > Pitch Radius > Dedendum Radius'
        	    print ' '
            else: 
	            raise ValueError, 'R_ring > Ra > Rp > Rd is not satisfied!!!'

            if R_ring>0: 
        	    pass
            else: 
        	    raise ValueError, 'Ring Radius should be greater than zero!!!'
    ##	---------------------------------------------------------------    
    Narc = int(10)
    beta_p = math.sqrt((Rp**2)/(Rb**2)-1)
    delta = 0.5*math.pi/N1 
    gamma = (0.5*math.pi/N1) + (beta_p - math.atan(beta_p))

    print ' '
    print 'Number of points for Arc     : ' + str(Narc)
    print 'Tooth Sector Angle           : ' + str(2*gamma*180./math.pi) + ' Degree'
    print 'The Whole Sector Angle       : ' + str(4*delta*180./math.pi) + ' Degree'
    #### ----------------------------------------------------------------------------

    ##################################################################################
    ####  Involute Points
    ####	       For detail algorithm, please check "Involute Algorithm.pptx"
    Involute_x = []
    Involute_y = []
    if Rd >= Rb:
	    Asector = (Rp-Rd)/Narc
    else: 
	    Asector = (Rp-Rb)/Narc
    print ' '
    print '--------------------------------------------------------- '
    print '   Involute Points: X, Y Values'
    print ' '
    for i in range(Narc+1): 
	    if Rd >= Rb: 
		    Rn = Rd+Asector*i 
	    else: 
		    Rn = Rb+Asector*i
	    beta_n = math.sqrt( (Rn**2)/(Rb**2) - 1. )
	    Kapa_n = gamma - ( beta_n - math.atan(beta_n) )
	    Involute_x.append(Rn*math.sin(Kapa_n))
	    Involute_y.append(Rn*math.cos(Kapa_n))
	    print str(Rn*math.sin(Kapa_n))+', '+str(Rn*math.cos(Kapa_n))
    Asector = (Ra-Rp)/Narc
    for i in range(Narc):
	    Rn = Rp+Asector*(i+1)
	    beta_n = math.sqrt( (Rn**2)/(Rb**2) - 1. )
	    Kapa_n = gamma - ( beta_n - math.atan(beta_n) )
	    Involute_x.append(Rn*math.sin(Kapa_n))
	    Involute_y.append(Rn*math.cos(Kapa_n))
	    print str(Rn*math.sin(Kapa_n))+', '+str(Rn*math.cos(Kapa_n))
    print ' '
    print '--------------------------------------------------------- '
    print ' '
    Ipnts = []
    if Rd<Rb:
	    Ipnts.append((Rd*math.sin(gamma),Rd*math.cos(gamma)))
    for i in range(len(Involute_x)): 
	    Ipnts.append((Involute_x[i], Involute_y[i]))
    ##	---------------------------------------------------------------
    ##	sector end edge point
    p2_x = Rr*math.sin(2.*delta)
    p2_y = Rr*math.cos(2.*delta)
    ##	---------------------------------------------------------------
    ##	teeth end edge point crossed by root
    if Rd>=Rb:
	    p3_x = Involute_x[0]
	    p3_y = Involute_y[0]
    else:
	    p3_x = Rd*math.sin(gamma)
	    p3_y = Rd*math.cos(gamma)
    ##      rename of gamma angle
    g_angle = gamma
    print ' Sector Edge End point: '+str(p2_x)+', '+str(p2_y)
    print ' Teeth  Edge End point: '+str(p3_x)+', '+str(p3_y)
    print ' '
    print '--------------------------------------------------------- '
    print ' '
    p4_x = Involute_x[len(Involute_x)-1]
    p4_y = Involute_y[len(Involute_y)-1]
    ##################################################################################
    ##################################################################################
    ####	A/CAE Model Creation
    from abaqus import *
    from abaqusConstants import *
    session.viewports['Viewport: 1'].makeCurrent()
    session.viewports['Viewport: 1'].maximize()
    session.journalOptions.setValues(replayGeometry=COORDINATE, recoverGeometry=COORDINATE)
    import sys
    from caeModules import *
    from driverUtils import executeOnCaeStartup
    executeOnCaeStartup()
    session.viewports['Viewport: 1'].partDisplay.geometryOptions.setValues( referenceRepresentation=ON)
    Mdb()
    session.memoryReductionOptions.setValues(percentThreshold=90)
    #### ---------------------------------------------------------------
    #### 	Create Involute Profile (Half)
    session.viewports['Viewport: 1'].setValues(displayedObject=None)
    mdb.models.changeKey(fromName='Model-1', toName=ModelName)
    s = mdb.models[ModelName].ConstrainedSketch(name='__profile__', sheetSize=200.0)
    g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
    s.setPrimaryObject(option=STANDALONE)
    s.Line(point1=(0.0, 0.0), point2=(p2_x,p2_y))
    s.Line(point1=(p3_x,p3_y), point2=Ipnts[0])
    s.ArcByCenterEnds(center=(0.0, 0.0), point1=(p3_x, p3_y), point2=(p2_x,p2_y), direction=CLOCKWISE)
    s.Line(point1=(0.0, 0.0), point2=(0.0, Ra))
    s.Spline(points=Ipnts, )
    s.ArcByCenterEnds(center=(0.0, 0.0), point1=(0.0, Ra), point2=Ipnts[len(Ipnts)-1], direction=CLOCKWISE)
    p = mdb.models[ModelName].Part(name=PartName, dimensionality=THREE_D, type=DEFORMABLE_BODY)
    p = mdb.models[ModelName].parts[PartName]
    p.BaseSolidExtrude(sketch=s, depth=Width)
    s.unsetPrimaryObject()
    del mdb.models[ModelName].sketches['__profile__']
    #### ---------------------------------------------------------------
    ####	Mirror for another half of Involute teeth profile 
    f = p.faces
    face = f.findAt((0,p2_y,1))
    p.Mirror(mirrorPlane=face, keepOriginal=ON)
    #### ---------------------------------------------------------------    
    #### 	Bore Cut
    p = mdb.models[ModelName].parts[PartName]
    f, e = p.faces, p.edges
    face = f.findAt((0,p2_y*0.5,0))
    edge = e.findAt((-p2_x*0.99,p2_y*0.99,0),)
    t = p.MakeSketchTransform(sketchPlane=face, sketchUpEdge=edge, sketchPlaneSide=SIDE1, 
		    sketchOrientation=RIGHT, origin=(0.0, 0.0, 0.0))
    s = mdb.models[ModelName].ConstrainedSketch(name='__profile__', sheetSize=100.00, gridSpacing=5, 
		    transform=t)
    g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
    s.setPrimaryObject(option=SUPERIMPOSE)
    p = mdb.models[ModelName].parts[PartName]
    p.projectReferencesOntoSketch(sketch=s, filter=COPLANAR_EDGES)
    s.CircleByCenterPerimeter(center=(0, 0), point1=(0, Rc))
    p = mdb.models[ModelName].parts[PartName]
    f1, e1 = p.faces, p.edges
    p.CutExtrude(sketchPlane=face, sketchUpEdge=edge, sketchPlaneSide=SIDE1, sketchOrientation=RIGHT, 
		    sketch=s, flipExtrudeDirection=OFF)
    s.unsetPrimaryObject()
    del mdb.models[ModelName].sketches['__profile__']    
    #### ---------------------------------------------------------------
    if GearType=='External':
		print ' '
		print ' ------------ External Gear Type'
		print ' '
		if HelixType==0:
			print ' '
		        print ' ---------------- Spur Gear'
        		print ' '
        		flag = externalSpur.externalSpur(ModelName, PartName, p2_x, p2_y, Rr, Rc, Ra, Rp, delta, WSC, GTN, Width, MeshType, meshsize, 
					App, GearRotX, GearRotY, GearRotZ, GearSubIdent, Recovery_flag, GearNumTeeth, JobName, GearNumEig)
        	elif HelixType==1:
        		print ' '
        		print ' ---------------- Helical Gear'
        		print ' '
        		flag = externalHelical.externalHelical(ModelName, PartName, p2_x, p2_y, Rr, Rc, Ra, Rp, delta, WSC, GTN, Width, MeshType, meshsize, 
					App, GearRotX, GearRotY, GearRotZ, GearSubIdent, Recovery_flag, GearNumTeeth, JobName, GearNumEig, hangle)
    elif GearType=='Internal':
		print ' '
		print ' ------------ Internal Gear Type'
		print ' '
        	if HelixType==0:
        		print ' '
        		print ' ---------------- Spur Gear'
        		print ' '		
        		flag = internalSpur.internalSpur(ModelName, PartName, p2_x, p2_y, Rr, Rc, Ra, Rp, delta, WSC, GTN, Width, MeshType, meshsize, 
					App, GearRotX, GearRotY, GearRotZ, GearSubIdent, Recovery_flag, GearNumTeeth, JobName, GearNumEig, D_ring)
        	elif HelixType==1:
        		print ' '
        		print ' ---------------- Helical Gear'
        		print ' '
        		flag = internalHelical.internalHelical(ModelName, PartName, p2_x, p2_y, Rr, Rc, Ra, Rp, Rd, delta, WSC, GTN, Width, MeshType, meshsize, 
					App, GearRotX, GearRotY, GearRotZ, GearSubIdent, Recovery_flag, GearNumTeeth, JobName, GearNumEig, hangle, D_ring)
    #### ---------------------------------------------------------------
    print ' '
    os.system('copy abaqus.rpy script.log')
    print ' '
    print ' '
    print ' Plug-in Complete'
    print ' '
    print ' '
