from abaqus import *
from abaqusConstants import *
import customKernel
import sys, os
from caeModules import *
import uti, math

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def internalHelical(ModelName, PartName, p2_x, p2_y, Rr, Rc, Ra, Rp, Rd, delta, WSC, GTN, Width, MeshType, meshsize, 
		App, AngX, AngY, AngZ, GearSubIdent, Recovery_flag, number_sector, JobName, NumEig, hangle, D_ring):

	R_ring = D_ring/2.
	p2_x = R_ring*math.sin(2.*delta)
	p2_y = R_ring*math.cos(2.*delta)

	#### ---------------------------------------------------------------
	#### 	Create Involute Profile
	PartName_2 = 'Part-2'
	session.viewports['Viewport: 1'].setValues(displayedObject=None)
	s = mdb.models[ModelName].ConstrainedSketch(name='__profile__', sheetSize=200.0)
	g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
	s.setPrimaryObject(option=STANDALONE)
	s.Line(point1=(0.0, 0.0), point2=(p2_x,p2_y))
	s.ArcByCenterEnds(center=(0.0, 0.0), point1=(-p2_x, p2_y), point2=(p2_x,p2_y) , direction=CLOCKWISE)
	s.Line(point1=(-p2_x, p2_y), point2=(0.0, 0.0))
	p = mdb.models[ModelName].Part(name=PartName_2, dimensionality=THREE_D, type=DEFORMABLE_BODY)
	p = mdb.models[ModelName].parts[PartName_2]
	p.BaseSolidExtrude(sketch=s, depth=Width)
	s.unsetPrimaryObject()
	del mdb.models[ModelName].sketches['__profile__']

	#### ---------------------------------------------------------------
	#### 	Bore Cut
	p = mdb.models[ModelName].parts[PartName_2]
	f, e = p.faces, p.edges
	face = f.findAt((0,p2_y*0.5,0))
	edge = e.findAt((-p2_x*0.99,p2_y*0.99,0),)
	t = p.MakeSketchTransform(sketchPlane=face, sketchUpEdge=edge, 
	    sketchPlaneSide=SIDE1, sketchOrientation=RIGHT, origin=(0.0, 0.0, 0.0))
	s = mdb.models[ModelName].ConstrainedSketch(name='__profile__', 
	    sheetSize=100.00, gridSpacing=5, transform=t)
	g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
	s.setPrimaryObject(option=SUPERIMPOSE)
	p = mdb.models[ModelName].parts[PartName_2]
	p.projectReferencesOntoSketch(sketch=s, filter=COPLANAR_EDGES)
	s.CircleByCenterPerimeter(center=(0, 0), point1=(0, Rc))
	p = mdb.models[ModelName].parts[PartName_2]
	f1, e1 = p.faces, p.edges
	p.CutExtrude(sketchPlane=face, sketchUpEdge=edge, sketchPlaneSide=SIDE1, 
	    sketchOrientation=RIGHT, sketch=s, flipExtrudeDirection=OFF)
	s.unsetPrimaryObject()
	del mdb.models[ModelName].sketches['__profile__']

	#### ---------------------------------------------------------------
	#### 	Create Sector 
	a = mdb.models[ModelName].rootAssembly
	p   = mdb.models[ModelName].parts[PartName]
	a.Instance(name=PartName+'-1', part=p, dependent=ON)
	p_2 = mdb.models[ModelName].parts[PartName_2]
	a.Instance(name=PartName_2+'-1', part=p_2, dependent=ON)
	a.InstanceFromBooleanCut(name='Part-3', 
	    instanceToBeCut=mdb.models[ModelName].rootAssembly.instances[PartName_2+'-1'], 
	    cuttingInstances=(a.instances[PartName+'-1'], ), originalInstances=DELETE)

	a = mdb.models[ModelName].rootAssembly
	a.DatumAxisByPrincipalAxis(principalAxis=ZAXIS)
	a.RadialInstancePattern(instanceList=('Part-3-1', ), point=(0.0, 0.0, 0.0), 
	    axis=(0.0, 0.0, 1.0), number=2, totalAngle=4.*delta*180/3.1416)
	a.InstanceFromBooleanMerge(name='Part-4', instances=(a.instances['Part-3-1'], 
	    a.instances['Part-3-1-rad-2'], ), originalInstances=DELETE, 
	    domain=GEOMETRY)

	a.rotate(instanceList=('Part-4-1', ), axisPoint=(0.0, 0.0, 0.0), 
	    axisDirection=(0.0, 0.0, 1.0), angle=-2.*delta*180/3.14159)
	a.RadialInstancePattern(instanceList=('Part-4-1', ), point=(0.0, 0.0, 0.0), 
	    axis=(0.0, 0.0, 1.0), number=2, totalAngle=180.0)
	a.InstanceFromBooleanMerge(name='Part-5', instances=(a.instances['Part-4-1'], 
	    a.instances['Part-4-1-rad-2'], ), originalInstances=DELETE, 
	    domain=GEOMETRY)
	a.deleteFeatures(('Part-4-1', 'Part-4-1-rad-2', ))
	p = mdb.models[ModelName].parts['Part-5']
	f = p.faces
	p.RemoveFaces(faceList=(f.getByBoundingBox(-1.e20,-1.e20,-1.e20,1.e20,0.0,1.e20)), deleteCells=False)

	tmp = []
	for i in range(len(f)):
		if f[i].pointOn[0][2]!=0.0:
			tmp.append(f[i])

	p.RemoveFaces(faceList=(tmp), deleteCells=False)

	p3_x = 1.2*R_ring*math.sin(2.*delta)
	p3_y = 1.2*R_ring*math.cos(2.*delta)

	session.viewports['Viewport: 1'].setValues(displayedObject=None)
	s = mdb.models[ModelName].ConstrainedSketch(name='__profile__', sheetSize=200.0)
	g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
	s.setPrimaryObject(option=STANDALONE)
	s.Line(point1=(0.0, 0.0), point2=(p3_x,p3_y))
	s.Line(point1=(-p3_x, p3_y), point2=(0.0, 0.0))
	p = mdb.models[ModelName].Part(name='Part-6', dimensionality=THREE_D, type=DEFORMABLE_BODY)
	p = mdb.models[ModelName].parts['Part-6']
	p.BaseShellExtrude(sketch=s, depth=Width)
	s.unsetPrimaryObject()
	del mdb.models[ModelName].sketches['__profile__']

	#### ---------------------------------------------------------------
	a = mdb.models[ModelName].rootAssembly
	p   = mdb.models[ModelName].parts['Part-5']
	a.Instance(name='Part-5-1', part=p, dependent=ON)
	p_2 = mdb.models[ModelName].parts['Part-6']
	a.Instance(name='Part-6-1', part=p_2, dependent=ON)
	a.InstanceFromBooleanCut(name='Part-7', 
	    instanceToBeCut=mdb.models[ModelName].rootAssembly.instances['Part-5-1'], 
	    cuttingInstances=(a.instances['Part-6-1'], ), originalInstances=DELETE)

	#### ---------------------------------------------------------------
	p = mdb.models[ModelName].parts['Part-7']
	f = p.faces
	face_tmp=[]
	for i in range(len(f)):
		face_tmp.append(f[i])
			
	for ii in range(len(face_tmp)):
		for jj in range(len(face_tmp)-ii):
			a = face_tmp[ii].pointOn[0][0]
			b = face_tmp[ii+jj].pointOn[0][0]

			if a>b:
				tmp = face_tmp[ii]
				face_tmp[ii] = face_tmp[ii+jj]
				face_tmp[ii+jj] = tmp 	

	p.RemoveFaces(faceList=(face_tmp[0],), deleteCells=False)
	p.RemoveFaces(faceList=(face_tmp[2],), deleteCells=False)

	#### ---------------------------------------------------------------
	#### 	Copy Part
	p = mdb.models[ModelName].Part(name='Part-7'+'-Copy', 
			objectToCopy=mdb.models[ModelName].parts['Part-7'])

	#raise ValueError, "CHECK 001 !!"
	#### ---------------------------------------------------------------
	PartName = 'Part-7'+'-Copy'
	p = mdb.models[ModelName].parts[PartName]
	f, e, d = p.faces, p.edges, p.datums
	face = f.findAt((0,p2_y*0.99,0))
	edge = e.findAt((-p2_x*0.99,p2_y*0.99,0),)
	t = p.MakeSketchTransform(sketchPlane=face, sketchUpEdge=edge, 
	    sketchPlaneSide=SIDE1, origin=(0.0, 0.0, 0.0))
	s = mdb.models[ModelName].ConstrainedSketch(name='__profile__', 
	    sheetSize=100.00, gridSpacing=5, transform=t)
	g, v, d1, c = s.geometry, s.vertices, s.dimensions, s.constraints
	s.setPrimaryObject(option=SUPERIMPOSE)
	p.projectReferencesOntoSketch(sketch=s, filter=COPLANAR_EDGES)
	s.CircleByCenterPerimeter(center=(0.0, 0.0), point1=(0.0, Ra))
	p.PartitionFaceBySketch(sketchUpEdge=edge, faces=face, sketch=s)
	s.unsetPrimaryObject()
	del mdb.models[ModelName].sketches['__profile__']

	#raise ValueError, "CHECK 002 !!"
	#### ---------------------------------------------------------------
	#### 	Copy Part
	p = mdb.models[ModelName].Part(name='RING01-01', 
			objectToCopy=mdb.models[ModelName].parts[PartName])

	PartName = 'RING01-01'

	del mdb.models[ModelName].parts['Part-2']
	del mdb.models[ModelName].parts['Part-3']
	del mdb.models[ModelName].parts['Part-4']
	del mdb.models[ModelName].parts['Part-5']
	del mdb.models[ModelName].parts['Part-6']
	a = mdb.models[ModelName].rootAssembly
	a.regenerate()
	del a.features['Part-7-1']
	del mdb.models[ModelName].parts['Part-7']
	del mdb.models[ModelName].parts['Part-7-Copy']

	#raise ValueError, "CHECK 003 !!"
	#### ---------------------------------------------------------------
	#### 	Create Angle for Helix Angle 
	#rotangle = float(Width)*tan(float(hangle))/float(Ra)
	rotangle = float(Width)*tan(float(hangle))/float(Rp)

	print ' '
	print 'Rotation Angle for '+str(hangle*180./math.pi)+' Helix Angle: '+str(rotangle*180./math.pi)+' Degree'
	print ' '
	
	if WSC!=1:
		rotangle = rotangle/WSC
		Width = Width/WSC

	print ' '
	print 'Divided by '+str(WSC)+' Section in width direction '
	print ' '
	#raise ValueError, "CHECK 004 !!"
	#### ---------------------------------------------------------------
	#### 	Create Sector with Helical Teeth 
	a = mdb.models[ModelName].rootAssembly
	p = mdb.models[ModelName].parts[PartName]
	a.Instance(name=PartName+'-1', part=p, dependent=ON)
	a.Instance(name=PartName+'-2', part=p, dependent=ON)
	a.translate(instanceList=(PartName+'-2', ), vector=(0.0, 0.0, Width))
	a.rotate(instanceList=(PartName+'-2', ), axisPoint=(0.0, 0.0, 0.0), axisDirection=(
		0.0, 0.0, 1.0), angle=rotangle*180./math.pi)
	a.InstanceFromBooleanMerge(name=PartName+'01', instances=(a.instances[PartName+'-1'], 
	    a.instances[PartName+'-2'], ), originalInstances=DELETE, domain=GEOMETRY)

	#raise ValueError, "CHECK 005 !!"
	#### ---------------------------------------------------------------
	#### 	Create Wire for Helical reference line
	p = mdb.models[ModelName].parts[PartName+'01']

	numPoints = WSC*2
	Helix = []
	Helix_angle = []
	####

	for ip in range(numPoints+1):
		theta = (float(ip)/float(numPoints))*float(-1.*rotangle)

		if App==1:
			fx = float(Ra)*math.sin(theta)
			fy = float(Ra)*math.cos(theta)
		elif App==2:
			fx = float(Rp)*math.sin(theta)
			fy = float(Rp)*math.cos(theta)
		else:
			raise ValueError, "Approach should be 1 or 2 !!"			
		fz = (float(ip)/float(numPoints))*float(Width)
		
		if ip%2==0.0:
			Helix.append((fx,fy,fz))
			Helix_angle.append(theta)

	d = p.datums
	l = len(p.datums)

	#raise ValueError, "CHECK 006 !!"
	#### ---------------------------------------------------------------
	#### 	Create Loft Sector with Helical Teeth 
	p = mdb.models[ModelName].parts[PartName+'01']
	e = p.edges
	edges01 = e.getByBoundingBox(-1e20,-1e20,-1,1e20,1e20,Width/2.)

	if len(edges01)!=11:
		print len(edges01)
		raise ValueError, " Number of Edges should be 11 - Check 01 !!"

	ed01 = []
	for nume01 in range(len(edges01)):
		ed01.append(edges01[nume01])

	for nume01 in range(len(ed01)):
		for nume02 in range(len(ed01)-1):
			a = ed01[nume01].pointOn[0][1]
			b = ed01[nume02].pointOn[0][1]
			if a > b:
				temp = ed01[nume01]
				ed01[nume01] = ed01[nume02]
				ed01[nume02] = temp 

	edges001 = []
	edges001.append(ed01[4])
	edges001.append(ed01[5])
	edges001.append(ed01[6])

	for nume01 in range(len(edges001)):
		for nume02 in range(len(edges001)-1):
			a = edges001[nume01].pointOn[0][0]
			b = edges001[nume02].pointOn[0][0]
			if a > b:
				temp = edges001[nume01]
				edges001[nume01] = edges001[nume02]
				edges001[nume02] = temp 

	edges01 = []
	for nume01 in range(4):
		edges01.append(ed01[nume01])
	#edges01.append(edges001[1])
	for nume01 in range(4):
		edges01.append(ed01[nume01+7])
	edges01.append(edges001[0])
	edges01.append(edges001[2])

	edges02 = e.getByBoundingBox(-1e20,-1e20,Width/2.,1e20,1e20,Width+1.)

	if len(edges02)!=11:
		print len(edges02)
		raise ValueError, " Number of Edges should be 11 - Check 02 !!"

	ed02 = []
	for nume01 in range(len(edges02)):
		ed02.append(edges02[nume01])

	for nume01 in range(len(ed02)):
		for nume02 in range(len(ed02)-1):
			a = ed02[nume01].pointOn[0][1]
			b = ed02[nume02].pointOn[0][1]
			if a > b:
				temp = ed02[nume01]
				ed02[nume01] = ed02[nume02]
				ed02[nume02] = temp 
	edges002 = []
	edges002.append(ed02[4])
	edges002.append(ed02[5])
	edges002.append(ed02[6])

	for nume01 in range(len(edges002)):
		for nume02 in range(len(edges002)-1):
			a = edges002[nume01].pointOn[0][0]
			b = edges002[nume02].pointOn[0][0]
			if a > b:
				temp = edges002[nume01]
				edges002[nume01] = edges002[nume02]
				edges002[nume02] = temp 

	edges02 = []
	for nume01 in range(4):
		edges02.append(ed02[nume01])
	#edges02.append(edges002[1])
	for nume01 in range(4):
		edges02.append(ed02[nume01+7])
	edges02.append(edges002[0])
	edges02.append(edges002[2])

	e1 = p.edges
	for i in range(len(e1)):
		if 0<e1[i].pointOn[0][2] and e1[i].pointOn[0][2]<Width:
			pathEdges = e1[i]
	fx = pathEdges.pointOn[0][0]
	fy = pathEdges.pointOn[0][1]
	fz = pathEdges.pointOn[0][2]

	p.SolidLoft(loftsections=((edges01),(edges02)), globalSmoothing=ON, keepInternalBoundaries=True)

	#raise ValueError, "CHECK 007 !!"
	#### ---------------------------------------------------------------
	####	Selecting internal 4 edges
	p = mdb.models[ModelName].parts[PartName+'01']
	session.viewports['Viewport: 1'].setValues(displayedObject=p)
	e = p.edges

	temp01_y = Ra*math.cos((2.*delta))
	edges01 = e.getByBoundingBox(-1e20,temp01_y*0.999,-1,1e20,Ra*1.001,Width/2.)
	temp_y = Ra*math.cos((2.*delta+rotangle))
	edges02 = e.getByBoundingBox(-1e20,temp_y*0.999,Width/2.,1e20,Ra*1.001,Width+1.)
	edges03 = e.getByBoundingBox(-1e20,temp_y*0.999,-1,1e20,Ra*1.001,Width+1.)

	if len(edges01)!=3:
		print len(edges01)
		raise ValueError, "Number of Edge should be 3 !! - Track01"
	if len(edges02)!=3:
		print len(edges02)
		raise ValueError, "Number of Edge should be 3 !! - Track02"
	if len(edges03)!=10:
		print len(edges03)
		raise ValueError, "Number of Edge should be 10 !! - Track03"

	final4edges = []
	ed01 = []
	for nume01 in range(len(edges01)):
		ed01.append(edges01[nume01])
	for nume01 in range(len(ed01)):
		for nume02 in range(len(ed01)-1):
			a = ed01[nume01].pointOn[0][0]
	 		b = ed01[nume02].pointOn[0][0]
	 		if a > b:
	 			temp = ed01[nume01]
	 			ed01[nume01] = ed01[nume02]
	 			ed01[nume02] = temp 
	final4edges.append(ed01[1])
	ed02 = []
	for nume01 in range(len(edges02)):
	 	ed02.append(edges02[nume01])
	for nume01 in range(len(ed02)):
	 	for nume02 in range(len(ed02)-1):
	 		a = ed02[nume01].pointOn[0][0]
	 		b = ed02[nume02].pointOn[0][0]
	 		if a > b:
	 			temp = ed02[nume01]
	 			ed02[nume01] = ed02[nume02]
	 			ed02[nume02] = temp 
	final4edges.append(ed02[1])
	ed03 = []
	for nume01 in range(len(edges03)):
		if 0.<edges03[nume01].pointOn[0][2] and edges03[nume01].pointOn[0][2]<Width*0.999:
			ed03.append(edges03[nume01])
	for nume01 in range(len(ed03)):
	 	for nume02 in range(len(ed03)-1):
	 		a = ed03[nume01].pointOn[0][0]
	 		b = ed03[nume02].pointOn[0][0]
	 		if a > b:
	 			temp = ed03[nume01]
	 			ed03[nume01] = ed03[nume02]
	 			ed03[nume02] = temp 

	if len(ed03)!=4:
		raise ValueError, "Edge length should be 4 !! - Track04"

	final4edges.append(ed03[1])
	final4edges.append(ed03[2])
		
	p.CoverEdges(edgeList=(final4edges), tryAnalytical=False)

	#raise ValueError, "CHECK 008 !!"
	#### ---------------------------------------------------------------
	a1 = mdb.models[ModelName].rootAssembly
	p = mdb.models[ModelName].parts[PartName+'01']
	for iflag in range(WSC):
		a1.Instance(name=PartName+'01'+'-'+str(iflag+1), part=p, dependent=ON)
		if iflag>0:
			a1.translate(instanceList=(PartName+'01'+'-'+str(iflag+1), ), vector=(0.0, 0.0, Width*iflag))
			a1.rotate(instanceList=(PartName+'01'+'-'+str(iflag+1), ), axisPoint=(0.0, 0.0, 0.0), 
			    axisDirection=(0.0, 0.0,1.0), angle=(rotangle*float(iflag)*180./math.pi))

	if WSC==1:
		p = mdb.models[ModelName].Part(name=PartName+'01_FINAL', 
			objectToCopy=mdb.models[ModelName].parts[PartName+'01'])
		pass
	elif WSC==3:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==5:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==7:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==9:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], a1.instances[PartName+'01'+'-8'], a1.instances[PartName+'01'+'-9'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==11:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], a1.instances[PartName+'01'+'-8'], a1.instances[PartName+'01'+'-9'], 
	            a1.instances[PartName+'01'+'-10'], a1.instances[PartName+'01'+'-11'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==13:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], a1.instances[PartName+'01'+'-8'], a1.instances[PartName+'01'+'-9'], 
	            a1.instances[PartName+'01'+'-10'], a1.instances[PartName+'01'+'-11'], a1.instances[PartName+'01'+'-12'], 
	            a1.instances[PartName+'01'+'-13'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==15:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], a1.instances[PartName+'01'+'-8'], a1.instances[PartName+'01'+'-9'], 
	            a1.instances[PartName+'01'+'-10'], a1.instances[PartName+'01'+'-11'], a1.instances[PartName+'01'+'-12'], 
	            a1.instances[PartName+'01'+'-13'], a1.instances[PartName+'01'+'-14'], a1.instances[PartName+'01'+'-15'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==17:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], a1.instances[PartName+'01'+'-8'], a1.instances[PartName+'01'+'-9'], 
	            a1.instances[PartName+'01'+'-10'], a1.instances[PartName+'01'+'-11'], a1.instances[PartName+'01'+'-12'], 
	            a1.instances[PartName+'01'+'-13'], a1.instances[PartName+'01'+'-14'], a1.instances[PartName+'01'+'-15'], 
	            a1.instances[PartName+'01'+'-16'], a1.instances[PartName+'01'+'-17'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)
	elif WSC==19:
		a1.InstanceFromBooleanMerge(name=PartName+'01_FINAL', instances=(
	            a1.instances[PartName+'01'+'-1'], a1.instances[PartName+'01'+'-2'], a1.instances[PartName+'01'+'-3'],  
	            a1.instances[PartName+'01'+'-4'], a1.instances[PartName+'01'+'-5'], a1.instances[PartName+'01'+'-6'],  
	            a1.instances[PartName+'01'+'-7'], a1.instances[PartName+'01'+'-8'], a1.instances[PartName+'01'+'-9'], 
	            a1.instances[PartName+'01'+'-10'], a1.instances[PartName+'01'+'-11'], a1.instances[PartName+'01'+'-12'], 
	            a1.instances[PartName+'01'+'-13'], a1.instances[PartName+'01'+'-14'], a1.instances[PartName+'01'+'-15'], 
	            a1.instances[PartName+'01'+'-16'], a1.instances[PartName+'01'+'-17'], a1.instances[PartName+'01'+'-18'], 
	            a1.instances[PartName+'01'+'-19'], ), 
		    keepIntersections=ON, originalInstances=DELETE, domain=GEOMETRY)

	#raise ValueError, "CHECK 009 !!"
	#### ---------------------------------------------------------------

	if WSC!=1:
		rotangle = rotangle*WSC
		Width = Width*WSC

	Helix = []
	Helix_angle = []
	####

	for ip in range(numPoints+1):
		theta = (float(ip)/float(numPoints))*float(-1.*rotangle)

		if App==1:
			fx = float(Ra)*math.sin(theta)
			fy = float(Ra)*math.cos(theta)
		elif App==2:
			fx = float(Rp)*math.sin(theta)
			fy = float(Rp)*math.cos(theta)
		else:
			raise ValueError, "Approach should be 1 or 2 !!"			

		fz = (float(ip)/float(numPoints))*float(Width)
	
		if ip%2==0.0:
			Helix.append((fx,fy,fz))
			Helix_angle.append(theta)

	#raise ValueError, "CHECK 010 !!"
	#### ---------------------------------------------------------------
	####	Set up Geometry Set for DCOUP
	FinalSectorPart = PartName+'01_FINAL'
	p = mdb.models[ModelName].parts[FinalSectorPart]
	session.viewports['Viewport: 1'].setValues(displayedObject=p)
	session.viewports['Viewport: 1'].view.setValues(session.views['Front'])
	f = p.faces
	face = f.getByBoundingBox(-1.e20,-1.e20,-1.e20, 1.e20,1.e20,1.e20)
	p.Set(faces=face, name='TEST')

	for i in range(len(Helix)):
		for j in range(len(Helix)-1):
			a = Helix[i][2]
			b = Helix[j][2]
			if a<b:
				tmp = Helix[i]
				Helix[i] = Helix[j]
				Helix[j] = tmp 	

	#print ' '
	#print ' ------ Helix Coordinate Values at Width Section '
	#print ' '
	#for i in range(len(Helix)):
	#	print Helix[i]
	#	print Helix_angle[i]
	#print ' '

	test = []
	for i in range(len(p.sets['TEST'].faces)):
		test.append(p.sets['TEST'].faces[i])		

	for i in range(len(test)):
		for j in range(len(test)-1):
			a = test[i].pointOn[0][2]
			b = test[j].pointOn[0][2]
			if a<b:
				tmp = test[i]
				test[i] = test[j]
				test[j] = tmp 	

	CN0F = []
	CN1F = []
	CN2F = []	
	for i in range(len(Helix)-1):
		z01 = Helix[i][2]
		z02 = Helix[i+1][2]

		face_tmp = []
		for j in range(len(test)):
			if z02>test[j].pointOn[0][2] and test[j].pointOn[0][2]>z01:

				tmp1 = abs(z01/float(test[j].pointOn[0][2]))
				tmp2 = abs(z02/float(test[j].pointOn[0][2]))
				flag = 0
				if (0.9999<tmp1 and tmp1<1.00001) or (0.9999<tmp2 and tmp2<1.00001):
					flag=1
				if flag==0:
					face_tmp.append(test[j])

		if len(face_tmp)!=11:
			print len(face_tmp)
			raise ValueError, "Face should be 11 !!"


		for ii in range(len(face_tmp)):
			for jj in range(len(face_tmp)-ii):
				a = face_tmp[ii].pointOn[0][1]
				b = face_tmp[ii+jj].pointOn[0][1]

				a_x = face_tmp[ii].pointOn[0][0]
				a_y = face_tmp[ii].pointOn[0][1]
				b_x = face_tmp[ii+jj].pointOn[0][0]
				b_y = face_tmp[ii+jj].pointOn[0][1]

				tmp_angle = Helix_angle[i+1]
				#tmp_angle = (Helix_angle[i]+Helix_angle[i+1])/2.

				a1_x = math.cos(tmp_angle)*a_x     + math.sin(tmp_angle*-1.)*a_y
				a1_y =-math.sin(tmp_angle*-1.)*a_x + math.cos(tmp_angle)*a_y

				b1_x = math.cos(tmp_angle)*b_x     + math.sin(tmp_angle*-1.)*b_y
				b1_y =-math.sin(tmp_angle*-1.)*b_x + math.cos(tmp_angle)*b_y

				if a1_y>b1_y:
					tmp = face_tmp[ii]
					face_tmp[ii] = face_tmp[ii+jj]
					face_tmp[ii+jj] = tmp 	

		CN0F.append(face_tmp[2])
		CN0F.append(face_tmp[3])

		face_tmp01 = []
		face_tmp01.append(face_tmp[4])
		face_tmp01.append(face_tmp[5])
		face_tmp01.append(face_tmp[6])

		for ii in range(len(face_tmp01)):
		 	for jj in range(len(face_tmp01)-ii):
		  		a = face_tmp01[ii].pointOn[0][0]
		  		b = face_tmp01[ii+jj].pointOn[0][0]
 
 		 		a_x = face_tmp01[ii].pointOn[0][0]
 		 		a_y = face_tmp01[ii].pointOn[0][1]
 		 		b_x = face_tmp01[ii+jj].pointOn[0][0]
 		 		b_y = face_tmp01[ii+jj].pointOn[0][1]
 
 		 		tmp_angle = Helix_angle[i+1]
				print tmp_angle
 		 		#tmp_angle = (Helix_angle[i]+Helix_angle[i+1])/2.

		 		a1_x = math.cos(tmp_angle)*a_x     + math.sin(tmp_angle*-1.)*a_y
 		 		a1_y =-math.sin(tmp_angle*-1.)*a_x + math.cos(tmp_angle)*a_y
 
 		 		b1_x = math.cos(tmp_angle)*b_x     + math.sin(tmp_angle*-1.)*b_y
 		 		b1_y =-math.sin(tmp_angle*-1.)*b_x + math.cos(tmp_angle)*b_y
 
 		 		if a1_x>b1_x:
 		 			tmp = face_tmp01[ii]
 		 			face_tmp01[ii] = face_tmp01[ii+jj]
 		 			face_tmp01[ii+jj] = tmp 

		CN1F.append(face_tmp01[1])

		CN2F.append(face_tmp[9])
		CN2F.append(face_tmp[10])
			
	iflag = 0
	for i in range(WSC):
		face = f.findAt((CN0F[iflag].pointOn[0], ))
		p.Set(faces=face, name='TF01Set0'+str(i+1))
		face = f.findAt((CN0F[iflag+1].pointOn[0],))
		p.Set(faces=face, name='TF02Set0'+str(i+1))
		p.SetByBoolean(name='TFSet0'+str(i+1), sets=(p.sets['TF01Set0'+str(i+1)], p.sets['TF02Set0'+str(i+1)],))
		iflag = iflag+2
		del mdb.models[ModelName].parts[FinalSectorPart].sets['TF01Set0'+str(i+1)]
		del mdb.models[ModelName].parts[FinalSectorPart].sets['TF02Set0'+str(i+1)]

	for i in range(WSC):
		face = f.findAt((CN1F[i].pointOn[0], ))
		p.Set(faces=face, name='TFBSet0'+str(i+1))

	iflag = 0
	for i in range(WSC):
		face = f.findAt((CN2F[iflag].pointOn[0], ))
		p.Set(faces=face, name='TFTSet01'+str(i+1))		
		face = f.findAt((CN2F[iflag+1].pointOn[0], ))
		p.Set(faces=face, name='TFTSet02'+str(i+1))		
		p.SetByBoolean(name='TFTSet0'+str(i+1), sets=(p.sets['TFTSet01'+str(i+1)], p.sets['TFTSet02'+str(i+1)],))
		iflag = iflag+2
		del mdb.models[ModelName].parts[FinalSectorPart].sets['TFTSet01'+str(i+1)]
		del mdb.models[ModelName].parts[FinalSectorPart].sets['TFTSet02'+str(i+1)]

	#raise ValueError, "CHECK 011 !!"
	#### ---------------------------------------------------------------
	####	Material
	mdb.models[ModelName].Material(name='STEEL')
	mdb.models[ModelName].materials['STEEL'].Density(table=((7.8e-09, ), ))
	mdb.models[ModelName].materials['STEEL'].Elastic(table=((200000.0, 0.3), ))
	mdb.models[ModelName].HomogeneousSolidSection(name='SOLID', material='STEEL', thickness=None)			

	#### ---------------------------------------------------------------
	####	Meshing
	p = mdb.models[ModelName].parts[FinalSectorPart]
	c = p.cells
	cells = c.getByBoundingBox(-1e20,-1e20,-1e20,1e20,1e20,1e20)
	region = p.Set(cells=cells, name='ALL')
	p = mdb.models[ModelName].parts[FinalSectorPart]
	p.SectionAssignment(region=region, sectionName='SOLID', offset=0.0, 
			offsetType=MIDDLE_SURFACE, offsetField='', 
			thicknessAssignment=FROM_SECTION)
	c = p.cells
	if MeshType=='Hex-dominant':
		p.setMeshControls(regions=c, elemShape=HEX_DOMINATED)
		#p.setMeshControls(regions=c, technique=SWEEP, algorithm=ADVANCING_FRONT)
	elif MeshType=='Tetra':
		p.setMeshControls(regions=c, elemShape=TET, technique=FREE)
	p = mdb.models[ModelName].parts[FinalSectorPart]
	e = p.edges		
	side_edge = []
	for i in range(len(e)):
		for j in range(WSC):
			a = float(j)*float(Width)/float(WSC)
			b = float(j+1)*float(Width)/float(WSC)
			if ((a+.1/WSC)<e[i].pointOn[0][2] and e[i].pointOn[0][2]<(b-(.1/WSC))): 
				side_edge.append(e[i])

	p = mdb.models[ModelName].parts[FinalSectorPart]
	e = p.edges
	if meshsize==0.0:
		tmp_meshsize = (Ra-Rr)/8.
		print 'Meshsize: '+str(tmp_meshsize)
		org_meshsize = meshsize
		meshsize = tmp_meshsize

		p.seedEdgeByNumber(edges=side_edge, number=1, constraint=FINER)
		p.seedPart(size=meshsize, minSizeFactor=0.1)

	else:
		p.seedEdgeByNumber(edges=side_edge, number=1, constraint=FINER)
		p.seedPart(size=meshsize, minSizeFactor=0.1)

	p.generateMesh()			
	print ' '
	print ' --------------------------------------------------------------- '
	print ' '		

	#raise ValueError, "CHECK 012 !!"
	#### -----------------------------------------------------------------------------------------
	####	Reference Node for DCOUP Generation - Approach A and B
	Helix2 = []
	for ip in range(numPoints+1):
		theta = (float(ip)/float(numPoints))*float(-1.*rotangle)

		if App==1:
			fx = float(Ra)*math.sin(theta)
			fy = float(Ra)*math.cos(theta)
		elif App==2:
			fx = float(Rp)*math.sin(theta)
			fy = float(Rp)*math.cos(theta)
		else:
			raise ValueError, "Approach should be 1 or 2 !!"
		fz = (float(ip)/float(numPoints))*float(Width)
	
		if ip%2==0.0:
			Helix2.append((fx,fy,fz))

	p = mdb.models[ModelName].parts[FinalSectorPart]
	print ' '
	print ' -------------  DCOUP Reference Nodes'
	print ' '
	iflag = 0
	for i in range(GTN):
		fx = (Helix2[iflag][0]+Helix2[iflag+1][0])*0.5
		fy = (Helix2[iflag][1]+Helix2[iflag+1][1])*0.5
		fz = (Helix2[iflag][2]+Helix2[iflag+1][2])*0.5
		if App==1:
			p.Node(coordinates=(fx, fy-meshsize*0.1, fz))
			print 'DCOUP Reference Node: '+str(fx)+', '+str(fy-meshsize*0.1)+', '+str(fz)
			nodes = p.nodes[-1:]
			p.Set(nodes=nodes, name='Ref-CN0'+str(i+1))
		elif App==2:
			p.Node(coordinates=(fx, fy-(Ra-Rd)*0.5, fz))
			print 'DCOUP Reference Node: '+str(fx)+', '+str(fy-(Ra-Rd)*0.5)+', '+str(fz)
			nodes = p.nodes[-1:]
			p.Set(nodes=nodes, name='Ref-CN0'+str(i+1))
		else:
			raise ValueError, "Approach should be 1 or 2 !!"
		iflag += 2

	pass

	#### ---------------------------------------------------------------
	####	Assembly 
	a = mdb.models[ModelName].rootAssembly
	a.DatumCsysByDefault(CARTESIAN)
	p = mdb.models[ModelName].parts[FinalSectorPart]
	a.Instance(name=FinalSectorPart+'-1', part=p, dependent=ON)
	#### ---------------------------------------------------------------
	####	Constraint for DCOUP
	Set_tmp = []
	for i in range(WSC):
		if float(i)%2.==0:
			if App==1:
				Set_tmp.append('TFBSet0'+str(i+1))
			elif App==2:
				Set_tmp.append('TFSet0'+str(i+1))
			else:
				raise ValueError, "Approach should be 1 or 2 !!"
	a = mdb.models[ModelName].rootAssembly
	for i in range(GTN):
		region1=a.instances[FinalSectorPart+'-1'].sets['Ref-CN0'+str(i+1)]
		region2=a.instances[FinalSectorPart+'-1'].sets[Set_tmp[i]]
		mdb.models[ModelName].Coupling(name='Constraint-'+str(i+1), controlPoint=region1, 
		    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=DISTRIBUTING, 
		    weightingMethod=UNIFORM, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
		    ur2=ON, ur3=ON)	

	#### ---------------------------------------------------------------
	#### 	Assembly Radial Sweep 360 degree	
	a.RadialInstancePattern(instanceList=(FinalSectorPart+'-1', ), point=(0.0, 0.0, 0.0), 
			axis=(0.0, 0.0, 1.0), number=number_sector, totalAngle=360.0)

	#### ---------------------------------------------------------------
	####  	Constraint Creation for Radial Instance	
	## instanace number in circumferential direction
	tot_rad_num = int(number_sector)
	#rad_num = 2
	##	constraint ID
	c_num = int(GTN)+1
	#c_num = 6
	##	reference node number for teeth
	tot_p_num = int(GTN)
	#p_num = 1
	##
	a = mdb.models[ModelName].rootAssembly
	for r in range(tot_rad_num-1): 
		rad_num = r + 2
		for p in range(tot_p_num): 
			p_num = p + 1

			region1=a.instances[FinalSectorPart+'-1-rad-'+str(rad_num)].sets['Ref-CN0'+str(p_num)]
			region2=a.instances[FinalSectorPart+'-1-rad-'+str(rad_num)].sets[Set_tmp[p]]
			mdb.models[ModelName].Coupling(name='Constraint-'+str(c_num), controlPoint=region1, 
			    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=DISTRIBUTING, 
			    weightingMethod=UNIFORM, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
			    ur2=ON, ur3=ON)
			c_num += 1			

	tmp = []
	for i in range(WSC):
		tmp.append(a.instances[FinalSectorPart+'-1'].sets['TFTSet0'+str(i+1)])

	for r in range(tot_rad_num-1): 
		rad_num = r + 2
		for p in range(WSC): 
			tmp.append(a.instances[FinalSectorPart+'-1-rad-'+str(rad_num)].sets['TFTSet0'+str(p+1)])

	a.SetByBoolean(name='RingSurface', sets=(tmp))

	#raise ValueError, "CHECK 013 !!"
	#### ---------------------------------------------------------------
	####   	RETAINED NSET	
	a = mdb.models[ModelName].rootAssembly
	a.regenerate()	
	reftmp = []
	inst = a.allInstances
	for i in range(len(inst.keys())):
		for j in range(len(inst[inst.keys()[i]].sets.keys())):
			if ('CN0' in inst[inst.keys()[i]].sets.keys()[j])==TRUE:
				reftmp.append(inst[inst.keys()[i]].sets[inst[inst.keys()[i]].sets.keys()[j]])

	if int(GTN)*int(number_sector)!=int(len(reftmp)):
		raise ValueError, ' Reference Search ERROR !!'

	a.SetByBoolean(name='Set-RETAINED01', sets=((reftmp)))

	pass

	#raise ValueError, "CHECK 014 !!"
	#### ---------------------------------------------------------------
	####   	Dummy Input file Creation 	
	mdb.models[ModelName].setValues(noPartsInputFile=ON)
	mdb.Job(name=JobName+'-TMP', model=ModelName, description='', type=ANALYSIS, 
			atTime=None, waitMinutes=0, waitHours=0, queue=None, memory=90, 
			memoryUnits=PERCENTAGE, getMemoryFromAnalysis=True, 
			explicitPrecision=SINGLE, nodalOutputPrecision=SINGLE, echoPrint=OFF, 
			modelPrint=OFF, contactPrint=OFF, historyPrint=OFF, userSubroutine='', 
			scratch='', resultsFormat=ODB, parallelizationMethodExplicit=DOMAIN, 
			numDomains=1, activateLoadBalancing=False, multiprocessingMode=DEFAULT, 
			numCpus=1, numGPUs=0)
	mdb.jobs[JobName+'-TMP'].writeInput(consistencyChecking=OFF)

	#### ---------------------------------------------------------------
	####   	Read Dummy Input file and Merge Coincidence Nodes 
	ModelName = JobName+'-TMP'
	PartName2 = 'PART-1'
	mdb.ModelFromInputFile(name=ModelName, inputFileName=JobName+'-TMP.inp')	
	a = mdb.models[ModelName].rootAssembly
	p = mdb.models[ModelName].parts[PartName2]
	mdb.meshEditOptions.setValues(enableUndo=True, maxUndoCacheElements=0.5)
	n = p.nodes
	nodes = n[0:len(n)]
	p.mergeNodes(nodes=nodes, tolerance=0.01)
	#### ---------------------------------------------------------------
	####    Move Model to match origin (0,0,0)	
	a=mdb.models[ModelName].rootAssembly
	a.translate(instanceList=(PartName2+'-1', ), vector=(0.0, 0.0, -Width/2.))
	#### ---------------------------------------------------------------
	####    RETAINED NODE SET-UP 
	a.DatumPointByCoordinate(coords=(0.0, 0.0, 0.0))
	d1 = a.datums
	a.ReferencePoint(point=d1[d1.keys()[len(d1.keys())-1]].pointOn)
	r1 = a.referencePoints
	refPoints1=(r1[r1.keys()[0]], )
	region1=a.Set(referencePoints=refPoints1, name='SET-RETAINED10')
	a.SetByBoolean(name='RETAINED', sets=(a.sets['SET-RETAINED01'], a.sets['SET-RETAINED10'],))

	#### ---------------------------------------------------------------
	####    Center Constraint Build 
	a=mdb.models[ModelName].rootAssembly
	region2=a.sets['RINGSURFACE']

	mdb.models[ModelName].Coupling(name='Constraint-Center', controlPoint=region1, 
	    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=DISTRIBUTING, 
	    weightingMethod=UNIFORM, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
	    ur2=ON, ur3=ON)	

	#### ---------------------------------------------------------------
	####    Rotation Assembly at + Z axis 
	a = mdb.models[ModelName].rootAssembly
	a.rotate(instanceList=(PartName2+'-1', ), axisPoint=(0.0, 0.0, 0.0), 
	    axisDirection=(1.0, 0.0, 0.0), angle=AngX)
	a.rotate(instanceList=(PartName2+'-1', ), axisPoint=(0.0, 0.0, 0.0), 
	    axisDirection=(0.0, 1.0, 0.0), angle=AngY)
	a.rotate(instanceList=(PartName2+'-1', ), axisPoint=(0.0, 0.0, 0.0), 
	    axisDirection=(0.0, 0.0, 1.0), angle=AngZ)

	#### ---------------------------------------------------------------
	####	FREQ and SUBSTR GENERATION STEP Build 
	a = mdb.models[ModelName].rootAssembly
	mdb.models[ModelName].FrequencyStep(name='FREQ', previous='Initial',
			simLinearDynamics=ON, normalization=MASS, numEigen=NumEig)
	region = a.sets['RETAINED']
	mdb.models[ModelName].EncastreBC(name='RETAINED', createStepName='FREQ', 
			region=region, localCsys=None)
	mdb.models[ModelName].SubstructureGenerateStep(name='SUBSTR_GEN', 
			previous='FREQ', substructureIdentifier=GearSubIdent, computeReducedMassMatrix=ON, 
			computeReducedViscousDampingMatrix=ON, computeReducedStructuralDampingMatrix=ON, 
			modeRange=((1, NumEig, 1), ), retainedEigenmodesMethod=MODE_RANGE)
	region = a.sets['RETAINED']
	mdb.models[ModelName].RetainedNodalDofsBC(name='RETAINDOF', createStepName='SUBSTR_GEN', 
			region=region, u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, ur3=ON)
	mdb.models[ModelName].steps['SUBSTR_GEN'].setValues(computeReducedViscousDampingMatrix=OFF)
	mdb.models[ModelName].steps['SUBSTR_GEN'].setValues( computeReducedStructuralDampingMatrix=OFF)
	if Recovery_flag==0:
		mdb.models[ModelName].steps['SUBSTR_GEN'].setValues(recoveryMatrix=NONE)
	mdb.models[ModelName].setValues(noPartsInputFile=ON)
	mdb.Job(name=JobName, model=ModelName, description='', type=ANALYSIS, atTime=None, 
			waitMinutes=0, waitHours=0, queue=None, memory=90, memoryUnits=PERCENTAGE, 
			getMemoryFromAnalysis=True, explicitPrecision=SINGLE, nodalOutputPrecision=SINGLE, 
			echoPrint=OFF, modelPrint=OFF, contactPrint=OFF, historyPrint=OFF, 
			userSubroutine='', scratch='', resultsFormat=ODB, multiprocessingMode=DEFAULT, 
			numCpus=1, numGPUs=0)
	mdb.jobs[JobName].writeInput(consistencyChecking=OFF)	

	#raise ValueError, "CHECK 014 !!"
	#### ---------------------------------------------------------------
	#### 	FLEXIBLE BODY Keyword Add.	
	if Recovery_flag==0:
		readfile = open(JobName+'.inp', 'r')
		writefile = open(JobName+'-SimpackFlex.inp', 'w')

		read_line = readfile.readlines()
		num_line = 1
		for line in read_line:
			if num_line==len(read_line): 
				writefile.write('*FLEXIBLE BODY, TYPE=SIMPACK\n')
				writefile.write(line)
			else:
				writefile.write(line)
			num_line += 1
		
		readfile.close()
		writefile.close()	
	elif Recovery_flag==1:
		readfile = open(JobName+'.inp', 'r')
		writefile = open(JobName+'-SimpackFlex.inp', 'w')

		read_line = readfile.readlines()
		num_line = 1
		for line in read_line:
			if num_line==len(read_line): 
				writefile.write('*FLEXIBLE BODY, TYPE=SIMPACK\n')
				writefile.write('*ELEMENT RECOVERY MATRIX, POSITION=AVERAGED AT NODES\n')
				writefile.write('S, E\n')
				writefile.write(line)
			else:
				writefile.write(line)
			num_line += 1
		
		readfile.close()
		writefile.close()
	#### ---------------------------------------------------------------
	####  	Final Input File for Abaqus to Simpack: JobName+'-SimpackFlex.inp'	
	os.system('del '+JobName+'-TMP.inp')

	p = mdb.models[ModelName].parts[PartName2]
        session.viewports['Viewport: 1'].setValues(displayedObject=p)

	###################################################################################

	print ' '
	print JobName+'.inp is created                 -- Substructure Generation'
	print JobName+'-SimpackFlex.inp is created     -- Substructure Generation for Simpack Flex Body'
	print ' '

	return 1
