from abaqus import *
from abaqusConstants import *
import customKernel
import sys, os
from caeModules import *
import uti, math

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def internalSpur(ModelName, PartName, p2_x, p2_y, Rr, Rc, Ra, Rp, delta, WSC, GTN, Width, MeshType, meshsize, 
		App, AngX, AngY, AngZ, GearSubIdent, Recovery_flag, number_sector, JobName, NumEig, D_ring):

	R_ring = D_ring/2.
	p2_x = R_ring*math.sin(2.*delta)
	p2_y = R_ring*math.cos(2.*delta)

	PartName_2 = 'Part-2'
	#### ---------------------------------------------------------------
	#### 	Create Involute Profile
	session.viewports['Viewport: 1'].setValues(displayedObject=None)
	s = mdb.models[ModelName].ConstrainedSketch(name='__profile__', sheetSize=200.0)
	g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
	s.setPrimaryObject(option=STANDALONE)
	s.Line(point1=(0.0, 0.0), point2=(p2_x,p2_y))
	s.ArcByCenterEnds(center=(0.0, 0.0), point1=(-p2_x, p2_y), point2=(p2_x,p2_y), direction=CLOCKWISE)
	s.Line(point1=(-p2_x, p2_y), point2=(0.0, 0.0))
	p = mdb.models[ModelName].Part(name=PartName_2, dimensionality=THREE_D, type=DEFORMABLE_BODY)
	p = mdb.models[ModelName].parts[PartName_2]
	p.BaseSolidExtrude(sketch=s, depth=Width)
	s.unsetPrimaryObject()
	del mdb.models[ModelName].sketches['__profile__']

	#raise ValueError, "CHECK 001 !!"
	#### ---------------------------------------------------------------
	#### 	Bore Cut
	p = mdb.models[ModelName].parts[PartName_2]
	f, e = p.faces, p.edges
	face = f.findAt((0,p2_y*0.5,0))
	edge = e.findAt((-p2_x*0.99,p2_y*0.99,0),)
	t = p.MakeSketchTransform(sketchPlane=face, sketchUpEdge=edge, 
	    sketchPlaneSide=SIDE1, sketchOrientation=RIGHT, origin=(0.0, 0.0, 0.0))
	s = mdb.models[ModelName].ConstrainedSketch(name='__profile__', 
	    sheetSize=100.00, gridSpacing=5, transform=t)
	g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
	s.setPrimaryObject(option=SUPERIMPOSE)
	p = mdb.models[ModelName].parts[PartName_2]
	p.projectReferencesOntoSketch(sketch=s, filter=COPLANAR_EDGES)
	s.CircleByCenterPerimeter(center=(0, 0), point1=(0, Rc))
	p = mdb.models[ModelName].parts[PartName_2]
	f1, e1 = p.faces, p.edges
	p.CutExtrude(sketchPlane=face, sketchUpEdge=edge, sketchPlaneSide=SIDE1, 
	    sketchOrientation=RIGHT, sketch=s, flipExtrudeDirection=OFF)
	s.unsetPrimaryObject()
	del mdb.models[ModelName].sketches['__profile__']

	#raise ValueError, "CHECK 002 !!"
	#### ---------------------------------------------------------------
	#### 	Create Sector 
	a = mdb.models[ModelName].rootAssembly
	p   = mdb.models[ModelName].parts[PartName]
	a.Instance(name=PartName+'-1', part=p, dependent=ON)
	p_2 = mdb.models[ModelName].parts[PartName_2]
	a.Instance(name=PartName_2+'-1', part=p_2, dependent=ON)
	a.InstanceFromBooleanCut(name='Part-3', 
	    instanceToBeCut=mdb.models[ModelName].rootAssembly.instances[PartName_2+'-1'], 
	    cuttingInstances=(a.instances[PartName+'-1'], ), originalInstances=DELETE)

	a = mdb.models[ModelName].rootAssembly
	a.DatumAxisByPrincipalAxis(principalAxis=ZAXIS)
	a.RadialInstancePattern(instanceList=('Part-3-1', ), point=(0.0, 0.0, 0.0), 
	    axis=(0.0, 0.0, 1.0), number=2, totalAngle=4.*delta*180/3.1416)
	a.InstanceFromBooleanMerge(name='Part-4', instances=(a.instances['Part-3-1'], 
	    a.instances['Part-3-1-rad-2'], ), originalInstances=DELETE, 
	    domain=GEOMETRY)

	a.rotate(instanceList=('Part-4-1', ), axisPoint=(0.0, 0.0, 0.0), 
	    axisDirection=(0.0, 0.0, 1.0), angle=-2.*delta*180/3.14159)
	a.RadialInstancePattern(instanceList=('Part-4-1', ), point=(0.0, 0.0, 0.0), 
	    axis=(0.0, 0.0, 1.0), number=2, totalAngle=180.0)
	a.InstanceFromBooleanMerge(name='Part-5', instances=(a.instances['Part-4-1'], 
	    a.instances['Part-4-1-rad-2'], ), originalInstances=DELETE, 
	    domain=GEOMETRY)
	a.deleteFeatures(('Part-4-1', 'Part-4-1-rad-2', ))
	p = mdb.models[ModelName].parts['Part-5']
	f = p.faces
	p.RemoveFaces(faceList=(f.getByBoundingBox(-1.e20,-1.e20,-1.e20,1.e20,0.0,1.e20)), deleteCells=False)

	session.viewports['Viewport: 1'].setValues(displayedObject=None)
	s = mdb.models[ModelName].ConstrainedSketch(name='__profile__', 
	    sheetSize=200.0)
	g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
	s.setPrimaryObject(option=STANDALONE)
	s.Line(point1=(0.0, 0.0), point2=(p2_x,p2_y))
	s.ArcByCenterEnds(center=(0.0, 0.0), point1=(p2_x, p2_y), point2=(-p2_x,p2_y)
	    , direction=CLOCKWISE)
	s.Line(point1=(-p2_x, p2_y), point2=(0.0, 0.0))
	p = mdb.models[ModelName].Part(name='Part-6', dimensionality=THREE_D, 
	    type=DEFORMABLE_BODY)
	p = mdb.models[ModelName].parts['Part-6']
	p.BaseSolidExtrude(sketch=s, depth=Width)
	s.unsetPrimaryObject()
	del mdb.models[ModelName].sketches['__profile__']		

	a1 = mdb.models[ModelName].rootAssembly
	a1.DatumCsysByDefault(CARTESIAN)
	p = mdb.models[ModelName].parts['Part-5']
	a1.Instance(name='Part-5-1', part=p, dependent=ON)
	p = mdb.models[ModelName].parts['Part-6']
	a1.Instance(name='Part-6-1', part=p, dependent=ON)
	a1 = mdb.models[ModelName].rootAssembly
	a1.InstanceFromBooleanCut(name='Part-7', 
	    instanceToBeCut=mdb.models[ModelName].rootAssembly.instances['Part-5-1'], 
	    cuttingInstances=(a1.instances['Part-6-1'], ), originalInstances=DELETE)

	del mdb.models[ModelName].parts['Part-2']		
	del mdb.models[ModelName].parts['Part-3']		
	del mdb.models[ModelName].parts['Part-4']		
	del mdb.models[ModelName].parts['Part-5']		
	del mdb.models[ModelName].parts['Part-6']		

	session.viewports['Viewport: 1'].setValues(displayedObject=None)
	s = mdb.models[ModelName].ConstrainedSketch(name='__profile__', sheetSize=200.0)
	g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
	s.setPrimaryObject(option=STANDALONE)
	s.CircleByCenterPerimeter(center=(0, 0), point1=(0, Ra))
	p = mdb.models[ModelName].Part(name='Part-8', dimensionality=THREE_D, type=DEFORMABLE_BODY)
	p = mdb.models[ModelName].parts['Part-8']
	p.BaseShellExtrude(sketch=s, depth=Width)
	s.unsetPrimaryObject()
	del mdb.models[ModelName].sketches['__profile__']			

	FinalSectorPart = 'GEAR01_FinalSector'

	a1 = mdb.models[ModelName].rootAssembly
	a1.DatumCsysByDefault(CARTESIAN)
	p = mdb.models[ModelName].parts['Part-7']
	a1.Instance(name='Part-7-1', part=p, dependent=ON)
	p = mdb.models[ModelName].parts['Part-8']
	a1.Instance(name='Part-8-1', part=p, dependent=ON)
	a1 = mdb.models[ModelName].rootAssembly
	a1.InstanceFromBooleanCut(name=FinalSectorPart, 
	    instanceToBeCut=mdb.models[ModelName].rootAssembly.instances['Part-7-1'], 
	    cuttingInstances=(a1.instances['Part-8-1'], ), originalInstances=DELETE)		

	del mdb.models[ModelName].parts['Part-7']		
	del mdb.models[ModelName].parts['Part-8']		


	#raise ValueError, "CHECK 003 !!"
	#### ---------------------------------------------------------------
	####	Partition
	p = mdb.models[ModelName].parts[FinalSectorPart]
	c = p.cells
	cells = c.getByBoundingBox(-1e20,-1e20,-1e20,1e20,1e20,1e20)
	e, v, d = p.edges, p.vertices, p.datums
	p = mdb.models[ModelName].parts[FinalSectorPart]
	d_points = []
	tmp = float(Width)/float(WSC)
	for i in range(WSC):
		p.DatumPointByCoordinate(coords=(0.0, 0.0, tmp*(i+1)))
		d_points.append(p.datums[p.datums.keys()[i]])
	d1 = p.datums
	p.DatumAxisByTwoPoint(point1=d_points[WSC-2], point2=d_points[WSC-1])
	d_normal = p.datums[p.datums.keys()[WSC]]
	for i in range(WSC-1):
		cells = c.getByBoundingBox(-1e20,-1e20,-1e20,1e20,1e20,1e20)
		p.PartitionCellByPlanePointNormal(point=d_points[i], normal=d_normal, cells=cells)	

	#raise ValueError, "CHECK 004 !!"
	#### ---------------------------------------------------------------
	####	Set up Geometry Set for DCOUP
	p = mdb.models[ModelName].parts[FinalSectorPart]
	session.viewports['Viewport: 1'].view.setValues(session.views['Front'])
	p = mdb.models[ModelName].parts[FinalSectorPart]
	f = p.faces
	#face = f.getByBoundingBox(-R_ring,0.0,-1.0, R_ring,R_ring-meshsize,Width)
	face = f.getByBoundingBox(-1.e20,-1.e20,-1.0, 1.e20,1.e20,Width)
	p.Set(faces=face, name='TEST')

	test = []
	for i in range(len(p.sets['TEST'].faces)):
		test.append(p.sets['TEST'].faces[i])		

	CN0F = []
	CN1F = []
	CN2F = []
	for i in range(len(d_points)):
		if i==0:
			z01 = 0.0 
			z02 = d_points[0].pointOn[2]
		else:
			z01 = d_points[i-1].pointOn[2]
			z02 = d_points[i].pointOn[2]
			
		#print z01, z02

		face_tmp = []
		for j in range(len(test)):

			if z02>test[j].pointOn[0][2] and test[j].pointOn[0][2]>z01:

				tmp1 = abs(z01/float(test[j].pointOn[0][2]))
				tmp2 = abs(z02/float(test[j].pointOn[0][2]))
				flag = 0
				if (0.9999<tmp1 and tmp1<1.00001) or (0.9999<tmp2 and tmp2<1.00001):
					flag=1
				if flag==0:
					face_tmp.append(test[j])
		if len(face_tmp)!=9:
			raise ValueError, "Face should be 9 !!"


		for ii in range(len(face_tmp)):
			for jj in range(len(face_tmp)-ii):
				a = face_tmp[ii].pointOn[0][1]
				b = face_tmp[ii+jj].pointOn[0][1]

				if a>b:
					tmp = face_tmp[ii]
					face_tmp[ii] = face_tmp[ii+jj]
					face_tmp[ii+jj] = tmp 	

		CN0F.append(face_tmp[1])
		CN0F.append(face_tmp[2])

		for ii in range(3):
			for jj in range(3-ii):
				a = face_tmp[3+ii].pointOn[0][0]
				b = face_tmp[3+ii+jj].pointOn[0][0]

				if a>b:
					tmp = face_tmp[3+ii]
					face_tmp[3+ii] = face_tmp[3+ii+jj]
					face_tmp[3+ii+jj] = tmp 	

		CN1F.append(face_tmp[4])
		CN2F.append(face_tmp[8])
			
	iflag = 0
	for i in range(WSC):
		face = f.findAt((CN0F[iflag].pointOn[0], ))
		p.Set(faces=face, name='TF01Set0'+str(i+1))
		face = f.findAt((CN0F[iflag+1].pointOn[0],))
		p.Set(faces=face, name='TF02Set0'+str(i+1))
		p.SetByBoolean(name='TFSet0'+str(i+1), sets=(p.sets['TF01Set0'+str(i+1)], p.sets['TF02Set0'+str(i+1)],))
		iflag = iflag+2
		del mdb.models[ModelName].parts[FinalSectorPart].sets['TF01Set0'+str(i+1)]
		del mdb.models[ModelName].parts[FinalSectorPart].sets['TF02Set0'+str(i+1)]

	for i in range(WSC):
		face = f.findAt((CN1F[i].pointOn[0], ))
		p.Set(faces=face, name='TFBSet0'+str(i+1))

	for i in range(WSC):
		face = f.findAt((CN2F[i].pointOn[0], ))
		p.Set(faces=face, name='TFTSet0'+str(i+1))

	#raise ValueError, "CHECK 005 !!"
	#### ---------------------------------------------------------------
	####	Material
	mdb.models[ModelName].Material(name='STEEL')
	mdb.models[ModelName].materials['STEEL'].Density(table=((7.8e-09, ), ))
	mdb.models[ModelName].materials['STEEL'].Elastic(table=((200000.0, 0.3), ))
	mdb.models[ModelName].HomogeneousSolidSection(name='SOLID', material='STEEL', thickness=None)
	#### ---------------------------------------------------------------
	####	Meshing
	p = mdb.models[ModelName].parts[FinalSectorPart]
	c = p.cells
	cells = c.getByBoundingBox(-1e20,-1e20,-1e20,1e20,1e20,1e20)
	region = p.Set(cells=cells, name='ALL')
	p = mdb.models[ModelName].parts[FinalSectorPart]
	p.SectionAssignment(region=region, sectionName='SOLID', offset=0.0, 
			offsetType=MIDDLE_SURFACE, offsetField='', 
			thicknessAssignment=FROM_SECTION)
	c = p.cells
	if MeshType=='Hex-dominant':
		p.setMeshControls(regions=c, elemShape=HEX_DOMINATED)
		#p.setMeshControls(regions=c, technique=SWEEP, algorithm=ADVANCING_FRONT)
	elif MeshType=='Tetra':
		p.setMeshControls(regions=c, elemShape=TET, technique=FREE)
	p = mdb.models[ModelName].parts[FinalSectorPart]
	e = p.edges		
	side_edge = []
	for i in range(len(e)):
		for j in range(WSC):
			a = float(j)*float(Width)/float(WSC)
			b = float(j+1)*float(Width)/float(WSC)
			if ((a+.1/WSC)<e[i].pointOn[0][2] and e[i].pointOn[0][2]<(b-(.1/WSC))): 
				side_edge.append(e[i])

	p = mdb.models[ModelName].parts[FinalSectorPart]
	e = p.edges
	if meshsize==0.0:
		tmp_meshsize = (Ra-Rr)/8.
		print 'Meshsize: '+str(tmp_meshsize)
		org_meshsize = meshsize
		meshsize = tmp_meshsize

		p.seedEdgeByNumber(edges=side_edge, number=1, constraint=FINER)
		p.seedPart(size=meshsize, minSizeFactor=0.1)

	else:
		p.seedEdgeByNumber(edges=side_edge, number=1, constraint=FINER)
		p.seedPart(size=meshsize, minSizeFactor=0.1)

	p.generateMesh()		
	
	print ' '
	print ' --------------------------------------------------------------- '
	print ' '

	#raise ValueError, "CHECK 006 !!"
	#### -----------------------------------------------------------------------------------------
	####	Reference Node for DCOUP Generation - Approach A and B
	p = mdb.models[ModelName].parts[FinalSectorPart]
	ref_z = []
	for i in range(WSC):
		ip = i
		if float(ip)%2.==0:
			ref_z.append(0.5*((2*ip+1)*(Width/WSC)))
	if len(ref_z)!=GTN:
		raise ValueError, "DCOUP Ref node number should match with Partition Number in Width direction!!"
	for i in range(len(ref_z)):
		for j in range(len(ref_z)-1):
			a = ref_z[i]
			b = ref_z[j]
			if a>b:
				tmp = ref_z[i]
				ref_z[i] = ref_z[j]
				ref_z[j] = tmp 	

	print ' '
	print ' -------------  DCOUP Reference Nodes'
	print ' '
	for i in range(len(ref_z)):
		if App==1:
			p.Node(coordinates=(0.0, Ra-meshsize*.2, ref_z[i]))
			print 'DCOUP Reference Node: 0.0, '+str(Ra-meshsize*.2)+', '+str(ref_z[i])
			nodes = p.nodes[-1:]
			p.Set(nodes=nodes, name='Ref-CN0'+str(i+1))
		elif App==2:
			p.Node(coordinates=(0.0, Rp, ref_z[i]))
			print 'DCOUP Reference Node: 0.0, '+str(Rp)+', '+str(ref_z[i])
			nodes = p.nodes[-1:]
			p.Set(nodes=nodes, name='Ref-CN0'+str(i+1))
		else:
			raise ValueError, "Approach should be 1 or 2 !!"

	#### ---------------------------------------------------------------
	####	Assembly 
	a = mdb.models[ModelName].rootAssembly
	a.DatumCsysByDefault(CARTESIAN)
	p = mdb.models[ModelName].parts[FinalSectorPart]
	a.Instance(name=FinalSectorPart+'-1', part=p, dependent=ON)
	#### ---------------------------------------------------------------
	####	Constraint for DCOUP
	Set_tmp = []
	for i in range(WSC):
		if float(i)%2.==0:
			if App==1:
				Set_tmp.append('TFBSet0'+str(i+1))
			elif App==2:
				Set_tmp.append('TFSet0'+str(i+1))
			else:
				raise ValueError, "Approach should be 1 or 2 !!"
	a = mdb.models[ModelName].rootAssembly
	for i in range(GTN):
		region1=a.instances[FinalSectorPart+'-1'].sets['Ref-CN0'+str(i+1)]
		region2=a.instances[FinalSectorPart+'-1'].sets[Set_tmp[GTN-i-1]]
		mdb.models[ModelName].Coupling(name='Constraint-'+str(i+1), controlPoint=region1, 
		    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=DISTRIBUTING, 
		    weightingMethod=UNIFORM, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
		    ur2=ON, ur3=ON)	

	#raise ValueError, "CHECK 007 !!"
	#### ---------------------------------------------------------------
	#### 	Assembly Radial Sweep 360 degree	
	a.RadialInstancePattern(instanceList=(FinalSectorPart+'-1', ), point=(0.0, 0.0, 0.0), 
			axis=(0.0, 0.0, 1.0), number=number_sector, totalAngle=360.0)

	#### ---------------------------------------------------------------
	####  	Constraint Creation for Radial Instance	
	## instanace number in circumferential direction
	tot_rad_num = int(number_sector)
	#rad_num = 2
	##	constraint ID
	c_num = int(GTN)+1
	#c_num = 6
	##	reference node number for teeth
	tot_p_num = int(GTN)
	#p_num = 1
	##
	a = mdb.models[ModelName].rootAssembly
	for r in range(tot_rad_num-1): 
		rad_num = r + 2
		for p in range(tot_p_num): 
			p_num = p + 1

			region1=a.instances[FinalSectorPart+'-1-rad-'+str(rad_num)].sets['Ref-CN0'+str(p_num)]
			region2=a.instances[FinalSectorPart+'-1-rad-'+str(rad_num)].sets[Set_tmp[tot_p_num-p-1]]
			mdb.models[ModelName].Coupling(name='Constraint-'+str(c_num), controlPoint=region1, 
			    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=DISTRIBUTING, 
			    weightingMethod=UNIFORM, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
			    ur2=ON, ur3=ON)
			c_num += 1	

	tmp = []
	for i in range(WSC):
		tmp.append(a.instances[FinalSectorPart+'-1'].sets['TFTSet0'+str(i+1)])

	for r in range(tot_rad_num-1): 
		rad_num = r + 2
		for p in range(WSC): 
			tmp.append(a.instances[FinalSectorPart+'-1-rad-'+str(rad_num)].sets['TFTSet0'+str(p+1)])

	a.SetByBoolean(name='BoreSurface', sets=(tmp))

	a = mdb.models[ModelName].rootAssembly
	#### ---------------------------------------------------------------
	####   	RETAINED NSET	
	reftmp = []
	inst = a.allInstances
	for i in range(len(inst.keys())):
		for j in range(len(inst[inst.keys()[i]].sets.keys())):
			if ('CN0' in inst[inst.keys()[i]].sets.keys()[j])==TRUE:
				reftmp.append(inst[inst.keys()[i]].sets[inst[inst.keys()[i]].sets.keys()[j]])

	if int(GTN)*int(number_sector)!=int(len(reftmp)):
		raise ValueError, ' Reference Search ERROR !!'

	a.SetByBoolean(name='Set-RETAINED01', sets=(reftmp))

	#raise ValueError, "CHECK 008 !!"
	#### ---------------------------------------------------------------
	####   	Dummy Input file Creation 	
	mdb.models[ModelName].setValues(noPartsInputFile=ON)
	mdb.Job(name=JobName+'-TMP', model=ModelName, description='', type=ANALYSIS, 
			atTime=None, waitMinutes=0, waitHours=0, queue=None, memory=90, 
			memoryUnits=PERCENTAGE, getMemoryFromAnalysis=True, 
			explicitPrecision=SINGLE, nodalOutputPrecision=SINGLE, echoPrint=OFF, 
			modelPrint=OFF, contactPrint=OFF, historyPrint=OFF, userSubroutine='', 
			scratch='', resultsFormat=ODB, parallelizationMethodExplicit=DOMAIN, 
			numDomains=1, activateLoadBalancing=False, multiprocessingMode=DEFAULT, 
			numCpus=1, numGPUs=0)
	mdb.jobs[JobName+'-TMP'].writeInput(consistencyChecking=OFF)

	#### ---------------------------------------------------------------
	####   	Read Dummy Input file and Merge Coincidence Nodes 
	ModelName = JobName+'-TMP'
	PartName2 = 'PART-1'
	mdb.ModelFromInputFile(name=ModelName, inputFileName=JobName+'-TMP.inp')	
	a = mdb.models[ModelName].rootAssembly
	p = mdb.models[ModelName].parts[PartName2]
	mdb.meshEditOptions.setValues(enableUndo=True, maxUndoCacheElements=0.5)
	n = p.nodes
	nodes = n[0:len(n)]
	p.mergeNodes(nodes=nodes, tolerance=meshsize*0.1)
	#### ---------------------------------------------------------------
	####    Move Model to match origin (0,0,0)	
	a=mdb.models[ModelName].rootAssembly
	a.translate(instanceList=(PartName2+'-1', ), vector=(0.0, 0.0, -Width/2.))
	#### ---------------------------------------------------------------
	####    RETAINED NODE SET-UP 
	a.DatumPointByCoordinate(coords=(0.0, 0.0, 0.0))
	d1 = a.datums
	a.ReferencePoint(point=d1[d1.keys()[len(d1.keys())-1]].pointOn)
	r1 = a.referencePoints
	refPoints1=(r1[r1.keys()[0]], )
	region1=a.Set(referencePoints=refPoints1, name='SET-RETAINED10')
	a.SetByBoolean(name='RETAINED', sets=(a.sets['SET-RETAINED01'], a.sets['SET-RETAINED10'],))

	#### ---------------------------------------------------------------
	####    Center Constraint Build 
	a=mdb.models[ModelName].rootAssembly
	region2=a.sets['BORESURFACE']

	mdb.models[ModelName].Coupling(name='Constraint-Center', controlPoint=region1, 
	    surface=region2, influenceRadius=WHOLE_SURFACE, couplingType=DISTRIBUTING, 
	    weightingMethod=UNIFORM, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
	    ur2=ON, ur3=ON)	

	#### ---------------------------------------------------------------
	####    Rotation Assembly at + Z axis 
	a = mdb.models[ModelName].rootAssembly
	a.rotate(instanceList=(PartName2+'-1', ), axisPoint=(0.0, 0.0, 0.0), 
	    axisDirection=(1.0, 0.0, 0.0), angle=AngX)
	a.rotate(instanceList=(PartName2+'-1', ), axisPoint=(0.0, 0.0, 0.0), 
	    axisDirection=(0.0, 1.0, 0.0), angle=AngY)
	a.rotate(instanceList=(PartName2+'-1', ), axisPoint=(0.0, 0.0, 0.0), 
	    axisDirection=(0.0, 0.0, 1.0), angle=AngZ)

	#### ---------------------------------------------------------------
	####	FREQ and SUBSTR GENERATION STEP Build 
	a = mdb.models[ModelName].rootAssembly
	mdb.models[ModelName].FrequencyStep(name='FREQ', previous='Initial', 
			simLinearDynamics=ON, normalization=MASS, numEigen=NumEig)
	region = a.sets['RETAINED']
	mdb.models[ModelName].EncastreBC(name='RETAINED', createStepName='FREQ', 
			region=region, localCsys=None)
	mdb.models[ModelName].SubstructureGenerateStep(name='SUBSTR_GEN', 
			previous='FREQ', substructureIdentifier=GearSubIdent, computeReducedMassMatrix=ON, 
			computeReducedViscousDampingMatrix=ON, computeReducedStructuralDampingMatrix=ON, 
			modeRange=((1, NumEig, 1), ), retainedEigenmodesMethod=MODE_RANGE)
	region = a.sets['RETAINED']
	mdb.models[ModelName].RetainedNodalDofsBC(name='RETAINDOF', createStepName='SUBSTR_GEN', 
			region=region, u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, ur3=ON)
	mdb.models[ModelName].steps['SUBSTR_GEN'].setValues(computeReducedViscousDampingMatrix=OFF)
	mdb.models[ModelName].steps['SUBSTR_GEN'].setValues( computeReducedStructuralDampingMatrix=OFF)
	if Recovery_flag==0:
		mdb.models[ModelName].steps['SUBSTR_GEN'].setValues(recoveryMatrix=NONE)
	mdb.models[ModelName].setValues(noPartsInputFile=ON)
	mdb.Job(name=JobName, model=ModelName, description='', type=ANALYSIS, atTime=None, 
			waitMinutes=0, waitHours=0, queue=None, memory=90, memoryUnits=PERCENTAGE, 
			getMemoryFromAnalysis=True, explicitPrecision=SINGLE, nodalOutputPrecision=SINGLE, 
			echoPrint=OFF, modelPrint=OFF, contactPrint=OFF, historyPrint=OFF, 
			userSubroutine='', scratch='', resultsFormat=ODB, multiprocessingMode=DEFAULT, 
			numCpus=1, numGPUs=0)
	mdb.jobs[JobName].writeInput(consistencyChecking=OFF)	

	#raise ValueError, "CHECK 009 !!"
	#### ---------------------------------------------------------------
	#### 	FLEXIBLE BODY Keyword Add.	
	if Recovery_flag==0:
		readfile = open(JobName+'.inp', 'r')
		writefile = open(JobName+'-SimpackFlex.inp', 'w')

		read_line = readfile.readlines()
		num_line = 1
		for line in read_line:
			if num_line==len(read_line): 
				writefile.write('*FLEXIBLE BODY, TYPE=SIMPACK\n')
				writefile.write(line)
			else:
				writefile.write(line)
			num_line += 1
		
		readfile.close()
		writefile.close()	
	elif Recovery_flag==1:
		readfile = open(JobName+'.inp', 'r')
		writefile = open(JobName+'-SimpackFlex.inp', 'w')

		read_line = readfile.readlines()
		num_line = 1
		for line in read_line:
			if num_line==len(read_line): 
				writefile.write('*FLEXIBLE BODY, TYPE=SIMPACK\n')
				writefile.write('*ELEMENT RECOVERY MATRIX, POSITION=AVERAGED AT NODES\n')
				writefile.write('S, E\n')
				writefile.write(line)
			else:
				writefile.write(line)
			num_line += 1
		
		readfile.close()
		writefile.close()
	#### ---------------------------------------------------------------
	####  	Final Input File for Abaqus to Simpack: JobName+'-SimpackFlex.inp'	
	os.system('del '+JobName+'-TMP.inp')

	p = mdb.models[ModelName].parts[PartName2]
        session.viewports['Viewport: 1'].setValues(displayedObject=p)

	###################################################################################

	print ' '
	print JobName+'.inp is created                 -- Substructure Generation'
	print JobName+'-SimpackFlex.inp is created     -- Substructure Generation for Simpack Flex Body'
	print ' '

	return 1
