from abaqusGui import *
import os

thisPath = os.path.abspath(__file__)
thisDir = os.path.dirname(thisPath)

nlgIcon = afxCreatePNGIcon(os.path.join(thisDir, r'NLG.png'))
mlgIcon = afxCreatePNGIcon(os.path.join(thisDir, r'MLG.png'))
mlgIcon.create()


###########################################################################
# Class definition
###########################################################################

class LgsDB(AFXDataDialog):

    [ID_TIP,] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST + 1)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        # Construct the base class.
        #

        AFXDataDialog.__init__(self, form, 'Create Landing Gear Model',
            self.APPLY|self.DISMISS,
            LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|DIALOG_ACTIONS_SEPARATOR|DECOR_RESIZE)
            
        self.tipDB = None
            
        # Create icons
        #
        landingIcon = afxCreatePNGIcon(os.path.join(thisDir, r'landing.png'))
        tipIcon = afxCreatePNGIcon(os.path.join(thisDir, r'tipSmall.png'))
        fileSelectIcon = afxCreatePNGIcon(os.path.join(thisDir, r'fileSelectSmall.png'))

        # Mapping functions
        #
        FXMAPFUNC(self, SEL_COMMAND, self.ID_TIP, LgsDB.onTip)

        # Model name
        #
        AFXTextField(p=self, ncols=0,
            labelText='Name:', tgt=form.modelNameKw, sel=0,
            opts=AFXTEXTFIELD_STRING|LAYOUT_FILL_X)        

        # Tab book
        #            
        TabBook_1 = FXTabBook(p=self, tgt=None, sel=0,
            opts=TABBOOK_NORMAL|LAYOUT_FILL_X,
            x=0, y=0, w=0, h=0, pl=DEFAULT_SPACING, pr=DEFAULT_SPACING,
            pt=DEFAULT_SPACING, pb=DEFAULT_SPACING)
           
        # First tab - Analysis Parameters
        #
        tabItem = FXTabItem(p=TabBook_1, text='Analysis Parameters', ic=None, opts=TAB_TOP_NORMAL,
            x=0, y=0, w=0, h=0, pl=0, pr=0, pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        TabItem_1 = FXVerticalFrame(p=TabBook_1,
            opts=FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X,
            x=0, y=0, w=0, h=0, pl=DEFAULT_SPACING, pr=DEFAULT_SPACING,
            pt=DEFAULT_SPACING, pb=DEFAULT_SPACING, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        GroupBox_4 = FXGroupBox(p=TabItem_1, text='Aircraft Condition', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        HFrame_1 = FXHorizontalFrame(p=GroupBox_4, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_1, ncols=9, labelText='Pitch Angle (deg)', tgt=form.pitch_degKw, sel=0)
        AFXTextField(p=HFrame_1, ncols=9, labelText='Downward Initial Velocity ', tgt=form.initial_vel_yKw, sel=0)
        AFXTextField(p=HFrame_1, ncols=9, labelText='Forward Initial Velocity ', tgt=form.initial_vel_zKw, sel=0)
        HFrame_2 = FXHorizontalFrame(p=GroupBox_4, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_2, ncols=9, labelText='Gravity                ', tgt=form.gravity_accKw, sel=0)
        HFrame_6 = FXHorizontalFrame(p=GroupBox_4, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_6, ncols=9, labelText='CG X', tgt=form.CG_xKw, sel=0)
        AFXTextField(p=HFrame_6, ncols=9, labelText='CG Y', tgt=form.CG_yKw, sel=0)
        AFXTextField(p=HFrame_6, ncols=9, labelText='CG Z', tgt=form.CG_zKw, sel=0)
        GroupBox_3 = FXGroupBox(p=TabItem_1, text='Analysis Condition', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        HFrame_3 = FXHorizontalFrame(p=GroupBox_3, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_3, ncols=9, labelText='Time Constant for Landing', tgt=form.time_constantKw, sel=0)
        AFXTextField(p=HFrame_3, ncols=9, labelText='Delay Time for Lift', tgt=form.delay_time_constantKw, sel=0)
        HFrame_4 = FXHorizontalFrame(p=GroupBox_3, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_4, ncols=9, labelText='Time Step', tgt=form.time_stepKw, sel=0)
        AFXTextField(p=HFrame_4, ncols=9, labelText='End Time', tgt=form.end_timeKw, sel=0)
        HFrame_5 = FXHorizontalFrame(p=GroupBox_3, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_5, ncols=9, labelText='Field Output Number', tgt=form.field_output_numbersKw, sel=0)
        GroupBox_301 = FXGroupBox(p=TabItem_1, text='Aircraft Mass and Inertia', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        HFrame_301 = FXHorizontalFrame(p=GroupBox_301, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_301, ncols=9, labelText='Airplane Mass', tgt=form.airplane_massKw, sel=0)
        HFrame_401 = FXHorizontalFrame(p=GroupBox_301, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_401, ncols=9, labelText='Airplane Ixx', tgt=form.airplane_inertia_xKw, sel=0)
        AFXTextField(p=HFrame_401, ncols=9, labelText='Airplane Iyy', tgt=form.airplane_inertia_yKw, sel=0)
        AFXTextField(p=HFrame_401, ncols=9, labelText='Airplane Izz', tgt=form.airplane_inertia_zKw, sel=0)
        
        #FXLabel(p=TabItem_1, text='', ic=landingIcon)

        # Second tab - Nose Gear
        #
        tabItem = FXTabItem(p=TabBook_1, text='Nose Gear', ic=None, opts=TAB_TOP_NORMAL,
            x=0, y=0, w=0, h=0, pl=6, pr=6, pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        TabItem_3 = FXVerticalFrame(p=TabBook_1,
            opts=FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X,
            x=0, y=0, w=0, h=0, pl=DEFAULT_SPACING, pr=DEFAULT_SPACING,
            pt=DEFAULT_SPACING, pb=DEFAULT_SPACING, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)

        # Tip
        #
        tipFrame = FXHorizontalFrame(p=TabItem_3, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        FXLabel(p=tipFrame, text='Diagram:')
        self.tipBtn = FXButton(p=tipFrame, text='', ic=tipIcon, tgt=self, sel=self.ID_TIP, opts=FRAME_NONE)
        self.tipBtn.setTipText('Nose gear attached points')


        GroupBox_5 = FXGroupBox(p=TabItem_3, text='Reference Aircraft Attached Points ', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        HFrame_6 = FXHorizontalFrame(p=GroupBox_5, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_6, ncols=9, labelText='RP-NLG1 X', tgt=form.RP_NLG1_xKw, sel=0)
        AFXTextField(p=HFrame_6, ncols=9, labelText='RP-NLG1 Y', tgt=form.RP_NLG1_yKw, sel=0)
        AFXTextField(p=HFrame_6, ncols=9, labelText='RP-NLG1 Z', tgt=form.RP_NLG1_zKw, sel=0)
        HFrame_7 = FXHorizontalFrame(p=GroupBox_5, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_7, ncols=9, labelText='RP-NLG2 X', tgt=form.RP_NLG2_xKw, sel=0)
        AFXTextField(p=HFrame_7, ncols=9, labelText='RP-NLG2 Y', tgt=form.RP_NLG2_yKw, sel=0)
        AFXTextField(p=HFrame_7, ncols=9, labelText='RP-NLG2 Z', tgt=form.RP_NLG2_zKw, sel=0)
        HFrame_8 = FXHorizontalFrame(p=GroupBox_5, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_8, ncols=9, labelText='RP-NLG3 X', tgt=form.RP_NLG3_xKw, sel=0)
        AFXTextField(p=HFrame_8, ncols=9, labelText='RP-NLG3 Y', tgt=form.RP_NLG3_yKw, sel=0)
        AFXTextField(p=HFrame_8, ncols=9, labelText='RP-NLG3 Z', tgt=form.RP_NLG3_zKw, sel=0)
        HFrame_9 = FXHorizontalFrame(p=GroupBox_5, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_9, ncols=9, labelText='RP-NLG4 X', tgt=form.RP_NLG4_xKw, sel=0)
        AFXTextField(p=HFrame_9, ncols=9, labelText='RP-NLG4 Y', tgt=form.RP_NLG4_yKw, sel=0)
        AFXTextField(p=HFrame_9, ncols=9, labelText='RP-NLG4 Z', tgt=form.RP_NLG4_zKw, sel=0)
        GroupBox_6 = FXGroupBox(p=TabItem_3, text='NLG Modeling Parameters', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        HFrame_10 = FXHorizontalFrame(p=GroupBox_6, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_10, ncols=9, labelText='ALPHA_nose (deg)', tgt=form.alpha_noseKw, sel=0)
        HFrame_11 = FXHorizontalFrame(p=GroupBox_6, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_11, ncols=9, labelText='SB_NLG     ', tgt=form.SB_NLGKw, sel=0)
        AFXTextField(p=HFrame_11, ncols=9, labelText='MF_NLG', tgt=form.MF_NLGKw, sel=0)
        AFXTextField(p=HFrame_11, ncols=9, labelText='Piston_NLG', tgt=form.Piston_NLGKw, sel=0)
        AFXTextField(p=HFrame_11, ncols=9, labelText='WA_NLG', tgt=form.WA_NLGKw, sel=0)
        AFXTextField(p=HFrame_11, ncols=9, labelText='Tire_NLG', tgt=form.Tire_NLGKw, sel=0)
        HFrame_13 = FXHorizontalFrame(p=GroupBox_6, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_13, ncols=9, labelText='S.A. Stroke', tgt=form.SA_NLG_StrokeKw, sel=0)
        AFXTextField(p=HFrame_13, ncols=9, labelText='Tire_Damping_Coeff', tgt=form.nosetire_damping_coeffKw, sel=0)
        HFrame_131 = FXHorizontalFrame(p=GroupBox_6, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        fileHandler = LgsDBFileHandler(form, 'spring_nose_filename', 'Property Files (*.property)\nAll Files (*.*)')
        fileTextHf = FXHorizontalFrame(p=HFrame_131, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)

        AFXTextField(p=fileTextHf, ncols=12,
            labelText='Spring Data File', tgt=form.spring_nose_filenameKw, sel=0,
            opts=AFXTEXTFIELD_STRING|LAYOUT_CENTER_Y)
        FXButton(p=fileTextHf, text='\tSelect File\nFrom Dialog', ic=fileSelectIcon, tgt=fileHandler, sel=AFXMode.ID_ACTIVATE,
            opts=BUTTON_NORMAL|LAYOUT_CENTER_Y, x=0, y=0, w=0, h=0, pl=1, pr=1, pt=1, pb=1)
        fileHandler = LgsDBFileHandler(form, 'damper_nose_filename', 'Property Files (*.property)\nAll Files (*.*)')
        fileTextHf = FXHorizontalFrame(p=HFrame_131, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)

        AFXTextField(p=fileTextHf, ncols=12,
            labelText='Damper Data File', tgt=form.damper_nose_filenameKw, sel=0,
            opts=AFXTEXTFIELD_STRING|LAYOUT_CENTER_Y)
        FXButton(p=fileTextHf, text='\tSelect File\nFrom Dialog', ic=fileSelectIcon, tgt=fileHandler, sel=AFXMode.ID_ACTIVATE,
            opts=BUTTON_NORMAL|LAYOUT_CENTER_Y, x=0, y=0, w=0, h=0, pl=1, pr=1, pt=1, pb=1)
        fileHandler = LgsDBFileHandler(form, 'tire_nose_filename', 'Property Files (*.property)\nAll Files (*.*)')
        fileTextHf = FXHorizontalFrame(p=HFrame_131, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)

        AFXTextField(p=fileTextHf, ncols=12,
            labelText='Tire Data File', tgt=form.tire_nose_filenameKw, sel=0,
            opts=AFXTEXTFIELD_STRING|LAYOUT_CENTER_Y)
        FXButton(p=fileTextHf, text='\tSelect File\nFrom Dialog', ic=fileSelectIcon, tgt=fileHandler, sel=AFXMode.ID_ACTIVATE,
            opts=BUTTON_NORMAL|LAYOUT_CENTER_Y, x=0, y=0, w=0, h=0, pl=1, pr=1, pt=1, pb=1)
        GroupBox_601 = FXGroupBox(p=TabItem_3, text='NLG Mass and Inertia', opts=FRAME_GROOVE|LAYOUT_FILL_X)

        hf = FXHorizontalFrame(p=GroupBox_601, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)

        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=va, ncols=9, labelText='FB mass', tgt=form.nose_FB_mKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='TP mass', tgt=form.nose_TP_mKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='FB Ixx    ', tgt=form.nose_FB_m_xKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Piston Ixx', tgt=form.nose_piston_m_xKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='TB Ixx    ', tgt=form.nose_TB_m_xKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Wheel Ixx', tgt=form.nose_Wheel_m_xKw, sel=0)


        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=va, ncols=9, labelText='MF mass', tgt=form.nose_MF_mKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='TB mass', tgt=form.nose_TB_mKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='FB Iyy    ', tgt=form.nose_FB_m_yKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Piston Iyy', tgt=form.nose_piston_m_yKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='TB Iyy    ', tgt=form.nose_TB_m_yKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Wheel Iyy', tgt=form.nose_Wheel_m_yKw, sel=0)


        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=va, ncols=9, labelText='piston mass', tgt=form.nose_piston_mKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='WA mass', tgt=form.nose_WA_mKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='FB Izz    ', tgt=form.nose_FB_m_zKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Piston Izz', tgt=form.nose_piston_m_zKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='TB Izz    ', tgt=form.nose_TB_m_zKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Wheel Izz', tgt=form.nose_Wheel_m_zKw, sel=0)


        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        FXLabel(p=va, text=' ', pt=8)
        AFXTextField(p=va, ncols=9, labelText='Wheel mass', tgt=form.nose_Wheel_mKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='MF Ixx', tgt=form.nose_MF_m_xKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='TP Ixx', tgt=form.nose_TP_m_xKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='WA Ixx', tgt=form.nose_WA_m_xKw, sel=0)


        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        FXLabel(p=va, text=' ', pt=8)
        FXLabel(p=va, text=' ', pt=8)
        AFXTextField(p=va, ncols=9, labelText='MF Iyy', tgt=form.nose_MF_m_yKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='TP Iyy', tgt=form.nose_TP_m_yKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='WA Iyy', tgt=form.nose_WA_m_yKw, sel=0)


        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        FXLabel(p=va, text=' ', pt=8)
        FXLabel(p=va, text=' ', pt=8)
        AFXTextField(p=va, ncols=9, labelText='MF Izz', tgt=form.nose_MF_m_zKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='TP Izz', tgt=form.nose_TP_m_zKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='WA Izz', tgt=form.nose_WA_m_zKw, sel=0)


        
        #FXLabel(p=TabItem_3, text='', ic=nlgIcon)

        # Third tab - Main Gear
        #
        tabItem = FXTabItem(p=TabBook_1, text='Main Gear', ic=None, opts=TAB_TOP_NORMAL,
            x=0, y=0, w=0, h=0, pl=6, pr=6, pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        TabItem_2 = FXVerticalFrame(p=TabBook_1,
            opts=FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X,
            x=0, y=0, w=0, h=0, pl=DEFAULT_SPACING, pr=DEFAULT_SPACING,
            pt=DEFAULT_SPACING, pb=DEFAULT_SPACING, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)


        # Tip
        #
        tipFrame = FXHorizontalFrame(p=TabItem_2, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        FXLabel(p=tipFrame, text='Diagram:')
        self.tipBtn = FXButton(p=tipFrame, text='', ic=tipIcon, tgt=self, sel=self.ID_TIP, opts=FRAME_NONE)
        self.tipBtn.setTipText('Main gear attached points')
        

        GroupBox_51 = FXGroupBox(p=TabItem_2, text='Reference Aircraft Attached Points (mm)', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        HFrame_61 = FXHorizontalFrame(p=GroupBox_51, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_61, ncols=9, labelText='RP-MLG1 X', tgt=form.RP_MLG1_xKw, sel=0)
        AFXTextField(p=HFrame_61, ncols=9, labelText='RP-MLG1 Y', tgt=form.RP_MLG1_yKw, sel=0)
        AFXTextField(p=HFrame_61, ncols=9, labelText='RP-MLG1 Z', tgt=form.RP_MLG1_zKw, sel=0)
        HFrame_71 = FXHorizontalFrame(p=GroupBox_51, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_71, ncols=9, labelText='RP-MLG2 X', tgt=form.RP_MLG2_xKw, sel=0)
        AFXTextField(p=HFrame_71, ncols=9, labelText='RP-MLG2 Y', tgt=form.RP_MLG2_yKw, sel=0)
        AFXTextField(p=HFrame_71, ncols=9, labelText='RP-MLG2 Z', tgt=form.RP_MLG2_zKw, sel=0)
        HFrame_81 = FXHorizontalFrame(p=GroupBox_51, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        AFXTextField(p=HFrame_81, ncols=9, labelText='RP-MLG3 X', tgt=form.RP_MLG3_xKw, sel=0)
        AFXTextField(p=HFrame_81, ncols=9, labelText='RP-MLG3 Y', tgt=form.RP_MLG3_yKw, sel=0)
        AFXTextField(p=HFrame_81, ncols=9, labelText='RP-MLG3 Z', tgt=form.RP_MLG3_zKw, sel=0)

        gb = FXGroupBox(p=TabItem_2, text='MLG Modeling Parameters', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        hf = FXHorizontalFrame(p=gb, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)

        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)  
        AFXTextField(p=va, ncols=9, labelText='ALPHA_mf (deg)', tgt=form.alpha_mfKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='SB_MLG  ', tgt=form.SB_MLGKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='WA_MLG', tgt=form.WA_MLGKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='S.A. Stroke', tgt=form.SA_MLG_StrokeKw, sel=0)

        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)  
        AFXTextField(p=va, ncols=9, labelText='ALPHA_shock (deg)', tgt=form.alpha_shockKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='MF_MLG', tgt=form.MF_MLGKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Tire_MLG', tgt=form.Tire_MLGKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Tire_Damping_Coeff', tgt=form.maintire_damping_coeffKw, sel=0)

        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)  
        FXLabel(p=va, text=' ', pt=8)        
        AFXTextField(p=va, ncols=9, labelText='TA_MLG', tgt=form.TA_MLGKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Offset_SA_MLG', tgt=form.Offset_SA_MLGKw, sel=0)

        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)  
        FXLabel(p=va, text=' ', pt=8)        
        AFXTextField(p=va, ncols=9, labelText='SA_MLG', tgt=form.SA_MLGKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Offset_MF_MLG', tgt=form.Offset_MF_MLGKw, sel=0)

        HFrame_2131 = FXHorizontalFrame(p=gb, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        fileHandler = LgsDBFileHandler(form, 'spring_main_filename', 'Property Files (*.property)\nAll Files (*.*)')
        fileTextHf = FXHorizontalFrame(p=HFrame_2131, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        
        AFXTextField(p=fileTextHf, ncols=12,
            labelText='Spring Data File', tgt=form.spring_main_filenameKw, sel=0,
            opts=AFXTEXTFIELD_STRING|LAYOUT_CENTER_Y)
        FXButton(p=fileTextHf, text='\tSelect File\nFrom Dialog', ic=fileSelectIcon, tgt=fileHandler, sel=AFXMode.ID_ACTIVATE,
            opts=BUTTON_NORMAL|LAYOUT_CENTER_Y, x=0, y=0, w=0, h=0, pl=1, pr=1, pt=1, pb=1)
        fileHandler = LgsDBFileHandler(form, 'damper_main_filename', 'Property Files (*.property)\nAll Files (*.*)')
        fileTextHf = FXHorizontalFrame(p=HFrame_2131, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)

        AFXTextField(p=fileTextHf, ncols=12,
            labelText='Damper Data File', tgt=form.damper_main_filenameKw, sel=0,
            opts=AFXTEXTFIELD_STRING|LAYOUT_CENTER_Y)
        FXButton(p=fileTextHf, text='\tSelect File\nFrom Dialog', ic=fileSelectIcon, tgt=fileHandler, sel=AFXMode.ID_ACTIVATE,
            opts=BUTTON_NORMAL|LAYOUT_CENTER_Y, x=0, y=0, w=0, h=0, pl=1, pr=1, pt=1, pb=1)
        fileHandler = LgsDBFileHandler(form, 'tire_main_filename', 'Property Files (*.property)\nAll Files (*.*)')
        fileTextHf = FXHorizontalFrame(p=HFrame_2131, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)

        AFXTextField(p=fileTextHf, ncols=12,
            labelText='Tire Data File      ', tgt=form.tire_main_filenameKw, sel=0,
            opts=AFXTEXTFIELD_STRING|LAYOUT_CENTER_Y)
        FXButton(p=fileTextHf, text='\tSelect File\nFrom Dialog', ic=fileSelectIcon, tgt=fileHandler, sel=AFXMode.ID_ACTIVATE,
            opts=BUTTON_NORMAL|LAYOUT_CENTER_Y, x=0, y=0, w=0, h=0, pl=1, pr=1, pt=1, pb=1)
        gb = FXGroupBox(p=TabItem_2, text='MLG Mass and Inertia', opts=FRAME_GROOVE|LAYOUT_FILL_X)

        hf = FXHorizontalFrame(p=gb, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)

        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)        
        AFXTextField(p=va, ncols=9, labelText='FB mass', tgt=form.main_FB_mKw, sel=0)        
        AFXTextField(p=va, ncols=9, labelText='WA mass', tgt=form.main_WA_mKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='FB Ixx  ', tgt=form.main_FB_m_xKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='TA Ixx  ', tgt=form.main_TA_m_xKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='SAB Ixx', tgt=form.main_SAB_m_xKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Wheel Ixx', tgt=form.main_Wheel_m_xKw, sel=0)        
        
        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)        
        AFXTextField(p=va, ncols=9, labelText='MF mass', tgt=form.main_MF_mKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Wheel mass', tgt=form.main_Wheel_mKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='FB Iyy', tgt=form.main_FB_m_yKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='TA Iyy', tgt=form.main_TA_m_yKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='SAB Iyy', tgt=form.main_SAB_m_yKw, sel=0)        
        AFXTextField(p=va, ncols=9, labelText='Wheel Iyy', tgt=form.main_Wheel_m_yKw, sel=0)
        
        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)        
        AFXTextField(p=va, ncols=9, labelText='TA mass', tgt=form.main_TA_mKw, sel=0)
        FXLabel(p=va, text=' ', pt=8)
        AFXTextField(p=va, ncols=9, labelText='FB Izz', tgt=form.main_FB_m_zKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='TA Izz', tgt=form.main_TA_m_zKw, sel=0)        
        AFXTextField(p=va, ncols=9, labelText='SAB Izz', tgt=form.main_SAB_m_zKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='Wheel Izz', tgt=form.main_Wheel_m_zKw, sel=0)
        
        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)        
        AFXTextField(p=va, ncols=9, labelText='SAT mass', tgt=form.main_SAT_mKw, sel=0)
        FXLabel(p=va, text=' ', pt=8)
        AFXTextField(p=va, ncols=9, labelText='MF Ixx', tgt=form.main_MF_m_xKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='SAT Ixx', tgt=form.main_SAT_m_xKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='WA Ixx', tgt=form.main_WA_m_xKw, sel=0)

        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)        
        AFXTextField(p=va, ncols=9, labelText='SAB mass', tgt=form.main_SAB_mKw, sel=0)
        FXLabel(p=va, text=' ', pt=8)
        AFXTextField(p=va, ncols=9, labelText='MF Iyy', tgt=form.main_MF_m_yKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='SAT Iyy', tgt=form.main_SAT_m_yKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='WA Iyy', tgt=form.main_WA_m_yKw, sel=0)
        
        va = AFXVerticalAligner(hf, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)        
        FXLabel(p=va, text=' ', pt=8)
        FXLabel(p=va, text=' ', pt=8)        
        AFXTextField(p=va, ncols=9, labelText='MF Izz', tgt=form.main_MF_m_zKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='SAT Izz', tgt=form.main_SAT_m_zKw, sel=0)
        AFXTextField(p=va, ncols=9, labelText='WA Izz', tgt=form.main_WA_m_zKw, sel=0)

        # Check box to automaticall import an input file for visualization
        #
        #FXCheckButton(p=self, text='Import visualization model', tgt=form.visKw, sel=0)
        
        #FXLabel(p=TabItem_2, text='', ic=mlgIcon)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onTip(self, sender, sel, ptr):

        if not self.tipDB:
            self.tipDB = TipDB(self)
            self.tipDB.create()

        txt = sender.getTipText()
        self.tipDB.label.setText(txt)
        if 'Main' in txt:
            self.tipDB.img.setIcon(mlgIcon)
        else:
            self.tipDB.img.setIcon(nlgIcon)
        self.tipDB.show()


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        if self.tipDB:
            self.tipDB.hide()
        AFXDataDialog.hide(self)


##########################################################################
# Class definition
###########################################################################

class TipDB(AFXDialog):
    
    #numInstance = 0
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, dbOwner):

        AFXDialog.__init__(self, 'Points Attached Tip',
            self.DISMISS, DIALOG_NORMAL|DIALOG_UNPOST_DESTROY|DECOR_RESIZE)

        self.label = FXLabel(p=self, text='')
        self.img = FXLabel(p=self, text='', ic=nlgIcon)


###########################################################################
# Class definition
###########################################################################

class LgsDBFileHandler(FXObject):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form, keyword, patterns='*'):

        self.form = form
        self.patterns = patterns
        self.patternTgt = AFXIntTarget(0)
        exec('self.fileNameKw = form.%sKw' % keyword)
        self.readOnlyKw = AFXBoolKeyword(None, 'readOnly', AFXBoolKeyword.TRUE_FALSE)
        FXObject.__init__(self)
        FXMAPFUNC(self, SEL_COMMAND, AFXMode.ID_ACTIVATE, LgsDBFileHandler.activate)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self, sender, sel, ptr):

       fileDb = AFXFileSelectorDialog(getAFXApp().getAFXMainWindow(), 'Select a File',
           self.fileNameKw, self.readOnlyKw,
           AFXSELECTFILE_ANY, self.patterns, self.patternTgt)
       fileDb.setReadOnlyPatterns('*.property')
       fileDb.create()
       fileDb.showModal()
        
