from abaqusGui import *

###########################################################################
# Class definition
###########################################################################

class LgsForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        
        # Construct the base class.
        #
        AFXForm.__init__(self, owner)

        self.cmd = AFXGuiCommand(mode=self, method='createLGSModel',
            objectName='abq_Lgs.lgsUtils', registerQuery=False)
        pickedDefault = ''

        self.modelNameKw = AFXStringKeyword(self.cmd, 'modelName', True,  'Landing_Gear')
        self.pitch_degKw = AFXFloatKeyword(self.cmd, 'pitch_deg', True, 0)
        self.initial_vel_yKw = AFXFloatKeyword(self.cmd, 'initial_vel_y', True, -2.44e3)
        self.initial_vel_zKw = AFXFloatKeyword(self.cmd, 'initial_vel_z', True, 2e3)
        self.gravity_accKw = AFXFloatKeyword(self.cmd, 'gravity_acc', True, 9800)
        self.CG_xKw = AFXFloatKeyword(self.cmd, 'CG_x', True, 0)
        self.CG_yKw = AFXFloatKeyword(self.cmd, 'CG_y', True,  -8.3)
        self.CG_zKw = AFXFloatKeyword(self.cmd, 'CG_z', True, -11.233e3)
        self.time_constantKw = AFXFloatKeyword(self.cmd, 'time_constant', True, 10)
        self.delay_time_constantKw = AFXFloatKeyword(self.cmd, 'delay_time_constant', True, 0.4)
        self.time_stepKw = AFXFloatKeyword(self.cmd, 'time_step', True, 2.e-5)
        self.end_timeKw = AFXFloatKeyword(self.cmd, 'end_time', True, 1)
        self.field_output_numbersKw = AFXIntKeyword(self.cmd, 'field_output_numbers', True, 50)
        self.RP_NLG1_xKw = AFXFloatKeyword(self.cmd, 'RP_NLG1_x', True,  222.75)
        self.RP_NLG1_yKw = AFXFloatKeyword(self.cmd, 'RP_NLG1_y', True, -756.)
        self.RP_NLG1_zKw = AFXFloatKeyword(self.cmd, 'RP_NLG1_z', True, -2.552e3)
        self.RP_NLG2_xKw = AFXFloatKeyword(self.cmd, 'RP_NLG2_x', True, -222.75)
        self.RP_NLG2_yKw = AFXFloatKeyword(self.cmd, 'RP_NLG2_y', True, -756.)
        self.RP_NLG2_zKw = AFXFloatKeyword(self.cmd, 'RP_NLG2_z', True, -2.552e3)
        self.RP_NLG3_xKw = AFXFloatKeyword(self.cmd, 'RP_NLG3_x', True,  230.5)
        self.RP_NLG3_yKw = AFXFloatKeyword(self.cmd, 'RP_NLG3_y', True, -1.115e3)
        self.RP_NLG3_zKw = AFXFloatKeyword(self.cmd, 'RP_NLG3_z', True, -3.145e3)
        self.RP_NLG4_xKw = AFXFloatKeyword(self.cmd, 'RP_NLG4_x', True, -230.5)
        self.RP_NLG4_yKw = AFXFloatKeyword(self.cmd, 'RP_NLG4_y', True, -1.115e3)
        self.RP_NLG4_zKw = AFXFloatKeyword(self.cmd, 'RP_NLG4_z', True, -3.145e3)
        self.alpha_noseKw = AFXFloatKeyword(self.cmd, 'alpha_nose', True,  7.5)
        self.SB_NLGKw = AFXFloatKeyword(self.cmd, 'SB_NLG', True,  600)
        self.MF_NLGKw = AFXFloatKeyword(self.cmd, 'MF_NLG', True,  690)
        self.Piston_NLGKw = AFXFloatKeyword(self.cmd, 'Piston_NLG', True,  550)
        self.WA_NLGKw = AFXFloatKeyword(self.cmd, 'WA_NLG', True,  240)
        self.Tire_NLGKw = AFXFloatKeyword(self.cmd, 'Tire_NLG', True,  212)
        self.SA_NLG_StrokeKw = AFXFloatKeyword(self.cmd, 'SA_NLG_Stroke', True,  300)
        self.nosetire_damping_coeffKw = AFXFloatKeyword(self.cmd, 'nosetire_damping_coeff', True,  10)
        self.spring_nose_filenameKw = AFXStringKeyword(self.cmd, 'spring_nose_filename', True,  'spring_nose.property')
        self.damper_nose_filenameKw = AFXStringKeyword(self.cmd, 'damper_nose_filename', True,  'damper_nose.property')
        self.tire_nose_filenameKw = AFXStringKeyword(self.cmd, 'tire_nose_filename', True,  'tire_nose.property')
        self.RP_MLG1_xKw = AFXFloatKeyword(self.cmd, 'RP_MLG1_x', True,  2.169e3)
        self.RP_MLG1_yKw = AFXFloatKeyword(self.cmd, 'RP_MLG1_y', True, -1.074e3)
        self.RP_MLG1_zKw = AFXFloatKeyword(self.cmd, 'RP_MLG1_z', True, -11.567e3)
        self.RP_MLG2_xKw = AFXFloatKeyword(self.cmd, 'RP_MLG2_x', True,  2.233e3)
        self.RP_MLG2_yKw = AFXFloatKeyword(self.cmd, 'RP_MLG2_y', True, -1.136e3)
        self.RP_MLG2_zKw = AFXFloatKeyword(self.cmd, 'RP_MLG2_z', True, -11.065e3)
        self.RP_MLG3_xKw = AFXFloatKeyword(self.cmd, 'RP_MLG3_x', True,  1.429e3)
        self.RP_MLG3_yKw = AFXFloatKeyword(self.cmd, 'RP_MLG3_y', True, -1.136e3)
        self.RP_MLG3_zKw = AFXFloatKeyword(self.cmd, 'RP_MLG3_z', True, -10.806e3)
        self.alpha_mfKw = AFXFloatKeyword(self.cmd, 'alpha_mf', True,  4.)
        self.alpha_shockKw = AFXFloatKeyword(self.cmd, 'alpha_shock', True,  9.)
        self.SB_MLGKw = AFXFloatKeyword(self.cmd, 'SB_MLG', True,  880)
        self.MF_MLGKw = AFXFloatKeyword(self.cmd, 'MF_MLG', True,  950)
        self.TA_MLGKw = AFXFloatKeyword(self.cmd, 'TA_MLG', True,  460)
        self.SA_MLGKw = AFXFloatKeyword(self.cmd, 'SA_MLG', True,  1330)
        self.WA_MLGKw = AFXFloatKeyword(self.cmd, 'WA_MLG', True,  480)
        self.Tire_MLGKw = AFXFloatKeyword(self.cmd, 'Tire_MLG', True,  370)
        self.Offset_SA_MLGKw = AFXFloatKeyword(self.cmd, 'Offset_SA_MLG', True,  146)
        self.Offset_MF_MLGKw = AFXFloatKeyword(self.cmd, 'Offset_MF_MLG', True,  270)
        self.SA_MLG_StrokeKw = AFXFloatKeyword(self.cmd, 'SA_MLG_Stroke', True,  360)
        self.maintire_damping_coeffKw = AFXFloatKeyword(self.cmd, 'maintire_damping_coeff', True,  10)
        self.spring_main_filenameKw = AFXStringKeyword(self.cmd, 'spring_main_filename', True,  'spring_main.property')
        self.damper_main_filenameKw = AFXStringKeyword(self.cmd, 'damper_main_filename', True,  'damper_main.property')
        self.tire_main_filenameKw = AFXStringKeyword(self.cmd, 'tire_main_filename', True,  'tire_main.property')
    #
        self.airplane_massKw = AFXFloatKeyword(self.cmd, 'airplane_mass', True, 22.0)
        self.airplane_inertia_xKw = AFXFloatKeyword(self.cmd, 'airplane_inertia_x', True, 65.e6)
        self.airplane_inertia_yKw = AFXFloatKeyword(self.cmd, 'airplane_inertia_y', True, 100.e6)
        self.airplane_inertia_zKw = AFXFloatKeyword(self.cmd, 'airplane_inertia_z', True, 100.e6)
    #
        self.nose_FB_mKw = AFXFloatKeyword(self.cmd, 'nose_FB_m', True, 0.001)
        self.nose_MF_mKw = AFXFloatKeyword(self.cmd, 'nose_MF_m', True, 0.003)
        self.nose_piston_mKw = AFXFloatKeyword(self.cmd, 'nose_piston_m', True, 0.003)
        self.nose_TP_mKw = AFXFloatKeyword(self.cmd, 'nose_TP_m', True, 0.001)
        self.nose_TB_mKw = AFXFloatKeyword(self.cmd, 'nose_TB_m', True, 0.001)
        self.nose_WA_mKw = AFXFloatKeyword(self.cmd, 'nose_WA_m', True, 0.001)
        self.nose_Wheel_mKw = AFXFloatKeyword(self.cmd, 'nose_Wheel_m', True, 0.1)
        self.main_FB_mKw = AFXFloatKeyword(self.cmd, 'main_FB_m', True, 0.006)
        self.main_MF_mKw = AFXFloatKeyword(self.cmd, 'main_MF_m', True, 0.006)
        self.main_TA_mKw = AFXFloatKeyword(self.cmd, 'main_TA_m', True, 0.005)
        self.main_SAT_mKw = AFXFloatKeyword(self.cmd, 'main_SAT_m', True, 0.001)
        self.main_SAB_mKw = AFXFloatKeyword(self.cmd, 'main_SAB_m', True, 0.001)
        self.main_WA_mKw = AFXFloatKeyword(self.cmd, 'main_WA_m', True, 0.001)
        self.main_Wheel_mKw = AFXFloatKeyword(self.cmd, 'main_Wheel_m', True, 0.1)
    #
        self.nose_FB_m_xKw = AFXFloatKeyword(self.cmd, 'nose_FB_m_x', True, 1.e4)
        self.nose_FB_m_yKw = AFXFloatKeyword(self.cmd, 'nose_FB_m_y', True, 1.e4)
        self.nose_FB_m_zKw = AFXFloatKeyword(self.cmd, 'nose_FB_m_z', True, 2.e3)
        self.nose_MF_m_xKw = AFXFloatKeyword(self.cmd, 'nose_MF_m_x', True, 6.e2)
        self.nose_MF_m_yKw = AFXFloatKeyword(self.cmd, 'nose_MF_m_y', True, 6.e2)
        self.nose_MF_m_zKw = AFXFloatKeyword(self.cmd, 'nose_MF_m_z', True, 8.e1)
        self.nose_piston_m_xKw = AFXFloatKeyword(self.cmd, 'nose_piston_m_x', True, 4.e1)
        self.nose_piston_m_yKw = AFXFloatKeyword(self.cmd, 'nose_piston_m_y', True, 4.e1)
        self.nose_piston_m_zKw = AFXFloatKeyword(self.cmd, 'nose_piston_m_z', True, 1.e-1)
        self.nose_TP_m_xKw = AFXFloatKeyword(self.cmd, 'nose_TP_m_x', True, 1.e-1)
        self.nose_TP_m_yKw = AFXFloatKeyword(self.cmd, 'nose_TP_m_y', True, 1.e-1)
        self.nose_TP_m_zKw = AFXFloatKeyword(self.cmd, 'nose_TP_m_z', True, 1.e-3)
        self.nose_TB_m_xKw = AFXFloatKeyword(self.cmd, 'nose_TB_m_x', True, 1.e-1)
        self.nose_TB_m_yKw = AFXFloatKeyword(self.cmd, 'nose_TB_m_y', True, 1.e-1)
        self.nose_TB_m_zKw = AFXFloatKeyword(self.cmd, 'nose_TB_m_z', True, 1.e-3)
        self.nose_WA_m_xKw = AFXFloatKeyword(self.cmd, 'nose_WA_m_x', True, 2.e0)
        self.nose_WA_m_yKw = AFXFloatKeyword(self.cmd, 'nose_WA_m_y', True, 2.e0)
        self.nose_WA_m_zKw = AFXFloatKeyword(self.cmd, 'nose_WA_m_z', True, 2.e-2)
        self.nose_Wheel_m_xKw = AFXFloatKeyword(self.cmd, 'nose_Wheel_m_x', True, 3.e3)
        self.nose_Wheel_m_yKw = AFXFloatKeyword(self.cmd, 'nose_Wheel_m_y', True, 3.e3)
        self.nose_Wheel_m_zKw = AFXFloatKeyword(self.cmd, 'nose_Wheel_m_z', True, 1.e3)
    #
        self.main_FB_m_xKw = AFXFloatKeyword(self.cmd, 'main_FB_m_x', True, 1.e3)
        self.main_FB_m_yKw = AFXFloatKeyword(self.cmd, 'main_FB_m_y', True, 1.e3)
        self.main_FB_m_zKw = AFXFloatKeyword(self.cmd, 'main_FB_m_z', True, 2.e0)
        self.main_MF_m_xKw = AFXFloatKeyword(self.cmd, 'main_MF_m_x', True, 5.e3)
        self.main_MF_m_yKw = AFXFloatKeyword(self.cmd, 'main_MF_m_y', True, 5.e3)
        self.main_MF_m_zKw = AFXFloatKeyword(self.cmd, 'main_MF_m_z', True, 1.e2)
        self.main_TA_m_xKw = AFXFloatKeyword(self.cmd, 'main_TA_m_x', True, 7.e0)
        self.main_TA_m_yKw = AFXFloatKeyword(self.cmd, 'main_TA_m_y', True, 7.e0)
        self.main_TA_m_zKw = AFXFloatKeyword(self.cmd, 'main_TA_m_z', True, 4.e-2)
        self.main_SAT_m_xKw = AFXFloatKeyword(self.cmd, 'main_SAT_m_x', True, 2.e2)
        self.main_SAT_m_yKw = AFXFloatKeyword(self.cmd, 'main_SAT_m_y', True, 2.e2)
        self.main_SAT_m_zKw = AFXFloatKeyword(self.cmd, 'main_SAT_m_z', True, 1.e0)
        self.main_SAB_m_xKw = AFXFloatKeyword(self.cmd, 'main_SAB_m_x', True, 4.e1)
        self.main_SAB_m_yKw = AFXFloatKeyword(self.cmd, 'main_SAB_m_y', True, 4.e1)
        self.main_SAB_m_zKw = AFXFloatKeyword(self.cmd, 'main_SAB_m_z', True, 7.e-2)
        self.main_WA_m_xKw = AFXFloatKeyword(self.cmd, 'main_WA_m_x', True, 2.e1)
        self.main_WA_m_yKw = AFXFloatKeyword(self.cmd, 'main_WA_m_y', True, 2.e1)
        self.main_WA_m_zKw = AFXFloatKeyword(self.cmd, 'main_WA_m_z', True, 5.e-2)
        self.main_Wheel_m_xKw = AFXFloatKeyword(self.cmd, 'main_Wheel_m_x', True, 8.e4)
        self.main_Wheel_m_yKw = AFXFloatKeyword(self.cmd, 'main_Wheel_m_y', True, 8.e4)
        self.main_Wheel_m_zKw = AFXFloatKeyword(self.cmd, 'main_Wheel_m_z', True, 4.e4)

        # Visualization input file keyword
        #
        #self.visKw = AFXBoolKeyword(self.cmd, 'vis', AFXBoolKeyword.TRUE_FALSE, True, False)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        from abq_Lgs.lgsDB import LgsDB
        return LgsDB(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        # Check for empty values
        #
        kws = [self.spring_main_filenameKw, self.damper_nose_filenameKw,
               self.tire_nose_filenameKw, self.spring_main_filenameKw, 
               self.damper_main_filenameKw, self.tire_main_filenameKw, ]
               
        db = self.getCurrentDialog()

        if self.modelNameKw.getValue() == '':
            showAFXErrorDialog(db, 'A model name must be entered.')
            db.onKeywordError(self.modelNameKw)
            return False        
        
        for kw in kws:
            if kw.getValue() == '':
                showAFXErrorDialog(db, 'A file name for must be entered.')
                db.onKeywordError(kw)
                return False
            else:
                if not os.path.isfile(kw.getValue()):
                    showAFXErrorDialog(db, 'File specified does not exist.')
                    db.onKeywordError(kw)
                    return False
                    
        return True