from abaqus import *
from abaqusConstants import *
import visualization, uti
import math
from odbAccess import *
from abaqus import getInputs

# opening odbs with and without far-field results 

odbPath = getInput('Enter original job name')
odbPath1 = str(odbPath) + ".odb"
odb = openOdb(path=odbPath1)
newodbPath = str(odbPath) + "_acvis.odb"
newodb = openOdb(path=newodbPath)

for step in odb.steps.values():
    plotdata = []
    print 'Working on step %s' % step.name
    isFirst = 1
	
    #computing incident side sum  

    for frame in step.frames:
        if isFirst:
            isFirst = 0
            continue
        if not frame.fieldOutputs.has_key('INTEN'):
            continue
        freq = frame.frameValue
        print '  Working on frequency %s' % freq
        IN_set = odb.rootAssembly.elementSets['IN_SIDE']		
        incident = frame.fieldOutputs['INTEN'].getSubset(position=CENTROID).getSubset(region=IN_set)	
        incident_sum = 0               
        for value in incident.values:	
            temp1 = value.data
            temp2 = value.conjugateData
            mag = 0.0
            for i in range(len(temp1)):
                mag += abs((temp1[i]**2) + (temp2[i]**2))				
            incident_sum += mag**0.5

        #computing transmitted side (far-field) sum  

        nframe = newodb.steps[step.name].frames[frame.incrementNumber]
        inst = newodb.rootAssembly.instances['INF-INST']	
        transmitted = nframe.fieldOutputs['INTEN_FAR']
        transmitted_sum = 0
        
        for elem in inst.elements:
            a = 0
            elemval = 0				
            for nodeNum in elem.connectivity:
                a += 1					
                for value in transmitted.values:
                    if nodeNum == value.nodeLabel:						
                        temp = value.data					
                        elemval += temp  
            # average of nodal INTEN_FAR values for the element						
            elemavg = elemval/a 	
            transmitted_sum += elemavg	
        stl = 10.0*math.log10(incident_sum/transmitted_sum)		
        plotdata.append((freq,stl))
	

    # create and plot data
    if len(plotdata) == 0:
        raise ValueError, 'No data could be calculated\nbetween given sets.'
    curveName = 'STL in %s' %step.name
    xy1 = session.XYData(name=curveName, data=plotdata)
    try:
        xyp = session.xyPlots['XYPlot-1']
    except:
        xyp = session.XYPlot('XYPlot-1')
    
    
    chartName = xyp.charts.keys()[0]
    chart = xyp.charts[chartName]
    c1 = session.Curve(xyData=xy1)
    chart.setValues(curvesToPlot=(c1, ), )
    chart.axes1[0].axisData.setValues(useSystemTitle=False, title='Frequency',)
    chart.axes2[0].axisData.setValues(useSystemTitle=False, title='STL (dB)',)
    
    vp = session.viewports[session.currentViewportName]
    vp.setValues(displayedObject=xyp)		