from abaqusGui import *
from abaqusConstants import *
from abq_BrakeNoise.computeContributionsConstants import *


###########################################################################
# Class definition
###########################################################################

class ComputeContributionsDB(AFXDataDialog):
    [
        ID_OUTPUT_FILE,
        ID_ADD,
        ID_REMOVE,
        ID_SELECT_ALL,
        ID_CLEAR_ALL,
        ID_TABLE,
        ID_RIGHT_CLICK,
        ID_COMPONENT_ODB,
        ID_CCF,
        ID_CMCF,
        ID_LAST,
    ] = range(AFXDataDialog.ID_LAST,AFXDataDialog.ID_LAST+11)


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'Compute Contributions', 
                               opts=DIALOG_ACTIONS_SEPARATOR|DECOR_RESIZE)

        self.form = form

        # Get the product version number
        #
        self.rightClick = TRUE
        major, minor, update = getAFXApp().getBaseVersionNumbers()
        isPrerelease = getAFXApp().getBasePrerelease()

        if (minor == 5) and (update < 6):
            self.rightClick = FALSE
        if (minor == 5) and isPrerelease:
            self.rightClick = FALSE
        if (minor == 6) and isPrerelease and (update < 6):
            self.rightClick = FALSE

        # Map functions
        #
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CCF, ComputeContributionsDB.onCmdComputeRB)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CMCF, ComputeContributionsDB.onCmdComputeRB)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_COMPONENT_ODB, ComputeContributionsDB.onCmdComponentOdb)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_TABLE, ComputeContributionsDB.onCmdTableChecked)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_RIGHT_CLICK, ComputeContributionsDB.onCmdSelectOdb)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_ADD, ComputeContributionsDB.onCmdAdd)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_SELECT_ALL, ComputeContributionsDB.onCmdSelectAll)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_REMOVE, ComputeContributionsDB.onCmdRemove)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CLEAR_ALL, ComputeContributionsDB.onCmdClearAll)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_OUTPUT_FILE, ComputeContributionsDB.onCmdOutfile)
        
        #GUILOG
        getAFXAliasMap().setPrefix(self, 'ComputeContributionsDB')
        #ENDGUILOG

        self.appendActionButton('Write to File', self, self.ID_CLICKED_APPLY)

        #GUILOG
        w = self.getActionButton(self.ID_CLICKED_APPLY)                
        getAFXAliasMap().setName(w, w.getText())
        #ENDGUILOG

        dismissBtn = self.appendActionButton(self.DISMISS)
        
        #GUILOG
        getAFXAliasMap().setName(dismissBtn, 'Dismiss')
        #ENDGUILOG
        
        # CCF and CMCF Group Box
        #       
        hf = FXHorizontalFrame(self,LAYOUT_FILL_X)
        FXLabel(hf, 'Type:', None)
        self.ccfRB = FXRadioButton(hf, 'CCF', self, self.ID_CCF)
        self.cmcfRB = FXRadioButton(hf, 'CMCF', self, self.ID_CMCF)
        self.ccfRB.setCheck(TRUE)
        self.form.computeTgt.setValue(CCF.getId())

        #GUILOG
        getAFXAliasMap().setName(self.ccfRB, 'CCF RB')
        getAFXAliasMap().setName(self.cmcfRB, 'CMCF RB')
        #ENDGUILOG

        # Create the table for components and ODBs
        #
        from kernelAccess import session

        componentNames = session.customData.componentNames
        self.numRows = len(componentNames)

        vf = FXVerticalFrame(self, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)

        table = AFXTable(vf, self.numRows+1, 3, self.numRows+1, 3, self, self.ID_TABLE, 
                         opts=LAYOUT_FILL_X|AFXTABLE_SINGLE_SELECT)

        table.setLeadingRows(1)
        if self.rightClick:
            table.setLeadingRowLabels('\tComponents\tComponent ODBs (Right-click to select)')
        else:
            table.setLeadingRowLabels('\tComponents\tComponent ODBs (Type in the full path to ODB)')
        
        # First column
        #
        table.setColumnType(0, table.BOOL)
        table.setColumnWidth(0, 25)
        table.setColumnJustify(0, AFXTable.CENTER)
        table.setColumnEditable(0, TRUE)

        # Second column
        #
        i = 1
        for cName in componentNames:
            table.setItemText(i, 1, cName)
            i += 1

        # Third column
        # Initially set the width to 0 for the third column for CCF
        #
        table.setStretchableColumn(1)
        table.setColumnWidth(2,0)

        if self.rightClick:
            table.appendClientPopupItem('Select...', None, self, self.ID_RIGHT_CLICK)
            
        
        self.table = table
        self.bgColor = self.getBackColor()
        self.cellColor = self.table.getBackColor()

        # Set the color of column 2 back to grey
        #
        for row in range(1, self.numRows+1):
            self.table.setItemBackColor(row, 2, self.bgColor)

        #GUILOG
        getAFXAliasMap().setName(self.table, 'Components Table')
        #ENDGUILOG

        # Unstable Modes
        #        
        mx = FXMatrix(self,1,MATRIX_BY_ROWS|LAYOUT_FILL_X,0,0,0,0,0,0,0,0)        
        gb = FXGroupBox(mx,'Unstable Modes', FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW)
        hf1 = FXHorizontalFrame(gb, LAYOUT_FILL_X|FRAME_SUNKEN|LAYOUT_FILL_Y|FRAME_THICK, 0,0,0,0, 0,0,0,0)     

        self.unstableModes = session.customData.unstableModes
        numModes = len(self.unstableModes)

        if numModes > 10:
            numModes = 10
        self.unstableList = AFXList(hf1, numModes, self.form.umSelKw, 0, 
                                    LAYOUT_FILL_X|LAYOUT_FILL_Y|AFXLIST_NO_AUTOCOMMIT)

        for um in self.unstableModes:
            self.unstableList.appendItem(str(um))        
        
        #GUILOG
        getAFXAliasMap().setName(self.unstableList, 'Unstable Modes')
        #ENDGUILOG

        # Add remove buttons
        #
        vf = FXVerticalFrame(mx,LAYOUT_CENTER_Y|LAYOUT_FILL_ROW)
        va = AFXVerticalAligner(vf)

        w = FXButton(va,'>\tAdd Selected',None,self,self.ID_ADD)
        
        #GUILOG
        getAFXAliasMap().setName(w, '>')
        #ENDGUILOG        

        w = FXButton(va,'>>\tAdd All',None,self,self.ID_SELECT_ALL)

        #GUILOG
        getAFXAliasMap().setName(w, '>>')
        #ENDGUILOG               
        
        w = FXButton(va,'<\tRemove Selected',None,self,self.ID_REMOVE)

        #GUILOG
        getAFXAliasMap().setName(w, '<')
        #ENDGUILOG        

        w = FXButton(va,'<<\tRemove All',None,self,self.ID_CLEAR_ALL)

        #GUILOG
        getAFXAliasMap().setName(w, '<<')
        #ENDGUILOG

        # Selection List
        #        
        gb = FXGroupBox(mx,'Selection', FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW)
        hf1 = FXHorizontalFrame(gb, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, 0,0,0,0)

        self.selectionList = AFXList(hf1, numModes,None,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|AFXLIST_NO_AUTOCOMMIT,0,0,0,0)

        #GUILOG
        getAFXAliasMap().setName(self.selectionList, 'SelectionList')
        #ENDGUILOG   

        # Output file name field        
        #        
        hf = FXHorizontalFrame(self,LAYOUT_FILL_X|LAYOUT_FILL_Y)
        w = AFXTextField(hf,28,'Results file name:',form.outputFileNameTgt,0,LAYOUT_FILL_X)

        #GUILOG
        getAFXAliasMap().setName(w, 'Results file name')
        #ENDGUILOG
        
        self.outfileDb = None        
        w = FXButton(hf,'Select...',None,self,self.ID_OUTPUT_FILE)
        
        #GUILOG
        getAFXAliasMap().setName(w, 'Select')
        #ENDGUILOG
        
        self.odbDb = None
               
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def onCmdOutfile(self, sender, sel, ptr):

        # Post select output file DB
        # 
        if not self.outfileDb:
            patterns = 'Component contributions (*.ccf)'
            self.outfileDb = AFXFileSelectorDialog(self, 'Select Output File', 
                self.form.outputFileNameTgt, None, 
                AFXSELECTFILE_ANY|AFXSELECTFILE_REMOTE_HOST, patterns)
            
            self.outfileDb.create()
            
        self.outfileDb.showModal()
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def onCmdAdd(self,sender,sel,ptr):
        
        numOfItems = self.form.umSelKw.getLength()
        for i in range(numOfItems):
            um = self.form.umSelKw.getValue(i)
            if self.selectionList.findItem(um) == -1:
                self.selectionList.appendItem(um)
        self.form.umSelKw.setValues('')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def onCmdComputeRB(self,sender,sel,ptr):
        tempId = SELID(sel)

        if tempId == self.ID_CCF:
            self.ccfRB.setCheck(TRUE)
            self.cmcfRB.setCheck(FALSE)
            self.form.computeTgt.setValue(CCF.getId())
            self.table.setColumnEditable(2, FALSE)  
            for row in range(1, self.numRows+1):
                self.table.setItemBackColor(row, 2, self.bgColor)
            
            self.table.setStretchableColumn(1)
            self.table.setColumnWidth(2,0)

            
        elif tempId == self.ID_CMCF:
            self.cmcfRB.setCheck(TRUE)
            self.ccfRB.setCheck(FALSE)
            self.form.computeTgt.setValue(CMCF.getId())
            self.table.setColumnEditable(2, TRUE)
            self.table.setStretchableColumn(2)
            self.table.setColumnWidth(2,200)
            self.table.setColumnWidth(1,100)

            for row in range(1, self.numRows+1):
                if self.table.getItemBoolValue(row, 0):
                    self.table.setItemEditable(row, 2, TRUE)
                    self.table.setItemBackColor(row, 2, self.cellColor)
                else:
                    self.table.setItemEditable(row, 2, FALSE) 
                    self.table.setItemBackColor(row, 2, self.bgColor)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def onCmdTableChecked(self,sender,sel,ptr):

        # Modify the item color depending on the selections.
        #
        if self.form.computeTgt.getValue() == CCF.getId():
            return 1
            
        col = self.table.getCurrentColumn()
        if col != 0:
            return 1
                
        for row in range(1, self.numRows+1):
            if self.table.getItemBoolValue(row, 0):
                self.table.setItemEditable(row, 2, TRUE)
                self.table.setItemBackColor(row, 2, self.cellColor)
            else:
                self.table.setItemEditable(row, 2, FALSE)
                self.table.setItemBackColor(row, 2, self.bgColor)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def onCmdRemove(self,sender,sel,ptr):
        
        numOfItems = self.selectionList.getNumItems()
        index = 0
        for i in range(numOfItems):             
            if self.selectionList.isItemSelected(index):
                self.selectionList.removeItem(index)
            else:
                index += 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def onCmdSelectAll(self,semder,sel,ptr):        
        self.selectionList.clearItems()
        for um in self.unstableModes:
            self.selectionList.appendItem(str(um))   
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def onCmdClearAll(self,sender,sel,ptr):
        self.selectionList.clearItems() 
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def onCmdSelectOdb(self, sender, sel, ptr):

        # Post select Odb DB
        #
        self.form.componentOdbPathTgt.setValue('')       
        if self.form.computeTgt.getValue() == CCF.getId():
            return 1
            
        col = self.table.getCurrentColumn()
        if col != 2:
            return 1       
        row = self.table.getCurrentRow()
        if row <=0:
            return 1
            
        if not self.table.getItemBoolValue(row, 0):
            return 1
            
        if not self.odbDb:
            patterns = 'Output database (*.odb)'
            self.odbDb = AFXFileDialog('Select Database', 
                self.form.componentOdbPathTgt, None,self,self.ID_COMPONENT_ODB, 
                AFXSELECTFILE_ANY, patterns)            
            self.odbDb.create()            
        
        self.odbDb.setTarget(self)    
        self.odbDb.showModal()        
        return 1
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    def onCmdComponentOdb(self, sender, sel, ptr):

        self.odbDb.setTarget(None)
        if not self.form.componentOdbPathTgt.getValue():
            return 1
        
        odbPath = self.form.componentOdbPathTgt.getValue()
        row = self.table.getCurrentRow()
        self.table.setItemText(row, 2, odbPath)              

        return 1
