from abaqusGui import *
from abaqusConstants import *
from abq_BrakeNoise.computeContributionsConstants import *

###########################################################################
# Class definition
###########################################################################

class PlotContributionsDB(AFXDataDialog):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'Plot Contributions', opts=DECOR_RESIZE)

        self.form = form

        #GUILOG
        getAFXAliasMap().setPrefix(self, 'PlotContributionsDB')
        #ENDGUILOG
        
        plotBtn = self.appendActionButton('Plot', self, self.ID_CLICKED_APPLY)
        #GUILOG
        getAFXAliasMap().setName(plotBtn, 'plot Button')
        #ENDGUILOG 
        dismissBtn = self.appendActionButton(self.DISMISS)
        #GUILOG
        getAFXAliasMap().setName(dismissBtn, 'dismiss Button')
        #ENDGUILOG 
        
        
        # Add the ODB and the results file labels
        #
        FXLabel(self, 'Complex frequency ODB: %s' % form.complexFrequencyODBName)
        FXLabel(self, 'Results file name: %s' % form.resultsFileNameTgt.getValue())
                
        # CCF and CMCF Radio Buttons
        #
        hf = FXHorizontalFrame(self, LAYOUT_FILL_X,  0,0,0,0, 0,0,0,0)
        FXLabel(hf, 'Quantity to plot:')
        w = FXRadioButton(hf, 'CCF', form.computeTgt, CCF.getId())
        
        #GUILOG
        getAFXAliasMap().setName(w, 'CCF RB')
        #ENDGUILOG 
        
        w = FXRadioButton(hf, 'CMCF', form.computeTgt, CMCF.getId())
        
        #GUILOG
        getAFXAliasMap().setName(w, 'CMCF RB')
        #ENDGUILOG 

        switcher = FXSwitcher(self, LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0)  

        #GUILOG
        getAFXAliasMap().setName(switcher, 'Switcher')
        #ENDGUILOG 
        
        # CCF Switcher
        #
        hf = FXHorizontalFrame(switcher, LAYOUT_FILL_X|LAYOUT_FILL_Y)

        # Create the table for components 
        # 
        numCCFComponents = len(form.componentNamesForCCF)
        gb = FXGroupBox(hf,'Components', FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)              
        
        vf = FXVerticalFrame(gb, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0)
        table = AFXTable(vf, NUM_ROWS, 2, numCCFComponents, 2, None, 0, 
                         opts=LAYOUT_FILL_X|AFXTABLE_SINGLE_SELECT|LAYOUT_FILL_Y)

        #GUILOG
        getAFXAliasMap().setName(table, 'CCF Component Table')
        #ENDGUILOG 

        table.setColumnType(0, table.BOOL)
        table.setColumnWidth(0, 25)
        table.setColumnJustify(0, AFXTable.CENTER)
        table.setColumnEditable(0, TRUE)
        
        i = 0
        for cName in form.componentNamesForCCF:
            table.setItemText(i, 1, cName)
            i += 1

        self.table = table

        # Unstable Modes
        #                   
        gb = FXGroupBox(hf,'Unstable Modes', FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        hf1 = FXHorizontalFrame(gb, LAYOUT_FILL_X|FRAME_SUNKEN|LAYOUT_FILL_Y|FRAME_THICK, 0,0,0,0, 0,0,0,0)     
        
        unstableList = AFXList(hf1, NUM_ROWS, self.form.unstableModeTgt, 0, 
                                    LAYOUT_FILL_X|LAYOUT_FILL_Y|AFXLIST_NO_AUTOCOMMIT|LIST_BROWSESELECT)
        for um in form.unstableModes:
            unstableList.appendItem(str(um))
            
        # Select the first unstable mode
        #
        self.form.unstableModeTgt.setValue(str(form.unstableModes[0]))
        
        #GUILOG
        getAFXAliasMap().setName(unstableList, 'Unstable Modes CCF')
        #ENDGUILOG 
        
        # CMCF Switcher
        #
        vf = FXVerticalFrame(switcher, LAYOUT_FILL_X|LAYOUT_FILL_Y)        
        
        if len(form.componentNamesForCMCF) == 0:
            # Add a note saying there are no results for CMCF
            #
            AFXNote(vf, 'There are no components with CMCF \nresults in the results file.', LAYOUT_CENTER_X|LAYOUT_CENTER_Y)
            
        else:
            # Add components and the unstable modes
            #        
            self.maxNumModesSpinner = AFXSpinner(vf,3,'Maximum number of modes:',             
                                                 form.maxNumModesKw, 0, opts=LAYOUT_FILL_Y)
            self.maxNumModesSpinner.setRange(1, 10000)
                                                 
            #GUILOG
            getAFXAliasMap().setName(self.maxNumModesSpinner, 'Max Num Modes')
            #ENDGUILOG 

            hf = FXHorizontalFrame(vf, LAYOUT_FILL_X|LAYOUT_FILL_Y)

            # Create the list for components 
            #        
            gb = FXGroupBox(hf,'Components', FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
            hf1 = FXHorizontalFrame(gb, LAYOUT_FILL_X|FRAME_SUNKEN|LAYOUT_FILL_Y|FRAME_THICK, 0,0,0,0, 0,0,0,0)     
            self.componentList = AFXList(hf1, NUM_ROWS, self.form.componentNameKw, 0, 
                                        LAYOUT_FILL_X|LAYOUT_FILL_Y|AFXLIST_NO_AUTOCOMMIT|LIST_BROWSESELECT)

            for cn in form.componentNamesForCMCF:
                self.componentList.appendItem(cn)

            # Select first componentName
            #
            self.form.componentNameKw.setValue(form.componentNamesForCMCF[0])
            
            #GUILOG
            getAFXAliasMap().setName(self.componentList, 'CMCF Component List')
            #ENDGUILOG 
            
            # Unstable Modes
            #  
            gb = FXGroupBox(hf,'Unstable Modes', FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
            hf1 = FXHorizontalFrame(gb, LAYOUT_FILL_X|FRAME_SUNKEN|LAYOUT_FILL_Y|FRAME_THICK, 0,0,0,0, 0,0,0,0)     

            unstableList = AFXList(hf1, NUM_ROWS, self.form.unstableModeTgt, 0, 
                                    LAYOUT_FILL_X|LAYOUT_FILL_Y|AFXLIST_NO_AUTOCOMMIT|LIST_BROWSESELECT)
            for um in form.unstableModes:
                unstableList.appendItem(str(um))

            #GUILOG
            getAFXAliasMap().setName(unstableList, 'Unstable Modes CMCF')
            #ENDGUILOG

        # Add the transitions
        #       
        self.addTransition(form.computeTgt,  AFXTransition.EQ, CCF.getId(), 
                            switcher,  MKUINT(FXSwitcher.ID_OPEN_FIRST+0,  SEL_COMMAND),  None)
        self.addTransition(form.computeTgt,  AFXTransition.EQ,  CMCF.getId(), 
                            switcher,  MKUINT(FXSwitcher.ID_OPEN_FIRST+1,  SEL_COMMAND),  None)
