from abaqusGui import *
from abaqusConstants import *
from abq_BrakeNoise.computeContributionsConstants import *
from cPickle import Unpickler
import types

###########################################################################
# Class definition
###########################################################################

class PlotContributionsForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        # Construct the base class.
        #
        AFXForm.__init__(self, owner)
                  
        # Command to plotCcf
        #
        self.ccfCmd = AFXGuiCommand(self, 'plotCcf', 'abq_BrakeNoise.plotContributionsModule')        
        self.ccfResultsFileNameKw = AFXStringKeyword(self.ccfCmd, 'resultsFileName', TRUE)
        self.ccfUnstableModeKw = AFXIntKeyword(self.ccfCmd, 'unstableMode', TRUE)
        self.componentNamesKw = AFXTupleKeyword(self.ccfCmd, 'componentNames', TRUE)
        
        # Command to plotCmcf
        #
        self.cmcfCmd = AFXGuiCommand(self, 'plotCmcf', 'abq_BrakeNoise.plotContributionsModule')        
        self.cmcfResultsFileNameKw = AFXStringKeyword(self.cmcfCmd, 'resultsFileName', TRUE)
        self.cmcfUnstableModeKw = AFXIntKeyword(self.cmcfCmd, 'unstableMode', TRUE)
        self.componentNameKw = AFXStringKeyword(self.cmcfCmd, 'componentName', TRUE)
        self.maxNumModesKw = AFXIntKeyword(self.cmcfCmd, 'maxNumModes', TRUE, 10)
        
        # Temporary targets
        #
        self.computeTgt = AFXIntTarget(CCF.getId())    
        self.resultsFileNameTgt = AFXStringTarget()        
        self.unstableModeTgt = AFXStringTarget()  
        
        # Data structure to store actual data
        #
        self.componentNames = None
        self.unstableModes = None
        self.isCcfDataAvailable = FALSE
        self.isCmcfDataAvailable = FALSE
        self.maxLimit = 200

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        # Show the openOdb dialog
        #
        from abq_BrakeNoise.openResultsDB import OpenResultsDB
        self.firstDb = OpenResultsDB(self)
        return self.firstDb

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getNextDialog(self,previousDb):
        # Show the plot contributions dialog
        #
        if previousDb == self.firstDb:
            from abq_BrakeNoise.plotContributionsDB import PlotContributionsDB
            self.secondDb = PlotContributionsDB(self)
            return self.secondDb
        else:
            return None      
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyCurrentKeywordValues(self):        

        # Method to check if the results file is valid.
        #
        currentDb = self.getCurrentDialog()
        
        if currentDb == self.firstDb:
            if self.resultsFileNameTgt.getValue().strip() == '':
                showAFXErrorDialog(currentDb,'Results file name must be specified.')
                return FALSE
            
        # Collect all the information from the results file
        #
        status = self.gatherData()
        
        return status
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        currentDb = self.getCurrentDialog()
                
        db = self.secondDb
        table = db.table
        rows = table.getNumRows()
        cols = table.getNumColumns()
        
        resultsFileName = self.resultsFileNameTgt.getValue()
        unstableMode = int(self.unstableModeTgt.getValue())
        
        if self.computeTgt.getValue() == CCF.getId():
            # Set ccf as the command and deactivate the other command
            #           
            self.ccfCmd.activate()
            self.cmcfCmd.deactivate()
            self.ccfResultsFileNameKw.setValue(resultsFileName)
            self.ccfUnstableModeKw.setValue(unstableMode)
            self.componentNamesKw.setValues('')            
            
        else:            
            # Set cmcf as the command and deactivate the other command
            #
            self.ccfCmd.deactivate()
            self.cmcfCmd.activate()
            self.cmcfResultsFileNameKw.setValue(resultsFileName)
            self.cmcfUnstableModeKw.setValue(unstableMode)
        
        if self.computeTgt.getValue() == CCF.getId():
            selectedCNames = []
            for row in range(0,rows):
                isSelected = table.getItemBoolValue(row, 0)
                if isSelected:
                    cName = table.getItemText(row,1)
                    selectedCNames.append(cName)

            if not selectedCNames:
                errMsg = 'At least one component must be selected.'
                showAFXErrorDialog(currentDb, errMsg)
                return FALSE        
            index = 0
            for cName in selectedCNames:
                self.componentNamesKw.setValue(index,cName)
                self.componentNamesKw.setElementType(index,AFXTUPLE_TYPE_STRING)
                index += 1   
        else:
            if len(self.componentNamesForCMCF) == 0:
                errMsg = 'There are no components with CMCF results in the results file.'
                showAFXErrorDialog(currentDb, errMsg)
                return FALSE
        
        return TRUE

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
    def gatherData(self):

        # Read the results file and store information
        #
        currentDb = self.getCurrentDialog()
        resultsFileName = self.resultsFileNameTgt.getValue()  
        if not os.path.isfile(resultsFileName):
            showAFXErrorDialog(currentDb,'The file "%s" does not exist.' % resultsFileName)
            return FALSE
        
        resultsFile = open(resultsFileName)
        try:
            pfile = Unpickler(resultsFile)
            pickleDict = pfile.load()
        except:
            showAFXErrorDialog(currentDb,'"%s" is an invalid results file.' % resultsFileName)
            return FALSE

        # Collect CCF and CMCF data
        #
        try:
            self.complexFrequencyODBName = pickleDict['complexFrequencyODBName']
            if type(self.complexFrequencyODBName) != types.StringType:
                raise TypeError, '1'
            self.unstableModesDict = pickleDict['unstableModesDict'] 
            if type(self.unstableModesDict) != types.DictType:
                raise TypeError, '2'
            self.componentNamesForCCF = pickleDict['componentNamesForCCF']
            if type(self.componentNamesForCCF) != types.ListType and type(self.componentNamesForCCF) != types.TupleType: 
                raise TypeError, '3'
            self.ccfDict = pickleDict['ccfDict']
            if type(self.ccfDict) != types.DictType:
                raise TypeError , '4'       
            self.componentNamesForCMCF = pickleDict['componentNamesForCMCF']
            if type(self.componentNamesForCMCF) != types.ListType and type(self.componentNamesForCMCF) != types.TupleType: 
                raise TypeError, '5'
            self.cmcfDict = pickleDict['cmcfDict']
            if type(self.cmcfDict) != types.DictType:
                raise TypeError ,'6'       
             
        except:
            import sys
            msg = '%s: %s\n' % (sys.exc_info()[0], sys.exc_info()[1])
            msg += 'The results file is not in a valid format.'
            msg += '\nPlease check if the file was created using the compute component contributions plug-in.'
            showAFXErrorDialog(currentDb,msg)
            return FALSE
        
        self.unstableModes = self.unstableModesDict.keys()
        self.unstableModes.sort()
        
        return TRUE

