from abaqusGui import *
from abaqusTester import *
from kernelAccess import mdb, session
from abq_Hcore.hcoreConstants import *
import i18n

###########################################################################
# Class definition
###########################################################################

class HcoreDB(AFXDataDialog):
    
    [
       ID_MODEL,
       ID_MAT,
       ID_GLUE,
       ID_GLUE,
    ] = range(AFXDialog.ID_LAST, AFXDialog.ID_LAST+4)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        AFXDataDialog.__init__(self, form, 'Create Honeycomb',
            self.OK|self.CANCEL, DIALOG_ACTIONS_SEPARATOR)

        # Sets the  prefix key - usually the dialog box class name- of a widget
        
        #GUILOG
        getAFXAliasMap().setPrefix(self, 'HcoreDB') 
        #ENDGUILOG
        
        # Mapping the Flag-ID from FXButton to a command method
        self.form = form
        FXMAPFUNC(self, SEL_COMMAND, self.ID_MODEL, HcoreDB.onModelsChanged)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_MAT, HcoreDB.onMaterialsChanged)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_GLUE, HcoreDB.onGlueEffects)
                
    # ----------------Widgets to be used by the user-----------------------------------
    
    # Model combo
        # Since all forms will be canceled if the  model changes,
        # we do not need to register a query on the model.
        #
        va = AFXVerticalAligner(self, LAYOUT_SIDE_LEFT, 0,0,0,0, 0,0,0,0)

        self.modelcomboBox = AFXComboBox(va, 0, 1, 'Model name:', form.modelNamekw, 0)
        self.form.modelNamekw.setTarget(self)
        self.form.modelNamekw.setSelector(self.ID_MODEL)
        self.modelcomboBox.setMaxVisible(10)
        
        #GUILOG
        getAFXAliasMap().setName(self.modelcomboBox, 'Model name:')
        #ENDGUILOG

        self.currentModelName = getCurrentContext()['modelName']
        names = mdb.models.keys()
        names.sort()
        
        #self.onDummy1()

        for name in names:
            self.modelcomboBox.appendItem(name)
            if not form.modelNamekw.getValue() in names:
                form.modelNamekw.setValue(self.currentModelName)    

        matfunc = 'Material->Create'
        matfunc_target = getAFXApp().getAFXMainWindow().getTargetFromFunction(i18n.tr('Material')+'->'+i18n.tr('Create'))
        matfunc_selector = getAFXApp().getAFXMainWindow().getSelectorFromFunction(i18n.tr('Material')+'->'+i18n.tr('Create'))  
        # Icon
        
        hf = FXHorizontalFrame(va,LAYOUT_FILL_X,0,0,0,0,0,0,0,0)
        
        # Material comboBox:

        self.matcomboBox = AFXComboBox(hf, 1, 1, 'Material name:', form.materialNamekw, 0)
        self.form.materialNamekw.setSelector(self.ID_MAT)
        
        self.matcomboBox.setMaxVisible(10)
        
        #GUILOG
        getAFXAliasMap().setName(self.matcomboBox, 'Material name:')
        #ENDGUILOG
        
        thisPath = os.path.abspath(__file__)
        thisDir = os.path.dirname(thisPath)
        #print "os.path.join(thisDir, 'materialAdd.png')==",os.path.join(thisDir, 'materialAdd.png')
        #icon_mat = afxCreatePNGIcon(os.path.join(thisDir, 'materialAdd.png'))
        
        #print "icon_mat==",icon_mat
        #icon_mat.scale(24,24)
        matbutton = FXButton(hf, '', ic=None , tgt=matfunc_target, sel=matfunc_selector)
        
        #GUILOG
        getAFXAliasMap().setName(matbutton, 'Create Material')
        #ENDGUILOG
        
        self.gluecheck = FXCheckButton(va, 'Adhesive', self, self.ID_GLUE)
        self.form.gluekw.setValue(False)
        
        #GUILOG
        getAFXAliasMap().setName(self.gluecheck, 'Adhesive')
        #ENDGUILOG

        # ------------------------ Tab book ---------------------------
        TabBook_1 = FXTabBook(va, None, 0,TABBOOK_NORMAL|LAYOUT_FILL_X,
            0, 0, 0, 0, DEFAULT_SPACING, DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING)
        

        # ------------First tab - Geometric Parameters ------------------
        tabItem1 = FXTabItem(TabBook_1, 'Cell Geometry', None, TAB_TOP_NORMAL,
            0, 0, 0, 0, 0, 0, DEFAULT_PAD, DEFAULT_PAD)
            
        #GUILOG
        getAFXAliasMap().setName(tabItem1, 'Cell Geometry')
        #ENDGUILOG
            
        TabItem_1 = FXVerticalFrame(TabBook_1,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X,
            0, 0, 0, 0, DEFAULT_SPACING, DEFAULT_SPACING,DEFAULT_SPACING, DEFAULT_SPACING,
            DEFAULT_SPACING, DEFAULT_SPACING)

        gb = FXGroupBox(TabItem_1, '', FRAME_GROOVE|LAYOUT_FILL_X)
        va = AFXVerticalAligner(gb, LAYOUT_SIDE_LEFT, 0,0,0,0, 0,0,0,0)
        self.lengtha_text = AFXTextField(va, 8, 'Base length', form.length_akw, 0) 

        #GUILOG
        getAFXAliasMap().setName(self.lengtha_text, 'Base length')
        #ENDGUILOG
        
        self.lengthb_text = AFXTextField(va, 8, 'Inclined length', form.length_bkw, 0)
        
        #GUILOG
        getAFXAliasMap().setName(self.lengthb_text, 'Inclined length')
        #ENDGUILOG
        
        self.angle_text = AFXTextField(va, 8, 'Angle of inclination', form.angle_alveolekw, 0)
        
        #GUILOG
        getAFXAliasMap().setName(self.angle_text, 'Angle of inclination')
        #ENDGUILOG

        va = AFXVerticalAligner(gb, LAYOUT_SIDE_LEFT, 0,0,0,0, 0,0,0,0)
        self.heigth_text = AFXTextField(va, 8, 'Height', form.heightkw, 0)
        
        #GUILOG
        getAFXAliasMap().setName(self.heigth_text, 'Height')
        #ENDGUILOG
        
        self.thickness_text = AFXTextField(va, 8, 'Wall thickness(t)', form.thicknesskw, 0)
        
        
        #GUILOG
        getAFXAliasMap().setName(self.thickness_text, 'Wall thickness(t)')
        #ENDGUILOG
        
        
        # ------------Second tab - Pattern Parameters -------------------
        tabItem2 = FXTabItem(TabBook_1, 'Honeycomb Dimensions', None, TAB_TOP_NORMAL,
            0, 0, 0, 0, 0, 0, DEFAULT_PAD, DEFAULT_PAD)
            
        #GUILOG
        getAFXAliasMap().setName(tabItem2, 'Honeycomb Dimensions')
        #ENDGUILOG
            
        TabItem_2 = FXVerticalFrame(TabBook_1,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X,
            0, 0, 0, 0, DEFAULT_SPACING, DEFAULT_SPACING,DEFAULT_SPACING, DEFAULT_SPACING,
            DEFAULT_SPACING, DEFAULT_SPACING)
        
        hf = FXHorizontalFrame(TabItem_2, LAYOUT_FILL_X,0,0,0,0,0,0,0,0)
        
        FXLabel(hf, '',None, LAYOUT_SIDE_LEFT)
        lp_rb = FXRadioButton(hf, 'Number of Cells',form.pattern_rbkw, ID_LP.getId()) 
        
        #GUILOG
        getAFXAliasMap().setName(lp_rb, 'Number of Cells')
        #ENDGUILOG
        
        lp_rb = FXRadioButton(hf, 'In-plane Dimensions',form.pattern_rbkw, ID_AREA.getId()) 
        
        
        #GUILOG
        getAFXAliasMap().setName(lp_rb, 'In-plane Dimensions')
        #ENDGUILOG
                    
        gb = FXGroupBox(TabItem_2, '', FRAME_GROOVE)
        self.sw = FXSwitcher(gb,LAYOUT_FILL_Y|LAYOUT_FILL_X,0,0,0,0, 0,0,0,0) 
        
        va1 = AFXVerticalAligner(self.sw, LAYOUT_SIDE_LEFT, 0,0,0,0, 0,0,0,0)
        
        self.xdir_text = AFXTextField(va1, 8, 'X-direction', form.Xdirkw, 0)
        
        #GUILOG
        getAFXAliasMap().setName(self.xdir_text, 'X-direction')
        #ENDGUILOG
        
        self.ydir_text = AFXTextField(va1, 8, 'Y-direction', form.Ydirkw, 0)
        
        #GUILOG
        getAFXAliasMap().setName(self.ydir_text, 'Y-direction')
        #ENDGUILOG

        va2 = AFXVerticalAligner(self.sw, LAYOUT_SIDE_LEFT, 0,0,0,0, 0,0,0,0)
        
        self.length_text = AFXTextField(va2, 8, 'Ribbon or "L" dimension', form.Xdirkw, 0)
        
        
        #GUILOG
        getAFXAliasMap().setName(self.length_text, 'Ribbon or "L" dimension')
        #ENDGUILOG
        
        self.width_text = AFXTextField(va2, 8, 'Expansion or "W" dimension', form.Ydirkw, 0)
        
        #GUILOG
        getAFXAliasMap().setName(self.width_text, 'Expansion or "W" dimension')
        #ENDGUILOG
        
        # ------------Third tab - Geometric Parameters of glue (SHOWS ONLY WHEN gluecheck IS true)-------------------
        self.tabItem3 = FXTabItem(TabBook_1, 'Adhesive Properties', None, TAB_TOP_NORMAL,
            0, 0, 0, 0, 0, 0, DEFAULT_PAD, DEFAULT_PAD)
            
        #GUILOG
        getAFXAliasMap().setName(self.tabItem3, 'Adhesive Properties')
        #ENDGUILOG
            
        TabItem_3 = FXVerticalFrame(TabBook_1,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X,
            0, 0, 0, 0, DEFAULT_SPACING, DEFAULT_SPACING,DEFAULT_SPACING, DEFAULT_SPACING,
            DEFAULT_SPACING, DEFAULT_SPACING)
            
        gb = FXGroupBox(TabItem_3, '', FRAME_GROOVE|LAYOUT_FILL_X)
        
        va = AFXVerticalAligner(gb, LAYOUT_SIDE_LEFT, 0,0,0,0, 0,0,0,0)
        
        hf = FXHorizontalFrame(va,0, 0, 0, 0, 0)
        
         # Material comboBox:

        self.gluematcomboBox = AFXComboBox(hf, 0, 1, 'Material name', form.gluematerialNamekw, 0)
        self.gluematcomboBox.setMaxVisible(10)
        
        # Cannot have the same alias as Hcore material name
	#GUILOG
        getAFXAliasMap().setName(self.gluematcomboBox, 'Glue Material name')
        #ENDGUILOG
        
#       self.onGlueMaterialsChanged()
        
        gluematbutton = FXButton(hf, '', ic=None , tgt=matfunc_target, sel=matfunc_selector)
        
        #GUILOG
        getAFXAliasMap().setName(gluematbutton, 'Create Glue Material')
        #ENDGUILOG
        
        self.glueheight_text = AFXTextField(va, 8, 'Height', form.glueheightkw, 0)
        
        #GUILOG
        getAFXAliasMap().setName(self.glueheight_text, 'Glue Height')
        #ENDGUILOG
        
        self.gluethickness_singlewall = AFXTextField(va, 8, 'Thickness', form.gluethicknesskw, 0)
        
        #GUILOG
        getAFXAliasMap().setName(self.gluethickness_singlewall, 'Glue Thickness')
        #ENDGUILOG
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def processUpdates(self):
        
        if self.form.pattern_rbkw.getValue() == ID_LP.getId():
            self.sw.setCurrent(0)
            
        else:
            self.sw.setCurrent(1)
            
            
    def onDummy(self):
        
#        self.onModelsChanged(None, None, None)
        self.onMaterialsChanged(None, None, None)
     
    def onDummy1(self) :
        
        self.onModelsChanged(None, None, None)
        
    def models(self):
        
        self.modelcomboBox.clearItems()
        names=mdb.models.keys() 
        names.sort()
        for key in names:
            self.modelcomboBox.appendItem(key)

        if self.modelcomboBox.getNumItems():
            self.modelcomboBox.setCurrentItem(0)
        
        if self.modelcomboBox.getText():
            mdb.models[self.modelcomboBox.getText()].materials.registerQuery(self.onDummy)
        
    def onModelsChanged(self,sender,sel,ptr):
        
        
        for m in mdb.models.keys():
            mdb.models[m].materials.unregisterQuery(self.onDummy)
        text = self.modelcomboBox.getText()
        
        mdb.models[text].materials.registerQuery(self.onDummy,False)
        
        #self.onDymmy()
        self.onMaterialsChanged(None,None,None)
#        if self.gluecheck.getCheck():
#            self.onGlueMaterialsChanged()
                
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onMaterialsChanged(self,sender,sel,ptr):
        
        self.matcomboBox.clearItems()
        text = self.modelcomboBox.getText()
#       print 'The models repository changed.' 

#        mdb.models[self.modelcomboBox.getText()].materials.registerQuery(self.onDummy)
        
        names=mdb.models[text].materials.keys() 
        names.sort()

        for key in names:
            self.matcomboBox.appendItem(key)
            
        if self.matcomboBox.getNumItems():
            self.matcomboBox.setCurrentItem(0)
            
        if self.gluecheck.getCheck():
            self.onGlueMaterialsChanged()    

#        mdb.models[self.modelcomboBox.getText()].materials.unregisterQuery(self.onDummy)
            
    
    def onGlueEffects(self, sender, sel, ptr):
        
        if self.gluecheck.getCheck():
            self.form.gluekw.setValue(True)
            self.onGlueMaterialsChanged()
            self.tabItem3.show()
        else:
            self.form.gluekw.setValue(False)
            self.tabItem3.hide()
            
    def onGlueMaterialsChanged(self):
        
        self.gluematcomboBox.clearItems()
        
        text = self.modelcomboBox.getText()
#       print 'The models repository changed.' 
        
        names=mdb.models[text].materials.keys() 
        names.sort()
        
        for key in names:
            self.gluematcomboBox.appendItem(key)
            
        if self.gluematcomboBox.getNumItems():
            self.gluematcomboBox.setCurrentItem(0)
        
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
    
        text = self.modelcomboBox.getText()
        mdb.models.registerQuery(self.models)
#        mdb.models[text].materials.registerQuery(self.onDummy)
#        mdb.models[text].materials.registerQuery(self.onGlueMaterialsChanged)
#        registerCurrentContext(self.onDummy1)
        self.tabItem3.hide()
        AFXDataDialog.show(self)

                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):    

        text = self.modelcomboBox.getText()
        mdb.models.unregisterQuery(self.models)
        for m in mdb.models.keys():
            mdb.models[m].materials.unregisterQuery(self.onDummy)
#        mdb.models[text].materials.unregisterQuery(self.onDummy)
#        mdb.models[text].materials.unregisterQuery(self.onGlueMaterialsChanged)
#        unregisterCurrentContext(self.onDummy1)
        AFXDataDialog.hide(self)


        

            
            
            
        
