from abaqusGui import *
from abaqusConstants import *
from abq_Hcore.hcoreDB import HcoreDB
from abq_Hcore.hcoreConstants import *

###########################################################################
# Class definition
###########################################################################

class HcoreForm(AFXForm):

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        # Construct the base class.
        #
        AFXForm.__init__(self, owner)
                
        # Command and Keywords
        # First Tab - Geometric Parameters
        self.cmd = AFXGuiCommand(self, 'Hcore', 'abq_Hcore.hcoreKernel')  
        self.modelNamekw = AFXStringKeyword(self.cmd, 'hcoremodelName', True)       
        self.length_akw = AFXFloatKeyword(self.cmd, 'length_a', TRUE)
        self.length_bkw = AFXFloatKeyword(self.cmd, 'length_b', TRUE)
        self.angle_alveolekw = AFXFloatKeyword(self.cmd, 'angle_alveole', TRUE)
        self.heightkw = AFXFloatKeyword(self.cmd, 'height', TRUE)
        self.materialNamekw = AFXStringKeyword(self.cmd, 'hcorematerialName', True)  
        self.thicknesskw = AFXFloatKeyword(self.cmd, 'thickness', TRUE)
        
        self.gluekw = AFXBoolKeyword(self.cmd, name='glue', isRequired=TRUE, defaultValue=False)
                
        self.Xdirkw = AFXIntKeyword(self.cmd, 'x_dir', TRUE)
        self.Ydirkw = AFXIntKeyword(self.cmd, 'y_dir', TRUE)
        
        self.pattern_rbkw = AFXSymConstKeyword(self.cmd, 'pattern', TRUE, ID_LP.getId())
                
        self.glueheightkw = AFXFloatKeyword(self.cmd, 'glueheight',TRUE,0)
        self.gluethicknesskw = AFXFloatKeyword(self.cmd, 'gluethickness', TRUE,0)
        self.gluematerialNamekw = AFXStringKeyword(self.cmd, 'h_gluematerialName', TRUE,'') 
	
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):
	    
	AFXForm.activate(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        db = HcoreDB(self)
        return db
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    def doCustomChecks(self):
		
        db = self.getCurrentDialog()
	
        kw_list = []
        kw_gluelist = []
        dirkw_list = []
        flag = 0
        
        kw_list.append(self.length_akw)
        kw_list.append(self.length_bkw)
        kw_list.append(self.heightkw)
        kw_list.append(self.thicknesskw)
        kw_gluelist.append(self.glueheightkw)
        kw_gluelist.append(self.gluethicknesskw)
        
        
        dirkw_list.append(self.Xdirkw)
        dirkw_list.append(self.Ydirkw)
        
        #----- deletion of objects-------
        def onReturn():
            
            del kw_list[:]
            del dirkw_list[:]
            del kw_gluelist[:]
            
            return True
        
        #------ verifying if all dimensions are >0 --------
        for x in kw_list:
        
            if x.getValue() <= 0:
                
                showAFXErrorDialog(db, 'The value must be positive')
                db.onKeywordError(x)
                onReturn()
                return False  
                
        #------ verifying if all glue dimensions are >0 (if opted)--------
        
        if self.gluekw.getValue() == True:
            
            for x in kw_gluelist:
            
                if x.getValue() <= 0:
                    
                    showAFXErrorDialog(db, 'The value must be positive')
                    db.onKeywordError(x)
                    onReturn()
                    return False  
            
        
        #------- verifying if there are atleast 2 instances to create a linear pattern
        
        if self.pattern_rbkw.getValue == ID_AREA.getId():
            
            for temp in dirkw_list:
                
                if temp.getValue() >=2:
                    flag =1
                    
            if flag!=1:
                
                for temp in dirkw_list:
                    
                    if temp.getValue() <2:
                        showAFXErrorDialog(db, 'Atleast one of the values should be >=2')
                        db.onKeywordError(temp)
                        onReturn()
                        return False  
        
        
        if (self.length_akw.getValue() == self.length_bkw.getValue()):
            

            if (self.angle_alveolekw.getValue()!= 60.0):
                
                
                showAFXErrorDialog(db, 'The angle value must be 60.0 for a regulare hexagon')
                db.onKeywordError(self.angle_alveolekw)
                return False
                
        
            
        if (self.angle_alveolekw.getValue() > 90.0 or self.angle_alveolekw.getValue() < -90.0 or self.angle_alveolekw.getValue() == 0.0):
            
            showAFXErrorDialog(db, '-pi/2 <= angle < 0 or 0 < angle <= pi/2')
            db.onKeywordError(self.angle_alveolekw)
            return False
            
        
        
        onReturn()
            
        return True
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomTasks(self):
        
        mw = getAFXApp().getAFXMainWindow()
        mw.writeToMessageArea('A part has been created')
        
     
    """
    def verifyKeywordValues(self):
        
        db = self.getCurrentDialog()
        
        if self.length_akw.getValue() < 0:
            
            showAFXErrorDialog(db, 'A value entered is negative.')
            return False    

    """
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    """
    def activate(self):
	    
	mainWindow = getAFXApp().getAFXMainWindow()
        if getDisplayedObjectType() == SKETCH:
		showAFXErrorDialog(mainWindow,'Please close the Sketch Module inorder to proceed')
		return None   
    """		
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def okToCancel(self):

        # By default when the context changes in Abaqus/CAE, all forms/Procedures are cancelled
        # Overwriting the base class
        
        return False
        

    
    



        
    