
########################################################################
#                                                                      #
#                                                                      #
#                  This module constructs the hcore.                   #
#                                                                      #
#                                                                      #
########################################################################

from abaqus import *
from abaqusConstants import *
from caeModules import *
from math import *
from abq_Hcore.hcoreConstants import *
from random import*
import warnings

# global variables 
hex_coor=[];
sidewall_coor=[];
doublewall_coor_3d=[];
X_dir = 0;
Y_dir = 0;

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def HcorePartCreation(hcoremodelName, name, depth, offset_x, offset_y):
	
    sketch_sheet = offset_x*offset_y
    hex_vertices = []
    
    s = mdb.models[hcoremodelName].ConstrainedSketch(name='__profile__', sheetSize=sketch_sheet)
    s.setPrimaryObject(option=STANDALONE)
    g,v = s.geometry, s.vertices
	   
    #--- mking a list of Lines---

    for x in range(0,len(hex_coor)):

        if not x == len(hex_coor)-1:
            
            temp = s.Line(hex_coor[x],hex_coor[x+1])
            
            # for connecting the side wall line
            if x == 1: # data structure with string names would be better for access                    
                
                leftline_sidewall = s.Line(hex_coor[x],sidewall_coor[0])
                
            if x == 4:
                 rightline_sidewall = s.Line(hex_coor[x],sidewall_coor[1])
        else:
            
            temp = s.Line(hex_coor[x],hex_coor[0])
           
    for x in range(0,len(hex_coor)):
	    
		hex_vertices.append(v.findAt(hex_coor[x],))
    
    for y in range(0,len(sidewall_coor)):
	    
		hex_vertices.append(v.findAt(sidewall_coor[y],))
	 
     # Add exception if unable to find the vertex for any reason
    # Handling Constraints-------------------------------------
    # Sanity Check:
    
     # Equal Length Constraint:
    keys = g.keys() # obtaining the line geometry keys
    
    s.EqualLengthConstraint(g[keys[0]],g[keys[1]]) # oblique base left and top left
    s.EqualLengthConstraint(g[keys[1]],g[keys[4]]) # oblique top left and top right
    s.EqualLengthConstraint(g[keys[4]],g[keys[5]]) # oblique top right and top bottom
    
    s.EqualLengthConstraint(g[keys[2]],g[keys[6]]) # side left and side right
    
    s.EqualLengthConstraint(g[keys[3]],g[keys[7]]) # flat top and bottom
    
    # Horizontal Constraints
    
    s.HorizontalConstraint(g[keys[2]]) # side left
    s.HorizontalConstraint(g[keys[3]]) # flat top
    s.HorizontalConstraint(g[keys[6]]) # side right
    s.HorizontalConstraint(g[keys[7]]) # flat bottom
    s.ParallelConstraint(g[keys[3]],g[keys[7]]) # flat top and bottom
    
    # Angular Constraints (not required):

    p = mdb.models[hcoremodelName].Part(name=name,dimensionality=THREE_D, type=DEFORMABLE_BODY)
    p = mdb.models[hcoremodelName].parts[name]
    p.BaseShellExtrude(sketch=s, depth=depth)
    
    s.unsetPrimaryObject()
    
    del hex_vertices[:]
    #del mdb.models[hcoremodelName].sketches['__profile__']

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def HcoreSectionCreation(hcoremodelName,name, material,thickness):
    
    mdb.models[hcoremodelName].HomogeneousShellSection(name=name, 
    preIntegrate=OFF, material=material, thicknessType=UNIFORM, 
    thickness=thickness, thicknessField='', idealization=NO_IDEALIZATION, 
    poissonDefinition=DEFAULT, thicknessModulus=None, temperature=GRADIENT, 
    useDensity=OFF, integrationRule=SIMPSON, numIntPts=5)
    
 #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
def HcoreAssemblyInstance(hcoremodelName, name):
    
    m = mdb.models[hcoremodelName] 
    a = m.rootAssembly
    p = m.parts[name]
    
    n = randint(0,5000) # change when there are more assemby instances
    a.Instance(name=name+str(n), part=p, dependent=ON)    

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def HcoreGlueAssembly(hcoremodelName, name, assemblyname, distance):
    
    m = mdb.models[hcoremodelName] 
    a = m.rootAssembly
    # print "assemblyname", assemblyname
    a.translate(instanceList=(assemblyname, ), vector=(0.0, 0.0, distance))

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
def HcoreAssemblyMerge(hcoremodelName,name):
    
     
     m = mdb.models[hcoremodelName] 
     a = m.rootAssembly
     keys = a.instances.keys()
     instance_limit = 100 # Safety limit CAE could still merge with 3000 instances, but takes some time
     
     temp = []
     num_instances = len(keys)
     name_num = 1
     
     # To improve the CAE performance
     if num_instances < instance_limit: 
	     for x in range(0,num_instances):
		     
			
		     temp.append(a.instances[keys[x]])
			 
	     a.InstanceFromBooleanMerge(name=name, instances = (temp), keepIntersections=ON, 
	     originalInstances=DELETE, domain=GEOMETRY) 
	     temp[:] = []
      
           
     else:
        
        num_rem = num_instances/instance_limit # (int)quotient
        num_quo = num_instances%(instance_limit) # remainder 
        
        for x in range(0,num_rem):
            
            keys = a.instances.keys()
            for y in range(0,instance_limit):
                
                temp.append(a.instances[keys[y]])
                
            a.InstanceFromBooleanMerge(name="temp"+str(name_num), instances = (temp), keepIntersections=ON, 
            originalInstances=DELETE, domain=GEOMETRY) 
            
            name_num += 1
            temp[:] = []
               
        # final merging
        	
	keys = a.instances.keys()
	num_instances = len(keys)
	
	if num_instances>1:
		
		for x in range(0,num_instances):
		     
			
		     temp.append(a.instances[keys[x]])
			
		a.InstanceFromBooleanMerge(name=name, instances = (temp), keepIntersections=ON, 
		originalInstances=DELETE, domain=GEOMETRY) 
                temp[:] = []
	else:
	     try:   
	        del a.features[a.instances.keys()[0]]
	        #There should be a part with name temp1
	        m.parts.changeKey(fromName='temp1', toName=name)
		a = m.rootAssembly
		a.regenerate()
		p = m.parts[name]
		a.Instance(name=name, part=p, dependent=ON)
	     except:
	        raise ValueError, "Unable to create the honeycomb part"
        
     # deleting parts was the cause of break in ipc connection
     
     keys = m.parts.keys()
     
     for x in (keys):
	     
	     if ((x.find("temp")) ==0):
		     del m.parts[x]
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~       
def HcorePattern(X_dir,Y_dir,pattern,hcoremodelName,name,offset_x,offset_y): 
    
        if pattern == ID_LP:
    
            pass
            
        if pattern == ID_AREA:
            
            #print "Area is Under Construction"
            X_dir = X_dir/offset_x
            X_dir = int(X_dir)
            Y_dir = Y_dir/offset_y
            Y_dir = int(Y_dir)
	   
            if (X_dir == 0 and Y_dir == 0):
                
                raise ValueError, 'Area is not large enough to fit an Hcore'
		
	#print "number of cells in x_dir = ", X_dir
	#print "number of cells in x_dir = ", Y_dir
                
        if (X_dir*Y_dir >= 1000):
            
            warnings.warn("Instances more than 1000 can slow down performance.")
            
                
        m = mdb.models[hcoremodelName] 
        a = m.rootAssembly

        keys = a.instances.keys()
        
        a.LinearInstancePattern(instanceList=(keys[0], ), direction1=(1.0, 0.0, 
        0.0), direction2=(0.0, 1.0, 0.0), number1=X_dir, number2=Y_dir, 
        spacing1=offset_x, spacing2=offset_y)
        
        # session.viewports['Viewport: 1'].setValues(displayedObject=a)

        name = name+str(len(keys))
        HcoreAssemblyMerge(hcoremodelName,name)
	
	
	return X_dir, Y_dir, name

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
def HcoreBoundarySet(X_dir,Y_dir,hcoremodelName,name,partName,offset_x,offset_y):
    
    m = mdb.models[hcoremodelName]
    #keys = m.parts.keys()
    #part_name = keys[len(keys)-1]
    p = m.parts[partName]
    
    faces = []
    x_coor = 0
    y_coor = 0
    
    for y in range(0,2):
        y_coor = (y*offset_y*(Y_dir-1))
            
        for x in range(0,X_dir):
            
            x_coor = (x*offset_x)
	    #temp = p.faces.getClosest(((doublewall_coor_3d[y+2][0]+x_coor,doublewall_coor_3d[y+2][1]+y_coor,doublewall_coor_3d[y+2][2]),))
            #faces.append(temp[0][0].index)
	    faces.append(p.faces.findAt(((doublewall_coor_3d[y+2][0]+x_coor,doublewall_coor_3d[y+2][1]+y_coor,doublewall_coor_3d[y+2][2]),)))
	    
    #faces_sequence = GetSequence(faces,p)
        
    set_sidewall = p.Set(name, faces = faces)
    
    del faces[:]
    
    a = m.rootAssembly
    a.regenerate()

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
def GetSequence(faces, parts):
	    
	    p = parts
	    temp =''
	    for i in faces:
		    temp += 'p.faces[%s:%s]+'%(i,i+1)
		    
	    return eval(temp[:-1])

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def HcoreFinalSetSectionAssignment(hcoremodelName,partName,boundarywallsetName,singlewallsetName,doublewallsetName,hcorematerialName,thickness_single,thickness_double):
    
    m = mdb.models[hcoremodelName] 
    
    
    """
    a = m.rootAssembly
    keys = a.instances.keys()
    name = keys[0]
    del m.rootAssembly.instances[name]
    """
    
    
    #keys = m.parts.keys()
    #part_name = keys[len(keys)-1]
    p = m.parts[partName]
    
    # accessing the sets
    boundarywallset = p.sets[boundarywallsetName]
    singlewallset = p.sets[singlewallsetName]
    doublewallset = p.sets[doublewallsetName]
    
    # making new sets
    #newsinglewallset = p.SetByBoolean('hcore_single',(singlewallset,boundarywallset),UNION)
    newsinglewallset = p.sets[singlewallsetName]
    newdoublewallset = p.SetByBoolean('hcore_double',(doublewallset,boundarywallset), DIFFERENCE)
    
    #------------Section Creation -----------------------------------------#
    #------------single wall thickness -----------------
    
    name='SingleWall'
    material=hcorematerialName
    thickness=thickness_single
    HcoreSectionCreation(hcoremodelName,name, material,thickness)
    
    #---------double wall thickness --------------------
    name='DoubleWall'
    material=hcorematerialName
    thickness=thickness_double
    HcoreSectionCreation(hcoremodelName,name, material,thickness)
    
    p.SectionAssignment(region=newdoublewallset, sectionName='DoubleWall', 
        offset=0.0, offsetType=MIDDLE_SURFACE)
        
    p.SectionAssignment(region=newsinglewallset, sectionName='SingleWall', 
        offset=0.0, offsetType=MIDDLE_SURFACE)
	
    p.SectionAssignment(region=boundarywallset, sectionName='SingleWall', 
        offset=0.0, offsetType=MIDDLE_SURFACE)
    
    p.deleteSets(setNames=(doublewallsetName,))
    
    a = m.rootAssembly
    a.regenerate()



########################################################################                                                                      
#  Main Function of the Module                                         #
########################################################################  
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def Hcore(length_a, length_b, angle_alveole, height, hcoremodelName,hcorematerialName, thickness, x_dir,y_dir,pattern,glue,gluethickness,glueheight,h_gluematerialName):
	
	## 1.---------------------------------------
	# Sanity Checks, Memory Delete, Model Access
	## -----------------------------------------

	# printing the user inputs for sanity check 
	#print "Length (a) = ", length_a
	#print "Length (b) = ", length_b
	#print "Angle of alveole (deg) = ", angle_alveole
	#print "Slice Thickness (Heigth) = ", height
	#print "hcoremodelName = ", hcoremodelName
	#print "hcorematerialName = ", hcorematerialName
	#print "Wall Thickness = ", thickness
	#print "glue = ", glue
	#print "gluethickness =", gluethickness
	#print "glueheight =", glueheight
	#print "h_gluematerialName =", h_gluematerialName
	
	
	# Assigning required internal variables variables
	X_dir = x_dir
	Y_dir = y_dir
        
	thickness_single = thickness
	thickness_double = thickness*2
        
	# Accessing the model
	model = mdb.models[hcoremodelName]
	
	# Deleting any previous instances in the model if exists
	keys = model.rootAssembly.features.keys()
	l = len(keys)
	if l!=0:
		for x in range(0,l):
			
		    del(model.rootAssembly.features[keys[x]])

	## 2.---------------------------------------
	# Hexagon parameter calculation based upon user inputs
	## ----------------------------------------- 

	# Refer the document for details
	# Obtaining sides of the triangle
	h = length_b*sin (radians(angle_alveole))  #converting to radians
	l = length_b*cos (radians(angle_alveole))
	
	# Area of hexagon excluding side lengths
        w = 2*h
        L = (2*l) + length_a
        
       # Area of hexagon with side lengths
        offset_y = abs(w) 
        offset_x = abs(L + length_a)
        
	#----------Use the current model --------------------------------
        
        #---------Create the hexagon sketch from user inputs -------------------------------
        
        #---clockwise arrangement starting from baseflat left
        """              2__3
                        /       \
	      6___1/         \4___7
		       \         /
angle of alveole(\0___/5
        """
        
        left_baseflat = (-length_a/2.0,0)
        left_point = (-length_a/2.0-l,h)
        left_topflat = (-length_a/2.0,w)
        right_topflat = (length_a/2.0,w)
        right_point = (length_a/2.0+l,h)
        right_baseflat = (length_a/2.0,0)
        left_sidewall = (-2*length_a/2.0-l,h)
        right_sidewall = (2*length_a/2.0+l,h)
        
        left_sidewall_3d = (-2*length_a/2.0-l,h,0)
        right_sidewall_3d = (2*length_a/2.0+l,h,0)
        baseflat_midpoint_3d = ((-length_a/2.0+length_a/2.0)/2.0,0,0)
        topflat_midpoint_3d = ((-length_a/2.0+length_a/2.0)/2.0,w,0)
                
        #---making a list of tuple hex co-or points---
        
        hex_coor.append(left_baseflat)
        hex_coor.append(left_point)
        hex_coor.append(left_topflat)
        hex_coor.append(right_topflat)
        hex_coor.append(right_point)
        hex_coor.append(right_baseflat)
        
        sidewall_coor.append(left_sidewall)
        sidewall_coor.append(right_sidewall)
        
	# a very crude way of obtaining
        doublewall_coor_3d.append(left_sidewall_3d)
        doublewall_coor_3d.append(right_sidewall_3d)
        doublewall_coor_3d.append(baseflat_midpoint_3d)
        doublewall_coor_3d.append(topflat_midpoint_3d)
        
        #------------------------hcore part creation---------------------------#
        partname = 'hcell'
        depth = height
        name = partname
        
	HcorePartCreation(hcoremodelName, name, depth,offset_x,offset_y)  
        
        #------------Set Creation ---------------------------------------#

        #---------double wall set---------
        
        name = partname
        p = model.parts[name]
        faces = []
        for temp in doublewall_coor_3d:
            faces.append(p.faces.findAt((temp,)))
        
        name = 'doublewall'
        set_sidewall = p.Set(name, faces = faces)
        
        
        f = p.faces
        faces_centroid=[]
        faces_rest=[]
        flag = 0
        x = 0
        
        faces = p.sets[name].faces
        
        #---------single wall set---------
        for temp in f:
            
            for temp1 in faces:
                if (temp1 == temp):
                    flag = 1
                    # print "flag = ", flag
            
            if flag!=1:
                faces_centroid.append(temp.getCentroid())
                # print faces_centroid[x]
                x = x+1
                # print "counter = ",x 
                
            flag = 0
            
         
        for x in range(len(faces_centroid)):
            temp = f.findAt(faces_centroid[x])
            faces_rest.append(temp)
         
        set_hcorewall = p.Set('singlewall', faces = faces_rest)
      
        
        #-------------Assembly Instance----------------------------------------#
        name = partname
        HcoreAssemblyInstance(hcoremodelName, name)
	
	
        
        #--------------------------glue part creation (if opted)---------------#
        
        if glue == TRUE:

            gluepartname = 'h_glue'
            name = gluepartname
            depth = glueheight
            HcorePartCreation(hcoremodelName, name, depth,offset_x, offset_y)
                        
            #------------Section Creation -------------------------------------#
            
	
            sectionname='GlueWall'
            name = sectionname
            material=h_gluematerialName
            thickness=gluethickness
            HcoreSectionCreation(hcoremodelName,name, material,thickness)
            
            #-----------Section Assignment ------------------------------------#
            name = gluepartname
            p = model.parts[name]
            faces = p.faces
            
            set_gluepart = p.Set('gluepart', faces = faces)
            p.SectionAssignment(region=set_gluepart, sectionName=sectionname, 
            offset=0.0, offsetType=MIDDLE_SURFACE)
	    
	    
            #-----------Assembly Instance -------------------------------------#
            for x in range(0,2):
                
                name = gluepartname
                HcoreAssemblyInstance(hcoremodelName, name)
		
            
            #------ glue one layer --------------------------
            name = gluepartname
            distance = -glueheight
            a = model.rootAssembly.instances
            keys = a.keys()
            assemblyname = keys[1]
            
            HcoreGlueAssembly(hcoremodelName, name, assemblyname, distance)
	    	    
            
            #------ glue second layer -------------------------
            
            distance = height
            assemblyname = keys[2]
            HcoreGlueAssembly(hcoremodelName, name, assemblyname, distance)
            
        
            #--------------Assembly Merge (useful when glue included)--------------#
	    
                    
            name = 'hcore_withglue'
            HcoreAssemblyMerge(hcoremodelName,name)
        
        #--------------Assembly Pattern (Finaly Honeycomb struture)------------#
	
        
        name = 'honeycomb'   
	
        X_dir,Y_dir, partName = HcorePattern(X_dir,Y_dir,pattern,hcoremodelName,name,offset_x,offset_y)
	#HcorePattern(X_dir,Y_dir,pattern,hcoremodelName,name,offset_x,offset_y)
		
	
        name = 'boundarywall'
        HcoreBoundarySet(X_dir,Y_dir,hcoremodelName,name,partName,offset_x,offset_y)
	
	
        boundarywallsetName = 'boundarywall'
        singlewallsetName = 'singlewall'
        doublewallsetName = 'doublewall'
        
        HcoreFinalSetSectionAssignment(hcoremodelName,partName,boundarywallsetName,singlewallsetName,doublewallsetName,hcorematerialName,thickness_single,thickness_double)
	
	keys = model.parts.keys()
	l = len(keys)
	session.viewports['Viewport: 1'].setValues(displayedObject=None)
	p = model.parts[keys[l-1]]
	session.viewports['Viewport: 1'].setValues(displayedObject=p)
       
	hex_coor[:] = []
        sidewall_coor[:] = []
        doublewall_coor_3d[:] = []
	
	print "The honey comb was successfully created in part: ",keys[l-1]
	
	
        
        
        
        
        
        
           
    
    
    

        
        
        
    
        
            
            
        
        

        
        

        
        
        
        
        