from abaqusConstants import *
from abaqusGui import *
from kernelAccess import *
import os
from abq_UnwrapMesh.unwrapMeshConstants import *
thisPath = os.path.abspath(__file__)
thisDir = os.path.dirname(thisPath)


###########################################################################
# Class definition
###########################################################################

class UnwrapMeshDB(AFXDataDialog):
    [ ID_3,
    ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+1)
    
    numInstance =0
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        # Construct the base class.
        #

        AFXDataDialog.__init__(self, form, 'Create Unwrapped Part',
            self.OK|self.CANCEL, DIALOG_ACTIONS_SEPARATOR, DECOR_RESIZE)
            
        self.form=form
        UnwrapMeshDB.numInstance+=1
        

        #GuiLog Prefix
        getAFXAliasMap().setPrefix(self,'UnwrapMeshDB-%d'% UnwrapMeshDB.numInstance)


        mainWindow = getAFXApp().getAFXMainWindow()
        FXMAPFUNC(self, SEL_COMMAND, self.ID_3, UnwrapMeshDB.onCmdCreateGeom)
            

        #GroupBox for Datum Axis
        GroupBox_1 = FXGroupBox(p=self, text='Create/Select a Datum Axis', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        VFrame_1 = FXVerticalFrame(p=GroupBox_1, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        HFrame_2 = FXHorizontalFrame(p=VFrame_1, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        w=FXRadioButton(p=HFrame_2, text='Create based on two points', tgt=form.enterPickKw, sel=PICK_POINT.getId())
        getAFXAliasMap().setName(w, 'TwoPointDatumAxis') #For guiLog

        #Datum axis by picking in viewport 
        pickHf = FXHorizontalFrame(p=VFrame_1, opts=0, x=0, y=0, w=0, h=0,
                                   pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        pickHf.setSelector(99)

       
        label = FXLabel(p=pickHf, text='Point 1 (None)', ic=None, opts=LAYOUT_CENTER_Y|JUSTIFY_LEFT)        
        pickHandler = unWrapDBPickHandler(form, form.point1Kw, 'Pick a point', POINTS|VERTICES|INTERESTING_POINTS|DATUM_POINTS, ONE, label)
        icon = afxCreatePNGIcon(thisDir+r'/selectIcon.png')
         #Slecting Point 1
        self.btn1=FXButton(p=pickHf, text='\tPick Items in Viewport', ic=icon, tgt=pickHandler, sel=AFXMode.ID_ACTIVATE,
                           opts=BUTTON_NORMAL|LAYOUT_CENTER_Y, x=0, y=0, w=0, h=0, pl=2, pr=2, pt=1, pb=1)
        getAFXAliasMap().setName(self.btn1, 'Ponit1inVP') #For guiLog

        pickHf = FXHorizontalFrame(p=VFrame_1, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        pickHf.setSelector(99)
        label = FXLabel(p=pickHf, text='Point 2 (None)', ic=None, opts=LAYOUT_CENTER_Y|JUSTIFY_LEFT)
        pickHandler = unWrapDBPickHandler(form, form.point2Kw, 'Pick a point', POINTS|VERTICES|INTERESTING_POINTS|DATUM_POINTS, ONE, label)
        icon = afxCreatePNGIcon(thisDir+r'/selectIcon.png')
             #Slecting Point 2
        self.btn2=FXButton(p=pickHf, text='\tPick Items in Viewport', ic=icon, tgt=pickHandler, sel=AFXMode.ID_ACTIVATE,
            opts=BUTTON_NORMAL|LAYOUT_CENTER_Y, x=0, y=0, w=0, h=0, pl=2, pr=2, pt=1, pb=1)
        getAFXAliasMap().setName(self.btn2, 'Ponit2inVP') #For guiLog

        #Selcting existing datum axis
        HFrame_3 = FXHorizontalFrame(p=VFrame_1, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        w1=FXRadioButton(p=HFrame_3, text='Existing datum axes', tgt=form.enterPickKw, sel=SELECT_EXISTING.getId())
        getAFXAliasMap().setName(w1, 'ExistingDatumAxesRB') #For guiLog

        self.ComboBox_1 = AFXComboBox(p=HFrame_3, ncols=10, nvis=1, text='', tgt=form.datumAxes1Kw, sel=0)
        self.ComboBox_1.setMaxVisible(10)
        getAFXAliasMap().setName(self.ComboBox_1, 'ExisDatAxisName') #For guiLog
                        
        #Groupbox for entering the part data for created geometry
        GroupBox_2 = FXGroupBox(p=self, text='Enter Part Data', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        self.nametf=AFXTextField(p=GroupBox_2, ncols=12, labelText='New part name', tgt=form.partNameKw, sel=0)
        getAFXAliasMap().setName(self.nametf, 'partName') #For guiLog
        self.tf1=AFXTextField(p=GroupBox_2, ncols=12, labelText='Unwrapping reference cylinder radius',
                              tgt=form.partRadiusKw, sel=0,opts=AFXTEXTFIELD_FLOAT)
        getAFXAliasMap().setName(self.tf1, 'CylRadius') #For guiLog
        
        VFrame_2 = FXVerticalFrame(p=GroupBox_2, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
                                   pl=0, pr=0, pt=0, pb=0)
        HFrame_4 = FXHorizontalFrame(p=VFrame_2, opts=0, x=0, y=0, w=0, h=0,
                                     pl=0, pr=0, pt=0, pb=0)
        label = FXLabel(p=HFrame_4, text='Pick a point of unwrapping', ic=None, opts=LAYOUT_CENTER_Y|JUSTIFY_LEFT)
        HFrame_3 = FXHorizontalFrame(p=VFrame_2, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        label = FXLabel(p=HFrame_3, text='', ic=None, opts=LAYOUT_CENTER_Y|JUSTIFY_LEFT)            
        self.Tf1=AFXTextField(p=HFrame_3, ncols=7, labelText='X:', tgt=form.xCoordKw, sel=0, opts=AFXTEXTFIELD_READONLY)
        self.Tf2=AFXTextField(p=HFrame_3, ncols=7, labelText='Y:', tgt=form.yCoordKw, sel=0, opts=AFXTEXTFIELD_READONLY)
        self.Tf3=AFXTextField(p=HFrame_3, ncols=7, labelText='Z:', tgt=form.zCoordKw, sel=0, opts=AFXTEXTFIELD_READONLY)
        pickHandler = unWrapDBPickHandler(form, form.pointKw, 'Pick a point',
            AFXPickStep.POINTS|AFXPickStep.NODES, AFXPickStep.ONE, label)
        icon = afxCreatePNGIcon(thisDir+r'/selectIcon.png')
        self.btn3=FXButton(p=HFrame_3, text='\tPick Items in Viewport', ic=icon, tgt=pickHandler, sel=AFXMode.ID_ACTIVATE,
                           opts=BUTTON_NORMAL|LAYOUT_CENTER_Y, x=0, y=0, w=0, h=0, pl=2, pr=2, pt=1, pb=1)
        getAFXAliasMap().setName(self.btn3, 'UnwrapPtinVP') #For guiLog

        HFrame_5 = FXHorizontalFrame(p=VFrame_2, opts=0, x=0, y=0, w=0, h=0, pl=0, pr=0, pt=0, pb=0)
        
        self.chkBtn = FXCheckButton(HFrame_5, 'Generate geometry',  self, 
                                                    self.ID_3)
        self.chkBtn.setCheck(TRUE)
        getAFXAliasMap().setName(self.chkBtn, 'GengeomChkBtn') #For guiLog
          
        HFrame_6 = FXHorizontalFrame(p=VFrame_2, opts=0, x=0, y=0, w=0, h=0, pl=0, pr=0, pt=0, pb=0)
        label = FXLabel(p=HFrame_6, text='    ', ic=None, opts=LAYOUT_CENTER_Y|JUSTIFY_LEFT)
        self.lineRadBtn=FXRadioButton(p=HFrame_6, text='Using piecewise linear segment', tgt=form.splineLineKw, sel=USE_LINE.getId())
        getAFXAliasMap().setName(self.lineRadBtn, 'lineSegBtn') #For guiLog

        HFrame_7 = FXHorizontalFrame(p=VFrame_2, opts=0, x=0, y=0, w=0, h=0, pl=0, pr=0, pt=0, pb=0)
        label = FXLabel(p=HFrame_7, text='    ', ic=None, opts=LAYOUT_CENTER_Y|JUSTIFY_LEFT)
        self.splineRadBtn=FXRadioButton(p=HFrame_7, text='Using splines that break at feature Angle', tgt=form.splineLineKw, sel=USE_SPLINE.getId())
        getAFXAliasMap().setName(self.splineRadBtn, 'splineRadBtn') #For guiLog
        self.geomTf=AFXTextField(p=HFrame_7, ncols=10, labelText='', tgt=form.fAngleKw, sel=0)
        getAFXAliasMap().setName(self.geomTf, 'splineRadVal') #For guiLog
        afxDisableWidgetTree(self.geomTf)
        
        self.listPartNamesReg = []
     
    

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

    def processUpdates(self):

        if self.form.enterPickKw.getValue() == PICK_POINT.getId():
            afxEnableWidgetTree(self.btn1)
            afxEnableWidgetTree(self.btn2)
            afxDisableWidgetTree(self.ComboBox_1)            
        else:
            afxEnableWidgetTree(self.ComboBox_1)

            self.form.datumAxes1Kw.setValue(self.ComboBox_1.getText())
            afxDisableWidgetTree(self.btn1)
            afxDisableWidgetTree(self.btn2)
        if self.form.splineLineKw.getValue() == USE_SPLINE.getId() and self.chkBtn.getCheck():
            afxEnableWidgetTree(self.geomTf)
            self.form.geomKw.setValue(True)
        elif self.form.splineLineKw.getValue() == USE_LINE.getId() and self.chkBtn.getCheck():
            afxDisableWidgetTree(self.geomTf)
            self.form.geomKw.setValue(True)
        else:
            afxDisableWidgetTree(self.geomTf)
            self.form.geomKw.setValue(False)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdCreateGeom(self,sender,sel,ptr):
        if self.chkBtn.getCheck():
            self.form.geomKw.setValue(True)
            afxEnableWidgetTree(self.lineRadBtn)
            afxEnableWidgetTree(self.splineRadBtn)
            if self.form.splineLineKw.getValue() == USE_SPLINE.getId():
                afxEnableWidgetTree(self.geomTf)
            elif self.form.splineLineKw.getValue() == USE_LINE.getId():
                afxDisableWidgetTree(self.geomTf)
        else:
            afxDisableWidgetTree(self.lineRadBtn)
            afxDisableWidgetTree(self.splineRadBtn)
            afxDisableWidgetTree(self.geomTf)
            self.form.geomKw.setValue(False)
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def populateDatumAxes(self):
        
        from kernelAccess import session, mdb
        curViewport = session.viewports[session.currentViewportName]
        mainWindow = getAFXApp().getAFXMainWindow()


        #if str(type(curViewport.displayedObject)) == "<type 'NoneType'>":
        #    return False

        if getDisplayedObjectType() == None:
            return False #changed code

        if not hasattr(curViewport.displayedObject, 'modelName'):
            return False

        modName=curViewport.displayedObject.modelName

        if not hasattr(curViewport.displayedObject, 'name'):
            showAFXErrorDialog(mainWindow, 'The plug-in works only\n'
                                                 'in the Part module.\n\n')
            return False

        partName=curViewport.displayedObject.name
        model=mdb.models[modName]
        try:
            part = model.parts[partName]
        except:
            showAFXErrorDialog(mainWindow, 'The plug-in works only\n'
                                                  'in the Part module.\n\n')
            return False

        assembly=model.rootAssembly
 
        modName=curViewport.displayedObject.modelName
        partName=curViewport.displayedObject.name
        model=mdb.models[modName]
        assembly=model.rootAssembly
        part = model.parts[partName]

        ids=[]
        self.form.datumAxes1Kw.setValue('')

        self.ComboBox_1.clearItems()
        
        for feature in part.features.keys():
            ids.append(part.features[feature].id)
        datumAxisIds=[]

        #Appending the datum axis from the datum objects to the combo box
        #
        if part.datums:

            for x in part.datums.keys():

                part.datums[x] == part 
                if str(type(part.datums[x]))== "<class 'kernelAccess.DatumAxisProxy'>": #change
          
                    datumAxisIds.append(x)

        if datumAxisIds:
            for i in part.features.keys():
                if part.features[i].id in datumAxisIds:
                    featureName=part.features[i].name 
                    self.ComboBox_1.appendItem(featureName)
            self.form.datumAxes1Kw.setValue(featureName)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        unregisterCurrentContext(self.populateDatumAxes)

        from kernelAccess import session, mdb
        curViewport = session.viewports[session.currentViewportName]
        mainWindow = getAFXApp().getAFXMainWindow()

        #if str(type(curViewport.displayedObject)) !="<class 'kernelAccess.PartProxy'>": 
        if getDisplayedObjectType() != PART: #changed code from original

            return None 
        else:

            for partName in self.listPartNamesReg:
                mdb.models[partName[0]].parts[partName[1]].features.unregisterQuery(self.populateDatumAxes)
            self.listPartNamesReg=[]

           
        AFXDataDialog.hide(self)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
        
        registerCurrentContext(self.populateDatumAxes)
        from kernelAccess import session, mdb,part
        curViewport = session.viewports[session.currentViewportName]
        mainWindow = getAFXApp().getAFXMainWindow()
        #if str(type(curViewport.displayedObject)) !="<class 'kernelAccess.PartProxy'>": #change        

        if getDisplayedObjectType() != PART: #changed code from original


            showAFXErrorDialog(mainWindow,'There is no Part \n'
                                          'associated with the current viewport.\n\n'
                                              'Switch to Part module,\n'
                      'or Select a Part from the context bar.')

            return None #change        
        else:

            partName=curViewport.displayedObject.name
            model = getCurrentContext()['modelName']
            mdb.models[model].parts[partName].features.registerQuery(self.populateDatumAxes)
            self.listPartNamesReg.append((model,partName))

            
        AFXDataDialog.show(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def setPickedPoint(self, picked_point):

        # set the keyword in the form
        #
        self.form.xCoordKw.setValue(str(picked_point[0]))
        self.form.yCoordKw.setValue(str(picked_point[1]))
        self.form.zCoordKw.setValue(str(picked_point[2])) 

        self.setPointTFtoKw()  

        self.btn3.setState(STATE_UP)         
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        

    def setPointTFtoKw(self):
        
        self.PtTxt1 = self.form.xCoordKw.getValueAsString()
        self.PtTxt2 = self.form.yCoordKw.getValueAsString()
        self.PtTxt3 = self.form.zCoordKw.getValueAsString()
        self.Tf1.setText(self.PtTxt1)
        self.Tf2.setText(self.PtTxt2)
        self.Tf3.setText(self.PtTxt3)
       

###########################################################################
# Class definition
###########################################################################

class unWrapDBPickHandler(AFXProcedure):

        count = 0

        #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        def __init__(self, form, keyword, prompt, entitiesToPick, numberToPick, label):

                self.form = form
                self.keyword = keyword
                self.prompt = prompt
                self.entitiesToPick = entitiesToPick # Enum value
                self.numberToPick = numberToPick # Enum value
                self.label = label
                self.labelText = label.getText()

                AFXProcedure.__init__(self, form.getOwner())

                unWrapDBPickHandler.count += 1
                self.setModeName('unWrapDBPickHandler%d' % (unWrapDBPickHandler.count) )

        #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        def getFirstStep(self):

                return  AFXPickStep(self, self.keyword, self.prompt, 
                    self.entitiesToPick, self.numberToPick, sequenceStyle=TUPLE)

        #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        def getNextStep(self, previousStep):

                if self.label.getText() != '':
                    self.label.setText( self.labelText.replace('None', 'Picked') )
                    return None
                else:
                    return None            
       
       #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        def deactivate(self, force=False):
            if force:
                AFXProcedure.deactivate(self)
                return None

            try:
                if self.label.getText() =='':
                    cmd_list1 = self.keyword.getSetupCommands().splitlines()
                    str1 = cmd_list1[0].split('=')[-1].strip()
                    str2 = cmd_list1[1].split('.')[-1].split(']')[0]
                    str3 = cmd_list1[-1].split('[')[-1].split(']')[0]

                    cmd1 = str1+"."+str2+"["+str3+"]."+"coordinates"

                    coord1=eval(cmd1)
                    coord1x= "%.3f" %coord1[0]
                    coord1y= "%.3f" %coord1[1]
                    coord1z= "%.3f" %coord1[2]
                    coordinates=(coord1x, coord1y, coord1z)

                    self.form.getCurrentDialog().setPickedPoint(coordinates)

                    AFXProcedure.deactivate(self)
                    return None
                else:
                    AFXProcedure.deactivate(self)
                    return None                
            except:
                try:
                    self.form.getCurrentDialog().btn3.setState(STATE_UP)
                except:
                     showAFXErrorDialog(mainWindow, 'An error occurred, close the plug-in dialog and reopen.')
                AFXProcedure.deactivate(self)
                return None
          
