from abaqusGui import *
from abaqusConstants import *
import osutils, os
from abq_UnwrapMesh.unwrapMeshConstants import *

###########################################################################
# Class definition
###########################################################################

class UnwrapMeshForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        
        # Construct the base class.
        #
        AFXForm.__init__(self, owner)
        self.radioButtonGroups = {}

        self.cmd = AFXGuiCommand(mode=self, method='unwrapMesh',
            objectName='abq_UnwrapMesh.unwrapMeshModule', registerQuery=False)
        pickedDefault = ''
        self.point1Kw = AFXObjectKeyword(self.cmd, 'point1', True, pickedDefault)
        self.point2Kw = AFXObjectKeyword(self.cmd, 'point2', True, pickedDefault)
        self.datumAxes1Kw = AFXStringKeyword(self.cmd, 'datumAxes1', True)
        self.enterPickKw = AFXSymConstKeyword(self.cmd, 'entered', True, PICK_POINT.getId())
        self.partNameKw = AFXStringKeyword(self.cmd, 'unwrappedPartName', True, '')
        self.partRadiusKw = AFXStringKeyword(self.cmd, 'wrappedPartRadius', True, '')
        self.pointKw = AFXObjectKeyword(self.cmd, 'pickedPoint', TRUE, pickedDefault)
        self.xCoordKw = AFXStringKeyword(self.cmd, 'xCoord', True, '0')
        self.yCoordKw = AFXStringKeyword(self.cmd, 'yCoord', True, '0')
        self.zCoordKw = AFXStringKeyword(self.cmd, 'zCoord', True, '0')
        self.geomKw = AFXBoolKeyword(command=self.cmd,name='createGeom',isRequired=True,defaultValue=TRUE)
        self.fAngleKw = AFXStringKeyword(self.cmd, name='featureAngle', isRequired=True, defaultValue='20')
        
        self.splineLineKw = AFXSymConstKeyword(self.cmd, 'splineLine', True, USE_LINE.getId())
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        from abq_UnwrapMesh.unwrapMeshDB import UnwrapMeshDB
        self.unwrapMeshDB = UnwrapMeshDB(self)
        return self.unwrapMeshDB

    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def activate(self):
        mainWindow = getAFXApp().getAFXMainWindow()
        from kernelAccess import session
        from kernelAccess import mdb
        
        if getDisplayedObjectType() != PART:
            showAFXErrorDialog(mainWindow,
            'There is no part associated with the current viewport.\n\n'
            'Select a part from the context bar, or use "File->Import" to import a part.')
            return None

        vp = session.viewports[session.currentViewportName]
        vpModelName = vp.displayedObject.modelName
        vpPartName = vp.displayedObject.name

        part = mdb.models[vpModelName].parts[vpPartName]
        if part.queryGeometry(printResults=False)['category'] != 'OrphanMesh':
            showAFXErrorDialog(mainWindow,
                'The plug-in is supported only for orphan mesh.')
            return None

        if (part.verifyMeshQuality(criterion=ANALYSIS_CHECKS,elemShape=QUAD)['numElements']\
            or part.verifyMeshQuality(criterion=ANALYSIS_CHECKS,elemShape=TRI)['numElements'])\
            and not (part.verifyMeshQuality(criterion=ANALYSIS_CHECKS,elemShape=HEX)['numElements'] or\
            part.verifyMeshQuality(criterion=ANALYSIS_CHECKS,elemShape=TET)['numElements'] or \
            part.verifyMeshQuality(criterion=ANALYSIS_CHECKS,elemShape=LINE)['numElements'] or \
            part.verifyMeshQuality(criterion=ANALYSIS_CHECKS,elemShape=WEDGE)['numElements']):

            AFXForm.activate(self)
        else:

            showAFXErrorDialog(mainWindow,
                'The plug-in is supported only for orphan mesh parts with shell elements.')
            return None
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def verifyKeywordValues(self):

        db = self.getCurrentDialog()
        if self.enterPickKw.getValueAsString()=='PICK_POINT':
            if not self.point1Kw.getValue():
                errMsg = 'Define the circumferential direction.'
                showAFXErrorDialog(db, errMsg)
                return False
            if not self.point2Kw.getValue():
                errMsg = 'Define the circumferential direction.'
                showAFXErrorDialog(db, errMsg)
                return False            
        elif self.enterPickKw.getValueAsString()=='SELECT_EXISTING':
            if self.datumAxes1Kw.getValue()=='' or self.datumAxes1Kw.getValue()==None :
          
                errMsg = 'Select an existing datum axis.'
                showAFXErrorDialog(db, errMsg)
                return False
        if not self.partNameKw.getValue():    
            errMsg = 'Enter the name of the new part.'
            showAFXErrorDialog(db, errMsg)
            return False
        
        if not self.partRadiusKw.getValue():    
            errMsg = 'Enter the unwraping radius of the reference surface.'
            showAFXErrorDialog(db, errMsg)
            return False
        if self.geomKw.getValue():

            if self.splineLineKw == 'USE_SPLINE' and self.fAngleKw.getValue()=='':

                errMsg="Enter a feature angle."
                showAFXErrorDialog(db,errMsg)
                return False      
        return AFXForm.verifyKeywordValues(self)

