from abaqus import *
from abaqusConstants import *
from abq_UnwrapMesh.unwrapMeshConstants import *
import math
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def unwrapMesh(point1=None, point2=None, datumAxes1=None, entered=None, unwrappedPartName=None,
                wrappedPartRadius=None, pickedPoint=None, xCoord=None, yCoord=None, zCoord=None, createGeom=None, 
                featureAngle=None, splineLine=None):
    
    featureAngle=float(featureAngle)
    wrappedPartRadius=float(wrappedPartRadius)
   
    try:
        pLabel=pickedPoint.label
    except:
        raise ValueError, 'Pick a point of unwrapping'

    curViewport = session.viewports[session.currentViewportName]
    try:
        modName=curViewport.displayedObject.modelName
    except:
        raise ValueError, 'The displayed object must be a part in the Part module.'
    
    partName=curViewport.displayedObject.name
    model=mdb.models[modName]

    try:  
        part = model.parts[partName]
    except:
        raise ValueError, 'The plug-in works only in the Part module.'
    
    assembly=model.rootAssembly
    
    #An instance name is generated to create an instance of the displayed
    #part object
    #
    instanceName=generateInstanceName(partName, assembly)
    assembly.Instance(name=instanceName, part=part, dependent=ON)
    
    #Available Features in Assembly and part before creating  ones in the plug-in
    #i.e. edge to edge constraint, 
    #        
    asseFeatures=assembly.features.keys()
    prtFeatures= part.features.keys()
    
    if not datumAxes1:

        movableAxis1,name, movableAxisId=createDatumAxis(point1, point2, part, model, assembly, prtFeatures, 
                                                                                                asseFeatures)
        movableAxis =assembly.instances[instanceName].datums[movableAxisId]
    else:

        movableAxis1=part.features[datumAxes1]
        movableAxisId=part.features[datumAxes1].id
        
        movableAxis =assembly.instances[instanceName].datums[movableAxisId]

    
    #Create datum Csys by default,which will be used to create
    #Edge to edge constraint
    #
    datumCsys=assembly.DatumCsysByDefault(CARTESIAN)
    datumCsysId=datumCsys.id
    fixedAxis= assembly.datums[datumCsysId].axis1
      
    #assembly.features
    createEdge2EdgeConstraint(movableAxis, fixedAxis, assembly, part, asseFeatures, prtFeatures)    
    aNode=assembly.instances[instanceName].nodes.getFromLabel(pLabel)
    xCoord=aNode.coordinates[0]
    yCoord=aNode.coordinates[1]
    zCoord=aNode.coordinates[2]

    
    #Compute angle
    #

    x= zCoord/(math.sqrt((zCoord ** 2)+(yCoord ** 2)))
    alpha = math.acos(x)
    
    if yCoord > 0 or yCoord == 0:
        alpha1 = alpha * 180/pi
    else:
        alpha1 = (2*pi-alpha)*180/pi
 
    #Rotate the instance
    #
    try:
        assembly.rotate(instanceList=(instanceName,), axisPoint=(0.0, 0.0, 0.0), axisDirection=(1.0, 0.0, 0.0), angle=alpha1 ) 
    except:
        delFeatures(assembly,part, asseFeatures, prtFeatures)
        raise ValueError, 'Instance rotation failed.'
  
    
    #Create meshed part
    #
    unwrappedInstance=(assembly.instances[instanceName], )
    
    try:
        assembly.PartFromInstanceMesh(name=unwrappedPartName, partInstances=unwrappedInstance)
        print '"%s" is created.' % unwrappedPartName
        
    except:              
        #Call a function to delete features that were created in the plug-in,
        #since we do not want the features created by the plug-in to stay if the plug-in fails.
        #
        delFeatures(assembly,part, asseFeatures, prtFeatures)
        raise ValueError, 'An error occurred while creating an orphan mesh.'

    unwrappedPart = model.parts[unwrappedPartName]
    
    setName=editNodes(unwrappedPart,wrappedPartRadius)
    recreateNodes(setName, unwrappedPart, wrappedPartRadius, assembly,part, asseFeatures, prtFeatures)
    
    #Merge nodes to remove duplicate nodes
    #
    unwrappedPart.mergeNodes(nodes=unwrappedPart.nodes, tolerance=1e-06)
    
    #Deleting the instance created by the plug-in
    #
    
    del assembly.instances[instanceName]

    
    curViewport.setValues(displayedObject=unwrappedPart)
    
    
    #Checking if Create Geometry option is selected. If yes then using spline or using
    #line?
    #
    if createGeom and splineLine==USE_SPLINE:
        #Changing the modeling space to 2D planar so that part2DGeomFrom2DMesh can be used.
        #
        unwrappedPart.setValues(space=TWO_D_PLANAR, type=DEFORMABLE_BODY)
        try:
            geometryPart=model.Part2DGeomFrom2DMesh(name=unwrappedPartName+'_Geometry', part=unwrappedPart, featureAngle=featureAngle)
            curViewport.setValues(displayedObject=geometryPart)
            print '"%s"_Geometry is created.' % unwrappedPartName
        except:
            raise ValueError, 'Creating Geometry failed.'
    elif createGeom and splineLine==USE_LINE:       
    #Changing the modeling space to 2D planar so that part2DGeomFrom2DMesh can be used.
        #
        unwrappedPart.setValues(space=TWO_D_PLANAR, type=DEFORMABLE_BODY)
        try:
            geometryPart=model.Part2DGeomFrom2DMesh(name=unwrappedPartName+'_Geometry', part=unwrappedPart, featureAngle=0.0)
            curViewport.setValues(displayedObject=geometryPart)
            print unwrappedPartName+'_Geometry' +' is created.'
        except:
            raise ValueError, 'Creating Geometry failed'
        
  
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Function to create datum axis
#
def createDatumAxis(point1, point2, part, model, assembly, prtFeatures, asseFeatures):
    
    try:
        newDatumAxis=part.DatumAxisByTwoPoint(point1, point2)
    except:
        delFeatures(assembly,part, asseFeatures, prtFeatures)
        raise ValueError, 'Creation of datum axis failed.'
    movableAxisName=newDatumAxis.name
    
    movableAxisId = newDatumAxis.id
    
    return newDatumAxis, movableAxisName, movableAxisId 
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createEdge2EdgeConstraint(movableAxis, fixedAxis, assembly, part, asseFeatures, prtFeatures):    
    try:

        assembly.EdgeToEdge(movableAxis=movableAxis, fixedAxis=fixedAxis, flip=OFF)
    except:
        delFeatures(assembly,part, asseFeatures, prtFeatures)
        raise ValueError, 'Creation of Edge to edge constraint failed.'
    return
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""
#The function checks for exististing instance names and creates a new name
"""
def generateInstanceName(partName, assembly):
    
    existingInstNames=assembly.instances.keys()
    count=1        
    while 1:
        if partName+'_'+str(count) in existingInstNames:
            count+=1
            continue
        else:
            instanceName=partName+'_'+str(count)
        break
    return instanceName
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
"""
#The function deletes the features which were created by the plug-in
"""
def delFeatures(assembly, part, asseFeatures, prtFeatures):
    #Total features available in part and assembly after running the plug-in
    #
    totAsseFeatures = assembly.features.keys()
    totPrtFeatures = part.features.keys()
    
    for afeature in totAsseFeatures:
        if afeature not in asseFeatures:
            assembly.deleteFeatures((afeature,))
    for pfeature in totPrtFeatures:
        if pfeature not in prtFeatures:
            part.eleteFeatures((pfeature,))
    
    return   
   

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
"""
#The function that changes the coordinates of the nodes in cylindrical coordinate
#system
"""
def editNodes(unwrappedPart,wrappedPartRadius):   
    nodes =unwrappedPart.nodes
    coords = []
    for n in nodes:
        x,y,z = n.coordinates
        theta = math.acos(z/(math.sqrt((z** 2)+(y** 2))))
        newZ = 0
        if y > 0 or y==0:            
            newY = theta*wrappedPartRadius
        elif y < 0:
            newY = (2*pi - theta)*wrappedPartRadius
        newX = x
        coords.append((newX, newY, newZ))
    unwrappedPart.editNode(nodes=nodes,coordinates=coords)   
    el = unwrappedPart.verifyMeshQuality(criterion=ASPECT_RATIO, threshold=10)['failedElements']
    
    setName=getSetName('StretchedEls', unwrappedPart)
    a=unwrappedPart.Set(name=setName, elements=el)
    
    return setName
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
"""
#The function removes the stretched elements and  recreates nodes and using node connectivity
creates the original elements, which were stretched after the plug-in run and removed.
"""
def recreateNodes(setName, unwrappedPart, wrappedPartRadius, assembly, part, asseFeatures, prtFeatures):

    #Create a list of stretched element node labels and corresponding coordinates
    #
    stretElems=unwrappedPart.sets[setName].elements

    if not stretElems: return


    coOrds=[]
    coOrdLabs=[]
    labs=[]
    labs1=[]
    for el in stretElems:
        elstrNods=el.getNodes()
        coOrds=[]
        labs=[]
        for i in range(len(elstrNods)):
            x=elstrNods[i].coordinates
            lb=elstrNods[i].label
            coOrds.append(x)
            labs.append(lb)
            x=[]            
        coOrdLabs.append((coOrds,labs))
        labs1.append(labs)


    #Find perimeter of the wrapped mesh
    #
    peri=2*wrappedPartRadius*pi

    for i in coOrdLabs:
        for j in range(len(i[0])):
            if i[0][j][1] < wrappedPartRadius or i[0][j][1] == wrappedPartRadius:

                createdNode=unwrappedPart.Node(coordinates=i[0][j])

                unwrappedPart.editNode(nodes=(createdNode,), offset2=peri)

                newLab= createdNode.label

                i[1][j]= newLab

        #Create new QUAD4 elements using the newly created nodes
        #
        nds=[]
        for t in range(len(i[1])):
            nds.append(unwrappedPart.nodes.getFromLabel(i[1][t]))
        try:
            el1=unwrappedPart.Element(nds,QUAD4)
            
        except:
            #print 'test'
            #delFeatures(assembly, part, asseFeatures, prtFeatures)
            raise ValueError, 'The plug-in only supports QUAD4 elements.'


    #Deleting the stretched elements and its set
    #
    unwrappedPart.deleteElement(elements=unwrappedPart.sets[setName], deleteUnreferencedNodes=ON)

    del unwrappedPart.sets[setName]

    return

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
#Function to check if a setName exists and create a new set name if needed
#
def getSetName(setName, part):
    existingSetNames=part.sets.keys()
    count=1        
    while 1:
        if setName+'_'+str(count) in existingSetNames:
            count+=1
            continue
        else:
            newSetName=setName+'_'+str(count)
        break
    return setName  
    
    