from abaqusGui import *

#Used to generate correctly sized dialog
dummyText = """Laminate specification for for picked region using layup CompositeHull_Layup
  Ply        Material    Thickness Local Orientation Angle
-------------------------------------------------------------------------------------
    1     Glass-Epoxy      0.75000            0.00
    2     Glass-Epoxy      1.75000           45.00
    3     Glass-Epoxy      1.75000          -45.00
    4     Glass-Epoxy      1.00000           90.00
    5            Core     40.00000            0.00
    6     Glass-Epoxy      0.75000           90.00
    7     Glass-Epoxy      1.75000          -45.00
    8     Glass-Epoxy      1.75000           45.00
    9     Glass-Epoxy      0.75000            0.00

ABD Matrix Calculation for for picked region using layup CompositeHull_Layup
  1.7642E+005   6.6256E+004   6.1035E-004 |  -3.8182E+004  -1.1455E+004  -1.2817E-003
  6.6256E+004   1.8343E+005   6.1035E-004 |  -1.1455E+004  -1.7839E+005  -1.2817E-003
  6.1035E-004   6.1035E-004   7.9548E+004 |  -1.2817E-003  -1.2817E-003  -1.7995E+004
-------------------------------------------------------------------------------------
 -3.8182E+004  -1.1455E+004  -1.2817E-003 |   9.4381E+007   3.3893E+007   1.9429E+006
 -1.1455E+004  -1.7839E+005  -1.2817E-003 |   3.3893E+007   8.9096E+007   1.9429E+006
 -1.2817E-003  -1.2817E-003  -1.7995E+004 |   1.9429E+006   1.9429E+006   4.0722E+007

Equivalent extensional moduli for for picked region using layup CompositeHull_Layup
EXT   =   3.0346E+003
EYT   =   3.1552E+003
GXYT  =   1.5830E+003
NUXYT =   0.3612

Equivalent bending moduli for for picked region using layup CompositeHull_Layup
EXB   =   7.7067E+003
EYB   =   7.2751E+003
GXYB  =   3.8513E+003
NUXYB =   0.3804"""

###########################################################################
# Class definition
###########################################################################

class AbdDB(AFXDataDialog):
    [
        ID_CREATE,
        ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+1)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form, abdPlyProc, abdFaceSheetsProc):

        # Construct the base class.
        #
        self.form = form
        self.abdPlyProc = abdPlyProc
        self.abdFaceSheetsProc = abdFaceSheetsProc
        AFXDataDialog.__init__(self, form, 'Calculate ABD Matrix',
            self.DISMISS, DIALOG_ACTIONS_SEPARATOR|DECOR_RESIZE)
            
        getAFXAliasMap().setPrefix(self, 'calcABD')
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CREATE,
                  AbdDB.onCmdCreate)
        typeGB = FXGroupBox(p=self, text='Type', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        gb2 = FXGroupBox(p=self, text='Selection', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        resultsGB = FXGroupBox(p=self, text='Results', opts=FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)

        self.switcher = FXSwitcher(gb2, LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0)
        getAFXAliasMap().setName(self.switcher, 'type')
        
        hf1 = FXHorizontalFrame(p=typeGB, opts=LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
                                pl=0, pr=0, pt=0, pb=0)
        self.radioBtns = {}
        self.radioBtns['Lay-up'] = FXRadioButton(p=hf1, text='Lay-up', tgt=self.switcher, sel=FXSwitcher.ID_OPEN_FIRST)
        self.radioBtns['Face sheets'] = FXRadioButton(p=hf1, text='Face sheets', tgt=self.switcher, sel=FXSwitcher.ID_OPEN_FIRST+1)
        getAFXAliasMap().setName(self.radioBtns['Lay-up'], 'radLayButtons')
        getAFXAliasMap().setName(self.radioBtns['Face sheets'], 'radFaceButtons')

        hf4 = FXHorizontalFrame(p=self.switcher, opts=LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
                                pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        label = FXLabel(p=hf4, text='Select region:', ic=None)
        w = FXButton(p=hf4, text='Select ply region...', ic=None, tgt=abdPlyProc, sel=AFXForm.ID_ACTIVATE)
        getAFXAliasMap().setName(w, 'ply')
        hf5 = FXHorizontalFrame(p=self.switcher, opts=LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
                                pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        label = FXLabel(p=hf5, text='Select region:', ic=None)
        w=FXButton(p=hf5, text='Select face region...', ic=None, tgt=abdFaceSheetsProc, sel=AFXForm.ID_ACTIVATE)
        getAFXAliasMap().setName(w, 'face')
        
        self.text = FXText(resultsGB, None, 0,
                      LAYOUT_FILL_X|LAYOUT_FILL_Y)#_WIDTH|LAYOUT_FIX_HEIGHT, 0, 0, 300, 100)
        self.text.setVisRows(20)
        self.text.setVisCols(20)
        self.text.setText(dummyText)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def onCmdCreate(self, sender, sel, ptr):
        if SELID(sel) == self.ID_CREATE:
            self.form.doCustomChecks()
            AFXForm.issueCommands(self.form, writeToReplay=True, writeToJournal=True) 
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def onCustomDataChanged(self):

        self.text.setText(session.customData.compositePluginData['text'].text)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def show(self):
        self.showContext = getCurrentContext()
        registerCurrentContext(self.callbackFunction)
        modelName = self.showContext['modelName']
        p = eval(self.showContext['objectPath'])
        partName = p.name
        session.customData.compositePluginData['text'].registerQuery(self.onCustomDataChanged)
        AFXDataDialog.show(self)
        self.text.setText('')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):
        
        #Cancel any procedures.
        # Seems we have a bug if procedure is called more than once
        # the isActive check fails.  
        #
        if self.abdPlyProc.isActive():
            self.abdPlyProc.deactivate()
        if self.abdFaceSheetsProc.isActive():
            self.abdFaceSheetsProc.deactivate()
        
        session.customData.compositePluginData['text'].unregisterQuery(self.onCustomDataChanged)
        unregisterCurrentContext(self.callbackFunction)
        modelName = self.showContext['modelName']
        p = eval(self.showContext['objectPath'])
        partName = p.name
        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def callbackFunction(self):

        #Get sets, sections and layups associated with the part on show
        if getDisplayedObjectType() == PART:
            self.showContext = getCurrentContext()