from abaqusGui import *
from abq_AbdMatrix.abdSelectCoreDB import AbdSelectCoreDB

###########################################################################
# Class definition
###########################################################################

class AbdPlyProc(AFXProcedure):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        AFXProcedure.__init__(self, owner)
        self.pickedRegionCmd = AFXGuiCommand(mode=self, method='collectLayUpData',
                                             objectName='abq_AbdMatrix.abdKernel',
                                             registerQuery=False)
        self.pickedRegionKw = AFXObjectKeyword(command = self.pickedRegionCmd,
                                               name='region', isRequired=True,
                                               defaultValue='')
        self.unhighlight = 'None'

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):
        #Check part is being displayed in viewport and it has sections or layups
        msg = None
        if getDisplayedObjectType() == PART:
            msg = None
        else:
            msg = 'A part must be displayed in the current viewport.'
            
        if msg:
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(),msg)
        else:
            AFXProcedure.activate(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstStep(self):
        self.step1 = AFXPickStep(owner=self, 
                                 keyword=self.pickedRegionKw,
                                 prompt='Select a region',
                                 entitiesToPick=AFXPickStep.FACES|AFXPickStep.CELLS|AFXPickStep.ELEMENTS,
                                 numberToPick=ONE,
                                 highlightLevel=4)
        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getLoopStep(self):
        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def doCustomTasks(self):
        
        # Need to keep face, cell or element highlighted when going in to loop step.
        #
        cmdString = self.pickedRegionKw.getSetupCommands()
        highlight=cmdString+'highlight(pickedRegion)'
        sendCommand(self.unhighlight)
        
        # Remember what is highlighted so can unhighlight on cancel.
        #
        self.unhighlight =  cmdString+'unhighlight(pickedRegion)'
        sendCommand(highlight)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCancel(self):
        
        # Unhighlight previously selected region.
        #
        sendCommand(self.unhighlight)
        self.unhighlight = 'None'
        AFXProcedure.onCancel(self)


###########################################################################
# Class definition
###########################################################################

class AbdFaceSheetsProc(AFXProcedure):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        
        AFXProcedure.__init__(self, owner)
        
        self.pickedRegionCmd = AFXGuiCommand(mode=self, method='collectLayUpData',
                                             objectName='abq_AbdMatrix.abdKernel',
                                             registerQuery=False)
        self.pickedRegionKw = AFXObjectKeyword(command = self.pickedRegionCmd,
                                               name='region',
                                               isRequired=True, defaultValue='')
        self.coreSheetKw = AFXStringKeyword(command = self.pickedRegionCmd,
                                            name='core',
                                            isRequired=True, defaultValue=None)
        self.unhighlight = 'None'

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def activate(self):
        #Check a part is being displayed in viewport and it has sections or layups
        msg = None
        if getDisplayedObjectType() == PART:
            msg = None
        else:
            msg = 'A part must be displayed in the current viewport.'
            
        if msg:
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(),msg)
        else:
            self.coreSheetKw.setValueToDefault()
            AFXProcedure.activate(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def getFirstStep(self):
        self.coreSheetKw.setValueToDefault()
        self.step1 = AFXPickStep(owner=self, keyword=self.pickedRegionKw,
                                 prompt='Select a region',
                                 entitiesToPick=AFXPickStep.FACES|AFXPickStep.CELLS|AFXPickStep.ELEMENTS,
                                 numberToPick=ONE, highlightLevel=4)
        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def getNextStep(self, previousStep):
        
        if previousStep == self.step1:
            
            # Need to calculate details of ply before popping up dialog.
            #
            cmdString = self.pickedRegionCmd.getCommandString()
            try:
                sendCommand(cmdString)
                self.abdSelectCoreDB = AbdSelectCoreDB(self)
                self.step2 = AFXDialogStep(self, self.abdSelectCoreDB, 'Select Core Material')
                return self.step2
            except:
                return None

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def getLoopStep(self):
        self.coreSheetKw.setValueToDefault()
        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def doCustomTasks(self):
        cmdString = self.pickedRegionKw.getSetupCommands()
        highlight=cmdString+'highlight(pickedRegion)'
        sendCommand(self.unhighlight)
        self.unhighlight =  cmdString+'unhighlight(pickedRegion)'
        sendCommand(highlight)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                
    def onCancel(self):
        sendCommand(self.unhighlight)
        self.unhighlight = 'None'
        AFXProcedure.onCancel(self)
        

