from abaqusGui import *

###########################################################################
# Class definition
###########################################################################

class AbdSelectCoreDB(AFXDataDialog):

    # Dialog posted with list of plies in ply stack.
    # User to select the core material to allow face sheets calculation.
    #----------------------------------------------------------------------
    numInstance = 1
    def __init__(self, form):

        # Need to create list once kernel has worked out what plies are to be used.
        # A problem with kernelAccess as is seemed to be out of sync during the
        # procedure. However, pluginForm, used to populate the main dialog, is kept
        # up to date. Therefore, use attribute of plug-in form to create items in list
        # on this dialog.
        #
        AFXDataDialog.__init__(self, form, 'Select Core Material',
                               self.OK|self.CANCEL,
                               DIALOG_ACTIONS_SEPARATOR|DECOR_RESIZE)
        
        getAFXAliasMap().setPrefix(self, 'AbdSelectCoreDB-%d' % AbdSelectCoreDB.numInstance)
        nvis = 10
        vf = FXVerticalFrame(self, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y,
                             0, 0, 0, 0, 0, 0, 0, 0)
        self.plyList = AFXList(vf, nvis, tgt=form.coreSheetKw, sel=0,
                               opts=LIST_SINGLESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0)
        self.plyList.insertItem(0,30*' ')
        getAFXAliasMap().setName(self.plyList, 'plyList')
        
    def show(self):
        
        # Wanted to query custom data created when kernel script. Doesn't seem to stay in sync!
        # Use data that main plugin form accesses.
        #
        self.plyList.clearItems()
        from kernelAccess import session
        laminae = session.customData.compositePluginData['laminae'].laminae
        for l in laminae[0]:
            self.plyList.insertItem(0, l.plyName)
        AFXDataDialog.show(self)

            
    def hide(self):
        AFXDataDialog.hide(self)
