from abaqus import *
from abaqusConstants import *
import customKernel
from caeModules import *

def createAirfoilPart(modelName, fileName, numberOfPartitions):

    # Get viewport and model name
    #
    vpName = session.currentViewportName
    vp = session.viewports[vpName]
    model = mdb.models[modelName]
    
    # Create unique part name
    #
    i=1
    partName = 'Airfoil_%s' % i
    while partName in model.parts.keys():
        i+=1
        partName = 'Airfoil_%s' % i   

##
##      Read/Create the spline for blade from E-blade cloud point data 
##
    p = model.Part(name=partName, dimensionality=THREE_D, type=DEFORMABLE_BODY)
    p.ReferencePoint(point=(0.0, 0.0, 0.0))
    f_spn=open(fileName, 'r')
    FL_spn=f_spn.readlines()
    rose_data = []; sec=[]
    jflag=0; n_section=0; section_num=1.e10; n_point=[]; inum=0; ipoint=0

    print ' Data Information from Eblade'
    print ' '

    for line in FL_spn:
        FL = line.split()
        if (FL[0]=='Nsec'): 
            n_section=int(FL[1])
            print 'Total number of Section: ', n_section
        if (FL[0]=='SectionNo'): 
            section_num=int(FL[1])
            sec=[]
            inum=0
        if (FL[0]=='NP'): 
            n_point.append(int(FL[1]))
            ipoint = int(FL[1])
        if (section_num == jflag):
            dummy=[]
            dummy.append(float(FL[1]))
            dummy.append(float(FL[2]))
            dummy.append(float(FL[3]))
            sec.append(dummy)
            inum += 1
            if(inum == ipoint): rose_data.append(sec)   
            
        if (FL[0]=='PTNO'): 
            jflag += 1

    # Close file
    #
    f_spn.close()
    
    for kkk in range(n_section):
        p.WireSpline(points=rose_data[kkk], mergeWire=ON, meshable=ON, smoothClosedSpline=OFF)

##
##      Create Loft 
##

    e = p.edges
    NN = []
    total = len(e)

    for ii in range(total):
        aa = e[ii].pointOn
        #       Z direction is stacking direction.
        NN.append((float(aa[0][2]), ii))    

    KK = sorted(NN)

    loftsec =[]
    for ii in range(total):
        ii1 = KK[ii][1]
        loftsec.append((e[ii1], ))

    # Create solid feature between wire features
    #
    p.SolidLoft(loftsections=loftsec, startCondition=NONE, endCondition=NONE)

    if numberOfPartitions:

        numberOfPartitions = int(numberOfPartitions)                
##
##      Create the datum plane and partitions
##
        Min = int(KK[0][0]); Max = int(KK[total-1][0])
        step = int((Max-Min)/numberOfPartitions)
        for ii in range(numberOfPartitions):
            f = p.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=step*(ii+1)+Min)
            p.PartitionCellByDatumPlane(datumPlane=p.datums[f.id], cells=p.cells)

    # Show part
    #
    vp.setValues(displayedObject=p)
    vp.view.fitView()
