from abaqusGui import *
import abaqusConstants, os
from kernelAccess import session

###########################################################################
# Class definition
###########################################################################

class CreateAudioDB(AFXDataDialog):

    [
        ID_XYDATA_FILE,
        ID_WAVE_FILE,
    ] = range(AFXDataDialog.ID_LAST,AFXDataDialog.ID_LAST+2)
    
    numInstance = 0
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        CreateAudioDB.numInstance += 1
        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'Auralizing', self.OK|self.CANCEL,
                                DIALOG_ACTIONS_SEPARATOR)
       #GUILOG
        getAFXAliasMap().setPrefix(self, 'CreatAudio-%d'
                                          % CreateAudioDB.numInstance)
       #ENDGUILOG       

        # Map command to odb
        #
        FXMAPFUNC(self, SEL_COMMAND, self.ID_XYDATA_FILE, CreateAudioDB.onCmdFile)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_WAVE_FILE, CreateAudioDB.onCmdFile)


        # Model info
        #
        thisPath = os.path.abspath(__file__)
        thisDir = os.path.dirname(thisPath)
        icon = afxCreatePNGIcon(thisDir + r'/fileSelectSmall.png')

        self.form = form
        self.waveDataDb = None
        self.saveDataDb = None

        # Wave file options
        #
        hf = FXHorizontalFrame(p=self, x=0, y=0, w=0, h=0,
                               pl=0, pr=0, pt=0, pb=0,
                               hs=DEFAULT_SPACING, vs=DEFAULT_SPACING, opts=LAYOUT_FILL_X)
        w = AFXTextField(p=hf, ncols=12, labelText='Name:',
                        tgt=self.form.waveFileNameKw, sel=0,
                        opts=AFXTEXTFIELD_STRING|LAYOUT_CENTER_Y|LAYOUT_FILL_X)
        getAFXAliasMap().setName(w, 'Audio file name')
        w = FXButton(p=hf, text='', ic=icon, tgt=self, sel=self.ID_WAVE_FILE,
                 opts=BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_CENTER_Y|LAYOUT_RIGHT,
                 x=0, y=0, w=0, h=0, pl=1, pr=1, pt=1, pb=1)
        getAFXAliasMap().setName(w, 'Audio file name button')
        w = AFXTextField(p=self, ncols=11, labelText='Frame rate:',
                     tgt=self.form.frameRateKw, sel=0, opts=AFXTEXTFIELD_INTEGER|LAYOUT_FILL_X)
        getAFXAliasMap().setName(w, 'Audio file frame rate')
        from kernelAccess import session
        vp = session.viewports[session.currentViewportName]
        odb = session.odbs[vp.odbDisplay.name]

        for key in odb.steps.keys():

            if odb.steps[key].procedure == '*STEADY STATE DYNAMICS':
                w = AFXTextField(p=self, ncols=12, labelText='Total time:(Only for SSD)',
                         tgt=self.form.totalTimeKw, sel=0, opts=AFXTEXTFIELD_INTEGER|LAYOUT_FILL_X)
                getAFXAliasMap().setName(w, 'Audio file total time')
                break

        # Step
        #
        gb = FXGroupBox(p=self, text='Step', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        self.stepCombo = AFXComboBox(p=gb, ncols=0, nvis=1, text='SSD/Dynamic step:', tgt=self.form.stepKw, sel=0,
            opts=LAYOUT_FILL_X)
        self.stepCombo.setMaxVisible(10)
        getAFXAliasMap().setName(self.stepCombo, 'Audio file step')

        # Node
        #
        gb = FXGroupBox(p=self, text='Node', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        self.instCombo = AFXComboBox(p=gb, ncols=0, nvis=1, text='Instance:',
                                     tgt=self.form.instanceNameKw, sel=0, opts=AFXTEXTFIELD_INTEGER|LAYOUT_FILL_X)
        self.instCombo.setMaxVisible(10)
        getAFXAliasMap().setName(self.instCombo, 'Audio file instance')
        w = AFXTextField(p=gb, ncols=19, labelText='Node label:',
                     tgt=self.form.nodeLabelKw, sel=0, opts=LAYOUT_FILL_X)
        getAFXAliasMap().setName(w, 'Audio file node label')

        # Post-processing options
        #
        gb = FXGroupBox(p=self, text='Options', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        hf = FXHorizontalFrame(p=gb, x=0, y=0, w=0, h=0,
                               pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING, opts=LAYOUT_FILL_X)
        self.saveTxt = AFXTextField(p=hf, ncols=14, labelText='Save XYData to file:',
                     tgt=self.form.xyDataFileNameKw, sel=0,
                     opts=AFXTEXTFIELD_STRING|LAYOUT_CENTER_Y|LAYOUT_FILL_X|AFXTEXTFIELD_CHECKBUTTON|AFXTEXTFIELD_VERTICAL)
        getAFXAliasMap().setName(self.saveTxt, 'Audio file save name')
        w = FXButton(p=hf, text='', ic=icon, tgt=self, sel=self.ID_XYDATA_FILE,
                 opts=BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_BOTTOM|LAYOUT_RIGHT,
                 x=0, y=0, w=0, h=0, pl=1, pr=1, pt=1, pb=1)
        getAFXAliasMap().setName(w, 'Audio file save name button')
        w = FXCheckButton(p=gb, text='Play audio', tgt=self.form.playAudioKw, sel=0)
        getAFXAliasMap().setName(w, 'Audio file play audio')

        # Set combo boxes options
        #
        self.onUpdate()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdFile(self, sender, sel, ptr):

        # Post select File DB
        #
        if SELID(sel) == self.ID_WAVE_FILE:
            self.fileDb = self.waveDataDb
            patterns = 'Wave Files (*.wav)\nAll Files (*.*)'
            self.form.fileKw = self.form.waveFileNameKw
        else:
            self.fileDb = self.saveDataDb
            patterns = 'Text Files (*.txt)\nAll Files (*.*)'
            self.form.fileKw = self.form.xyDataFileNameKw

        if not self.fileDb:
            self.fileDb = AFXFileSelectorDialog(self, 'File Selection', 
                self.form.fileKw, None, 
                AFXSELECTFILE_ANY|AFXSELECTFILE_REMOTE_HOST, patterns)
            self.fileDb.create()
        self.fileDb.showModal()
        getAFXAliasMap().setPrefix(self.fileDb, 'Audio file open')
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):

        registerCurrentContext(self.onUpdate)
        AFXDataDialog.show(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        unregisterCurrentContext(self.onUpdate)
        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onUpdate(self):

        # Set combo boxes options
        #
        from kernelAccess import session
        vp = session.viewports[session.currentViewportName]
        odb = session.odbs[vp.odbDisplay.name]
        
        self.stepCombo.clearItems()
        self.form.stepKw.setValue('')
        for key in odb.steps.keys():
            if odb.steps[key].procedure == '*STEADY STATE DYNAMICS' or \
               odb.steps[key].procedure == '*DYNAMIC, EXPLICIT' or \
               odb.steps[key].procedure == '*DYNAMIC':
                self.stepCombo.appendItem(text=key)
        if self.stepCombo.getNumItems() > 0:
            self.stepCombo.insertItem(0, '_All Steps')
        
        self.instCombo.clearItems()
        self.form.instanceNameKw.setValue('')
        for key in odb.rootAssembly.instances.keys():
            self.instCombo.appendItem(text=key)


###########################################################################
# Class definition
###########################################################################

class AudioForm(AFXForm):
    [
        ID_OVERWRITE,
        ID_LAST
    ] = range(AFXForm.ID_LAST, AFXForm.ID_LAST+2)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        # Construct the base class.
        #
        AFXForm.__init__(self, owner)
        
        FXMAPFUNC(self, SEL_COMMAND, self.ID_OVERWRITE, AudioForm.onCmdOverwrite)
        
        # Command to createAudioFile
        #             
        self.cmd = AFXGuiCommand(self,'createAudioFile','abq_Auralizing.createAudioFile')
        self.stepKw = AFXStringKeyword(self.cmd, 'stepName', True, '')
        self.instanceNameKw = AFXStringKeyword(self.cmd, 'instanceName', True)
        self.nodeLabelKw = AFXIntKeyword(self.cmd, 'nodeLabel', True)
        self.frameRateKw = AFXFloatKeyword(self.cmd, 'frameRate', True, 2000.0)
        self.totalTimeKw = AFXFloatKeyword(self.cmd, 'totalTime', True, 4.0)
        self.waveFileNameKw = AFXStringKeyword(self.cmd, 'waveFileName', True)
        self.xyDataFileNameKw = AFXStringKeyword(self.cmd, 'xyDataFileName', True)
        self.playAudioKw = AFXBoolKeyword(self.cmd, 'playAudio', AFXBoolKeyword.TRUE_FALSE, 1)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):

        # Reimplemented to check if an ODB is displayed in the viewport or not
        # 
        from kernelAccess import session
        vp = session.viewports[session.currentViewportName]
        od = vp.odbDisplay
        odbName = ''
        try:
            odbName = od.name
        except:
            odbName = ''

        if not odbName:
            errMsg = 'Viewport is not associated with any ODB.'
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)
            return
        AFXForm.activate(self)
        return

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        # Show the dialog
        #
        self.firstDb = CreateAudioDB(self)
        return self.firstDb

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        # Method to check various input
        #
        db = self.getCurrentDialog()

        if not self.waveFileNameKw.getValue():
            errMsg = 'A wave file name must be specified.'
            showAFXErrorDialog(db, errMsg)
            return FALSE
        if not self.frameRateKw.getValue():
            errMsg = 'Frame rate must be specified.'
            showAFXErrorDialog(db, errMsg)
            return FALSE
        from kernelAccess import session
        vp = session.viewports[session.currentViewportName]
        odb = session.odbs[vp.odbDisplay.name]

        for key in odb.steps.keys():
            if not self.totalTimeKw.getValue() and odb.steps[key].procedure == '*STEADY STATE DYNAMICS':
                errMsg = 'Total time must be specified.'
                showAFXErrorDialog(db, errMsg)
                return FALSE
        if not self.stepKw.getValue():
            errMsg = 'A step must be specified.'
            showAFXErrorDialog(db, errMsg)
            return FALSE
        if not self.instanceNameKw.getValue():
            errMsg = 'An instance name must be specified.'
            showAFXErrorDialog(db, errMsg)
            return FALSE
        if not self.nodeLabelKw.getValue() or self.nodeLabelKw.getValue() <= 0:
            errMsg = 'A positive integer node label must be specified.'
            showAFXErrorDialog(db, errMsg)
            return FALSE
        if not db.saveTxt.getCheck():
            self.xyDataFileNameKw.setValue('')
        elif not self.xyDataFileNameKw.getValue():
            errMsg = 'XYData file name must be specified.'
            showAFXErrorDialog(db, errMsg)
            return FALSE

        return AFXForm.verifyKeywordValues(self)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        # Check if the files already exist.
        #
        errFiles = []
        
        wfn = self.waveFileNameKw.getValue()
        if wfn[-4:].lower() != '.wav':
            wfn += '.wav'
        
        if os.path.isfile(wfn):
            errFiles.append(wfn)
        
        xyfn = self.xyDataFileNameKw.getValue()
        
        if os.path.isfile(xyfn):
            errFiles.append(xyfn)
        
        if errFiles:
            db = self.getCurrentDialog()
            errorMsg = 'The file(s) %s already exist. Ok to overwrite?' % errFiles        
            showAFXWarningDialog(db, errorMsg,AFXDialog.YES|AFXDialog.NO, 
                                 self, self.ID_OVERWRITE)
            return FALSE
                            
        return TRUE
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdOverwrite(self, sender, sel, ptr):

        # If user allowed overwrite, then continue, otherwise just return
        #
        if sender.getPressedButtonId() == AFXDialog.ID_CLICKED_YES:
            AFXForm.issueCommands(self)
        return TRUE