from abaqus import *
import wave
import math
try:
    import numpy.oldnumeric as N
except:
    import Numeric as N
from abaqusConstants import *
import struct
import os


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createAudioFile(stepName, instanceName, nodeLabel, frameRate, totalTime, waveFileName, xyDataFileName=None, playAudio= TRUE):

    # Method that create the Audio file
    #
    vp = session.viewports[session.currentViewportName]
    odbName = vp.odbDisplay.name
    
    if not odbName:
        errMsg = 'The current viewport is not associated with an output databse.'
        raise ValueError, errMsg
    
    odb = session.odbs[odbName]    
    
    if not odb.rootAssembly.instances.has_key(instanceName):
        raise KeyError, 'The odb does not have instance named: "%s"."' % instanceName
    
    instance = odb.rootAssembly.instances[instanceName]
    try:
        node = instance.getNodeFromLabel(label=nodeLabel)
    except:
        raise ValueError, 'Node %s not found' % nodeLabel
    
    if waveFileName[-4:].lower() != '.wav':
            waveFileName += '.wav'

    # List to store sequence of (frequency, mag, phi),
    #
    dataList = []
    numValues = 0
    sumList = []
    sumList_vol = []

    # List to keep steps
    #
    stepNames = []
    
    if stepName == '_All Steps':
        for stepKey in odb.steps.keys():
            if odb.steps[stepKey].procedure == '*STEADY STATE DYNAMICS' or \
              odb.steps[stepKey].procedure == '*DYNAMIC, EXPLICIT' or \
              odb.steps[stepKey].procedure == '*DYNAMIC':
                stepNames.append(stepKey)
        if len(stepNames) == 0:
            raise ValueError, 'No ssd or dynamic step found.'

        # At least one step found.  Take first type and filter for only this type
        #
        stepProc = odb.steps[stepNames[0]].procedure
        stepNames = [s for s in stepNames if odb.steps[s].procedure == stepProc]
    else:
        stepNames.append(stepName)

    ssdProc = 0

    for stepName in stepNames:

        # Check step for *SSD
        #
        if not odb.steps.has_key(stepName):
            raise KeyError, 'The odb does not have step named: "%s"."' % step

        step = odb.steps[stepName]
        print 'Processing step: "%s"...' % step.name

        # Check if SSD
        #
        if step.procedure != '*STEADY STATE DYNAMICS' and step.procedure != '*DYNAMIC, EXPLICIT'and step.procedure !='*DYNAMIC':
            raise ValueError, 'The step "%s" is not a steady state dynamic step, ' \
                              'or a dynamic (explicit) step or dynamic(implicit)' % step.name

        # Process all frames of this step
        #
        if step.procedure == '*STEADY STATE DYNAMICS':
            ssdProc = 1
            for frame in step.frames:
                numValues += 1
                frequency = frame.frequency
                if not frame.fieldOutputs.has_key('POR'):
                    raise KeyError, 'The frame with frequency %f does not have POR field output.' % frequency

                fo = frame.fieldOutputs['POR'].getSubset(region=node)
                if len(fo.values) == 0:
                    raise ValueError,' The node "%s".%d does not have output for frame with frequency %f.' % (instanceName, nodeLabel, frequency)

                value = fo.values[0]
                if value.precision == SINGLE_PRECISION:
                    real = value.data
                    imag = value.conjugateData
                else:
                    real = value.dataDouble
                    imag = value.conjugateDataDouble

                mag = ((real*real) + (imag*imag)) ** 0.5
                try:
                    phi = atan2(imag,real)
                except:
                    phi = 0.0
                dataList.append((frequency, mag, phi))
                pmax = -1.e32
                pmin = 1.e32
        
        #Get history data for POR. Calculate sumList by calling 'interpolatePOR'function for *Dynamic,
        #which will be written to the WAV file
        #        
        elif step.procedure == '*DYNAMIC':
            hrName = 'Node %s.%s' % (instanceName, nodeLabel)
            #Checking if history output has POR
            #
            if step.historyRegions.has_key(hrName):
                hr = step.historyRegions[hrName]            
                sumList1 = hr.historyOutputs['POR'].data
                endPoint = sumList1[-1][0]

                if frameRate > len(sumList1):
                    raise ValueError, 'Frame rate greater than '+str(len(sumList1))+' is not allowed.'

                if frameRate==1.:
                    raise ValueError,'Frame rate cannot be '+ str(frameRate) +'.'
                deltaT = 1.0/frameRate
                sumList= interpolatePOR(sumList1, deltaT, endPoint, frameRate)            
            #Checking if field output has POR
            #            
            else: 
                try:
                    for frame in step.frames:
                        numValues += 1
                        time = frame.frameValue
                        if not frame.fieldOutputs.has_key('POR'):
                            raise KeyError, 'The frame with Time %f does not have POR field output.' % time

                        fo = frame.fieldOutputs['POR'].getSubset(region=node)
                        if len(fo.values) == 0:
                            raise ValueError,' The node "%s".%d does not have output for frame with Time %f.' % (instanceName, nodeLabel, time)

                        value = fo.values[0]
                        if value.precision == SINGLE_PRECISION:
                            real = value.data

                        else:
                            real = value.dataDouble
                        dataList.append((time, real))
                    sumList1=dataList         
                    if frameRate > len(sumList1):
                        raise ValueError, 'Frame rate greater than '+str(len(sumList1))+' is not allowed.'

                    if frameRate==1.:
                        raise ValueError,'Frame rate cannot be '+ str(frameRate) +'.'                    
                    endPoint = sumList1[-1][0]
                    deltaT = 1.0/frameRate
                    sumList=interpolatePOR(sumList1,deltaT, endPoint, frameRate) 

                except:
                    raise ValueError, "ODB does not have POR as history output or field output"
        #Get history data or field output data for PORCalculate sumList by calling 'interpolatePOR'function for *Dynamic,
        #Explicit, which will be written to the WAV file
        #
            
        
        elif step.procedure == '*DYNAMIC, EXPLICIT':
            hrName = 'Node %s.%s' % (instanceName, nodeLabel)
            #Checking if history output has POR
            #            
            if step.historyRegions.has_key(hrName):
                hr = step.historyRegions[hrName]            
                sumList1 = hr.historyOutputs['POR'].data
                endPoint = sumList1[-1][0]

                if frameRate > len(sumList1):
                    raise ValueError, 'Frame rate greater than '+str(len(sumList1))+' is not allowed.'

                if frameRate==1.:
                    raise ValueError,'Frame rate cannot be '+ str(frameRate) +'.'
                deltaT = 1.0/frameRate
                sumList= interpolatePOR(sumList1, deltaT, endPoint, frameRate)            
            #Checking if field output has POR
            #                 
            else: 
                try:
                    for frame in step.frames:
                        numValues += 1
                        time = frame.frameValue
                        if not frame.fieldOutputs.has_key('POR'):
                            raise KeyError, 'The frame with Time %f does not have POR field output.' % time

                        fo = frame.fieldOutputs['POR'].getSubset(region=node)
                        if len(fo.values) == 0:
                            raise ValueError,' The node "%s".%d does not have output for frame with Time %f.' % (instanceName, nodeLabel, time)

                        value = fo.values[0]
                        if value.precision == SINGLE_PRECISION:
                            real = value.data

                        else:
                            real = value.dataDouble

                        dataList.append((time, real))
                    sumList1=dataList
                    if frameRate > len(sumList1):
                        raise ValueError, 'Frame rate greater than '+str(len(sumList1))+' is not allowed.'

                    if frameRate==1.:
                        raise ValueError,'Frame rate cannot be '+ str(frameRate) +'.'            
                    endPoint = sumList1[-1][0]
                    deltaT = 1.0/frameRate
                    sumList=interpolatePOR(sumList1,deltaT, endPoint, frameRate) 

                except:
                    raise ValueError, "ODB does not have POR as history output or field output"
                        


    # Now calculate the sumList, pmax and pmin for SSD and for *Dynamic and
    # *Dynamic Explicit calculate    

    
    if ssdProc:
        deltaT = 1.0/frameRate 
        numDataPts = int(totalTime/deltaT)
        print 'Number of data points (bytes) = %d' % numDataPts
        count=0
        time=0.0        
        while (count<numDataPts):
            sum = 0.0
            for frameIndex in range(numValues):
                freq = dataList[frameIndex][0]
                mag = dataList[frameIndex][1]
                phi = dataList[frameIndex][2]
                omega = 2.0*pi*freq
                sum += mag*cos(omega*time + phi)

            sumList.append((time,sum))
            if (sum>pmax):
                pmax = sum
            elif (sum<pmin):
                pmin = sum
            count += 1
            time += deltaT
        
    
    else:

        count=len(sumList)        
        pmax = max([abs(i[1]) for i in sumList])
        pmin = min([i[1] for i in sumList])

    if pmax < abs(pmin):
        pmax = abs(pmin)
        
    del dataList
    
    # Scale the values for the wave file
    #
    waveFile = wave.open(waveFileName, 'wb')
   # waveFile.setnchannels(1)
   # waveFile.setsampwidth(1)
   # waveFile.setframerate(frameRate)
   # waveFile.setnframes(len(sumList))
   # waveFile.setcomptype('NONE', 'not compressed')
    waveFile.setparams((1,2,frameRate, len(sumList),'NONE','noncompressed'))
    volume = float(32767/pmax)
    sumListY = [s[1] for s in sumList] 
    signal = volume*N.resize(sumListY, (len(sumListY),))
#    signal = N.resize(sumListY, (len(sumListY),))
                          
    ssignal = ''.join((wave.struct.pack('h', item) for item in signal))

    waveFile.writeframes(ssignal)
    waveFile.close()

    # Plot XYData in a new viewport
    #
        
    name = 'Pressure Plot'
    xyData = session.XYData(name = name, data = sumList)
    
    if name in session.xyPlots.keys():
        xyp = session.xyPlots[name]
    else:
        xyp = session.XYPlot(name)

    chartName = xyp.charts.keys()[0]
    chart = xyp.charts[chartName]
    c1 = session.Curve(xyData=xyData)
    chart.setValues(curvesToPlot=(c1, ), )
    chart.axes1[0].axisData.setValues(useSystemTitle=False, title='Time')
    chart.axes2[0].axisData.setValues(useSystemTitle=False, title='Pressure')

    vp.setValues(displayedObject=xyp)
    
    # Save XYData
    #
    if xyDataFileName:
        xyFile = open(xyDataFileName, 'wb')
        for item in sumList:
            xyFile.write('%e %e\n' % (item[0], item[1]))

        xyFile.close()

    # Play Audio
    #
    if playAudio:
        os.system(waveFileName)


def interpolatePOR(sumList1, deltaT, endPoint, frameRate):

    #Function that finds interpolated Y value of history data pair, where deltaT is
    #interpolated X value  
    #
    iterSumList=[(0,0)]
    iterList = iter(sumList1)
    a = iterList.next()
    b = iterList.next()
    while deltaT <= endPoint:
        if deltaT >= a[0] and deltaT <= b[0]:
            Yint=a[1]+(deltaT-a[0])*((b[1]-a[1])/(b[0]-a[0]))
            iterSumList.append((deltaT,Yint))
            deltaT+=1./frameRate
        else:
            a=b

            if b==endPoint:
                continue
            else:
                b=iterList.next()
    sumList=iterSumList    
    return(sumList)
