from abaqusGui import *

###########################################################################
# Class definition
###########################################################################

class CreateEditForm(AFXForm):
                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, objectDB, mode):
    
        self.objectDB = objectDB
        self.mode = mode
                                
        AFXForm.__init__(self, owner)
        
        if mode == 'EDIT':
            self.cmd = AFXGuiCommand(self, 'setValues', '', True)
            self.required = False
        else:
            self.required = True

        # The repository and baseName get set later by the managerForm
        # via the setValues method in this class.
        #
        self.repository = None
        self.baseName = None
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def setValues(self, repository, baseName):

        # This method is called by managerForm when it is constructed.
        
        self.repository = repository
        self.baseName = baseName
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getDefaultName(self):
    
        # This method returns the next available default name for a
        # a repository, based on the baseName and number of objects
        # in the repository.
        from kernelAccess import mdb, session
        exec('keys = %s.keys()' % self.repository )
        index = len(keys) + 1
        defaultName = '%s-%d' % (self.baseName, index)
        while defaultName in keys:
            index += 1
            defaultName = '%s-%d' % (self.baseName, index)
        return defaultName

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def setName(self, name):
    
        # This method is called by managerDB when the user selects
        # an object to edit.
        
        self.cmd.setObjectName("%s['%s']" % (self.repository, name) )
        self.nameKw.setValue(name)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        db = self.objectDB(self) 
        if self.mode == 'CREATE':
            self.cmd.setKeywordValuesToDefaults()
            self.nameKw.setValue( self.getDefaultName() )
            
        elif self.mode == 'EDIT':
            title = db.getTitle().replace('Create', 'Edit')
            db.setTitle(title)
            db.nameText.setReadOnlyState()
        return db
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):
    
        if self.nameKw.getValue().strip() == '':
            showAFXErrorDialog(self.getCurrentDialog(),
                'A name must be entered.')
            self.getCurrentDialog().onKeywordError(self.nameKw)
            return False
            
        return True
