from abaqusGui import *
from abq_CaeManagerTemplate.renameForm import RenameForm
from abq_CaeManagerTemplate.copyForm import CopyForm

###########################################################################
# Class definition
###########################################################################

class ManagerDB(AFXDataDialog):

    [
        ID_CREATE,
        ID_EDIT,
        ID_COPY,
        ID_RENAME,        
        ID_DELETE,
        ID_DELETE_WARNING,
        ID_TABLE,
        ID_EXECUTE_FUNCTION,
        ID_LAST
    ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+9)
    numInstance = 0

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form, repository, title, leadingRowLabels='Name'):

        ManagerDB.numInstance+=1
        self.form = form
        self.repository = repository
        self.oldSummary = []
        self.oldKeys = []
        self.doubleClickFunction = self.ID_EDIT
        
        self.form.copyForm = CopyForm(self.form.getOwner(), self, self.repository)
        self.form.renameForm = RenameForm(self.form.getOwner(), self, self.repository)
        
        AFXDataDialog.__init__(self, form, title, 0, 
            DIALOG_UNPOST_DESTROY|DECOR_RESIZE|DIALOG_ACTIONS_RIGHT)

        # Set the prefix
        #
        getAFXAliasMap().setPrefix(self, 'caeManagerTemplateManagerDB-%d' % ManagerDB.numInstance)

        # Message map
        #
        FXMAPFUNC(self, SEL_SELECTED, self.ID_TABLE, ManagerDB.onCmdTable)
        
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CREATE, ManagerDB.onCheckActiveFunctions)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_EDIT, ManagerDB.onCheckActiveFunctions)
        FXMAPFUNC(self, SEL_DOUBLECLICKED, 0, ManagerDB.onCheckActiveFunctions)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_COPY, ManagerDB.onCheckActiveFunctions)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_RENAME, ManagerDB.onCheckActiveFunctions)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_DELETE, ManagerDB.onCheckActiveFunctions)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_DELETE_WARNING, ManagerDB.onDeleteWarning)
        
        FXMAPFUNC(self, SEL_COMMAND, self.ID_EXECUTE_FUNCTION, ManagerDB.executeFunction)
                
        # Table
        #
        hf = FXHorizontalFrame(self, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK|FRAME_SUNKEN,
            0,0,0,0, 0,0,0,0)
        self.numColumns = len(leadingRowLabels.split('\t'))
        self.table = AFXTable(hf, 1, self.numColumns, 1, self.numColumns, 
            self, self.ID_TABLE, LAYOUT_FILL_X|LAYOUT_FILL_Y|AFXTABLE_RESIZE|AFXTABLE_ROW_MODE)
        self.table.setLeadingRows(1)
        self.table.setLeadingRowLabels(leadingRowLabels)
        self.table.setColumnWidth(0,150)
        self.table.setStretchableColumn(self.numColumns-1)
                
        # Action area
        #
        self.createBtn = self.appendActionButton('Create...', self, self.ID_CREATE)
        self.editBtn = self.appendActionButton('Edit...', self, self.ID_EDIT)
        self.copyBtn = self.appendActionButton('Copy...', self, self.ID_COPY)
        self.renameBtn = self.appendActionButton('Rename...', self, self.ID_RENAME)
        self.deleteBtn = self.appendActionButton('Delete...', self, self.ID_DELETE)
        dismissBtn = self.appendActionButton(self.DISMISS)

        # Name the action buttons and table
        #
        getAFXAliasMap().setName(self.createBtn, 'Create')
        getAFXAliasMap().setName(self.editBtn, 'Edit')
        getAFXAliasMap().setName(self.copyBtn, 'Copy')
        getAFXAliasMap().setName(self.renameBtn, 'Rename')
        getAFXAliasMap().setName(self.deleteBtn, 'Delete')
        getAFXAliasMap().setName(dismissBtn, 'Dismiss')
        getAFXAliasMap().setName(self.table, 'Table')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def cmdCopy(self, name):
    
        self.form.copyForm.setValues(name)
        self.form.copyForm.activate()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def cmdCreate(self):

        self.form.createForm.activate()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getDefaultWidth(self):
    
        # Set an initial width to the dialog, otherwise the default will
        # be too narrow.
        #
        return 250

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def cmdDelete(self):

        showAFXWarningDialog(self, 'OK to delete the selected items?',
            self.YES|self.NO, self, self.ID_DELETE_WARNING)
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def cmdEdit(self, name):
    
        self.form.editForm.setName(name)
        self.form.editForm.activate()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def cmdRename(self, name):
    
        self.form.renameForm.setValues(name)
        self.form.renameForm.activate()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def executeFunction(self, sender, sel, ptr):
    
        # If the previous function had a bailout, and the user chose to stay
        # in that function, then we do not want to activate the new function.
        #
        # A client could decide to not offer one of the management functions,
        # so check if they exist before checking if they are still active.
        
        if (self.form.createForm and self.form.createForm.isActive() ) or \
           (self.form.editForm   and self.form.editForm.isActive() ) or \
           (self.form.copyForm   and self.form.copyForm.isActive() ) or \
           (self.form.renameForm and self.form.renameForm.isActive() ):
                return 1

        selections = self.getSelections()
        if selections:
            name = selections[0]

        # Call individual functions instead of activating the approrpriate
        # form in this function so that clients have the option of overwriting
        # the individual functions to specialize their behavior.
        #
        if self.selectedFunctionId == self.ID_CREATE:
           self.cmdCreate()
        elif self.selectedFunctionId == self.ID_EDIT:
           self.cmdEdit(name)
        elif self.selectedFunctionId == self.ID_COPY:
           self.cmdCopy(name)
        elif self.selectedFunctionId == self.ID_RENAME:
           self.cmdRename(name)
        elif self.selectedFunctionId == self.ID_DELETE:
           self.cmdDelete()
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getSelections(self):
    
        # Returns a list of names of the selected items in the manager.
        
        numRows = self.table.getNumRows()
        selections = []
        for i in range(1, numRows):
            if self.table.isItemSelected(i, 0):
                selections.append(self.table.getItemText(i,0))

        return selections

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getSummary(self):

        from kernelAccess import mdb, session
        self.repository = self.form.cmd.getExpandedObjectName()

        # Update the form repositories
        #
        self.form.createForm.repository = self.repository
        self.form.copyForm.repository = self.repository
        self.form.renameForm.repository = self.repository
        self.form.editForm.repository = self.repository


        exec('keys = %s.keys()' % self.repository)
        keys.sort()
        return [ [key] for key in keys]

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        unregisterCurrentContext(self.onUpdate)
        exec('%s.unregisterQuery(self.onUpdate)' % self.repository)
        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCheckActiveFunctions(self, sender, sel, ptr):
    
        self.selectedFunctionId = SELID(sel)
        if self.selectedFunctionId == 0:
            # User double-clicked in the table
            self.selectedFunctionId = self.doubleClickFunction
        
        # Cancel any currently running functions.
        # A client could decide to not offer one of the management functions,
        # so check if they exist before checking if they are active.
        #
        if self.form.createForm and self.form.createForm.isActive():
            self.form.createForm.cancel(self, MKUINT(self.ID_EXECUTE_FUNCTION, SEL_COMMAND))
            
        elif self.form.editForm and self.form.editForm.isActive():
            self.form.editForm.cancel(self, MKUINT(self.ID_EXECUTE_FUNCTION, SEL_COMMAND))

        elif self.form.copyForm and self.form.copyForm.isActive():
            self.form.copyForm.cancel(self, MKUINT(self.ID_EXECUTE_FUNCTION, SEL_COMMAND))
            
        elif self.form.renameForm and self.form.renameForm.isActive():
            self.form.renameForm.cancel(self, MKUINT(self.ID_EXECUTE_FUNCTION, SEL_COMMAND))
            
        else:
            self.executeFunction(sender, sel, ptr)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdTable(self, sender, sel, ptr):
        
        numSelections = len(self.getSelections())
        self.setButtonStates(numSelections)
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onUpdate(self):

        # Check to see if there are any changes since the 
        # dialog was last posted.
        #
        summary = self.getSummary()
        if summary == self.oldSummary:
            return
            
        keys = [items[0] for items in summary]
            
        # Clear the table, 
        # then add the keys and select the appropriate items.
        #
        numRows = self.table.getNumRows()
        if numRows > 1:
            self.table.deleteRows(1, numRows-1)
        
        numSelections = 0
        if keys:
            self.table.insertRows(1, len(keys))
            i = 1
            for items in summary:
                for j in range( self.numColumns ):
                    self.table.setItemText(i, j, items[j])
                i += 1
                    
            if len(keys) == numRows or \
               len(keys) == numRows-1:    # New item or renamed item
            
                i = 1
                for key in keys:
                    if key not in self.oldKeys:
                        self.table.selectRow(i)
                        numSelections += 1
                        break
                    i += 1

            elif len(keys) < numRows-1:   # Deleted item
            
                self.table.selectRow(1)
                numSelections += 1
                
        if numSelections==0 and keys:
            self.table.selectRow(1)
            numSelections = numSelections + 1
                            
        self.oldSummary = summary
        self.oldKeys = keys
        self.setButtonStates(numSelections)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onDeleteWarning(self, sender, sel, ptr):
    
        selections = self.getSelections()
        btnPressed = sender.getPressedButtonId()
        if btnPressed == self.ID_CLICKED_YES:
            for selection in selections:
                cmd = "del %s['%s']" % (self.repository,selection)
                sendCommand(cmd)
        return 1
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def setButtonStates(self, numSelections):
    
        # Set action area button states based on
        # whether one or more item is selected.
        #
        if numSelections == 0:
            self.editBtn.disable()
            self.copyBtn.disable()
            self.renameBtn.disable()
            self.deleteBtn.disable()
            
        elif numSelections == 1:
            self.editBtn.enable()
            self.copyBtn.enable()
            self.renameBtn.enable()
            self.deleteBtn.enable()
        
        elif numSelections > 1:
            self.editBtn.disable()
            self.copyBtn.disable()
            self.renameBtn.disable()
            self.deleteBtn.enable()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
    
        numSelections = len(self.getSelections())
        self.setButtonStates(numSelections)
        registerCurrentContext(self.onUpdate)
        exec('%s.registerInclusive(self.onUpdate)' % self.repository)
        AFXDataDialog.show(self)