"""
This script will create the DIALOG for the Campbell Plug-In
"""
from abaqusGui import *
import kernelAccess, os,string


from abq_CampbellDiagram.setConstants import (
    PLUGIN_NAME, BLK2, DEBUG_RUN_MODE, REF_CURVE_LIST,
    USE_MASS_STATE, USE_ALL_MODES_STATE, USE_NSET_STATE)

_defaultStepItems = [[0, '', '']]

# be verbose if:
_verbose = (DEBUG_RUN_MODE >= 1)
_verboseDev = (DEBUG_RUN_MODE >= 2)
_namePrefix = __name__.split('.')[-1]
_verboseBase = BLK2+_namePrefix+':'

# Colors
#
from abaqusGui import FXRGB
clr_Black       = FXRGB(0,0,0)
clr_Ocker       = FXRGB(255,222,173)
clr_Pink        = FXRGB(255,192,203) #"Pink"
clr_LightBlue   = FXRGB(173,216,230) #"LightBlue"
clr_Blue        = FXRGB(200,230,255) #"Blue"
clr_NavajoWhite = FXRGB(255,222,173) #"NavajoWhite"
clr_GreenYellow = FXRGB(173,230,80)  #"GreenYellow" modified.
clr_Violett     = FXRGB(190,160,230) # modified.
clr_Grey80      = FXRGB(204,204,204) # "Grey80".
clr_Grey88      = FXRGB(224,224,224) # "Grey88".
clr_Grey91      = FXRGB(232,232,232) # "Grey91".
clr_SeaYellow   = FXRGB(235,235,100) # "sea yellow".

###########################################################################
# Class definition
###########################################################################

class CampbellDB(AFXDataDialog):

    [   ID_CLICKED_CREATE,
        ID_SELECT_MODES,
        ID_SELECT_NSET,
        ID_CHECK_TABLE_BTN,
        ID_REF_MULTIPLIER,
        ID_UPDATE_PLOT,
        ID_LAST,
        
    ] = range(AFXDialog.ID_LAST, AFXDialog.ID_LAST+7)


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'Plot '+ PLUGIN_NAME, 0,
                               DECOR_RESIZE|DIALOG_ACTIONS_SEPARATOR)

        self.appendActionButton('Plot', self, self.ID_CLICKED_CREATE)
        self.appendActionButton('Dismiss', self, self.ID_CLICKED_DISMISS)

        self.form = form
        
        # We have two variables to keep track of the odb name
        # odbName should be used when an odb is shown.
        # plottedOdbName should be used when XY plot is shown
        # 
        self.odbName = None
        self.plottedOdbName = None
        
        # Variable to keep track if user going from XY plot
        # to Odb
        #
        self.plotted = 0
        
        
        self.currentContext = {}
        #
        self._steps = {'usedSteps':{},
                       'filledOnce': False}
        self._modes = {}
        self._nodeSet = {}
        self._keywords = {}
        self.refMultiplier = []
        self._lastActiveStep = None
        #
        self.moduleImported = False
        self.fileNameTgt = AFXStringTarget()

        # MAP the select FX button to the method of CLASS FastenerDB.onCmdSelect
        #
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CLICKED_CREATE,    CampbellDB.onCmdOkCreate)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_SELECT_MODES,      CampbellDB.onCmdSelectModes)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_SELECT_NSET,       CampbellDB.onCmdSelectNset)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CHECK_TABLE_BTN,   CampbellDB.onCmdTableChk)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_REF_MULTIPLIER,    CampbellDB.onCmdRefMultiplier)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_UPDATE_PLOT,       CampbellDB.onCmdUpdatePlot)
        

        #GUILOG
        getAFXAliasMap().setPrefix(self, 'Campbell Diagram')
        self.createBtn = self.getActionButton(self.ID_CLICKED_CREATE)
        getAFXAliasMap().setName(self.createBtn, 'Create')
        #ENDGUILOG
        #
        # Disable btn initially. Will be enabled, if odb is read.
        self.createBtn.disable()

        vfMain = FXVerticalFrame(self, FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y,
                                 0,0,0,0,0,0,0,0,0,0)

        vf = FXVerticalFrame(vfMain, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK|FRAME_SUNKEN,
                                 0,0,0,0,0,0,0,0,0,0)                                                                  
        #
        # Insert table to display and select steps
        stepLabels = 'Select\tStep Name\tSpin Speed'
        numRows = 2
        visibleRows = 8
        numColumns = 3
        stepTable = AFXTable(vf, visibleRows, numColumns, 
                             numRows, numColumns, 
                             tgt=self, sel=CampbellDB.ID_CHECK_TABLE_BTN, 
                             opts=AFXTABLE_EDITABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        
        
        #
        # store all data relativ to step table in one dictionary
        self._steps['numRows'] = numRows
        self._steps['numColumns'] = numColumns
        self._steps['table'] = stepTable
        
        #
        # Add mode list box
        hf = FXHorizontalFrame(vfMain, pl=0, pr=0, pb=0, opts=FRAME_NONE|LAYOUT_FILL_X)
        modeChk = FXCheckButton(hf, 'Plot all modes'+
                                '\tIf a reduced number of modes eg. 10 is used, '+
                                '\ncorrelation will only be searched in these '+
                                '\nmodes. If the correlation looked strange '+
                                '\nthe number of modes should be increased!',
                                tgt=self, sel=CampbellDB.ID_SELECT_MODES,
                                opts=CHECKBUTTON_NORMAL|LAYOUT_CENTER_Y)
        # update combo later on.
        modeCombo = AFXComboBox(hf, ncols=3, nvis=15, text='or select number of modes to plot',
                                tgt=None, sel=0, pt=0,
                                opts=LAYOUT_CENTER_Y)
        #
        self._modes['chkBtn'] = modeChk
        self._modes['modeCombo'] = modeCombo
        #
        # Add nodeSet list box
        hf = FXHorizontalFrame(vfMain, pl=0, pr=0, pb=0, opts=FRAME_NONE|LAYOUT_FILL_X)
        nodeSetChk = FXCheckButton(hf, 'Use node set only'+
                                   '\tSelect node set for evaluation.'+
                                   '\nNode sets are only available from '+
                                   '\ninstance "PART-1-1" in the current'+
                                   '\nrelease.',
                                   tgt=self, sel=CampbellDB.ID_SELECT_NSET,
                                   opts=CHECKBUTTON_NORMAL|LAYOUT_CENTER_Y)
        nodeSetCombo = AFXComboBox(hf, ncols=0, nvis=5, text='',
                                   tgt=self.form.nodeSetKw, sel=0,
                                   opts=LAYOUT_CENTER_Y)
        
        
        #
        self._nodeSet['chkBtn'] = nodeSetChk
        self._nodeSet['nsetCombo'] = nodeSetCombo
        self._nodeSet['setNames'] = []
        #
        self.massChk = FXCheckButton(vfMain, 'Include mass matrix\tif available',
                                     tgt=self.form.useMassKw, sel=0)
        self.form.useMassKw.setValue(USE_MASS_STATE)
        #
        hf01 = FXHorizontalFrame(vfMain, FRAME_NONE|LAYOUT_FILL_X,
                                0,0,0,0,0,0,0,0,0,0)
        self.refFrameCombo=AFXComboBox(p=hf01, ncols=6, nvis=2, text='Reference frame', tgt=self.form.refFrameKw, sel=0)
                              
        #hf1 = FXHorizontalFrame(vfMain, FRAME_NONE|LAYOUT_FILL_X,
        #                        0,0,0,0,0,0,0,0,0,0)
        self.dirnCombo=AFXComboBox(p=hf01, ncols=3, nvis=4, text='Spin axis direction', tgt=self.form.dirnKw, sel=0)                                        
        
        # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
        # Initialize some objects:
        self.initCustomData()
        self.readCustomData()
        #
        self.initStepTable(stepLabels)
        #
        self.initNsetCombo()
        self.switchNsetCombo(active=USE_NSET_STATE)
        #
        self.initModeCombo()
        self.switchModeCombo(active=USE_ALL_MODES_STATE)
        #
        # Output group box
        # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
        outputGb = FXGroupBox(p=vfMain, text='Plot Options',
                              opts=FRAME_GROOVE|LAYOUT_FILL_X)
        """hf01 = FXHorizontalFrame(outputGb, FRAME_NONE|LAYOUT_FILL_X,
                                0,0,0,0,0,0,0,0,0,0)
        self.refFrameCombo=AFXComboBox(p=hf01, ncols=6, nvis=2, text='Reference frame', tgt=self.form.refFrameKw, sel=0)"""
                              
        hf1 = FXHorizontalFrame(outputGb, FRAME_NONE|LAYOUT_FILL_X,
                                0,0,0,0,0,0,0,0,0,0)
        #self.dirnCombo=AFXComboBox(p=hf1, ncols=3, nvis=2, text='Spin axis direction', tgt=self.form.dirnKw, sel=0)                                
        self.xyPrefixTf = AFXTextField(hf1, 20, '  XY Data prefix'+
                                       '\tEnter string to be added as start prefix'+
                                       '\nfor all curves in the later diagram.',
                                       tgt=self.form.xyPrefixKw, sel=0,
                                       opts=AFXTEXTFIELD_STRING)
        self.updatePlotButton = FXButton(p=hf1, text='Redraw Plot Options...'+
                                        '\tUpdate the current XY-Plot'+
                                        '\nbased on modified settings!',
                                        tgt=self, sel=self.ID_UPDATE_PLOT,
                                        opts=BUTTON_NORMAL|LAYOUT_RIGHT)
        #
        # Add reference curve multiplier:
        hf2 = FXHorizontalFrame(outputGb, FRAME_NONE,
                                0,0,0,0,0,0,0,0,0,0)
        FXLabel(hf2, 'Create reference curve(s):'
                +'\tmultiply whirl frequency with factor and'
                +'\ndisplay it versus spin speed.')
        for iRef, status in REF_CURVE_LIST:
            checkBtn = FXCheckButton(hf2, '%iX\tmultiplied with whirl vs speed'%iRef,
                                     tgt=self, sel=CampbellDB.ID_REF_MULTIPLIER)
            self.refMultiplier += [(iRef, checkBtn)]
            checkBtn.setCheck(status)
        #
        self.calcCritPtIntersectionChk = FXCheckButton(outputGb,
                                                       'Calculate intersection of curves to reference line(s)',
                                                       tgt=None,)
        self.calcCritPtIntersectionChk.setCheck(True)
        #
        
        #
        #GUILOG
        getAFXAliasMap().setName(self.updatePlotButton,  'Update XY-Plot')
        getAFXAliasMap().setName(self._steps['table'],   'Step Table')
        getAFXAliasMap().setName(self.massChk,           'Mass CheckButton')
        getAFXAliasMap().setName(self._modes['chkBtn'],  'Mode CheckButton')
        getAFXAliasMap().setName(self._modes['modeCombo'],'Mode ComboBox')
        getAFXAliasMap().setName(self._nodeSet['chkBtn'],   'Nset CheckButton')
        getAFXAliasMap().setName(self._nodeSet['nsetCombo'],'Nset ComboBox')
        getAFXAliasMap().setName(self.xyPrefixTf,        'XY Data Prefix')
        #
        # add map for multiplier
        for iRef, btn in self.refMultiplier:
            getAFXAliasMap().setName(btn, 'RefCurve multiplier Nr%i'%(iRef))
        #
        getAFXAliasMap().setName(self.calcCritPtIntersectionChk, 'CalcIntersect CheckButton')
        #
        #ENDGUILOG

        #
        # do some initial checks:
        self._checkRefMultiplier()
        self._checkUpdatePlotButton()


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onContextCallback(self):

        if _verbose:
            print '\n',_verboseBase,'onContextCallback'

        #
        # extract current context
        newContext = getCurrentContext()
        #
        vpName = newContext['viewportName']
        objectPath = newContext['objectPath']
        objectType = newContext['objectType']
        
        dirnKeys=['X', 'Y', 'Z', '-X', '-Y', '-Z']
        self.dirnCombo.clearItems()
        for dirn in dirnKeys:
            self.dirnCombo.appendItem(dirn)
        self.form.dirnKw.setValue(self.dirnCombo.getText())         
        refFrames=['Stationary', 'Moving']
        self.refFrameCombo.clearItems()
        for refFrame in refFrames:
            self.refFrameCombo.appendItem(refFrame)
        self.form.refFrameKw.setValue(self.refFrameCombo.getText())            
        
        #
        # Since there's other object type
        # set to disable and turn on only 
        # when XYPlot
        #
        self.updatePlotButton.disable()
        if objectType == "Odb":
            if not(self.currentContext):
                self.currentContext['objectType'] = objectType
                self.currentContext['objectPath'] = objectPath
                self.currentContext['viewportName'] = vpName
            self.odbName = os.path.basename(objectPath).replace("']","")
            self.plottedOdbName = self.odbName
            self.scanOdbForSteps()
            self.plotted = 0
        else:
            # Clear table, disable widgets, set odb name to none
            #
            self.setStepTableItems(items=[])
            self.odbName = None
            self.createBtn.disable()
            if objectType == "XYPlot":
                self.plotted = 1
                self.updatePlotButton.enable()
            
            

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdOkCreate(self, sender, sel, ptr):

        if _verbose:
            print '\n',_verboseBase,'onCmdOkCreate'

        #
        # extract keywords and apply them:
        status,keywords = self.extractKeywords()
        #
        if status == False:
            return
        #
        for name,kwValue in keywords.items():
            _kw = getattr(self.form, name)
            _kw.setValue(kwValue)
        #
        # Execute command
        self.handle(self, MKUINT(self.ID_CLICKED_APPLY, SEL_COMMAND), None)

        return 1


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdUpdatePlot(self, sender, sel, ptr):

        if _verbose:
            print '\n',_verboseBase,'onCmdUpdatePlot'

        #
        self._checkUpdatePlotButton()
        if not(self.updatePlotButton.isEnabled()):
            return
        #
        # extract keywords and apply them:
        status,keywords = self.extractKeywords(plotted=1)
        #
        if status == False:
            return
        #
        from kernelAccess import session
        #
        args = (keywords['odbPathKw'],
                keywords['modesToPlotKw'],
                keywords['xyPrefixKw'],
                keywords['calcCritPtIntersectionKw'],
                keywords['refMultiplierKw'],
                )
        getAFXApp().beginWaitCursor()
        getAFXApp().forceRefresh()
        #
        try:
            cmd = 'abq_CampbellDiagram.campbellModule.updatePlot(*'+str(args)+')'
            sendCommand(cmd)
        except:
            getAFXApp().endWaitCursor()
            showAFXErrorDialog(self,
                'An update of the plot was not possible.\n'+
                'Use "Create" to do a full Campbell-run.')
        #
        getAFXApp().endWaitCursor()


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def extractKeywords(self, plotted=0):

        if _verbose:
            print _verboseBase,'extractKeywords'

        # extractKeywords may be called when calculating
        # values or updating plot. 
        #
        if not plotted:
        
            # odbPathKw
            #
            self._keywords['odbPathKw'] = self.plottedOdbName

            # useStepsKw:
            #
            table = self._steps['table']
            stepData = self.getTableData(table)
 
            workOnSteps = [data[0] for data in stepData
                                   if data[0]]
            #
            # check spin speed:
            missingSpeed = [data[2] for data in stepData
                                   if data[0] and (data[2] == '')]
            #
            if len(workOnSteps) < 2:
                showAFXErrorDialog(self,'At least two steps must be activated.')
                return False,self._keywords
            elif missingSpeed:
                showAFXErrorDialog(self,'Please specify the spin speed for all active steps.')
                return False,self._keywords
            else:
                self._keywords['useStepsKw'] = str(stepData)

            # modesKw:
            #
            modeCombo = self._modes['modeCombo']
            allModesSelected = self._modes['chkBtn'].getCheck()
            firstActiveStep = self.getFirstActiveStep()
            freqItems = self._steps['odbStepFreqs'][firstActiveStep]

            modes = [float(freq) for freq in freqItems]

            # Always use all modes available. Plot only number of modes
            # selected
            if not(allModesSelected):
                numberModesToPlot = int(modeCombo.getItemText(modeCombo.getCurrentItem()))
            else:
                numberModesToPlot = len(modes)

            self._keywords['modesKw'] = str(modes)
            self._keywords['modesToPlotKw'] = numberModesToPlot

            # nodeSetKw:
            #
            nsetCombo = self._nodeSet['nsetCombo']
            nsetSelected = self._nodeSet['chkBtn'].getCheck()
            if not(nsetSelected):
                self._keywords['nodeSetKw'] = None

        # Grab changes to plotting self._keywords
        #
        # xyPrefixKw:
        #
        xyPrefix = self.form.xyPrefixKw.getValue()
        if xyPrefix:
            if not(xyPrefix[0] in string.letters):
                showAFXErrorDialog(self, 'The xy-data prefix must begin with a letter.')
                self.xyPrefixTf.setFocusToTextField()
                return False,self._keywords
        self._keywords['xyPrefixKw'] = xyPrefix

        # refMultiplierKw: (no check necessary)
        #
        refMultiplier = []
        for iRef,checkBtn in self.refMultiplier:
            status = checkBtn.getCheck()
            refMultiplier += [(iRef,status)]
        self._keywords['refMultiplierKw'] = str(refMultiplier)

        # calcCritPtIntersectionKw
        #
        if (self.calcCritPtIntersectionChk.getCheck() and
            self.calcCritPtIntersectionChk.isEnabled()):
            self._keywords['calcCritPtIntersectionKw'] = True
        else:
            self._keywords['calcCritPtIntersectionKw'] = False

        return True, self._keywords

    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def initCustomData(self):

        if _verbose:
            print _verboseBase,'initCustomData'
        #
        try:
            # Send command to initialize customData
            #
            if not self.moduleImported:
                sendCommand('import abq_CampbellDiagram.campbellModule')
                self.moduleImported = True
            #
            sendCommand(
                'abq_CampbellDiagram.campbellModule.initCustomData()' )
            
        except:
            errText = ('Information from Plug-In: "%s":\n'%PLUGIN_NAME+
                       'An error occurred while trying to process '+
                       'the data:\n\n%s' % sys.exc_info()[1] )
            showAFXErrorDialog(self, errText )
            return


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def readCustomData(self):

        if _verbose:
            print _verboseBase,'readCustomData'
        #
        from kernelAccess import session
        #
        # reload propertyEvaluation frequencies.
        try:
            usedSteps = session.customData.campbell.get('useSteps', None)
            if usedSteps:
                _used = [(data[1],data[2]) for data in usedSteps]
                self._steps['usedSteps'] = dict(_used)
        except:
            if _verbose:
                print 'CampbellDiagramWarning:',
                print 'Used frequencies for steps could not be read from stored.'
        #
        # reload node set names
        try:
            nodeSetNames = session.customData.campbell.get('odbNodeSets', [])
            if nodeSetNames:
                self._nodeSet['setNames'] = nodeSetNames[:]
        except:
            if _verbose:
                print 'CampbellDiagramWarning:',
                print 'Node set names could not be read from stored.'
            



    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def scanOdbForSteps(self):

        if _verbose:
            print _verboseBase,'scanOdbForSteps'
        #
        getAFXApp().beginWaitCursor()
        getAFXApp().forceRefresh()

        #
        # access create button and disable it, when scan is unsuccessful
        self.createBtn.disable()
        #
        #
        try:
            # Send command to parse odb and take last step from customData
            #
            if not self.moduleImported:
                sendCommand('import abq_CampbellDiagram.campbellModule')
                self.moduleImported = True
            sendCommand(
                'abq_CampbellDiagram.campbellModule.openOdb(odbPath="%s")'%(
                self.odbName))
            
            from kernelAccess import session
            odbData = session.customData.campbell['odbs']
            if odbData.has_key(self.odbName):
                if self.plotted:
                    self._steps['odbSteps'] = session.customData.campbell['useSteps']
                else:
                    self._steps['odbSteps'] = odbData[self.odbName]['odbSteps']
                self._steps['odbStepFreqs'] = odbData[self.odbName]['odbStepFreqs']
                #
                self.updateStepTable()
                nodeSetNames = odbData[self.odbName].get('odbNodeSets', [])
                self._nodeSet['setNames'] = nodeSetNames[:]
                self.setNsetItems(nodeSetNames)
            
        except:
            getAFXApp().endWaitCursor()
            errText = ('Information from Plug-In: "%s":\n\n'%PLUGIN_NAME+
                       'An error occurred while trying to process '+
                       'the data:\n\n%s' % sys.exc_info()[1] )
            self.setStepTableItems(items=_defaultStepItems)
            showAFXErrorDialog(self, errText )
            return

        getAFXApp().endWaitCursor()

        

    #---------------------------------------------------------------------
    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdSelectModes(self, sender, sel, ptr):
        """
        based on the settings for the mode check button the mode list is activated
        """

        if _verbose:
            print '\n',_verboseBase,'onCmdSelectModes'
        #
        #
        self.switchModeCombo()
        return

    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def initModeCombo(self):
        """
        initializes mode combo
        """
        if _verbose:
            print _verboseBase, 'initModeCombo'
        #
        modeCombo = self._modes['modeCombo']
        #
        modeCombo.clearItems()

    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def switchModeCombo(self, active=None):
        """
        based on the settings for the mode check button the mode combo is activated
        check == True -> ModeCombo inactive
        """

        if _verbose:
            print _verboseBase,'switchModeCombo'
        #
        #
        if active == None:
            isActivated = self._modes['chkBtn'].getCheck()
        else:
            self._modes['chkBtn'].setCheck(active)
            isActivated = active
        #
        if isActivated:
            self._modes['modeCombo'].disable()
        else:
            self._modes['modeCombo'].enable()
        return

    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def setModeItems(self, nModes):
        """
        define objects for mode combo
        """
        if _verbose:
            print _verboseBase, 'setModeItems'
        #
        modeCombo = self._modes['modeCombo']
        modeCombo.clearItems()
        #
        for iMode in range(nModes):
            modeCombo.appendItem(str(iMode+1))


    #---------------------------------------------------------------------
    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdSelectNset(self, sender, sel, ptr):
        """
        based on the settings for the mode check button the mode list is activated
        """

        if _verbose:
            print '\n',_verboseBase,'onCmdSelectNset'
        #
        #
        self.switchNsetCombo()
        return

    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def initNsetCombo(self):
        """
        initializes nset combo box
        """
        if _verbose:
            print _verboseBase, 'initNsetCombo'
        #
        nsetCombo = self._nodeSet['nsetCombo']
        #
        nsetCombo.clearItems()


    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def switchNsetCombo(self, active=None):
        """
        based on the settings for the nset check button the nset combo is activated
        check == True -> NsetCombo active
        """

        if _verbose:
            print _verboseBase,'switchNsetCombo active:',active,"!"
        #
        #
        if active == None:
            isActivated = self._nodeSet['chkBtn'].getCheck()
        else:
            self._nodeSet['chkBtn'].setCheck(active)
            isActivated = active
        #
        if isActivated:
            self._nodeSet['nsetCombo'].enable()
        else:
            self._nodeSet['nsetCombo'].disable()
        return


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def setNsetItems(self, items):
        """
        define objects for nset combo
        """
        if _verbose:
            print _verboseBase, 'setNsetItems'
        #
        nsetCombo = self._nodeSet['nsetCombo']
        nsetCombo.clearItems()
        #
        for item in items:
            nsetCombo.appendItem(str(item))
        #
        if not(items):
            nsetCombo.appendItem("NO_PART_SET_FOUND")
            nsetCombo.setCurrentItem(0)
            self.switchNsetCombo(active=False)


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdTableChk(self, sender, sel, ptr):
        """
        handle clicking on checks in step table
        redraw row background and redisplay mode list.
        """

        if _verbose:
            print '\n',_verboseBase,'onCmdTableChk'
        #
        if not(self._steps['filledOnce']):
            # the table is empty, do nothing.
            return
        
        #
        lastActiveStep = self._lastActiveStep
        #
        table = self._steps['table']
        tableChks = self.getBoolTableData(table)
        stepData = self._steps['odbSteps']
        #
        hasTableChanged = False
        #
        for iRow,chkBtn in enumerate(tableChks):
            if stepData[iRow][0] != chkBtn:
                stepData[iRow] = [chkBtn]+stepData[iRow][1:]
                self.setTableRowProperties(table, iRow+1, active=chkBtn)
                hasTableChanged = True
                break
        #
        if hasTableChanged:
            firstActiveStep = self.getFirstActiveStep()
            if firstActiveStep != lastActiveStep:
                #
                # Fill freq list based on active step:
                freqItems = self._steps['odbStepFreqs'][firstActiveStep]
                self.setModeItems(nModes=len(freqItems))
        #
        return

    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateStepTable(self):
        """
        Reaccesses custom data to fill step table 
        """
        if _verbose:
            print _verboseBase,'updateStepTable'
        #
        stepData = self._steps['odbSteps']
        #
        self.setStepTableItems(stepData)
        #
        # detect first active step:
        firstActiveStep = self.getFirstActiveStep()
        #
        # Fill freq list based on active step:
        freqItems = self._steps['odbStepFreqs'][firstActiveStep]
        self.setModeItems(nModes=len(freqItems))
        #
        #
        if stepData:
            #
            # Now re-enable btn.
            self.createBtn.enable()
            self._steps['filledOnce'] = True
        else:
            self.createBtn.disable()
            self._steps['filledOnce'] = False

    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstActiveStep(self):
        """
        Returns name of first step to be worked on.
        """
        if _verbose:
            print _verboseBase,'getFirstActiveStep'
        #
        stepData = self._steps['odbSteps']
        #
        # detect first active step:
        firstActiveStep = 0
        #print 'stepData=', stepData
        #print 'enumeratestepdata=', enumerate(stepData)
        for iRow,rowData in enumerate(stepData):
            if rowData[0]:
                firstActiveStep = iRow
                break
        #
        self._lastActiveStep = firstActiveStep
        return firstActiveStep
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def initStepTable(self, stepLabels):
        """
        define objects for step table
        """
        if _verbose:
            print _verboseBase, 'initStepTable'
        #
        table = self._steps['table']
        #
        table.showHorizontalGrid(TRUE)
        table.showVerticalGrid(TRUE)

        table.setLeadingRows(1)
        table.setLeadingRowLabels(stepLabels)
        table.setLeadingColumns(0)
        table.setColumnWidth(0, width=45)
        table.setColumnWidth(1, width=180)
        table.setColumnWidth(2, width=100)
        table.setColumnJustify(-1, AFXTable.MIDDLE)
        table.setColumnJustify(0, AFXTable.CENTER)

        # define type and edit-mode for items
        table.setColumnType(0, AFXTable.BOOL)
        table.setColumnType(1, AFXTable.TEXT)
        table.setColumnType(2, AFXTable.FLOAT)
        table.setColumnEditable(1, editable=FALSE)
        #

        table.setDefaultColor("Black")
        table.setStretchableColumn(1)


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def setStepTableItems(self, items):
        """
        Set contents for step table
        """

        if _verbose:
            print _verboseBase,'setStepTableItems'

        table = self._steps['table']
        usedSteps = self._steps.get('usedSteps', {})
        #
        numRows = table.getNumRows()
        #
        numItems = len(items)+1
        #
        if numItems != numRows:
            table.deleteRows(1,numRows-1)
            table.insertRows(1,numItems-1)
            
        for iRow,itemData in enumerate(items):
            jRow = iRow+1
            self.setTableRowProperties(table, iRow+1, active=itemData[0])
            #
            # fill items got from custom data for this ODB
            table.setItemBoolValue(jRow, 0, itemData[0])
            table.setItemValue(jRow, 1, str(itemData[1]))
            table.setItemValue(jRow, 2, str(itemData[2]))
            #
            # if job was committed before, fill also
            # spin speed if possible.
            propEvaluation = usedSteps.get(itemData[1], '')
            if _verboseDev:
                print 'Add propEvaluation for step:',itemData[1],propEvaluation
            if propEvaluation != '':
                table.setItemValue(jRow, 2, str(propEvaluation))


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def setTableRowProperties(self, table, iRow, active):
        """
        Set properties for current row, depending on active flag.
        """
        if _verboseDev:
            print _verboseBase,'setTableRowProperties'
            
        if active:
            backColor = clr_NavajoWhite
            stepNameClr = clr_Grey91
        else:
            backColor = clr_Grey80
            stepNameClr = clr_Grey80
        #
        colors = [backColor, stepNameClr, backColor]
        #
        for iCol in range(3):
            table.setItemBackColor(row=iRow, column=iCol, color=colors[iCol])
        #
        table.setItemEditable(iRow, 2, active)



    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getTableData(self, table):
        """
        Returns the current contents of table in list of lists.
        """
        tableData = []
        for jRow in range(1,table.getNumRows()):
            line = [table.getItemBoolValue(jRow,0),
                    table.getItemValue(jRow,1)]
            try:
                itemValue = table.getItemValue(jRow,2)
                line.append(float(itemValue))
            except ValueError:
                #
                # add empty, if float fails.
                line.append('')
            tableData.append(line)
        #
               
        return tableData


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getBoolTableData(self, table):
        """
        Returns the current contents of table in list of lists.
        first col only.
        """
        tableData = []
        for jRow in range(1,table.getNumRows()):
            tableData += [table.getItemBoolValue(jRow,0)]
        #
        return tableData


    #---------------------------------------------------------------------
    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdRefMultiplier(self, sender, sel, ptr):
        """
        Checks, if any ref multiplier is active.
        If not, disable some check buttons.
        """

        if _verbose:
            print '\n',_verboseBase,'onCmdRefMultiplier'
        #
        self._checkRefMultiplier()
        return

    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def _checkRefMultiplier(self):
        """
        Checks, if any ref multiplier is active.
        If not, disable some check buttons.
        """

        if _verbose:
            print _verboseBase,'_checkRefMultiplier'
        #
        anyBtnSeleted = False
        for iRef, chkBtn in self.refMultiplier:
            if chkBtn.getCheck():
                anyBtnSeleted = True
                break
        if anyBtnSeleted:
            self.calcCritPtIntersectionChk.enable()
        else:
            self.calcCritPtIntersectionChk.disable()
        #
        return

    #---------------------------------------------------------------------
    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdCalcIntersection(self, sender, sel, ptr):
        """
        Checks, if intersections of ref curve and whirl speed should be calculated.
        If not, disable arrows.
        """

        if _verbose:
            print '\n',_verboseBase,'onCmdCalcIntersection'
        #


    #---------------------------------------------------------------------
    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def _checkStatusMessages(self):
        """
        Handles status mesages generated during evaluation.
        """

        if _verbose:
            print _verboseBase,'_checkStatusMessages'
        #
        from kernelAccess import session
        if ( not(hasattr(session, 'customData')) or
             not(hasattr(session.customData, 'campbell'))):
            info = ('Warning from Plug-In: "%s":\n\n'%PLUGIN_NAME+
                    'The customData could not be extracted.'+
                    '\nPlease ask your local Support Office.')
            dialog = showAFXErrorDialog
            self.updatePlotButton.disable()
            return
        #
        #
        odbData = session.customData.campbell['odbs'][self.plottedOdbName]
        #
        generatedMessages = ''
        error = False
        #
        massStatus = odbData['massStatus']
        if massStatus and massStatus.find(', FAILED!:') > -1:
            generatedMessages += massStatus
        #
        odbStatus = odbData['odbStatus']
        if odbStatus and odbStatus.find(', FAILED!:') > -1:
            generatedMessages += odbStatus
            error = True
        #
        if generatedMessages:
            if error:
                info = ('Warning from Plug-In: "%s":\n\n'%PLUGIN_NAME+
                        'An error occured during data generation.'+
                        '\nPlease doublecheck the ODB content.'+
                        '\nSee detailed messages below:\n\n')
                dialog = showAFXErrorDialog
                self.updatePlotButton.disable()
            else:
                info = ('Information from Plug-In: "%s":\n\n'%PLUGIN_NAME+
                        'An issue occured during data generation.'+
                        '\nPlease doublecheck the ODB content.'+
                        '\nSee detailed messages below:\n\n')
                dialog = showAFXInformationDialog
                self._checkUpdatePlotButton()
            #
            dialog(self, info+generatedMessages)
            return

        self._checkUpdatePlotButton()
        if _verbose:
            if massStatus == None:
                massStatus = ''
            showAFXInformationDialog(self, massStatus+'\n'+odbStatus)
        
    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def _checkUpdatePlotButton(self):
        """
        Checks, all data is available to enable the update plot btn.
        If not, disable btn
        """
        if _verbose:
            print _verboseBase,'_checkUpdatePlotButton'
        #
        from kernelAccess import session
        if ( not(hasattr(session, 'customData')) or
             not(hasattr(session.customData, 'campbell')) or
             not(session.customData.campbell.get('odbs', None))):
             self.updatePlotButton.disable()
        else:
            odbData = session.customData.campbell['odbs']
            if (odbData.has_key(self.plottedOdbName) and
                odbData[self.plottedOdbName].get('modeCorrelation', None) and
                odbData[self.plottedOdbName].get('odbSteps', None) and
                odbData[self.plottedOdbName].get('odbStepFreqs', None)):
                   self.updatePlotButton.enable()
            else:
                self.updatePlotButton.disable()

    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
        """
        register context callback
        """
        registerCurrentContext(self.onContextCallback)
        #
        AFXDataDialog.show(self)           

    #
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):
        """
        unregister context callback
        """
        unregisterCurrentContext(self.onContextCallback)
        #
        AFXDataDialog.hide(self)
