"""
calculate mac values based on complex criteria only
"""
from abaqus import *
from abaqusConstants import *

from abq_CampbellDiagram import macUtilities
from abq_CampbellDiagram.setConstants import (REF_CURVE_PREFIX,
                                                   CROSS_POINT_PREFIX)

#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createXYData(odbPath, modeCorrelation, modesToPlot, xyPrefix,
                 calcCritPtIntersection, refMultiplier):
    """
    Create session xyData for curve plotting for each frequency.
    """
    if xyPrefix and not(xyPrefix .endswith('_')):
        xyPrefix += '_'
    plotTitle = xyPrefix+'CampbellDiagram'
    #
    if session.xyPlots.has_key(plotTitle):
        del session.xyPlots[plotTitle]
    xyp = session.XYPlot(plotTitle)
    chartName = xyp.charts.keys()[0]
    chart = xyp.charts[chartName]
    #
    vp = session.viewports[session.currentViewportName]
    vp.setValues(displayedObject=xyp)
    #
    curves = []
    #
    # Create curves for each mode requested
    modes = sorted(modeCorrelation.keys())
    i=1
    for iMode in modes:
        modeData = modeCorrelation[iMode]
        curveName = xyPrefix+'Mode_%i'%(iMode+1)
        xy1 = session.XYData(name=curveName, data=modeData)
        curve1 = session.Curve(xyData=xy1)
        curve1.symbolStyle.setValues(show=True)
        curves += [curve1]
        if i == modesToPlot: break
        i+=1
    #
    # Create rmp-hz reference curve:
    refStartPoint = (0., 0.)
    lastModeSpeed = modeData[-1][0]
    allReferenceData = []
    allCrossPoints = []
    #
    for multiplier,status in refMultiplier:
        if not(status):
            continue
        #
        factor = int(multiplier)
        referenceData = [refStartPoint,
                         (lastModeSpeed,lastModeSpeed/60.*factor)]
        #
        curveName = xyPrefix+'%s%i'%(REF_CURVE_PREFIX, factor)
        xy1 = session.XYData(name=curveName, data=referenceData)
        allReferenceData += [referenceData]
        #
        curve1 = session.Curve(xyData=xy1)
        curve1.lineStyle.setValues(show=True, style=DASHED)
        curves += [curve1]
        _curveColor = curve1.lineStyle.color
        #
        # Create cross points if needed
        if calcCritPtIntersection:
            #
            crossPoints = []
            maxRefFreq = referenceData[1][1]
            #
            i=1
            for iMode in modes:
                modeData = modeCorrelation[iMode]
                
                #
                # do quick check, if intersection is possible:
                freqValues = sorted([data[1] for data in modeData])
                if maxRefFreq < freqValues[0]:
                    continue
                #
                # calculate intersection

                for iSeg in range(len(modeData)-1):
                    line1 = (modeData[iSeg],modeData[iSeg+1])
                    #print 'line1=', line1
                    crossPoint = macUtilities.calculateLineLineIntersection2D(
                        line1, line2=referenceData)
                    if crossPoint:
                        crossPoints += [crossPoint]
                        break
                if i == modesToPlot: break
                i+=1

            #
            # if crossPoints, create curve:
            if crossPoints:
                curveName = xyPrefix+'%s%i'%(CROSS_POINT_PREFIX, factor)
                xy1 = session.XYData(name=curveName, data=crossPoints)
                curve1 = session.Curve(xyData=xy1)
                curves += [curve1]
                curve1.symbolStyle.setValues(show=True,size=2, marker=HOLLOW_CIRCLE,
                                             color=_curveColor)
                curve1.lineStyle.setValues(show=False, color=_curveColor)
                #
                allCrossPoints += [(factor, _curveColor, crossPoints)]
    #
    # store cross points for later reusage:
    session.customData.campbell['odbs'][odbPath]['allCrossPoints'] = allCrossPoints
    #
    # Create Chart
    chart.setValues(curvesToPlot=curves)
    #
    xTitle = 'Spin Speed [rpm]'
    yTitle = 'Whirl Frequency [Hz]'
    xMaxValue = ''
    yMaxValue = ''
    #
    chart.axes1[0].axisData.setValues(labelFormat=DECIMAL, labelNumDigits=0,
                                      useSystemTitle=False,
                                      title=xTitle)
    chart.axes2[0].axisData.setValues(labelFormat=DECIMAL, labelNumDigits=0,
                                      useSystemTitle=False,
                                      title=yTitle,
                                      tickMode=INCREMENT,
                                      tickIncrement=2000.)
    if xMaxValue != '':
        chart.axes1[0].axisData.setValues(maxAutoCompute=False, maxValue=xMaxValue)
    else:
        chart.axes1[0].axisData.setValues(maxAutoCompute=True)
    #
    if yMaxValue != '':
        chart.axes2[0].axisData.setValues(maxAutoCompute=False, maxValue=yMaxValue)
    else:
        chart.axes2[0].axisData.setValues(maxAutoCompute=True)
    #
    chart.majorAxis2GridStyle.setValues(show=True)

    #
    #
    odb = session.odbs[odbPath]
    crossAnnotations = [_n for _n in odb.userData.annotations.keys()
                           if _n.startswith(xyPrefix+CROSS_POINT_PREFIX)]
    for _n in crossAnnotations:
        del odb.userData.annotations[_n]