"""
Some utilities used during mac calculation
"""

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getModeOrder(macMatrix, printResults):
    """
    Extract max mac value and print the three biggest ones and the sum.

    @param macMatrix     LIST of LIST floats
    @param printResults  BOOL if True print information

    @return modeOrder    LIST of integer id and mode value
    """
    modeOrder = []
    cutOff = 8
    for line in macMatrix:
        extend = [(_x,i+1) for i,_x in enumerate(line)]
        lineSort = sorted(extend, reverse=True)
        modeOrder += [lineSort[:cutOff]]
        modeOrder[-1] += [sum(line)]

    if printResults:
        for iFreq,x in enumerate(modeOrder):
            print 'Freq: %2i, Maximum: '%(iFreq+1),
            for a in x:
                try:
                    print '%12.9f, %3i |'%(a[0],a[1]),
                except:
                    print '%12.9f'%(a),
            print 'Sum = %10.6f'%x[-1],
            if iFreq+1 != x[0][1]:
                print ' -> swap freq!'
            else:
                print ' == keep place.'
    #
    # return the indicator for the maximum mode by row
    return modeOrder


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def linearInterpolation(xValues, yValues, xPoint):
    """
    Returns the interpolated value yPoint.
    xValues sorted ascending, both SEQ have the same length!

    @param xValues  SEQ: of Floats the independent variable - sorted ascending
    @param yValues  SEQ: of Floats the dependent variable.
    @param xPoint   FLOAT: the position to get a y-variable for. 

    @return yPoint   FLOAT: the related dependent value to xPoint.
    """
    #
    if xPoint >= xValues[-1]:
        return yValues[-1]
    elif xPoint <= xValues[0]:
        return yValues[0]
    #    
    for i in range(len(xValues)-1):
        if xValues[i] < xPoint <= xValues[i+1]:
            break
    j = i+1
    yPoint = yValues[i]+(yValues[j]-yValues[i])/(xValues[j]-xValues[i])*(
      xPoint-xValues[i])
    #
    return yPoint
# end linearInterpolation

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def calculateLineLineIntersection2D(line1, line2):
    """
    Calculates the intersection point between two lines given by two points each.
    
    @param line1  TUPLE of TUPLE of two floats ((x1,y1),(x2,y2)) first line
    @param line2  TUPLE of TUPLE of two floats ((x1,y1),(x2,y2)) second line

    @return crossPoint TUPLE of floats or None
    """
    #print 'line1, line2=', line1, line2
    #
    slope1 = getSlope2D(line1)
    #print 'slope1=', slope1
    slope2 = getSlope2D(line2)
    #print 'slope2=', slope2
    #
    if slope1 == slope2:
        print 'both lines have same slope'
        # both lines should not be parallel
        return None
    #
    x1_1 = line1[0][0]
    x1_2 = line1[1][0]
    x2_1 = line2[0][0]
    x2_2 = line2[1][0]
    #
    y1_1 = line1[0][1]
    y1_2 = line1[1][1]
    y2_1 = line2[0][1]
    y2_2 = line2[1][1]
    #
    #print 'slope1, x1_1, slope2, x2_1, y2_1, y1_1=', slope1, x1_1, slope2, x2_1, y2_1, y1_1
    xCross = (slope1*x1_1-slope2*x2_1+y2_1-y1_1)/(slope1-slope2)
    #
    yCross = slope1*(xCross-x1_1)+y1_1
    #
    crossPoint = (xCross,yCross)
    #
    allX1 = sorted((x1_1,x1_2))
    allX2 = sorted((x2_1,x2_2))
    allY1 = sorted((y1_1,y1_2))
    allY1 = sorted((y2_1,y2_2))
    #
    if ((allX1[0] < xCross < allX1[1]) and
        (allY1[0] < yCross < allY1[1])):
        return crossPoint
    else:
        return None
# end calculateLineLineIntersection2D


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getSlope2D(line1):
    """
    Return the slope of the line, line is given by two points.
    
    @param line1  TUPLE of TUPLE of two floats ((x1,y1),(x2,y2)) first line

    @return slope TUPLE of floats or None
    """
    #
    p1,p2 = line1
    if p1 == p2:
        # the two points should not be equal
        return None
    elif p1[0] == p2[0]:
        # the two x-coord should not be equal
        return None
    #
    #print 'p2, p1=', p2,p1
    slope = ((p2[1]-p1[1])/(p2[0]-p1[0]))
    #print 'slope=', slope
    return slope

