from abaqusConstants import *
from abaqusGui import *
from kernelAccess import mdb,session
import os
from abq_ContourIntegral.contourIntegralConsts import DISTANCE_PICKED, CFP_PICKED, LABEL_PICKED
#from ababltin import getMdbVersionNumbers
thisPath = os.path.abspath(__file__)
thisDir = os.path.dirname(thisPath)

###########################################################################
# Class definition
###########################################################################

class ContourIntegralDB(AFXDataDialog):
    [ID_COMBO,ID_AVG]= range(AFXDataDialog.ID_LAST,AFXDataDialog.ID_LAST+2)
    numInstance = 0
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):
        ContourIntegralDB.numInstance += 1
        self.form=form
        # Construct the base class.
        #

        AFXDataDialog.__init__(self, form, 'XY Data from Contour Integral',
            self.APPLY|self.CANCEL, DECOR_RESIZE)
        #GUILOG
        getAFXAliasMap().setPrefix(self, 'ContourIntegralDB-%d'
                                   % ContourIntegralDB.numInstance)
        #ENDGUILOG                       
            
        self.crackNames =[]
        self.combineList = []

        self.okBtn = self.getActionButton(self.ID_CLICKED_APPLY)
        #getAFXAliasMap().setName(self.okBtn, 'plot')
        self.okBtn.setText('Plot')
        #Find current .odb name    
        vp=session.currentViewportName
        GroupBox_2 = FXGroupBox(p=self, text='Data Extraction', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        #Combobox for Crack names
        self.crackComboBox = AFXComboBox(p=GroupBox_2, ncols=12, nvis=1, text='Crack name:', tgt=self, sel=self.ID_COMBO,opts=LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.crackComboBox, 'crackcombobox')
        FXMAPFUNC(self, SEL_COMMAND, self.ID_COMBO, ContourIntegralDB.populateLists)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_AVG,ContourIntegralDB.onAvgCheck)
        self.crackComboBox.setMaxVisible(10)       
        
        GroupBox_3 = FXGroupBox(p=self, text='X Values', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        
        VFrame_1 = FXVerticalFrame(p=GroupBox_3, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        
        #Radio buttons for Labels or Distance along crack front        
        self.rad1=FXRadioButton(p=VFrame_1, text='Distance', tgt=form.distOrNodeKw, sel=DISTANCE_PICKED.getId())
        getAFXAliasMap().setName(self.rad1, 'radio1')
        self.rad3=FXRadioButton(p=VFrame_1, text='Position on Crack Front', tgt=form.distOrNodeKw, sel=CFP_PICKED.getId())
        getAFXAliasMap().setName(self.rad3, 'radio3')
        self.rad2=FXRadioButton(p=VFrame_1, text='Node ID/XFEM crack front ID', tgt=form.distOrNodeKw, sel=LABEL_PICKED.getId())        
        getAFXAliasMap().setName(self.rad2, 'radio2')
        GroupBox_4 = FXGroupBox(p=self, text='Y Values', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        HFrame_1 = FXHorizontalFrame(p=GroupBox_4, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        VFrame_2 = FXVerticalFrame(p=HFrame_1, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        label1 = FXLabel(p=VFrame_2, text='Parameters', opts=JUSTIFY_LEFT)
        listVf = FXVerticalFrame(p=VFrame_2, opts=FRAME_SUNKEN|FRAME_THICK, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        listVf.setSelector(99)
        #paraList for Parameters
        self.paraList = AFXList(p=listVf, nvis=8, tgt=form.fracParaKw, sel=0, opts=HSCROLLING_OFF|LIST_EXTENDEDSELECT)
        getAFXAliasMap().setName(self.paraList, 'paraList')
        VFrame_3 = FXVerticalFrame(p=HFrame_1, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        label2 = FXLabel(p=VFrame_3, text='Contours', opts=JUSTIFY_LEFT)   
        listVf = FXVerticalFrame(p=VFrame_3, opts=FRAME_SUNKEN|FRAME_THICK, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        listVf.setSelector(99)
        #contourList for Contours
        self.contourList = AFXList(p=listVf, nvis=8, tgt=form.contourNumKw, sel=0, opts=HSCROLLING_OFF|LIST_EXTENDEDSELECT)
        getAFXAliasMap().setName(self.contourList, 'contourList')
        HFrame_2 = FXHorizontalFrame(p=self, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        self.chkBtn = FXCheckButton(HFrame_2, 'Plot the average data',  self, self.ID_AVG)
        getAFXAliasMap().setName(self.chkBtn, 'chkBtn')
        self.chkBtn.setCheck(FALSE)
       
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def populateLists(self,sender,sel,ptr):
        self.paraList.clearItems()
        self.contourList.clearItems()

        self.crackNames,self.combineList=self.findCrackParamtersContours()
        item = self.crackComboBox.getText()
        self.form.crackNameKw.setValue(item)
        for i in range(len(self.combineList)):
            if self.combineList[i][0]==item and self.paraList.findItem(str(self.combineList[i][1]))==-1:
                self.paraList.appendItem(text=str(self.combineList[i][1]))
            if self.combineList[i][0]==item and self.contourList.findItem(str(self.combineList[i][-1]))==-1:
                self.contourList.appendItem(text=str(self.combineList[i][-1]))
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):
        unregisterCurrentContext(self.populateComboboxLists)
        AFXDataDialog.hide(self)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self): 
        registerCurrentContext(self.populateComboboxLists)
        AFXDataDialog.show(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def findCrackParamtersContours(self):
        vp=session.currentViewportName
        odbName=session.viewports[vp].odbDisplay.name
        hVars=session.odbData[odbName].historyVariables
        hVarsKeys = hVars.keys()
        contPara=['J-integral:','J-integral estimated from Ks:','T-stress:','Stress intensity factor K1:',
        'Stress intensity factor K2:','Stress intensity factor K3:','Crack propagation direction:','Ct-integral:']
       
        # Find history variable strings 'hList' which has above contour integral parameters
       
        hList=[hVarsKeys[t] for t in range(len(hVarsKeys)) for k in range(len(contPara)) if contPara[k] in hVarsKeys[t]]
        # Use strings in above obtained hList to parse, so that 'crack name','fracture paramter' and 'contour'
        #can be obtained.And they can be used in populating widgets.

        crackNames=[]

        combineList=[]

        for i in hList:
            lLabel= hVars[i].legendLabel
            splitLabel=(lLabel).split(' ')
            fpara=splitLabel[0]

            if majorVersion!=6 :
                msg =  ( i18n.tr('Error: Invalid ODB version: %s.py\n'))
                getAFXApp().beep()
                getAFXApp().getAFXMainWindow().writeToMessageArea(msg)

                break
            elif majorVersion==6 and minorVersion> 11: 
                splitString=(splitLabel[-5]).split('_')          
            else:
                splitString=(splitLabel[-1]).split('_')
            if splitString[-3]=='':
                crack=splitString[1]
            elif splitString[0].isupper()==False:
                crack=splitString[0]+'_'+splitString[1]
            else:
                crack=splitString[0]
            contu=splitString[-1]
            combine=[crack,fpara,contu]
            crackNames.append(crack)
            if combine not in combineList:
                combineList.append(combine)

        crackNames=list(set(crackNames))
        return (crackNames,combineList)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def populateComboboxLists(self):
        self.crackComboBox.clearItems()
        self.paraList.clearItems()
        self.contourList.clearItems()
        self.crackNames,self.combineList=self.findCrackParamtersContours()
        Flag=True
        for i in self.crackNames:        
            self.crackComboBox.appendItem(text=str(i))
            if Flag==True:
                self.form.crackNameKw.setValue(i)
                Flag=False
        self.form.fracParaKw.setValue(0,'')
        self.form.contourNumKw.setValue(0,'')                
        item = self.crackComboBox.getText()
        self.form.crackNameKw.setValue(item)
        for i in range(len(self.combineList)):
            if self.combineList[i][0]==item and self.paraList.findItem(str(self.combineList[i][1]))==-1:
                self.paraList.appendItem(text=str(self.combineList[i][1]))
            if self.combineList[i][0]==item and self.contourList.findItem(str(self.combineList[i][-1]))==-1:
                self.contourList.appendItem(text=str(self.combineList[i][-1]))               
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onAvgCheck(self, sender, sel, ptr):
        if self.chkBtn.getCheck():

            self.form.avgKw.setValue(True)
        else:

            self.form.avgKw.setValue(False)                
