from abaqusGui import *
from abaqusConstants import *
from abq_ContourIntegral.contourIntegralConsts import DISTANCE_PICKED

###########################################################################
# Class definition
###########################################################################

class ContourIntegralForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        
        # Construct the base class.
        #
        AFXForm.__init__(self, owner)

        self.cmd = AFXGuiCommand(mode=self, method='plotFractureParameters',
            objectName='abq_ContourIntegral.contourIntegralModule', registerQuery=False)
        pickedDefault = ''
        self.crackNameKw = AFXStringKeyword(self.cmd, 'crackName', True)
        self.fracParaKw = AFXTupleKeyword(self.cmd, 'fracPara', True,opts=AFXTUPLE_TYPE_STRING)
        self.contourNumKw = AFXTupleKeyword(self.cmd, 'contourNum', True,opts=AFXTUPLE_TYPE_STRING)
        self.distOrNodeKw = AFXSymConstKeyword(self.cmd,'distOrNode', True, defaultValue=DISTANCE_PICKED.getId())
        self.avgKw = AFXBoolKeyword(command=self.cmd,name='avg',isRequired=True,defaultValue=False)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        import contourIntegralDB
        return contourIntegralDB.ContourIntegralDB(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def activate(self):
        mainWindow = getAFXApp().getAFXMainWindow()
        vp=session.viewports[session.currentViewportName]
        
        if not getDisplayedObjectType() == ODB:
            showAFXErrorDialog(mainWindow,'There is no output database (ODB) \n'
                                          'associated with the current viewport.\n\n'
                                              'Select an ODB from the context bar,\n'
                                  'or use "File->Open" to open an ODB.')  
        else:
            #Check to see if contour integral parameter is available in the ODB
            odbName=vp.odbDisplay.name
            hVars=session.odbData[odbName].historyVariables
            hVarsKeys = hVars.keys()

            contPara=['J-integral:','J-integral estimated from Ks:','T-stress:','Stress intensity factor K1:',
            'Stress intensity factor K2:','Stress intensity factor K3:','Crack propagation direction:','Ct-integral:']            
           
            para=[contPara[k] for k in range(len(contPara)) for t in range(len(hVarsKeys)) if contPara[k] in hVarsKeys[t]]

            if para==[]:    
                showAFXErrorDialog(mainWindow,'The ODB does not have any contour integral paramters.')
            else:
                AFXForm.activate(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def okToCancel(self):

        # No need to close the dialog when a file operation (such
        # as New or Open) or model change is executed.
        #
        return False
