from odbAccess import *
import xyPlot
import visualization
from abaqus import *
from abq_ContourIntegral.contourIntegralConsts import *
from abaqusConstants import *


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   
#Find ODB, step and frame in the current viewport
def getCurOdbStepFrame():    
    vp=session.currentViewportName
    odbDisp=session.viewports[vp].odbDisplay
    odbName=odbDisp.name
    odb=session.odbs[odbName]
    stepKeys=odb.steps.keys()
    curStepFrame=odbDisp.fieldFrame
    curStep= curStepFrame[0]
    curFrame= curStepFrame[1]

    #Checking for Frame=0, since for frame#0 no paramter is available
    if curFrame==0:
        raise ValueError,' No contour integral parameter is available for frame=0 '

    return odbName,curStep,curFrame,odb,stepKeys
   
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Main function which gets arguments from the dialog box.
def plotFractureParameters(crackName,fracPara,contourNum,distOrNode, avg):
    global avg1
    avg1 = avg
    fracPara=list(fracPara)
    contourNum=list(contourNum)
    if distOrNode==DISTANCE_PICKED:
        alongCrackFront='Use distance as X axis for plotting'
    elif distOrNode==CFP_PICKED:	
        alongCrackFront='Use crack front position as X axis for plotting'
    elif distOrNode==LABEL_PICKED:
        alongCrackFront='Use labels as X axis for plotting'
    global odbName,curStep,curFrame,odb,stepKeys    
    odbName,curStep,curFrame,odb,stepKeys=getCurOdbStepFrame()    
    if fracPara==[] or '' in fracPara:
        raise ValueError,'At least one parameter must be specified.'
    if contourNum==[] or '' in contourNum:
        raise ValueError,'At least one contour must be specified.'

    contours=[]
    for i in contourNum:
        contours.append(int(i))
        
    fracParameters=getFracParaStrings(fracPara)
    
    histVars,contourHistVarsKeys=getHistoryVariableKeys(crackName, fracParameters)
    
    xfem,xfemLabels,forElsetCheck, conturs=goToConvenOrXfem(contourHistVarsKeys,histVars)
    

    if xfem==0:
        #calls a function for a conventional crack 
        collectConvKeys(forElsetCheck, contourHistVarsKeys,crackName,fracParameters,contours,alongCrackFront)
    else:
        #Calls a function for an XFEM crack
        collectXfemKeys(forElsetCheck, histVars,contourHistVarsKeys,xfemLabels,crackName,fracParameters,contours,alongCrackFront)            
        
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Gets contour integral complete parameter strings form the short names given 
#in the dialog box which will be passed as history variable name string
def getFracParaStrings(fracPara):

    dict={'J-integral estimated from Ks:':'JKs','Crack propagation direction:':'Cpd',\
                 'J-integral:':'J','Stress intensity factor K1:':'K1','Stress intensity factor K2:':'K2',\
                 'Stress intensity factor K3:':'K3','Ct-integral:':'Ct','T-stress:':'T'}
    
    fracParameters=[j for j in dict if dict[j] in fracPara]
    
    return fracParameters
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getHistoryVariableKeys(crackName, fracParameters):
    
    histVars=session.odbData[odbName].historyVariables
    histVarsKeys=histVars.keys()
    

    #Using a list for XFEM crack to append labels

    contourHistVarsKeys=[]    
    #Find history varialbe keys with crackname and contour integral parameters for conventional/XFEM crack 
    
    for j in histVarsKeys:
        for i in range(len(fracParameters)):
            if fracParameters[i] in j and crackName in j:
                contourHistVarsKeys.append(j)
    return histVars,contourHistVarsKeys                
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Decides whether it is a conventional or an XFEM crack. For an XFEM crack it also finds labels by
#parsing the strings.
#
def goToConvenOrXfem(contourHistVarsKeys,histVars):
    
    xfem=0
    xfemLabels=[]
    conturs = []
    forElsetCheck=contourHistVarsKeys[0]

    for i in contourHistVarsKeys:
        if 'XFEM' in i:
            xfem=1
            break    
    xLabels=[]        
    if xfem==1:
        for i in contourHistVarsKeys:
            leglabel=histVars[i].legendLabel
            x=(leglabel).split(' ')
            if minorVersion > 11:
                y=(x[-5]).split('_')
                contur = y[-1]
            else:    
                y=(x[-1]).split('_')#-5 instead of -1
                contur = y[-1]
            xLabels.append(int(y[-3]))
            xfemLabels=sorted(list(set(xLabels)))
            if contur not in conturs:
                conturs.append(contur)
    else:
         for i in contourHistVarsKeys:
            leglabel=histVars[i].legendLabel
            x=(leglabel).split(' ')
            if minorVersion > 11:
                y=(x[-5]).split('_')
                contur = y[-1]
            else:    
                y=(x[-1]).split('_')#-5 instead of -1
                contur = y[-1]
            if contur not in conturs:
                conturs.append(contur)
       
    global conturs1
    conturs1 = conturs
    return xfem,xfemLabels, forElsetCheck, conturs        
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Extracts results for a Conventional crack
#the strings will become historyvariable keys strings, which can be used as outputVariableName

def resultsConv(convHistVarsKeys,distanceConv,char,convNodeSetKeys,crackTipLabels,paraConv,alongCrackFront,contours,crackName):    
    modifiedList=[]
    
    #from each convNodeSetkey 'char' (e.g., _J_ etc.) is removed
    #
    for entity in convNodeSetKeys:
       splitEntity=entity.split(char)
       reconstitution='_'.join(splitEntity)
       modifiedList.append(reconstitution)

    results=[]
    
    for member in convHistVarsKeys:
        for name in modifiedList:
            if name in member and paraConv in member:
                try:
                    histResultConv = (xyPlot.XYDataFromHistory(odb=odb, outputVariableName=member, name= 'temp',steps=(stepKeys[curStep], ), )).data[curFrame-1]
                    combine=[name,histResultConv]
                    results.append(combine)
                except:
                    raise ValueError, 'Specified contour integral parameters are not available for the Step = '+ str(stepKeys[curStep])

    distLabel(results,paraConv,distanceConv,crackTipLabels,contours,crackName,alongCrackFront)    


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Collect strings to find labels and coordinates With coordinates, find the distance 
#between crack tips  
# Send distance, labels and additional variables to resultsConv function
# 
def collectConvKeys(forElsetCheck, convHistVarsKeys, crackName,
                   fracParameters, contours, alongCrackFront):    

    # List of parameter strings with matching
    # identifiers for node sets
    #
    paramsStringsList = [
        (('J-integral:',),                   '_J_'),
        (('T-stress:',)  ,                   '_T_'),
        (('Stress intensity factor K1:',
          'Stress intensity factor K2:',
          'Stress intensity factor K3:',
          'J-integral estimated from Ks:',
          'Crack propagation direction:',),   '_K_'),
        (('Ct-integral',),                   '_C_')]

    # List of string variables with value if identifier found
    # It is 'empty' otherwise
    #
    char = ['empty' for i in range(len(paramsStringsList))]

    for key in convHistVarsKeys:
        for param in paramsStringsList:
            for string in param[0]:
                if string in key:
                    char[paramsStringsList.index(param)] = param[1]

    # Modify list of parameter strings to include
    # if identifier is found
    #
    paramsStringsList = zip(paramsStringsList, char)
    nSets=odb.rootAssembly.nodeSets

    # Iterate through parameter strings to create list of conventional
    # crack node set keys and list of conventional crack node set keys 
    # that contain 'Contour_1' in it
    #
    # Find coordinates, labels and distance, then call resultsConv function
    # 

    for param in paramsStringsList:

        for string in param[0][0]:
            if string in fracParameters:        
                convNodeSetKeys = []
                contour1ConvNodeSetKeys = []
                if param[1] == param[0][1]:
                    convNodeSetKeys = [nSetKey for nSetKey in nSets.keys() if param[1] in nSetKey and crackName in nSetKey]
                    contour1ConvNodeSetKeys = [case for case in convNodeSetKeys if 'Contour_1' in case]
                    crackTipCoord = [[nom, nSets[nom].nodes[0][0].coordinates] for nom in contour1ConvNodeSetKeys]
                    crackTipLabels = [nSets[nom1].nodes[0][0].label for nom1 in contour1ConvNodeSetKeys]

                    # Find the distance between crack tips    
                    #     
                    distanceConv = distconv(crackTipCoord,alongCrackFront)

                    resultsConv(convHistVarsKeys, distanceConv, param[1], 
                                convNodeSetKeys, crackTipLabels, string, 
                                alongCrackFront, contours, crackName)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# creation of the list which gives the fracture parameter value for the 
#corresponding distance,for conventional and XFEM cracks

def distLabel(results,para,distance,labels,contours,crackName,alongCrackFront):
   
    labels=sorted(labels)
    assocDist= []
    assocLabel=[]
    
    for i in range(len(contours)):
        if len(conturs1)==10 or len(conturs1)>10:
            if contours[i]<10:
                resultContour = [resu for resu in results if 'Contour_0'+str(contours[i]) in resu[0]]
            else:
                resultContour = [resu for resu in results if 'Contour_'+str(contours[i]) in resu[0]]
        else:        
            resultContour = [resu for resu in results if 'Contour_'+str(contours[i]) in resu[0]]

        histResultList=[A[-1][-1] for A in resultContour]
        for j in range(len(histResultList)):
            if alongCrackFront=='Use distance as X axis for plotting':
                assocDist.append([distance[j], histResultList[j]])
            elif alongCrackFront=='Use crack front position as X axis for plotting':	
                assocDist.append([distance[j],histResultList[j]])   
            elif alongCrackFront=='Use labels as X axis for plotting':
                assocLabel.append([labels[j],histResultList[j]])      
                
    plotPrep(assocDist,assocLabel,para,alongCrackFront,contours,crackName)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Extracts results of an XFEM crack with and sends results to 'distLabel' function

def resultsXfem(stringList,paraList,para,distanceXfem,xfemLabels,contours,crackName,alongCrackFront):
   
    resultsX=[]

    for entityX in paraList:         
      for nameX in stringList:
          if nameX in entityX and para in entityX:
              try:
                histResultsXfem=(xyPlot.XYDataFromHistory(odb=odb, outputVariableName=entityX, name='temp1', steps=(stepKeys[curStep], ), )).data[curFrame-1]
              except:
                raise ValueError, 'Specified contour integral parameters are not available for the Step = '+ str(stepKeys[curStep])
              combineX=[nameX,histResultsXfem]
              resultsX.append(combineX)
   
    distLabel(resultsX,para,distanceXfem,xfemLabels,contours,crackName,alongCrackFront)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Collect strings to find coordinates
# With coordinates, find the distance between crack tips  
# Send distance, labels and additional variables to resultsConv function
# 
def collectXfemKeys(forElsetCheck,histVars,xfemHistVarsKeys,xfemLabels,crackName,fracParameters,contours,alongCrackFront):

    coord=[]
    if minorVersion > 11:
        elset1 = (xfemHistVarsKeys[0].split('in'))[-1]
        elset = ' in'+ elset1
        for co in xfemLabels:
            x1=xyPlot.XYDataFromHistory(odb=odb, outputVariableName='Coordinate: X at '+crackName+'_XFEM_'+ str(co) + elset,name='tempx', steps=(stepKeys[curStep], )).data[-1][-1]
            y1=xyPlot.XYDataFromHistory(odb=odb, outputVariableName='Coordinate: Y at '+crackName+'_XFEM_'+ str(co) + elset,name='tempy', steps=(stepKeys[curStep], )).data[-1][-1] 
            z1=xyPlot.XYDataFromHistory(odb=odb, outputVariableName='Coordinate: Z at '+crackName+'_XFEM_'+ str(co) + elset,name='tempz', steps=(stepKeys[curStep], )).data[-1][-1] 
            coordTemp=[x1,y1,z1]
            coord.append(coordTemp)
    else:
        for co in xfemLabels:
            x1=xyPlot.XYDataFromHistory(odb=odb, outputVariableName='Coordinate: X at '+crackName+'_XFEM_'+ str(co),name='tempx', steps=(stepKeys[curStep], )).data[-1][-1] 
            y1=xyPlot.XYDataFromHistory(odb=odb, outputVariableName='Coordinate: Y at '+crackName+'_XFEM_'+ str(co),name='tempy', steps=(stepKeys[curStep], )).data[-1][-1] 
            z1=xyPlot.XYDataFromHistory(odb=odb, outputVariableName='Coordinate: Z at '+crackName+'_XFEM_'+ str(co),name='tempz', steps=(stepKeys[curStep], )).data[-1][-1] 
            coordTemp=[x1,y1,z1]
            coord.append(coordTemp)

    #Call a function to Write cooordinates of an xfem crack to .csv file
    writeCoordsXfem(coord,xfemLabels,crackName)
       
    # Call a function to Compute the distance between each crack front and the first crack front 
    #for an xfem crack

    distanceXfem=distxfem(coord,alongCrackFront)

    #Seperate out  XfemHistVarskeys with different parameter names (e.g. 'J-integral:' etc)
    #and append the keys to the parameter lists. Call resultsXfem function for the non-empty lists.
    #    
    
    jList=[]
    tList=[]
    jkList=[]
    cpdList=[]
    ctList=[]
    k1List=[]
    k2List=[]
    k3List=[]


    for var in xfemHistVarsKeys:
        if 'J-integral:' in var:
            jList.append(var)
        if 'T-stress:' in var:
            tList.append(var)
        if 'J-integral estimated from Ks:' in var:
            jkList.append(var)
        if 'Crack propagation direction:' in var:
            cpdList.append(var)
        if 'Ct-integral:' in var:
            ctList.append(var)
        if 'Stress intensity factor K1:' in var:
            k1List.append(var)
        if 'Stress intensity factor K2:' in var:
            k2List.append(var)
        if 'Stress intensity factor K3:' in var:
            k3List.append(var)
           
    #A list which has parameter-list and corresponding parameter name for different parameters
    
    stringList=[(jList,'J-integral:'),(tList,'T-stress:'),(jkList,'J-integral estimated from Ks:'),\
    (cpdList,'Crack propagation direction:'),(ctList,'Ct-integral:'),\
    (k1List,'Stress intensity factor K1:'),(k2List,'Stress intensity factor K2:'),\
    (k3List,'Stress intensity factor K3:')]
    
    strings=[]
    for i in range(len(stringList)):
        strings=[]
        if stringList[i][0]!=[]:            
            for j in stringList[i][0]:
                label=histVars[j].legendLabel
                if minorVersion > 11:
                    splitLabel = (label.split(' '))[-5]
                else:    
                    splitLabel=(label.split())[-1]
                strings.append(splitLabel)
            resultsXfem(strings,stringList[i][0],stringList[i][1],distanceXfem,xfemLabels,contours,
                        crackName,alongCrackFront)           
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#Plot Fracture parameter vs Distance and Labels (node labels in case of 
#conventional crack). Each viewport has only one contour integral parameter and respective contours 
#specified by a user.
#
def plotDistnLabe(data,para,contours,crackName,divider,title,xLegend):

    dict={'J-integral estimated from Ks:':'JKs','Crack propagation direction:':'Cpd',\
         'J-integral:':'J','Stress intensity factor K1:':'K1','Stress intensity factor K2:':'K2',\
         'Stress intensity factor K3:':'K3','Ct-integral:':'Ct','T-stress:':'T'}
    
    if not len(data):
        errMsg = "A plot cannot be created."\
                  "\nCheck if the type of the contour integral specified in the input file is supported by the plug-in."
        raise ValueError, errMsg
    
    if avg1 == ON and len(contours)> 1:
        #Find average XY data
        #
        avgLen=len(data)/len(contours)
        div=range(0, len(data), avgLen)
        div.remove(div[0])
        xavgData = []
        yavgData=[]
        sum=0
        for j in range(avgLen):
            sum=0
            x1=0
            for i in div:
                sum = sum+data[j+i][1]  
            x1 = data[j][1] + sum
            yavgData.append(x1/len(contours))
            xavgData.append(data[j][0])
        avgXY=zip(xavgData,yavgData)
    elif avg1==ON and len(contours)==1:
        print 'The average data will be plotted only when more than one contours are selected.'
        
 
    #Plotting on a viewport for each fracture parameter vs Labels/Distance (with specified contours)
    #for a particular crack name
    #
    
    vPort= session.Viewport(name=dict[para]+':', origin=(10,10), width=150, height=100)
    plotname1=dict[para] +title
    if not session.xyPlots.has_key(plotname1):
        xyPlot1 = session.XYPlot(plotname1)
    else:
        xyPlot1 = session.xyPlots[plotname1]
    chart1 = xyPlot1.charts.values()[0]

    curveList1=[]
    
    for tm in range(len(contours)):
        
        plotData=[]
        for tv in data[tm*divider:(tm+1)*divider]:
            plotData.append(tv)
        
        xyData1 = session.XYData(name=crackName+' Contour-'+str(contours[tm])+'-'+str(dict[para])\
        +title, data=plotData)

        curve1 = session.Curve(xyData1)

        curveList1.append(curve1)

        chart1.setValues(curvesToPlot=curveList1)
        chart1.axes1[0].axisData.setValues(useSystemTitle=False,title=xLegend)
        chart1.axes2[0].axisData.setValues(useSystemTitle=False,title=dict[para])
    
    if avg1 == ON and len(contours)> 1:
        #Add average XY data to the plot
        #
        xyData2 = session.XYData(name=crackName+' Contour-avg-'+str(dict[para])\
            +title, data=avgXY)
        curve2 = session.Curve(xyData2)
        curveList1.append(curve2)
        chart1.setValues(curvesToPlot=curveList1)
    vPort.setValues(displayedObject=xyPlot1)


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#For both conventional and xfem cracks prepares for a plot, for a particular parameter appending XY data for 
#each contour and sending it to plotDistnLabe function for final plotting.
#
def plotPrep(assocDist,assocLabel,para,alongCrackFront,contours,crackName):
    dataLabelPlot=[]
    dataDistPlot=[]
    dataPosPlot=[]
    
    #The XY data coming from 'distLabel' function is for all the contours, 'divider' is used to divide 
    #the data for an each contour.
    if alongCrackFront=='Use labels as X axis for plotting':
        divider = len(assocLabel)/len(contours)
        for tz in range(len(contours)):                     
            for tv in assocLabel[tz*divider:(tz+1)*divider]:
                datLab=(tv[0],tv[1])
                dataLabelPlot.append(datLab)
        xLegend='Labels'
        plotDistnLabe(dataLabelPlot,para,contours,crackName,divider,' vs Labels',xLegend)    
    
    elif alongCrackFront=='Use distance as X axis for plotting':
        divider = len(assocDist)/len(contours)
        for tz in range(len(contours)):
                    for tr in assocDist[tz*divider:(tz+1)*divider]:
                        datDist=(tr[0],tr[1])
                        dataDistPlot.append(datDist)        
        xLegend='Distance from 1st node on crack front'		
        plotDistnLabe(dataDistPlot,para,contours,crackName,divider,' vs Distance from 1st Node',xLegend)	
		
    elif alongCrackFront=='Use crack front position as X axis for plotting':	
        divider = len(assocDist)/len(contours)
        for tz in range(len(contours)):
                    for tr in assocDist[tz*divider:(tz+1)*divider]:
                        datPos=(tr[0],tr[1])
                        dataPosPlot.append(datPos)        
        xLegend='Distance along crack front'
        plotDistnLabe(dataPosPlot,para,contours,crackName,divider,' vs Crack Front Position',xLegend)    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Write cooordinates of an xfem crack to .csv file
def writeCoordsXfem(coord,labels,crackName):
    fname='Coordinates_'+crackName+'.csv'
    try:
      fop=open(fname,'w')
      fop.write('Labels')
      fop.write(',')
      fop.write('X-coord')
      fop.write(',')
      fop.write('Y-coord')
      fop.write(',')
      fop.write('Z-coord')
      fop.write('\n')
      for k in range(len(coord)):
          fop.write(str(labels[k]))
          fop.write(',')
          fop.write(str(coord[k][0]))
          fop.write(',')
          fop.write(str(coord[k][1]))
          fop.write(',')
          fop.write(str(coord[k][2]))
          fop.write('\n')
      fop.close()
    except:
        raise ValueError, fname+' '+'is open and needs to be closed.'
    return
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
#Write cooordinates of a conventional crack to .csv file
def writeCoordsConv(crackTipCoord,crackTipLabels,crackName):
    fname='Coordinates_'+crackName+'.csv'
    try:
        fop=open(fname,'w')
        fop.write('Labels')
        fop.write(',')
        fop.write('X-coord')
        fop.write(',')
        fop.write('Y-coord')
        fop.write(',')
        fop.write('Z-coord')
        fop.write('\n')
        for k in range(len(crackTipCoord)):
            fop.write(str(crackTipLabels[k]))
            fop.write(',')
            fop.write(str(crackTipCoord[k][1][0]))
            fop.write(',')
            fop.write(str(crackTipCoord[k][1][1]))
            fop.write(',')
            fop.write(str(crackTipCoord[k][1][2]))
            fop.write('\n')
        fop.close()
    except:
        raise ValueError, fname+' '+'is open and needs to be closed.'
    return
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Compute the distance/postion for each crack front node for XFEM contour integral
def distxfem(coord,alongCrackFront):   
    if alongCrackFront=='Use distance as X axis for plotting':
        distanceXfem=[]
        count=0
        while count<len(coord):
            xTemp=coord[count][0]-coord[0][0]
            yTemp=coord[count][1]-coord[0][1]
            zTemp=coord[count][2]-coord[0][2]
            distanceXfem.append(sqrt(xTemp**2+yTemp**2+zTemp**2))
            count += 1
        return (distanceXfem)

    elif alongCrackFront=='Use crack front position as X axis for plotting':	
        posXfem, count = [], 0
        while count<len(coord):
            if count == 0:
			    posXfem.append(0.0)
            else:
                xTemp=coord[count][0]-coord[count-1][0]
                yTemp=coord[count][1]-coord[count-1][1]
                zTemp=coord[count][2]-coord[count-1][2]
                posXfem.append(posXfem[-1]+sqrt(xTemp**2+yTemp**2+zTemp**2))
            count += 1	
        return(posXfem)
		
    else:	
        return ([])
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Find the distance/position for each crack front node for a CONVENTIONAL contour integral
def distconv(crackTipCoord,alongCrackFront):   
    if alongCrackFront=='Use distance as X axis for plotting':
        distanceConv=[]
        for cnt in range(len(crackTipCoord)):
            xtemp=crackTipCoord[cnt][1][0]-crackTipCoord[0][1][0]
            ytemp=crackTipCoord[cnt][1][1]-crackTipCoord[0][1][1]
            ztemp=crackTipCoord[cnt][1][2]-crackTipCoord[0][1][2]
            distanceConv.append(sqrt(xtemp**2+ytemp**2+ztemp**2))
        return(distanceConv)

    elif alongCrackFront=='Use crack front position as X axis for plotting':	
        posConv=[]
        for cnt in range(len(crackTipCoord)):
            if cnt==0:
                posConv.append(0.0)
            else:
                xtemp=crackTipCoord[cnt][1][0]-crackTipCoord[cnt-1][1][0]
                ytemp=crackTipCoord[cnt][1][1]-crackTipCoord[cnt-1][1][1]
                ztemp=crackTipCoord[cnt][1][2]-crackTipCoord[cnt-1][1][2]
                posConv.append(posConv[-1]+sqrt(xtemp**2+ytemp**2+ztemp**2))
        return(posConv)
    else:	
        return ([])
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#NOT required now as moinorVersion Constant is used instead of the version info
#Find the version of the ODB
#
# def getVersionInfo():
    # odbName,curStep,curFrame,odb,stepKeys = getCurOdbStepFrame()

    # #####update after SR00376190#####
    # #####Corrector : SKI7#####
    # #####Reason: to accomodate higher versions of Abaqus#####
    # OdbPath=session.odbs[odbName].path
    # versionInfo=getMdbVersionNumbers(OdbPath)
    # jobData = odb.jobData
    # # versionInfo = (((jobData.version).split(" "))[1]).split("-")[0]
    # # if versionInfo== '3DEXPERIENCE':
        # # versionInfo=6.16
    # # else:
        # # versionInfo = float(versionInfo)
    # #####end of correction####### 
    # return versionInfo