from abaqusGui import *
from kernelAccess import *
from abaqusConstants import *

###########################################################################
# Class definition
###########################################################################

class createPartstoolsetGui(AFXToolsetGui):
    [ID_CALL]=range(AFXModuleGui.ID_LAST)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self):

        # Construct the CAE file toolset.
        #
        AFXToolsetGui.__init__(self,'createMultipleParts')
        self.group = AFXToolbarGroup(owner=self,title="Create Parts by Picking Cells")
        (head, tail) = os.path.split(__file__)
        fileLoadPath1 = os.path.join(head, "multiple_parts.png")        
        createIcon = afxCreateIcon(fileLoadPath1)
        createIcon.scale(24,24)
        tgt=pickHandler(self)        
        self.btn = AFXToolButton(self.group,'\tCreate Parts\n by Picking Cells',createIcon, tgt=tgt, sel=0)

        # set the prefix of the menu pane
        getAFXAliasMap().setPrefix(self.group, 'Create Multiple Parts')
        
        # name the component test button
        getAFXAliasMap().setName(self.btn, 'Create Multi Parts')


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getKernelInitializationCommand(self):
        if not getAFXApp().isProductViewer():
            try:
                return 'import abq_CreatePartsFromCells.createPartsFromCellsUtils'
            except:
                return ''
        else:
            return ''
   
###########################################################################
# Class definition
###########################################################################

class pickHandler(AFXProcedure):

    count = 0

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        AFXProcedure.__init__(self,owner)

        self.cmd = AFXGuiCommand(mode=self, method='createMultipleParts', objectName='abq_CreatePartsFromCells.createPartsFromCellsUtils', registerQuery=False)                   
        pickedDefault = ''
        self.pickCellsKw = AFXObjectKeyword(self.cmd, 'pickedCells', TRUE, pickedDefault)
        self.keyword = self.pickCellsKw
        self.prompt = 'Pick Cells'
        self.entitiesToPick = CELLS 
        self.numberToPick = MANY 
        self.highlightLevel=3
        pickHandler.count += 1
        self.setModeName('pickHandler%d' % (pickHandler.count) )

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstStep(self):                  
                
        self.step1=AFXPickStep(self, self.keyword, self.prompt, 
            self.entitiesToPick, self.numberToPick,self.highlightLevel, sequenceStyle=TUPLE)                                
        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getNextStep(self, previousStep):
        
        return None
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getLoopStep(self):
        return None
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def activate(self):
        mainWindow = getAFXApp().getAFXMainWindow()
        if getDisplayedObjectType() == PART:
            self.vp = session.viewports[session.currentViewportName]
            self.vpModelName = self.vp.displayedObject.modelName
            self.vpPartName = self.vp.displayedObject.name
            self.part = mdb.models[self.vpModelName].parts[self.vpPartName]
            if self.part.queryGeometry(printResults=False)['category'] == 'Geometry':
                if self.part.space == THREE_D:
                    if not self.part.cells:
                        showAFXErrorDialog(mainWindow,
                            'The part does not have cells.')
                        return None                                                  
                else:
                    showAFXErrorDialog(mainWindow,
                        'The plug-in is supported only for 3D parts.')
                    return None
            else:
                showAFXErrorDialog(mainWindow,
                    'The plug-in is supported only for native parts.')
                return None
            AFXProcedure.activate(self)
        else:
            showAFXErrorDialog(mainWindow,
            'A part must be displayed in the current viewport.')
            return None
        
