from abaqusConstants import *
from abaqus import *


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createMultipleParts(pickedCells=None):
    
    
    #Get the information regarding the displayed object in the current viewport
    #
    curViewport = session.viewports[session.currentViewportName]
    modName=curViewport.displayedObject.modelName
    partName=curViewport.displayedObject.name
    model=mdb.models[modName]
    origPart=model.parts[partName]
    
        
    #Check wheather number of picked cells are same as number of the total cells
    #
    if len(pickedCells)==len(model.parts[partName].cells):
        msg = "The number of picked cells cannot be same as the total number of the part cells."
        raise ValueError, msg
    

    #Generate part names of picked and not picked cells
    #
    notPickedPartName=generatePartName(model, partName,'notPicked')
    pickedPartName=generatePartName(model, partName, 'picked')
    
    #Create two parts by copying the original part which the customer picked
    #the 1st one is for picked cells and the 2nd is for non picked cells
    #
    model.Part(name=pickedPartName, objectToCopy=model.parts[partName])
    model.Part(name=notPickedPartName, objectToCopy=model.parts[partName])
    
       
    part1=model.parts[pickedPartName]
    part=model.parts[notPickedPartName] 

    #Create a list of the total face indexes for both the parts. Below we have used 'part faces'
    #we could have used 'part1 faces' as well because since they are copied parts they have same face indexes.
    #

    f=part.faces
    f1=part1.faces    
    
    fIndex=[]          
    fIndex=[i.index for i in f]
    
    # Create a list of cells for the 'nonpickedPart' using the indexs of the picked cells of the original part
    # We could have used 'pickedPart' also.
    #

    inx=[]
    pickedCells1=[]      
    cells=part.cells
    
    inx=[i.index for i in pickedCells]
    pickedCells1=[cells[i] for i in inx]
           

    #Create a list of face indexes in the picked and non picked cells in the 
    #'part'
    pickedfaces, nonpickedfaces, tFCellsIndexes = getPickedNonPickedFaces(cells, pickedCells1)
      
    #Creating a list of the face indexes to be removed from part1
    #to make 'part' with the non-picked cells
    #    
    
    rmvList=[]            
    rmvList=[i for i in pickedfaces if i not in nonpickedfaces]       
    fList=generateString(rmvList,"f[")   
    faceList=eval(fList)
    
    #Creating a list of the face indexes to be removed from part1
    #to make part1 with only picked cells
    #
    rmvList1=[] 

    rmvList1=[i for i in nonpickedfaces if i not in pickedfaces]
    
    #Creating a list of the face indexes, which are in the part1(supposed to be the pickedPart) 
    #but not in the picked cells,and those faces should be removed.It's done to remove mixed 
    #dimensionality of the newly created part
    #
    
    rmvList2=[]
    if part1.queryAttributes(printResults=False)['numShellFaces']: 
        rmvList2=[i for i in fIndex if i not in tFCellsIndexes]             

        
    #Adding the two lists to create a new list of face indxes, which will be used to
    #remove faces from part1 and create 'part1' with only picked cells.
    #
   
    rmvList3=rmvList2+rmvList1
    f1List=generateString(rmvList3,"f1[")
    faceList1=eval(f1List)
 
    
    #Remove the required faces of the parts so that parts are created
    #according to the requirements
    #
  
    part.RemoveFaces(faceList, False)
  
    part1.RemoveFaces(faceList1, False)


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""
Function generates a unique part name
"""
def generatePartName(model, partName, Flag):
    existingPartNames=model.parts.keys()
    count=1        
    while 1:
        if partName+'_'+Flag+'_'+str(count) in existingPartNames:
            count+=1
            continue
        else:
            newPartName=partName+'_'+Flag+'_'+str(count)
        break
    if len(newPartName) > 38 or len(newPartName)==38:
        msg='Length of the newly created part name exceeds 38 characters.\n' \
            'Try reducing the length of the original part name.'
        raise ValueError, msg
    return newPartName    
    

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""
Function generates a string that will be used as the geometric sequence of 
the faces to be removed.
"""    
def generateString(rmvList,faceString):

    string=''
    for fc in range(len(rmvList)):
        
        if rmvList[fc]!=rmvList[-1]:
    
            string=string+faceString+str(rmvList[fc])+":"+str(rmvList[fc]+1)+"]+"
        else:
    
            string=string+faceString+str(rmvList[fc])+":"+str(rmvList[fc]+1)+"]"    
    return string
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""
Function generates indexes of picked and nonpicked faces to be removed along 
with total face indexes
"""     
def getPickedNonPickedFaces(cells, pickedCells1):
    nonpickedfaces=[]
    pickedfaces = [] 
    tFCellsIndexes=[]
    for cell in cells:
        for j in cell.getFaces():
            tFCellsIndexes.append(j)
        if cell not in pickedCells1:      
            for x in cell.getFaces():
                nonpickedfaces.append(x)
        else:
            for y in cell.getFaces():
                pickedfaces.append(y)

    pickedfaces=set(pickedfaces)
    nonpickedfaces=set(nonpickedfaces)

    return pickedfaces, nonpickedfaces, tFCellsIndexes
