from abaqusGui import *
from dymolaConstants import *

###########################################################################
# Class definition
###########################################################################

class CreateDymolaForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
                
        AFXForm.__init__(self, owner)
        self.dymNameKw = AFXStringKeyword(command=None,
                                          name='dymFileName',
                                          isRequired=False,
                                          defaultValue="LogicalJob-1")
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        from abq_Dymola.dymolaDB import DymolaDB
        self.dymNameKw.setValue(self.getUniqueJobName())
        dtext = 'Create New Dymola File'
        self.dymolaDB = DymolaDB(self, NEW, dtext)
        return self.dymolaDB

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getUniqueJobName(self):
        """
        This function gets a unique name for the dymola job
        """
        import os.path
        baseStr = "LogicalJob-"
        counter = 1
        while True: # Infinite Loop
            dymJobName = baseStr + str(counter)
            extFileName = dymJobName + '.mo'
            if not os.path.exists(extFileName):
                return dymJobName
            counter += 1
        return None
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):
        """
        """
        
        mainWindow = self.getCurrentDialog()

        # Check valid Dymola model name
        dymFName = self.dymNameKw.getValue()
        if not dymFName:
            errMsg = "Dymola file name should be specified."
            showAFXErrorDialog(mainWindow, errMsg)
            self.dymolaDB.dymFileNameTF.setFocusToTextField()
            return 0

        status = self.validNameCheck(dymFName, 'Dymola file ')
        if not status:
            self.dymolaDB.dymFileNameTF.setFocusToTextField()
            return 0
        
        errMsg = 'No %s have been selected for export.\nSelect at least '\
                 'one %s to export from Abaqus to Dymola.'

        # Check if sensors and actuators are specified
        numSenItems = self.dymolaDB.expSenList.getNumItems()
        numActItems = self.dymolaDB.expActList.getNumItems()
        
        if not numSenItems:
            showAFXErrorDialog(mainWindow, errMsg % ('sensors', 'sensor'))
            return 0
        
        if not numActItems:
            showAFXErrorDialog(mainWindow, errMsg % ('actuators', 'actuator'))
            return 0

        self.dymolaDB.plantExport()        
        return 1
                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def validNameCheck(self, name, type):
        """
        This method checks text fields for valid characters.
        """

        errMsg = 'Invalid name.\n'
        errMsg = errMsg + '\n'
        errMsg = errMsg + type + ' names must be 1-38 characters long,\n'
        errMsg = errMsg + 'may not begin or end with an underscore,\n'
        errMsg = errMsg + 'may not beign with a dash, and may not,\n'
        errMsg = errMsg + 'contain non-printable characters or the\n'
        errMsg = errMsg + 'following characters:\n'
        errMsg = errMsg + '\n'
        errMsg = errMsg + '<space> . $ & * ~ ! ()\n'
        errMsg = errMsg + '[] {} | ; : ` < > ? " / \' \\'
        
        invalidList = [' ', '.', '$', '&', '*', '~', '`', '!',
                       '(', ')', '[', ']', '{', '}', '|', '/',
                       ';', ':', "'", '"', '<', '>', '?', '.', '\\']

        mainWindow = self.getCurrentDialog()

        if len(name) > 38:
            showAFXErrorDialog(mainWindow, errMsg)
            return 0
        elif name[0] in ['_', '-'] or name[-1] in ['_', '-']:
            showAFXErrorDialog(mainWindow, errMsg)
            return 0
        else:
            for i in name:
                if i in invalidList:
                    showAFXErrorDialog(mainWindow, errMsg)
                    return 0
        
        return 1
