from abaqusGui import *

###########################################################################
# Class definition
###########################################################################

class DymolaActNameDB(AFXDataDialog):
    numInstance = 0
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):
        DymolaActNameDB.numInstance += 1
        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'Create Actuator Amplitude',
                               0, DECOR_RESIZE|DIALOG_ACTIONS_SEPARATOR)
        self.form = form
        self.appendActionButton(text='OK',tgt=self,
                                sel=self.ID_CLICKED_OK)
        self.appendActionButton(text='Cancel',tgt=self,
                                sel=self.ID_CLICKED_CANCEL)
        getAFXAliasMap().setPrefix(self, 'dymolaActName-%d'
                                           % DymolaActNameDB.numInstance)
        FXMAPFUNC(self,SEL_COMMAND,self.ID_CLICKED_CANCEL,DymolaActNameDB.onCancel)
        FXMAPFUNC(self,SEL_COMMAND,self.ID_CLICKED_OK,DymolaActNameDB.onOK)
        self.actNameTF = AFXTextField(p=self, ncols=20, labelText='Actuator name:', tgt=None, sel=0,
                                      opts=AFXTEXTFIELD_STRING|LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
                                      pl=DEFAULT_PAD, pr=DEFAULT_PAD, pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.actNameTF, 'actNameTF')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onOK(self,sender,sel,ptr):
        """
        """
        from kernelAccess import mdb
        
        modelName = getCurrentContext()['modelName']
        mainWindow = getAFXApp().getAFXMainWindow()
        message0 = "A name must be entered."
        message1 = '"%s" already exists and cannot be overwritten.'
        message2 = "Names are case sensitive, but case retaining.\n\n"
        
        if not self.actNameTF.getText():
            showAFXErrorDialog(mainWindow, message0)
            return 0

        retVal = self.form.validNameCheck(self.actNameTF.getText(), 'Amplitude')
        if not retVal:
            return 0
        amps = mdb.models[modelName].amplitudes.keys()
        if self.actNameTF.getText() in amps:
            showAFXErrorDialog(mainWindow, message1 %self.actNameTF.getText())
            return 0

        ampsl = [i.lower() for i in amps]
        if self.actNameTF.getText().lower() in ampsl:
            msg = message2 + message1
            showAFXErrorDialog(mainWindow, msg %self.actNameTF.getText())
            return 0
        
        mdb.models[modelName].ActuatorAmplitude(name=self.actNameTF.getText())
        mdb.models[modelName].amplitudes.keys()
        self.hide()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCancel(self,sender,sel,ptr):
        """
        """
        
        self.hide()
