from abaqusGui import *
from dymolaConstants import *
import rwDymFile

###########################################################################
# Class definition
###########################################################################

class DymolaDB(AFXDataDialog):
    [
        ID_SENSOR,
        ID_ACTUATOR,
        ID_ADD_S,
        ID_SELECT_ALL_S,
        ID_REMOVE_S,
        ID_CLEAR_ALL_S,
        ID_ADD_A,
        ID_SELECT_ALL_A,
        ID_REMOVE_A,
        ID_CLEAR_ALL_A,
        ID_OVERWRITE_FILE,
        ID_OVER_SIGNALS,
        ID_SELECT_FILE,
        ID_FILE_OK,
        ID_WARN_IMPACT,
        ID_FILE_DYMOLA,
        ID_LIST_CLICKED
        ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+17)
    numInstance = 0
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form, mode, title):
        DymolaDB.numInstance += 1
        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, title,0,
                               DECOR_RESIZE)

        self.mode = mode
        self.form = form
        if self.mode == NEW:
            textok = "OK"
        elif self.mode == UPDATE:
            textok = 'Check Signals'
        self.okBtn = self.appendActionButton(text=textok,
                                                tgt=self,
                                                sel=self.ID_CLICKED_APPLY)
        self.okBtn.setDefault(enable=True)
        self.cancelBtn = self.appendActionButton(text='Cancel',
                                                tgt=self,
                                                sel=self.ID_CLICKED_CANCEL)
        #getAFXAliasMap().setPrefix(self, 'dymolaDB')
        #GUILOG
        getAFXAliasMap().setPrefix(self, 'dymola-%d'
                                           % DymolaDB.numInstance)
        #ENDGUILOG
        if self.mode == NEW:
            self.chkBtn = FXCheckButton(p=self, text='Launch Dymola on OK', tgt=None, sel=0,
                                        opts=CHECKBUTTON_NORMAL|LAYOUT_SIDE_BOTTOM, x=0, y=0, w=0, h=0,
                                        pl=DEFAULT_PAD, pr=DEFAULT_PAD, pt=DEFAULT_PAD,
                                        pb=DEFAULT_PAD)
        
            getAFXAliasMap().setName(self.chkBtn, 'chkBtn1')
            self.chkBtn.setCheck(True)

        vfs0 = FXVerticalFrame(p=self, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
                               pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING,
                               vs=DEFAULT_SPACING)
        if self.mode == NEW:
            self.dymFileNameTF = AFXTextField(p=vfs0, ncols=20, labelText='Dymola model name:',
                                              tgt=self.form.dymNameKw, sel=0,
                                              opts=AFXTEXTFIELD_STRING|LAYOUT_FILL_X, x=0, y=0, w=0,
                                              h=0, pl=DEFAULT_PAD, pr=DEFAULT_PAD, pt=DEFAULT_PAD,
                                              pb=DEFAULT_PAD)
            getAFXAliasMap().setName(self.dymFileNameTF, 'dymFileNameTF1')
        elif self.mode == UPDATE:
            self.fileDb = None
            hf = FXHorizontalFrame(p=vfs0, opts=0, x=0, y=0, w=0, h=0,
                                   pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING,
                                   vs=DEFAULT_SPACING)
            self.dymFileNameTF = AFXTextField(p=hf, ncols=20, labelText='Dymola file name:',
                                              tgt=form.dymNameKw, sel=0,opts=AFXTEXTFIELD_STRING|JUSTIFY_RIGHT,
                                              x=0, y=0, w=0, h=0, pl=DEFAULT_PAD, pr=DEFAULT_PAD,
                                              pt=DEFAULT_PAD, pb=DEFAULT_PAD)
            getAFXAliasMap().setName(self.dymFileNameTF, 'dymFileNameTF2')
            self.dymFileNameTF.setEditable(False)
            self.selecFileBtn=FXButton(p=hf, text='Select...', ic=None, tgt=self, sel=self.ID_SELECT_FILE,
                     opts=BUTTON_NORMAL, x=0, y=0, w=0, h=0, pl=DEFAULT_PAD, pr=DEFAULT_PAD,
                     pt=DEFAULT_PAD, pb=DEFAULT_PAD)
            getAFXAliasMap().setName(self.selecFileBtn, 'selectFileBtn')
            FXMAPFUNC(self, SEL_COMMAND, self.ID_SELECT_FILE, DymolaDB.onCmdFile)
            self.dbtn = FXButton(p=vfs0, text='Open file in Dymola...', ic=None, tgt=self,
                                 sel=self.ID_FILE_DYMOLA, opts=BUTTON_NORMAL, x=0, y=0,
                                 w=0, h=0, pl=DEFAULT_PAD, pr=DEFAULT_PAD,
                                 pt=DEFAULT_PAD, pb=DEFAULT_PAD)
            getAFXAliasMap().setName(self.dbtn, 'dymolabtn')
            FXMAPFUNC(self, SEL_COMMAND, self.ID_FILE_DYMOLA, DymolaDB.openDymolaFile)
            self.dbtn.disable()            
        gb = FXGroupBox(vfs0, 'Signal Options', FRAME_GROOVE|LAYOUT_FILL_X|
                            LAYOUT_FILL_Y|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW)        
        tabBook = FXTabBook(p=gb, tgt=None, sel=0, opts=TABBOOK_NORMAL|LAYOUT_FILL_X|LAYOUT_FILL_Y,
            x=0, y=0, w=0, h=0, pl=DEFAULT_SPACING, pr=DEFAULT_SPACING,
            pt=DEFAULT_SPACING, pb=DEFAULT_SPACING)
        self.tb2=FXTabItem(p=tabBook, text='Sensors', ic=None, opts=TAB_TOP_NORMAL|LAYOUT_FILL_Y,
            x=0, y=0, w=0, h=0, pl=6, pr=6, pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.tb2, 'tb2')
        tabVf = FXVerticalFrame(p=tabBook, opts=FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,
            x=0, y=0, w=0, h=0, pl=DEFAULT_SPACING, pr=DEFAULT_SPACING,
            pt=DEFAULT_SPACING, pb=DEFAULT_SPACING, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        self.sensBtn=FXButton(p=tabVf, text='Create Sensor...', ic=None, tgt=self, sel=self.ID_SENSOR, opts=BUTTON_NORMAL,
            x=0, y=0, w=0, h=0, pl=DEFAULT_PAD, pr=DEFAULT_PAD, pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.sensBtn, 'sensorBtn')
        FXMAPFUNC(self,SEL_COMMAND,self.ID_SENSOR,DymolaDB.createSensor)
        hfM = FXMatrix(tabVf,3,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0)
        if self.mode == NEW:
            text1 = 'Available Signals'
            text2 = 'Selected Signals'
        elif self.mode == UPDATE:
            text1 = 'Abaqus Signals'
            text2 = 'Dymola Signals'
            
        gb = FXGroupBox(hfM, text1, FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y|
                        LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW)
        hfM1 = FXHorizontalFrame(gb, LAYOUT_FILL_X|FRAME_SUNKEN|LAYOUT_FILL_Y|FRAME_THICK,
                                 0,0,0,0, 0,0,0,0)             
        self.abqSenList = AFXList(hfM1, 5, self, self.ID_LIST_CLICKED, 
            LAYOUT_FILL_X|LAYOUT_FILL_Y|AFXLIST_NO_AUTOCOMMIT)
        getAFXAliasMap().setName(self.abqSenList, 'abqsenList')
        if self.mode == NEW:
            vfM = FXVerticalFrame(hfM,LAYOUT_CENTER_Y|LAYOUT_FILL_ROW)
            va = AFXVerticalAligner(vfM)
        
            FXMAPFUNC(self,SEL_COMMAND,self.ID_ADD_S,DymolaDB.onCmdAdd)
            self.sasBtn = FXButton(va,'>\tAdd selected',None,self,self.ID_ADD_S)
            getAFXAliasMap().setName(self.sasBtn, 'sasBtn1')
            FXMAPFUNC(self,SEL_COMMAND,self.ID_SELECT_ALL_S,DymolaDB.onCmdSelectAll)
            sabtn = FXButton(va,'>>\tAdd all',None,self,self.ID_SELECT_ALL_S)
            getAFXAliasMap().setName(sabtn, 'sabtn1')
            self.saaBtn = sabtn
            FXMAPFUNC(self,SEL_COMMAND,self.ID_REMOVE_S,DymolaDB.onCmdRemove)
            self.srsBtn = FXButton(va,'<\tRemove selected',None,self,self.ID_REMOVE_S)
            getAFXAliasMap().setName(self.srsBtn, 'srsBtn')
            FXMAPFUNC(self,SEL_COMMAND,self.ID_CLEAR_ALL_S,DymolaDB.onCmdClearAll)
            rabtn = FXButton(va,'<<\tRemove all',None,self,self.ID_CLEAR_ALL_S)
            getAFXAliasMap().setName(rabtn, 'rabtn')
            self.sraBtn = rabtn
            
        gb = FXGroupBox(hfM, text2, FRAME_GROOVE|LAYOUT_FILL_X|
                        LAYOUT_FILL_Y|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW)
        hfM2 = FXHorizontalFrame(gb, LAYOUT_FILL_X|LAYOUT_FILL_Y|
                                 FRAME_SUNKEN|FRAME_THICK,0,0,0,0, 0,0,0,0)
        self.expSenList = AFXList(hfM2, 5, self, self.ID_LIST_CLICKED, LAYOUT_FILL_X|
                                  LAYOUT_FILL_Y|AFXLIST_NO_AUTOCOMMIT,0,0,0,0)
        getAFXAliasMap().setName(self.expSenList, 'expSenlist')
        
        self.tb1=FXTabItem(p=tabBook, text='Actuators', ic=None, opts=TAB_TOP_NORMAL|LAYOUT_FILL_Y,
            x=0, y=0, w=0, h=0, pl=6, pr=6, pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.tb1, 'tb1')
        tabVf = FXVerticalFrame(p=tabBook, opts=FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_Y,
            x=0, y=0, w=0, h=0, pl=DEFAULT_SPACING, pr=DEFAULT_SPACING,
            pt=DEFAULT_SPACING, pb=DEFAULT_SPACING, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        self.actBtn = FXButton(p=tabVf, text='Create Actuator...', ic=None, tgt=self, sel=self.ID_ACTUATOR, opts=BUTTON_NORMAL,
            x=0, y=0, w=0, h=0, pl=DEFAULT_PAD, pr=DEFAULT_PAD, pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        FXMAPFUNC(self,SEL_COMMAND,self.ID_ACTUATOR,DymolaDB.createActuator)
        getAFXAliasMap().setName(self.actBtn, 'actbtn')

        hfMa = FXMatrix(tabVf,3,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0)
        gb = FXGroupBox(hfMa, text1, FRAME_GROOVE|LAYOUT_FILL_X|
                        LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_Y)
        hfMa1 = FXHorizontalFrame(gb, LAYOUT_FILL_X|LAYOUT_FILL_Y|
                                 FRAME_SUNKEN|FRAME_THICK,0,0,0,0, 0,0,0,0)
        self.abqActList = AFXList(hfMa1, 5, self, self.ID_LIST_CLICKED, LAYOUT_FILL_X|
                                  LAYOUT_FILL_Y|AFXLIST_NO_AUTOCOMMIT,0,0,0,0)
        getAFXAliasMap().setName(self.abqActList, 'abqactlist')
        if self.mode == NEW:
            vfMa = FXVerticalFrame(hfMa,LAYOUT_CENTER_Y|LAYOUT_FILL_ROW)
            va = AFXVerticalAligner(vfMa)
               
            self.aasBtn = FXButton(va,'>\tAdd selected',None,self,self.ID_ADD_A)
            getAFXAliasMap().setName(self.aasBtn, 'aasBtn')
            sabtn = FXButton(va,'>>\tAdd all',None,self,self.ID_SELECT_ALL_A)
            getAFXAliasMap().setName(sabtn, 'sabBtn')
            self.aaaBtn = sabtn
            self.arsBtn = FXButton(va,'<\tRemove selected',None,self,self.ID_REMOVE_A)
            getAFXAliasMap().setName(self.arsBtn, 'arsbtn')
            rabtn = FXButton(va,'<<\tRemove all',None,self,self.ID_CLEAR_ALL_A)
            getAFXAliasMap().setName(rabtn, 'rabtn1')
            self.araBtn = rabtn

            FXMAPFUNC(self,SEL_COMMAND,self.ID_ADD_A,DymolaDB.onCmdAdd)
            FXMAPFUNC(self,SEL_COMMAND,self.ID_SELECT_ALL_A,DymolaDB.onCmdSelectAll)
            FXMAPFUNC(self,SEL_COMMAND,self.ID_REMOVE_A,DymolaDB.onCmdRemove)
            FXMAPFUNC(self,SEL_COMMAND,self.ID_CLEAR_ALL_A,DymolaDB.onCmdClearAll)

        gb = FXGroupBox(hfMa, text2, FRAME_GROOVE|LAYOUT_FILL_X|
                        LAYOUT_FILL_Y|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW)
        hfMa2 = FXHorizontalFrame(gb, LAYOUT_FILL_X|LAYOUT_FILL_Y|
                                 FRAME_SUNKEN|FRAME_THICK,0,0,0,0, 0,0,0,0)
        self.expActList = AFXList(hfMa2, 5, self, self.ID_LIST_CLICKED, LAYOUT_FILL_X|
                                  LAYOUT_FILL_Y|AFXLIST_NO_AUTOCOMMIT,0,0,0,0)
        getAFXAliasMap().setName(self.expActList, 'expActList')
            
        FXMAPFUNC(self,SEL_COMMAND,self.ID_OVERWRITE_FILE,DymolaDB.overwrite)
        FXMAPFUNC(self,SEL_COMMAND,self.ID_OVER_SIGNALS,DymolaDB.overwrite)
        self.initVal()
        self.fileNameTgt = AFXStringTarget()
        FXMAPFUNC(self,SEL_COMMAND,self.ID_FILE_OK,DymolaDB.appendTable)
        self.impSenList = []
        self.impActList = []
        FXMAPFUNC(self,SEL_COMMAND,self.ID_WARN_IMPACT,DymolaDB.overwrite)
        FXMAPFUNC(self, SEL_SELECTED, self.ID_LIST_CLICKED, DymolaDB.disableButtons)
        FXMAPFUNC(self, SEL_DESELECTED, self.ID_LIST_CLICKED, DymolaDB.disableButtons)
        self.disableButtons(None, None, None)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def disableButtons(self, sender, sel, ptr):
        """
        This function enable/disables buttons based on whether any list item is selected or not
        """
        if self.mode != NEW:
            return
        listObjs = [self.abqSenList, self.expSenList, self.abqActList, self.expActList]
        listObjSelected = [False, False, False, False]
        listObjEmpty = [False, False, False, False]
        selectedBtns = [self.sasBtn, self.srsBtn, self.aasBtn, self.arsBtn]
        allBtns = [self.saaBtn, self.sraBtn, self.aaaBtn, self.araBtn]
        
        l = len(listObjs)
        for i in range(l):
            listObj = listObjs[i]
            num = listObj.getNumItems()
            if num == 0:
                listObjEmpty[i] = True
            for j in range(num):
                if listObj.isItemSelected(j):
                    listObjSelected[i] = True
                    break
        for i in range(l):
            if listObjSelected[i]:
                selectedBtns[i].enable()
            else:
                selectedBtns[i].disable()
            if listObjEmpty[i]:
                allBtns[i].disable()
            else:
                allBtns[i].enable()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
        """
        """
        from kernelAccess import session
        from kernelAccess import mdb

        modelName = self.returnModelName()
        mdb.models[modelName].amplitudes.registerQuery(self.updateAbqActList, False)
        mdb.models[modelName].historyOutputRequests.registerQuery(self.updateAbqSenList, False)
        
        AFXDataDialog.show(self)
         
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):
        """
        """
        from kernelAccess import session
        
        modelName = self.returnModelName()
        mdb.models[modelName].amplitudes.unregisterQuery(self.updateAbqActList)
        mdb.models[modelName].historyOutputRequests.unregisterQuery(self.updateAbqSenList)
        # No harm in unregistering twice. What is happening is that in some cases that registerquery has been called more twice before coming here. So by unregistering query twice we avoid our callback function to be called
        mdb.models[modelName].amplitudes.unregisterQuery(self.updateAbqActList)
        mdb.models[modelName].historyOutputRequests.unregisterQuery(self.updateAbqSenList)
        
        AFXDataDialog.hide(self)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def initVal(self):
        """
        """
        self.updateAbqSenList()
        self.updateAbqActList()
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateList(self, stype):
        """
        Get the sensor and actuator infor from repositories.
        """
        
        outList = []
        
        from kernelAccess import mdb
        modelName = self.returnModelName()
        
        if stype == 'S':                                
            hoReqKeys = mdb.models[modelName].historyOutputRequests.keys()
            for key in hoReqKeys:
                hoReq = mdb.models[modelName].historyOutputRequests[key]
                if hoReq.sensor and hoReq.suppressed!=True:
                    outList.append(key)
        elif stype == 'A':
            from kernelAccess import kernelAccessClient
            ampKeys = mdb.models[modelName].amplitudes.keys()

            for key in ampKeys:
                ampObj = mdb.models[modelName].amplitudes[key]                       
                if str(type(ampObj))=="<class 'kernelAccess.ActuatorAmplitudeProxy'>" or str(type(ampObj))=="<class 'kernelAccessClient.ActuatorAmplitudeProxy'>" :
                    outList.append(key)                                        
        return outList
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateAbqSenList(self):
        """
        """

        # This function should not be called when the mode is inactive
        if not self.form.isActive():
            return 0

        self.allSenList = []
        self.probSenList = []
        self.probSList = []
        
        self.asErrMsg = "The following %s names contains charact"\
                        "ers that are not supported in Dymola\nand "\
                        "will not be listed in the plug-in:"
	self.asNoteMsg = "Note: Sensor and actuator names cannot contain "\
                         "special characters other than '_'."
        self.asErrMsg1 = "Do you want these %s to be automatically renamed"\
                         " to a valid name in Abaqus?\n"
        
        # Initialize the list
        noItem = self.abqSenList.getNumItems()
        if noItem:
            self.abqSenList.clearItems()
                        
        from kernelAccess import mdb
        
        self.allSenList = self.updateList('S')
        
        self.probSenList = self.probListFunc(self.allSenList)

        if self.probSenList:
            self.probSList = self.validSenActName(self.probSenList)
            if self.probSList:
                topMsg = self.asErrMsg %('sensor')
                tList = [i[0] for i in self.probSList]
                items = ','.join(tList)
                btmMsg = self.asErrMsg1 %('sensors') + '\n' + self.asNoteMsg
                
                mainWindow = getAFXApp().getAFXMainWindow()
                modelName = self.returnModelName()
                mdb.models[modelName].historyOutputRequests.unregisterQuery(self.updateAbqSenList)
                self.disableButtons(None, None, None)
                showAFXItemsWarningDialog(mainWindow, topMsg, items, btmMsg, 
                                          AFXDialog.YES|AFXDialog.NO,
                                          self, self.ID_OVER_SIGNALS)
                return 0
            else:
                for item in self.allSenList:
                    self.abqSenList.appendItem(item)
        else:
            for item in self.allSenList:
                self.abqSenList.appendItem(item)
        self.disableButtons(None, None, None)
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateAbqActList(self):
        """
        """

        # This function should not be called when the mode is inactive
        if not self.form.isActive():
            return 0

        self.allActList = []
        self.probActList = []
        self.probAList = []
        
        self.asErrMsg = "The following %s names contains charact"\
                        "ers that are not supported in Dymola\nand "\
                        "will not be listed in the plug-in:"
	self.asNoteMsg = "Note: Sensor and actuator names cannot contain "\
                         "special characters other than '_'."
        self.asErrMsg1 = "Do you want these %s to be automatically renamed"\
                         " to a valid name in Abaqus?\n"
        
        # Initialize the list
        noItem = self.abqActList.getNumItems()       
        if noItem:
            self.abqActList.clearItems()
                        
        self.allActList = self.updateList('A')
        
        self.probActList = self.probListFunc(self.allActList)
        if self.probActList:
            self.probAList = self.validSenActName(self.probActList)
            if self.probAList:                
                topMsg = self.asErrMsg %('actuator')
                tList = [i[0] for i in self.probAList]
                items = ','.join(tList)
                btmMsg = self.asErrMsg1 %('actuators') + '\n' + self.asNoteMsg
                
                mainWindow = getAFXApp().getAFXMainWindow()
                modelName = self.returnModelName()
                mdb.models[modelName].amplitudes.unregisterQuery(self.updateAbqActList)
                self.disableButtons(None, None, None)
                showAFXItemsWarningDialog(mainWindow, topMsg, items, btmMsg, 
                                          AFXDialog.YES|AFXDialog.NO,
                                          self, self.ID_OVER_SIGNALS)
                return 0
            else:
                for item in self.allActList:
                    self.abqActList.appendItem(item)
        else:
            for item in self.allActList:
                self.abqActList.appendItem(item)
        self.disableButtons(None, None, None)
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdAdd(self,sender,sel,ptr):
        """
        To add data to the list
        """

        if SELID(sel) == self.ID_ADD_S:
            numItems = self.abqSenList.getNumItems()
            for i in range(numItems): 
                if self.abqSenList.isItemSelected(i):
                    comp = self.abqSenList.getItemText(i)
                    if self.mode == NEW:
                        if self.expSenList.findItem(comp) == -1:
                            self.expSenList.appendItem(comp)
                    elif self.mode == UPDATE:                        
                        if comp.upper() not in self.impSenList:
                            self.expSenList.appendItem(comp)
                            
            self.abqSenList.killSelection()
        elif SELID(sel) == self.ID_ADD_A:
            numItems = self.abqActList.getNumItems()
            for i in range(numItems): 
                if self.abqActList.isItemSelected(i):
                    comp = self.abqActList.getItemText(i)
                    if self.mode == NEW:
                        if self.expActList.findItem(comp) == -1:                
                            self.expActList.appendItem(comp)
                    elif self.mode == UPDATE:
                        if comp.upper() not in self.impActList:
                            self.expActList.appendItem(comp)
                            
            self.abqActList.killSelection()
        self.disableButtons(None, None, None)
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdSelectAll(self,sender,sel,ptr):
        """
        Move all items between lists.
        """

        if SELID(sel) == self.ID_SELECT_ALL_S:
            self.expSenList.clearItems()
            numItems = self.abqSenList.getNumItems()
            for i in range(numItems):
                comp = self.abqSenList.getItemText(i)
                self.expSenList.appendItem(comp)
        elif SELID(sel) == self.ID_SELECT_ALL_A:
            self.expActList.clearItems()
            numItems = self.abqActList.getNumItems()
            for i in range(numItems):
                comp = self.abqActList.getItemText(i)
                self.expActList.appendItem(comp)
        self.disableButtons(None, None, None)        
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdRemove(self,sender,sel,ptr):
        """
        Remove item from list.
        """
        
        mainWindow = getAFXApp().getAFXMainWindow()
        errmsg = "This operation is not valid for %s imported from Dymola."
        if SELID(sel) == self.ID_REMOVE_S:
            numItems = self.expSenList.getNumItems()
            index = 0
            for i in range(numItems):             
                if self.expSenList.isItemSelected(index):
                    if self.mode == UPDATE:
                        sensor = self.expSenList.getItemText(index)
                        if sensor in self.impSenList:
                            showAFXErrorDialog(mainWindow, errmsg % 'Sensors')
                            return 0
                    self.expSenList.removeItem(index)
                    self.disableButtons(None, None, None)
                else:
                    index += 1
        elif SELID(sel) == self.ID_REMOVE_A:
            numItems = self.expActList.getNumItems()
            index = 0
            for i in range(numItems):             
                if self.expActList.isItemSelected(index):
                    if self.mode == UPDATE:
                        actuator = self.expActList.getItemText(index)
                        if actuator in self.impActList:
                            showAFXErrorDialog(mainWindow, errmsg % 'Actuators')
                            return 0
                    self.expActList.removeItem(index)
                    self.disableButtons(None, None, None)
                else:
                    index += 1

        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdClearAll(self,sender,sel,ptr):
        """
        Clear all items from list.
        """

        if SELID(sel) == self.ID_CLEAR_ALL_S:
            self.expSenList.clearItems()
        elif SELID(sel) == self.ID_CLEAR_ALL_A:
            self.expActList.clearItems()
        self.disableButtons(None, None, None)

        return 1
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def plantExport(self):
        
        import os, uti
        import checkFileExt
        
        self.senList = []
        self.actList = []
        
        mainWindow = getAFXApp().getAFXMainWindow()

        numItems = self.expSenList.getNumItems()
        for i in range(numItems):
            self.senList.append(self.expSenList.getItemText(i))
            
        numItems = self.expActList.getNumItems()
        for i in range(numItems):
            self.actList.append(self.expActList.getItemText(i))        
                
        # Check to see if file already exists
        thisDir = os.getcwd()
        dirSep = os.sep
        self.dymFileName = self.form.dymNameKw.getValue()
        extFileName = checkFileExt.addExt(self.dymFileName, '.mo')
        self.filePath = thisDir + dirSep + extFileName
        fileList = os.listdir(thisDir)
        self.extFileName = extFileName

        if extFileName in fileList:
            message = "Dymola file %s already exists. OK to overwrite?" %(self.dymFileName)
            self.write = 0
            showAFXWarningDialog(mainWindow, message, AFXDialog.YES|AFXDialog.NO,
                                 self, self.ID_OVERWRITE_FILE)
            return 0
            
        else:
            modelName = self.returnModelName()
            rwDymFile.writeDymFile(self.extFileName, modelName, self.senList, self.actList)
            if self.mode == NEW:
                if self.chkBtn.getCheck():
                    dymCmd = '%s "%s"'
                    try:
                        uti.Spawn(dymCmd %('Dymola.exe', self.filePath))
                    except:
                        msg = 'Unable to launch executable "Dymola.exe". Make sure that '\
                              'the complete path to\n"Dymola.exe" has been added to the '\
                              'environment variable "PATH".'
                        showAFXErrorDialog(mainWindow, msg)
                        return 0                    
            self.form.deactivate()
            return 1
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def createSensor(self,sender,sel,ptr):

        sel = getAFXApp().getAFXMainWindow().getSelectorFromFunction('Output->History Output Requests->Create')
        tgt = getAFXApp().getAFXMainWindow().getTargetFromFunction('Output->History Output Requests->Create')
        tgt.handle(self, MKUINT(sel, SEL_COMMAND), None)

        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def createActuator(self,sender,sel,ptr):

        from abq_Dymola.dymolaActNameDB import DymolaActNameDB
        self.actNameDB = DymolaActNameDB(self.form)
        self.actNameDB.create()
        self.actNameDB.showModal()

#        sel = getAFXApp().getAFXMainWindow().getSelectorFromFunction('Tools->Amplitude->Create')
#        tgt = getAFXApp().getAFXMainWindow().getTargetFromFunction('Tools->Amplitude->Create')
#        tgt.handle(self, MKUINT(sel, SEL_COMMAND), None)

#        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def overwrite(self,sender,sel,ptr):
        """
        1) Function to check if files can be overwritten. Currently deletes
        file for True and does nothing for false.
        2) Function to overwrite invalid sensors and actuators.
        """
        mainWindow = getAFXApp().getAFXMainWindow()
        
        if SELID(sel) == self.ID_OVERWRITE_FILE:
            import osutils
            import uti
            if sender.getPressedButtonId() == AFXDialog.ID_CLICKED_YES:
                osutils.remove(self.filePath, force=1)
                modelName = self.returnModelName()
                rwDymFile.writeDymFile(self.extFileName, modelName,
                                          self.senList, self.actList) 
                if self.mode == NEW:
                    if self.chkBtn.getCheck():
                        dymCmd = '%s "%s"'
                        try:
                            uti.Spawn(dymCmd %('Dymola.exe', self.filePath))
                        except:
                            msg = 'Unable to launch executable "Dymola.exe". Make sure that '\
                                  'the complete path to\n"Dymola.exe" has been added to the '\
                                  'environment variable "PATH".'
                            showAFXErrorDialog(mainWindow, msg)
                            return 0
                    self.form.deactivate()
            elif sender.getPressedButtonId() == AFXDialog.ID_CLICKED_NO:
                self.dymFileNameTF.setFocusToTextField()
                return 0

        elif SELID(sel) == self.ID_OVER_SIGNALS:
            from kernelAccess import mdb
            modelName = self.returnModelName()
            if sender.getPressedButtonId() == AFXDialog.ID_CLICKED_YES:
                if self.probSList:
                    newSList = self.findRepFunc(self.probSList)
                    newSList = self.findDupSig(newSList, "S")
                    for i in range(len(self.probSList)):
                        #self.form.sendCommandString("mdb.models[" + modelName.__repr__() + "].historyOutputRequests.changeKey(fromName=" + self.probSList[i][0].__repr__() + ", toName=" + newSList[i].__repr__() + ")", False, False)
                        mdb.models[modelName].historyOutputRequests.changeKey(fromName=self.probSList[i][0],
                                                                              toName=newSList[i])
                    self.probSList = []
                    self.allSenList = self.updateList('S')
                    self.abqSenList.clearItems()
                    for item in self.allSenList:
                        self.abqSenList.appendItem(item)
                    self.disableButtons(None, None, None)
                    mdb.models[modelName].historyOutputRequests.registerQuery(self.updateAbqSenList, False)
                elif self.probAList:
                    newAList = self.findRepFunc(self.probAList)
                    newAList = self.findDupSig(newAList, "A")
                    for i in range(len(self.probAList)):
                        #self.form.sendCommandString("mdb.models[" + modelName.__repr__() + "].amplitudes.changeKey(fromName=" + self.probAList[i][0].__repr__() + ", toName=" + newAList[i].__repr__() + ")", False, False)
                        mdb.models[modelName].amplitudes.changeKey(fromName=self.probAList[i][0], 
                                                                   toName=newAList[i])
                    self.probAList = []
                    self.allActList = self.updateList('A')
                    self.abqActList.clearItems()
                    for item in self.allActList:
                        self.abqActList.appendItem(item)
                    self.disableButtons(None, None, None)
                    mdb.models[modelName].amplitudes.registerQuery(self.updateAbqActList, False)
            elif sender.getPressedButtonId() == AFXDialog.ID_CLICKED_NO:
                if self.probSList:
                    probSList1 = [x[0] for x in self.probSList]
                    origList = self.findDiff(self.allSenList, probSList1)
                    for item in origList:
                        self.abqSenList.appendItem(item)
                    self.disableButtons(None, None, None)
                    mdb.models[modelName].historyOutputRequests.registerQuery(self.updateAbqSenList, False)
                elif self.probAList:
                    probAList1 = [x[0] for x in self.probAList]
                    origList = self.findDiff(self.allActList, probAList1)
                    for item in origList:
                        self.abqActList.appendItem(item)
                    self.disableButtons(None, None, None)
                    mdb.models[modelName].amplitudes.registerQuery(self.updateAbqActList, False)
                        
        elif SELID(sel) == self.ID_WARN_IMPACT:
            if sender.getPressedButtonId() == AFXDialog.ID_CLICKED_YES:
                from kernelAccess import mdb
                modelName = self.returnModelName()
                for name in self.misActList:
                    mdb.models[modelName].ActuatorAmplitude(name=name)
                self.updateAbqActList()
            elif sender.getPressedButtonId() == AFXDialog.ID_CLICKED_NO:
                return 0
            
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def returnModelName(self):
        """
        Function to return name of the displayed model in viewport.
        """

        modelName = getCurrentContext()['modelName']

        return modelName

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def validSenActName(self, nameList):
        """
        Function to check if Sensor and Actuator names are valid in
        Dymola.
        """
        
        returnList = []
        probChar = []
        
        invalidList = [' ', '.', '$', '&', '*', '~', '`', '!',
                       '(', ')', '[', ']', '{', '}', '|', '/',
                       ';', ':', "'", '"', '<', '>', '?', '.',
                       '\\', '@', '#', '%', '^', '*', '-', '+',
                       '=', ',']

        for name in nameList:            
            for letter in name:
                if letter in invalidList:
                    probChar.append(letter)
                    
            if probChar:
                returnList.append([name, probChar])
                probChar = []

        return returnList
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def probListFunc(self, nameList):
        """
        Function to return sensors and actuators with non alpha-numeric
        character.
        """

        returnList = []
        for item in nameList:
            if not item.isalnum():
                returnList.append(item)

        return returnList
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def findRepFunc(self, nameList):        
        """
        Function to replace invalid characters by '_'.
        """

        outList = []

        for item in nameList:
            s = item[0]
            for char in item[1]:
                s = s.replace(char, '_')

            outList.append(s)

        return outList
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def findDupSig(self, nameList, stype):
        """
        Function to check if history outputs or amplitudes keys have
        existing names in the list.
        """
        from kernelAccess import mdb
        modelName = self.returnModelName()

        if stype == "S":
            keyList =  mdb.models[modelName].historyOutputRequests.keys()
        elif stype == "A":
            keyList =  mdb.models[modelName].amplitudes.keys()

        for i in range(len(nameList)):
            if nameList[i] in keyList:
                count = 0
                signalName = nameList[i]
                while signalName in keyList:
                    signalName = nameList[i] + "_%s" % count
                    count += 1
                nameList[i] = signalName    

        return nameList

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def findDiff(self, list1, list2):
        """
        Function to find difference between two lists
        """
        oList = []
        if len(list1) > len(list2):
            lista = list1
            listb = list2
        else:
            lista = list2
            listb = list1

        for item in lista:
            if item not in listb:
                oList.append(item)

        return oList

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdFile(self, sender, sel, ptr):
        """
        Open File selector dialog.
        """
        
        if not self.fileDb:
            patterns = 'Dymola files (*.mo)\n'
            self.fileDb = AFXFileSelectorDialog(self, 'Select a Dymola file', 
                                                self.fileNameTgt,
                                                None, 
                                                AFXSELECTFILE_EXISTING,
                                                patterns,
                                                self.form.patternIndexTgt)
            self.fileDb.setReadOnlyPatterns('*.mo')
            self.fileDb.create()
            self.fileNameTgt.setTarget(self)
            self.fileNameTgt.setSelector(self.ID_FILE_OK)
            getAFXAliasMap().setPrefix(self.fileDb, 'FileDB1-%d' % DymolaDB.numInstance)

        #getAFXAliasMap().setPrefix(self.fileDb, 'dymola file open')

        self.fileDb.showModal()        
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def appendTable(self, sender, sel, ptr):
        """
        """
        import os

        self.impSenList = []
        self.impActList = []
        
        mainWindow = getAFXApp().getAFXMainWindow()
        
        fileName = self.fileNameTgt.getValue().strip()
        
        if not fileName:
            showAFXErrorDialog(mainWindow, 'A file name must be specified.')
            self.form.dymNameKw.setValue(None)
            self.dbtn.disable()
            return 0

        if not os.path.exists(fileName):
            showAFXErrorDialog(mainWindow, 'File not found: %s' % fileName)
            self.form.dymNameKw.setValue(None)
            self.dbtn.disable()
            return 0
        
        self.impSenList, self.impActList = rwDymFile.readDymFile(fileName)
        
        if not self.impSenList or not self.impActList:
            errStr = "Either Sensors or Actuators is missing in %s"
            showAFXErrorDialog(mainWindow, errStr % fileName)
            self.form.dymNameKw.setValue(None)
            self.dbtn.disable()
            return 0
        
        self.form.dymNameKw.setValue(fileName)
        self.dbtn.enable()        

        self.expSenList.clearItems()
        self.expActList.clearItems()

        for item in self.impSenList:
            self.expSenList.appendItem(item)

        for item in self.impActList:
            self.expActList.appendItem(item)
        self.disableButtons(None, None, None)

        self.processLists()
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def processLists(self):
        """
        """
        
        mainWindow = getAFXApp().getAFXMainWindow()
        numSen = self.abqSenList.getNumItems()
        numAct = self.abqActList.getNumItems()
        
        eAbqSenList = []
        eAbqActList = []

        for i in range(numSen):
            eAbqSenList.append(self.abqSenList.getItemText(i).upper())

        for i in range(numAct):
            eAbqActList.append(self.abqActList.getItemText(i).upper())
        
        self.misSenList = []
        self.misActList = []
        
        self.misSenList = [ x for x in self.impSenList if x not in eAbqSenList]
        self.misActList = [ x for x in self.impActList if x not in eAbqActList]
        
        modelName = self.returnModelName()
        warnMsgM = "The following %s imported from Dymola do not "\
                  "have a corresponding\n%s definition in Abaqus/CAE "\
                  "for the model '%s':"
        senMsg = "\nCreate History Outputs with the same name for these "\
                 "sensors in Abaqus/CAE."
        #senMsg = "Do you want the History Outputs to be created "\
        #         "automatically in Abaqus/CAE ?"
        actMsg = "Do you want the amplitudes to be created "\
                 "automatically in Abaqus/CAE?\n(Note: Define "\
                 "an actuator driven feature for the "\
                 "actuator to have any effect.)"
        if self.misActList and not self.misSenList:
            topMsg = warnMsgM %('actuators', 'Actuator amplitude', modelName)
            items = ','.join(self.misActList)
            btmMsg = actMsg
            showAFXItemsWarningDialog(mainWindow, topMsg, items, btmMsg, 
                                     AFXDialog.YES|AFXDialog.NO,
                                     self, self.ID_WARN_IMPACT)
           
        elif self.misSenList and not self.misActList:
            warnMsg = warnMsgM %('sensors', 'History Output', modelName)
            warnMsg += '\n\n'
            for item in self.misSenList:
                warnMsg += '%s\n' % item
            warnMsg += senMsg
            
            showAFXInformationDialog(mainWindow, warnMsg, tgt=None, sel=0)
            
        elif self.misSenList and self.misActList:
            warnMsg = warnMsgM %('sensors', 'History Output', modelName)
            warnMsg += '\n\n'
            for item in self.misSenList:
                warnMsg += '%s\n' % item
            warnMsg += senMsg
            warnMsg += '\n\n'
            warnMsg += warnMsgM %('actuators', 'Actuator amplitude', modelName)
            warnMsg += '\n\n'
            for item in self.misActList:
                warnMsg += '%s\n' % item
            warnMsg += '\n' + actMsg
            showAFXWarningDialog(mainWindow, warnMsg,AFXDialog.YES|AFXDialog.NO,
                                 self, self.ID_WARN_IMPACT)
        elif not self.misSenList and not self.misActList:
#            message = "The signals are consistent between Abaqus and Dymola."
            message = "All Dymola signals have corresponding signal names in Abaqus/CAE."
            showAFXInformationDialog(mainWindow, message, tgt=None, sel=0)
        else:
            pass

        return 1
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def openDymolaFile(self, sender, sel, ptr):
        """
        Open dymola file
        """
        import uti

        if not self.form.dymNameKw.getValue():
            return 0
        mainWindow = getAFXApp().getAFXMainWindow()
        dymCmd = '%s "%s"'
        try:
            uti.Spawn(dymCmd %('Dymola.exe', self.form.dymNameKw.getValue()))
        except:
            msg = 'Unable to launch executable "Dymola.exe". Make sure that '\
                  'the complete path to\n"Dymola.exe" has been added to the '\
                  'environment variable "PATH".'
            showAFXErrorDialog(mainWindow, msg)
            return 0                    
        
        return 1
        
        
        
