from abaqusGui import *
from dymolaConstants import *

###########################################################################
# Class definition
###########################################################################
class DymolaJobDB(AFXDataDialog):
    [
        ID_SELECT_FILE,
        ID_SCHEME,
        ID_FILE_OK,
        ID_MONITOR
        ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+4)
    numInstance = 0
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        DymolaJobDB.numInstance += 1

        # Construct the base class.
        #
            
        AFXDataDialog.__init__(self, form, 'Submit Abaqus Dymola Co-simulation',
                               0)
        
        self.form = form
        self.okBtn = self.appendActionButton(text='Submit',
                                             tgt=self,
                                             sel=self.ID_CLICKED_APPLY)
        self.cancelBtn = self.appendActionButton(text='Cancel',
                                                 tgt=self,
                                                 sel=self.ID_CLICKED_CANCEL)
        getAFXAliasMap().setPrefix(self, 'dymolaJob-%d'
                                           % DymolaJobDB.numInstance)
        
        vfs0 = FXVerticalFrame(p=self, opts=0, x=0, y=0, w=0, h=0,
                               pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING,
                               vs=DEFAULT_SPACING)
        self.abqJobNameTF = AFXTextField(p=vfs0, ncols=8,
                                         labelText='Abaqus job name:',
                                         tgt=self.form.jobNameKw, sel=0,
                                         opts=AFXTEXTFIELD_STRING|LAYOUT_FILL_X,
                                         x=0, y=0, w=0, h=0,
                                         pl=DEFAULT_PAD, pr=DEFAULT_PAD,
                                         pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.abqJobNameTF, 'jobNameTF')
        self.prevJobName = None
        hf = FXHorizontalFrame(p=vfs0, opts=0, x=0, y=0, w=0, h=0,
                               pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING,
                               vs=DEFAULT_SPACING)
        self.dymFileNameTF = AFXTextField(p=hf, ncols=30,
                                          labelText='Dymola file name:',
                                          tgt=self.form.dymNameKw, sel=0,
                                          opts=AFXTEXTFIELD_STRING,
                                          x=0, y=0, w=0, h=0, pl=DEFAULT_PAD,
                                          pr=DEFAULT_PAD,
                                          pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.dymFileNameTF, 'dymFileNameTF')
        self.dymFileNameTF.setEditable(False)
        self.fileSelecBtn=FXButton(p=hf, text='Select...', ic=None, tgt=self,
                 sel=self.ID_SELECT_FILE, opts=BUTTON_NORMAL, x=0, y=0, w=0,
                 h=0, pl=DEFAULT_PAD, pr=DEFAULT_PAD, pt=DEFAULT_PAD,
                 pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.fileSelecBtn, 'fileSelectBtn')
        FXMAPFUNC(self, SEL_COMMAND, self.ID_SELECT_FILE, DymolaJobDB.onCmdFile)
        self.fileNameTgt = AFXStringTarget()
        self.fileDb1 = None
        message = "The 'dymosim.dll' in the current directory should have \n"\
                  "been generated from the selected Dymola file."
        AFXNote(vfs0, message, NOTE_INFORMATION, 0, 0)
        self.stepCombo = AFXComboBox(p=vfs0, ncols=8, nvis=2,
                                     text='Co-simulation step:', tgt=self.form.stepKw, sel=0,
                                     opts=0, x=0, y=0, w=0, h=0, pl=DEFAULT_PAD,
                                     pr=DEFAULT_PAD, pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.stepCombo, 'stepcombo')       
        self.stepCombo.setMaxVisible(10)
        self.updateStep()

        gb = FXGroupBox(p=vfs0, text='Co-simulation Controls', opts=FRAME_GROOVE|LAYOUT_FILL_X,
            x=0, y=0, w=0, h=0, pl=DEFAULT_SPACING, pr=DEFAULT_SPACING,
            pt=DEFAULT_SPACING, pb=DEFAULT_SPACING, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        matrix = FXMatrix(p=gb, n=1, opts=MATRIX_BY_COLUMNS, x=0, y=0, w=0, h=0,
                          pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
##        tabVf = FXVerticalFrame(p=gb, opts=0,
##                                x=0, y=0, w=0, h=0, pl=DEFAULT_SPACING, pr=DEFAULT_SPACING,
##                                pt=DEFAULT_SPACING, pb=DEFAULT_SPACING, hs=DEFAULT_SPACING,
##                                vs=DEFAULT_SPACING)
##        self.timeStephf = FXHorizontalFrame(p=tabVf, opts=LAYOUT_FILL_X, x=0,
##                                            y=0, w=0, h=0,pl=0, pr=0, pt=0,
##                                            pb=0, hs=DEFAULT_SPACING,
##                                            vs=DEFAULT_SPACING)
        self.tsSchemeCombo = AFXComboBox(p=matrix, ncols=8,nvis=2,
                                         text='Time stepping scheme:',
                                         tgt=self.form.stepSchemeKw, sel=0, opts=0)
        getAFXAliasMap().setName(self.tsSchemeCombo, 'schemecombo')
	self.tsSchemeCombo.appendItem('FIXED')
	self.tsSchemeCombo.appendItem('VARIABLE')
        
        # text field made visible based on selection
        self.form.stepSchemeKw.setTarget(self)
        self.form.stepSchemeKw.setSelector(self.ID_SCHEME)
        FXMAPFUNC(self,SEL_COMMAND,self.ID_SCHEME,DymolaJobDB.onSchemeUpdate)
        
        self.stepValueTf = AFXTextField(p=matrix, ncols=8,
                                        labelText='Step value:',
                                        tgt=self.form.stepTimeKw,sel=0,
                                        opts=AFXTEXTFIELD_FLOAT|LAYOUT_FILL_X,
                                        x=0, y=0, w=0, h=0,
                                        pl=DEFAULT_PAD, pr=DEFAULT_PAD,
                                        pt=DEFAULT_PAD, pb=DEFAULT_PAD)
                                           
        getAFXAliasMap().setName(self.stepValueTf, 'stepValueTf')
        # Text field will be shown based on scheme selection
        # done in onSchemeUpdate()
        self.stepValueTf.show()
        self.stepValueFlag = 1
##        hf = FXHorizontalFrame(p=tabVf, opts=0, x=0, y=0, w=0, h=0,
##                               pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        enforceCombo = AFXComboBox(p=matrix, ncols=8, nvis=2,
                                   text='Enforcement type:',
                                   tgt=self.form.enforceKw, sel=0, opts=0)
        getAFXAliasMap().setName(enforceCombo, 'enforcecombo')                                   
	enforceCombo.appendItem('EXACT')
        enforceCombo.appendItem('LOOSE')

        solverCombo = AFXComboBox(p=matrix, ncols=8, nvis=6,
                                  text='Dymola solver:',
                                  tgt=self.form.dymSolverKw, sel=0, opts=0)
        getAFXAliasMap().setName(solverCombo, 'solverCombo')                                   
	solverCombo.appendItem('Dassl')
        solverCombo.appendItem('Lsodar')
        solverCombo.appendItem('Euler')
        solverCombo.appendItem('Rkfix2')
        solverCombo.appendItem('Rkfix3')
        solverCombo.appendItem('Rkfix4')
        matrix.setNumRows(2)
	matrix.setNumColumns(2)
        
        gb = FXGroupBox(p=vfs0, text='Job Controls', opts=FRAME_GROOVE|LAYOUT_FILL_X,
            x=0, y=0, w=0, h=0, pl=DEFAULT_SPACING, pr=DEFAULT_SPACING,
            pt=DEFAULT_SPACING, pb=DEFAULT_SPACING, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        precisionCombo = AFXComboBox(p=gb, ncols=8, nvis=2,
                                   text='Abaqus/Explicit precision:',
                                   tgt=self.form.precisionKw, sel=0, opts=0)
        getAFXAliasMap().setName(precisionCombo, 'precisioncombo')                                   
	precisionCombo.appendItem('Double')
        precisionCombo.appendItem('Single')
        self.spinner = AFXSpinner(p=gb, ncols=4, labelText='Use multiple processors:', tgt=self.form.ncpusKw,
                             sel=0, opts=AFXSPINNER_CHECKBUTTON,x=0, y=0, w=0, h=0, pl=DEFAULT_PAD,
                             pr=DEFAULT_PAD, pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.spinner, 'spinner')
        self.spinner.setRange(1,10000)
        
        gb = FXGroupBox(p=vfs0, text='Job Execution', opts=FRAME_GROOVE|LAYOUT_FILL_X,
                        x=0, y=0, w=0, h=0, pl=DEFAULT_SPACING, pr=DEFAULT_SPACING,
                        pt=DEFAULT_SPACING, pb=DEFAULT_SPACING, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        hf = FXHorizontalFrame(p=gb, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        self.radio1=FXRadioButton(p=hf, text='Write input', tgt=self.form.jobTypeKw,
                      sel=WRITE_INPUT.getId(), opts=RADIOBUTTON_NORMAL,
                      x=0, y=0, w=0, h=0, pl=DEFAULT_PAD, pr=DEFAULT_PAD,
                      pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.radio1, 'radio1')                                
        self.radio2=FXRadioButton(p=hf, text='Data check', tgt=self.form.jobTypeKw,
                      sel=DATACHECK.getId(), opts=RADIOBUTTON_NORMAL,
                      x=0, y=0, w=0, h=0, pl=DEFAULT_PAD, pr=DEFAULT_PAD,
                      pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.radio2, 'radio2')                                   
        self.radio3=FXRadioButton(p=hf, text='Full analysis', tgt=self.form.jobTypeKw,
                      sel=FULL_ANALYSIS.getId(), opts=RADIOBUTTON_NORMAL,
                      x=0, y=0, w=0, h=0, pl=DEFAULT_PAD, pr=DEFAULT_PAD,
                      pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.radio3, 'radio3')
        hf = FXHorizontalFrame(p=gb, opts=LAYOUT_SIDE_RIGHT, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)
        self.monitorBtn = FXButton(p=hf, text='Monitor...', ic=None, tgt=self, sel=self.ID_MONITOR, opts=BUTTON_NORMAL,
            x=0, y=0, w=0, h=0, pl=DEFAULT_PAD, pr=DEFAULT_PAD, pt=DEFAULT_PAD, pb=DEFAULT_PAD)
        getAFXAliasMap().setName(self.monitorBtn, 'monitorBtn')                                   
        self.monitorBtn.disable()
        FXMAPFUNC(self,SEL_COMMAND,self.ID_MONITOR,DymolaJobDB.launchMonitor)
        FXMAPFUNC(self,SEL_COMMAND,self.ID_FILE_OK,DymolaJobDB.checkcosim)
        self.patternIndexTgt = AFXIntTarget(0)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
        """
        """
        from kernelAccess import mdb

        modelName = self.returnModelName()
        mdb.models[modelName].steps.registerQuery(self.updateStep, False)
        
        AFXDataDialog.show(self)
         
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):
        """
        """
        from kernelAccess import session
        
        modelName = self.returnModelName()
        mdb.models[modelName].steps.unregisterQuery(self.updateStep)
        
        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def returnModelName(self):
        """
        Function to return name of the displayed model in viewport.
        """

        modelName = getCurrentContext()['modelName']

        return modelName

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateStep(self):
        """
        """
        from kernelAccess import mdb

        self.stepCombo.clearItems()
        modelName = self.returnModelName()
        stepNames = mdb.models[modelName].steps.keys()
        stepVal = self.form.stepKw.getValue()
        for stepName in stepNames:
            if stepName != 'Initial':
                self.stepCombo.appendItem(stepName)

        if stepVal:
            self.form.stepKw.setValue(stepVal)
            
        return 1
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdFile(self, sender, sel, ptr):
        """
        Open File selector dialog.
        """
       
        if not self.fileDb1:
            patterns = 'Dymola files (*.mo)\n'
            self.fileDb1 = AFXFileSelectorDialog(self, 'Select a Dymola file for co-simulation', 
                                                self.fileNameTgt,
                                                None, 
                                                AFXSELECTFILE_EXISTING,
                                                patterns,
                                                self.patternIndexTgt)
            self.fileDb1.setReadOnlyPatterns('*.mo')
            self.fileDb1.create()
            self.fileNameTgt.setTarget(self)
            self.fileNameTgt.setSelector(self.ID_FILE_OK)
            getAFXAliasMap().setPrefix(self.fileDb1, 'FileDB-%d' % DymolaJobDB.numInstance)
    
        #getAFXAliasMap().setPrefix(self.fileDb1, 'dymola file open1')
        self.fileDb1.showModal()        

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onSchemeUpdate(self,sender,sel,ptr):
        """
        This method updates step value based on scheme selection
        """
        
        # Get the scheme type
        scheme = self.tsSchemeCombo.getText()
        if scheme == 'FIXED':
            # Check if it is already created
            if not self.stepValueFlag:
                self.stepValueTf.create()
                self.stepValueFlag = 1
                self.stepValueTf.show()
            self.stepValueTf.enable()
            if not self.form.stepTimeKw.getValue():
                self.form.stepTimeKw.setValueToDefault()
#            self.timeStephf.recalc()
        elif scheme == 'VARIABLE':
            self.stepValueTf.disable()
            self.stepValueFlag = 0
            if not self.form.stepTimeKw.getValue():
                self.form.stepTimeKw.setValueToDefault()
            
        return 1
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def checkcosim(self,sender,sel,ptr):
        """
        """
        import os
        from kernelAccess import mdb
        
        mainWindow = getAFXApp().getAFXMainWindow()
        
        fileName = self.fileNameTgt.getValue().strip()
        
        if not fileName:
            showAFXErrorDialog(mainWindow, 'A file name must be specified.')
            self.form.dymNameKw.setValue(None)
            return 0

        if not os.path.exists(fileName):
            showAFXErrorDialog(mainWindow, 'File not found: %s' % fileName)
            self.form.dymNameKw.setValue(None)
            return 0        
        
        self.form.dymNameKw.setValue(fileName)
            
        return 1
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateList(self, stype):
        """
        Get the sensor and actuator info from repositories.
        """
        outList = []
        
        from kernelAccess import mdb
        modelName = self.returnModelName()
        
        if stype == 'S':                                
            hoReqKeys = mdb.models[modelName].historyOutputRequests.keys()
            for key in hoReqKeys:
                hoReq = mdb.models[modelName].historyOutputRequests[key]
                if hoReq.sensor:
                    outList.append(key)
        elif stype == 'A':
            ampKeys = mdb.models[modelName].amplitudes.keys()
            for key in ampKeys:
                ampObj = mdb.models[modelName].amplitudes[key]
            
            from kernelAccess import kernelAccessClient

            kacList = dir(kernelAccessClient)	    
            if 'ActuatorAmplitudeProxy' in kacList:
                for key in ampKeys:
                    ampObj = mdb.models[modelName].amplitudes[key]
                    if type(ampObj) == kernelAccessClient.ActuatorAmplitudeProxy:
                        outList.append(key)
            # The 'if' part was working in 6.8, but the proxy has changed in 6.9.
            # Implementing an alternate logic for 6.9
            elif 'abqRpoClient' in kacList:
                kacList1 = dir(kernelAccessClient.abqRpoClient)
                if 'ActuatorAmplitudeProxy' in kacList1:
                    for key in ampKeys:
                        ampObj = mdb.models[modelName].amplitudes[key]
                        if type(ampObj) == kernelAccessClient.abqRpoClient.ActuatorAmplitudeProxy:
                            outList.append(key)
	    	else:
            	    for key in ampKeys:
                        ampObj = mdb.models[modelName].amplitudes[key]
			if hasattr(kernelAccessClient.abqRpoClient, "ActuatorAmplitudeProxy"):
                            if type(ampObj) == kernelAccessClient.abqRpoClient.ActuatorAmplitudeProxy :
                                outList.append(key)
			elif  str(type(ampObj))=="<class 'kernelAccess.ActuatorAmplitudeProxy'>":
				outList.append(key)    
        return outList

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def launchMonitor(self,sender,sel,ptr):
        """
        """
        from abq_Dymola.jobMonitorDB import JobMonitorDB
        jobName = self.form.jobNameKw.getValue()
        self.jobMonitorDB = JobMonitorDB(self.form, jobName )
        self.jobMonitorDB.create()
        self.jobMonitorDB.showModal()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def processUpdates(self):
        """
        Disable monitoring if job name changes.
        """

        self.currJobName = self.form.jobNameKw.getValue()
        if self.currJobName != self.prevJobName:
            self.monitorBtn.disable()
            self.prevJobName = self.currJobName

        
        
        
