"""

"""

def writeData(saFlag, ioList):
    """
    Function to write sensor & actuator data.
    saFlag = 0 means actuator
    saFlag = 1 means sensor
    """

    fileList = []
    annStr = " annotation (extent=["

    endStr = "]);\n" 
    
    if saFlag:                
        iLimit = 10
    else:        
        iLimit = 11


    if len(ioList) <= iLimit:
        if saFlag:
            ioStr = "  Modelica.Blocks.Interfaces.RealInput "
            fixedStr = "-120,-10; -80,30]);\n"
            baseX1 = -120
            baseX2 = -80
            tbaseY1 = 70
            tbaseY2 = 110
            bbaseY1 = -110
            bbaseY2 = -70
        else:
            ioStr = "  Modelica.Blocks.Interfaces.RealOutput "
            fixedStr = "90,-10; 110,10]);\n"
            baseX1 = 90
            baseX2 = 110
            tbaseY1 = 10
            tbaseY2 = 30
            bbaseY1 = -30
            bbaseY2 = -10
            
        totPair = len(ioList)/2
        count = 0
        for i in range(totPair):            
            xPos1 = str(baseX1)
            xPos2 = str(baseX2)
            
            yPos1 = str(tbaseY1 - count)
            yPos2 = str(tbaseY2 - count)

            wrtStr = ioStr + ioList[i*2] + annStr + xPos1 + ',' + yPos1 + '; ' + xPos2 + ',' + yPos2 + endStr
            fileList.append(wrtStr)

            yPos1 = str(bbaseY1 + count)
            yPos2 = str(bbaseY2 + count)
            
            wrtStr = ioStr + ioList[(i*2) + 1] + annStr + xPos1 + ',' + yPos1 + '; ' + xPos2 + ',' + yPos2 + endStr
            fileList.append(wrtStr)

            if saFlag:
                count += 20
            else:
                count += -20

        if len(ioList)%2:
            wrtStr = ioStr + ioList[-1] + annStr + fixedStr
            fileList.append(wrtStr)
    else:        
        yLength = 220 # maximum height of sketch
        clearance = 1 # Gap between sensors or actuators
        totNum = yLength/len(ioList)
        asLen = totNum - clearance
        baseY = -110
        if saFlag:
            ioStr = "  Modelica.Blocks.Interfaces.RealInput "            
            baseX = -120
        else:
            ioStr = "  Modelica.Blocks.Interfaces.RealOutput "
            baseX = 110

        cumLen = asLen
        addendum = 0
        for i in range(len(ioList)):
            if saFlag:
                xPos1 = str(baseX)
                xPos2 = str(baseX + totNum)
            else:
                xPos1 = str(baseX - totNum)
                xPos2 = str(baseX)
            
            yPos1 = str(baseY + addendum)
            yPos2 = str(baseY + cumLen)

            wrtStr = ioStr + ioList[i] + annStr + xPos1 + ',' + yPos1 + '; ' + xPos2 + ',' + yPos2 + endStr
            fileList.append(wrtStr)
            
            cumLen += asLen + clearance
            addendum += asLen + clearance
            
    return fileList

def writeDymFile(fileName, modelName, senList, actList):
    """
    Function to write out the modelica input file.
    """
    import os
    import checkFileExt

    invalidList = [' ', '.', '$', '&', '*', '~', '`', '!',
                   '(', ')', '[', ']', '{', '}', '|', '/',
                   ';', ':', "'", '"', '<', '>', '?', '.',
                   '\\', '@', '#', '%', '^', '*', '-', '+',
                   '=', ',']

    ver = "3.0"
    headStr = '  annotation (uses(Modelica(version=%s)), Diagram);\n' %(ver)

    thisDir = os.getcwd()
    fileName = thisDir + os.sep + fileName
#    fileName = checkFileExt.splitFileName(fileName, '.mo')
    fileObj = open(fileName, 'w')
    if not modelName.isalnum():
        tList = []
        for i in modelName:
            if i in invalidList:
                tList.append('_')
            else:
                tList.append(i)
        tempName = ''.join(tList)
        modelName = tempName

    writeStr = "model " + modelName + "\n"
    fileObj.write(writeStr)
    fileObj.write(headStr)

    senOList = writeData(1, senList)
    for item in senOList:
        fileObj.write(item)

    actOList = writeData(0, actList)
    for item in actOList:
        fileObj.write(item)
        
    writeStr = "end " + modelName + ";\n"
    fileObj.write(writeStr)

    fileObj.close()

    return 1
    
    
def readDymFile(fileName):
    """
    Function to import Dymola .mo files
    """

    impSenList = []
    impActList = []

    searchMStr = 'package'.upper()
    mCount = 0
    searchStr1 = 'Blocks.Interfaces.RealInput'.upper()
    add1 = len(searchStr1)
    
    searchStr2 = 'Blocks.Interfaces.RealOutput'.upper()
    add2 = len(searchStr2)
    
    dymFile = open(fileName, 'r')
    
    dymFileList = dymFile.readlines()
    
    for line in dymFileList:
        # check for 2nd occurence of 'package' and exit
        if mCount == 2:
            break
        
        line = line.strip()
        line = line.upper()

        fvalp = line.find(searchMStr)
        if fvalp != -1:
            mCount += 1
            continue
        fvals = line.find(searchStr1)
        if fvals != -1:
            sensorName = line[fvals+add1:].strip()
            sensorL = sensorName.split()
            sensorName = sensorL[0].strip()
            impSenList.append(sensorName)
            continue

        fvala = line.find(searchStr2)
        if fvala != -1:
            actName = line[fvala+add2:].strip()
            actL = actName.split()
            actName = actL[0].strip()
            impActList.append(actName)
            continue
        
    dymFile.close()
    return impSenList, impActList 

