from abaqusGui import *
from dymolaConstants import *
import rwDymFile

###########################################################################
# Class definition
###########################################################################

class SubmitJobForm(AFXForm):
    [
        ID_OVERWRITE,
        ] = range(AFXForm.ID_LAST, AFXForm.ID_LAST+1)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        AFXForm.__init__(self, owner)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_OVERWRITE,
                  SubmitJobForm.onCmdOverwrite)
        self.dymCmd = AFXGuiCommand(mode=self,
                                    method='runJobs',
                                    objectName='abq_Dymola.dymUtils',
                                    registerQuery=False)
        self.jobNameKw = AFXStringKeyword(self.dymCmd,
                                          'jobName',
                                          True,
                                          'CosimJob-1')
        self.stepKw = AFXStringKeyword(self.dymCmd,
                                       'stepName',
                                       True)
        self.senListKw = AFXTupleKeyword(self.dymCmd,
                                         'senList',
                                         True, 0, -1,
                                         AFXTUPLE_TYPE_STRING)
        self.actListKw = AFXTupleKeyword(self.dymCmd,
                                         'actList',
                                         True, 0, -1,
                                         opts=AFXTUPLE_TYPE_STRING)
        self.enforceKw = AFXStringKeyword(self.dymCmd,
                                          'enforceType',
                                          True,
                                          'EXACT')
        self.stepSchemeKw = AFXStringKeyword(self.dymCmd,
                                             'schemeType',
                                             True,
                                             'FIXED')
        self.stepTimeKw = AFXFloatKeyword(self.dymCmd,
                                          'cosimStepTime',
                                          True,
                                          .0002)
        self.dymSolverKw = AFXStringKeyword(self.dymCmd,
                                            'dymSolver',
                                            True,
                                            'Dassl')
        self.precisionKw = AFXStringKeyword(self.dymCmd,
                                          'precision',
                                          True,
                                          'Double')
        self.ncpusKw = AFXIntKeyword(self.dymCmd,
                                        'nCpus',
                                        True,
                                        2)
        self.multiProcKw = AFXIntKeyword(self.dymCmd,
                                        'multiProc',
                                        True,
                                        0)
        self.jobTypeKw = AFXSymConstKeyword(self.dymCmd,
                                            'jobType',
                                            True,
                                            FULL_ANALYSIS.getId())
        self.dymNameKw = AFXStringKeyword(None, 'dymolaFile',
                                          isRequired=True)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):
        """
        Reimplemented to check if the displayed model has analysis steps
        """
        
        from kernelAccess import mdb
        
        modelName = getCurrentContext()['modelName']
        errMsg = 'There are no analysis steps defined in the model. Create an'\
                 'analysis step before submitting the job.'
        if len(mdb.models[modelName].steps.keys()) < 2:
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)
            return None
        
        return AFXForm.activate(self)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        """
        """
        from abq_Dymola.dymolaJobDB import DymolaJobDB
        self.jobNameKw.setValue(self.getUniqueJobName())
        self.dymolaJobDB = DymolaJobDB(self)
        return self.dymolaJobDB

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getUniqueJobName(self):
        """
        This function gets a unique name for the dymola job
        """
        import os.path
        baseStr = "CosimJob-"
        counter = 1
        dymJobName = baseStr + str(counter)
        return dymJobName
        #Below is commented out for QA test
        #
        """while True: # Infinite Loop
            dymJobName = baseStr + str(counter)
            extFileName = dymJobName + '.inp'
            
            if not os.path.exists(extFileName):
                return dymJobName
            counter += 1
        return None"""
    
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):
        """
        """
        import os
        from abaqusConstants import CREATED, PROPAGATED
        from kernelAccess import mdb, session
        
        # copy active signals to keyword
        mainWindow = getAFXApp().getAFXMainWindow()

        try:
            session.viewports[session.currentViewportName].displayedObject.modelName
        except:
            errMsg = "Switch to the Job Module before submitting Abaqus Dymola Co-simulation."
            showAFXErrorDialog(mainWindow, errMsg)
            return 0        
            
        if not self.jobNameKw.getValue():
            errMsg = "Abaqus job name should be specified."
            showAFXErrorDialog(mainWindow, errMsg)
            self.dymolaJobDB.abqJobNameTF.setFocusToTextField()
            return 0
        
        status = self.validNameCheck(self.jobNameKw.getValue(), 'Abaqus job ')
        if not status:
            self.dymolaJobDB.abqJobNameTF.setFocusToTextField()
            return 0

        if not self.dymNameKw.getValue():
            errMsg = "A valid Dymola file name should be selected."
            showAFXErrorDialog(mainWindow, errMsg)
            return 0
            
        stepName = self.stepKw.getValue()
        if not stepName:
            errMsg = "A co-simulation step should be specified."
            showAFXErrorDialog(mainWindow, errMsg)
            return 0

        status = self.verifySignals()
        
        if not status:
            return 0
        
        errMsgM = "The following %s are not active for the chosen \n"\
                 "co-simulation step '%s'. Activate these %s before\n"\
                 "submitting the job.\n\n"
        
        # Check if history outputs are active for chosen step
        modelName = getCurrentContext()['modelName']
        hoKeys = mdb.models[modelName].steps[stepName].historyOutputRequestStates.keys()
        hoDict = {}
        hFlag = 0
        for i in hoKeys:
            hoDict[i.upper()] = i
            
        errMsg = errMsgM %('history outputs', stepName, 'history outputs')
        for sensor in self.impSenList:
            hoObj = mdb.models[modelName].steps[stepName].historyOutputRequestStates[hoDict[sensor]]
            if hoObj.status not in [CREATED, PROPAGATED]:
                errMsg += '%s\n' %hoDict[sensor]
                hFlag = 1
        if hFlag:
            showAFXErrorDialog(mainWindow, errMsg)
            return 0

        item = ''
        for i in self.impSenList:
            item += i
            item += ','
        self.senListKw.setValues(item)
        item = ''
        for i in self.impActList:
            item += i + ','
        self.actListKw.setValues(item)

        if self.stepSchemeKw.getValue() == 'FIXED':
            if not self.stepTimeKw.getValue():
                errMsg = "Specify the 'Step value' for co-simulation."
                showAFXErrorDialog(mainWindow, errMsg)
                return 0        
        
        if self.dymolaJobDB.spinner.getCheck():
            self.multiProcKw.setValue(1)
        elif not self.dymolaJobDB.spinner.getCheck():
            self.multiProcKw.setValue(0)
            self.ncpusKw.setValue(2)

        #Below is commented out just for QA test and should be uncommented while running
        #the cosimulation
        #
        """if not os.path.exists('dymosim.dll'):
            errMsg  = "Unable to find 'dymosim.dll' in the current working "\
                      "directory '%s'." %(os.getcwd())
            showAFXErrorDialog(mainWindow, errMsg)
            return 0"""
               
        if os.path.exists(self.jobNameKw.getValue()+'.inp'):
            message = 'Job files already exist for %s.\nOK to overwrite?' %(self.jobNameKw.getValue())
            showAFXWarningDialog(mainWindow, message, AFXDialog.YES|AFXDialog.NO,
                                 self, self.ID_OVERWRITE)
            return 0
        
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomTasks(self):

        if self.jobTypeKw.getValue() in [FULL_ANALYSIS.getId(), DATACHECK.getId()]:
            self.dymolaJobDB.monitorBtn.enable()
        else:
            self.dymolaJobDB.monitorBtn.disable()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def validNameCheck(self, name, type):
        """
        This method checks text fields for valid characters.
        """

        errMsg = 'Invalid name.\n'
        errMsg = errMsg + '\n'
        errMsg = errMsg + type + ' names must be 1-38 characters long,\n'
        errMsg = errMsg + 'may not begin or end with an underscore,\n'
        errMsg = errMsg + 'may not beign with a dash, and may not,\n'
        errMsg = errMsg + 'contain non-printable characters or the\n'
        errMsg = errMsg + 'following characters:\n'
        errMsg = errMsg + '\n'
        errMsg = errMsg + '<space> . $ & * ~ ! ()\n'
        errMsg = errMsg + '[] {} | ; : ` < > ? " / \' \\'

        invalidList = [' ', '.', '$', '&', '*', '~', '`', '!',
                       '(', ')', '[', ']', '{', '}', '|', '/',
                       ';', ':', "'", '"', '<', '>', '?', '.', '\\']

        mainWindow = getAFXApp().getAFXMainWindow()

        if len(name) > 38:
            showAFXErrorDialog(mainWindow, errMsg)
            return 0
        elif name[0] in ['_', '-'] or name[-1] in ['_', '-']:
            showAFXErrorDialog(mainWindow, errMsg)
            return 0
        else:
            for i in name:
                if i in invalidList:
                    showAFXErrorDialog(mainWindow, errMsg)
                    return 0

        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifySignals(self):
        """
        """
        mainWindow = getAFXApp().getAFXMainWindow()
        fileName = self.dymNameKw.getValue()
        self.impSenList, self.impActList = rwDymFile.readDymFile(fileName)
        
        if not self.impSenList or not self.impActList:
            errStr = "Either Sensors or Actuators is missing in %s"
            showAFXErrorDialog(mainWindow, errStr % fileName)
            self.dymNameKw.setValue(None)
            return 0
        
        senList = self.dymolaJobDB.updateList('S')
        actList = self.dymolaJobDB.updateList('A')
        
        eAbqSenList = []
        eAbqActList = []
        
        for i in senList:
            eAbqSenList.append(i.upper())
            
        for i in actList:
            eAbqActList.append(i.upper())
        
        self.misSenList = []
        self.misActList = []
        
        self.misSenList = [ x for x in self.impSenList if x not in eAbqSenList]
        self.misActList = [ x for x in self.impActList if x not in eAbqActList]
        
        modelName = self.dymolaJobDB.returnModelName()
        fileName = self.dymNameKw.getValue()

        warnMsgM = "The following %s found in Dymola file '%s' \ndo not "\
                   "have a corresponding %s definition in Abaqus/CAE "\
                   "for the model '%s':\n\n"
        warnMsgS = "\nUse 'Plugins --> Dymola --> Match existing Dymola (*.mo) file ...' "\
                   "to match these %s \nwith %s in Abaqus/CAE before submitting the job."

        warnMsgA = "\n(Note: Define an actuator driven feature for the actuator to have any effect.)"

        if self.misSenList and self.misActList:
            warnMsg = warnMsgM %('sensors and actuators', fileName, 'History Output and Actuator amplitude', modelName)
            for item in self.misSenList:
                warnMsg += '%s\n' % item
            warnMsg += '\n'

            for item in self.misActList:
                warnMsg += '%s\n' % item
            warnMsg += warnMsgS %('sensors and actuators',
                                  'History Outputs and Actuator amplitudes')
            warnMsg += warnMsgA
            showAFXErrorDialog(mainWindow, warnMsg)
            return 0
            
        elif self.misSenList:
            warnMsg = warnMsgM %('sensors', fileName, 'History Output', modelName)
            for item in self.misSenList:
                warnMsg += '%s\n' % item
            warnMsg += warnMsgS %('sensors', 'History Outputs')
            
            showAFXErrorDialog(mainWindow, warnMsg)
            return 0
        elif self.misActList:
            warnMsg = warnMsgM %('actuators', fileName, 'Actuator amplitude', modelName)
            for item in self.misActList:
                warnMsg += '%s\n' % item
            warnMsg += warnMsgS %('actuators', 'Actuator amplitudes')
            warnMsg += warnMsgA
            showAFXErrorDialog(mainWindow, warnMsg)
            return 0

        return 1
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdOverwrite(self, sender, sel, ptr):
        """
        """
        if sender.getPressedButtonId() == AFXDialog.ID_CLICKED_YES:
            self.issueCommands(writeToReplay=True, writeToJournal=False)            
        return 1
