from abaqusConstants import *
from abaqusGui import *
from kernelAccess import mdb
from engFeaturesSymConsts import *


###########################################################################
# Class definition
###########################################################################

class EngFeaturesDB(AFXDataDialog):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        # Construct the base class.
        #

        AFXDataDialog.__init__(self, form, 'Create Engineering Features',
                               self.APPLY|self.DISMISS,
                               DIALOG_ACTIONS_SEPARATOR|DECOR_RESIZE)

        #GUILOG
        getAFXAliasMap().setPrefix(self, 'engFeatures')
        #ENDGUILOG

        cb = AFXComboBox(p=self, ncols=0, nvis=1, text='Type:',
                         tgt=form.featTypeKw, sel=0, opts=LAYOUT_FILL_X)
        cb.setMaxVisible(7)
        cb.appendItem('Springs/Dashpots', SPRINGS_DASHPOTS.getId())
        cb.appendItem('Wires for Connectors', WIRES_FOR_CONNECTORS.getId())
        cb.appendItem('Beam MPCs', BEAM_MPCS.getId())
        cb.appendItem('Tie MPCs', TIE_MPCS.getId())
        cb.appendItem('Link MPCs', LINK_MPCS.getId())
        cb.appendItem('Pin MPCs', PIN_MPCS.getId())
        cb.appendItem('Elbow MPCs', ELBOW_MPCS.getId())
        getAFXAliasMap().setName(cb, 'type')

        check = FXCheckButton(p=self, text='Create set of wires',
                              tgt=form.wireSetKw, )
        getAFXAliasMap().setName(check, 'createWireSet')


        # Feature names
        #
        w = AFXTextField(p=self, ncols=0, labelText='Feature name (prefix):',
                         tgt=form.featNameKw, sel=0, opts=LAYOUT_FILL_X)
        getAFXAliasMap().setName(w, 'names')

        # Create transitions to control selective graying out of check box
        # and Feature Name text box.
        self.addTransition(form.featTypeKw, AFXTransition.EQ,
                           WIRES_FOR_CONNECTORS.getId(), check,
                           MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND))
        self.addTransition(form.featTypeKw, AFXTransition.NE,
                           WIRES_FOR_CONNECTORS.getId(), check,
                           MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND))
        self.addTransition(form.featTypeKw, AFXTransition.NE,
                           WIRES_FOR_CONNECTORS.getId(), w,
                           MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND))
        self.addTransition(form.featTypeKw, AFXTransition.EQ,
                           WIRES_FOR_CONNECTORS.getId(), w,
                           MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND))

        # Group Boxes for the two node sets
        gb = FXGroupBox(p=self, text='Sets', opts=FRAME_GROOVE|LAYOUT_FILL_X)
	self.va = AFXVerticalAligner(gb,
            opts=LAYOUT_FILL_X)
        self.cbNSet1 = AFXComboBox(p=self.va, ncols=0, nvis=3, text='First:',
                                   tgt=form.nset1Kw, sel=0, opts=LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.cbNSet1, 'set1')
        self.cbNSet2 = AFXComboBox(p=self.va, ncols=0, nvis=3, text='Second:',
                                   tgt=form.nset2Kw, sel=0, opts=LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.cbNSet2, 'set2')


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
        """
        The show(), hide(), and onUpdate() methods work together to keep
        the assembly set combo boxes current as assembly sets are added
        to, or removed from, the database.  This function registers the
        onUpdate() method when the dialog box is shown.
        """
        registerCurrentContext(self.onUpdate)
        modelName = getCurrentContext()['modelName']
        mdb.models[modelName].rootAssembly.sets.registerQuery(self.onUpdate)
        AFXDataDialog.show(self)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):
        """
        The show(), hide(), and onUpdate() methods work together to keep
        the assembly set combo boxes current as assembly sets are added
        to, or removed from, the database.  This function unregisters the
        onUpdate() method when the dialog box is hidden.
        """
        unregisterCurrentContext(self.onUpdate)
        modelName = getCurrentContext()['modelName']
        if modelName in mdb.models.keys():
            model = mdb.models[modelName]
            model.rootAssembly.sets.unregisterQuery(self.onUpdate)
        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onUpdate(self):
        """
        The show(), hide(), and onUpdate() methods work together to keep
        the assembly set combo boxes current as assembly sets are added
        to, or removed from, the database.  This function is called when
        such a database change occurs.
        """
        modelName = getCurrentContext()['modelName']
        model = mdb.models[modelName]
        
        # Clear and append all sets
        #
        self.cbNSet1.clearItems()
        self.cbNSet2.clearItems()
        
        for setName in model.rootAssembly.sets.keys():
            self.cbNSet1.appendItem(setName)        
            self.cbNSet2.appendItem(setName)
