from abaqusGui import *
from abaqusConstants import ALL
from engFeaturesSymConsts import *
###########################################################################
# Class definition
###########################################################################

class EngFeaturesForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        
        # Construct the base class.
        #
        AFXForm.__init__(self, owner)

        self.cmd = AFXGuiCommand(mode=self, method='main',
                                 objectName='abq_EngFeatures.engFeaturesUtils',
                                 registerQuery=False)
        self.featTypeKw = AFXSymConstKeyword(self.cmd, 'featType', True,
                                             SPRINGS_DASHPOTS.getId())
        self.featNameKw = AFXStringKeyword(self.cmd, 'featName', True, 'Feat-')
        self.nset1Kw = AFXStringKeyword(self.cmd, 'nset1Name', True, '')
        self.nset2Kw = AFXStringKeyword(self.cmd, 'nset2Name', True, '')

        # The 'wireSet' parameter must be last because it has a default
        # value (False) in the kernel script parameter list.
        self.wireSetKw = AFXBoolKeyword(self.cmd, 'wireSet',
                                        booleanType=AFXBoolKeyword.ON_OFF,
                                        isRequired=True, defaultValue=False)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        from abq_EngFeatures.engFeaturesDB import EngFeaturesDB
        self.db = EngFeaturesDB(self)
        return self.db

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):
    
        # Check for empty values
        #
        if not self.hasValue( self.featNameKw, 'Feature name prefix' ):
            return False

        if not self.hasValue( self.nset1Kw, 'First set' ):
            return False
    
        if not self.hasValue( self.nset2Kw, 'Second set' ):
            return False

        return True

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hasValue(self, kw, prefix):
    
        # Check for empty string
        
        if kw.getValue() == '':
            showAFXErrorDialog(self.db, '%s must be specified.' % prefix)
            return False
            
        return True
