from abaqusGui import *
from symbolicConstants import *

outputVarName = range(6)
outputVarName[0] = 'Artificial strain energy: ALLAE for Whole Model'
outputVarName[1] = 'Static dissipation (stabilization): ALLSD for Whole Model'
outputVarName[2] = 'Kinetic energy: ALLKE for Whole Model'
outputVarName[3] = 'Viscous dissipation: ALLVD for Whole Model'
outputVarName[4] = 'Internal energy: ALLIE for Whole Model'
outputVarName[5] = 'Strain energy: ALLSE for Whole Model'


###########################################################################
# Class definition
###########################################################################

class EVDB(AFXDataDialog):

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        AFXDataDialog.__init__(self, form, title='Energy Ratio Verification')

        getAFXAliasMap().setPrefix(self, 'Energy Ratio Verification')
        self.form = form

        self.plotBtn = self.appendActionButton(text="Plot",tgt=self,sel=self.ID_CLICKED_APPLY)
        getAFXAliasMap().setName(self.plotBtn, 'Plot')
        self.dismissBtn = self.appendActionButton(text="Dismiss",tgt=self,sel=self.ID_CLICKED_CANCEL)
        getAFXAliasMap().setName(self.dismissBtn, 'Dismiss')

        FXLabel(p=self, text="This tool will plot as percentages the ratio\nof energies available in the current ODB.")
        gb1=FXGroupBox(p=self, text="Numerator", opts=FRAME_GROOVE|LAYOUT_FILL_X)
        hf1=FXHorizontalFrame(gb1, 0, 0,0,0,0, 0,0,0,0)
        self.CB = range(6)
        self.CB[0] = FXCheckButton(p=hf1,text="ALLAE",tgt=self.form.allaeKw)
        getAFXAliasMap().setName(self.CB[0], 'Check Box ALLAE')
        self.CB[1] = FXCheckButton(p=hf1,text="ALLSD",tgt=self.form.allsdKw)
        getAFXAliasMap().setName(self.CB[1], 'Check Box ALLSD')
        self.CB[2] = FXCheckButton(p=hf1,text="ALLKE",tgt=self.form.allkeKw)
        getAFXAliasMap().setName(self.CB[2], 'Check Box ALLKE')
        self.CB[3] = FXCheckButton(p=hf1,text="ALLVD",tgt=self.form.allvdKw)
        getAFXAliasMap().setName(self.CB[3], 'Check Box ALLVD')
        gb2=FXGroupBox(p=self,text="Denominator", opts=FRAME_GROOVE|LAYOUT_FILL_X)
        hf2=FXHorizontalFrame(gb2, 0, 0,0,0,0, 0,0,0,0)
        self.CB[4] = FXCheckButton(p=hf2,text="ALLIE",tgt=self.form.allieKw)
        getAFXAliasMap().setName(self.CB[4], 'Check Box ALLIE')
        self.CB[5] = FXCheckButton(p=hf2,text="ALLSE",tgt=self.form.allseKw)
        getAFXAliasMap().setName(self.CB[5], 'Check Box ALLSE')

        self.onUpdate()


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):

        registerCurrentContext(self.onUpdate)
        AFXDataDialog.show(self)


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        unregisterCurrentContext(self.onUpdate)
        AFXDataDialog.hide(self)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onUpdate(self):

        foundVar = False
        for cb in self.CB:
            cb.disable()
        if hasattr(session.viewports[session.currentViewportName].odbDisplay, 'name'):
            keys = session.odbData[session.viewports\
              [session.currentViewportName].odbDisplay.name].historyVariables.keys()
            for var in outputVarName:
                if keys.count(var):
                    self.CB[outputVarName.index(var)].enable()
                    foundVar = True
                    
        if not foundVar:
            self.plotBtn.disable()
        else:
            self.plotBtn.enable()
