from abaqusGui import *
from abaqusConstants import OFF

###########################################################################
# Class definition
###########################################################################

class EVForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        self.owner = owner
        AFXForm.__init__(self, owner=owner)
        
        self.sthCmd = AFXGuiCommand(mode=self,method="showPlot",objectName="abq_Erv.eVKernel")
        self.allaeKw  = AFXBoolKeyword(command=self.sthCmd,name="allae",isRequired=True,defaultValue=OFF)
        self.allsdKw  = AFXBoolKeyword(command=self.sthCmd,name="allsd",isRequired=True,defaultValue=OFF)
        self.allkeKw  = AFXBoolKeyword(command=self.sthCmd,name="allke",isRequired=True,defaultValue=OFF)
        self.allvdKw  = AFXBoolKeyword(command=self.sthCmd,name="allvd",isRequired=True,defaultValue=OFF)
        self.allieKw  = AFXBoolKeyword(command=self.sthCmd,name="allie",isRequired=True,defaultValue=OFF)
        self.allseKw  = AFXBoolKeyword(command=self.sthCmd,name="allse",isRequired=True,defaultValue=OFF)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        """Return the first dialog to be posted"""
        from abq_Erv.eVDB import EVDB
        return EVDB(self)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def handleException(self, exception):
        """Overload the exception handling method"""

        errorType = exception[0]
        errorValue = exception[1]
        db = self.getCurrentDialog()

        # Then post error dialog
        showAFXErrorDialog(db, str(errorValue))
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        mainWindow = getAFXApp().getAFXMainWindow()
        if not hasattr(session.viewports[session.currentViewportName].odbDisplay, 'name'):
            msg =  'There is no output database (ODB)\nassociated with the current viewport.\n\n'
            msg += 'Select an ODB from the context bar, or use "File->Open" to open an ODB.'
            showAFXErrorDialog(mainWindow, msg)
            return 0
        db = self.getCurrentDialog()
        if not db.CB[0].isEnabled():
            self.allaeKw.setValue(0)
        if not db.CB[1].isEnabled():
            self.allsdKw.setValue(0)
        if not db.CB[2].isEnabled():
            self.allkeKw.setValue(0)
        if not db.CB[3].isEnabled():
            self.allvdKw.setValue(0)
        if not db.CB[4].isEnabled():
            self.allieKw.setValue(0)
        if not db.CB[5].isEnabled():
            self.allseKw.setValue(0)
        allae = self.allaeKw.getValue()
        allsd = self.allsdKw.getValue()
        allke = self.allkeKw.getValue()
        allvd = self.allvdKw.getValue()
        allie = self.allieKw.getValue()
        allse = self.allseKw.getValue()
        if not (allae or allsd or allke or allvd):
            showAFXErrorDialog(mainWindow, "At least one numerator must be selected.")
            return 0
        if not (allie or allse):
            showAFXErrorDialog(mainWindow, "At least one denominator must be selected.")
            return 0
        return 1

