from abaqusGui import *
import exteriorMeshDB



###########################################################################
# Class definition
###########################################################################

class exteriorMeshForm(AFXForm):
    [
        ID_WARNING,
    ] = range(AFXForm.ID_LAST, AFXForm.ID_LAST+1)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        # Construct the base class.
        #
        AFXForm.__init__(self, owner)

        # Command
        #
  	self.cmd = AFXGuiCommand(self, 'extMesh', 'abq_ExteriorMesh.exteriorMeshUtils')
  	
	FXMAPFUNC(self, SEL_COMMAND, self.ID_WARNING,exteriorMeshForm.onCmdWarning)
	
	#Output file name
        self.setName = AFXStringKeyword(self.cmd, 'setName', True)
        self.surfName = AFXStringKeyword(self.cmd, 'surfName', True)
	
	#Model name
        self.modelsCombo = AFXStringKeyword(self.cmd, 'modelsCombo', True)
        self.partsCombo = AFXStringKeyword(self.cmd, 'partsCombo', True)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        return exteriorMeshDB.exteriorMeshDB(self)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    def validNameCheck(self, name):
        """
        Method to check Abaqus name syntax
        Attributes:
    	name - string
        Calling Methods: verifyKeywordValues
        """
    
        invalidList = [' ', '.', '$', '&', '@','#','+','*', '~', '`', '!',
    		   '(', ')', '[', ']', '{', '}', '|', '/',
    		   ';', ':', "'", '"', '<', '>', '?', '.', '^','\\']
    
        if len(name) > 38:
		return 0
        elif name[0] in ['_', '-'] or name[-1] in ['_', '-'] or name[0].isdigit()==TRUE:
		return 0
        else:
    	  for i in name:
    	    if i in invalidList:
    		return 0
    
    	return 1
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    def doCustomChecks(self):
    	if self.partsCombo.getValue()=="":
            mainWindow = getAFXApp().getAFXMainWindow()
            showAFXErrorDialog(mainWindow, 'No part selected')
            return False
        if self.setName.getValue()=="" or self.surfName.getValue()=="":
            showAFXErrorDialog( self.getCurrentDialog(), 'No set or surface name provided;')
            return False
        else:
            errMsg = 'Invalid name.\n'
            errMsg = errMsg + '\n'
            errMsg = errMsg + ' names must be 1-38 characters long,\n'
            errMsg = errMsg + 'may not begin or end with an underscore,\n'
            errMsg = errMsg + 'may not beign with a digit or hyphen, and may not,\n'
            errMsg = errMsg + 'contain non-printable characters or the\n'
            errMsg = errMsg + 'following characters:\n'
            errMsg = errMsg + '\n'
            errMsg = errMsg + '<space> . $ & * ~ ! ()\n'
            errMsg = errMsg + '[] {} | ; : ` < > ? " / \' \\'
            if self.validNameCheck(self.setName.getValue())==0:
            	showAFXErrorDialog( self.getCurrentDialog(), errMsg)
            	return False
            if self.validNameCheck(self.surfName.getValue())==0:
            	showAFXErrorDialog( self.getCurrentDialog(), errMsg)
            	return False
            

        return True
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdWarning(self, sender, sel, ptr):

        if sender.getPressedButtonId() == AFXDialog.ID_CLICKED_YES:
                self.issueCommands()
        elif sender.getPressedButtonId() == AFXDialog.ID_CLICKED_NO:
                self.deactivate()

    	    

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    
    def activate(self):
        #if getDisplayedObjectType() == PART:
        if len(mdb.models.values()[0].parts):
            AFXForm.activate(self)
        else:
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(),
                 'No part in the current model')