from abaqusGui import *
from math import ceil

###########################################################################
# Class definition
###########################################################################

class FftDB(AFXDataDialog):

    [
        ID_CLICKED_SAVEAS,
        ID_SELECT_FILE,
        ID_COMBO_CHANGE,
    ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+3)

    numInstance = 0

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        FftDB.numInstance += 1

        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'Convert To Frequency Domain', 0,
            DECOR_RESIZE)
        self.appendActionButton('Save As...', self, self.ID_CLICKED_SAVEAS)
        self.appendActionButton('Plot', self, self.ID_CLICKED_APPLY)
        self.appendActionButton('Cancel', self, self.ID_CLICKED_CANCEL)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CLICKED_SAVEAS, FftDB.onSaveAs)


        #GUILOG - Main buttons
        getAFXAliasMap().setPrefix(self, 'FftDB-%d' % FftDB.numInstance)
        w = self.getActionButton(self.ID_CLICKED_SAVEAS)
        getAFXAliasMap().setName(w, 'SaveAs')
        #ENDGUILOG


        self.form = form
        self.firstSaveAsTime = True

        # File name, Xy name
        #
        self.fileDb = None
        self.saveAsDb = None

        self.deci = 1 # Holder for scale factor min/max values

        # Input group box
        #
        gb = FXGroupBox(p=self, text='Source', opts=FRAME_GROOVE|LAYOUT_FILL_X)

        # Switcher
        #
        hf = FXHorizontalFrame(gb, FRAME_NONE|LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        sw = FXSwitcher(gb, opts=LAYOUT_FILL_X)
        self.readFile = FXRadioButton(hf, 'Read from file', sw, FXSwitcher.ID_OPEN_FIRST)
        self.hOutput = FXRadioButton(hf, 'History output', sw, FXSwitcher.ID_OPEN_FIRST+1)

        #GUILOG - Radio buttons
        getAFXAliasMap().setName(self.readFile, 'readFile')
        getAFXAliasMap().setName(self.hOutput, 'hOutput')
        #ENDGUILOG

        # Read from file
        #
        hf1 = FXHorizontalFrame(sw, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        self.fileName = AFXTextField(p=hf1, ncols=20, labelText='File name:',
                                     tgt=self.form.xyFilePathKw,
                                     sel=0, opts=LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.fileName, 'fileName')
        w = FXButton(p=hf1, text='Select...', ic=None, tgt=self,
                     sel=self.ID_SELECT_FILE, opts=BUTTON_NORMAL|LAYOUT_RIGHT)
        getAFXAliasMap().setName(w, 'Select')
        FXMAPFUNC(self, SEL_COMMAND, self.ID_SELECT_FILE, FftDB.onCmdSelectFile)

        # Read from xy data object
        #
        vf = FXVerticalFrame(sw, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        hf2 = FXHorizontalFrame(vf, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        self.xyDataCombo = AFXComboBox(p=hf2, ncols=-1, nvis=5, text='XY data objects:',
                                       tgt=self, sel=self.ID_COMBO_CHANGE,
                                       opts=LAYOUT_FILL_X)
        self.xyDataCombo.setMaxVisible(10)
        getAFXAliasMap().setName(self.xyDataCombo, 'xyDataCombo')
        FXMAPFUNC(self, SEL_COMMAND, self.ID_COMBO_CHANGE, FftDB.onCmbChange)

        # Time range group box
        #
        hf = FXHorizontalFrame(vf, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        self.timeMin = AFXComboBox(p=hf, ncols=-1, nvis=20, text='Min.:',
                                   tgt=self.form.xyMinKw, sel=0, opts=LAYOUT_FILL_X)
        self.timeMinSel = FXButton(p=hf, text='Pick min...',
            tgt=self.form.proc, sel=AFXMode.ID_ACTIVATE)

        hf = FXHorizontalFrame(vf, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)            
        self.timeMax = AFXComboBox(p=hf, ncols=-1, nvis=20, text='Max.:',
                                   tgt=self.form.xyMaxKw, sel=0, opts=LAYOUT_FILL_X)
        self.timeMaxSel = FXButton(p=hf, text='Pick max...',
            tgt=self.form.proc, sel=AFXMode.ID_ACTIVATE)

        self.timeMin.disable()
        self.timeMax.disable()

        #GUILOG - Time combo boxes
        getAFXAliasMap().setName(self.timeMin, 'timeMin')
        getAFXAliasMap().setName(self.timeMinSel, 'timeMinSel')
        getAFXAliasMap().setName(self.timeMax, 'timeMax')
        getAFXAliasMap().setName(self.timeMaxSel, 'timeMaxSel')
        #ENDGUILOG

        # Frequency Range
        #
        gb = FXGroupBox(p=self, text='Frequency Range', opts=FRAME_GROOVE|LAYOUT_FILL_X)

        hf = FXHorizontalFrame(gb, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        self.freqMin = AFXTextField(p=hf, ncols=10, labelText='Min:',
                                   tgt=self.form.freqMinKw, sel=0,
                                   opts=LAYOUT_FILL_X|AFXTEXTFIELD_FLOAT)
        self.freqMax = AFXTextField(p=hf, ncols=10, labelText='Max:',
                                   tgt=self.form.freqMaxKw, sel=0,
                                   opts=LAYOUT_FILL_X|AFXTEXTFIELD_FLOAT)

        #GUILOG - Frequency Text fields
        getAFXAliasMap().setName(self.freqMin, 'freqMin')
        getAFXAliasMap().setName(self.freqMax, 'freqMax')
        #ENDGUILOG

        self.onCmbChange(None, None, None)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        if self.saveAsDb:
            if self.saveAsDb.shown():
                self.saveAsDb.hide()
        session.xyDataObjects.unregisterQuery(self.updateCombo)
        self.form.xyNewDataNameKw.setValue('')
        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmbChange(self, sender, sel, ptr):


        # Do changes to combo box only if shown:
        #
        if self.hOutput.getCheck() == True:

            self.form.xyDataNameKw.setRequired(True)
            self.form.xyNewDataNameKw.setRequired(True)
            self.form.xyMinKw.setRequired(True)
            self.form.xyMaxKw.setRequired(True)
            self.form.xyFilePathKw.setRequired(False)
            self.form.xyFilePathKw.setValue('')

            item = self.xyDataCombo.getCurrentItem()
            timeMinItem = self.timeMin.getCurrentItem()
            timeMaxItem = self.timeMax.getCurrentItem()
            if item == -1:
                self.timeMin.disable()
                self.timeMax.disable()
            else:
                from kernelAccess import session
                d = session.xyDataObjects[self.xyDataCombo.getItemText(item)].data
                if len(d) < 3:
                    self.timeMin.disable()
                    self.timeMax.disable()
                    print 'XY Data object contains less than 3 data sets.'
                else:
                    self.timeMin.enable()
                    self.timeMax.enable()

                    self.timeMin.clearItems()
                    self.timeMax.clearItems()

                    for cb in (self.timeMin, self.timeMax):
                        for x in d:
                            cb.appendItem(str(x[0]))
                    if timeMinItem == -1:
                        timeMinItem = 0
                        self.form.xyMinKw.setValue(self.timeMin.getItemText(timeMinItem))
                    if timeMaxItem == -1:
                        timeMaxItem = len(d)-1
                        if timeMaxItem > 0:
                            self.form.xyMaxKw.setValue(self.timeMax.getItemText(timeMaxItem))        

        else:
            self.form.xyDataNameKw.setRequired(False)
            self.form.xyNewDataNameKw.setRequired(False)
            self.form.xyMinKw.setRequired(False)
            self.form.xyMaxKw.setRequired(False)
            self.form.xyFilePathKw.setRequired(True)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdSelectFile(self, sender, sel, ptr):
        
        # Post the file selection dialog
        #
        if not self.fileDb:
          patterns = 'Report files (*.rpt)'
          self.fileDb = AFXFileSelectorDialog(self, 'Select a File', 
              self.form.xyFilePathKw, None, 
              AFXSELECTFILE_ANY, patterns)
          getAFXAliasMap().setPrefix(self.fileDb, 'FileDB-%d' % FftDB.numInstance)
          self.fileDb.create()
        
        self.fileDb.showModal()
        
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onSaveAs(self, sender, sel, ptr):

        showError = ''

        # Check if file or xy data has been selected
        #
        if self.readFile.getCheck():

            # Check if path has been entered
            #
            if self.form.xyFilePathKw.getValue() == '':
                showError = 'A file name must be entered.'
        else:

            # Check if xy data has been selected
            #
            item = self.xyDataCombo.getCurrentItem()
            if item == -1:
                showError = 'Please select an XY data object.'
            else:
                self.form.xyDataNameKw.setValue(
                    self.xyDataCombo.getItemText(item))

        if showError:
            showAFXErrorDialog(self, showError)
        else:
            # Post the Save As dialog
            #
            if self.firstSaveAsTime:
                self.saveAsDb = SaveAsDB(self.form)
                self.saveAsDb.create()
                self.firstSaveAsTime = False

            from kernelAccess import session
            i=1
            while 'XYData-'+str(i) in session.xyDataObjects.keys():
                i+=1
            self.form.xyNewDataNameKw.setValue('XYData-'+str(i))
            self.saveAsDb.showModal()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
    
        session.xyDataObjects.registerQuery(self.updateCombo)
        AFXDataDialog.show(self)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateCombo(self):

        # Populate xy data combo box
        #
        text = self.xyDataCombo.getText()
        self.xyDataCombo.clearItems()
        from kernelAccess import session
        i = 0
        loc = -1
        for key in sorted(session.xyDataObjects.keys()):
            if text == key:
                loc = i    
            self.xyDataCombo.appendItem(key)
            i+=1
        try:
            self.xyDataCombo.setCurrentItem(loc)
            self.setText.setCurrentItem(loc)
        except:
            pass
            
        self.onCmbChange(None, None, None)
        


###########################################################################
# Class definition
###########################################################################

class SaveAsDB(AFXDataDialog):

    numInstance = 0

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        SaveAsDB.numInstance += 1

        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'Save XY Data As', 0,
            DIALOG_NORMAL|DIALOG_ACTIONS_SEPARATOR|DECOR_RESIZE)
        
        # To emulate the functionality in the Save XY Data As
        # we set display button text as OK, but works as APPLY
        # We do not want the main dialog to be dismissed by
        # infrastructure.
        #
        self.appendActionButton('OK', self, self.ID_CLICKED_APPLY)
        self.appendActionButton('Cancel', self, self.ID_CANCEL)

        self.form = form
        #GUILOG
        getAFXAliasMap().setPrefix(self, 'SaveAs-%d'
                                   % SaveAsDB.numInstance)
        #ENDGUILOG

        self.text = AFXTextField(p=self, ncols=20, labelText='Name:', 
                         tgt=form.xyNewDataNameKw, sel=0, opts=LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.text, 'xyNewDataName')
        AFXNote(self, "XY Data are saved only for the\ncurrent Abaqus session.")