from abaqusGui import *
import os
from abq_Fft.fftPickProcedure import FftPickProcedure

###########################################################################
# Class definition
###########################################################################

class FftForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        AFXForm.__init__(self, owner)

        self.owner = owner
        self.cmd = AFXGuiCommand(mode=self, method='convertToFreqDomain',
            objectName='abq_Fft.fftModule', registerQuery=False)

        self.xyDataNameKw = AFXStringKeyword(self.cmd, 'xyDataName', False, 'XYData-1')
        self.xyNewDataNameKw = AFXStringKeyword(self.cmd, 'xyNewDataName', False, '_fft_temp_1')
        self.xyFilePathKw = AFXStringKeyword(self.cmd, 'xyFilePath', True)
        self.xyMinKw = AFXFloatKeyword(self.cmd, 'minV', False)
        self.xyMaxKw = AFXFloatKeyword(self.cmd, 'maxV', False)
        self.freqMinKw = AFXFloatKeyword(self.cmd, 'freqMin', False)
        self.freqMaxKw = AFXFloatKeyword(self.cmd, 'freqMax', False)
        self.proc = FftPickProcedure(owner, self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        from abq_Fft.fftDB import FftDB
        self.db = FftDB(self)
        return self.db


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):


        # Set requirement for frequencies cutoffs
        #
        if self.db.freqMin.getText() == '':
            self.freqMinKw.setRequired(False)
            self.freqMinKw.setValueToDefault()
        else:
            self.freqMinKw.setRequired(True)

        if self.db.freqMax.getText() == '':
            self.freqMaxKw.setRequired(False)
            self.freqMaxKw.setValueToDefault()
        else:
            self.freqMaxKw.setRequired(True)

        return AFXForm.verifyKeywordValues(self)


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        if self.db.readFile.getCheck():
            readFile = True
        else:
            readFile = False

        # Check for empty values
        #
        if readFile:
            self.xyDataNameKw.setValue('')
            if self.xyFilePathKw.getValue() == '':
                showAFXErrorDialog(self.db, 'A file name must be entered.')
                return False
            else:
                if not os.path.isfile(self.xyFilePathKw.getValue()):
                    showAFXErrorDialog(self.db, 'File specified does not exist.')
                    return False
        else:

            item = self.db.xyDataCombo.getCurrentItem()
            if item == -1:
                showAFXErrorDialog(self.db, 'Please select an XY data object.')
                return False
            else:
                self.xyDataNameKw.setValue(
                    self.db.xyDataCombo.getItemText(item))
            if float(self.db.timeMin.getText()) >= float(self.db.timeMax.getText()):
                showAFXErrorDialog(self.db, 'Minimum value must be less than maximum.')
                return False

        if self.db.saveAsDb:  
            if self.db.saveAsDb.shown():
                if self.xyNewDataNameKw.getValue() == '':
                    showAFXErrorDialog(self.db.saveAsDb, 'A name must be specified.')
                    return False
                from kernelAccess import session
                if self.xyNewDataNameKw.getValue() in session.xyDataObjects.keys():
                    showAFXErrorDialog(self.db.saveAsDb,
                        '"%s" already exists and cannot be overwritten.' % \
                        self.xyNewDataNameKw.getValue())
                    return False
            else: # Plot
                self.xyNewDataNameKw.setValue('_fft_temp_1')

        return True

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomTasks(self):
    
        currentDb = self.getCurrentDialog()
        if currentDb.saveAsDb:
            if currentDb.saveAsDb.shown():
                currentDb.saveAsDb.hide()

