from abaqus import *
from visualization import *
from customKernel import *
import math, sys
try:
    import numpy.fft as FFT
except:
    import FFT

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def convertToFreqDomain(xyDataName=None, xyNewDataName=None, xyFilePath=None,
                        minV=None, maxV=None,
                        freqMin=None, freqMax=None):


    yQuantity = None
    if xyDataName in session.xyDataObjects.keys():

        # Data from xy data
        #
        data = list(session.xyDataObjects[xyDataName].data) # Plot called

        # Set the x-axis type to be the same as xy data
        #
        yQuantity = session.xyDataObjects[xyDataName].axis2QuantityType

        # Apply cutoff time values
        #
        if minV:
            data = filter(lambda x:x[0]>minV, data)
        if maxV:
            data = filter(lambda x:x[0]<maxV, data)

        data = tuple(data)
        if len(data) < 2:
            raise ValueError,'Not enough data points selected given time range.'
    elif xyFilePath:
            
        # Data from file
        #
        data = getXyDataFromFile(xyFilePath)
    else:
        raise ValueError, 'No XY data object or file specified.'

    if not xyNewDataName:
        xyNewDataName = '_fft_temp_1'

    # Use first time split to get frequencies values
    #
    N = len(data)
    N_2 = N/2
    totalTimeSplit = N*(data[1][0] - data[0][0])
    frequencies = [(x+1)/totalTimeSplit for x in range(N_2)]

    # Use FFT module to calculate displacement values
    # in frequency domain. Multiply by 2/N to recover 
    # amplitudes, then take absolute values
    #
    fyFft = FFT.fft([y[1]*2/N for y in data])
    absFyFft = [abs(f) for f in fyFft[1:N_2+1]]

    fyFftData = zip(frequencies, absFyFft)

    # Apply cutoff frequencies
    #
    if freqMin:
        fyFftData = filter(lambda x:x[0]>freqMin, fyFftData)
    if freqMax:
        fyFftData = filter(lambda x:x[0]<freqMax, fyFftData)

    if len(fyFftData) < 2:
        raise ValueError,'Not enough data points selected given frequency range.'

    session.XYData(data=tuple(fyFftData), name=xyNewDataName)    
    xQuantity = QuantityType(type=FREQUENCY)
    if not yQuantity:
        yQuantity = QuantityType(type=DISPLACEMENT)
    session.xyDataObjects[xyNewDataName].setValues(axis1QuantityType=xQuantity, 
        axis2QuantityType=yQuantity, )

    if  xyNewDataName != '_fft_temp_1':
        print 'The XY data "%s" has been created.' % xyNewDataName
    else: # Plot
        if len(session.xyPlots.keys()) == 0:
            xyp = session.XYPlot('XYPlot-1')
        else:
            xypName = session.xyPlots.keys()[0]
            xyp = session.xyPlots[xypName]
        chartName = xyp.charts.keys()[0]
        chart = xyp.charts[chartName]
        xy1 = session.xyDataObjects[xyNewDataName]
        c1 = session.Curve(xyData=xy1)
        chart.setValues(curvesToPlot=(c1, ), )
        session.viewports[session.currentViewportName].setValues(
            displayedObject=xyp)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getXyDataFromFile(fileName):

    infile = open(fileName)
    lines = infile.readlines()

    data = []
    for line in lines:
        row = line.split(' ')
        while '' in row:
            row.remove('')
        if '\n' in row:
            row.remove('\n')
        if '\r\n' in row:
            row.remove('\r\n')
        if len(row) == 2:
            try:
                data.append((float(row[0]), float(row[1])))
            except:
                pass

    infile.close()
    return data

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def timeValue(data=None, xyDataName='', timeRange='min'):

    if len(data) <> 1:
        raise ValueError, 'Select one data point only.'
    try:
        d = session.xyDataObjects[xyDataName].data
    except:
        raise ValueError, 'Please select an XY Data object.'

    xVal = data[0][0]

    idx = len(filter(lambda x:x[0]<xVal, d))

    # Place the index value in the XY data object. Back in the procedure
    # we remove it
    #
    session.customData.idx = idx
