from abaqusGui import *

###########################################################################
# Class definition
###########################################################################

class FftPickProcedure(AFXProcedure):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, form):

        AFXProcedure.__init__(self, owner, AFXProcedure.SUBPROCEDURE)
        self.cmd = AFXGuiCommand(mode=self,
            method='timeValue', objectName='abq_Fft.fftModule')
        self.dataKw = AFXObjectKeyword(command=self.cmd,
            name='data', isRequired=True)
        self.xyDataName = AFXStringKeyword(command=self.cmd,
            name='xyDataName', isRequired=True)
        self.timeRange = AFXStringKeyword(command=self.cmd,
            name='timeRange', isRequired=True)
        self.form = form
        self.cb = None
        self.kw = None

        FXMAPFUNC(self, SEL_COMMAND, self.ID_ACTIVATE, FftPickProcedure.onIdActivate)


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstStep(self):

        # Return the first dialog to be posted
        #

        prompt = 'Select time %s' % self.cb.getLabelText().lower()
        self.xyDataName.setValue(self.form.db.xyDataCombo.getText())
        self.timeRange.setValue(self.cb.getLabelText().lower())
        self.step1 = AFXPickStep(owner=self, keyword=self.dataKw,
                           prompt=prompt,
                           entitiesToPick=XY_POINT)

        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def deactivate(self):
    
        # Before exiting, update table
        #
        from kernelAccess import session
        if hasattr(session.customData,'idx'):
            self.cb.setCurrentItem(session.customData.idx)
            if session.customData.idx > 0:
                self.kw.setValue(self.cb.getItemText(session.customData.idx))
        AFXProcedure.deactivate(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onIdActivate(self, sender, sel, ptr):
    
        if 'min' in sender.getText().lower():
            self.cb = self.form.db.timeMin
            self.kw = self.form.xyMinKw
        else:
            self.cb = self.form.db.timeMax
            self.kw = self.form.xyMaxKw
        AFXProcedure.activate(self)        