from abaqusGui import *
from abaqusConstants import *
from kernelAccess import session
from abq_FieldDiffer.fieldDifferConstants import *


###########################################################################
# Class definition
###########################################################################

class FieldDifferDB(AFXDataDialog):

    [
        ID_BASE_ODB,
        ID_BASE_STEP,
        ID_BASE_FRAME,
        ID_BASE_VARIABLE,
        ID_BASE_INV_COMP,
        ID_OTHER_ODB,
        ID_OTHER_STEP,
     ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+7)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):
    
        self.form = form

        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'Plot Differences in Fields', 0, DECOR_RESIZE)

        #GUILOG
        getAFXAliasMap().setPrefix(self, 'fieldDiffer')
        #ENDGUILOG

        # Set targets for keywords
        #
        form.baseOdbKw.setTarget(self)
        form.baseOdbKw.setSelector(self.ID_BASE_ODB)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_BASE_ODB, FieldDifferDB.updateBaseSteps)
        
        form.baseStepKw.setTarget(self)
        form.baseStepKw.setSelector(self.ID_BASE_STEP)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_BASE_STEP, FieldDifferDB.updateBaseFrames)
        
        form.baseFrameKw.setTarget(self)
        form.baseFrameKw.setSelector(self.ID_BASE_FRAME)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_BASE_FRAME, FieldDifferDB.updateBaseVariables)
        
        form.baseVariableKw.setTarget(self)
        form.baseVariableKw.setSelector(self.ID_BASE_VARIABLE)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_BASE_VARIABLE, FieldDifferDB.updateBaseInvComps)
                
        form.otherOdbKw.setTarget(self)
        form.otherOdbKw.setSelector(self.ID_OTHER_ODB)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_OTHER_ODB, FieldDifferDB.updateOtherSteps)
        
        form.otherStepKw.setTarget(self)
        form.otherStepKw.setSelector(self.ID_OTHER_STEP)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_OTHER_STEP, FieldDifferDB.updateOtherFrames)
        
        hf = FXHorizontalFrame(p=self, opts=LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)

        # Base results
        #
        gb = FXGroupBox(p=hf, text='Base Result', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        va = AFXVerticalAligner(gb, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        
        self.baseOdbCombo = AFXComboBox(p=va, ncols=-1, nvis=5, text='ODB:', tgt=form.baseOdbKw, sel=0)
        self.baseOdbCombo.setMaxVisible(10)
        self.baseStepCombo = AFXComboBox(p=va, ncols=-1, nvis=5, text='Step:', tgt=form.baseStepKw, sel=0)
        self.baseStepCombo.setMaxVisible(10)
        self.baseFrameCombo = AFXComboBox(p=va, ncols=-1, nvis=5, text='Frame:', tgt=form.baseFrameKw, sel=0)
        self.baseFrameCombo.setMaxVisible(10)
        self.baseVariableCombo = AFXComboBox(p=va, ncols=-1, nvis=5, text='Variable:', tgt=form.baseVariableKw, sel=0)
        self.baseVariableCombo.setMaxVisible(10)
        self.baseInvCompCombo = AFXComboBox(p=va, ncols=-1, nvis=5, text='Inv/Comp:', tgt=form.baseInvCompKw, sel=0)
        self.baseInvCompCombo.setMaxVisible(10)
        
        # Other results
        #
        gb = FXGroupBox(p=hf, text='Other Result', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        va = AFXVerticalAligner(gb, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)

        self.otherOdbCombo = AFXComboBox(p=va, ncols=-1, nvis=5, text='ODB:', tgt=form.otherOdbKw, sel=0)
        self.otherOdbCombo.setMaxVisible(10)
        self.otherStepCombo = AFXComboBox(p=va, ncols=-1, nvis=5, text='Step:', tgt=form.otherStepKw, sel=0)
        self.otherStepCombo.setMaxVisible(10)
        self.otherFrameCombo = AFXComboBox(p=va, ncols=-1, nvis=5, text='Frame:', tgt=form.otherFrameKw, sel=0)
        self.otherFrameCombo.setMaxVisible(10)
        self.otherVariableText = AFXTextField(p=va, ncols=-1, labelText='Variable:', opts=LAYOUT_FILL_X)
        self.otherVariableText.setText('Same as base')
        self.otherVariableText.setReadOnlyState(True)
        self.otherInvCompText = AFXTextField(p=va, ncols=-1, labelText='Inv/Comp:', opts=LAYOUT_FILL_X)
        self.otherInvCompText.setText('Same as base')
        self.otherInvCompText.setReadOnlyState(True)
        
        # Options
        #
        gb = FXGroupBox(p=self, text='Options', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        w = FXRadioButton(p=gb, text='Plot only the difference (other result - base result)', tgt=form.plotKw, sel=DIFFERENCE.getId() )
        getAFXAliasMap().setName(w, 'plotOptions1')
        w = FXRadioButton(p=gb, text='Plot the base, other, and difference in separate viewports', tgt=form.plotKw, sel=BASE_OTHER_DIFFERENCE.getId() )
        getAFXAliasMap().setName(w, 'plotOptions2')
        self.mutliPlotVf = FXVerticalFrame(gb, 0, 0,0,0,0, 20,0,0,0)
        hf = FXHorizontalFrame(self.mutliPlotVf, 0, 0,0,0,0, 0,0,0,0)
        w = FXRadioButton(p=hf, text='Tile vertically', tgt=form.tileModeKw, sel=VERTICAL.getId() )
        getAFXAliasMap().setName(w, 'plotOptions3')
        w = FXRadioButton(p=hf, text='Tile horizontally', tgt=form.tileModeKw, sel=HORIZONTAL.getId() )
        getAFXAliasMap().setName(w, 'plotOptions4')
        w = FXCheckButton(p=self.mutliPlotVf, text='Link the viewports', tgt=form.linkVpsKw, sel=0)
        getAFXAliasMap().setName(w, 'chkOptions')
        
        # Action buttons
        #
        w = self.appendActionButton('Plot', self, self.ID_CLICKED_APPLY)
        getAFXAliasMap().setName(w, 'plot')
        w = self.appendActionButton(self.CANCEL)
        getAFXAliasMap().setName(w, 'cancel')
        
        self.firstTime = True

        # Set up GUILOG names
        #
        getAFXAliasMap().setName(self.baseOdbCombo, 'baseOdbCombo')
        getAFXAliasMap().setName(self.baseStepCombo, 'baseStepCombo')
        getAFXAliasMap().setName(self.baseFrameCombo, 'baseFrameCombo')
        getAFXAliasMap().setName(self.baseVariableCombo, 'baseVariableCombo')
        getAFXAliasMap().setName(self.baseInvCompCombo, 'baseInvCompCombo')

        getAFXAliasMap().setName(self.otherOdbCombo, 'otherOdbCombo')
        getAFXAliasMap().setName(self.otherStepCombo, 'otherStepCombo')
        getAFXAliasMap().setName(self.otherFrameCombo, 'otherFrameCombo')
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
    
        session.odbs.registerQuery(self.updateOdbCombos)
        AFXDataDialog.show(self)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):
    
        session.odbs.unregisterQuery(self.updateOdbCombos)
        AFXDataDialog.hide(self)
                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def clearBaseSteps(self):
    
        self.baseStepNames = []
        self.baseStepCombo.clearItems()
        self.baseStep = None
        self.form.baseStepKw.setValue('')
        
        self.clearBaseFrames()
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def clearBaseFrames(self):
                
        self.baseNumFrames = 0
        self.baseFrameCombo.clearItems()
        self.baseFrame = None
        self.form.baseFrameKw.setValue('')

        self.clearBaseVariables()
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def clearBaseVariables(self):
                
        self.baseVariableNames = []
        self.baseVariableCombo.clearItems()
        self.baseVariable = None
        self.form.baseVariableKw.setValue('')

        self.clearBaseInvComps()
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def clearBaseInvComps(self):
                
        self.invariants = []
        self.components = []
        self.baseInvCompCombo.clearItems()
        self.baseInvComp = None
        self.form.baseInvCompKw.setValue('')
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def clearOtherSteps(self):
    
        self.otherStepNames = []
        self.otherStepCombo.clearItems()
        self.otherStepCombo.appendItem('Same as base')
        self.form.otherStepKw.setValue('Same as base')
        self.otherStep = None
        
        self.clearOtherFrames()
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def clearOtherFrames(self):
                
        self.otherNumFrames = 0
        self.otherFrameCombo.clearItems()
        self.otherFrameCombo.appendItem('Same as base')
        self.form.otherFrameKw.setValue('Same as base')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateOdbCombos(self):

        # Something changed in session.odbs, so update the ODB combo boxes
        
        self.odbNames = session.odbs.keys()
        self.odbNames.sort()
        # Populate base ODB combo box
        #
        self.baseOdbCombo.clearItems()
        for odbName in self.odbNames:
            self.baseOdbCombo.appendItem(odbName)
        
        # If a base ODB was previously specified and it still exists
        # in the list of ODBs, select it; otherwise, choose the first
        # ODB in the list.
        #
        baseOdbName = self.form.baseOdbKw.getValue()
        if baseOdbName in self.odbNames:
            self.baseOdb = session.odbs[baseOdbName]
            if self.firstTime:
                # Set step, frame, and variable
                self.updateBaseSteps(None, None, None)
                self.firstTime = False
        elif self.odbNames:
            baseOdbName = self.odbNames[0]
            self.baseOdb = session.odbs[baseOdbName]
            self.form.baseOdbKw.setValue(baseOdbName)
            # Reset step, frame, and variable
            self.updateBaseSteps(None, None, None)
        else:
            self.baseOdb = None
            self.form.baseOdbKw.setValue('')
            self.clearBaseSteps()
            
        # Populate other ODB combo box
        #
        self.otherOdbCombo.clearItems()
        self.otherOdbCombo.appendItem('Same as base')
        for odbName in self.odbNames:
            self.otherOdbCombo.appendItem(odbName)

        # If the other ODB name is currently set to 'Same as base', then
        # set the other ODB to be the same as the base ODB. 
        # If an other ODB was previously specified and it still exists
        # in the list of ODBs, select it; otherwise, if the previous name
        # no longer exists in the list, set the other ODB to be the same
        # as the base ODB.
        #
        otherOdbName = self.form.otherOdbKw.getValue()
        if otherOdbName == 'Same as base':
            self.otherOdb = self.baseOdb
            # Base ODB may have changed, so set other step and frame
            self.updateOtherSteps(None, None, None)
        elif otherOdbName in self.odbNames:
            self.otherOdb = session.odbs[otherOdbName]
            self.updateOtherSteps(None, None, None)
        elif self.odbNames:
            self.otherOdb = self.baseOdb
            # Base ODB may have changed, so set other step and frame
            self.form.otherOdbKw.setValue('Same as base')
            self.updateOtherSteps(None, None, None)
        else:
            self.otherOdb = None
            self.form.otherOdbKw.setValue('')
            self.clearOtherSteps()
                    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateBaseSteps(self, sender, sel, ptr):
    
        self.baseOdb = session.odbs[self.form.baseOdbKw.getValue()]
        
        # Populate base steps combo box
        #
        self.baseStepCombo.clearItems()
        self.baseStepNames = self.baseOdb.steps.keys()
        for baseStepName in self.baseStepNames:
            self.baseStepCombo.appendItem(baseStepName)
        
        # If a base step was previously specified and it still exists
        # in the list of steps, select it; otherwise, choose the first
        # step in the list.
        #
        baseStepName = self.form.baseStepKw.getValue()
        if baseStepName in self.baseStepNames:
            self.baseStep = self.baseOdb.steps[baseStepName]
        elif self.baseStepNames:
            baseStepName = self.baseStepNames[0]
            self.baseStep = self.baseOdb.steps[baseStepName]
            self.form.baseStepKw.setValue(baseStepName)
        else:
            self.baseStep = None
            self.form.baseStepKw.setValue('')
            self.clearBaseFrames()
        
        # Update the base frames.
        #
        if self.baseStep:
            self.updateBaseFrames(None, None, None)
            
        # If the other ODB is the same as base, update it too
        #
        if self.form.otherOdbKw.getValue() == 'Same as base':
            self.updateOtherSteps(None, None, None)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateBaseFrames(self, sender, sel, ptr):
            
        self.baseStep = self.baseOdb.steps[self.form.baseStepKw.getValue()]
        
        # Populate base frames combo box
        #
        self.baseFrameCombo.clearItems()
        self.baseNumFrames = len(self.baseStep.frames)
        for i in range(self.baseNumFrames):
            self.baseFrameCombo.appendItem('%s' % i, i)
                
        # Determine the base frame index
        #
        try:
            baseFrameId = int( self.form.baseFrameKw.getValue() )
        except:
            # Keyword value may be an empty string
            baseFrameId = 0

        # If a base frame index was previously specified and it still exists
        # in the list of frames, select it; otherwise, choose the first
        # frame in the list. Then set the base frame.
        #
        if baseFrameId in range(self.baseNumFrames):
            self.baseFrame = self.baseStep.frames[baseFrameId]
            # Set the keyword value in case the exception above was hit
            self.form.baseFrameKw.setValue(str(baseFrameId))
        elif self.baseNumFrames > 0:
            self.baseFrame = self.baseStep.frames[0]
            self.form.baseFrameKw.setValue('0')
        else:
            self.baseFrame = None
            self.form.baseFrameKw.setValue('')
            self.clearBaseVariables()
        
        # Update the base variables.
        #
        if self.baseFrame:
            self.updateBaseVariables(None, None, None)
        
        # If the other step is the same as base, update it too
        #
        if self.form.otherStepKw.getValue() == 'Same as base':
            self.updateOtherFrames(None, None, None)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateBaseVariables(self, sender, sel, ptr):
            
        self.baseFrame = self.baseStep.frames[int( self.form.baseFrameKw.getValue() )]
    
        # Populate base variables combo box
        #
        self.baseVariableCombo.clearItems()
        self.baseVariableNames = self.baseFrame.fieldOutputs.keys()
        self.baseVariableNames.sort()
        for baseVariableName in self.baseVariableNames:
            self.baseVariableCombo.appendItem(baseVariableName)
        
        # If a base variable was previously specified and it still exists
        # in the list of variables, select it; otherwise, choose the first
        # variable in the list. Then set the base variable.
        #
        baseVariableName = self.form.baseVariableKw.getValue()
        if baseVariableName in self.baseVariableNames:
            self.baseVariable = self.baseFrame.fieldOutputs[baseVariableName]
        elif self.baseVariableNames:
            baseVariableName = self.baseVariableNames[0]
            self.baseVariable = self.baseFrame.fieldOutputs[baseVariableName]
            self.form.baseVariableKw.setValue( baseVariableName )
        else:
            self.baseVariable = None
            self.form.baseVariableKw.setValue('')
            self.clearBaseInvComps()
                
        # Update the base inv/comp
        #
        if self.baseVariable:
            self.updateBaseInvComps(None, None, None)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateBaseInvComps(self, sender, sel, ptr):
    
        self.baseVariable = self.baseFrame.fieldOutputs[self.form.baseVariableKw.getValue()]
            
        # Populate base inv/comp combo box
        #
        self.baseInvCompCombo.clearItems()
        self.invariants = self.baseVariable.validInvariants
        self.components = self.baseVariable.componentLabels

        if self.invariants or self.components:
            self.baseInvCompCombo.enable()
            for invariant in self.invariants:
                self.baseInvCompCombo.appendItem( invariantsMap[str(invariant)] )

            for component in self.components:
                self.baseInvCompCombo.appendItem(component)

            # If a base variable was previously specified and it still exists
            # in the list of variables, select it; otherwise, choose the first
            # variable in the list. Then set the base variable.
            #
            baseInvCompName = self.form.baseInvCompKw.getValue()
            if baseInvCompName in self.invariants or baseInvCompName in self.components:
                pass
            elif self.baseInvCompCombo.getNumItems() > 0:
                self.form.baseInvCompKw.setValue( self.baseInvCompCombo.getItemText(0) )
            else:
                self.form.baseInvCompKw.setValue('')
        else:
            self.form.baseInvCompKw.setValueToDefault()
            self.baseInvCompCombo.disable()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateOtherSteps(self, sender, sel, ptr):
    
        otherOdbName = self.form.otherOdbKw.getValue()
        if otherOdbName == 'Same as base':
            self.otherOdb = self.baseOdb
            if not self.otherOdb:
                self.clearOtherSteps()
                return
        else:
            self.otherOdb = session.odbs[otherOdbName]
        
        # Populate other steps combo box
        #
        self.otherStepCombo.clearItems()
        self.otherStepNames = self.otherOdb.steps.keys()
        if self.otherStepNames:
            self.otherStepCombo.appendItem('Same as base')
        for otherStepName in self.otherStepNames:
            self.otherStepCombo.appendItem(otherStepName)
        
        # If an other step was previously specified and it still exists
        # in the list of steps, select it; otherwise, choose the first
        # step in the list. Then set the other step.
        #
        otherStepName = self.form.otherStepKw.getValue()
        if otherStepName == 'Same as base':
            try:
                self.otherStep = self.otherOdb.steps[self.form.baseStepKw.getValue()]
            except:
                # Ignore for now since user may not be done specifying inputs.
                # If this condition exists when the dialog is committed, the
                # kernel will raise an error.
                self.otherStep = None
                self.clearOtherFrames()
                
        elif otherStepName in self.otherStepNames:
            self.otherStep = self.otherOdb.steps[otherStepName]
        elif self.otherStepNames:
            otherStepName = self.otherStepNames[0]
            self.otherStep = self.otherOdb.steps[otherStepName]
            self.form.otherStepKw.setValue(otherStepName)
        else:
            self.otherStep = None
            self.form.otherStepKw.setValue('')
            self.clearOtherFrames()

        # Update the other frames
        #
        if self.otherStep:
            self.updateOtherFrames(None, None, None)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateOtherFrames(self, sender, sel, ptr):
            
        otherStepName = self.form.otherStepKw.getValue()
        if otherStepName == 'Same as base':
            try:
                self.otherStep = self.otherOdb.steps[self.form.baseStepKw.getValue()]
            except:
                # Ignore for now since user may not be done specifying inputs.
                # If this condition exists when the dialog is committed, the
                # kernel will raise an error.
                self.otherStep = None
                self.clearOtherFrames()
                return
                
        else:
            self.otherStep = self.otherOdb.steps[otherStepName]
            
        # Populate other frames combo box
        #
        self.otherFrameCombo.clearItems()
        self.otherNumFrames = len(self.otherStep.frames)
        otherFrames = []
        if self.otherNumFrames > 0:
            self.otherFrameCombo.appendItem('Same as base')
            otherFrames.append('Same as base')
        for i in range(self.otherNumFrames):
            self.otherFrameCombo.appendItem('%s' % i, i)
            otherFrames.append( str(i) )
            
        # If an other frame index was previously specified and it still exists
        # in the list of frames, select it; otherwise, choose the first
        # frame in the list.
        #
        otherFrameId = self.form.otherFrameKw.getValue()
        if otherFrameId not in otherFrames:
            if self.otherNumFrames > 0:
                self.form.otherFrameKw.setValue( otherFrames[0] )
            else:
                self.form.otherFrameKw.setValue('')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def processUpdates(self):
    
        if self.form.plotKw.getValue() == DIFFERENCE.getId():
            afxDisableWidgetTree(self.mutliPlotVf)
        else:
            afxEnableWidgetTree(self.mutliPlotVf)
