from abaqusGui import *
from abaqusConstants import *
from abq_FieldDiffer.fieldDifferConstants import *


###########################################################################
# Class definition
###########################################################################

class FieldDifferForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        
        # Construct the base class.
        #
        AFXForm.__init__(self, owner)

        self.cmd = AFXGuiCommand(mode=self, method='plotFieldDiffs',
            objectName='abq_FieldDiffer.fieldDifferUtils', registerQuery=False)
            
        self.baseOdbKw = AFXStringKeyword(self.cmd, 'baseOdbName', True)
        self.baseStepKw = AFXStringKeyword(self.cmd, 'baseStepName', True)
        self.baseFrameKw = AFXStringKeyword(self.cmd, 'baseFrameId', True)
        self.baseVariableKw = AFXStringKeyword(self.cmd, 'baseVariableName', True)
        self.baseInvCompKw = AFXStringKeyword(self.cmd, 'baseInvCompName', True)

        self.otherOdbKw = AFXStringKeyword(self.cmd, 'otherOdbName', True, 'Same as base')
        self.otherStepKw = AFXStringKeyword(self.cmd, 'otherStepName', True, 'Same as base')
        self.otherFrameKw = AFXStringKeyword(self.cmd, 'otherFrameId', True, 'Same as base')
        
        self.plotKw = AFXSymConstKeyword(self.cmd, 'fieldsToPlot', True, DIFFERENCE.getId())
        self.tileModeKw = AFXSymConstKeyword(self.cmd, 'tileMode', True, VERTICAL.getId())
        self.linkVpsKw = AFXBoolKeyword(self.cmd, 'linkVps', AFXBoolKeyword.TRUE_FALSE, True, True)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        from abq_FieldDiffer.fieldDifferDB import FieldDifferDB
        self.db = FieldDifferDB(self)
        return self.db
                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):
    
        # Check for empty values
        #
        if not self.hasValue( self.baseOdbKw, 'A base ODB' ):
            return False
    
        if not self.hasValue( self.baseStepKw, 'A base step' ):
            return False
    
        if not self.hasValue( self.baseFrameKw, 'A base frame' ):
            return False
    
        if not self.hasValue( self.baseVariableKw, 'A base variable' ):
            return False
        
        currentDb = self.getCurrentDialog()
        if currentDb.baseInvCompCombo.getNumItems():
            if not self.hasValue( self.baseInvCompKw, 'A base invariant/component' ):
                return False
    
        if not self.hasValue( self.otherOdbKw, 'An other ODB' ):
            return False
    
        if not self.hasValue( self.otherStepKw, 'An other step' ):
            return False
    
        if not self.hasValue( self.otherFrameKw, 'An other frame' ):
            return False
    
        # Check if the "same as base" values exist in the other ODB
        #
        if self.otherStepKw.getValue() == 'Same as base':
            if self.baseStepKw.getValue() in self.db.otherStepNames:
                otherStepName = self.baseStepKw.getValue()
            else:
                showAFXErrorDialog(self.db,
                    'Step "%s" does not exist in the other ODB.' % self.baseStepKw.getValue() )
                return False
        else:
            otherStepName = self.otherStepKw.getValue()
                
        if self.otherFrameKw.getValue() == 'Same as base':
            if int( self.baseFrameKw.getValue() ) in range(self.db.otherNumFrames):
                otherFrameId = self.baseFrameKw.getValue()
            else:
                showAFXErrorDialog(self.db,
                    'Frame %s does not exist in step "%s" of the other ODB.' % \
                    (self.baseFrameKw.getValue(), otherStepName) )
                return False
        else:
            otherFrameId = self.otherFrameKw.getValue()
                
        if self.otherOdbKw.getValue() == 'Same as base':
            otherOdbName = self.baseOdbKw.getValue()
        else:
            otherOdbName = self.otherOdbKw.getValue()
        
        otherFrame = session.odbs[otherOdbName].steps[otherStepName].frames[ int(otherFrameId) ]
        otherVariableNames = otherFrame.fieldOutputs.keys()
        if self.baseVariableKw.getValue() not in otherVariableNames:
            showAFXErrorDialog(self.db,
                'Variable %s does not exist in step "%s", frame %s of the other ODB.' % \
                (self.baseVariableKw.getValue(), otherStepName, otherFrameId) )
            return False

        return True
                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hasValue(self, kw, prefix):
    
        # Check for empty string
        
        if kw.getValue() == '':
            showAFXErrorDialog(self.db,
                '%s must be specified.' % prefix)
            return False
            
        return True
