from abaqus import *
from abaqusConstants import *
from visualization import *
from abq_FieldDiffer.fieldDifferConstants import *

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def plotFieldDiffs(
    baseOdbName, baseStepName, baseFrameId, baseVariableName, baseInvCompName,
    otherOdbName, otherStepName, otherFrameId, fieldsToPlot, tileMode, linkVps):
    
    # Get ODB's
    #
    baseOdb = session.odbs[baseOdbName]
    if otherOdbName == 'Same as base':
        otherOdb = baseOdb
    else:
        otherOdb = session.odbs[otherOdbName]
        
        # Check to see if the ODB's are compatible. This check is
        # not very thorough, but it's the simplest thing to do.
        #
        numBaseNodes = 0
        numBaseElements = 0
        for instance in baseOdb.rootAssembly.instances.values():
            numBaseNodes += len(instance.nodes)
            numBaseElements += len(instance.nodes)
        numOtherNodes = 0
        numOtherElements = 0
        for instance in otherOdb.rootAssembly.instances.values():
            numOtherNodes += len(instance.nodes)
            numOtherElements += len(instance.nodes)
        if numBaseNodes != numOtherNodes or numBaseElements != numOtherElements:
            raise ValueError, 'The difference could not be calculated\n' \
                              'because the mesh in the other ODB does\n' \
                              'not match the mesh in the base ODB.'
    
    # Get steps
    #
    baseStep = baseOdb.steps[baseStepName]
    if otherStepName == 'Same as base':
        otherStepName = baseStepName
    otherStep = otherOdb.steps[otherStepName]

    # Get frames
    #
    baseFrameIndex = int(baseFrameId)
    baseFrame = baseStep.frames[baseFrameIndex]
    if otherFrameId == 'Same as base':
        otherFrameId = baseFrameId
    otherFrameIndex = int(otherFrameId)
    otherFrame = otherStep.frames[ int(otherFrameId) ]
        
    # Get field outputs
    #
    baseFieldOutput = baseFrame.fieldOutputs[baseVariableName]
    otherFieldOutput = otherFrame.fieldOutputs[baseVariableName]
    
    # Get scratch ODB
    #
    if baseOdb.name in session.scratchOdbs.keys():
        scratchOdb = session.scratchOdbs[baseOdb.name]
    else:
        scratchOdb = session.ScratchOdb(odb=baseOdb)
    
    # Get scratch step, frame
    #
    if 'Field Differences' in scratchOdb.steps.keys():
        sessionStep = scratchOdb.steps['Field Differences']
        sessionFrame = sessionStep.frames[0]
    else:
        sessionStep = scratchOdb.Step(name='Field Differences',
            description='Results of differencing two fields', 
            domain=TIME, timePeriod=1.0)
        sessionFrame = sessionStep.Frame(frameId=0, frameValue=0.0,
            description='Results of differencing two fields' )

    # Create difference field
    #
    baseOdbName = os.path.split(baseOdb.name)[1]
    field1 = '%s_%s_%s_%s' % (baseOdbName, baseStepName, baseFrameId, baseVariableName)
    
    otherOdbName = os.path.split(otherOdb.name)[1]
    field2 = '%s_%s_%s_%s' % (otherOdbName, otherStepName, otherFrameId, baseVariableName)
    
    vDiff = otherFieldOutput - baseFieldOutput
    index = len(sessionFrame.fieldOutputs)
    fieldName = '%s-%d' % (baseVariableName, index)
    sessionField = sessionFrame.FieldOutput(name=fieldName,
        description='Difference between %s and %s' % (field1, field2), 
        field=vDiff)

    # Create difference plot
    #
    if 'Difference Plot' not in session.viewports.keys():
        diffVp = session.Viewport(name='Difference Plot', origin=(0,0), 
        width=100, height=150)
    else:
        diffVp = session.viewports['Difference Plot']
        
    baseStepIndex = baseOdb.steps.keys().index(baseStepName)
    diffVp.setValues(displayedObject=baseOdb)
    diffVp.odbDisplay.setFrame(frame=sessionFrame)
    
    position = sessionField.locations[0].position

    if baseInvCompName:
        if baseInvCompName in  sessionField.componentLabels:
            diffVp.odbDisplay.setPrimaryVariable(variableLabel=fieldName,
                outputPosition=position, refinement=(COMPONENT, baseInvCompName))
        else:
            diffVp.odbDisplay.setPrimaryVariable(variableLabel=fieldName,
                outputPosition=position, refinement=(INVARIANT, baseInvCompName))
    else:
        diffVp.odbDisplay.setPrimaryVariable(variableLabel=fieldName,
            outputPosition=position,)

    diffVp.odbDisplay.display.setValues(plotState=(CONTOURS_ON_DEF, ))
    
    # Create other plots
    #
    if fieldsToPlot == DIFFERENCE:
    
        diffVp.makeCurrent()
        diffVp.maximize()
        
    else:

        # Plot the base ODB
        #
        if 'Base ODB' not in session.viewports.keys():
            baseVp = session.Viewport(name='Base ODB', origin=(0,0), 
            width=100, height=150)
        else:
            baseVp = session.viewports['Base ODB']

        baseVp.setValues(displayedObject=baseOdb)
        baseVp.odbDisplay.setFrame(step=baseStepIndex, frame=baseFrameIndex)
        
        if baseInvCompName:
            if baseInvCompName in  baseFieldOutput.componentLabels:
                baseVp.odbDisplay.setPrimaryVariable(variableLabel=baseVariableName,
                    outputPosition=position, 
                    refinement=(COMPONENT, baseInvCompName))
            else:
                baseVp.odbDisplay.setPrimaryVariable(variableLabel=baseVariableName,
                    outputPosition=position, 
                    refinement=(INVARIANT, baseInvCompName))
        else:
            baseVp.odbDisplay.setPrimaryVariable(variableLabel=baseVariableName,
                outputPosition=position,)

        baseVp.odbDisplay.display.setValues(plotState=(CONTOURS_ON_DEF, ))
        
        # Plot the other ODB
        #
        if 'Other ODB' not in session.viewports.keys():
            otherVp = session.Viewport(name='Other ODB', origin=(0,0), 
            width=100, height=150)
        else:
            otherVp = session.viewports['Other ODB']

        otherStepIndex = otherOdb.steps.keys().index(otherStepName)
        otherVp.setValues(displayedObject=otherOdb)
        otherVp.odbDisplay.setFrame(step=otherStepIndex, frame=otherFrameIndex)

        if baseInvCompName:        
            if baseInvCompName in  otherFieldOutput.componentLabels:
                otherVp.odbDisplay.setPrimaryVariable(variableLabel=baseVariableName,
                    outputPosition=position, 
                    refinement=(COMPONENT, baseInvCompName))
            else:
                otherVp.odbDisplay.setPrimaryVariable(variableLabel=baseVariableName,
                    outputPosition=position, 
                    refinement=(INVARIANT, baseInvCompName))
        else:
            otherVp.odbDisplay.setPrimaryVariable(variableLabel=baseVariableName,
                outputPosition=position,)

        otherVp.odbDisplay.display.setValues(plotState=(CONTOURS_ON_DEF, ))
        
        baseVp.restore()
        otherVp.restore()
        diffVp.restore()
        
        # Tile viewports
        #
        originX,originY = session.drawingArea.origin
        
        if tileMode==VERTICAL:
        
            vpWidth = int(session.drawingArea.width/3)
            vpHeight = session.drawingArea.height
            
            baseVp.setValues(origin=(originX,originY), width=vpWidth, height=vpHeight)
            originX += vpWidth
            otherVp.setValues(origin=(originX,originY), width=vpWidth, height=vpHeight)
            originX += vpWidth
            diffVp.setValues(origin=(originX,originY), width=vpWidth, height=vpHeight)
            
        else:
        
            vpWidth = session.drawingArea.width
            vpHeight = int(session.drawingArea.height/3)
            
            diffVp.setValues(origin=(originX,originY), width=vpWidth, height=vpHeight)
            originY += vpHeight
            otherVp.setValues(origin=(originX,originY), width=vpWidth, height=vpHeight)
            originY += vpHeight
            baseVp.setValues(origin=(originX,originY), width=vpWidth, height=vpHeight)
        
        # Link viewports
        #
        for viewport in session.viewports.values():
            viewport.setValues(applyLinkedCommands=False)
        if linkVps:
            session.linkedViewportCommands.setValues(linkViewports=True)
            baseVp.setValues(applyLinkedCommands=True)
            otherVp.setValues(applyLinkedCommands=True)
            diffVp.setValues(applyLinkedCommands=True)
