"Dialog box for the Butterworth high-pass filter"

from abaqusGui import *
from kernelAccess import session

###########################################################################
# Class definition
###########################################################################

class HighPassDB(AFXDataDialog):
    "The class for definition of the dialog-box"
    
    # The constructor for the dialog box
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):
        "Constructor for the dialog box"
        
        AFXDataDialog.__init__(self,
                               form,
                               'Butterworth High Pass Filter',
                               self.OK|self.APPLY|self.CANCEL,
                               DIALOG_ACTIONS_SEPARATOR)

        getAFXAliasMap().setPrefix(self, 'HighPassDB')
        
        aligner = AFXVerticalAligner(self)
        xynames = session.xyDataObjects.keys()
        self.inplst = AFXComboBox(aligner,
                                  0,
                                  0,
                                  'XYData to filter:',
                                  tgt=form.inpXYKw,
                                  opts=LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.inplst, 'InpXYData')
        
        for xyname in xynames:
            self.inplst.appendItem(xyname)
            
        outXYData = AFXTextField(aligner,
                                 ncols=20,
                                 labelText='New XYData name:',
                                 tgt=form.outXYKw,
                                 sel=0,
                                 opts=AFXTEXTFIELD_STRING|LAYOUT_FILL_X)
        getAFXAliasMap().setName(outXYData, 'OutXYData')
        
        fc = AFXTextField(aligner,
                          ncols=5,
                          labelText='Cut-off frequency:',
                          tgt=form.fcKw,
                          sel=0,
                          opts=AFXTEXTFIELD_FLOAT|LAYOUT_FILL_X)
        getAFXAliasMap().setName(fc, 'fc')
        
        self.inplst.setMaxVisible(8)

    # Function to update the dialog box if XY data objects in the current
    # session are changed
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onXYDataChanged(self):
        "Function to update the dialog box if the XY data objects in the current session are changed"
        
        xynames = session.xyDataObjects.keys()
        self.inplst.clearItems()
        for xyname in xynames:
            self.inplst.appendItem(xyname)

    # Overloaded show function
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
        "Overloaded Show function"
        
        session.xyDataObjects.registerQuery(self.onXYDataChanged)
        AFXDataDialog.show(self)

    # Overloaded hide function
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):
        "Overloaded hide function"
        
        session.xyDataObjects.unregisterQuery(self.onXYDataChanged)
        AFXDataDialog.hide(self)
