"""
ABAQUS/CAE plug-in to perform Butterworth high-pass filtering of the XY-Data
"""

from abaqusGui import *
import os, uti

majorNumber, minorNumber, updateNumber = getAFXApp().getBaseVersionNumbers()
verAbaqus = majorNumber * 1.0 + minorNumber * 0.1
if verAbaqus > 6.6:
    import _abqPluginUtils

thisPath = os.path.abspath(__file__)
thisDir = os.path.dirname(thisPath)

# Use package syntax to avoid possible name clashes with other plugins
#
from abq_HighPass.highPassForm import HighPassForm

# Register commands
#
version = '1.2-1'
if verAbaqus < 6.6:
    status = "Plug-in " + str(version) + " version was not registered because "\
             "it is not supported in Abaqus version " + str(uti.getVersion())
elif verAbaqus ==6.6:
    status = 'OK'
else:
    status = _abqPluginUtils.checkCompatibility('Butterworth High-Pass Filter', version)
if status == 'OK':
    toolset=getAFXApp().getAFXMainWindow().getPluginToolset()
    toolset.registerGuiMenuButton(
        object=HighPassForm(toolset),
        buttonText='Tools|Butterworth High-Pass Filter...',
        messageId=AFXMode.ID_ACTIVATE,
        kernelInitString='import abq_HighPass.highPassKernel',
        applicableModules=["Visualization",],
        version=version,
        author='ABAQUS, Inc.',
        description='This plug-in performs Butterworth high-pass filtering of an XYData',
        helpUrl=os.path.join(thisDir, 'abq_HighPass', 'HighPass.html'),
        )
else:
    getAFXApp().getAFXMainWindow().writeToMessageArea(status)
    getAFXApp().beep()
