from abaqus import *

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def writeInputFromViewport(fileName):

    vp = session.viewports[session.currentViewportName]
    try:
        odbName = vp.odbDisplay.name
        odb = session.odbs[odbName]
    except:
        raise ValueError, 'An ODB must be displayed in the current viewport.'

    nodes = {}
    elements = {}
    activeElementInfo = vp.getActiveElementLabels()
    instanceNames = activeElementInfo.keys()
    instanceNames.sort()
    u = odb.steps.values()[-1].frames[-1].fieldOutputs['U']
    
    try:
        for instanceName in instanceNames:

            disp = getDisplacements(u, instanceName)
           
            elementLabels = activeElementInfo[instanceName]
            for elementLabel in elementLabels:

                element = odb.rootAssembly.instances[instanceName].getElementFromLabel(elementLabel)
                connectivity = element.connectivity

                instanceName = element.instanceNames[0]
                sectionCategory = element.sectionCategory
                elemType = element.type

                # Elements
                #

                if not elements.has_key(instanceName):
                    elements[instanceName] = {}

                if not elements[instanceName].has_key(elemType):
                    elements[instanceName][elemType] = []

                s = '%s, ' % elementLabel
                for nodeLabel in connectivity:
                    s += '%s, ' % nodeLabel
                elements[instanceName][elemType].append(s[:-2])


                # Nodes
                #

                for nodeNumber in connectivity:

                    if not nodes.has_key(instanceName):
                        nodes[instanceName] = {}

                    if not nodes[instanceName].has_key(nodeNumber):
                        nodes[instanceName][nodeNumber] = []


                    coords = odb.rootAssembly.instances[instanceName].getNodeFromLabel(nodeNumber).coordinates
                    s = '%s, ' % nodeNumber
                    i = 0
                    deform = disp[nodeNumber]
                    deformSize = len(deform)
                    for coord in coords:
                        udef = 0
                        if i < deformSize:
                            udef = disp[nodeNumber][i]
                            
                        s += '%s, ' % (coord + udef )
                        i += 1
                                       
                    nodes[instanceName][nodeNumber] = s[:-2]

        file = open(fileName, 'w')

        # Heading
        #
        file.write('*Heading\n')
        file.write('** File created by input file utility\n')

        for instanceName in instanceNames:


            # Instance
            #
            file.write('*Part, name=%s\n' % (instanceName[:-2]) )


            # Nodes
            #
            file.write('*Node\n')
            nodeNumbers = nodes[instanceName].keys()
            nodeNumbers.sort()
            for nodeNumber in nodeNumbers:
                file.write('%s\n' % nodes[instanceName][nodeNumber] )

            # Elements
            #

            elementTypes = elements[instanceName].keys()
            elementTypes.sort()
            for elementType in elementTypes:
                file.write('*Element, Type=%s\n' % elementType)
                for element in elements[instanceName][elementType]:
                    file.write('%s\n' % element)

            file.write('*End Part\n' )


        #file.write('*End Assembly\n')

        # Materials
        #
        # TBD

        file.close()

        print 'The input file "%s" has been written.' % fileName
    except ValueError:
        raise ValueError,"An error occurred while writing the input file.\n"\
                         "Please contact SIMULIA support for further assistance."

def getDisplacements(u, instanceName):
    
    disp = {}
    blocks = u.bulkDataBlocks
    for block in blocks:
        if block.instance.name != instanceName:
            continue
        
        nodeLabels = block.nodeLabels
        data = block.data
        i = 0
        for node in nodeLabels:
            disp[node] = data[i]
            i += 1
    return disp
        
    
