from abaqusGui import *
import abaqusConstants

###########################################################################
# Class definition
###########################################################################

class MacDB(AFXDataDialog):

    [
        ID_ODBA,
        ID_ODBB,
        ID_FILE_OK,
        ID_COMP,
        ID_STEPS,
        ID_CLICKED_APPLY_CONTOUR,
        ID_CLICKED_APPLY_BAR,
        ID_LAST
    ] = range(AFXDialog.ID_LAST, AFXDialog.ID_LAST+8)

    numInstance = 0

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        MacDB.numInstance += 1

        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'MAC', 0,DECOR_RESIZE|DIALOG_ACTIONS_SEPARATOR)

        self.appendActionButton('Contour', self, self.ID_CLICKED_APPLY_CONTOUR)
        self.appendActionButton('Bar Graph', self, self.ID_CLICKED_APPLY_BAR)
        self.appendActionButton('Dismiss', self, self.ID_CLICKED_DISMISS)

        self.form = form
        self.odbDb = None
        self.patterns = None
        self.A = False
        self.B = False
        self.moduleImported = False
        self.fileNameTgt = AFXStringTarget()

        # MAP the select FX button to the method of CLASS FastenerDB.onCmdSelect
        #
        FXMAPFUNC(self, SEL_COMMAND, self.ID_ODBA, MacDB.onCmdSelectFile)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_ODBB, MacDB.onCmdSelectFile)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_FILE_OK, MacDB.onCmdOdb)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_COMP, MacDB.onChkComp)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_STEPS, MacDB.onGetSteps)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CLICKED_APPLY_CONTOUR, MacDB.onCmdContour)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CLICKED_APPLY_BAR, MacDB.onCmdBar)

        #GUILOG
        getAFXAliasMap().setPrefix(self, 'MacDB-%d'
                                   % MacDB.numInstance)
        #ENDGUILOG

        #GUILOG
        w = self.getActionButton(self.ID_CLICKED_APPLY_CONTOUR)
        getAFXAliasMap().setName(w, 'Contour')
        w = self.getActionButton(self.ID_CLICKED_APPLY_BAR)
        getAFXAliasMap().setName(w, 'Bar')
        #ENDGUILOG

        # Odb A name field
        #
        gb = FXGroupBox(p=self, text='Data A', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        hf = FXHorizontalFrame(p=gb, opts=LAYOUT_FILL_X)
        FXLabel(p=hf, text='ODB A ')
        self.odbALabelField = FXLabel(p=hf, text='None')
        self.buttonA = FXButton(p=hf, text='Select...', ic=None, tgt=self, 
                                sel=self.ID_ODBA, opts=BUTTON_NORMAL|LAYOUT_RIGHT)
        self.comboBoxA = AFXComboBox(p=gb, ncols=0, nvis=3, text='*FREQUENCY steps', 
                                     tgt=form.dataAStepKw, sel=0, opts=LAYOUT_FILL_X)

        # Odb B name field
        #        
        gb = FXGroupBox(p=self, text='Data B', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        hf = FXHorizontalFrame(p=gb, opts=LAYOUT_FILL_X)
        FXLabel(p=hf, text='ODB A ')
        self.odbBLabelField = FXLabel(p=hf, text='None')
        self.buttonB = FXButton(p=hf, text='Select...', ic=None, tgt=self, 
                                sel=self.ID_ODBB, opts=BUTTON_NORMAL|LAYOUT_RIGHT)
        self.comboBoxB = AFXComboBox(p=gb, ncols=0, nvis=3, text='*FREQUENCY steps', 
                                     tgt=form.dataBStepKw, sel=0, opts=LAYOUT_FILL_X)

        # Get steps button
        #
        hf = FXHorizontalFrame(p=self, opts=LAYOUT_FILL_X)
        self.getSteps = FXButton(p=hf, text='Get steps...', ic=None, tgt=self, 
                                sel=self.ID_STEPS, opts=BUTTON_NORMAL)
        self.getSteps.disable()

        # Compatibility Check button
        #
        hf = FXHorizontalFrame(p=self, opts=LAYOUT_FILL_X)
        self.compBut = FXButton(p=hf, text='Check compatibility:', ic=None, tgt=self, 
                                sel=self.ID_COMP, opts=BUTTON_NORMAL)
        self.compBut.disable()
        self.compFactorLabel = FXLabel(p=hf, text='Please select Output databases.')
        self.compFactorLabel.setTextColor(FXRGB(139,0,0))

        #GUILOG
        getAFXAliasMap().setName(self.odbALabelField, 'ODB A')
        getAFXAliasMap().setName(self.buttonA, 'Select A')
        getAFXAliasMap().setName(self.comboBoxA, 'Data A')
        getAFXAliasMap().setName(self.odbBLabelField, 'ODB B')
        getAFXAliasMap().setName(self.buttonB, 'Select B')
        getAFXAliasMap().setName(self.comboBoxB, 'Data B')
        getAFXAliasMap().setName(self.getSteps, 'getSteps')
        getAFXAliasMap().setName(self.compBut, 'compBut')
        #ENDGUILOG

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdContour(self, sender, sel, ptr):

        # Send command with contour id
        #
        self.form.plotType.setValue(abaqusConstants.CONTOURS_ON_UNDEF.getId())
        self.form.commit()
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdBar(self, sender, sel, ptr):

        # Send command with bar id
        #
        self.form.plotType.setValue(abaqusConstants.BANDED.getId()) # Using BANDED because 6.6 does not have BAR
        self.form.commit()
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdSelectFile(self, sender, sel, ptr):

        if SELID(sel) == self.ID_ODBA:
            self.patterns = 'Output A database (*.odb)'
        else:
            self.patterns = 'Output B database (*.odb)'

        # Post select Odb DB
        #
        if not self.odbDb:
            self.odbDb = AFXFileSelectorDialog(self, 'Select Database', 
                self.fileNameTgt, None, 
                AFXSELECTFILE_ANY|AFXSELECTFILE_REMOTE_HOST, self.patterns)
            getAFXAliasMap().setPrefix(self.odbDb, 'FileDB-%d' % MacDB.numInstance)
            
            self.odbDb.setReadOnlyPatterns('*.odb')
            self.odbDb.create()
            self.fileNameTgt.setTarget(self)
            self.fileNameTgt.setSelector(self.ID_FILE_OK)

        self.odbDb.showModal()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdOdb(self, sender, sel, ptr):

        # Check if paths are valid
        #
        fileName = self.fileNameTgt.getValue().strip()
        if not fileName:
            showAFXErrorDialog(self,
                'A file name must be specified.')
            return
        if not os.path.exists(fileName):
            showAFXErrorDialog(self,
                'File not found: %s' % fileName)
            return
        if self.patterns.startswith('Output A'):
            self.odbALabelField.setText(fileName)
            self.A = True
        else:
            self.odbBLabelField.setText(fileName)
            self.B = True

        if self.A & self.B:
            self.getSteps.enable()
            self.compFactorLabel.setText('Please select steps by click on Get steps...')

        return

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onGetSteps(self, sender, sel, ptr):

        getAFXApp().beginWaitCursor()
        getAFXApp().forceRefresh()

        try:
            # Send command to parse odb and take last step from customData
            #
            if not self.moduleImported:
                sendCommand('import abq_Mac.macModule')
                self.moduleImported = True
            sendCommand('abq_Mac.macModule.openOdb(odbAPath="' + self.odbALabelField.getText() + 
                        '", odbBPath="' + self.odbBLabelField.getText() + '")')
            from kernelAccess import session
            self.comboBoxA.clearItems()
            self.comboBoxB.clearItems()
            for step in session.customData.odbSteps[0]:
                self.comboBoxA.appendItem(step)
            self.form.dataAStepKw.setValue(
                self.comboBoxA.getItemText(len(session.customData.odbSteps[0])-1))
            for step in session.customData.odbSteps[1]:
                self.comboBoxB.appendItem(step)
            self.form.dataBStepKw.setValue(
                self.comboBoxB.getItemText(len(session.customData.odbSteps[1])-1))
            self.compBut.enable()
            self.compFactorLabel.setText('Compatibility factor not calculated')
            self.compFactorLabel.setTextColor(FXRGB(0,0,0))
        except:
            getAFXApp().endWaitCursor()
            showAFXErrorDialog(self,
                'An error occurred while trying to process the data:\n\n%s' % sys.exc_info()[1] )
            return

        getAFXApp().endWaitCursor()



    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onChkComp(self, sender, sel, ptr):

        getAFXApp().beginWaitCursor()
        getAFXApp().forceRefresh()

        try:
            # If both ODBs have steps selected, activate button
            #
            if self.comboBoxA.getCurrentItem() == -1 & self.comboBoxB.getCurrentItem() == -1:
                self.compFactorLabel.setText('Please select steps for compatibility factor calculation')
                getAFXApp().endWaitCursor()
                return

            stepA = self.comboBoxA.getItemText(self.comboBoxA.getCurrentItem())
            stepB = self.comboBoxB.getItemText(self.comboBoxB.getCurrentItem())
            self.compFactorLabel.setText('Calculating compatibility factor')
            sendCommand('session.customData.compFactor = abq_Mac.macModule.macPlot(dataAStep="' + stepA + 
                        '", dataBStep="' + stepB + '", plotType=COMP_DEFAULT)')
            self.compFactorLabel.setText('%*.*f' % (1,3,session.customData.compFactor))

        except:
            getAFXApp().endWaitCursor()
            showAFXErrorDialog(self,
                'An error occurred while trying to process the data:\n\n%s' % sys.exc_info()[1] )
            return

        getAFXApp().endWaitCursor()


