from abaqusGui import *
import abaqusConstants

###########################################################################
# Class definition
###########################################################################

class MacForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        # Construct the base class
        #
        AFXForm.__init__(self, owner)

        # Command to openOdb
        #            
        self.cmd = AFXGuiCommand(self, 'macPlot', 'macOdb = abq_Mac.macModule')
        self.dataAStepKw = AFXStringKeyword(self.cmd, 'dataAStep', TRUE, '')
        self.dataBStepKw = AFXStringKeyword(self.cmd, 'dataBStep', TRUE, '')
        self.plotType = AFXSymConstKeyword(self.cmd, 'plotType', TRUE, abaqusConstants.COMP_DEFAULT.getId())

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        from abq_Mac.macDB import MacDB
        return MacDB(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        # Method to check if the odbName is valid
        # 
        currentDb = self.getCurrentDialog()
        
        if currentDb.odbALabelField.getText().strip() == '':
            showAFXErrorDialog(currentDb,'ODB file name must be specified.')
            currentDb.odbATextLabel.setFocusToTextField()
            return FALSE
        if currentDb.odbBLabelField.getText().strip() == '':
            showAFXErrorDialog(currentDb,'ODB file name must be specified.')
            currentDb.odbBTextLabel.setFocusToTextField()
            return FALSE
        if self.dataAStepKw.getValue() == "":
            showAFXErrorDialog(currentDb, 'A step must be specified.')
            currentDb.comboBoxA.setFocusToComboBox()
            return FALSE
        if self.dataBStepKw.getValue() == "":
            showAFXErrorDialog(currentDb, 'A step must be specified.')
            currentDb.comboBoxB.setFocusToComboBox()
            return FALSE

        return TRUE

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def handleException(self, exception):

        errorType = exception[0]
        errorValue = exception[1]
        db = self.getCurrentDialog()
        if errorType == ZeroDivisionError:
            showAFXErrorDialog(db,
                'Division by zero has occurred.\n'
                'A possible reason is mismatching node numbers.\n'
                'Please ensure that node numbers match between '
                'output databases.')
        else:
            showAFXErrorDialog(db, str(errorValue))
