## Automatically adapted for numpy.oldnumeric Mar 12, 2012 by ABQvwrK.exe

"""
Module to compute modal assurance component (MAC) and
generate a graphical representation of it.
"""
from abaqus import *
from abaqusConstants import *
try:
    import numpy.oldnumeric as Numeric
except:
	import numpy as Numeric
#from numpy.oldnumeric import array, Int32, Float64

from odbAccess import *
import os, uti, customKernel

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def openOdb(odbAPath, odbBPath):
    """
    Method to open *FREQUENCY ODBs and store step names in the session.customData object.
    Required arguments:
    odbPath: The path to the *FREQUENCY ODB
    
    Returns the odb object.
    """
    
    # Check the steps
    #
    session.customData.odbPaths = [odbAPath, odbBPath]
    session.customData.odbSteps = [[],[]]
    pathIndex = 0
    for odbPath in session.customData.odbPaths:
        if type(odbPath) is not str:
            raise TypeError, 'odbPath must be a string; got %s.' % type(odbPath)

        if odbPath.strip() == '':
            raise ValueError, 'Path cannot be an empty string.'

        if session.isUpgradeRequiredForOdb(upgradeRequiredOdbPath=odbPath):
            raise ValueError, ('%s\nis from a previous '
                'version of Abaqus.\nPlease close the plug-in and upgrade '
                'database before continuing.' % odbPath)

        odb = session.openOdb(odbPath)

        # Do odb type and step checks
        #
        if type(odb) != OdbType:
            raise TypeError, 'odb must be of type OdbType; got %s.' % type(odb)
        for step in odb.steps.values():
            if step.procedure == '*FREQUENCY':
                session.customData.odbSteps[pathIndex].append(step.name)

        # Make sure variables exist
        #
        for step in session.customData.odbSteps[pathIndex]:
            if not hasattr(odb.steps[step], "frames"):
                session.customData.odbSteps[pathIndex].remove(step)
            elif not hasattr(odb.steps[step].frames[-1], "fieldOutputs"):
                session.customData.odbSteps[pathIndex].remove(step)
            elif not odb.steps[step].frames[-1].fieldOutputs.has_key('U'):
                session.customData.odbSteps[pathIndex].remove(step)
            elif not hasattr(odb.steps[step].frames[-1].fieldOutputs['U'], "values"):
                session.customData.odbSteps[pathIndex].remove(step)

        if not session.customData.odbSteps[pathIndex]:
            raise ValueError, 'The ODB "%s" does not contain a *FREQUENCY step with field output "U" values.' % odbPath
            return None

        # Increase counter of pathIndex.  This avoids if same odb is used
        #
        pathIndex += 1

    return 1

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def macPlot(dataAStep, dataBStep, plotType=CONTOURS_ON_UNDEF):

    # Open odbs
    #
    odbA = session.odbs[session.customData.odbPaths[0]]
    odbB = session.odbs[session.customData.odbPaths[1]]

    # Read values from both odbs
    valA = odbA.steps[dataAStep].frames[-1].fieldOutputs['U'].values
    valB = odbB.steps[dataBStep].frames[-1].fieldOutputs['U'].values

    # Check that odb A is bigger than B, swap if needed
    #
    swap=False
    if len(valA) < len(valB):
        valA,valB = valB,valA
        odbA,odbB = odbB,odbA
        dataAStep, dataBStep = dataBStep, dataAStep
        swap=TRUE

    # Check which B node labels exist in A and add it to takeList
    #
    i=0
    j=0
    takeList=[]
    for a in valA:
        if i >= len(valB): break
        if a.nodeLabel == valB[i].nodeLabel:
            takeList.append(j)
            i+=1
        j+=1

    # Retrieve values and take only the necessary node labels
    # Note we filter only frames (modes) that contain 'Increment'
    # in the description.
    #
    arrayA=Numeric.array([[v.data for v in f.fieldOutputs['U'].values]
                                  for f in odbA.steps[dataAStep].frames
                                  if f.description[:9] != 'Increment'])
    arrayB=Numeric.array([[v.data for v in f.fieldOutputs['U'].values] 
                                  for f in odbB.steps[dataBStep].frames
                                  if f.description[:9] != 'Increment'])
    arrayA=Numeric.take(arrayA,takeList,1)

    # Take number of modes from the odb that has the fewest
    # and take only those modes from each array
    #
    matrixLen = min(len(arrayA),len(arrayB))
  
    arrayA = arrayA[:matrixLen]
    arrayB = arrayB[:matrixLen]

    if plotType==COMP_DEFAULT:
        imcVal = _calculateMacMatrix([arrayA[0]], [arrayB[0]], 1)
        print 'The compatibility check of the first modes gives ', imcVal[0][0]
        return imcVal[0][0]

    macMatrix = _calculateMacMatrix(arrayA, arrayB, matrixLen)

    # Send diagonal to be plotted (bar graph)
    #
    if plotType==BANDED:
        # Get diagonal of matrix
        #
        macDiag=[macMatrix[a][a] for a in range(matrixLen)]

        _makeMacBarGraph(macDiag, matrixLen)
    else:
        _createODB(macMatrix,matrixLen,swap)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _calculateMacMatrix( arrayA, arrayB, matrixLen):

    # Calculate macs and write to file
    #
    macMatrixS = ''
    macMatrix=[range(matrixLen) for a in range(matrixLen)]
    outfileText = open('MAC_AB1.txt','w')
    m=0

    # To find MAC:
    # Nnumerator (num): the sum of the dot product of components.
    # Denominator (denA and denB): the sum of the square
    # of the magnitude of components
    #
    try:
        for a in arrayA:
            for b in arrayB:
                k = 0
                num = 0
                denA = 0
                denB = 0
                for i in a:

                    # Find dot product of  components
                    #
                    j = b[k]
                    num += Numeric.dot(i, j)
                    denA += Numeric.dot(i,i)
                    denB += Numeric.dot(j,j)
                    k += 1

                # If any of the row or column is zero, we will
                # filter them out
                # 
                if denA == 0 or denB == 0:
                    MAC = 0.0
                    if denA == 0 and denB == 0: # Take only both are 0
                        matrixLen-=1
                else:
                    MAC = num**2/(denA*denB)
                    macMatrixS += '%8.5f, ' % MAC
                    macMatrix[m/matrixLen][m%matrixLen]=MAC
                    m+=1
            outfileText.write(macMatrixS +'\n')
            macMatrixS = ''
        outfileText.close()

    except ZeroDivisionError:
        print 'Division by zero has occurred.'
        print 'A possible reason is mismatching node numbers.'
        print 'Please ensure that node numbers match between output databases.'
        return False
    except:
        print 'A general error has occurred.\nPlease contact customer service.'
        return False

    return macMatrix

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _makeMacBarGraph(mac_diag, matrixLen):

    """
    Method to create the bar graphs for the specified macs.
    The bar graphs are plotted in the current viewport.    
    """ 

    # Get current viewport
    #
    vp = session.viewports[session.currentViewportName]
    bgName = vp.name

    # Create XYPlot
    #
    try:
        xyp = session.XYPlot(name =bgName)
    except:
        xyp = session.xyPlots[bgName]

    # Now create the appropriate curves               
    #
    start = (0.1,0.0)
    width = .8
    inc = .2    
    curvesToPlot = []
    name = 'Mode '

    i=1
    for mag in mac_diag:
        name = 'mode '+str(i)
        i+=1
        xyData = []
        xyData.append(start)
        first = start[0],mag
        xyData.append(first)
        xyData.append(first)
        second = start[0]+width, mag
        xyData.append(second)
        xyData.append(second)
        third = start[0]+width,start[1]
        xyData.append(third)

        # So that the xy data does not interfere with viewports
        #
        xyDataName = '%s-%s' % (bgName, name)
        xyData = session.XYData(name = xyDataName,data = xyData)

        if (uti.getVersion()).startswith('6.6'):
            curve = xyp.Curve(name, xyData)
            curve.curveOptions.setValues(lineThickness=MEDIUM,
                                     legendTextSource=USER_DEFINED,
                                     legendText=name)
        else:
            curve = session.Curve(name, xyData)

        curvesToPlot.append(name)
        start = third[0]+inc,start[1]


    # Differences between 6.6 and 6.7
    if (uti.getVersion()).startswith('6.6'):
        xyp.setValues(curvesToPlot=curvesToPlot)
        xyp.xyPlotOptions.setValues(xAxisMaxAutoCompute=OFF,xAxisMaxValue=matrixLen)
        xyp.xyPlotOptions.setValues(xAxisMinAutoCompute=OFF,xAxisMinValue=0)
        xyp.xyPlotOptions.setValues(yAxisMinAutoCompute=OFF,yAxisMaxAutoCompute=OFF)
        xyp.xyPlotOptions.setValues(xAxisMajorTickMode=TOTAL_NUMBER,
                                    numXAxisMajorTicks=matrixLen-1)
        xyp.xyPlotOptions.setValues(xAxisLabelDecimalPlaces=0,
                                    xAxisLabelFrequency=0,
                                    yAxisMajorTickMode=TOTAL_NUMBER,
                                    yAxisLabelDecimalPlaces=0,
                                    yAxisLabelFrequency=1)
        xyp.xyPlotOptions.setValues(numYAxisMajorTicks=0)
        xyp.xyPlotOptions.setValues(numYAxisMinorTicks=0)
        xyp.xyPlotOptions.setValues(numXAxisMinorTicks=0)
        xyp.setValues(xAxisTitleSource=USER_DEFINED,
                      yAxisTitleSource=USER_DEFINED, xAxisTitle='Modes', 
                      yAxisTitle='MAC diagonal')

    else:
        chartName = xyp.charts.keys()[0]
        chart = xyp.charts[chartName]
        chart.setValues(curvesToPlot=(curvesToPlot), )
        chart.gridArea.style.setValues(color='#FFFFFF')

        chart.axes1[0].axisData.setValues(maxAutoCompute=False,maxValue=matrixLen)
        chart.axes1[0].axisData.setValues(minAutoCompute=False,minValue=0)
        chart.axes1[0].setValues(labelFrequency=0)
        chart.axes1[0].axisData.setValues(tickMode=TOTAL_NUMBER,
                                    tickCount=matrixLen-1)
        chart.axes1[0].axisData.setValues(minorTickCount=0)
        chart.axes1[0].axisData.setValues(useSystemTitle=False, title='Modes')

        chart.axes2[0].axisData.setValues(maxAutoCompute=False, maxValue=1)
        chart.axes2[0].setValues(labelFrequency=1)
        chart.axes2[0].axisData.setValues(tickMode=TOTAL_NUMBER)
        chart.axes2[0].axisData.setValues(tickCount=0)
        chart.axes2[0].axisData.setValues(minorTickCount=0)
        chart.axes2[0].axisData.setValues(useSystemTitle=False, title='MAC Diagonal')
        chart.axes2[0].axisData.setValues(labelNumDigits=0)

    vp.setValues(displayedObject=xyp)
    vp.viewportAnnotationOptions.setValues(state=OFF, title=OFF, annotations=OFF)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def _createODB(macMatrix,matrixLen,swap):

    """
    This method will create an ODB with shell elements.
    The elements will be used to view the MAC matrix in a contour plot
    """

    # Create an ODB (which also creates the rootAssembly)
    #
    index = 1
    macOdbFileName = 'contourMAC' + str(index)
    while os.path.isfile(macOdbFileName + '.odb'):
        index += 1
        macOdbFileName = 'contourMAC' + str(index)

    odb = Odb(name=macOdbFileName,
              analysisTitle='ODB created with NVH MAC plug-in',
              description='shell elements for contour plot ',
              path=macOdbFileName+'.odb')

    # Set up the section categories.
    #
    sCat = odb.SectionCategory(name='M', description='MAC')
    
    #  Create shell model,
    #
    matrixLenE=matrixLen-1
    nodeData = [(n+1,n%matrixLen,n/matrixLen) for n in range(matrixLen*matrixLen)]
    part1 = odb.Part(name='part-1', embeddedSpace=TWO_D_PLANAR, type=DEFORMABLE_BODY)
    part1.addNodes(nodeData=nodeData, nodeSetName='nset-1')
    
    elementData = [(n+1, (n/matrixLenE)+n+1,
                         (n/matrixLenE)+n+2,
                         (n/matrixLenE)+n+3+matrixLenE,
                         (n/matrixLenE)+n+2+matrixLenE)
                         for n in range(matrixLenE*matrixLenE)]

    part1.addElements(elementData=elementData, type='S4')
    
    # Instance the part.
    #
    instance1 = odb.rootAssembly.Instance(name='part-1-1', object=part1)

    # Create instance level sets for section assignment
    #
    elLabels = [n+1 for n in range(matrixLenE*matrixLenE)]
    elset_1 = odb.rootAssembly.instances['part-1-1'].\
        ElementSetFromElementLabels(name='MAC',
        elementLabels=elLabels)
    #instance1.assignSection(region=elset_1,section=section_1)
    
    # Field data:
    #
    
    # Create a step and a frame.
    #
    step1 = odb.Step(name='mac-1', description='MAC step', domain=TIME, timePeriod=1.0)
    frame1 = step1.Frame(incrementNumber=1,frameValue=1, description='MAC')
    
    # Write nodal displacements.
    #
    uField = frame1.FieldOutput(name='MAC', description='Modal Assurance Criterion', type=SCALAR)
    nodeLabelData = [n+1 for n in range(matrixLen*matrixLen)]
    dispDataN=[]
    for i in range(matrixLen):
        for j in range(matrixLen):
            dispDataN.append((macMatrix[matrixLen-i-1][j],))
    uField.addData(position=NODAL, instance=instance1, labels=nodeLabelData, data=dispDataN)

    # Save and close odb
    #
    odb.save()
    odb.close()

    o1 = session.openOdb(name=macOdbFileName+'.odb')
    vp = session.viewports[session.currentViewportName]
    vp.setValues(displayedObject=o1)
    vp.odbDisplay.display.setValues(plotState=(CONTOURS_ON_DEF, ))
    vp.view.fitView()
    vp.view.setValues(session.views['Front'])
    vp.viewportAnnotationOptions.setValues(state=OFF, title=OFF, annotations=ON)

    # Create annotations for mode labels
    #
    c=0
    temp=matrixLen/10
    while temp:
        c+=1
        temp/=10

    modA = 'Model A'
    modB = 'Model B'

    # Swap names if odbs were swapped
    #
    if swap:
        modA,modB=modB,modA

    for k in sorted(o1.userData.annotations.keys()):
        del o1.userData.annotations[k]

    for l in range(matrixLen):
        ls = str(l+1)
        t = o1.userData.Text(name=modA + ' -'+ls, text=ls, offset=(-7-c*2, -5), anchor=(0, matrixLen-l-1, 0))
        vp.plotAnnotation(annotation=t)
        t = o1.userData.Text(name=modB + ' -'+ls, text=ls, offset=(-2-c, -6), anchor=(l, 0, 0))
        vp.plotAnnotation(annotation=t)

    t = o1.userData.Text(name=modA, text=modA, offset=(-24-c*2, -10), anchor=(0, matrixLen/2, 0))
    vp.plotAnnotation(annotation=t)
    t = o1.userData.Text(name=modB, text=modB, offset=( -12, -16), anchor=(matrixLen/2, 0, 0))
    vp.plotAnnotation(annotation=t)
