"""
Module to compute modal assurance component (MAC) and
generate a graphical representation of it.
"""
from abaqus import *
from abaqusConstants import *
from odbAccess import *
import math
import os, uti,types
import customKernel
from time import clock

from abq_Mac import macViewerResults
from abq_Mac import macComplexTools

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def openOdb(odbAPath, odbBPath):
    """
    Method to open *FREQUENCY ODBs and store step names in the session.customData object.
    Required arguments:
    odbPath: The path to the *FREQUENCY ODB
    
    Returns the odb object.
    """
    
    # Check the steps
    #
    session.customData.odbPaths = [odbAPath, odbBPath]
    session.customData.odbSteps = [[],[]]
    pathIndex = 0
    for odbPath in session.customData.odbPaths:
        if type(odbPath) != types.StringType:
            raise TypeError, 'odbPath must be a string; got %s.' % type(odbPath)

        if odbPath.strip() == '':
            raise ValueError, 'Path cannot be an empty string.'

        if session.isUpgradeRequiredForOdb(upgradeRequiredOdbPath=odbPath):
            raise ValueError, ('%s\nis from a previous '
                'version of Abaqus.\nPlease close the plug-in and upgrade '
                'database before continuing.' % odbPath)

        odb = session.openOdb(odbPath)

        # Do odb type and step checks
        #
        if type(odb) != OdbType:
            raise TypeError, 'odb must be of type OdbType; got %s.' % type(odb)
        for step in odb.steps.values():
            if (step.procedure == '*FREQUENCY' or step.procedure == '*COMPLEX FREQUENCY'):
                session.customData.odbSteps[pathIndex].append(step.name)

        # Make sure variables exist
        #
        for step in session.customData.odbSteps[pathIndex]:
            if not hasattr(odb.steps[step], "frames"):
                session.customData.odbSteps[pathIndex].remove(step)
            elif not hasattr(odb.steps[step].frames[-1], "fieldOutputs"):
                session.customData.odbSteps[pathIndex].remove(step)
            elif not odb.steps[step].frames[-1].fieldOutputs.has_key('U'):
                session.customData.odbSteps[pathIndex].remove(step)
            elif not hasattr(odb.steps[step].frames[-1].fieldOutputs['U'], "values"):
                session.customData.odbSteps[pathIndex].remove(step)

        if not session.customData.odbSteps[pathIndex]:
            raise ValueError, 'The ODB "%s" does not contain a *FREQUENCY step with field output "U" values.' % odbPath
            return None

        # Increase counter of pathIndex.  This avoids if same odb is used
        #
        pathIndex += 1

    return 1

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
def macPlot(dataAStep, dataBStep, plotType=CONTOURS_ON_UNDEF):

    # Open odbs
    #

    odbA = session.odbs[session.customData.odbPaths[0]]
    odbB = session.odbs[session.customData.odbPaths[1]]
    stepA = odbA.steps[dataAStep]
    stepB = odbB.steps[dataBStep]
    #
    # Extract list of nodes from stepA and stepB
    swap, takeList, valA, valB = macComplexTools.extractNodeList(stepA, stepB)
    
    # is swap, odb A is smaller than B
    #
    if swap:
        # valA and valB have already been swapped in extractNodeList !
        odbA,odbB = odbB,odbA
        dataAStep, dataBStep = dataBStep, dataAStep

    #
    #
    # Retrieve values and take only the necessary node labels
    # Note we filter only frames (modes) that contain 'Increment'
    # in the description.
    
    # Take number of modes from the odb that has the fewest
    # and take only those modes from each array
    arrayA, arrayB, isComplex = macComplexTools.accessNodeData(takeList,
                                                               stepA,
                                                               stepB)

    matrixLen = len(arrayA)
  
    #
    #
    if plotType==COMP_DEFAULT:
        imcVal = macComplexTools.calculateMacMatrix([arrayA[0]], [arrayB[0]], 1,
                                                    isComplex=isComplex)
        print 'The compatibility check of the first modes gives ', imcVal[0][0]
        return imcVal[0][0]

    macMatrix = macComplexTools.calculateMacMatrix(arrayA, arrayB, matrixLen,
                                                   isComplex=isComplex,
                                                   genTextOutput=True)

    #
    # Send diagonal to be plotted (bar graph)
    #
    if isComplex:
        fieldName = 'MAC Complex'
        fieldDescription = ('Modal Assurance Criterion based on '+
                            '*Frequency or *Complex Frequency steps.')
    else:
        fieldName = 'MAC Real'
        fieldDescription = ('Modal Assurance Criterion based on '+
                            '*Frequency steps.')
    macViewerResults.plotResults(matrixLen, macMatrix, dataAStep, dataBStep, swap,
                              name=fieldName,
                              description=fieldDescription,
                              plotType=plotType)

