from abaqusGui import *
import os

fileName = os.path.join(os.path.split(__file__)[0],'simple_muffler_diagram_1.png')
mufflerIcon = afxCreatePNGIcon(fileName)

###########################################################################
# Class definition
###########################################################################

class MufflerStlDB(AFXDataDialog):

    [ID_TIP_BTN,
     ID_LAST] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+2)
    
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        # Construct the base class.
        #
        self.form=form 
        self.tipDB = None
        AFXDataDialog.__init__(self, form, 'Plot STL ',
            self.APPLY|self.DISMISS, DECOR_RESIZE)
           

        self.okBtn = self.getActionButton(self.ID_CLICKED_APPLY)
        self.okBtn.setText('Plot')
        self.okBtn.disable()
            
        FXMAPFUNC(self, SEL_COMMAND, self.ID_TIP_BTN,MufflerStlDB.onCmdTip)
        from kernelAccess import session
        self.vp = session.viewports[session.currentViewportName]
        self.odb = self.vp.displayedObject
        
        GroupBox_1 = FXGroupBox(p=self, text='Inputs',
            opts=FRAME_GROOVE|LAYOUT_FILL_X)
        VFrame_1 = FXVerticalFrame(p=GroupBox_1, opts=LAYOUT_FILL_X,
            x=0, y=0, w=0, h=0, pl=0, pr=0, pt=0, pb=0)
        self.piCombo = AFXComboBox(p=VFrame_1, ncols=20, nvis=1,
            text='Pi', tgt=form.piKw, sel=0, opts=LAYOUT_FILL_X)
        self.piCombo.setMaxVisible(10)

        self.p1Combo = AFXComboBox(p=VFrame_1, ncols=20, nvis=1,
            text='P1', tgt=form.p1Kw, sel=0, opts=LAYOUT_FILL_X)
        self.p1Combo.setMaxVisible(10)

        self.p2Combo = AFXComboBox(p=VFrame_1, ncols=20, nvis=1,
            text='P2', tgt=form.p2Kw, sel=0, opts=LAYOUT_FILL_X)
        self.p2Combo.setMaxVisible(10)

        self.p3Combo = AFXComboBox(p=VFrame_1, ncols=20, nvis=1
            , text='P3', tgt=form.p3Kw, sel=0, opts=LAYOUT_FILL_X)
        self.p3Combo.setMaxVisible(10)
        hf = FXHorizontalFrame(p=VFrame_1, opts=LAYOUT_FILL_X,
            x=0, y=0, w=0, h=0, pl=0, pr=0, pt=0, pb=0)
        FXLabel(hf, 'Diagram:')
        FXButton(hf, 'Show', None, self, self.ID_TIP_BTN)
    
        GroupBox_2 = FXGroupBox(p=self, text='Additional Data',
            opts=FRAME_GROOVE|LAYOUT_FILL_X)
        VFrame_3 = FXVerticalFrame(p=GroupBox_2, opts=LAYOUT_FILL_X,
            x=0, y=0, w=0, h=0, pl=0, pr=0, pt=0, pb=0)
        
        AFXTextField(p=VFrame_3, ncols=12, labelText='Speed of sound',
            tgt=self.form.CKw, sel=0,
            opts=AFXTEXTFIELD_FLOAT|LAYOUT_FILL_X)
        AFXTextField(p=VFrame_3, ncols=12, labelText='Inlet tube area',
            tgt=self.form.inletAKw, sel=0,
            opts=AFXTEXTFIELD_FLOAT|LAYOUT_FILL_X)
        AFXTextField(p=VFrame_3, ncols=12, labelText='Outlet tube area',
            tgt=self.form.outletAKw, sel=0,
            opts=AFXTEXTFIELD_FLOAT|LAYOUT_FILL_X)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def populateCombos(self):
        """Method to populate the combo boxes"""
        
        setsToIgnore = [' ALL NODES']

        try:
            # Get assembly and instance sets
            #
            allSets = self.odb.rootAssembly.nodeSets.keys()

            for instance in self.odb.rootAssembly.instances.values():
                for nodesetName in instance.nodeSets.keys():
                    allSets.append('%s.%s' % (instance.name, nodesetName))
            allSets.sort()

            # Repopulate combos
            #
            self.piCombo.clearItems()
            self.p1Combo.clearItems()
            self.p2Combo.clearItems()
            self.p3Combo.clearItems()
            for setName in allSets:
                if setName not in setsToIgnore:
                    self.piCombo.appendItem(text=setName)
                    self.p1Combo.appendItem(text=setName)
                    self.p2Combo.appendItem(text=setName)
                    self.p3Combo.appendItem(text=setName)
            if self.form.piKw.getValue() not in allSets:
                self.form.piKw.setValue(None)
            if self.form.p1Kw.getValue() not in allSets:
                self.form.p1Kw.setValue(None)
            if self.form.p2Kw.getValue() not in allSets:
                self.form.p2Kw.setValue(None)
            if self.form.p3Kw.getValue() not in allSets:
                self.form.p3Kw.setValue(None)                    
        except:
            self.form.piKw.setValue(None)
            self.form.p1Kw.setValue(None)
            self.form.p2Kw.setValue(None)
            self.form.p3Kw.setValue(None)
            self.piCombo.clearItems()
            self.p1Combo.clearItems()
            self.p2Combo.clearItems()
            self.p3Combo.clearItems()
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdTip(self, sender, sel, ptr):
        if not self.tipDB:
            self.tipDB = TipDB(self)
            self.tipDB.create()

        self.tipDB.show()            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):

        # Register query on odbDisplay, populate combo boxes and call show()
        #
        registerCurrentContext(self.onUpdate)
        self.populateCombos()
        AFXDataDialog.show(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        # Unregister query on odbDisplay
        #
        unregisterCurrentContext(self.onUpdate)
        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onUpdate(self):

        self.piCombo.clearItems()
        self.p1Combo.clearItems()
        self.p2Combo.clearItems()
        self.p3Combo.clearItems()
        self.form.piKw.setValue(None)
        self.form.p1Kw.setValue(None)
        self.form.p2Kw.setValue(None)
        self.form.p3Kw.setValue(None)
        self.okBtn.disable()
        if hasattr(self.vp.odbDisplay, 'name'):
            self.populateCombos()
            self.okBtn.enable()


##########################################################################
# Class definition
###########################################################################

class TipDB(AFXDialog):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, dbOwner):


        # Construct the base class.
        #
        AFXDialog.__init__(self, 'Simple Muffler Tip',
            self.DISMISS, DIALOG_NORMAL|DIALOG_UNPOST_DESTROY|DECOR_RESIZE)


        # Set the muffler icon for DB
        #
        icon = mufflerIcon
        self.label = FXLabel(self,'',icon)
