from abaqusGui import *
from abaqusConstants import ALL
import osutils, os


###########################################################################
# Class definition
###########################################################################

class MufflerStlForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        
        # Construct the base class.
        #
        AFXForm.__init__(self, owner)

        self.cmd = AFXGuiCommand(mode=self, method='mufflerStlKernel',
            objectName='abq_MufflerStl.mufflerStlKernel', registerQuery=False)
        pickedDefault = ''
        self.piKw = AFXStringKeyword(self.cmd, 'inPressure', True, defaultValue=None)
        self.p1Kw = AFXStringKeyword(self.cmd, 'p1Pressure', True, defaultValue=None)
        self.p2Kw = AFXStringKeyword(self.cmd, 'p2Pressure', True, defaultValue=None)
        self.p3Kw = AFXStringKeyword(self.cmd, 'p3Pressure', True, defaultValue=None)
        
        self.CKw = AFXFloatKeyword(self.cmd, 'C', True, defaultValue=1)
        self.inletAKw = AFXFloatKeyword(self.cmd, 'inletArea', True, defaultValue=1)
        self.outletAKw = AFXFloatKeyword(self.cmd, 'outletArea', True, defaultValue=1)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        import mufflerStlDB
        return mufflerStlDB.MufflerStlDB(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        db = self.getCurrentDialog()
        if (self.piKw.getValue() == '' or
            self.p1Kw.getValue() == '' or
            self.p2Kw.getValue() == '' or
            self.p3Kw.getValue() == ''):
            showAFXErrorDialog(db,
                'Select node sets for Pi, P1, P2 and P3.')
            return False
        if (self.piKw.getValue()==self.p1Kw.getValue() or
            self.piKw.getValue()==self.p2Kw.getValue() or
            self.piKw.getValue()==self.p3Kw.getValue() or
            self.p1Kw.getValue()==self.p2Kw.getValue() or
            self.p2Kw.getValue()==self.p3Kw.getValue() or
            self.p1Kw.getValue()==self.p3Kw.getValue()):
            showAFXErrorDialog(db,
                'Select different node sets for Pi, P1, P2 and P3.')
            return False
        if self.CKw.getValue()==0:
            showAFXErrorDialog(db,
                'Sound of speed must be non-zero.')
            return False
        if self.inletAKw.getValue()<=0:
            showAFXErrorDialog(db,
                'Inlet tube area must be positive.')
            return False
        if self.outletAKw.getValue()<=0:
            showAFXErrorDialog(db,
                'Outlet tube area must be positive.')
            return False
        
        return True
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def okToCancel(self):

        return False


