from abaqus import *
from abaqusConstants import *
from odbAccess import *
import xyPlot
import math
import cmath
import visualization, uti

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Function plots 'data' obtained from function calcPressuresStl, 'data' is basically xy data pairs of 
#(freq,stl)
def plot(data,vp):
    if len(data) == 0:
        raise ValueError, 'No data could be calculated\nbetween given sets.'
    curveName = 'STL-%s-%s-%s' % ('p1', 'p2', 'p3')
    xy1 = session.XYData(name=curveName, data=data)
    try:
        xyp = session.xyPlots['XYPlot-1']
    except:
        xyp = session.XYPlot('XYPlot-1')

    chartName = xyp.charts.keys()[0]
    chart = xyp.charts[chartName]
    c1 = session.Curve(xyData=xy1)
    chart.setValues(curvesToPlot=(c1, ), )
    chart.axes1[0].axisData.setValues(useSystemTitle=False, title='Frequency',)
    chart.axes2[0].axisData.setValues(useSystemTitle=False, title='STL (dB)',)
    
    vp.setValues(displayedObject=xyp)


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Function uses, node labels of p1,p2 and p3 nodes, distances x1,x2 and inlet and outlet areas
#to compute pressures p1,p2,p3 from field output and and finally p-inlet and STL
def calcPressuresStl(p1Label,p2Label,p3Label,x1,x2,sIn,sOut,inPressure,odb,C):
    
    j=cmath.sqrt(-1)
    e=math.e      
    steps=odb.steps.keys()
    data=[]
    try:
        for step in steps:            
            frames=odb.steps[step].frames
            for i in range(len(frames)-1):    

                p1Dat=frames[i+1].fieldOutputs['POR'].values[int(p1Label)-1].data
                p1Conju=frames[i+1].fieldOutputs['POR'].values[int(p1Label)-1].conjugateData
                p1=(p1Dat+p1Conju*j)

                p2Dat=frames[i+1].fieldOutputs['POR'].values[int(p2Label)-1].data
                p2Conju=frames[i+1].fieldOutputs['POR'].values[int(p2Label)-1].conjugateData
                p2=(p2Dat+p2Conju*j)

                p3Dat=frames[i+1].fieldOutputs['POR'].values[int(p3Label)-1].data
                p3Conju=frames[i+1].fieldOutputs['POR'].values[int(p3Label)-1].conjugateData
                p3=(p3Dat+p3Conju*j)            

                freq=frames[i+1].frameValue

                w=2*math.pi*freq

                k=w/C

                prInCom=-( 1/(2*j*sin(k*(x2-x1))) * ( p1*(e ** (-j*k*x2)) - p2*(e ** (-j*k*x1)) ) )
                stl=20 * ( math.log10(abs(prInCom)/abs(p3)) ) + 10 * ( math.log10(sIn/sOut) )
                dat=[freq,stl]
                data.append(dat)
    except:
        raise ValueError,'An error has occurred.\n\nPlease review inputs. If issue persists, please'\
            ' contact technical support.'
    return data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Get node labels and coordinates for the nodes of the node sets
def getLabelCoords(ns,odb):
    
    if len(ns)==1:
        # must be an assembly set      
        PressureNset=odb.rootAssembly.nodeSets[ns[0]]
        NsetLabel=PressureNset.nodes[0][0].label
        NodeCoord=PressureNset.nodes[0][0].coordinates
    else:
        # must be an instance set
        PressureNset = odb.rootAssembly.instances[ns[0]].nodeSets[ns[1]]
        NsetLabel=PressureNset.nodes[0].label
        NodeCoord=PressureNset.nodes[0].coordinates        
    return NsetLabel,NodeCoord
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#The function gets arguments from the dialog box
def mufflerStlKernel(inPressure=None, p1Pressure=None, p2Pressure=None,
                     p3Pressure=None, C=None, inletArea=None, outletArea=None):
                
    vp=session.viewports[session.currentViewportName]
    odb=vp.displayedObject
    if type(odb) !=OdbType:
        raise ValueError, 'ODB must be displayed in active viewport.'

    curStepFrame=vp.odbDisplay.fieldFrame
    curStep= curStepFrame[0]
    curFrame= curStepFrame[1]


    #Inlet and outlet tube area should be in same units
    sIn=inletArea  #Inlet tube area
    sOut=outletArea #Outlet tube area
    
    inComCoord=[]
    
    p1ns = p1Pressure.split('.')    
    p1Label,p1Coord=getLabelCoords(p1ns,odb)
    
    p2ns = p2Pressure.split('.')
    p2Label,p2Coord=getLabelCoords(p2ns,odb)
    
    p3ns = p3Pressure.split('.')    
    p3Label,p3Coord=getLabelCoords(p3ns,odb)
    

    ns = inPressure.split('.')
    if len(ns) == 1:
        # must be an assembly set
        inPressure = odb.rootAssembly.nodeSets[ns[0]]
        nodesPinCom=inPressure.nodes
 
        for i in range(len(nodesPinCom[0])):
            coord=nodesPinCom[0][i].coordinates
            label=nodesPinCom[0][i].label
            inComCoord.append(coord)
        
        #Compute distance of p1 and p2 from nodeset 'Load'
        if inComCoord[0][0]==inComCoord[1][0]:
           distP1=abs(p1Coord[0]-inComCoord[0][0])
           distP2=abs(p2Coord[0]-inComCoord[0][0]) 
        elif inComCoord[0][1]==inComCoord[1][1]:
            distP1=abs(p1Coord[1]-inComCoord[0][1])
            distP2=abs(p2Coord[1]-inComCoord[0][1])        
    else: 
        # must be an instance set
        inPressure = odb.rootAssembly.instances[ns[0]].nodeSets[ns[1]]
        nodesPinCom=inPressure.nodes
        for i in range(len(nodesPinCom)):
            coord=nodesPinCom[i].coordinates
            label=nodesPinCom[i].label
            inComCoord.append(coord)

        #Compute distance of p1 and p2 from nodeset 'Load', 

        if inComCoord[0][0]==inComCoord[1][0]:
           distP1=abs(p1Coord[0]-inComCoord[0][0])
           distP2=abs(p2Coord[0]-inComCoord[0][0]) 
        elif inComCoord[0][1]==inComCoord[1][1]:
            distP1=abs(p1Coord[1]-inComCoord[0][1])
            distP2=abs(p2Coord[1]-inComCoord[0][1])  
    
    # x1 and x2 will be used in calcPressuresStl function
    x1=distP1
    x2=distP2

    data=calcPressuresStl(p1Label,p2Label,p3Label,x1,x2,sIn,sOut,inPressure,odb,C)
    plot(data,vp)
