from abaqusGui import *
from symbolicConstants import SymbolicConstant
from octaveBandConstants import *

###########################################################################
# Class definition
###########################################################################
class OctaveBandForm(AFXForm):
    def __init__(self, owner):
        AFXForm.__init__(self, owner)
        self.owner = owner

        self.cmd = AFXGuiCommand(mode=self, method='calculateOctaveBands',
                                 objectName='octaveBand')

        self.xyNameKw = AFXStringKeyword(command=self.cmd, name='xyName',
                                         isRequired=TRUE, defaultValue='')

        self.octaveTypeKw = AFXSymConstKeyword(command=self.cmd, name='octaveType',
                                               isRequired=TRUE,
                                               defaultValue=THIRDOCTAVE.getId())
        
        self.outputTypeKw = AFXSymConstKeyword(command=self.cmd, name='outputType',
                                               isRequired=TRUE,
                                               defaultValue=BOTH.getId())

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        return CreateSSDStepDb(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        if self.xyNameKw.getValue() == '':
            showAFXErrorDialog(self.getCurrentDialog(),'Please choose an STL curve to work from.')
            return FALSE
            
        return AFXForm.verifyKeywordValues(self)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def handleException(self, exc):
        """Overload the exception handling method"""

        errorType = exc[0]
        errorValue = exc[1]

        # Then post error dialog
        showAFXErrorDialog(self.getCurrentDialog(), str(errorValue))
        
###########################################################################
# Class definition
###########################################################################
class CreateSSDStepDb(AFXDataDialog):
    numInstance = 0	
    def __init__(self, form):
    	
    	CreateSSDStepDb.numInstance +=1
        self.form = form
        
        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form,
                               title='Create Octave Band XY Curves',
                               actionButtonIds=self.OK|self.APPLY|self.CANCEL,
                               opts=DIALOG_ACTIONS_SEPARATOR|DATADIALOG_BAILOUT)
        getAFXAliasMap().setPrefix(self, 'octave-%d' %CreateSSDStepDb.numInstance)
        # octave group box
        gb = FXGroupBox(p=self, text='STL Curve',
                        opts=FRAME_GROOVE|LAYOUT_FILL_X)

        self.stlCb = AFXComboBox(p=gb, ncols=40, nvis=5, text='STL curve:',
                                 tgt=self.form.xyNameKw, sel=0, opts=LAYOUT_FILL_X)
	getAFXAliasMap().setName(self.stlCb, 'combobox1')
        note= AFXNote(gb, 'STL data must be in dB units.')
        hf = FXHorizontalFrame(self, pl=0, pr=0, pt=0, pb=0, hs=0, vs=0,
                              opts=LAYOUT_FILL_X)
        
        
        # octave group box
        gb = FXGroupBox(p=hf, text='Octave',
                        opts=FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)

        vf = FXVerticalFrame(gb, pl=0, pr=0, pt=0, pb=0, hs=0, vs=0,
                             opts=LAYOUT_FILL_Y)
        
        self.rd1 = FXRadioButton(p=vf, text='One', tgt=self.form.octaveTypeKw,
                      sel=ONEOCTAVE.getId())
        getAFXAliasMap().setName(self.rd1, 'radio1btn')              
        
        self.rd2 = FXRadioButton(p=vf, text='One Third', tgt=self.form.octaveTypeKw,
                      sel=THIRDOCTAVE.getId())
        getAFXAliasMap().setName(self.rd2, 'radio2btn')
        
        # octave group box
        gb = FXGroupBox(p=hf, text='Output Type',
                        opts=FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        
        vf = FXVerticalFrame(gb, pl=0, pr=0, pt=0, pb=0, hs=0, vs=0,
                             opts=LAYOUT_FILL_Y)
        
        self.rd3 = FXRadioButton(p=vf, text='Line', tgt=self.form.outputTypeKw,
                      sel=LINE.getId())
	getAFXAliasMap().setName(self.rd3, 'radio3btn')
        self.rd4 = FXRadioButton(p=vf, text='Bar', tgt=self.form.outputTypeKw,
                      sel=BAR.getId())
        getAFXAliasMap().setName(self.rd4, 'radio4btn')
        self.rd5 = FXRadioButton(p=vf, text='Both', tgt=self.form.outputTypeKw,
                      sel=BOTH.getId())
        getAFXAliasMap().setName(self.rd5, 'radio5btn')
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def show(self):
        """Overload the show method"""
        #kernelInitString="import nvh_octaveBandKernel; reload(nvh_octaveBandKernel); octaveBand=nvh_octaveBandKernel.OctaveKernel(); from nvh_octaveBandConstants import *"
        #sendCommand(kernelInitString)
        
        from kernelAccess import session 
        session.xyDataObjects.registerQuery(self.onXYChanged)
        
        AFXDataDialog.show(self)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def hide(self):
        "Overload the hide method"""

        from kernelAccess import session 
        session.xyDataObjects.unregisterQuery(self.onXYChanged)
        
        AFXDataDialog.hide(self)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def onXYChanged(self):
        "Method to reload the xy combo box"""

        from kernelAccess import session 

        self.stlCb.clearItems()
        curveNames = session.xyDataObjects.keys()
        if self.form.xyNameKw.getValue() not in curveNames:
            self.form.xyNameKw.setValue('')
        for curveName in curveNames:
            self.stlCb.appendItem(text=curveName)

