from abaqusConstants import *
from orbitPlotConsts import *
from abaqusGui import *
from kernelAccess import mdb, session
import os

thisPath = os.path.abspath(__file__)
thisDir = os.path.dirname(thisPath)


###########################################################################
# Class definition
###########################################################################

class OrbitPlotDB(AFXDataDialog):
    [ID_STEPCOMBO,
     ID_FRAMECOMBO, 
     ID_NODESETCOMBO,
     ID_STARTSTEP,
     ID_ENDSTEP,
     ID_STARTFRAME,
     ID_ENDFRAME,
     ID_CLRANNTS
    ]= range(AFXDataDialog.ID_LAST,AFXDataDialog.ID_LAST+8)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        # Construct the base class.
        #
        self.form=form
        AFXDataDialog.__init__(self, form, 'Plot Orbits',0,opts=DIALOG_ACTIONS_SEPARATOR|DECOR_RESIZE|DIALOG_NORMAL)
            
            
        
        applyBtn = self.appendActionButton(text='Plot', tgt=self, sel=self.ID_CLICKED_APPLY)
        
        clearOrbitsBtn=self.appendActionButton(text='Clear', tgt=self, sel=self.ID_CLRANNTS)
        dismissBtn = self.appendActionButton(text='Dismiss', tgt=self, sel=self.ID_CLICKED_DISMISS)
        
        self.GroupBox_3 = FXGroupBox(p=self, text='Image Options', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        self.VFrame_1 = FXVerticalFrame(p=self.GroupBox_3, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        self.singleHFrame =FXHorizontalFrame(p=self.VFrame_1, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)     
        self.rb1=FXRadioButton(p=self.singleHFrame, text='Single image', tgt=self.form.numImageKw, sel=SINGLE_IMAGE.getId())
        self.HFrame_1 = FXHorizontalFrame(p=self.VFrame_1, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        self.stepCombo = AFXComboBox(p=self.HFrame_1, ncols=15, nvis=5, text='Step', tgt=self.form.stepKw, sel=0)    
        self.frameTF = AFXTextField(p=self.HFrame_1, ncols=5, labelText='Frame', tgt=self.form.frameTFKw, sel=0, opts=AFXTEXTFIELD_INTEGER)    
        
        self.HFrame_3 = FXHorizontalFrame(p=self.VFrame_1, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        self.rb2=FXRadioButton(p=self.HFrame_3, text='Multiple images', tgt=self.form.numImageKw, sel=MULTI_IMAGE.getId())
        self.HFrame_4 = FXHorizontalFrame(p=self.VFrame_1, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)        
        self.startStepCombo=AFXComboBox(p=self.HFrame_4, ncols=10, nvis=5, text='Start step', tgt=self.form.startStepKw, sel=0)
        self.endStepCombo=AFXComboBox(p=self.HFrame_4, ncols=10, nvis=5, text='End Step', tgt=self.form.endStepKw, sel=0)
        self.HFrame_5 = FXHorizontalFrame(p=self.VFrame_1, opts=10, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)       
        self.startFrameTF=AFXTextField(p=self.HFrame_5, ncols=5, labelText='Start frame', tgt=self.form.startFrameTFKw, sel=0, opts=AFXTEXTFIELD_INTEGER) 
        self.endFrameTF=AFXTextField(p=self.HFrame_5, ncols=5, labelText='End frame', tgt=self.form.endFrameTFKw, sel=0, opts=AFXTEXTFIELD_INTEGER)        

        GroupBox_2 = FXGroupBox(p=self, text='Orbit Data', opts=FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        self.HFrame_2 = FXHorizontalFrame(p=GroupBox_2, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        self.nodeSetCombo = AFXComboBox(p=self.HFrame_2, ncols=10, nvis=5, text='Node set', tgt=self.form.nodeSetKw, sel=0)
        self.dirnCombo=AFXComboBox(p=self.HFrame_2, ncols=5, nvis=4, text='Spin axis direction', tgt=self.form.dirnKw, sel=0)
        self.HFrame_7 = FXHorizontalFrame(p=GroupBox_2, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        
        self.resoChkBtn=FXCheckButton(p=self.HFrame_7, text='High resolution orbits', tgt=self.form.resoCheckKw, sel=0)
        self.resoChkBtn.setCheck(FALSE)
        self.enlargeTF = AFXTextField(p=self.HFrame_7, ncols=5, labelText='Orbit enlargment factor', tgt=self.form.enlargeTFKw, sel=0, opts=AFXTEXTFIELD_INTEGER)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CLRANNTS, OrbitPlotDB.clearOrbits)
        afxDisableWidgetTree(self.HFrame_4)
        afxDisableWidgetTree(self.HFrame_5)
        self.curOdbName=None
        self.cvp=None
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def processUpdates(self):
        if self.form.numImageKw.getValue() ==SINGLE_IMAGE.getId():
            afxDisableWidgetTree(self.startStepCombo)
            afxDisableWidgetTree(self.endStepCombo)
 
            afxDisableWidgetTree(self.startFrameTF)
            afxDisableWidgetTree(self.endFrameTF)
            afxEnableWidgetTree(self.stepCombo)
            afxEnableWidgetTree(self.frameTF)
       
        else:
            afxEnableWidgetTree(self.startStepCombo)
            afxEnableWidgetTree(self.endStepCombo)

            afxEnableWidgetTree(self.startFrameTF)
            afxEnableWidgetTree(self.endFrameTF)   
            afxDisableWidgetTree(self.stepCombo)
            afxDisableWidgetTree(self.frameTF) 
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def populateComboBoxes(self):
       from kernelAccess import session
       mainWindow = getAFXApp().getAFXMainWindow()
       curViewport = session.viewports[session.currentViewportName]       
       odbName=curViewport.odbDisplay.name
       self.cvp=session.currentViewportName
       odb=session.odbs[odbName]
       steps=odb.steps
       stepKeys=odb.steps.keys()

       dirnKeys=['X', 'Y', 'Z', '-X', '-Y', '-Z']
       self.dirnCombo.clearItems()
       for dirn in dirnKeys:
           self.dirnCombo.appendItem(dirn)
       self.form.dirnKw.setValue(self.dirnCombo.getText())         
       if self.curOdbName==odbName:
           pass
       else:
           self.curOdbName=odbName
           self.stepCombo.clearItems()
           self.startStepCombo.clearItems()
           self.endStepCombo.clearItems()
           self.nodeSetCombo.clearItems()

           for step in stepKeys:

               if steps[step].procedure=='*COMPLEX FREQUENCY' or steps[step].procedure == '*STEADY STATE DYNAMICS':
                   self.stepCombo.appendItem(step)
                   self.startStepCombo.appendItem(step)
                   self.endStepCombo.appendItem(step)
           self.form.stepKw.setValue(self.stepCombo.getText())
           self.form.startStepKw.setValue(self.startStepCombo.getText())
           self.form.endStepKw.setValue(self.endStepCombo.getText())
           nodeSetKeys=session.odbData[self.curOdbName].nodeSets.keys()
           for nset in nodeSetKeys:
               self.nodeSetCombo.appendItem(nset)

           self.form.nodeSetKw.setValue(self.nodeSetCombo.getText())    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
       
    def clearOrbits(self, sender, sel, ptr):
       from kernelAccess import session
       mainWindow = getAFXApp().getAFXMainWindow()
       curViewport = session.viewports[session.currentViewportName]     
       j = 0
       if curViewport.annotationsToPlot:
           for i in curViewport.annotationsToPlot[:]:
               if i.name.startswith('orbitPlot'):
                   del curViewport.annotationsToPlot[j]
               else:
                   j += 1
       else:
           showAFXErrorDialog(mainWindow, "No orbits are present in the viewport")
           

        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):
        #session.odbs.unregisterQuery(self.populateComboBoxes)
        unregisterCurrentContext(self.populateComboBoxes)
        AFXDataDialog.hide(self)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
        #session.odbs.registerQuery(self.populateComboBoxes)
        registerCurrentContext(self.populateComboBoxes)
        AFXDataDialog.show(self)
    