from abaqusGui import *
from abaqusConstants import *
from abq_OrbitPlot.orbitPlotConsts import *
###########################################################################
# Class definition
###########################################################################

class OrbitPlotForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        
        # Construct the base class.
        #
        AFXForm.__init__(self, owner)
        self.radioButtonGroups = {}

        self.cmd = AFXGuiCommand(mode=self, method='createOrbitPlot',
            objectName='abq_OrbitPlot.orbitPlotKernel', registerQuery=False)
        pickedDefault = ''
        self.stepKw = AFXStringKeyword(self.cmd, 'step', True)
        self.frameTFKw = AFXStringKeyword(self.cmd, 'frameTF', True, '1')
        self.nodeSetKw = AFXStringKeyword(self.cmd, 'nodeSet', True)
        self.startFrameTFKw = AFXStringKeyword(self.cmd, 'startFrameTF', True,'1')       
        self.endFrameTFKw = AFXStringKeyword(self.cmd, 'endFrameTF', True,'2')
        self.startStepKw = AFXStringKeyword(self.cmd, 'startStep', True)
        self.endStepKw = AFXStringKeyword(self.cmd, 'endStep', True)
        self.numImageKw =  AFXSymConstKeyword(self.cmd, 'numImage', True, SINGLE_IMAGE.getId())
        self.enlargeTFKw=AFXStringKeyword(self.cmd, 'enlargeOrbits', True, '1')
        self.resoCheckKw=AFXBoolKeyword(self.cmd, 'resoCheck', AFXBoolKeyword.TRUE_FALSE, True, False)
        self.dirnKw = AFXStringKeyword(self.cmd, 'spinDirn', True)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        import orbitPlotDB
        return orbitPlotDB.OrbitPlotDB(self)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def doCustomChecks(self):
        mainWindow = getAFXApp().getAFXMainWindow()
        from kernelAccess import session
        
        db=self.getCurrentDialog()
        
        if self.nodeSetKw.getValue()=='':
            showAFXErrorDialog(mainWindow, "Select the node set.")
        
        if self.numImageKw.getValue()==SINGLE_IMAGE.getId():
            self.startStepKw.setValue('')
            self.endStepKw.setValue('')
            self.startFrameTFKw.setValue('')
            self.endFrameTFKw.setValue('')
            if self.stepKw.getValue()=='':
                showAFXErrorDialog(mainWindow, "Select the step name.")
                return False
            if self.frameTFKw.getValue()=='':
                showAFXErrorDialog(mainWindow, "Frame number input is missing.")  
                return False
        else:
            self.stepKw.setValue('')
            self.frameTFKw.setValue('')
            if self.startStepKw.getValue()=='' and self.endStepKw.getValue()=='':
                showAFXErrorDialog(mainWindow, "Select the start and step names.")
                return False
            if self.startStepKw.getValue()=='':
                showAFXErrorDialog(mainWindow, "Select the start step name.")
                return False
            if self.endStepKw.getValue()=='':
                showAFXErrorDialog(mainWindow, "Select the end step name.")
                return False            
            if self.startFrameTFKw.getValue()=='' and self.endFrameTFKw.getValue()=='':
                showAFXErrorDialog(mainWindow, "Start and end frame number inputs are missing.")
                return False
            if self.startFrameTFKw.getValue()=='':
                showAFXErrorDialog(mainWindow, "Start frame number input is missing.")  
                return False
            if self.endFrameTFKw.getValue()=='':
                showAFXErrorDialog(mainWindow, "End frame number input is missing.")       
                return False                            
        if self.enlargeTFKw.getValue()=='':
            showAFXErrorDialog(mainWindow, "Orbit enlargement factor is missing")
            return False
        enlargeTF=int(self.enlargeTFKw.getValue())    
        if enlargeTF<1:
            showAFXErrorDialog(mainWindow, "Orbit enlargement factor cannot be less than 1.")
            return False            

        return True            
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def activate(self):
        mainWindow = getAFXApp().getAFXMainWindow()
        from kernelAccess import session
        vp=session.viewports[session.currentViewportName]
        
        if not getDisplayedObjectType() == ODB:
            showAFXErrorDialog(mainWindow,'There is no output database (ODB) \n'
                                          'associated with the current viewport.\n\n'
                                              'Select an ODB from the context bar,\n'
                                  'or use "File->Open" to open an ODB.')  
        else:
            #Check to see if *Complex Frequency is available in the ODB
            odbName=vp.odbDisplay.name
            steps=session.odbs[odbName].steps.keys()
            odb=session.odbs[odbName]
            flag=0
            for step in steps:
                if odb.steps[step].procedure == '*COMPLEX FREQUENCY' or odb.steps[step].procedure == '*STEADY STATE DYNAMICS':
                    flag=1
                    break
            if flag==0:
                msg="The ODB must have either the Complex Frequency extraction step\n"\
                    "or the Steady State Dynamics step."
                showAFXErrorDialog(mainWindow,msg)
            else:
                AFXForm.activate(self)                

