from abaqus import *
from abaqusConstants import *
from math import *
from abq_OrbitPlot.orbitPlotConsts import *
import os
from numpy import *


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createOrbitPlot(step=None, frameTF=None, nodeSetCheck=None, nodeSet=None, startFrameTF=None, \
                    endFrameTF=None, startStep=None, endStep=None, numImage=None, enlargeOrbits=None, resoCheck=None, spinDirn=None):
                       
    cvpName=session.currentViewportName                  
    curViewport = session.viewports[cvpName]
    try:
        odbName=curViewport.odbDisplay.name
    except:
        msg="ODB must be displayed in the current viewport."
        raise ValueError, msg
    odb=session.odbs[odbName] 
    
    if frameTF=='' or frameTF==None:
        frameTF=None
    else:
        frameTF=int(frameTF)
        if frameTF > (len(odb.steps[step].frames)-1) or frameTF<1:
            msg="Enter the correct non-zero frame number."
            raise ValueError, msg                
    if startFrameTF=='' or startFrameTF==None:
        startFrameTF=None
    else:
        startFrameTF=int(startFrameTF)
        if startFrameTF > (len(odb.steps[startStep].frames)-1) or startFrameTF<1:
            msg="Enter the correct non-zero frame number."
            raise ValueError, msg                
        
    if endFrameTF=='' or endFrameTF==None:
        endFrameTF=None
    else:
        endFrameTF=int(endFrameTF)    
        if endFrameTF > (len(odb.steps[endStep].frames)-1) or endFrameTF<1:
            msg="Enter the correct non-zero frame number."
            raise ValueError, msg                        
        
    createAnnotations(numImage, frameTF, step, startStep, endStep, startFrameTF, endFrameTF, nodeSet, odb, enlargeOrbits, resoCheck, cvpName, spinDirn)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   
def getSingleImageResultData(odb, step, frame, nodeSet, startFrameTF, endFrameTF, startStep, endStep, numImage):
    reqdFrame=odb.steps[step].frames[int(frame)]
    frameDict=getXyHistoryData(odb, frmae, step)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getMultiImageResultData(odb, nodeSet, startFrameTF, endFrameTF, startStep, endStep, numImage):
    steps = odb.steps.keys() 
    odbSteps=odb.steps
    startStepNum=odbSteps[startStep].number
    endStepNum=odbSteps[endStep].number
    startStepProc=odbSteps[startStep].procedure
    
    if startStepProc == "*COMPLEX FREQUENCY":
        totalStepNums=range(startStepNum,endStepNum+3, 3)
        stpframes = getReqdStepKeys(odb, steps, totalStepNums, startStep, endStep, startFrameTF, endFrameTF) 
    elif startStepProc == "*STEADY STATE DYNAMICS":        
        totalStepNums=range(startStepNum, endStepNum+2, 2)        
        stpframes =getReqdStepKeys(odb, steps, totalStepNums, startStep, endStep, startFrameTF, endFrameTF)
    else:
        print "Specified steps are must be Complex Frequency Extraction or Steady State Dynamics"
    
    return startStepProc, stpframes #assuming start and end step procedure are same
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
def getReqdStepKeys(odb, steps, totalStepNums, startStep, endStep, startFrameTF, endFrameTF):
    stpframes={}
    reqdStepKeys=[]
    odbSteps=odb.steps
    if startStep!=endStep:
        for step in steps:
            if odbSteps[step].number in totalStepNums:
                reqdStepKeys.append(step)
                stpframes[step]=range(len(odbSteps[step].frames))   
                
                if step==startStep:
                    strtInx=stpframes[step].index(int(startFrameTF))
                    stpframes[step]=stpframes[step][strtInx:]
                    
                elif step==endStep:
                    endInx=stpframes[step].index(int(endFrameTF))
                    stpframes[step]=stpframes[step][:endInx+1]
               
    else:
        frames = odb.steps[startStep].frames
        lenFrames=len(frames)
        fstInx = startFrameTF
        sndInx = endFrameTF+1
        stpframes[startStep]=range(len(frames))[fstInx:sndInx]
                
    return stpframes
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
def getXyHistoryData(odb, step, frame):
    
    eigFreqData=list(session.XYDataFromHistory(name='eigFreqTemp', odb=odb, outputVariableName='Eigenfrequency: EIGFREQ for Whole Model', steps=(step,),) )
    eigFreqData.insert(0,(0,0))        
    eDampRatioData= list(session.XYDataFromHistory(name='eDampTemp', odb=odb, outputVariableName='Effective damping ratio: DAMPRATIO for Whole Model', steps=(step,),) )
    eDampRatioData.insert(0,(0,0))
    eigMagData= list(session.XYDataFromHistory(name='eigMagTemp', odb=odb, outputVariableName='Imaginary part of eigenvalue: EIGIMAG for Whole Model', steps=(step,),) )
    eigMagData.insert(0,(0,0))
    eigRealData= list(session.XYDataFromHistory(name='eigRealTemp', odb=odb, outputVariableName='Real part of eigenvalue: EIGREAL for Whole Model', steps=(step,),) )
    eigRealData.insert(0,(0,0))
    frameDict={}

    for i in xrange(len(eigFreqData)):
        frameDict[i]={'eigFreq':eigFreqData[i][1], 'logDecrement':pi*eDampRatioData[i][1], 'eigMagData':eigMagData[i][1], 'eigRealData':eigRealData[i][1]}
    
    return frameDict       
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def prepForWhirlDir(nodeSet, odb, frame, step, spinDirn):
    """
    nodes_obj=getNodes(nodeSet, odb)
    x_avg = y_avg = z_avg = 0.
    nodeLen=len(nodes_obj)
    for i in xrange(nodeLen):
        tmp_node_odj = nodes_obj[i]
        x = tmp_node_odj.coordinates[0]
        y = tmp_node_odj.coordinates[1]
        z = tmp_node_odj.coordinates[2]
        if i == 0:
            x_i = x
            y_i = y
            z_i = z
        x_avg = x_avg + x_i - x
        y_avg = y_avg + y_i - y
        z_avg = z_avg + z_i - z
    if y_avg == 0 and z_avg == 0:
        spin_axis = 'X'
        ID1 = 1; ID2 = 2
    elif x_avg == 0 and z_avg == 0:
        spin_axis = 'Y'
        ID1 = 2; ID2 = 0
    elif x_avg == 0 and y_avg == 0:
        spin_axis = 'Z'
        ID1 = 0; ID2 = 1
    else:
        raise ValueError, "The spin axis is not Global."
    """
    spin_axis=spinDirn
    if spin_axis=='X' or spin_axis=='-X':
        ID1 = 1; ID2 = 2
    elif spin_axis=='Y' or spin_axis=='-Y':
        ID1 = 2; ID2 = 0
    elif spin_axis=='Z' or spin_axis=='-Z':
        ID1 = 0; ID2 = 1
    return spin_axis, ID1, ID2
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
def getWhirlDir(odb, ID1, ID2, step, frame, startStepProc, spin_axis):
    ref_real = 2
    pos_real = 1
    ref_whirl =  1E-16
    
    tmp_freq_frames=odb.steps[step].frames[frame]
    temp2 = []
    tmp_whirl = 0.
    tmp_whirl_1 = 0.
    outputVals = tmp_freq_frames.fieldOutputs['U'].values
    outputValsLen=len(outputVals)
    for iii in xrange(outputValsLen):
        outputVal=outputVals[iii]
        n = outputVal.nodeLabel
        a = outputVal.data
        b = outputVal.conjugateData
        Uc = float(a[ID1]); Us = -float(b[ID1])
        Vc = float(a[ID2]); Vs = -float(b[ID2])
        if Uc+Us+Vc+Vs != 0.:
            W_f = 0.5*sqrt((Uc+Vs)**2 + (-Us+Vc)**2)
            W_b = 0.5*sqrt((Uc-Vs)**2 + (Us+Vc)**2)

            semimajor = W_f + W_b
            semiminor = W_f - W_b

            tmp_whirl +=semiminor
            if semiminor == 0.0:
                tmp_whirl_1 = tmp_whirl_1 + 1.
            temp2.append((n, semimajor, semiminor))

    if(tmp_whirl > 0. and abs(tmp_whirl) > ref_whirl):
        if spin_axis.startswith('-'):
            wDir="BACKWARD"
        else:    
            print 'in else'
            wDir="FORWARD"
    elif (tmp_whirl < 0. and abs(tmp_whirl) > ref_whirl):
        if spin_axis.startswith('-'):
            wDir="FORWARD"
        else:
            print 'in else'
            wDir="BACKWARD"
    else:
        if tmp_whirl_1 == float(len(temp2)):
            wDir="No Whirl"
        else:           
            wDir="MIXED"
    return wDir
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createAnnotations(numImage, singleFrameNum, singleStepName, startStep, endStep, startFrameNum, endFrameNum, nodeSet, odb, enlargeOrbits, resoCheck, cvpName, spinDirn):

    nodes=getNodes(nodeSet, odb)
    viewportName='orbit_plot_viewport'
    
    degreeFact=pi/180
    angDict={}
    if resoCheck:
        max=380
        div=20
    else:
        max=390
        div=30
    for phaseAngle in xrange(0,max,div):
        angDict[str(phaseAngle)]={'cosData':cos(phaseAngle*degreeFact), 'sinData':sin(phaseAngle*degreeFact)}    

    if numImage==SINGLE_IMAGE:
        step=odb.steps[singleStepName]
        frame=step.frames[singleFrameNum]
        disp=frame.fieldOutputs['U']
        startStepProc=step.procedure
        j=1
        plotText(startStepProc, odb, viewportName, singleStepName, singleFrameNum, frame, nodeSet, disp, j, nodes, enlargeOrbits, resoCheck, angDict, max, div, cvpName, spinDirn)
    
    elif numImage==MULTI_IMAGE: 
        
        startStepProc, stpframes = getMultiImageResultData(odb, nodeSet,  startFrameNum, endFrameNum, startStep, endStep, numImage)
        steps=odb.steps
        stpKeys=stpframes.keys()    
        
        j=1
        for sKey in stpKeys:
            step=steps[sKey]
            for frameNum in stpframes[sKey]:                
                frame=steps[sKey].frames[frameNum]
                disp=frame.fieldOutputs['U']
                plotText(startStepProc, odb, viewportName, sKey, frameNum, frame, nodeSet, disp, j, nodes, enlargeOrbits, resoCheck, angDict, max, div, cvpName, spinDirn)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getNodes(nodeSet, odb):
    assembly=odb.rootAssembly
    if len(nodeSet.split('.'))==1:
        try:
            nodes_obj=assembly.nodeSets[nodeSet].nodes[0]
        except:
            msg="Error when accessing nodes of the selected node set."
            raise ValueError, msg
        
    else:
        instanceName = nodeSet.split('.')[0]
        nodeSetName = nodeSet.split('.')[1]
        try:
            nodes_obj=assembly.instances[instanceName].nodeSets[nodeSetName].nodes
        except:
            msg="Error when accessing nodes of the selected node set."
            raise ValueError, msg                    
    if not nodes_obj:
        msg="Empty node set cannot be used."
        raise ValueError, msg
    return nodes_obj
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def plotText(startStepProc, odb, viewportName, sKey, frameNum, frame, nodeSet, disp, j, nodes, enlargeOrbits, resoCheck, angDict, max, div, cvpName, spinDirn):
    pngName='Step_'+sKey+'_Frame_'+str(frameNum)+'.png'
    vp=session.viewports[cvpName]
    spinAxis, ID1,ID2=prepForWhirlDir(nodeSet, odb, frameNum, sKey, spinDirn)
    whirlDirData= getWhirlDir(odb, ID1, ID2, sKey, frameNum, startStepProc, spinAxis)
    if startStepProc == "*COMPLEX FREQUENCY":
        frameDict=getXyHistoryData(odb, sKey, frameNum)
        eigMagData=frameDict[frameNum]['eigMagData']            
        t1="%0.3f" %(eigMagData/(2*pi))+"Hz"
        eigRealData='%0.3f' %(frameDict[frameNum]['eigRealData'])
        logDecrement="%0.3f" %frameDict[frameNum]['logDecrement']
        text1="Step = "+sKey+"\n"\
      "Mode = "+str(frameNum)+"\n"\
      "Log decrement = "+logDecrement+"\n"\
      "Whirl direction ="+str(whirlDirData)+"\n"\
      "Complex eigenvalue(Real) = "+eigRealData+"\n"\
      "Complex eigenvalue(Imag) = "+t1
    elif startStepProc == "*STEADY STATE DYNAMICS":

        text1="Step = "+sKey+"\n"\
        "Frame = "+str(frameNum)+"\n"\
        "Frequency ="+str(frame.frequency)+"\n"\
        "Whirl direction = "+ str(whirlDirData)
    else:
        print "The analyis procedure is not supported for orbit plots."
    
    j1 = 0
    curViewport=session.viewports[cvpName]
    if curViewport.annotationsToPlot:
        for i in curViewport.annotationsToPlot[:]:
            if i.name.startswith('orbitPlot'):
                del curViewport.annotationsToPlot[j1]
            else:
                j1 += 1
                                                                           
    text=odb.userData.Text(name='orbitPlot'+'_text', offset=(5, -5), text=text1, anchor=TOP_LEFT, font='-*-helvetica-medium-r-normal--16-*', referencePoint=TOP_LEFT,rotationAngle=0.0,color='Red',backgroundStyle=TRANSPARENT,backgroundColor='Black',box=OFF,justification=LEFT)
    vp.plotAnnotation(text) 
    
    plotAnnotations(nodes, odb, viewportName, pngName, j, disp, spinAxis, enlargeOrbits, resoCheck, angDict, max, div, cvpName)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def plotAnnotations(nodes, odb, viewportName, pngName, j, disp, spinAxis, enlargeOrbits, resoCheck, angDict, max, div, cvpName):


    
    factorDict={'X':[1,int(enlargeOrbits),int(enlargeOrbits)],\
                'Y':[int(enlargeOrbits), 1, int(enlargeOrbits)],\
                'Z':[int(enlargeOrbits), int(enlargeOrbits),1],\
                '-X':[1,int(enlargeOrbits),int(enlargeOrbits)],\
                '-Y':[int(enlargeOrbits), 1, int(enlargeOrbits)],\
                '-Z':[int(enlargeOrbits), int(enlargeOrbits),1]}
    
    nodeLen=len(nodes)
    
    userData=odb.userData
    
    orbitVp=session.viewports[cvpName]
    spinAxisDict=factorDict[spinAxis]
  
    labelValDict={}

    labelValDict=dict([(val.nodeLabel,val) for val in disp.values])
    zArray=zeros((nodeLen, 3), 'float64')
    for k in xrange(nodeLen):     
        node=nodes[k]
        coords=node.coordinates
        label=node.label
        u1Orig=coords[0]
        u2Orig=coords[1]
        u3Orig=coords[2]
        
        val=labelValDict[label]
        valData=val.data
        u1Real=valData[0]
        u2Real=valData[1]
        u3Real=valData[2]
        valCongugateData=val.conjugateData
        u1Imag=valCongugateData[0]
        u2Imag=valCongugateData[1]
        u3Imag=valCongugateData[2] 

        for phaseAngle in xrange(0,max,div):  
            u1Defom=spinAxisDict[0]*(u1Orig+(u1Real * angDict[str(phaseAngle)]['cosData']) - (u1Imag * angDict[str(phaseAngle)]['sinData']))
            u2Defom=spinAxisDict[1]*(u2Orig+(u2Real * angDict[str(phaseAngle)]['cosData']) - (u2Imag * angDict[str(phaseAngle)]['sinData']))
            u3Defom=spinAxisDict[2]*(u3Orig+(u3Real * angDict[str(phaseAngle)]['cosData']) - (u3Imag * angDict[str(phaseAngle)]['sinData']))
            
            if phaseAngle == 0:
                u11Orig=u1Orig
                u22Orig=u2Orig
                u33Orig=u3Orig
                
                zArray[k][0]=u1Defom
                zArray[k][1]=u2Defom
                zArray[k][2]=u3Defom
            
            arrow=userData.Arrow(color='Blue', name='orbitPlot'+str(j), startAnchor=(u11Orig,u22Orig,u33Orig), \
                        endAnchor=(u1Defom,u2Defom, u3Defom), endHeadStyle=NONE, lineStyle=SOLID)                
            
            orbitVp.plotAnnotation(arrow)
            u11Orig=u1Defom
            u22Orig=u2Defom
            u33Orig=u3Defom            
            j=j+1


    k1=1

    annotLen=len(zArray)
    sortDict={'X':0, 'Y':1, 'Z':2, '-X':0, '-Y':1, '-Z':2}
    zArray=sorted(zArray, key=lambda x:x[sortDict[spinAxis]])

    if annotLen >1:
        for z in xrange(len(zArray)-1):
            arrow1=userData.Arrow(color='Red', name='orbitPlot'+'_zPhase_'+str(k1), startAnchor=zArray[z], \
                                            endAnchor=(zArray[z+1]), endHeadStyle=NONE, lineStyle=SOLID)             
            orbitVp.plotAnnotation(arrow1) 
            k1=k1+1

    orbitVp.maximize()
    orbitVp.viewportAnnotationOptions.setValues(legend=OFF, title=OFF, state=OFF, compass=OFF)    
    orbitVp.setValues(displayedObject=odb)
    
    if spinAxis=='X' or spinAxis=='-X':
        session.View(name='User-1', nearPlane=288.59, farPlane=390.57, width=7.5592, height=2.835, projection=PERSPECTIVE, cameraPosition=(-290.94, 5.7871, 6.6778), cameraUpVector=(0.3426, 0.86842, -0.35843), cameraTarget=(48.501,-0.79369, -0.62705), viewOffsetX=1.2164, viewOffsetY=0.033981, autoFit=OFF)
    elif spinAxis=='Z' or spinAxis=='-Z':        
        session.View(name='User-1', nearPlane=288.59, farPlane=390.57, width=7.5592, height=2.835, projection=PERSPECTIVE, cameraPosition=(6.6778,5.7871,-290.94), cameraUpVector=(-0.35843, 0.86842, 0.3426), cameraTarget=(-0.62705, -0.79369, 48.501),  viewOffsetX=-0.47534, viewOffsetY=0.60938, autoFit=OFF)
    elif spinAxis=='Y' or spinAxis=='-Y':
        session.View(name='User-1', nearPlane=288.59, farPlane=390.57, width=7.5592, height=2.835, projection=PERSPECTIVE, cameraPosition=(6.6778,-290.94, 5.7871), cameraUpVector=(-0.35843, 0.3426, 0.86842), cameraTarget=(-0.62705, 48.501, -0.79369), viewOffsetX=1.2164, viewOffsetY=0.033981, autoFit=OFF)
    
    orbitVp.view.setValues(session.views['User-1'])
    session.printOptions.setValues(vpDecorations=OFF)
    curDir=os.getcwd()
    dirPath=os.path.join(curDir,"orbitPlotImages")

    try:
        os.makedirs(dirPath)
    except:
        pass
    fPath=os.path.join(dirPath,pngName)

    try:
        dirPath=os.path.join(curDir,"orbitPlotImages")
        
        try:
            os.makedirs(dirPath)
        except:
            pass
        fPath=os.path.join(dirPath,pngName)
        
        if os.path.exists(fPath):

            os.remove(fPath)            
        session.printToFile(fileName=fPath, format=PNG, canvasObjects=(orbitVp, ))  
    except:
        msg='Error when saving the orbit plot images.'
        raise ValueError, msg
