from abaqusGui import *
import os,copy,re


###########################################################################
# Class definition
###########################################################################

class PlotMultiPathsDB(AFXDataDialog):
    [  ID_DEL,
       ID_FILTER,
       ID_TIP,
       ID_PL
    ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+4)
    AFXDataDialog.ID_LAST = AFXDataDialog.ID_LAST+4

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        # Construct the base class.
        #

        AFXDataDialog.__init__(self, form, 'Plot Paths', 0,
           DECOR_RESIZE|DIALOG_ACTIONS_SEPARATOR)

        applyBtn = self.appendActionButton('Plot', tgt=self, sel=self.ID_PL)
        deleteBtn = self.appendActionButton('Delete', tgt=self, sel=self.ID_DEL)
        cancelBtn = self.appendActionButton('Cancel', tgt=self, sel=self.ID_CLICKED_CANCEL)
            
        getAFXAliasMap().setPrefix(self, 'plotMultiPaths')
        self.form = form
        self.filterStrTgt = AFXStringTarget()
        self.oldFilterString = ''
        self.allPathKeys = []

        #
        # define map function for command selection 
        FXMAPFUNC(self, SEL_COMMAND, PlotMultiPathsDB.ID_DEL, 
            PlotMultiPathsDB.onCmdDelete)
        FXMAPFUNC(self, SEL_COMMAND, PlotMultiPathsDB.ID_FILTER, 
            PlotMultiPathsDB.onCmdFilter)
        FXMAPFUNC(self, SEL_COMMAND, PlotMultiPathsDB.ID_PL, 
            PlotMultiPathsDB.onCmdPlot)
        FXMAPFUNC(self, SEL_COMMAND, PlotMultiPathsDB.ID_TIP,
            PlotMultiPathsDB.onTip)
            
        vf = FXVerticalFrame(p=self, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        gb = FXGroupBox(p=vf, text='Name',
                                opts=FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)

        # List widget
        #
        self.myListBox = AFXList(gb, 15,
                                 tgt=None, sel=0,
                                 opts=LIST_EXTENDEDSELECT|HSCROLLING_OFF|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        getAFXAliasMap().setName(self.myListBox, 'myListBox')

        # Check boxes
        #
        gb = FXGroupBox(p=vf, text='Options',
                                opts=FRAME_GROOVE|LAYOUT_FILL_X)
        w=FXCheckButton(gb, 'Plot names', tgt=self.form.addNameToPathKw, sel=0)
        getAFXAliasMap().setName(w, 'addName')
        w=FXCheckButton(gb, 'Plot end marker', tgt=self.form.useEndMarkerKw, sel=0)
        getAFXAliasMap().setName(w, 'useEnd')
        w=FXCheckButton(gb, 'Use unique colors', tgt=self.form.useColorKw, sel=0)
        getAFXAliasMap().setName(w, 'useDiffer')
        va = AFXVerticalAligner(gb, opts=LAYOUT_FILL_X)
        w=AFXTextField(va, 4, 'Font size', tgt=self.form.fontSizeKw, opts=AFXTEXTFIELD_INTEGER)
        getAFXAliasMap().setName(w, 'fontSize')
        hf = FXHorizontalFrame(va, opts=FRAME_NONE|LAYOUT_FILL_X, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        w=FXButton(hf, 'Filter:\tActivate Filter function.\n'+
                     'If field is empty, or equal *, all paths are displayed',
                 tgt=self, sel=PlotMultiPathsDB.ID_FILTER)
        getAFXAliasMap().setName(w, 'activateFilter')
        w=AFXTextField(hf, 20, '\tIf field is empty, or equal *, all paths are displayed',
                     tgt=self.filterStrTgt, sel=0, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y)
        getAFXAliasMap().setName(w, 'emptyField')
        self.tipBtn = FXButton(hf, "Tip...", tgt=self, sel=self.ID_TIP, opts=LAYOUT_RIGHT)
        getAFXAliasMap().setName(self.tipBtn, 'tip')
        
        #self.pathDict = pathDict

        # This function checks if the string matches the filter pattern
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def checkFilterPattern(self, pattern, string):
        "This function checks if the string matches the filter pattern"
        try:
            match = re.search(pattern, string)
        except:
            match = None
        if match:
            return True
        else:

            return False
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def _defineListContent(self, newFilterString):

        try:
            self.filterPattern = re.compile(self.filterStrTgt.getValue())
        except:
            self.filterPattern = ""        
        
        newContent = [key for key in self.allPathKeys
                          if self.checkFilterPattern(self.filterPattern, key)]
        if not newContent:
            newContent = 'Empty'
        self.changeListContent(newContent)
                                     
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def changeListContent(self, listContent=None):
        """Displays list content based on filter function.
        """

        # In case session.paths has been changed, update
        # allPathKeys
        #
        from kernelAccess import session
        self.allPathKeys = sorted(session.paths.keys())        
        keys = self.allPathKeys

        # Display none if 'Empty' from regex
        # 
        if listContent == 'Empty':
            keys = []
        elif listContent:
            keys = listContent
            
        # Append items
        #
        self.myListBox.clearItems()
        for ip, name in enumerate(keys):
            self.myListBox.appendItem(name)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdFilter(self, sender, sel, ptr):
        """
        activate command for Filter path
        """                 
        newFilterString = self.filterStrTgt.getValue()
        if newFilterString != self.oldFilterString:
            self.oldFilterString = newFilterString
            self._defineListContent(newFilterString)
        

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdDelete(self, sender, sel, ptr):
        """
        activate command for delete annotations . 
        """                 
        self.form.deleteCmd.activate()
        self.form.plotCmd.deactivate()
        self.handle(self, MKUINT(self.ID_CLICKED_APPLY, SEL_COMMAND), None)
        self.form.deleteCmd.deactivate()          
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdPlot(self, sender, sel, ptr):
        """
        activate command for delete annotations . 
        """
        #
        pathDict = {}
        myList = self.myListBox
        addToPlot = False
        for i in range(myList.getNumItems()):
            name = myList.getItemText(i)
            isSelected = myList.isItemSelected(i)
            if isSelected:
                addToPlot = True
            pathDict['path_%03i'%(i+1)] = (isSelected, name)
        #
        if not(addToPlot):
            showAFXInformationDialog(self, 'Please select paths.')
        #
        else:
            self.form.path1Kw.setValue(str(pathDict))
            self.form.plotCmd.activate()
            self.form.deleteCmd.deactivate()
            self.handle(self, MKUINT(self.ID_CLICKED_APPLY, SEL_COMMAND), None)
            self.form.plotCmd.deactivate()          

    # This function is called when user preses the tip button
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onTip(self, sender, sel, ptr):
        "This function is called when user preses the tip button"
        messageTuple = (('Apple','Matches the string Apple'),
                       ('.', 'Matches any character except a newline'),
                       ('^', 'Matches the start of the string'),
                       ('$', 'Matches the end of the string'),
                       ('*', '"Any number of occurrences of what just preceded me"'),
                       ('+', '"One or more occurrences of what just preceded me"'),
                       ('|', '"Either the thing before me or the thing after me"'),
                       ('\w', 'Matches any alphanumeric character'),
                       ('\d', 'Matches any decimal digit')
                       )
        message = "Specify regular expression pattern as the filter\n"
        for m in messageTuple:
            message = message + "\n" + m[0].ljust(10) + "\t" + m[1]
        showAFXInformationDialog(self, message)            
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):

        AFXDataDialog.show(self)

        # Register a query on parts
        #
        from kernelAccess import session
        session.paths.registerQuery(self.changeListContent)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        from kernelAccess import session        
        session.paths.unregisterQuery(self.changeListContent)
        AFXDataDialog.hide(self)