from abaqus import *
from abaqusConstants import *
import visualization

replaceList = [()]
#
_USE_COLOR = False
_USE_END_MARKER = False
_ADD_NAME_TO_PATH = False
_FONT_SIZE=12

_fontBasis = '-*-arial-medium-r-normal-*-*-%s0-*-*-p-*-*-*'

allColors = [ 
 '#458B74',
 '#F5F5DC',
 '#BB3A3A',
 '#005E9D',
 '#C1CDCD',
 '#DEB887',
 '#B1C6ED',
 '#8B8878',
 '#E9967A',
 '#BDB76B',
 '#8B0A50',
 '#313A97',
 '#CAD9BB',
 '#EEC900',
 '#ADD8E6',
 '#6E7B8B',
 '#8B4789',
 '#EEE8AA',
 '#B8CEC6',
 '#B8B8DB',
 '#CEA46B',
 '#6A5ACD',
 '#EEE9E9',
 '#003366',
 '#458B74',
 '#F5F5DC',
 '#BB3A3A',
 '#005E9D',
 '#C1CDCD',
 '#DEB887',
 '#B1C6ED',
 '#8B8878',
 '#E9967A',
 '#BDB76B',
 '#8B0A50',
 '#313A97',
 '#CAD9BB',
 '#EEC900',
 '#ADD8E6',
 '#6E7B8B',
 '#8B4789',
 '#EEE8AA',
 '#B8CEC6',
 '#B8B8DB',
 '#CEA46B',
 '#6A5ACD',
 '#EEE9E9',
 '#003366',
 '#458B74',
]


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def deletePlotPaths():
    #&& remove all plotted paths                        
    #
    i = 0
    pathList = session.paths.keys()
    vp = session.viewports[session.currentViewportName]
    o1 = vp.displayedObject
    if not(o1) or type(o1) != visualization.OdbType:
        return
    #
    for key in o1.userData.annotations.keys():
        for path in pathList:
            if key.find(path) > -1:
                del o1.userData.annotations[key] 
                                               

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def plotPaths(path1, useColor=_USE_COLOR, useEndMarker=_USE_END_MARKER,
              fontSize=_FONT_SIZE,
              addNameToPath=_ADD_NAME_TO_PATH):
    #
    useFont = _fontBasis%(fontSize)
    #
    pathDir = eval(path1)
    pathList = []
    for key in pathDir.keys():
        if pathDir[key][0]:  
            pathList += [pathDir[key][1],]
    vp = session.viewports[session.currentViewportName]
    nodeDir = {}   
    o1 = vp.displayedObject
    if not(o1) or type(o1) != visualization.OdbType:
        return

    nColorSets,remain = divmod(len(pathList),len(allColors))
    useColors = reduce(list.__add__,[allColors]*(nColorSets+1))

    if useEndMarker:
        myEndHeadStyle = FILLED_ARROW
        intermediateStyle = HOLLOW_CIRCLE
    else:
        myEndHeadStyle = FILLED_CIRCLE
        intermediateStyle = FILLED_CIRCLE

    for iPath,path in enumerate(pathList):
        
        if useColor:
            myColor = useColors[iPath]
        else:
            myColor = '#FF0000'

        # Replace path name segments, if required.
        #
        if addNameToPath:
            if replaceList and replaceList[0]:
                xPath = path.upper()
                for fromText,toText in replaceList:
                    xPath.replace(fromText, toText)
            else:
                xPath = path[:]
        else:
            xPath = ''

        # Collect data
        #
        if session.paths[path].type in (NODE_LIST, EDGE_LIST):
            instName = session.paths[path].expression[0][0]
            space = o1.rootAssembly.instances[instName].embeddedSpace

            # Get node list for path type NODE_LIST
            #
            if session.paths[path].type == NODE_LIST:
                eTypeFlag = 1
                nodeListOri = session.paths[path].expression[0][1]
                nodeList = []
                for item in nodeListOri:
                    if type(item) == StringType:
                        n1 = int(item.split(':')[0])
                        n2 = int(item.split(':')[1])
                        m = int(item.split(':')[2])
                        if m > 0:
                            for i in range(n1,n2+1,m):
                                nodeList += [i,]
                        else:
                            for i in range(n1,n2-1,m):
                                nodeList += [i,]                        
                    else:
                        nodeList += [item,]
            #
            # get node list for path type EDGE_LIST            
            elif  session.paths[path].type == EDGE_LIST:
                edgePathList = session.paths[path].expression[0][1]
                elemList =[]
                elemDict = {}    
                for item in edgePathList:
                    elemList += [item[0],]
                #
                # create element set to get element type    
                if o1.rootAssembly.elementSets.has_key(path):
                    pass
                else:    
                    o1.rootAssembly.ElementSetFromElementLabels(name=path, elementLabels=((instName,elemList),))
                #
                # stor element type and connectivity in dictionary      
                for elem in o1.rootAssembly.elementSets[path].elements[0]:
                    elemDict[elem.label] = (elem.type, elem.connectivity, elem.sectionCategory.name)
                j = 0                    
                nodeList = []
                for elem in elemList:
                    #
                    # node search for hexahedrons    
                    #if elemDict[elem][0] in elemTypeList1: 
                    if (elemDict[elem][2].find('solid') > -1 and space == THREE_D and 
                      (len(elemDict[elem][1]) == 8 or len(elemDict[elem][1]) == 20)): 
                        eTypeFlag = 1
                        #
                        # node search for face 1
                        if edgePathList[j][1] == 1:
                            addNodeToListHex(edgePathList, j, elemDict, elem, nodeList, 0, 3, 2, 1)
                       #
                        # node search for face 2          
                        elif edgePathList[j][1] == 2:
                            addNodeToListHex(edgePathList, j, elemDict, elem, nodeList, 4, 5, 6, 7)
                        #
                        # node search for face 3          
                        elif edgePathList[j][1] == 3:
                            addNodeToListHex(edgePathList, j, elemDict, elem, nodeList, 0, 1, 5, 4)
                        #
                        # node search for face 4          
                        elif edgePathList[j][1] == 4:
                            addNodeToListHex(edgePathList, j, elemDict, elem, nodeList, 1, 2, 6, 5)
                        #
                        # node search for face 5          
                        elif edgePathList[j][1] == 5:
                            addNodeToListHex(edgePathList, j, elemDict, elem, nodeList, 3, 7, 6, 2)
                        #
                        # node search for face 6          
                        elif edgePathList[j][1] == 6:
                            addNodeToListHex(edgePathList, j, elemDict, elem, nodeList, 4, 7, 3, 0)
                    #
                    # node search for tetrahedrons    
                    #elif elemDict[elem][0] in elemTypeList2:    
                    elif (elemDict[elem][2].find('solid') > -1 and space == THREE_D and 
                      (len(elemDict[elem][1]) == 4 or len(elemDict[elem][1]) == 10)): 
                        eTypeFlag = 1
                        #
                        # node search for face 1
                        if edgePathList[j][1] == 1:
                            addNodeToListTet(edgePathList, j, elemDict, elem, nodeList, 0, 2, 1)
                        #
                        # node search for face 2
                        if edgePathList[j][1] == 2:
                            addNodeToListTet(edgePathList, j, elemDict, elem, nodeList, 0, 1, 3)
                        #
                        # node search for face 3
                        if edgePathList[j][1] == 3:
                            addNodeToListTet(edgePathList, j, elemDict, elem, nodeList, 1, 2, 3)
                        #
                        # node search for face 4
                        if edgePathList[j][1] == 4:
                            addNodeToListTet(edgePathList, j, elemDict, elem, nodeList, 0, 3, 2)
                    #
                    # node search for 2D solids or shells (quad)    
                    elif ((elemDict[elem][2].find('solid') > -1 and space in (TWO_D_PLANAR, AXISYMMETRIC) 
                      and (len(elemDict[elem][1]) == 4 or len(elemDict[elem][1]) == 8)) or 
                      (elemDict[elem][2].find('shell') > -1 and space == THREE_D and 
                      (len(elemDict[elem][1]) == 4 or len(elemDict[elem][1]) == 8))): 
                        eTypeFlag = 1
                        if edgePathList[j][2] == 1:
                            node1 = elemDict[elem][1][0]
                            node2 = elemDict[elem][1][1]
                        elif edgePathList[j][2] == 2:
                            node1 = elemDict[elem][1][1]
                            node2 = elemDict[elem][1][2]
                        elif edgePathList[j][2] == 3:
                            node1 = elemDict[elem][1][2]
                            node2 = elemDict[elem][1][3]
                        elif edgePathList[j][2] == 4:
                            node1 = elemDict[elem][1][3]
                            node2 = elemDict[elem][1][0]
                        #
                        # second get direction    
                        if edgePathList[j][3] == 1:
                             if nodeList and nodeList[-1] == node1:
                                 nodeList += [node2]                                         
                             else:                                        
                                 nodeList += [node1, node2]
                        else:
                             if nodeList and nodeList[-1] == node2:
                                 nodeList += [node1]
                             else:
                                 nodeList += [node2, node1]
                    #
                    # node search for 2D solids or shells (tri)    
                    elif ((elemDict[elem][2].find('solid') > -1 and space in (TWO_D_PLANAR, AXISYMMETRIC) 
                      and (len(elemDict[elem][1]) == 3 or len(elemDict[elem][1]) == 6)) or 
                      (elemDict[elem][2].find('shell') > -1 and space == THREE_D and 
                      (len(elemDict[elem][1]) == 3 or len(elemDict[elem][1]) == 6))): 
                        eTypeFlag = 1
                        if edgePathList[j][2] == 1:
                            node1 = elemDict[elem][1][0]
                            node2 = elemDict[elem][1][1]
                        elif edgePathList[j][2] == 2:
                            node1 = elemDict[elem][1][1]
                            node2 = elemDict[elem][1][2]
                        elif edgePathList[j][2] == 3:
                            node1 = elemDict[elem][1][2]
                            node2 = elemDict[elem][1][0]
                        #
                        # second get direction    
                        if edgePathList[j][3] == 1:
                             if nodeList and nodeList[-1] == node1:
                                 nodeList += [node2]                                         
                             else:                                        
                                 nodeList += [node1, node2]
                        else:
                             if nodeList and nodeList[-1] == node2:
                                 nodeList += [node1]
                             else:
                                 nodeList += [node2, node1]
                    else:
                        eTypeFlag = 0
                    j += 1                         
            #
            # create annotations for selected path
            if eTypeFlag:                                          
                if o1.rootAssembly.nodeSets.has_key(path):
                    pass
                else:    
                   o1.rootAssembly.NodeSetFromNodeLabels(name=path, nodeLabels=((instName,nodeList),))
                for node in o1.rootAssembly.nodeSets[path].nodes[0]:
                    nodeDir[node.label] = node.coordinates
                flag = 1
                for key in o1.userData.annotations.keys():             
                    if key.find(path) > -1:
                        flag = 0     
                if flag:
                    for i in range(len(nodeList)-1):
                        if i == len(nodeList)-2:
                            _endHeadStyle = myEndHeadStyle
                        else:
                            _endHeadStyle = FILLED_CIRCLE
                        #
                        a1 = o1.userData.Arrow(name=path+'_'+str(nodeList[i])+'_'+str(nodeList[i+1]), 
                            startPoint=(0,0), startAnchor=nodeDir[nodeList[i]],endPoint=(0,0), 
                            endAnchor=nodeDir[nodeList[i+1]], startHeadStyle=FILLED_CIRCLE, 
                            endHeadStyle=_endHeadStyle, color=myColor, lineThickness=THIN)
                        if i != len(nodeList)-2:
                            a1.setValues(endHeadStyle=intermediateStyle)
                        if i > 0:
                            a1.setValues(startHeadStyle=intermediateStyle)
                        
                        vp.plotAnnotation(annotation=a1)
                        if i == 0:
                            t = o1.userData.Text(name=path+'_Start',
                                                 text='Start_'+xPath,
                                                 offset=(0, 0),
                                                 anchor=nodeDir[nodeList[i]],
                                                 color=myColor,
                                                 referencePoint=BOTTOM_CENTER,
                                                 font=useFont)
                            vp.plotAnnotation(annotation=t)
                        if i == len(nodeList)-2:
                            t = o1.userData.Text(name=path+'_End',
                                                 text='End_'+xPath,
                                                 offset=(0, 0), 
                                                 anchor=nodeDir[nodeList[i+1]],
                                                 color=myColor,
                                                 referencePoint=BOTTOM_CENTER,
                                                 font=useFont)
                            vp.plotAnnotation(annotation=t)
            else:
                print 'Element type not yet supported. Path "%s" not plotted' % path                
        elif session.paths[path].type == POINT_LIST:        
            coordList = session.paths[path].expression
            flag = 1
            for key in o1.userData.annotations.keys():             
                if key.find(path) > -1:
                    flag = 0     
            if flag:
                for i in range(len(coordList)-1):
                    if i == len(coordList)-2:
                        _endHeadStyle = myEndHeadStyle
                    else:
                        _endHeadStyle = FILLED_CIRCLE
                    #
                    a1 = o1.userData.Arrow(name=path+'_'+str(i)+'_'+str(i+1), 
                        startPoint=(0,0), startAnchor=coordList[i],endPoint=(0,0), 
                        endAnchor=coordList[i+1], startHeadStyle=FILLED_CIRCLE, 
                        endHeadStyle=_endHeadStyle, color=myColor, lineThickness=THIN) 
                    if i != len(coordList)-2:
                        a1.setValues(endHeadStyle=intermediateStyle)
                    if i > 0:
                        a1.setValues(startHeadStyle=intermediateStyle)
                    vp.plotAnnotation(annotation=a1)
                    if i == 0:
                        t = o1.userData.Text(name=path+'_Start',
                                             text='Start_'+xPath,
                                             offset=(0, 0), 
                                             anchor=coordList[i],
                                             color=myColor,
                                             referencePoint=BOTTOM_CENTER,
                                             font=useFont)
                        vp.plotAnnotation(annotation=t)
                    if i == len(coordList)-2:
                        t = o1.userData.Text(name=path+'_End',
                                             text='End',
                                             offset=(0, 0), 
                                             anchor=coordList[i+1],
                                             color=myColor,
                                             referencePoint=BOTTOM_CENTER,
                                             font=useFont)
                        vp.plotAnnotation(annotation=t)
        else:
            print 'Path type "%s" not yet supported. Path "%s" not plotted.' % (session.paths[path].type, path)
            
                     
def addNodeToListHex(edgePathList, j, elemDict, elem, nodeList, n1,n2,n3,n4):                        
    #&& gets nodes for a selected path if tetrahedron elements are included                        
    #
    # first get edge                                     
    if edgePathList[j][2] == 1:
        node1 = n1
        node2 = n2
    elif edgePathList[j][2] == 2:
        node1 = n2
        node2 = n3
    elif edgePathList[j][2] == 3:
        node1 = n3
        node2 = n4
    elif edgePathList[j][2] == 4:
        node1 = n4
        node2 = n1
    #
    # second get direction    
    if edgePathList[j][3] == 1:
         if nodeList and nodeList[-1] == elemDict[elem][1][node1]:
             nodeList += [elemDict[elem][1][node2]]                                         
         else:                                        
             nodeList += [elemDict[elem][1][node1],elemDict[elem][1][node2]]
    else:
         if nodeList and nodeList[-1] == elemDict[elem][1][node2]:
             nodeList += [elemDict[elem][1][node1]]
         else:
             nodeList += [elemDict[elem][1][node2],elemDict[elem][1][node1]]
    #
    return nodeList
                                                     
                 
def addNodeToListTet(edgePathList, j, elemDict, elem, nodeList, n1,n2,n3):
    #&& gets nodes for a selected path if tetrahedron elements are included                        
    #
    # first get edge                                     
    if edgePathList[j][2] == 1:
        node1 = n1
        node2 = n2
    elif edgePathList[j][2] == 2:
        node1 = n2
        node2 = n3
    elif edgePathList[j][2] == 3:
        node1 = n3
        node2 = n1
    #
    # second get direction    
    if edgePathList[j][3] == 1:
         if nodeList and nodeList[-1] == elemDict[elem][1][node1]:
             nodeList += [elemDict[elem][1][node2]]                                         
         else:                                        
             nodeList += [elemDict[elem][1][node1],elemDict[elem][1][node2]]
    else:
         if nodeList and nodeList[-1] == elemDict[elem][1][node2]:
             nodeList += [elemDict[elem][1][node1]]
         else:
             nodeList += [elemDict[elem][1][node2],elemDict[elem][1][node1]]
    #
    return nodeList
