from abaqus import *
from abaqusConstants import *
from caeModules import *

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createSequenceMesh(seq, entities, entitySeq):
    for count in seq[1]:
        entitySeq.append(eval("entities["+str(count)+":"+str(count+1)+"]"))
    return entitySeq

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createSequenceGeom(seq, entities, entitySeq):
    for count in seq[1]:
        entitySeq.append(eval("entities["+str(count-1)+":"+str(count)+"]"))

    return entitySeq

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def callSequence(cellSeq,cells,faceSeq,faces,edgeSeq,edges,vertexSeq,vertices):
    cellsSetSeq, facesSetSeq, edgesSetSeq, verticesSetSeq, entitySeq = [], [], [], [], []
    if cellSeq:
        cellsSetSeq = createSequenceGeom(cellSeq[0], cells, entitySeq)
    if faceSeq:
        facesSetSeq = createSequenceGeom(faceSeq[0], faces, entitySeq)
    if edgeSeq:
        edgesSetSeq = createSequenceGeom(edgeSeq[0], edges, entitySeq)
    if vertexSeq:
        verticesSetSeq = createSequenceGeom(vertexSeq[0], vertices, entitySeq)

    return(cellsSetSeq, facesSetSeq, edgesSetSeq, verticesSetSeq)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def formGeometricSequencePart(cellSeq, faceSeq, edgeSeq, vertexSeq, part):
    cells = part.cells
    faces = part.faces
    edges = part.edges
    vertices = part.vertices
    (cellsSetSeq, facesSetSeq, edgesSetSeq, verticesSetSeq) = callSequence(cellSeq,cells,faceSeq,faces,edgeSeq,edges,vertexSeq,vertices)

    return(cellsSetSeq, facesSetSeq, edgesSetSeq, verticesSetSeq)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def formGeometricSeqAsmbly(cellSeq, faceSeq, edgeSeq, vertexSeq):
    cellsSetSeq, facesSetSeq, edgesSetSeq, verticesSetSeq, entitySeq = [], [], [], [], []

    for cell in cellSeq:
        cells = tempInstance[instanceMap[cell[0]]].cells
        cellsSetSeq = (createSequenceGeom(cell, cells, entitySeq))
    for edge in edgeSeq:
        edges = tempInstance[instanceMap[edge[0]]].edges
        edgesSetSeq = (createSequenceGeom(edge, edges, entitySeq))
    for face in faceSeq:
        faces = tempInstance[instanceMap[face[0]]].faces
        facesSetSeq = (createSequenceGeom(face, faces, entitySeq))
    for vertex in vertexSeq:
        vertices = tempInstance[instanceMap[vertex[0]]].vertices
        verticesSetSeq = (createSequenceGeom(vertex, vertices, entitySeq))
    return (cellsSetSeq, facesSetSeq, edgesSetSeq, verticesSetSeq)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getGeometricSequence(geometry):
    cellSeq = list(geometry.cellSeq)
    edgeSeq = list(geometry.edgeSeq)
    faceSeq = list(geometry.faceSeq)
    vertexSeq = list(geometry.vertexSeq)

    return(cellSeq, faceSeq, edgeSeq, vertexSeq)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def formElementNodeSequencePart(meshSeq, part, attr):
    entitySeq = []
    meshEntity = part.__getattribute__(attr)
    if meshSeq:
        meshSetSeq = createSequenceMesh(meshSeq[0], meshEntity, entitySeq)
        return meshSetSeq

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def formElementNodeSequenceAsmbly(meshSeq, attr):
    meshSetSeq = []
    entitySeq = []
    for meshEntity in meshSeq:
        mesh = tempInstance[instanceMap[meshEntity[0]]].__getattribute__(attr)
        meshSetSeq = (createSequenceMesh(meshEntity, mesh, entitySeq))

    return meshSetSeq

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def appendLeafActionList(leaf, action):
    leafList.append(leaf)
    actionPartList.append(str(action).lower())

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createSetFromLeaf(part, action, setCount, cellsSetSeq, facesSetSeq, edgesSetSeq, verticesSetSeq, elementSetSeq, nodeSetSeq):
    try:
        setDG = part.Set(cells = cellsSetSeq, faces = facesSetSeq, edges = edgesSetSeq, vertices = verticesSetSeq, elements = elementSetSeq, nodes = nodeSetSeq, name = 'myInternalSetDG-'+str(setCount))
    except:
        raise ValueError, 'Error occured while creating a set from viewport entities. Please contact Simulia support for further assistance.'
    leaf = dgm.LeafFromSets(sets=(setDG, ))
    leafList.append(leaf)
    actionPartList.append(action)
    part.markSetInternal('myInternalSetDG-'+str(setCount), True)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
viewportName = session.currentViewportName
modelName = session.viewports[session.currentViewportName].displayedObject.modelName
mdb1 = mdb.models[modelName]
assembly = mdb1.rootAssembly
displayType = type(session.viewports[session.currentViewportName].displayedObject)
partType = type(mdb.models[modelName].parts.values()[0])
assemblyType = type(mdb.models[modelName].rootAssembly)
if displayType == partType:
    dg = session.viewports[viewportName].partDisplay.displayGroup
    dgRoot = dg.root
    partName = session.viewports[session.currentViewportName].displayedObject.name
    part = mdb.models[modelName].parts[partName]
else:
    dg = session.viewports[viewportName].assemblyDisplay.displayGroup
    dgRoot = dg.root
flag = False
setCount = 0
cellsSetSeq, facesSetSeq, edgesSetSeq, verticesSetSeq, elementSetSeq, nodeSetSeq = [],[],[],[],[],[]
actionPartList = []
leafList = []
instanceMap = {}
tempFeature = assembly.features
tempInstance = assembly.instances
for instance in tempInstance.keys():
    instanceMap[tempFeature[instance].id] = instance

while(1):

    geometry = dgRoot.leaf
    action = dgRoot.action

    if str(type(geometry)) == "<type 'EnumerationLeaf'>":
        pass
    elif str(type(geometry)) == "<type 'GeometryLeaf'>":
        (cellSeq, faceSeq, edgeSeq, vertexSeq) = getGeometricSequence(geometry)
        if displayType == partType:
            (cellsSetSeq, facesSetSeq, edgesSetSeq, verticesSetSeq) = formGeometricSequencePart(cellSeq, faceSeq, edgeSeq, vertexSeq, part)
        else:
            (cellsSetSeq, facesSetSeq, edgesSetSeq, verticesSetSeq) = formGeometricSeqAsmbly(cellSeq, faceSeq, edgeSeq, vertexSeq)
            part = assembly
        createSetFromLeaf(part, str(action).lower(), setCount, cellsSetSeq, facesSetSeq, edgesSetSeq, verticesSetSeq, [], [])
    elif str(type(geometry)) == "<type 'ElementLabelLeaf'>":
        if displayType == partType:
            elementSetSeq = formElementNodeSequencePart(list(geometry.elementSeq), part, 'elements')
        else:
            elementSetSeq = formElementNodeSequenceAsmbly(list(geometry.elementSeq), 'elements')
        if elementSetSeq:
            leafElement = dgm.LeafFromMeshElementLabels(elementSeq = elementSetSeq)
            appendLeafActionList(leafElement, action)
    elif str(type(geometry)) == "<type 'NodeLabelLeaf'>":
        if displayType == partType:
            nodeSetSeq = formElementNodeSequencePart(list(geometry.nodeSeq), part, 'nodes')
        else:
            nodeSetSeq = formElementNodeSequenceAsmbly(list(geometry.nodeSeq), 'nodes')
        if nodeSetSeq:
            leafNode = dgm.LeafFromMeshNodeLabels(nodeSeq = nodeSetSeq)
            appendLeafActionList(leafNode, action)
    elif str(type(geometry)) == "<type 'PartElemLabelsLeaf'>":
        elSeq = geometry.elementLabels
        elementSetSeq = part.elements.sequenceFromLabels(elSeq)
        leafElementLabel = dgm.LeafFromMeshElementLabels(elementSeq = elementSetSeq)
        appendLeafActionList(leafElementLabel, action)
    elif str(type(geometry)) == "<type 'PartNodeLabelsLeaf'>":
        nSeq = geometry.elementLabels
        nodeSetSeq = part.nodes.sequenceFromLabels(nSeq)
        leafNodeLabel = dgm.LeafFromMeshNodeLabels(nodeSeq = nodeSetSeq)
        appendLeafActionList(leafNodeLabel, action)
    elif str(type(geometry)) == "<type 'InstanceElemLabelsLeaf'>":
        for seq in geometry.elementLabels:
            instanceObject = tempInstance[instanceMap[seq[0]]]
            elementSetSeq.append(instanceObject.elements.sequenceFromLabels(seq[1]))
        leafElementLabelAsmbly = dgm.LeafFromMeshElementLabels(elementSeq = elementSetSeq)
        appendLeafActionList(leafElementLabelAsmbly, action)
    elif str(type(geometry)) == "<type 'InstanceNodeLabelsLeaf'>":
        for seq in geometry.nodeLabels:
            instanceObject = tempInstance[instanceMap[seq[0]]]
            nodeSetSeq.append(instanceObject.nodes.sequenceFromLabels(seq[1]))
        leafNodeLabelAsmbly = dgm.LeafFromMeshNodeLabels(nodeSeq = nodeSetSeq)
        appendLeafActionList(leafNodeLabelAsmbly, action)
    elif str(type(geometry)) == "<type 'SetsLeaf'>":
        setList = []
        for setName in geometry.sets:
            if displayType == partType:
                setList.append(part.sets[setName])
            else:
                setList.append(ssembly.allSets[setName])
        leafSet = dgm.LeafFromSets(setList)
        leafList.append(leafSet)
        actionPartList.append(str(action).lower())
    elif str(type(geometry)) == "<type 'SurfaceSetsLeaf'>":
        surfaceList = []
        for surfaceName in geometry.surfaceSets:
            if displayType == partType:
                part = part
                surfaceList.append(part.surfaces[surfaceName])
            else:
                surfaceList.append(assembly.allSurfaces[surfaceName])
                part = assembly
        leafSurface = dgm.LeafFromMeshSurfaceSets(surfaceList)
        leafList.append(leafSurface)
        actionPartList.append(str(action).lower())
    elif str(type(geometry)) == "<type 'InstanceLeaf'>":
        instanceList= []
        part = assembly
        flagSet = True
        for instanceId in geometry.instances:
            instanceList.append(tempInstance[instanceMap[instanceId]])
        leafInstance = dgm.LeafFromInstance(instanceList)
        leafList.append(leafInstance)
        actionPartList.append(str(action).lower())
    else:
        raise ValueError, "The current leaf object "+str(type(geometry))+" is not supported."

    if dgRoot.next == None:
        break
    else:
        dgRoot = dgRoot.next
        setCount += 1

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def restoreDisplayGroup():
    countActionPart = 0
    for items in leafList:
        action = actionPartList[countActionPart]
        try:
            if displayType == partType:
                displayStr = "session.viewports[session.currentViewportName].partDisplay.displayGroup."+action+"(leaf=items)"
            else:
                displayStr = "session.viewports[session.currentViewportName].assemblyDisplay.displayGroup."+action+"(leaf=items)"
        except:
            raise ValueError, "Error occured while restoring the display group. Please contact Simulia support for assistance."
        eval(displayStr)
        countActionPart += 1
    if not leafList:
        print "No display group is available for restoring."
    else:
        print "Display group has been successfully restored."
