from abaqusConstants import *
from abaqusGui import *
import os

thisPath = os.path.abspath(__file__)
thisDir = os.path.dirname(thisPath)


###########################################################################
# Class definition
###########################################################################

class SetOperationDB(AFXDataDialog):

    [
        ID_SET_TYPE,
     ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+1)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        self.form = form

        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'Operate on Sets',
            self.APPLY|self.CANCEL, DECOR_RESIZE)

        #GUILOG
        getAFXAliasMap().setPrefix(self, 'setOperation')
        #ENDGUILOG

        gb = FXGroupBox(p=self, text='Sets', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        hf = FXHorizontalFrame(gb, 0, 0,0,0,0, 0,0,0,0)
        self.radioE = FXRadioButton(p=hf, text='Element', tgt=self, sel=self.ID_SET_TYPE )
        self.radioE.setCheck(True)
        getAFXAliasMap().setName(self.radioE, 'element')
        self.radioN = FXRadioButton(p=hf, text='Node', tgt=self, sel=self.ID_SET_TYPE )
        getAFXAliasMap().setName(self.radioN, 'node')
        FXMAPFUNC(self, SEL_COMMAND, self.ID_SET_TYPE, SetOperationDB.updateSets)

        self.setA = AFXComboBox(p=gb, ncols=0, nvis=1, text='A:', tgt=form.setAKw, sel=0, opts=LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.setA, 'setA')
        self.setA.setMaxVisible(10)

        self.setB = AFXComboBox(p=gb, ncols=0, nvis=1, text='B:', tgt=form.setBKw, sel=0, opts=LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.setB, 'setB')
        self.setB.setMaxVisible(10)

        gb = FXGroupBox(p=self, text='Operations', opts=FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)

        vf2 = FXVerticalFrame(gb, LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0 )
        w = FXCheckButton(vf2, text='Union of A and B', tgt=form.unionKw, sel=0)
        getAFXAliasMap().setName(w, 'Union')
        w = FXCheckButton(vf2, text='Intersection of A and B', tgt=form.interKw, sel=0)
        getAFXAliasMap().setName(w, 'Intersect')
        w = FXCheckButton(vf2, text='Complement of B in A (A - B)', tgt=form.compBAKw, sel=0)
        getAFXAliasMap().setName(w, 'Comp B in A')
        w = FXCheckButton(vf2, text='Complement of A in B (B - A)', tgt=form.compABKw, sel=0)
        getAFXAliasMap().setName(w, 'Comp A in B')
        w = FXCheckButton(vf2, text='Unique of A and B', tgt=form.uniqueKw, sel=0)
        getAFXAliasMap().setName(w, 'Unique')

        # Update combo boxes
        #
        self.updateSets(self.setA, None, None)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):
    
        unregisterCurrentContext(self.update)
        exec(getCurrentContext()['objectPath'] + '.sets.unregisterQuery(self.update)')
        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):
    
        registerCurrentContext(self.update)
        exec(getCurrentContext()['objectPath'] + '.sets.registerQuery(self.update)')
        AFXDataDialog.show(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def update(self):

        self.updateSets(self.setA, None, None)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateSets(self, sender, sel, ptr):

        self.setA.clearItems()
        self.form.setAKw.setValue('')
        self.setB.clearItems()
        self.form.setBKw.setValue('')

        cmd = 'sets=' + getCurrentContext()['objectPath'] + '.sets'
        exec(cmd)
        # Update radio buttons
        #
        if sender.getText() == 'Node':
            self.radioE.setCheck(False)
        elif sender.getText() == 'Element':
            self.radioN.setCheck(False)

        # Update combo boxes
        #
        if self.radioE.getCheck():
            for key in sorted(sets.keys()):
                if sets[key].elements and (
                    not sets[key].faces and
                    not sets[key].edges and
                    not sets[key].vertices):
                    self.setA.appendItem(text=key)
                    self.setB.appendItem(text=key)
        else:
            for key in sorted(sets.keys()):
                if not sets[key].elements and sets[key].nodes:
                    self.setA.appendItem(text=key)
                    self.setB.appendItem(text=key)
        if self.setA.getNumItems():
            self.form.setAKw.setValue(self.setA.getItemText(0))
        if self.setB.getNumItems():
            self.form.setBKw.setValue(self.setB.getItemText(0))