from abaqusGui import *


###########################################################################
# Class definition
###########################################################################

class SetOperationForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        
        # Construct the base class.
        #
        AFXForm.__init__(self, owner)

        self.cmd = AFXGuiCommand(mode=self, method='setsByOperations',
            objectName='abq_SetOperations.setOperationUtils', registerQuery=False)
            
        self.partKw = AFXStringKeyword(self.cmd, 'part', True, '')
        self.operationKw = AFXStringKeyword(self.cmd, 'setType', True, 'node')
        self.setAKw = AFXStringKeyword(self.cmd, 'setA', True)
        self.setBKw = AFXStringKeyword(self.cmd, 'setB', True)
        self.unionKw = AFXBoolKeyword(self.cmd, 'union', AFXBoolKeyword.TRUE_FALSE, False, True)
        self.interKw = AFXBoolKeyword(self.cmd, 'inter', AFXBoolKeyword.TRUE_FALSE, False, True)
        self.compBAKw = AFXBoolKeyword(self.cmd, 'compBA', AFXBoolKeyword.TRUE_FALSE, False, True)
        self.compABKw = AFXBoolKeyword(self.cmd, 'compAB', AFXBoolKeyword.TRUE_FALSE, False, True)
        self.uniqueKw = AFXBoolKeyword(self.cmd, 'unique', AFXBoolKeyword.TRUE_FALSE, False, True)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):

        errMsg = 'A part must be displayed in the current viewport.'
        if getCurrentContext()['objectPath']:
            if getCurrentContext()['objectType'] == 'Part':
                AFXForm.activate(self)
            else:
                showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)
        else:
            showAFXErrorDialog(getAFXApp().getAFXMainWindow(), errMsg)


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        # Set value of part to object path string
        #
        self.partKw.setValue(getCurrentContext()['objectPath'])

        currentDb = self.getCurrentDialog()

        if currentDb.radioE.getCheck():
            self.operationKw.setValue('Element')
        else:
            self.operationKw.setValue('Node')

        if self.setAKw.getValue() == '':
            showAFXErrorDialog(currentDb, 'Set A must be specified.')
            return False
        if self.setBKw.getValue() == '':
            showAFXErrorDialog(currentDb, 'Set B must be specified.')
            return False

        if self.setAKw.getValue() == self.setBKw.getValue():
            showAFXErrorDialog(currentDb, 'You must select two different set names.')
            return False
        
        if not (self.unionKw.getValue() or self.interKw.getValue() or
                self.compBAKw.getValue() or self.compABKw.getValue() or
                self.uniqueKw.getValue() ):
            showAFXErrorDialog(currentDb, 'At least one operation must be specified.')
            return False

        return True

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        from abq_SetOperations.setOperationDB import SetOperationDB
        self.db = SetOperationDB(self)
        return self.db
        
