"""
Script to create new nsets and elsets based on Boolean operations
"""

from math import *
from abaqusConstants import *
from abaqus import *

def setsByOperations(part, setType, setA, setB,
                  union=True, inter=True, compBA=True,
                  compAB=True, unique=True):
    """
    Method to create node or element set given two sets and 
    a boolean operation.  
    Required arguments: part, type of set (node or element), 
                        set A, set B, <5 boolean opeartions>
   
    Returns a list of sets created
    """

    p = eval(part)
    
    # Generate a list of labels
    #
    err = 0
    try:
        if setType == 'Element':
            a2 = map( lambda e: e.label, p.sets[setA].elements )
            b2 = map( lambda e: e.label, p.sets[setB].elements )
        else:
            a2 = map( lambda n: n.label, p.sets[setA].nodes )
            b2 = map( lambda n: n.label, p.sets[setB].nodes )
        keys = p.sets.keys()
    except:
        print 'An error occurred while operating on sets. Please contact support.'
        err = 1 

    # Generate sets in a list
    #
    setsCreated = []
    if not err:
        operationList = ((compBA, difference(b2,a2), setB + '_minus_' + setA),
                         (compAB, difference(a2,b2), setA + '_minus_' + setB), 
                         (inter, intersection(a2,b2), setA + '_intersect_' + setB),
                         (union, difference(a2, intersection(a2, b2)) + b2, setA + '_union_' + setB),
                         (unique, difference((a2 + b2), intersection(a2, b2)), setA + '_uncommon_' + setB),
                         )

        for operation in operationList:
            if operation[0]:
                ow = 0
                if setType == 'Element':
                    ret = p.SetFromElementLabels(operation[2], operation[1])
                else:
                    ret = p.SetFromNodeLabels(operation[2], operation[1])
                if operation[2] in keys:
                    ow = 1
                if ret:
                    if ow:
                        print "The set '%s' has been overwritten." % operation[2]
                    else:
                        print "The set '%s' has been created." % operation[2]
                    setsCreated.append(ret)
                else:
                    print "Unable to create set '%s'." % operation[2]

    return setsCreated

def difference(setA, setB):
    """
    Method to perform boolean difference

    Returns a set
    """
    result = {}
    for element in setA:
        result[element] = 1
    for element in setB:
        if result.has_key(element):
            del result[element]


    return result.keys()

def intersection(setA, setB):
    """
    Method to perform boolean intersection

    Returns a set
    """
    temp = {}
    result = []
    for elt in setA:
        temp[elt] = 1
    for elt in setB:
        if temp.has_key(elt):
            result.append(elt)
    return result