from abaqusConstants import *
from abaqusGui import *
from kernelAccess import mdb, session
import os

#from optionsForm import OptionsForm
#toolset = getAFXApp().getAFXMainWindow().getPluginToolset()
#optionsForm = OptionsForm( toolset )

thisPath = os.path.abspath(__file__)
thisDir = os.path.dirname(thisPath)

createdIcon = afxCreateIcon( os.path.join( thisDir, 'created.png') )
propagatedIcon = afxCreateIcon( os.path.join( thisDir, 'propagated.png') )
modifiedIcon = afxCreateIcon( os.path.join( thisDir, 'modified.png') )
# deactivatedIcon = afxCreateIcon( os.path.join( thisDir, 'deactivated.png') )
deactivatedIcon = None
noLongerActiveIcon = afxCreateIcon( os.path.join( thisDir, 'blank.png') )
builtIntoBaseStateIcon = afxCreateIcon( os.path.join( thisDir, 'builtIntoBaseState.png') )
propagatedFromBaseStateIcon = afxCreateIcon( os.path.join( thisDir, 'propagatedFromBaseState.png') )
instanceNotApplicableIcon = afxCreateIcon( os.path.join( thisDir, 'notApplicable.png') )
typeNotApplicableIcon = afxCreateIcon( os.path.join( thisDir, 'notApplicable.png') )

dividerIcon = afxCreateIcon( os.path.join( thisDir, 'divider.png') )
dividerIcon.create()

blankIcon = afxCreateIcon( os.path.join( thisDir, 'blank.png') )

lightBlue   = FXRGB( 128, 192, 255 )
green       = FXRGB(   0, 192, 0 )
darkGreen   = FXRGB(   0, 128, 0 )
gray        = FXRGB( 170, 170, 170 )
darkGray    = FXRGB( 120, 120, 120 )
lightGray   = FXRGB( 220, 220, 220 )
lightYellow = FXRGB( 255, 255, 196 )
black       = FXRGB( 0, 0, 0 )
white       = FXRGB( 255, 255, 255 )


###########################################################################
# Class definition
###########################################################################

class StatusTableDB(AFXDataDialog):

    [
        ID_TABLE,
        ID_MODEL,
        ID_LOADS,
        ID_BCS,
        ID_INTERACTIONS,
        ID_PREDEFINED_FIELDS,
        ID_FIELD_OUTPUT_REQUESTS,
        ID_VALUE,
        ID_STATE,
        ID_MODE,
        ID_PERTURBATION,
        ID_CONTEXT,
        ID_CURRENT_MODULE,
        ID_ALL_MODULES,
        ID_SUPPRESSED,
        ID_DELETE,
        ID_ACTIVATE,
        ID_DEACTIVATE
    ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+18)
  
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'Propagating Object Manager', self.DISMISS, DECOR_RESIZE | DIALOG_UNPOST_DESTROY )
        
        hf = FXHorizontalFrame(self, hs=10, pt=0, pb=0 )
        
        # Context
        #
        #self.contextOnIcon = afxCreateIcon( os.path.join( thisDir, 'filter.png') )
        #self.contextOffIcon = afxCreateIcon( os.path.join( thisDir, 'filter.png') )
        #self.contextOffIcon.create()
        #FXButton( hf, '\tSet/Unset Module Context', self.contextOnIcon, self, self.ID_CONTEXT )
        #FXMAPFUNC( self, SEL_COMMAND, self.ID_CONTEXT, StatusTableDB.onCmdToggleContext )
        
        self.modulesHf = FXHorizontalFrame(hf, 0, 0,0,0,0, 0,0,0,0 )
        
        FXVerticalSeparator( self.modulesHf )
        
        #Collapse
        #
        self.addIcon = afxCreateIcon( os.path.join( thisDir, 'icoR_add.png') )
        self.addIcon.scale( 16, 16 )
        self.addIcon.create()
        self.subtractIcon = afxCreateIcon( os.path.join( thisDir, 'icoR_subtract.png') )
        self.subtractIcon.scale( 16, 16 )
        self.subtractIcon.create()
        
        # Field Output
        #
        self.fieldOutputOnIcon = afxCreateIcon( os.path.join( thisDir, 'icoR_fieldOutput.png') )
        self.fieldOutputOnIcon.scale( 24, 24 )
        self.fieldOutputOffIcon = afxCreateIcon( os.path.join( thisDir, 'icoR_fieldOutput.png') )
        self.fieldOutputOffIcon.scale( 24, 24 )
        self.fieldOutputOffIcon.create()
        self.fieldOutputBtn = FXButton( self.modulesHf, '\tShow/Hide Output Requests', self.fieldOutputOnIcon, self, self.ID_FIELD_OUTPUT_REQUESTS )
        FXMAPFUNC( self, SEL_COMMAND, self.ID_FIELD_OUTPUT_REQUESTS, StatusTableDB.onCmdToggleFieldOutputRequests )
        
        # Interactions
        #
        self.interactionsOnIcon = afxCreateIcon( os.path.join( thisDir, 'icoR_interaction.png') )
        self.interactionsOnIcon.scale( 24, 24 )
        self.interactionsOffIcon = afxCreateIcon( os.path.join( thisDir, 'icoR_interaction.png') )
        self.interactionsOffIcon.scale( 24, 24 )
        self.interactionsOffIcon.create()
        self.interactionsBtn = FXButton( self.modulesHf, '\tShow/Hide Interactions', self.interactionsOnIcon, self, self.ID_INTERACTIONS )
        FXMAPFUNC( self, SEL_COMMAND, self.ID_INTERACTIONS, StatusTableDB.onCmdToggleInteractions )
        
        # Boundary conditions
        #
        self.bcsOnIcon = afxCreateIcon( os.path.join( thisDir, 'icoR_boundaryCondition.png') )
        self.bcsOnIcon.scale( 24, 24 )
        self.bcsOffIcon = afxCreateIcon( os.path.join( thisDir, 'icoR_boundaryCondition.png') )
        self.bcsOffIcon.scale( 24, 24 )
        self.bcsOffIcon.create()
        self.bcsBtn = FXButton( self.modulesHf, '\tShow/Hide Boundary Conditions', self.bcsOnIcon, self, self.ID_BCS )
        FXMAPFUNC( self, SEL_COMMAND, self.ID_BCS, StatusTableDB.onCmdToggleBCs )
        
        # Loads
        #
        self.loadsOnIcon = afxCreateIcon( os.path.join( thisDir, 'icoR_load.png') )
        self.loadsOnIcon.scale( 24, 24 )
        self.loadsOffIcon = afxCreateIcon( os.path.join( thisDir, 'icoR_load.png') )
        self.loadsOffIcon.scale( 24, 24 )
        self.loadsOffIcon.create()
        self.loadsBtn = FXButton( self.modulesHf, '\tShow/Hide Loads', self.loadsOnIcon, self, self.ID_LOADS )
        FXMAPFUNC( self, SEL_COMMAND, self.ID_LOADS, StatusTableDB.onCmdToggleLoads )
        
        # Predefined fields
        #
        self.predefinedFieldsOnIcon = afxCreateIcon( os.path.join( thisDir, 'icoR_predefinedField.png') )
        self.predefinedFieldsOnIcon.scale( 24, 24 )
        self.predefinedFieldsOffIcon = afxCreateIcon( os.path.join( thisDir, 'icoR_predefinedField.png') )
        self.predefinedFieldsOffIcon.scale( 24, 24 )
        self.predefinedFieldsOffIcon.create()
        self.predefinedFieldsBtn = FXButton( self.modulesHf, '\tShow/Hide Predefined Fields', self.predefinedFieldsOnIcon, self, self.ID_PREDEFINED_FIELDS )
        FXMAPFUNC( self, SEL_COMMAND, self.ID_PREDEFINED_FIELDS, StatusTableDB.onCmdTogglePredefinedFields )
        
        FXLabel( hf, '', blankIcon )
        
        # Suppressed features
        #
        self.suppressedOnIcon = afxCreateIcon( os.path.join( thisDir, 'suppressed.png') )
        self.suppressedOnIcon.scale( 24, 24 )
        self.suppressedOffIcon = afxCreateIcon( os.path.join( thisDir, 'suppressed.png') )
        self.suppressedOffIcon.scale( 24, 24 )
        self.suppressedOffIcon.create()
        self.suppressedBtn = FXButton( hf, '\tShow/Hide Suppressed Features', self.suppressedOnIcon, self, self.ID_SUPPRESSED )
        FXMAPFUNC( self, SEL_COMMAND, self.ID_SUPPRESSED, StatusTableDB.onCmdToggleSuppressed )
        
        # Perturbation steps
        #
        self.perturbationsOnIcon = afxCreateIcon( os.path.join( thisDir, 'perturbations.png') )
        self.perturbationsOnIcon.scale( 24, 24 )
        self.perturbationsOnIcon.create()
        self.perturbationsOffIcon = afxCreateIcon( os.path.join( thisDir, 'perturbations.png') )
        self.perturbationsOffIcon.scale( 24, 24 )
        self.perturbationsOffIcon.create()
        self.perturbationsBtn = FXButton( hf, '\tShow/Hide Perturbation Steps', self.perturbationsOnIcon, self, self.ID_PERTURBATION )
        FXMAPFUNC( self, SEL_COMMAND, self.ID_PERTURBATION, StatusTableDB.onCmdTogglePerturbations )
        
        hf = FXHorizontalFrame(self, hs=10)
        
        # Create dummy labels to blend icons into background, otherwise they 
        # don't look so good in the table.
        #
        FXLabel( hf, 'dummy', createdIcon )
        FXLabel( hf, 'dummy', propagatedIcon )
        FXLabel( hf, 'dummy', modifiedIcon )
        FXLabel( hf, 'dummy', deactivatedIcon )
        FXLabel( hf, 'dummy', noLongerActiveIcon )
        FXLabel( hf, 'dummy', builtIntoBaseStateIcon )
        FXLabel( hf, 'dummy', propagatedFromBaseStateIcon )
        FXLabel( hf, 'dummy', instanceNotApplicableIcon )
        FXLabel( hf, 'dummy', typeNotApplicableIcon )
        
        hf.hide()
                
        hf = FXHorizontalFrame(self, opts=LAYOUT_FILL_X | LAYOUT_FILL_Y )
        
        self.bShowLoads = True
        self.bShowBCs = True
        self.bShowInteractions = True
        self.bShowPredefinedFields = True
        self.bShowFieldOutputRequests = True
        
        self.bModuleContext = True
        self.bShowSuppressed = True
        self.bShowPerturbations = True

        # Table
        #
        vf = FXVerticalFrame(hf, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,
            0,0,0,0, 0,0,0,0)
        self.table = AFXTable(vf, 1, 1, 1, 1, self, self.ID_TABLE, 
        AFXTABLE_RESIZE | LAYOUT_FILL_X | LAYOUT_FILL_Y)
        
        self.table.setLeadingRows(1)
        self.table.showVerticalGrid( True )
        self.table.showHorizontalGrid( True )
        
        #self.table.appendClientPopupItem('Delete', None, self, self.ID_DELETE)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_DELETE, StatusTableDB.onCmdDelete)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_ACTIVATE, StatusTableDB.onCmdActivate)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_DEACTIVATE, StatusTableDB.onCmdDeactivate)
        
        FXMAPFUNC( self, SEL_CLICKED, self.ID_TABLE, StatusTableDB.onClickTable )
        FXMAPFUNC( self, SEL_DOUBLECLICKED, self.ID_TABLE, StatusTableDB.onDoubleClickTable )
        FXMAPFUNC(self, SEL_RIGHTBUTTONPRESS, self.ID_TABLE, StatusTableDB.onCmdRightClick)
        
        self.firstTime = True
        
        #self.modulesHf.hide()
        
        self.oldModelName = ''
        self.currentModelName = ''
        self.currentModel = None
        self.isFoCollapsed = True
        self.isIntCollapsed = True
        self.isLoadCollapsed = True
        self.isBcCollapsed = True
        self.isPfCollapsed = True
        
        self.mw = getAFXApp().getAFXMainWindow()
        
        setSwitchModuleHook( self.onModuleSwitch )
        #registerCurrentContext( self.updateCurrentContext )

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdRightClick(self, sender, sel, ptr):

        # menu = [ ((text, enabled_flag, sel), ...), (....)]
        #   each group separated by AFXSeparator

        status, x, y, buttons = self.table.getCursorPosition()
        column = self.table.getColumnAtX( x )
        row = self.table.getRowAtY( y )
        

        if row == 0 or \
           row == self.itnStartRow-1 or \
           row == self.bcStartRow-1 or \
           row == self.loadStartRow-1 or \
           row == self.forStartRow-1 or \
           row == self.pdfStartRow-1:
                           
           return
        
        self.table.setPopupOptions(opts=AFXTable.POPUP_NONE)
        self.table.clearClientPopupItems()
        self.fullRows = self.getFullItemsSelected()
        if self.fullRows == None:
            return
        self.column = column
        if len(self.fullRows) > 0:
            if column > 2 and not row > self.pdfStartRow-1:
                fullMenu = ( (("Delete", True, self, self.ID_DELETE),
                              ("Activate", True, self, self.ID_ACTIVATE),
                              ("Deactivate", True, self, self.ID_DEACTIVATE)
                         ),
                         )
            else:
                fullMenu = ( (("Delete", True, self, self.ID_DELETE),
                         ),
                         )
            # Create menu
            #
            menuPane = FXMenuPane(self)

            kItem = 0
            for kGroup in range(len(fullMenu)):
                group = fullMenu[kGroup]
                if kGroup > 0:
                    FXHorizontalSeparator(menuPane)
                for item in group:
                    kItem = kItem + 1
                    title, enable, itemTgt, itemSel = item
                    mc = FXMenuCommand(menuPane, title, None, itemTgt, itemSel)                
                    if not enable:
                        mc.disable()
                    itemName = 'Item_%d' % kItem
                    getAFXAliasMap().setName(mc, itemName)
                    setattr(menuPane, itemName, mc)
            menuPane.create()
            status, x, y, buttons = self.mw.getCursorPosition()
            x, y = self.mw.translateCoordinatesTo(self.mw.getRoot(), x, y )
            menuPane.popup(None, x, y)
                #getAFXAliasMap().setPrefix(menuPane, 'PME RMB menu-%s' % PmeModelDB.menuPaneItr)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFullItemsSelected(self):
        rows = []
        for kcol in range(1,self.table.getNumColumns()):
            for krow in range(1,self.table.getNumRows()):
                if self.table.isItemSelected(krow,kcol):
                    if krow == 0 or \
                       krow == self.itnStartRow-1 or \
                       krow == self.bcStartRow-1 or \
                       krow == self.loadStartRow-1 or \
                       krow == self.forStartRow-1 or \
                       krow == self.pdfStartRow-1:
                        pass
                    else:    
                       #showAFXErrorDialog(self.mw, "Header rows cannot be deleted.")
                       #return None,None
                        rows.append(krow)
        
        return list(set(rows))
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdDelete(self,sender,sel,ptr):
        
        typeName = ''
        deleteObjects = {}
        
        if self.fullRows:
            for row in self.fullRows:
                typeName,states = self.getTypeAndStates(row)
                if not typeName in deleteObjects.keys():
                    deleteObjects[typeName] = []
                if typeName:
                    objectName = self.table.getItemText( row, 1 ).strip()
                    if not objectName in deleteObjects[typeName]:
                        deleteObjects[typeName].append(objectName)
            modelName = getCurrentContext()[ 'modelName' ]
            if deleteObjects:
                for item in deleteObjects:
                    exec("fNames = mdb.models['%s'].%s.keys()"%(modelName,item))
                    for objName in deleteObjects[item]: 
                        if objName in fNames:
                            try:
                                sendCommand("del mdb.models['%s'].%s['%s']"%(modelName,item,objName))
                            except:
                                self.mw.writeToMessageArea("Failed to delete '%s' from '%s' repository"%(objName,item))
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getTypeAndStates(self,row):
        
        if row > self.pdfStartRow-1:
            typeName = 'predefinedFields'
            states = 'predefinedFieldStates'
        elif row > self.loadStartRow-1:
            typeName = 'loads'
            states = 'loadStates'
        elif row > self.bcStartRow-1:
            typeName = 'boundaryConditions'
            states = 'boundaryConditionStates'
        elif row > self.itnStartRow-1:
            typeName = 'interactions'
            states = 'interactionStates'
        elif row > self.forStartRow-1:
            typeName = 'fieldOutputRequests'
            states = 'fieldOutputRequestStates'
        
        return typeName,states
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdActivate(self,sender,sel,ptr):
        
        if self.fullRows:
            for row in self.fullRows:
                typeName,states = self.getTypeAndStates(row)
                status,step,model,stepName,objectName,modelName = self.getStatus(row,states)
                if status == DEACTIVATED:
                    sendCommand("mdb.models['%s'].%s['%s'].reset('%s')"%(modelName,typeName,objectName,stepName))
                else:
                    self.mw.writeToMessageArea("Feature '%s' is already activated in model '%s'."%(objectName,modelName))
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getStatus(self,row,states):
        
        stepName = self.table.getItemText( 0, self.column ).strip()
        if stepName.startswith( 'LP\n' ):
            stepName = stepName[ 3: ]
        objectName = self.table.getItemText( row, 1 ).strip()
        modelName = getCurrentContext()[ 'modelName' ]
        model = mdb.models[modelName]
        step = model.steps[stepName]
        exec( "status = step.%s[ %r ].status" % (states, objectName) )
        
        return status,step,model,stepName,objectName,modelName
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdDeactivate(self,sender,sel,ptr):
        
        if self.fullRows:
            for row in self.fullRows:
                typeName,states = self.getTypeAndStates(row)
                status,step,model,stepName,objectName,modelName = self.getStatus(row,states)
                if status != CREATED:
                    sendCommand("mdb.models['%s'].%s['%s'].deactivate('%s')"%(modelName,typeName,objectName,stepName))
                else:
                    self.mw.writeToMessageArea("Feature '%s' cannot be deactivated in step '%s' where it was created."%(objectName,stepName))
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show( self ):
        
        registerCurrentContext( self.updateCurrentContext )
        
        AFXDataDialog.show( self )
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide( self ):
        
        self.unregisterQueries()
        unregisterCurrentContext( self.updateCurrentContext )
        AFXDataDialog.hide( self )
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateCurrentContext( self ):
    
        currentModel = mdb.models[ getCurrentContext()[ 'modelName' ] ]
        
        #if currentModel != self.currentModel:
            
        if self.currentModel:
            try:
                self.unregisterQueries()
            except:
                pass

        self.oldModelName = self.currentModelName
        self.currentModelName = getCurrentContext()[ 'modelName' ]
        self.currentModel = mdb.models[ self.currentModelName ]

        self.currentModel.steps.registerQuery( self.updateTable, False )
        self.currentModel.fieldOutputRequests.registerQuery( self.updateTable, False )
        self.currentModel.interactions.registerQuery( self.updateTable, False )
        self.currentModel.boundaryConditions.registerQuery( self.updateTable, False )
        self.currentModel.loads.registerQuery( self.updateTable, False )
        self.currentModel.predefinedFields.registerQuery( self.updateTable, False )

        self.updateTable()
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def unregisterQueries(self):
        
        self.currentModel.steps.unregisterQuery( self.updateTable )
        self.currentModel.fieldOutputRequests.unregisterQuery( self.updateTable )
        self.currentModel.interactions.unregisterQuery( self.updateTable )
        self.currentModel.boundaryConditions.unregisterQuery( self.updateTable )
        self.currentModel.loads.unregisterQuery( self.updateTable )
        self.currentModel.predefinedFields.unregisterQuery( self.updateTable )
        self.currentModel = None
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdToggleContext(self, sender, sel, ptr):
    
        if self.bModuleContext:
            sender.setIcon( self.contextOffIcon )
            sender.setButtonStyle( BUTTON_TOOLBAR )
            self.bModuleContext = False
            self.modulesHf.show()
        else:
            sender.setIcon( self.contextOnIcon )
            sender.setButtonStyle( BUTTON_NORMAL )
            self.bModuleContext = True
            self.modulesHf.hide()

        self.onModuleSwitch( getCurrentContext()[ 'moduleName' ] )

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdToggleFieldOutputRequests(self, sender, sel, ptr):
    
        if self.bShowFieldOutputRequests:
            sender.setIcon( self.fieldOutputOffIcon )
            sender.setButtonStyle( BUTTON_TOOLBAR )
            self.bShowFieldOutputRequests = False
        else:
            sender.setIcon( self.fieldOutputOnIcon )
            sender.setButtonStyle( BUTTON_NORMAL )
            self.bShowFieldOutputRequests = True

        self.setItemVisibility( self.forStartRow-1, self.forEndRow, self.bShowFieldOutputRequests )

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdToggleInteractions(self, sender, sel, ptr):
    
        if self.bShowInteractions:
            sender.setIcon( self.interactionsOffIcon )
            sender.setButtonStyle( BUTTON_TOOLBAR )
            self.bShowInteractions = False
        else:
            sender.setIcon( self.interactionsOnIcon )
            sender.setButtonStyle( BUTTON_NORMAL )
            self.bShowInteractions = True

        self.setItemVisibility( self.itnStartRow-1, self.itnEndRow, self.bShowInteractions )
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdToggleBCs(self, sender, sel, ptr):
    
        if self.bShowBCs:
            sender.setIcon( self.bcsOffIcon )
            sender.setButtonStyle( BUTTON_TOOLBAR )
            self.bShowBCs = False
        else:
            sender.setIcon( self.bcsOnIcon )
            sender.setButtonStyle( BUTTON_NORMAL )
            self.bShowBCs = True

        self.setItemVisibility( self.bcStartRow-1, self.bcEndRow, self.bShowBCs )
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdToggleLoads(self, sender, sel, ptr):
    
        if self.bShowLoads:
            sender.setIcon( self.loadsOffIcon )
            sender.setButtonStyle( BUTTON_TOOLBAR )
            self.bShowLoads = False
        else:
            self.bShowLoads = True
            sender.setButtonStyle( BUTTON_NORMAL )
            sender.setIcon( self.loadsOnIcon )

        self.setItemVisibility( self.loadStartRow-1, self.loadEndRow, self.bShowLoads )

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdTogglePredefinedFields(self, sender, sel, ptr):
    
        if self.bShowPredefinedFields:
            sender.setIcon( self.predefinedFieldsOffIcon )
            sender.setButtonStyle( BUTTON_TOOLBAR )
            self.bShowPredefinedFields = False
        else:
            sender.setIcon( self.predefinedFieldsOnIcon )
            sender.setButtonStyle( BUTTON_NORMAL )
            self.bShowPredefinedFields = True

        self.setItemVisibility( self.pdfStartRow-1, self.pdfEndRow, self.bShowPredefinedFields )
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdToggleSuppressed(self, sender, sel, ptr):
    
        if self.bShowSuppressed:
            sender.setIcon( self.suppressedOffIcon )
            sender.setButtonStyle( BUTTON_TOOLBAR )
            self.bShowSuppressed = False
        else:
            sender.setIcon( self.suppressedOnIcon )
            sender.setButtonStyle( BUTTON_NORMAL )
            self.bShowSuppressed = True

        # Rebuild table with or without suppressed features
        #
        self.updateTable()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdTogglePerturbations(self, sender, sel, ptr):
    
        # Show/hide perturbation step columns by setting column width
        #
        self.setPerturbationColumnWidth()

        if self.bShowPerturbations:
            self.perturbationsBtn.setIcon( self.perturbationsOffIcon )
            self.perturbationsBtn.setButtonStyle( BUTTON_TOOLBAR )
            self.bShowPerturbations = False
        else:
            self.perturbationsBtn.setIcon( self.perturbationsOnIcon )
            self.perturbationsBtn.setButtonStyle( BUTTON_NORMAL )
            self.bShowPerturbations = True

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def setPerturbationColumnWidth(self):
    
        steps = self.currentModel.steps.values()
        i = 0
        for column in range( 2, self.table.getNumColumns() ):
            if steps[ i ].perturbation:
                if self.bShowPerturbations:
                    self.tableColumnWidths[ i ] = self.table.getColumnWidth( column )
                    self.table.setColumnWidth( column, 0 )
                else:
                    self.table.setColumnWidth( column, self.tableColumnWidths[ i ] )
            i += 1
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onDoubleClickTable(self, sender, sel, ptr):
    
        status, x, y, buttons = self.table.getCursorPosition()
        column = self.table.getColumnAtX( x )
        row = self.table.getRowAtY( y )
        
        if self.table.getItemBackColor( row, column ) == white:
            return
        
        stepName = self.table.getItemText( 0, column ).strip()
        if stepName.startswith( 'LP\n' ):
            stepName = stepName[ 3: ]
            
        menuName = ''
        moduleName = ''
        
        # Create object
        #
        if row == self.pdfStartRow-1:
            menuName = 'Predefined Field->Create'
            moduleName = 'Load'
        elif row == self.loadStartRow-1:
            menuName = 'Load->Create'
            moduleName = 'Load'
        elif row == self.bcStartRow-1:
            menuName = 'BC->Create'
            moduleName = 'Load'
        elif row == self.itnStartRow-1:
            menuName = 'Interaction->Create'
            moduleName = 'Interaction'
        elif row == self.forStartRow-1:
            menuName = 'Output->Field Output Requests->Create'
            moduleName = 'Step'
            
        if menuName != '':
            createObject( menuName, moduleName, stepName, self )
            return
 
        # Edit object
        #
        if row > self.pdfStartRow-1:
            menuName = 'Predefined Field'
            moduleName = 'Load'
        elif row > self.loadStartRow-1:
            menuName = 'Load'
            moduleName = 'Load'
        elif row > self.bcStartRow-1:
            menuName = 'BC'
            moduleName = 'Load'
        elif row > self.itnStartRow-1:
            menuName = 'Interaction'
            moduleName = 'Interaction'
        elif row > self.forStartRow-1:
            menuName = 'Output'
            moduleName = 'Step'
            
        objectName = self.table.getItemText( row, 1 ).strip()
        print "names",menuName,moduleName,objectName
        editObject( moduleName, menuName, objectName, stepName )

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onClickTable(self, sender, sel, ptr):
    
        status, x, y, buttons = self.table.getCursorPosition()
        column = self.table.getColumnAtX( x )
        row = self.table.getRowAtY( y )

        # Only process clicks on the first column of a non-header row
        #
        if column != 0:  return

        if row == self.itnStartRow-1:
            
            if self.isIntCollapsed:
                self.table.setItemIcon(row, column, self.addIcon )
                self.isIntCollapsed = False
            else:
                self.table.setItemIcon(row, column, self.subtractIcon)
                self.isIntCollapsed = True
            
            self.setItemVisibility( self.itnStartRow, self.itnEndRow, self.isIntCollapsed )
            return
            
        elif row == self.bcStartRow-1:
            
            if self.isBcCollapsed:
                self.table.setItemIcon(row, column, self.addIcon )
                self.isBcCollapsed = False
            else:
                self.table.setItemIcon(row, column, self.subtractIcon )
                self.isBcCollapsed = True
            
            self.setItemVisibility( self.bcStartRow, self.bcEndRow,self.isBcCollapsed  )
            return
            
        elif row == self.loadStartRow-1:
            
            if self.isLoadCollapsed:
                self.table.setItemIcon(row, column, self.addIcon )
                self.isLoadCollapsed = False
            else:
                self.table.setItemIcon(row, column, self.subtractIcon )
                self.isLoadCollapsed = True
            
            self.setItemVisibility( self.loadStartRow, self.loadEndRow,self.isLoadCollapsed )
            return
            
        elif row == self.forStartRow-1:
            
            if self.isFoCollapsed:
                self.table.setItemIcon(row, column, self.addIcon )
                self.isFoCollapsed = False
            else:
                self.table.setItemIcon(row, column, self.subtractIcon )
                self.isFoCollapsed = True
            
            self.setItemVisibility( self.forStartRow, self.forEndRow,self.isFoCollapsed )
            return
            
        elif row == self.pdfStartRow-1:
            if self.isPfCollapsed:
                self.table.setItemIcon(row, column, self.addIcon )
                self.isPfCollapsed = False
            else:
                self.table.setItemIcon(row, column, self.subtractIcon )
                self.isPfCollapsed = True
            
            self.setItemVisibility( self.pdfStartRow, self.pdfEndRow, self.isPfCollapsed )
            return
            
        try:
            self.setRowState( row )
        except:
            pass
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def setRowState(self, row):
    
        # TBD: Why -1 for startRows everywhere rather than just setting properly to begin with?
        
        if row > self.pdfStartRow-1:
            typeName = 'predefinedFields'
            states = 'predefinedFieldStates'
        elif row > self.loadStartRow-1:
            typeName = 'loads'
            states = 'loadStates'
        elif row > self.bcStartRow-1:
            typeName = 'boundaryConditions'
            states = 'boundaryConditionStates'
        elif row > self.itnStartRow-1:
            typeName = 'interactions'
            states = 'interactionStates'
        elif row > self.forStartRow-1:
            typeName = 'fieldOutputRequests'
            states = 'fieldOutputRequestStates'
        
        objectName = self.table.getItemText( row, 1 ).strip()
        modelName = getCurrentContext()[ 'modelName' ]
        
        if self.bShowSuppressed:
            self.colorRow( row, typeName, states )
        else:
            self.table.setRowHeight( row, 0 )
        
        if self.table.getItemBoolValue( row, 0 ):
            sendCommand( "mdb.models[ %r ].%s[ %r ].resume()" % ( modelName, typeName, objectName ) )
        else:
            sendCommand( "mdb.models[ %r ].%s[ %r ].suppress()" % ( modelName, typeName, objectName ) )
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def setItemVisibility( self, startRow, endRow, isVisible ):
    
        if isVisible:
            height = self.rowHeight
        else:
            height = 0
            
        for i in range( startRow, endRow ):
            self.table.setRowHeight( i, height )
                                                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateTable( self ):
    
        # TBD: Why does this take so long?

        # Initialize table
        #
        self.table.deleteColumns( 2, self.table.getNumColumns()-2 )
        self.table.deleteRows( 1, self.table.getNumRows()-1 )
        
        # Create a column for suppress/resume button
        #
        if self.firstTime:
            self.table.insertColumns(0, 1)
            self.table.setColumnWidth(0, 24)
            self.table.setColumnType( 0, AFXTable.BOOL )
        
        # Create a column for each step
        #
        i = 2
        labels = ' \t'
        for name,step in self.currentModel.steps.items():
        
            self.table.insertColumns(i, 1)
            self.table.setColumnJustify(i, self.table.CENTER)
            if step.perturbation:
                name = 'LP\n' + name
            labels += '\t' + name
            i += 1

        self.table.setLeadingRowLabels( labels )
           
        if self.firstTime:
            self.tableColumnWidths = []
            for i in range( self.table.getNumColumns() ):
                self.tableColumnWidths.append( self.table.getColumnWidth( i ) )
                if i > 1:
                    self.table.setColumnType(i, self.table.ICON )
            self.firstTime = False
        
        # Create rows for each object type
        #
        self.numSteps = len( self.currentModel.steps )
        
        self.forStartRow, self.forEndRow = self.createSection( 'Field Output Requests', 'fieldOutputRequests', 'fieldOutputRequestStates', 1 )
        self.itnStartRow, self.itnEndRow = self.createSection( 'Interactions', 'interactions', 'interactionStates', self.forEndRow )
        self.bcStartRow, self.bcEndRow = self.createSection( "BC's", 'boundaryConditions', 'boundaryConditionStates', self.itnEndRow )
        self.loadStartRow, self.loadEndRow = self.createSection( 'Loads', 'loads', 'loadStates', self.bcEndRow )
        self.pdfStartRow, self.pdfEndRow = self.createSection( 'Predefined Fields', 'predefinedFields', 'predefinedFieldStates', self.loadEndRow )
        
        self.rowHeight = self.table.getRowHeight( 1 )
        
        # Filter table based on current visibility settings
        #
        self.onModuleSwitch( getCurrentContext()[ 'moduleName' ] )
        steps = self.currentModel.steps.values()
        cnt = 0
        for column in range( 2, self.table.getNumColumns() ):
            if steps[ cnt ].perturbation:
                if not self.bShowPerturbations:
                    self.table.setColumnWidth( column, 0 )
            cnt += 1        

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def createSection(self, name, type, states, startRow ):
    
        # Add header row 
        #
        i = self.table.getNumRows()
        self.table.insertRows(i, 1)
        self.table.setItemType(i, 0, AFXTable.ICON )
        self.table.setItemIcon(i,0,self.subtractIcon)
        self.table.setItemText(i, 1, name )
        for j in range( self.numSteps + 2 ):
            self.table.setItemBackColor(i, j, lightBlue )
        
        self.table.setItemEditable( i, 0, True )
        self.table.setItemBoolValue( i, 0, True )    
        # Add feature rows 
        #
        exec( "featureNames = self.currentModel.%s.keys()" % type )
        featureNames.sort()
        i = self.table.getNumRows()
        startRow = i
        
        for featureName in featureNames:
        
            exec( "suppressed = self.currentModel.%s[ %r ].suppressed" % ( type , featureName ) )
            
            # Create row, populate first two columns (suppress/resume and feature name)
            #
            if not self.bShowSuppressed and suppressed:
                continue
                
            else:
                self.table.insertRows(i, 1)
                self.table.setItemText(i, 1, '   ' + featureName )
                self.table.setItemEditable( i, 0, True )

                if suppressed:
                    self.table.setItemBoolValue( i, 0, False )
                else:
                    self.table.setItemBoolValue( i, 0, True )

            self.colorRow( i, type, states )

            # Populate step columns
            #
            j = 2
            for stepName,step in self.currentModel.steps.items():

                if ( type == 'loads' or type == 'fieldOutputRequests' ) and stepName == 'Initial':
                    j += 1
                    continue            

                if 1:  #self.modeTgt.getValue() == self.ID_STATE:

                    exec( "status = step.%s[ %r ].status" % (states, featureName) )

                    if status == NOT_YET_ACTIVE:
                        self.table.setItemIcon(i, j, None )
                    elif status == CREATED:
                        self.table.setItemIcon(i, j, createdIcon )
                    elif status == PROPAGATED:
                        self.table.setItemIcon(i, j, propagatedIcon )
                    elif status == MODIFIED:
                        self.table.setItemIcon(i, j, modifiedIcon )
                    elif status == DEACTIVATED:
                        self.table.setItemIcon(i, j, deactivatedIcon )
                    elif status == NO_LONGER_ACTIVE:
                        self.table.setItemIcon(i, j, noLongerActiveIcon )
                    elif status == BUILT_INTO_BASE_STATE:
                        self.table.setItemIcon(i, j, builtIntoBaseStateIcon )
                    elif status == PROPAGATED_FROM_BASE_STATE:
                        self.table.setItemIcon(i, j, propagatedFromBaseStateIcon )
                    elif status == INSTANCE_NOT_APPLICABLE:
                        self.table.setItemIcon(i, j, None )
                    elif status == TYPE_NOT_APPLICABLE:
                        self.table.setItemIcon(i, j, None )
                    else:
                        self.table.setItemText(i, j, str( status ) )
                        
#                 else:
# 
#                     if type == 'loads':
# 
#                         for featureName in featureNames:
#                             if not self.bShowSuppressed:
#                                 exec( "suppressed = self.currentModel.%s[ %r ].suppressed" % ( type , featureName ) )
#                                 if suppressed:  continue
# 
#                             exec( "status = step.%s[ %r ].status" % (states, featureName) )
# 
#                             if  status == NOT_YET_ACTIVE or \
#                                 status == DEACTIVATED or \
#                                 status == NO_LONGER_ACTIVE or \
#                                 status == BUILT_INTO_BASE_STATE or \
#                                 status == PROPAGATED_FROM_BASE_STATE or \
#                                 status == INSTANCE_NOT_APPLICABLE or \
#                                 status == TYPE_NOT_APPLICABLE:
# 
#                                 magnitude = 'n/a'
# 
#                             elif self.currentModel.loads[ featureName ].suppressed:
# 
#                                 magnitude = 'sup'
# 
#                             else:
# 
#                                 if 'magnitude' in dir( step.loadStates[ featureName ] ):
#                                     magnitude = step.loadStates[ featureName ].magnitude
# 
#                                 elif 'cf1' in dir( step.loadStates[ featureName ] ):
#                                     magnitude = step.loadStates[ featureName ].cf1
# 
#                                 else:
#                                     magnitude = 'Unk'
# 
#                             self.table.setItemText(i, j, str( magnitude ) )
#                             i += 1
# 
#                     else:
# 
#                         for featureName in featureNames:
#                             if not self.bShowSuppressed:
#                                 exec( "suppressed = self.currentModel.%s[ %r ].suppressed" % ( type , featureName ) )
#                                 if suppressed:  continue
# 
#                             exec( "status = step.%s[ %r ].status" % (states, featureName) )
# 
#                             if  status == NOT_YET_ACTIVE or \
#                                 status == DEACTIVATED or \
#                                 status == NO_LONGER_ACTIVE or \
#                                 status == BUILT_INTO_BASE_STATE or \
#                                 status == PROPAGATED_FROM_BASE_STATE or \
#                                 status == INSTANCE_NOT_APPLICABLE or \
#                                 status == TYPE_NOT_APPLICABLE:
# 
#                                 magnitude = 'n/a'
# 
#                             else:
# 
#                                 magnitude = 'tbd'
# 
#                             self.table.setItemText(i, j, str( magnitude ) )
#                             i += 1

                j += 1
            i += 1

        endRow = i
        
        return ( startRow, endRow )
                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def colorRow( self, row, type, states ):
    
        featureName = self.table.getItemText( row, 1 ).strip()
        exec( "suppressed = self.currentModel.%s[ %r ].suppressed" % ( type , featureName ) )
        
        if suppressed:
            self.table.setItemBackColor(row, 0, gray )
            self.table.setItemBackColor(row, 1, gray )
        else:
            self.table.setItemBackColor(row, 0, white )
            self.table.setItemBackColor(row, 1, white )
        
        j = 2
        for stepName,step in self.currentModel.steps.items():

            if ( type == 'loads' or type == 'fieldOutputRequests' ) and stepName == 'Initial':
            
                if suppressed:
                    self.table.setItemBackColor(row, j, gray )
                else:
                    self.table.setItemBackColor(row, j, white )
                    
                j += 1
                continue            

            exec( "status = step.%s[ %r ].status" % (states, featureName) )

            if suppressed:
            
                self.table.setItemBackColor(row, j, gray )
                
            else:
            
                if status == NOT_YET_ACTIVE:
                    self.table.setItemBackColor(row, j, white )
                elif status == CREATED:
                    self.table.setItemBackColor(row, j, lightYellow )
                elif status == PROPAGATED:
                    self.table.setItemBackColor(row, j, lightYellow )
                elif status == MODIFIED:
                    self.table.setItemBackColor(row, j, lightYellow )
                elif status == DEACTIVATED:
                    self.table.setItemBackColor(row, j, lightGray )
                elif status == NO_LONGER_ACTIVE:
                    self.table.setItemBackColor(row, j, lightGray )
                elif status == BUILT_INTO_BASE_STATE:
                    self.table.setItemBackColor(row, j, lightYellow )
                elif status == PROPAGATED_FROM_BASE_STATE:
                    self.table.setItemBackColor(row, j, lightYellow )
                elif status == INSTANCE_NOT_APPLICABLE:
                    self.table.setItemBackColor(row, j, white )
                elif status == TYPE_NOT_APPLICABLE:
                    self.table.setItemBackColor(row, j, white )
                else:
                    self.table.setItemBackColor(row, j, black )
                    
            j += 1
                        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onModuleSwitch( self, moduleName ):
    
        if self.bModuleContext:
        
            if moduleName == 'Step':

                self.setItemVisibility( self.forStartRow-1, self.forEndRow, True )
                self.setItemVisibility( self.itnStartRow-1, self.itnEndRow, False )
                self.setItemVisibility( self.bcStartRow-1, self.bcEndRow, False )
                self.setItemVisibility( self.loadStartRow-1, self.loadEndRow, False )
                self.setItemVisibility( self.pdfStartRow-1, self.pdfEndRow, False )

                self.fieldOutputBtn.setButtonStyle( BUTTON_NORMAL )
                self.interactionsBtn.setButtonStyle( BUTTON_TOOLBAR )
                self.bcsBtn.setButtonStyle( BUTTON_TOOLBAR )
                self.loadsBtn.setButtonStyle( BUTTON_TOOLBAR )
                self.predefinedFieldsBtn.setButtonStyle( BUTTON_TOOLBAR )
                
                self.bShowFieldOutputRequests = True
                self.bShowInteractions = False
                self.bShowBCs = False
                self.bShowLoads = False
                self.bShowPredefinedFields = False

            elif moduleName == 'Interaction':

                self.setItemVisibility( self.forStartRow-1, self.forEndRow, False )
                self.setItemVisibility( self.itnStartRow-1, self.itnEndRow, True )
                self.setItemVisibility( self.bcStartRow-1, self.bcEndRow, False )
                self.setItemVisibility( self.loadStartRow-1, self.loadEndRow, False )
                self.setItemVisibility( self.pdfStartRow-1, self.pdfEndRow, False )

                self.fieldOutputBtn.setButtonStyle( BUTTON_TOOLBAR )
                self.interactionsBtn.setButtonStyle( BUTTON_NORMAL )
                self.bcsBtn.setButtonStyle( BUTTON_TOOLBAR )
                self.loadsBtn.setButtonStyle( BUTTON_TOOLBAR )
                self.predefinedFieldsBtn.setButtonStyle( BUTTON_TOOLBAR )

                self.bShowFieldOutputRequests = False
                self.bShowInteractions = True
                self.bShowBCs = False
                self.bShowLoads = False
                self.bShowPredefinedFields = False

            elif moduleName == 'Load':

                self.setItemVisibility( self.forStartRow-1, self.forEndRow, False )
                self.setItemVisibility( self.itnStartRow-1, self.itnEndRow, False )
                self.setItemVisibility( self.bcStartRow-1, self.bcEndRow, True )
                self.setItemVisibility( self.loadStartRow-1, self.loadEndRow, True )
                self.setItemVisibility( self.pdfStartRow-1, self.pdfEndRow, True )

                self.fieldOutputBtn.setButtonStyle( BUTTON_TOOLBAR )
                self.interactionsBtn.setButtonStyle( BUTTON_TOOLBAR )
                self.bcsBtn.setButtonStyle( BUTTON_NORMAL )
                self.loadsBtn.setButtonStyle( BUTTON_NORMAL )
                self.predefinedFieldsBtn.setButtonStyle( BUTTON_NORMAL )

                self.bShowFieldOutputRequests = False
                self.bShowInteractions = False
                self.bShowBCs = True
                self.bShowLoads = True
                self.bShowPredefinedFields = True
            
            elif moduleName == 'Assembly':

                self.setItemVisibility( self.forStartRow-1, self.forEndRow, True )
                self.setItemVisibility( self.itnStartRow-1, self.itnEndRow, True )
                self.setItemVisibility( self.bcStartRow-1, self.bcEndRow, True )
                self.setItemVisibility( self.loadStartRow-1, self.loadEndRow, True )
                self.setItemVisibility( self.pdfStartRow-1, self.pdfEndRow, True )

                self.fieldOutputBtn.setButtonStyle( BUTTON_NORMAL )
                self.interactionsBtn.setButtonStyle( BUTTON_NORMAL )
                self.bcsBtn.setButtonStyle( BUTTON_NORMAL )
                self.loadsBtn.setButtonStyle( BUTTON_NORMAL )
                self.predefinedFieldsBtn.setButtonStyle( BUTTON_NORMAL )

                self.bShowFieldOutputRequests = True
                self.bShowInteractions = True
                self.bShowBCs = True
                self.bShowLoads = True
                self.bShowPredefinedFields = True    

        else:
        
            if moduleName == 'Step' or moduleName == 'Interaction' or moduleName == 'Load' or moduleName == 'Assembly':

                self.setItemVisibility( self.forStartRow-1, self.forEndRow, self.bShowFieldOutputRequests )
                self.setItemVisibility( self.itnStartRow-1, self.itnEndRow, self.bShowInteractions )
                self.setItemVisibility( self.bcStartRow-1, self.bcEndRow, self.bShowBCs )
                self.setItemVisibility( self.loadStartRow-1, self.loadEndRow, self.bShowLoads )
                self.setItemVisibility( self.pdfStartRow-1, self.pdfEndRow, self.bShowPredefinedFields )

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createObject( menuName, moduleName, stepName, db ):

    switchModule( moduleName )

    if stepName != '':
        switchStep( stepName )
    
    # Invoke create command
    #
    tgt = getAFXApp().getAFXMainWindow().getTargetFromFunction( menuName )
    sel = getAFXApp().getAFXMainWindow().getSelectorFromFunction( menuName )
    
    if tgt:
        tgt.handle( db, MKUINT( sel, SEL_COMMAND ), None )
        
    else:
    
        showAFXInformationDialog( getAFXApp().getAFXMainWindow(),
            'Creating from the table is not allowed until after the\n'
            'corresponding module has been visited once.\n\n'
            'Try double-clicking again now.' )

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def switchStep( stepName ):

    """stepCombo = getAFXAliasMap().getWidgetFromAlias( 'Context + Step' ).getParent().getParent()
    upBtn = getAFXAliasMap().getWidgetFromAlias( 'Context + Step' ).getParent().getFirst().getFirst()
    downBtn = upBtn.getNext()
    
    currentItemIndex = stepCombo.getCurrentItem()
    newItemIndex = None
    
    for i in range( stepCombo.getNumItems() ):
        if stepCombo.getItemText( i ) == stepName:
            newItemIndex = i
            break
    
    if newItemIndex:
        if newItemIndex > currentItemIndex:
            for i in range( newItemIndex - currentItemIndex ):
                stepCombo.handle( downBtn, MKUINT( AFXComboBox.ID_DECREMENT, SEL_COMMAND ), None )
        elif newItemIndex < currentItemIndex:
            for i in range( currentItemIndex - newItemIndex ):
                stepCombo.handle( upBtn,   MKUINT( AFXComboBox.ID_INCREMENT, SEL_COMMAND ), None )"""
                
    try:
        sendCommand("session.viewports[session.currentViewportName].assemblyDisplay.setValues(step='%s')"%stepName)
    except:
        pass

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def editObject( moduleName, menuName, objectName, stepName ):

    switchModule( moduleName )

    switchStep( stepName )
    
    # Activate the edit command from the appropriate menu
    #
    menuBar = getAFXApp().getAFXMainWindow().getMenubar()
    menuTitle = getWidgetFromText( menuBar, menuName )
    menuPane = menuTitle.getMenu()
    if menuName == 'Output':
        menuPane = getWidgetFromText( menuPane, 'Field Output Requests' )

    editMenuCascade = getWidgetFromText( menuPane, 'Edit' )
    
    editMenu = editMenuCascade.getMenu()
    firstItem = editMenu.getFirst()
    menuCommand = getWidgetFromText( editMenu, objectName )
    
    if not menuCommand:
        menuCommand = AFXMenuCommand(owner=editMenu.getOwner(),p=editMenu, label=objectName, ic=None, 
                                     tgt=firstItem.getTarget(), sel=firstItem.getSelector())
        menuCommand.hide()
    if menuCommand:

        tgt = menuCommand.getTarget()
        sel = menuCommand.getSelector()

        tgt.handle( menuCommand, MKUINT( sel, SEL_COMMAND ), None )
        
    else:
    
        showAFXInformationDialog( getAFXApp().getAFXMainWindow(),
            'Editing from the table is not allowed until after the\n'
            'corresponding module has been visited once.\n\n'
            'Try double-clicking again now.' )
