from abaqusGui import *
from abq_SumRF.sumRFSymbConsts import CURRENT_PICKED
import abaqusConstants

###########################################################################
# Class definition
###########################################################################

class SumRFForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        self.owner = owner
        AFXForm.__init__(self, owner=owner)
        self.getRFSubsetCmd = AFXGuiCommand(
            mode=self,
            method='getRFSubset',
            objectName='abq_SumRF.sumRFUtils',
            registerQuery=True)
        self.nodesPickedKw = AFXObjectKeyword(
            command=self.getRFSubsetCmd,
            name='nodesPicked',
            isRequired=True,
            defaultValue=None)
        self.timeKw = AFXSymConstKeyword(
            command=self.getRFSubsetCmd,
            name='timeParamter',
            isRequired=True,
            defaultValue=CURRENT_PICKED.getId())
        self.rf1Kw = AFXBoolKeyword(
            command=self.getRFSubsetCmd,
            name='rf1',
            isRequired=False)
        self.rf2Kw = AFXBoolKeyword(
            command=self.getRFSubsetCmd,
            name='rf2',
            isRequired=False)
        self.rf3Kw = AFXBoolKeyword(
            command=self.getRFSubsetCmd,
            name='rf3',
            isRequired=False)
        self.rfmKw = AFXBoolKeyword(
            command=self.getRFSubsetCmd,
            name='rfm',
            isRequired=False)
        self.setNameKw = AFXTupleKeyword(
            command=self.getRFSubsetCmd,
            name='setNames',
            isRequired=True,
            opts=AFXTUPLE_TYPE_STRING)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        """Return the first dialog to be posted"""

        from abq_SumRF.sumRFPickDB import SumRFPickDB
        self.sumRFSubsetDB = SumRFPickDB(self)
        self.getRFSubsetCmd.setKeywordValuesToDefaults(ignoreUnspecified=False)

        return self.sumRFSubsetDB

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):

        mainWindow = getAFXApp().getAFXMainWindow()
        from kernelAccess import session
        vp=session.viewports[session.currentViewportName]
        if getDisplayedObjectType() == XY_PLOT:
            if vp.odbDisplay.name: #display the odb if present, only if it does not have perturbation steps in it

                # Check for perturbation steps in currrent odb
                #
                pertrubationStepExists = 0
                odbName = vp.odbDisplay.name
                o = session.odbs[odbName]
                stepNameList = o.steps.keys()

                # Check if the stepName list has any steps which are perturbation, if there are we issue an error and not let the
                # user to use the plug-in
                """for name in stepNameList:
                    if o.steps[name].domain == abaqusConstants.TIME:
                        if o.steps[name].timePeriod == 0:
                            pertrubationStepExists = 0
                            break
                if pertrubationStepExists:
                    showAFXErrorDialog(mainWindow,'The plug-in only works for an analysis '
                                                  'which does not have any linear '
                                                  'perturbation step in it.')"""
                st = 'vp=session.viewports[session.currentViewportName]\nodbName=vp.odbDisplay.name\n' \
                     'odb1=session.odbs[odbName]\nvp.setValues(displayedObject=odb1)'
                sendCommand(st)
                AFXForm.activate(self)
            else:
                showAFXErrorDialog(mainWindow,'There is no output database (ODB) \n'
                                              'associated with the current viewport.\n\n'
                                              'Select an ODB from the context bar,\n'
                                              'or use "File->Open" to open an ODB.')
        elif getDisplayedObjectType() == None: # No odb in viewport
            showAFXErrorDialog(mainWindow,'There is no output database (ODB) \n'
                                          'associated with the current viewport.\n\n'
                                          'Select an ODB from the context bar,\n'
                                          'or use "File->Open" to open an ODB.')
        else:
            # Odb is being displayed now
            # Check for perturbation steps in currrent odb
            """pertrubationStepExists = 0
            odbName = vp.odbDisplay.name
            o = session.odbs[odbName]
            stepNameList = o.steps.keys()

            # Check if the stepName list has any steps which are perturbation, if there are we issue an error and not let the
            # user to use the plug-in
            for name in stepNameList:
                if o.steps[name].domain == abaqusConstants.TIME:
                    if o.steps[name].timePeriod == 0:
                        pertrubationStepExists = 0
                        break
            if pertrubationStepExists:
                showAFXErrorDialog(mainWindow,'The plug-in only works for an analysis which does not have any linear perturbation step in it.')"""
            #else:
            AFXForm.activate(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def deactivate(self):
        mainWindow = getAFXApp().getAFXMainWindow()
        from kernelAccess import session
        vp = session.viewports[session.currentViewportName]
        try:
            odbName = vp.odbDisplay.name
            o = session.odbs[odbName]
            vpTmp = vp.odbDisplay.display.plotState[0]
            vp.odbDisplay.display.setValues(plotState=abaqusConstants.NONE)
            vp.odbDisplay.display.setValues(plotState=vpTmp)
        except:
            AFXForm.deactivate(self)

        AFXForm.deactivate(self)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        # Make sure that one of setNames or nodesPicked is not empty
        #
        mainWindow = getAFXApp().getAFXMainWindow()
        commandString=AFXForm.getCommandString(self)

        # Either picking must be done, or nodeset must be provided
        #
        if self.nodesPickedKw.getValue() or self.setNameKw.getValues():
            pass
        else:
            st = 'Please select nodes either from viewport or from sets.'
            showAFXErrorDialog(mainWindow,st)
            return 0

        # Make sure that one of the RF's is checked
        #
        if self.rf1Kw.getValue() or self.rf2Kw.getValue() or self.rf3Kw.getValue() or self.rfmKw.getValue():
            pass
        else:
            showAFXErrorDialog(mainWindow,'Please choose one of the Reaction Forces for output.')
            return 0

        # Need to see if there is only one instance nodes picked
        #
        if len(commandString.split("instances"))>2:
            showAFXErrorDialog(mainWindow,'This functionality works only for one '
                                          'instance currently, please pick from one instance only.')
            return 0
        return 1