from abaqusGui import *

###########################################################################
# Class definition
###########################################################################

class SumRFNsetTableDB(AFXDialog):

    numInstance = 0

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        SumRFNsetTableDB.numInstance += 1

        # Construct the base class.
        #
        AFXDialog.__init__(self, title='Select Node Set',opts=DECOR_RESIZE)

        #GUILOG
        getAFXAliasMap().setPrefix(self, 'SumRFTableDB-%d'
                                   % SumRFNsetTableDB.numInstance)
        #ENDGUILOG

        self.form = form
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CLICKED_OK,
            SumRFNsetTableDB.onCmdOk)
        w = self.appendActionButton('OK', self, self.ID_CLICKED_OK)
        getAFXAliasMap().setName(w, 'OK')
        w = self.appendActionButton(self.DISMISS)
        getAFXAliasMap().setName(w, 'Dismiss')
        vf = FXVerticalFrame(p=self,pl=0,pr=0,pb=0,pt=0,opts=FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        self.list = AFXList(p=vf,nvis=10,opts=LAYOUT_FILL_X|LAYOUT_FILL_Y|HSCROLLING_OFF)
        getAFXAliasMap().setName(self.list, 'RF1')

        mainWindow = getAFXApp().getAFXMainWindow()
        self.msg='There is no output database (ODB) \n associated '\
                'with the current viewport.\n\nSelect an ODB from the '\
                'context bar,\nor use "File->Open" to open an ODB.'
        from kernelAccess import session
        try:
            odbName = session.viewports[session.currentViewportName].odbDisplay.name
            o = session.odbs[odbName]
            temp=o.rootAssembly.nodeSets.keys()
            for index in range(len(temp)):
                if temp[index]==' ALL NODES' : # Do not want to have this complication
                    continue
                self.list.appendItem(temp[index])
            for inst in o.rootAssembly.instances.keys():
                for nsetName in o.rootAssembly.instances[inst].nodeSets.keys():
                    setName="%s.%s"%(inst,nsetName)
                    self.list.appendItem(setName)
        except:
            pass

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdOk(self, sender, sel, ptr):

        items = ''
        numItems = self.list.getNumItems()
        for i in range(numItems):
            if self.list.isItemSelected(i):
                items += self.list.getItemText(i)
                items += ','
        self.form.setNameKw.setValues(items[:-1])
        self.hide()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def showModal(self):

        mainWindow = getAFXApp().getAFXMainWindow()
        from kernelAccess import session
        vp=session.viewports[session.currentViewportName]

        # Code below makes sure that XY_PLOT is not in viewport,
        # and an odb exists for the current viewport, then it activates
        # the procedure or else not
        #
        if getDisplayedObjectType() == XY_PLOT:
            if vp.odbDisplay.name:

                # Display the odb
                #
                st = 'vp=session.viewports[session.currentViewportName]\n' \
                     'odbName=vp.odbDisplay.name\nodb1=' \
                     'session.odbs[odbName]\n' \
                     'vp.setValues(displayedObject=odb1)'
                sendCommand(st)
                AFXDialog.showModal(self)
            else:
                showAFXErrorDialog(mainWindow,self.msg)
        elif getDisplayedObjectType() == None: # No odb in viewport
            showAFXErrorDialog(mainWindow,self.msg)
        else:
            AFXDialog.showModal(self)
