from abaqusGui import *
from abq_SumRF.sumRFPickNodesProc import PickElementsProcedure
from abq_SumRF.sumRFSymbConsts import CURRENT_PICKED, FTCURVE_PICKED


###########################################################################
# Class definition
###########################################################################

class SumRFPickDB(AFXDataDialog):
    [
        CREATE_NODESET_DIALOG,
        CLEAR_SELECTION]=range(AFXDataDialog.ID_LAST,AFXDataDialog.ID_LAST+2)

    numInstance = 0

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        SumRFPickDB.numInstance += 1

        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, title='Sum Reaction Forces')

        #GUILOG
        getAFXAliasMap().setPrefix(self, 'SumRF-%d'
                                   % SumRFPickDB.numInstance)
        #ENDGUILOG

        self.form = form
        self.applyBtn = self.appendActionButton(text='Calculate/Plot',tgt=self,sel=self.ID_CLICKED_APPLY)
        getAFXAliasMap().setName(self.applyBtn, 'Calculate/Plot')
        w = self.appendActionButton(text='Dismiss',tgt=self,sel=self.ID_CLICKED_DISMISS)
        getAFXAliasMap().setName(w, 'Dismiss')

        # Group box for Time radio buttons
        #
        gb=FXGroupBox(p=self,text='Time',opts=FRAME_GROOVE|LAYOUT_FILL_X)
        w = FXRadioButton(p=gb,text="Current step/frame",tgt=self.form.timeKw, sel=CURRENT_PICKED.getId())
        getAFXAliasMap().setName(w, 'step rb')
        w = FXRadioButton(p=gb,text="All steps",tgt=self.form.timeKw, sel=FTCURVE_PICKED.getId())
        getAFXAliasMap().setName(w, 'all steps rb')

        # Group box for Force Component
        #
        gb1=FXGroupBox(p=self, text='Force Component',opts=FRAME_GROOVE|LAYOUT_FILL_X)
        self.cb1=FXCheckButton(p=gb1,text="RF1", tgt=self.form.rf1Kw, sel=0)
        self.cb2=FXCheckButton(p=gb1,text="RF2", tgt=self.form.rf2Kw, sel=0)
        self.cb3=FXCheckButton(p=gb1,text="RF3", tgt=self.form.rf3Kw, sel=0)
        self.cbm=FXCheckButton(p=gb1,text="RF magnitude", tgt=self.form.rfmKw, sel=0)

        gb = FXGroupBox(p=self, text='Nodes', opts=FRAME_GROOVE)
        mx = FXMatrix(p=gb, n=2, opts=MATRIX_BY_COLUMNS, x=0,y=0,w=0,h=0, pl=0,pr=0,pt=0,pb=0)

        # Set up Select From Viewport
        #
        w = FXButton(p=mx,
                    text='Select From Viewport',
                    tgt=PickElementsProcedure(self.form.owner, self.form),
                    sel=AFXMode.ID_ACTIVATE, opts=LAYOUT_FILL_X|BUTTON_NORMAL)
        getAFXAliasMap().setName(w, 'Select From Viewport')
        self.sw1 = FXSwitcher(p=mx, opts=0, x=0,y=0,w=0,h=0, pl=0,pr=0,pt=0,pb=0)
        FXLabel(p=self.sw1,text='(None)', opts=JUSTIFY_LEFT)
        FXLabel(p=self.sw1,text='Picked', opts=JUSTIFY_LEFT)
        FXMAPFUNC(self, SEL_COMMAND, self.CREATE_NODESET_DIALOG, SumRFPickDB.createNodeSetDialog)

        # Set up Select From Node Sets
        #
        w = FXButton(p=mx,
                     text='Select From Node Sets',opts=LAYOUT_FILL_X|BUTTON_NORMAL,
                     tgt=self,
                     sel=self.CREATE_NODESET_DIALOG)
        getAFXAliasMap().setName(w, 'Select From Node Sets')
        self.sw2 = FXSwitcher(p=mx, opts=0, x=0,y=0,w=0,h=0, pl=0,pr=0,pt=0,pb=0)
        FXLabel(p=self.sw2,text='(None)', opts=JUSTIFY_LEFT)
        FXLabel(p=self.sw2,text='Selected', opts=JUSTIFY_LEFT)
        FXMAPFUNC(self, SEL_COMMAND, self.CLEAR_SELECTION, SumRFPickDB.onClearSelection)
        w = FXButton(p=gb,text='Clear Selection',tgt=self,sel=self.CLEAR_SELECTION, opts=BUTTON_NORMAL)
        getAFXAliasMap().setName(w, 'Clear')

        #GUILOG
        getAFXAliasMap().setName(self.cb1, 'RF1')
        getAFXAliasMap().setName(self.cb2, 'RF2')
        getAFXAliasMap().setName(self.cb3, 'RF3')
        getAFXAliasMap().setName(self.cbm, 'RFm')
        #ENDGUILOG


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def createNodeSetDialog(self, sender, sel, ptr):
        from abq_SumRF.sumRFNsetTableDB import SumRFNsetTableDB
        self.nsetDB = SumRFNsetTableDB(self.form)
        self.nsetDB.create()
        self.nsetDB.showModal()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onClearSelection(self, sender, sel, ptr):
        self.form.nodesPickedKw.setValue(None)
        self.form.setNameKw.setValues(None)
        sendCommand("abq_SumRF.sumRFUtils.unhighlightNodes()")

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def processUpdates(self):
        if self.form.timeKw.getValue() == CURRENT_PICKED.getId():
            if self.applyBtn.getText() != 'Calculate':
                self.applyBtn.setText('Calculate')
        else:
            if self.applyBtn.getText() != 'Plot':
                self.applyBtn.setText('Plot')
        # To provide feedback if nodes picked or nodeSets selected
        if self.form.nodesPickedKw.getValue():
            self.sw1.setCurrent(1)
        else:
            self.sw1.setCurrent(0)

        if self.form.setNameKw.getValues():
            self.sw2.setCurrent(1)
        else:
            self.sw2.setCurrent(0)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onOpenCloseOdb(self):
        self.onClearSelection(0,0,0)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        unregisterCurrentContext(self.onOpenCloseOdb)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):

        registerCurrentContext(self.onOpenCloseOdb)
        AFXDataDialog.show(self)

