from abaqusGui import *

###########################################################################
# Class definition
###########################################################################

class PickElementsProcedure(AFXProcedure):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, form):

        # Construct the base class.
        #
        AFXProcedure.__init__(self, owner=owner, type=AFXProcedure.SUBPROCEDURE)
        self.form = form

        #GUILOG
        getAFXAliasMap().setPrefix(self, 'PickElem' )
        #ENDGUILOG

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstStep(self):
        self.step1  = AFXPickStep(owner=self,
                               keyword=self.form.nodesPickedKw,
                               prompt='Select the nodes',
                               entitiesToPick=NODES,
                               numberToPick=MANY, highlightLevel=1,
                               sequenceStyle=ARRAY)
        
        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):

        mainWindow = getAFXApp().getAFXMainWindow()
        from kernelAccess import session
        vp=session.viewports[session.currentViewportName]

        # Code below makes sure that XY_PLOT is not in viewport,
        # and an odb exists for the current viewport, then it activates
        # the procedure or else not
        #
        if getDisplayedObjectType() == XY_PLOT:
            if vp.odbDisplay.name:
                
                # Display the odb
                #
                st = 'vp=session.viewports[session.currentViewportName]\n' \
                     'odbName=vp.odbDisplay.name\nodb1=' \
                     'session.odbs[odbName]\n' \
                     'vp.setValues(displayedObject=odb1)'
                sendCommand(st)
                AFXProcedure.activate(self)
            else:
                showAFXErrorDialog(mainWindow,
                    'There is no output database (ODB) \nassociated with '
                    'the current viewport.\n\nSelect an ODB from the '
                    'context bar,\nor use "File->Open" to open an ODB.')
        elif getDisplayedObjectType() == None: # No odb in viewport
            showAFXErrorDialog(mainWindow,
                'There is no output database (ODB) \n associated '
                'with the current viewport.\n\nSelect an ODB from the '
                'context bar,\nor use "File->Open" to open an ODB.')
        else:
            AFXProcedure.activate(self)
