"""
Kernel module for the test module
"""
from abaqus import *
from abaqusConstants import *
from abq_SumRF.sumRFSymbConsts import *
from customKernel import *
import visualization

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def plotRF(curvesToPlot_RF,xyp):
    chartName = xyp.charts.keys()[0]
    xyp.charts[chartName].setValues(curvesToPlot=curvesToPlot_RF)
    session.viewports[session.currentViewportName].setValues(displayedObject=xyp)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def printRF(sumXYDataObjectRF,forceDir):
    if forceDir==3:
        print "   RFM: %s" %sumXYDataObjectRF
    else:
        print "   RF"+str(forceDir+1)+": %s" %sumXYDataObjectRF

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def curveRF(stepOutputRF,curvesToPlot_RF,forceDir):
    xQuantity = visualization.QuantityType(type=TIME)
    yQuantity = visualization.QuantityType(type=FORCE)
    if forceDir==3:
        nameXY_RF = "Sum of RFM for selected nodes"
    else:
        nameXY_RF = "Sum of RF"+str(forceDir+1)+" for selected nodes"
    xy_rf = session.XYData(name=nameXY_RF, data=stepOutputRF,sourceDescription=\
    'sum desired',axis1QuantityType=xQuantity,axis2QuantityType=yQuantity,)
    session.Curve(nameXY_RF, xy_rf)
    curvesToPlot_RF.append(nameXY_RF)
    return(curvesToPlot_RF)
'''
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def timeRF(sumXYDataObjectRF,o,s,f,stepOutputRF,time0):
	timef=o.steps[s].frames[f].frameValue
    time0+=o.steps[s].timePeriod
	if timef < time0
    TempRF=(time0,sumXYDataObjectRF)
    stepOutputRF.append(TempRF)
    return(time0,stepOutputRF)
'''
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def allSteps(o,forceDir,curvesToPlot_RF,nodesLabel):
	time0=0.0
	timef=0.0
	ttime=0.0
	stepOutputRF=[(0,0),]
	flagIndex=0
	dicNode={}
	for s in o.steps.keys():
		ifr = o.steps[s].frames
		nframe = len(ifr)
		for f in range(0,nframe):
			try:
				temp=o.steps[s].frames[f].fieldOutputs['RF'].values
				timef=o.steps[s].frames[f].frameValue
				ttime=timef+time0
			except KeyError:
				return(None)
			for j in range(len(temp)):
				dicNode[temp[j].nodeLabel]=j
			sumXYDataObjectRF=sumXYDataObjectRF1=sumXYDataObjectRF2=sumXYDataObjectRF3=0
			for count in nodesLabel:
				if forceDir==3:
					try:
						sumXYDataObjectRF1+= temp[dicNode[count]].data[0]
						sumXYDataObjectRF2+= temp[dicNode[count]].data[1]
						try:
							sumXYDataObjectRF3+= temp[dicNode[count]].data[2]
						except IndexError:
							sumXYDataObjectRF3=0
					except KeyError:
						msg3= 'No RF Field Output found in the ODB for current'\
							   'node set selection'
						raise ValueError, msg3
				else:
					try:
						sumXYDataObjectRF+= temp[dicNode[count]].data[forceDir]
					except KeyError:
						return(None)
					except IndexError:
						sumXYDataObjectRF=0
			if forceDir==3:
				sumXYDataObjectRFM=(sumXYDataObjectRF1**2+sumXYDataObjectRF2**2+\
				sumXYDataObjectRF3**2)**0.5
				TempRF=(ttime,sumXYDataObjectRFM)
				stepOutputRF.append(TempRF)
			else:
				TempRF=(ttime,sumXYDataObjectRF)
				stepOutputRF.append(TempRF)
		time0+=o.steps[s].timePeriod
	curvesToPlot_RF=curveRF(stepOutputRF,curvesToPlot_RF,forceDir)
	return(curvesToPlot_RF)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def currentStepFrame(o,nodesLabel,forceDir,stepName,frameNumber):
    sumXYDataObjectRF=0
    try:
        temp=o.steps[stepName].frames[frameNumber].fieldOutputs['RF'].values
    except KeyError:
        msg1='No RF'+str(forceDir+1)+ ' Field Output found in'\
                     'the ODB for current node set selection.'
        raise ValueError, msg1
    nodeValues=[]
    dicNode={}
    for j in range(len(temp)):
        dicNode[temp[j].nodeLabel]=j
    try:
        for count in nodesLabel:
            sumXYDataObjectRF+= temp[dicNode[count]].data[forceDir]
    except KeyError:
        msg1='No RF'+str(forceDir+1)+ ' Field Output found in'\
             'the ODB for current node set selection.'
        raise ValueError, msg1

    return(sumXYDataObjectRF)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def unhighlightNodes():

    if "sumRFUtilsTemp" in session.customData.__members__:
        for node in session.customData.sumRFUtilsTemp:
            unhighlight(node)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getRFSubset(timeParamter,rf1=False,rf2=False,rf3=False,rfm=False,
                nodesPicked=None,setNames=[]):
    # Need to change code to allow instance level sets for v6.6, bug in v6.5

    vp = session.viewports[session.currentViewportName]
    odbName = vp.odbDisplay.name
    o = session.odbs[odbName]
    vpTmp = vp.odbDisplay.display.plotState[0]
    vp.odbDisplay.display.setValues(plotState=NONE)
    vp.odbDisplay.display.setValues(plotState=vpTmp)

    mySetName = None
    if nodesPicked:
        mySetBaseName = '_getRFSubset_'
        index = 0
        while mySetBaseName+str(index) in o.rootAssembly.nodeSets.keys():
            index += 1
        mySetName = mySetBaseName+str(index)
        mySet = o.rootAssembly.NodeSet(name=mySetName,nodes=(nodesPicked,))

    # SetName list needed for extracting data
    setNameList = []
    for name in setNames:
        setNameList.append(name)
    if nodesPicked:
        setNameList.append(mySetName)
    stepNameList = o.steps.keys()

    # returns tuple, (current step,current frame)
    stepName1, frameNumber = session.viewports[session.currentViewportName]\
    .odbDisplay.fieldFrame
    if type(stepName1)!=type(""):
        stepName = o.steps.values()[stepName1].name

    nodesList=[]
    for setName in setNameList:
        if '.' in setName:
            inst,nSet=setName.split('.')
            highSet = o.rootAssembly.instances[inst].nodeSets[nSet]
            nodeList=highSet.nodes
        else:
            highSet = o.rootAssembly.nodeSets[setName]
            nodeList=highSet.nodes[0]

        nodeIndex=0
        while nodeIndex < len(nodeList):
            nodesList.append(nodeList[nodeIndex])
            nodeIndex = nodeIndex + 1

    LenNodes=len(nodesList)
    nodesLabel=[]
    for i in range(len(nodesList)):
        nodesLabel.append(nodesList[i].label)
    nodesLabel=list(set(nodesLabel))
    # This check has been put in the activate() method of the form code for the plugin,
    #thus this is not needed here
    # Check if the stepName list has any steps which are perturbation,
    #if there are we issue an error and not let the
    # user to use the plug-in
    #for name in stepNameList:
    #    if o.steps[name].domain == TIME:
    #        if o.steps[name].timePeriod == 0:
    #            raise 'error', 'The plug-in only works for an analysis
    # which does not have any linear perturbation step in it.'

    # This check I remove, as it takes time for large odbs with many steps
    #and frames, and the data extraction is always
    # within a try. If data not found for any reason we raise an error already
    # Check if RF has been requested for the whole analysis as fieldOutput.
    # There is no way to request RF1, RF2, RF3 as field output separately,
    #if RF exists then all of these exist
    # Thus one check is enough below.
    #rf_found = 0
    #for stepName in stepNameList:
    #    for frame in o.steps[stepName].frames:
    #        if 'RF' in frame.fieldOutputs.keys():
    #            rf_found = 1
    #if not rf_found:
    #    raise ValueError,'No RF Field Output found in the ODB.'

    # RFM can be done in two ways for a node set
    # 1) First sum the components separately for each node in node set and
    #    then create a magnitude from the three components
    # For this I need to sum the components separately
    # 2) First find the magnitudes and sum the magnitudes.

    time1 = o.steps[stepName].frames[frameNumber].frameValue
    time0 = o.steps[stepName].totalTime
    currentTime = time0 + time1

    if timeParamter==FTCURVE_PICKED: # For force time curve

        curvesToPlot_RF=[]

        if not session.xyPlots.has_key('XYPlot-1'):
            xyp = session.XYPlot('XYPlot-1')
        else:
            xyp = session.xyPlots['XYPlot-1']

        if rf1:
            forceDir=0
            curvesToPlot_RF=allSteps(o,forceDir,curvesToPlot_RF,nodesLabel)
        if rf2:
            forceDir=1
            curvesToPlot_RF=allSteps(o,forceDir,curvesToPlot_RF,nodesLabel)
        if rf3:
            forceDir=2
            curvesToPlot_RF=allSteps(o,forceDir,curvesToPlot_RF,nodesLabel)
        if rfm:
            forceDir=3
            curvesToPlot_RF=allSteps(o,forceDir,curvesToPlot_RF,nodesLabel)
        plotRF(curvesToPlot_RF,xyp)

    else: # For Current time
        flagNoRF=0
        print "\nReaction Force sums for the selected nodes"
        print "   Step: %s, Frame: %s, Current total time: %s" %(stepName,
        frameNumber,currentTime)
        if rf1:
            forceDir=0
            (sumXYDataObjectRF1)=currentStepFrame(o,nodesLabel,forceDir,
            stepName,frameNumber)
            printRF(sumXYDataObjectRF1,forceDir)
        if rf2:
            forceDir=1
            (sumXYDataObjectRF2)=currentStepFrame(o,nodesLabel,forceDir,
            stepName,frameNumber)
            printRF(sumXYDataObjectRF2,forceDir)
        if rf3:
            forceDir=2
            try:
                (sumXYDataObjectRF3)=currentStepFrame(o,nodesLabel,forceDir,
                                                      stepName,frameNumber)
            except IndexError:
                msg2='No RF3 Field Output found in the ODB for current node'\
                     'set selection.\nThis is a 2D/Axi-symmetric analysis. RF3=0'
                raise ValueError, msg2
            printRF(sumXYDataObjectRF3,forceDir)
        if rfm:
            forceDir=0
            (sumXYDataObjectRF1)=currentStepFrame(o,nodesLabel,forceDir,
            stepName,frameNumber)
            forceDir=1
            (sumXYDataObjectRF2)=currentStepFrame(o,nodesLabel,forceDir,stepName,
            frameNumber)
            forceDir=2
            try:
                (sumXYDataObjectRF3)=currentStepFrame(o,nodesLabel,forceDir,stepName,
                                                  frameNumber)
            except IndexError:
                sumXYDataObjectRF3=0
            forceDir=3
            sumXYDataObjectRFM = (sumXYDataObjectRF1**2+sumXYDataObjectRF2**2+\
            sumXYDataObjectRF3**2)**0.5
            printRF(sumXYDataObjectRFM,forceDir)

    # Unhighlight nodes
    unhighlightNodes()

    for node in nodesList:
        highlight(node)
    session.customData.sumRFUtilsTemp=nodesList
