from abaqusConstants import *
from abaqusGui import *
from kernelAccess import mdb, session
import os

thisPath = os.path.abspath(__file__)
thisDir = os.path.dirname(thisPath)


###########################################################################
# Class definition
###########################################################################

class SymmModGenDB(AFXDataDialog):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        # Construct the base class.
        #

        AFXDataDialog.__init__(self, form, 'Setup Symmetric Model',
            self.OK|self.CANCEL, DIALOG_ACTIONS_SEPARATOR)
            

        okBtn = self.getActionButton(self.ID_CLICKED_OK)
        okBtn.setText('OK')
            
        frame = FXHorizontalFrame(self, 0, 0,0,0,0, 0,0,0,0)

        # Model combo
        # Since all forms will be canceled if the  model changes,
        # we do not need to register a query on the model.
        #
        self.RootComboBox_1 = AFXComboBox(p=frame, ncols=0,
            nvis=1, text='CCL Model:', tgt=form.modelNameKw, sel=0)
        self.RootComboBox_1.setMaxVisible(10)
        self.currentModelName = getCurrentContext()['modelName']
        names = mdb.models.keys()
        names.sort()
        for name in names:
            self.RootComboBox_1.appendItem(name)
        if not form.modelNameKw.getValue() in names:
            form.modelNameKw.setValue(self.currentModelName)
        msgCount = 93
        form.modelNameKw.setTarget(self)
        form.modelNameKw.setSelector(AFXDataDialog.ID_LAST+msgCount)

        self.form = form
        GroupBox_1 = FXGroupBox(p=self, text='Hoop Mesh Bias', opts=FRAME_GROOVE|LAYOUT_FILL_X)
        FXRadioButton(p=GroupBox_1, text='Uniform', tgt=form.GroupBox1Kw1, sel=UNIFORM.getId())
        HFrame_1 = FXHorizontalFrame(p=GroupBox_1, opts=0, x=0, y=0, w=0, h=0,
            pl=30, pr=64, pt=0, pb=0)
        self.spinner=AFXSpinner(HFrame_1, 6,'Elements in hoop direction:', tgt=form.keyword03Kw, sel=0)
        self.spinner.setRange(1,1000)
        self.spinner.setIncrement(1)
        HFrame_2 = FXHorizontalFrame(p=GroupBox_1, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        FXRadioButton(p=HFrame_2, text='Biased', tgt=form.GroupBox1Kw1, sel=BIASED.getId())
        vf = FXVerticalFrame(HFrame_2, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X,
            0,0,0,0, 0,0,0,0)
        # Note: Set the selector to indicate that this widget should not be
        #       colored differently from its parent when the 'Color layout managers'
        #       button is checked in the RSG Dialog Builder dialog.
        vf.setSelector(99)
        self.table = AFXTable(vf, 4, 4, 4, 4, form.keyword04Kw, 0, AFXTABLE_EDITABLE|LAYOUT_FILL_X)
        self.table.setPopupOptions(
            AFXTable.POPUP_CUT|
            AFXTable.POPUP_COPY|
            AFXTable.POPUP_PASTE|
            AFXTable.POPUP_INSERT_ROW|
            AFXTable.POPUP_DELETE_ROW|
            AFXTable.POPUP_CLEAR_CONTENTS|
            AFXTable.POPUP_READ_FROM_FILE|
            AFXTable.POPUP_WRITE_TO_FILE)
        self.table.setLeadingRows(1)
        self.table.setLeadingColumns(1)
        self.table.setColumnWidth(1, 65)
        self.table.setColumnType(1, AFXTable.FLOAT)
        self.table.setColumnWidth(2, 75)
        self.table.setColumnType(2, AFXTable.INT)
        self.table.setColumnWidth(3, 60)
        self.table.setColumnType(3, AFXTable.FLOAT)
        self.table.setLeadingRowLabels('Degrees\tElements\tBias')
        self.table.setStretchableColumn( self.table.getNumColumns()-1 )
        self.table.showHorizontalGrid(True)
        self.table.showVerticalGrid(True)
        HFrame_2 = FXHorizontalFrame(p=self, opts=0, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0)
        frame = AFXVerticalAligner(self, 0, 0,0,0,0, 0,0,0,0)
        Hframe = FXHorizontalFrame(self, 0, 0,0,0,0, 0,0,0,0)
        # Model combo
        # Since all forms will be canceled if the  model changes,
        # we do not need to register a query on the model.
        #
        self.RootComboBox_2 = AFXComboBox(p=frame, ncols=0, nvis=1, text='CAX Model:', tgt=form.modelName2Kw, sel=0)
        self.RootComboBox_2.setMaxVisible(10)
        self.currentModelName = getCurrentContext()['modelName']
        names = mdb.models.keys()
        names.sort()
        for name in names:
            self.RootComboBox_2.appendItem(name)
        if not form.modelName2Kw.getValue() in names:
            if names[-1] != self.currentModelName:
                form.modelName2Kw.setValue( names[-1] )
            else:
                form.modelName2Kw.setValue(names[0])
        msgCount = 94

        form.modelName2Kw.setTarget(self)
        form.modelName2Kw.setSelector(AFXDataDialog.ID_LAST+msgCount)
        msgHandler = str(self.__class__).split('.')[-1] + '.onComboBox_2StepsChanged'
        exec('FXMAPFUNC(self, SEL_COMMAND, AFXDataDialog.ID_LAST+%d, %s)' % (msgCount, msgHandler) )

        # Steps combo
        #
        self.ComboBox_2 = AFXComboBox(p=Hframe, ncols=0, nvis=1, text='Result transfer step:', tgt=form.stepNameKw, sel=0)
        self.ComboBox_2.setMaxVisible(10)
        modelName = self.form.modelName2Kw.getValue()

        # Update the names in the Steps combo
        #
        self.ComboBox_2.clearItems()
        names = mdb.models[modelName].steps.keys()
        for name in names:
            self.ComboBox_2.appendItem(name)
        if names:
            if not self.form.stepNameKw.getValue() in names:
                self.form.stepNameKw.setValue( names[-1] )
        else:
            self.form.stepNameKw.setValue('')
        self.form = form
        self.ComboBox_3 = AFXComboBox(p=Hframe, ncols=0, nvis=1, text='Old job:', tgt=form.jobNameKw, sel=0)
        self.ComboBox_3.setMaxVisible(10)
        
        modelName = self.form.modelName2Kw.getValue()
        # Update the names in the Steps combo
        #
        self.ComboBox_3.clearItems()
        names = mdb.jobs.keys()
        modeljobs = []
        for name in names:
            if mdb.jobs[name].model == modelName:
                self.ComboBox_3.appendItem(name)
                modeljobs.append(name)
        if modeljobs:
            if not self.form.jobNameKw.getValue() in modeljobs:
                self.form.jobNameKw.setValue( modeljobs[-1] )
        else:
            self.form.jobNameKw.setValue('')


    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onComboBox_2StepsChanged(self, sender, sel, ptr):

        self.updateComboBox_2Steps()
        self.updateComboBox_3Jobs()
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateComboBox_2Steps(self):

        modelName = self.form.modelName2Kw.getValue()

        # Update the names in the Steps combo
        #
        self.ComboBox_2.clearItems()
        names = mdb.models[modelName].steps.keys()
        for name in names:
            
            self.ComboBox_2.appendItem(name)
        if names:
            if not self.form.stepNameKw.getValue() in names:
                self.form.stepNameKw.setValue( names[-1] )
        else:
            self.form.stepNameKw.setValue('')

        self.resize( self.getDefaultWidth(), self.getDefaultHeight() )

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateComboBox_3Jobs(self):

        modelName = self.form.modelName2Kw.getValue()
        # Update the names in the Steps combo
        #
        self.ComboBox_3.clearItems()
        names = mdb.jobs.keys()
        modeljobs = []
        for name in names:
            if mdb.jobs[name].model == modelName:
                self.ComboBox_3.appendItem(name)
                modeljobs.append(name)
        if modeljobs:
            if not self.form.jobNameKw.getValue() in modeljobs:
                self.form.jobNameKw.setValue( modeljobs[-1] )
        else:
            self.form.jobNameKw.setValue('')

        self.resize( self.getDefaultWidth(), self.getDefaultHeight() )
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def processUpdates(self):
        """
        """
        # Check the radio button and grey out the alternate option.
        #
        meshdefinition = self.form.GroupBox1Kw1.getValue()
        if meshdefinition == UNIFORM.getId():
            self.disableBiasDef()
        else:
            self.disableUniformDef()
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def disableUniformDef(self):
        """
        """
        self.spinner.disable()
        self.table.enable()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def disableBiasDef(self):
        """
        """
        self.spinner.enable()
        self.table.disable()
