from abaqusGui import *
from abaqusConstants import *
import os


###########################################################################
# Class definition
###########################################################################

class SymmModelGenForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        
        # Construct the base class.
        #
        AFXForm.__init__(self, owner)
        self.radioButtonGroups = {}

        self.cmd = AFXGuiCommand(mode=self, method='writeSymmModelGenParameters',
            objectName='abq_SymmModGenPlugin.symmModGenUtils', registerQuery=False)
        pickedDefault = ''
        self.modelNameKw = AFXStringKeyword(self.cmd, 'cclModelName', True)
        self.GroupBox1Kw1 = AFXSymConstKeyword(self.cmd, 'meshTypeDefinition',True,UNIFORM.getId())
        self.keyword03Kw = AFXIntKeyword(self.cmd, 'numEleminHoopDir', True, 4)
        self.keyword04Kw = AFXTableKeyword(self.cmd, 'elemBiasDefinition', True)
        self.keyword04Kw.setColumnType(0, AFXTABLE_TYPE_FLOAT)
        self.keyword04Kw.setColumnType(1, AFXTABLE_TYPE_INT)
        self.keyword04Kw.setColumnType(2, AFXTABLE_TYPE_FLOAT)
        self.modelName2Kw = AFXStringKeyword(self.cmd, 'caxModelName', True)
        self.stepNameKw = AFXStringKeyword(self.cmd, 'resultTransferStepName', True)
        self.jobNameKw = AFXStringKeyword(self.cmd, 'oldJobName', True)
        self.setModal(TRUE)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        import symmModGenDB
        return symmModGenDB.SymmModGenDB(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        # Try to set the appropriate radio button on. If the user did
        # not specify any buttons to be on, do nothing.
        #
        for kw1,kw2,d in self.radioButtonGroups.values():
            try:
                value = d[ kw1.getValue() ]
                kw2.setValue(value)
            except:
                pass
        return True

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def okToCancel(self):

        # No need to close the dialog when a file operation (such
        # as New or Open) or model change is executed.
        #
        return False

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        if (self.modelNameKw.getValue() == ''):
            showAFXErrorDialog(self.getCurrentDialog(),
                               'You must select a Model.')
            return FALSE
        if (self.stepNameKw.getValue() == ''):
            showAFXErrorDialog(self.getCurrentDialog(),
                               'You must select a Step.')
            return FALSE
        if (self.stepNameKw.getValue() == 'Initial'):
            showAFXErrorDialog(self.getCurrentDialog(),
                               'You must select a Step other than "Initial" Step.')
            return FALSE
        if (self.jobNameKw.getValue() == ''):
            showAFXErrorDialog(self.getCurrentDialog(),
                               'You must select a Job.')
            return FALSE
        if (self.modelNameKw.getValue() == self.modelName2Kw.getValue()):
            showAFXErrorDialog(self.getCurrentDialog(),
                               'CAX model and CCL model cannot be the same.')
            return FALSE
        currentDb = self.getCurrentDialog()
        mainWindow = getAFXApp().getAFXMainWindow()
        if self.GroupBox1Kw1.getValue() == UNIFORM.getId():

            if self.keyword03Kw.getValue()< 0:
                
                showAFXErrorDialog(self.getCurrentDialog(),
                               'You must enter the No. of Elements in Hoop Direction.')
                return FALSE
        if self.GroupBox1Kw1.getValue() == BIASED.getId():
            if self.keyword04Kw.getValues() == '':
                showAFXErrorDialog(self.getCurrentDialog(),
                               'You must enter the Bias definition.')
                return FALSE
            else:
                rowNum = self.keyword04Kw.getNumRows()
                for i in range(rowNum):
                    colNum = self.keyword04Kw.getNumColumns(i)
                    for j in range(colNum):
                        if j != 0:
                            try:
                                item = float(self.keyword04Kw.getValue(i,j))
                                if float(self.keyword04Kw.getValue(i,j)) <= 0.0:
                                    if j == 1:
                                        showAFXErrorDialog(self.getCurrentDialog(),
                                                           'Bias mesh definition is incomplete or incorrect.\n\n "%s" in "Elements" column should be corrected.' % item)
                                        return FALSE
                                    if j == 2:
                                        showAFXErrorDialog(self.getCurrentDialog(),
                                                           'Bias mesh definition is incomplete or incorrect.\n\n "%s" in "Bias" column should be corrected.' % item )
                                        return FALSE
                            except:
                                if j == 1:
                                    showAFXErrorDialog(self.getCurrentDialog(),
                                                       'Bias mesh definition is of incorrect type.\n\n "%s" in "Elements" column should be corrected.' % self.keyword04Kw.getValue(i,j))
                                    return FALSE
                                if j == 2:
                                    showAFXErrorDialog(self.getCurrentDialog(),
                                                       'Bias mesh definition is of incorrect type.\n\n "%s" in "Bias" column should be corrected.' % self.keyword04Kw.getValue(i,j))
                                    return FALSE
                        else:
                            try:
                                item = int(self.keyword04Kw.getValue(i,j))
                                if int(self.keyword04Kw.getValue(i,j)) <= 0:
                                    if j == 1:
                                        showAFXErrorDialog(self.getCurrentDialog(),
                                                           'Bias mesh definition is incomplete or incorrect. \n\n "%s" in "Elements" column should be corrected' % item )

                                    if j == 2:
                                        showAFXErrorDialog(self.getCurrentDialog(),
                                                           'Bias mesh definition is incomplete or incorrect. \n\n "%s" in "Bias" column should be corrected' % item )
                                    return FALSE
                            except:
                                if j == 1:
                                    showAFXErrorDialog(self.getCurrentDialog(),
                                                       'Bias mesh definition is of incorrect type.\n\n "%s" in "Elements" column should be corrected.' % self.keyword04Kw.getValue(i,j))
                                if j == 2:
                                    showAFXErrorDialog(self.getCurrentDialog(),
                                                       'Bias mesh definition is of incorrect type.\n\n "%s" in "Bias" column should be corrected.' % self.keyword04Kw.getValue(i,j))
                                return FALSE
                

                totBiasAngle = 0
                for i in range(rowNum):
                      if float(self.keyword04Kw.getValue(i,0)) > 180.0:
                          showAFXErrorDialog(self.getCurrentDialog(),
                                    'Bias Angle cannot be greater than 180 deg')
                          return FALSE
                      totBiasAngle += float(self.keyword04Kw.getValue(i,0))
                if not (totBiasAngle == 180.0):
                    print "inside"
                    showAFXErrorDialog(self.getCurrentDialog(),
                                       'Bias Angle should add up to 180 deg')
                    return FALSE

        return AFXForm.verifyKeywordValues(self)

from sessionGui import FileToolsetGui

###########################################################################
# Class definition
###########################################################################

class AxiFrom3DGui(AFXToolsetGui):

    [   ID_CALL2,
        ID_CALL]=range(AFXModuleGui.ID_LAST,AFXModuleGui.ID_LAST + 2)
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self):

        # Construct the CAE file toolset.
        #

        AFXToolsetGui.__init__(self,'AxiFrom3D')
        self.group = AFXToolbarGroup(owner=self,
            title="Symmetric Model Generation Workflow")
        (head, tail) = os.path.split(__file__)
        fileLoadPath = os.path.join(head, 'CAX.png')
        createIcon = afxCreateIcon(fileLoadPath)
        createIcon.scale(24,24)
        self.save = AFXToolButton(self.group,
            '\tCreate Symmetric \nModel', 
            createIcon,tgt=SymmModelGenForm(self), sel=AFXForm.ID_ACTIVATE)

        # Set the prefix of the menu pane
        #
        getAFXAliasMap().setPrefix(self.group, 'CAX TO CCL GROUP')
        
        # Name the component test button
        #
        getAFXAliasMap().setName(self.save, 'CAX TO CCL')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getKernelInitializationCommand(self):
        if not getAFXApp().isProductViewer():
            try:
                return 'import abq_SymmModGenPlugin.symmModGenUtils'
            except:
                return ''
            
        else:
            return ''

