from abaqusGui import *
from abq_Vcct.vcctConstants import *

###########################################################################
# Class definition
###########################################################################

class ContactClearanceDB(AFXDataDialog):

    numInstance = 0

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        ContactClearanceDB.numInstance+=1

        # Value to use in init
        #
        from kernelAccess import session, mdb
        textboxwidth = 7
        title='Create Contact Clearance'

        # Set form to self member and initialize dialog
        #
        self.form = form
        self.showWarning = True
        AFXDataDialog.__init__(self, form, title, self.OK|self.CANCEL, DIALOG_ACTIONS_BOTTOM)

        # Set the prefix
        #
        getAFXAliasMap().setPrefix(self, 'vcctContactClearanceDB-%d' % ContactClearanceDB.numInstance)

        # Built widgets
        #
        self.nameText = AFXTextField(self, 0, 'Name: ', form.nameKw, 0, LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.nameText, 'Name')

        # Group box
        #
        gb = FXGroupBox(self, 'Options', LAYOUT_FILL_X|FRAME_GROOVE)

        # Adjust
        #
        self.adjust = FXCheckButton(gb, 'Adjust', form.adjustKw, 0, )
        getAFXAliasMap().setName(self.adjust, 'adjust')

        # Clearance
        #
        self.clearance = AFXTextField(gb, -1, 'Clearance:', form.clearanceKw, 0, AFXTEXTFIELD_FLOAT|LAYOUT_FILL_X, )
        getAFXAliasMap().setName(self.clearance, 'clearance')
        
        # Search
        #
        self.cbSearch = AFXComboBox(gb, 5, 3, 'Search node set:', form.searchNSetKw, 0, LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.cbSearch, 'search')

        self.populateComboBoxes()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):

        from kernelAccess import mdb
        vpName = session.currentViewportName
        model = mdb.models[session.sessionState[vpName]['modelName']]

        # Register query on assembly sets
        #
        model.rootAssembly.sets.registerQuery(self.populateComboBoxes, False)

        # Register query on instances sets
        #
        ikeys = model.rootAssembly.instances.keys()
        for instance in ikeys:
            model.rootAssembly.instances[instance].sets.registerQuery(self.populateComboBoxes, False)

        registerCurrentContext(self.populateComboBoxes)

        AFXDataDialog.show(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        from kernelAccess import mdb
        vpName = session.currentViewportName
        model = mdb.models[session.sessionState[vpName]['modelName']]

        # Unregister query on assembly sets
        #
        model.rootAssembly.sets.unregisterQuery(self.populateComboBoxes)

        # Unregister query on instances sets
        #
        ikeys = model.rootAssembly.instances.keys()
        for instance in ikeys:
            model.rootAssembly.instances[instance].sets.unregisterQuery(self.populateComboBoxes)

        unregisterCurrentContext(self.populateComboBoxes)

        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def populateComboBoxes(self):

        from kernelAccess import mdb
        vpName = session.currentViewportName
        model = mdb.models[session.sessionState[vpName]['modelName']]
        vModel = mdb.customData.vcct.models[session.sessionState[vpName]['modelName']]

        self.cbSearch.clearItems()

        for name in sorted(model.rootAssembly.sets.keys()):
            if len(model.rootAssembly.sets[name].nodes):
                self.cbSearch.appendItem(name)


