from abaqusGui import *
from abq_CaeManagerTemplate.createEditForm import CreateEditForm
from abq_Vcct.contactClearanceDB import ContactClearanceDB
from abq_Vcct.vcctConstants import *
from abaqusConstants import *

###########################################################################
# Class definition
###########################################################################

class ContactClearanceForm(CreateEditForm):
                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, mode):

        self.mode = mode
        CreateEditForm.__init__(self, owner, ContactClearanceDB, mode)

        if mode == 'CREATE':
            self.cmd = AFXGuiCommand(self, 'ContactClearance', 'mdb.customData.vcct.models[%s]')

        self.nameKw = AFXStringKeyword(self.cmd, 'name', self.required)
        self.searchNSetKw = AFXStringKeyword(self.cmd, 'searchNSet', self.required)
        self.adjustKw = AFXBoolKeyword(self.cmd, 'adjust', AFXBoolKeyword.TRUE_FALSE, False, True)
        self.clearanceKw = AFXFloatKeyword(self.cmd, 'clearance', False, 0.0)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        db = self.getCurrentDialog()

        # Check the name.
        #
        if self.mode == 'CREATE':
            name = self.nameKw.getValue().strip()
            if not name:
                showAFXErrorDialog(db, 'A name must be specified.')
                db.nameText.setFocusToTextField()
                return False
            exec('existingNames = %s.keys()' % self.repository)
            existingNames = [ n.lower() for n in existingNames ]
            if name.lower() in existingNames:
                showAFXErrorDialog(self.getCurrentDialog(),
                    '"%s" already exists.\n\n' 
                    'Delete that object first, \n'
                    'or specify a different name.' % name
                )
                return False

        if db.clearance.getText() == '':
            showAFXErrorDialog(db, 'A value for clearance must be specified.')
            db.onKeywordError(self.clearanceKw)
            db.clearance.setFocusToTextField()
            return False

        if db.cbSearch.getText() == '':
            showAFXErrorDialog(db, 'A node set for search must be specified.')
            db.onKeywordError(self.searchNSetKw)
            return False

        return AFXForm.verifyKeywordValues(self)

