from abaqusGui import *
from abaqusConstants import FILE, BOTH, CREATED, MODIFIED
from abq_Vcct.vcctConstants import *

###########################################################################
# Class definition
###########################################################################

class InterfaceDB(AFXDataDialog):

    numInstance = 0

    [
        ID_ANALYSIS,
     ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+1)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):
        
        InterfaceDB.numInstance+=1

        # Value to use in init
        #
        textboxwidth = 13
        title = 'Create Interface'

        # Set form to self member and initialize dialog
        #
        self.form = form
        AFXDataDialog.__init__(self, form, title, self.OK|self.CANCEL,
                               DIALOG_NORMAL|
                               DIALOG_UNPOST_DESTROY|
                               DIALOG_ACTIONS_BOTTOM)

        # Set the prefix
        #
        getAFXAliasMap().setPrefix(self, 'vcctInterfaceDB-%d' % InterfaceDB.numInstance)

        # Built widgets
        #
        self.nameText = AFXTextField(self, 0, 'Name: ', form.nameKw, 0, LAYOUT_FILL_X)

        # Standard/Explicit
        #
        gb = FXGroupBox(self, 'Analysis Type', LAYOUT_FILL_X|FRAME_GROOVE, 0,0,0,0, 0,0,0,0)
        #hf = FXHorizontalFrame(gb, LAYOUT_FILL_X|FRAME_NONE, 0,0,0,0, 0,0,0,0)
        self.radioStandard = FXRadioButton(gb, 'Standard', self, self.ID_ANALYSIS, RADIOBUTTON_NORMAL)
        self.radioExplicit = FXRadioButton(gb, 'Explicit', self, self.ID_ANALYSIS, RADIOBUTTON_NORMAL)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_ANALYSIS, InterfaceDB.updateWidgets)
        self.radioStandard.setCheck(1)

        # Contact
        #
        self.comboBoxI = AFXComboBox(self, 0, 3, 'Contact pair interaction:', form.contactPairNameKw, 0, LAYOUT_FILL_X)

        # InterfaceProperty Tab
        #
        gb = FXGroupBox(self, 'Interface Property', LAYOUT_FILL_X|FRAME_GROOVE)
        self.comboBoxD = AFXComboBox(gb, 0, 3, 'Properties:', form.interfacePropertyKw, 0, LAYOUT_FILL_X)
        self.cbBonding = AFXComboBox(gb, 0, 3, 'Initially bonded Nset:', form.bondNsetKw, 0, LAYOUT_FILL_X)
        self.comboBoxS = AFXComboBox(gb, 0, 3, 'Initiation step: ', form.debondStepNameKw, 0, LAYOUT_FILL_X)
        self.linearScaling = AFXTextField(gb, textboxwidth, 'Linear scaling: ', form.linearScalingKw, 0,
                                          AFXTEXTFIELD_FLOAT|AFXTEXTFIELD_CHECKBUTTON)
        self.outputGb = FXGroupBox(self, 'Crack Propagation Output', LAYOUT_FILL_X|FRAME_GROOVE)
        hf = FXHorizontalFrame(self.outputGb, LAYOUT_FILL_X|FRAME_NONE, 0,0,0,0)
        self.rb1=FXRadioButton(hf, 'DAT', form.crackOutputFileKw, DAT.getId())
        getAFXAliasMap().setName(self.rb1, 'DAT')
        self.rb2=FXRadioButton(hf, 'FIL', form.crackOutputFileKw, FILE.getId())
        getAFXAliasMap().setName(self.rb2, 'FIL')
        self.rb3=FXRadioButton(hf, 'BOTH', form.crackOutputFileKw, BOTH.getId())
        getAFXAliasMap().setName(self.rb3, 'BOTH')
        self.freqText1 = AFXTextField(self.outputGb, textboxwidth, 'Frequency: ', form.crackOutputFrequencyKw, 0, AFXTEXTFIELD_INTEGER)

        # Set alias 
        #
        getAFXAliasMap().setName(self.nameText, 'Name')
        getAFXAliasMap().setName(self.comboBoxI, 'Contact Pair')
        getAFXAliasMap().setName(self.comboBoxD, 'Properties')
        getAFXAliasMap().setName(self.cbBonding, 'Contact bonding')
        getAFXAliasMap().setName(self.comboBoxS, 'Initiation step')
        getAFXAliasMap().setName(self.linearScaling, 'Linear Scaling text')
        getAFXAliasMap().setName(self.freqText1, 'Frequency text')

        # Populate Combo boxes
        #
        self.populateComboBoxes()

        if self.form.mode == 'EDIT':
            from kernelAccess import session, mdb
            vpName = session.currentViewportName
            modelName = session.sessionState[vpName]['modelName']
            model = mdb.models[modelName]
            vModel = mdb.customData.vcct.models[modelName]
            vInterface = vModel.interfaces[self.form.nameKw.getValue()]

            if vInterface.analysisType:
                self.radioExplicit.setCheck(1)
                self.radioStandard.setCheck(0)
                self.updateWidgets(None, None, None)
            elif vInterface.linearScaling:
                self.linearScaling.setCheck(True)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):

        from kernelAccess import mdb
        vpName = session.currentViewportName
        model = mdb.models[session.sessionState[vpName]['modelName']]

        # Register query on interactions
        #
        model.interactions.registerQuery(self.populateComboBoxes, False)

        # Register query on steps
        #
        model.steps.registerQuery(self.populateComboBoxes, False)

        # Register query on assembly sets
        #
        model.rootAssembly.sets.registerQuery(self.populateComboBoxes, False)

        # Register query on instances sets
        #
        ikeys = model.rootAssembly.instances.keys()
        for instance in ikeys:
            model.rootAssembly.instances[instance].sets.registerQuery(self.populateComboBoxes, False)

        registerCurrentContext(self.populateComboBoxes)

        AFXDataDialog.show(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        from kernelAccess import mdb
        vpName = session.currentViewportName
        model = mdb.models[session.sessionState[vpName]['modelName']]

        # Unregister query on interactions
        #
        model.interactions.unregisterQuery(self.populateComboBoxes)

        # Unregister query on steps
        #
        model.steps.unregisterQuery(self.populateComboBoxes)

        # Unregister query on assembly sets
        #
        model.rootAssembly.sets.unregisterQuery(self.populateComboBoxes)

        # Unregister query on instances sets
        #
        ikeys = model.rootAssembly.instances.keys()
        for instance in ikeys:
            model.rootAssembly.instances[instance].sets.unregisterQuery(self.populateComboBoxes)

        unregisterCurrentContext(self.populateComboBoxes)

        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def populateComboBoxes(self):

        from kernelAccess import mdb
        vpName = session.currentViewportName
        model = mdb.models[session.sessionState[vpName]['modelName']]
        vModel = mdb.customData.vcct.models[session.sessionState[vpName]['modelName']]

        self.comboBoxD.clearItems()
        self.comboBoxI.clearItems()
        self.cbBonding.clearItems()
        self.comboBoxS.clearItems()

        for name in sorted(vModel.interfaceProperties.keys()):
            self.comboBoxD.appendItem(name)

        if self.radioExplicit.getCheck():
            for name in sorted(vModel.contactClearances.keys()):
                self.cbBonding.appendItem(name)
        else:
            for name in sorted(model.rootAssembly.sets.keys()):
                self.cbBonding.appendItem(name)

            for instance in sorted(model.rootAssembly.instances.keys()):
                for name in sorted(model.rootAssembly.instances[instance].sets.keys()):
                    self.cbBonding.appendItem(instance + '.' + name)
        self.cbBonding.setText('')
        if self.cbBonding.getNumItems() > 8:
            self.cbBonding.setNumVisible(6)

        modelStep = model.steps['Initial']
        for nameI in sorted(model.interactions.keys()):
            if self.radioExplicit.getCheck():
                self.comboBoxI.appendItem(nameI)
            elif model.interactions[nameI].history[0] == CREATED:
                if hasattr(modelStep.interactionStates[nameI],'interactionProperty'):
                    nameIP = modelStep.interactionStates[nameI].interactionProperty
                    self.comboBoxI.appendItem(nameI)
        self.comboBoxI.setText('')

        keys = model.steps.keys()
        keys.remove('Initial')
        for name in keys:
            self.comboBoxS.appendItem(name)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def updateWidgets(self, sender, sel, ptr):
    
        self.populateComboBoxes()
        if self.radioStandard.getCheck():
            self.comboBoxI.setLabelText('Contact pair interaction:')
            self.cbBonding.setLabelText('Initially bonded Nset:')
            self.linearScaling.enable()
            self.outputGb.enable()
            self.rb1.enable()
            self.rb2.enable()
            self.rb3.enable()
            self.freqText1.enable()
        else:
            self.comboBoxI.setLabelText('Contact interaction:')
            self.cbBonding.setLabelText('Contact clearance:')
            self.linearScaling.disable()
            self.outputGb.disable()
            self.rb1.disable()
            self.rb2.disable()
            self.rb3.disable()
            self.freqText1.disable()
