from abaqusGui import *
from abq_CaeManagerTemplate.createEditForm import CreateEditForm
from abq_Vcct.interfaceDB import InterfaceDB
from abq_Vcct.vcctConstants import DAT
from abaqusConstants import *

###########################################################################
# Class definition
###########################################################################

class InterfaceForm(CreateEditForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, mode):

        self.mode = mode
        CreateEditForm.__init__(self, owner, InterfaceDB, mode)

        if mode == 'CREATE':
            self.cmd = AFXGuiCommand(self, 'Interface', 'mdb.customData.vcct.models[%s]')

        self.nameKw = AFXStringKeyword(self.cmd, 'name', self.required)
        self.contactPairNameKw = AFXStringKeyword(self.cmd, 'contactPairName', True)
        self.analysisTypeKw = AFXBoolKeyword(self.cmd, 'analysisType', AFXBoolKeyword.TRUE_FALSE, True, 0)
        self.interfacePropertyKw = AFXStringKeyword(self.cmd, 'interfaceProperty', True)
        self.bondNsetKw = AFXStringKeyword(self.cmd, 'bondNset', True)
        self.debondStepNameKw = AFXStringKeyword(self.cmd, 'debondStep', True)
        self.linearScalingKw = AFXFloatKeyword(self.cmd, 'linearScaling', True, 0.9)
        self.crackOutputFileKw = AFXSymConstKeyword(self.cmd, 'crackOutputFile', False, DAT.getId())
        self.crackOutputFrequencyKw = AFXIntKeyword(self.cmd, 'crackOutputFrequency', False, 1)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        db = self.getCurrentDialog()

        # Check the name
        #
        if self.mode == 'CREATE':
            name = self.nameKw.getValue().strip()
            if not name:
                showAFXErrorDialog(db, 'A name must be specified.')
                db.nameText.setFocusToTextField()
                return False

        if db.radioStandard.getCheck():
            msg1 = 'A selection for contact pair interaction must be specified.'
            msg2 = 'A selection for initially bonded NSET must be specified.'
            self.analysisTypeKw.setValue(0)
        else:
            msg1 = 'A selection for contact interaction must be specified.'
            msg2 = 'A selection for contact clearance must be specified.'
            self.analysisTypeKw.setValue(1)

        # Check if combo boxes have been selected
        #
        if self.contactPairNameKw.getValue() == '':

            showAFXErrorDialog(db, msg1)
            db.onKeywordError(self.contactPairNameKw)
            return False

        # Interface
        #
        if self.interfacePropertyKw.getValue() == '':
            showAFXErrorDialog(db, 'A selection for interface properties must be specified.')
            db.onKeywordError(self.interfacePropertyKw)
            return False

        # bonding
        #
        if self.bondNsetKw.getValue() == '':
            showAFXErrorDialog(db, msg2)
            db.onKeywordError(self.bondNsetKw)
            return False

        # Step
        #
        if self.debondStepNameKw.getValue() == '':
            showAFXErrorDialog(db, 'A selection for debond initiation step must be specified .')
            db.onKeywordError(self.debondStepNameKw)
            return False

        if db.radioStandard.getCheck():
            if db.linearScaling.getCheck():
                self.linearScalingKw.setRequired(True)
                val = self.linearScalingKw.getValue()
                if val == '' or db.linearScaling.getText() == '':
                    showAFXErrorDialog(db, 'A value for linear scalaing must be specified .')
                    db.onKeywordError(self.linearScalingKw)
                    return False
                if val < 0 or val > 1:
                    showAFXErrorDialog(db, 'A value for linear scalaing must be between and including 0 and 1 .')
                    db.onKeywordError(self.linearScalingKw)
                    return False
            else:
                #self.linearScalingKw.setRequired(False)
                pass

        if self.crackOutputFrequencyKw.getValue() == '' or self.crackOutputFrequencyKw.getValue() < 0 :
            showAFXErrorDialog(db, 'A non-negative value for frequency must be specified.')
            db.onKeywordError(self.crackOutputFrequencyKw)
            return False


        return AFXForm.verifyKeywordValues(self)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        db = self.getCurrentDialog()

        if not db.linearScaling.getCheck():
            self.linearScalingKw.setValue(0)

        return True
        