from abaqusGui import *
from abaqusConstants import NONE
from abq_Vcct.vcctConstants import *

###########################################################################
# Class definition
###########################################################################

class InterfacePropertyDB(AFXDataDialog):

    [ID_SHOWWARNING,] = range(AFXForm.ID_LAST, AFXForm.ID_LAST+1)
    numInstance = 0

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        InterfacePropertyDB.numInstance+=1

        # Value to use in init
        #
        from kernelAccess import session, mdb
        textboxwidth = 7
        title='Create InterfaceProperty'

        # Set form to self member and initialize dialog
        #
        self.form = form
        self.showWarning = True
        AFXDataDialog.__init__(self, form, title, self.OK|self.CANCEL,
                               DIALOG_UNPOST_DESTROY|
                               DIALOG_ACTIONS_BOTTOM|
                               LAYOUT_MIN_WIDTH|
                               LAYOUT_MIN_HEIGHT)

        # Set the prefix
        #
        getAFXAliasMap().setPrefix(self, 'vcctInterfacePropertyDB-%d' % InterfacePropertyDB.numInstance)

        FXMAPFUNC(self, SEL_COMMAND, self.ID_SHOWWARNING, InterfacePropertyDB.onShowWarning)

        # Built widgets
        #
        self.nameText = AFXTextField(self, 0, 'Name: ', form.nameKw, 0, LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.nameText, 'Name')

        # Create Tabs
        #
        tabBookSI1 = FXTabBook(self, None, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0)

        w=FXTabItem(tabBookSI1, 'General Options')
        getAFXAliasMap().setName(w, 'General Tab1')
        vfSI1 = FXVerticalFrame(tabBookSI1, FRAME_RAISED|FRAME_THICK, 0,0,0,0)
        w=FXTabItem(tabBookSI1, 'Properties')
        getAFXAliasMap().setName(w, 'Properties Tab')
        vfSI2 = FXVerticalFrame(tabBookSI1, FRAME_RAISED|FRAME_THICK, 0,0,0,0)

        # General Options Tab
        #
        vaSI1 = AFXVerticalAligner(vfSI1, FRAME_NONE, 0,0,0,0)

        gbSI1 = FXGroupBox(vfSI1, 'Fracture Criterion', LAYOUT_FILL_X|FRAME_GROOVE)
        hfSI1 = FXHorizontalFrame(gbSI1, LAYOUT_FILL_X|FRAME_NONE, 0,0,0,0, 0,0,0,0)
        vfSI3 = FXVerticalFrame(hfSI1, FRAME_NONE, 0,0,0,0, 0,0,0,0)
        self.radio1 = FXRadioButton(vfSI3, 'BK LAW', form.fractureCriterionKw,
                                    BK_LAW.getId(), RADIOBUTTON_NORMAL, 0,0,0,0, 4,4,4,4)
        self.radio2 = FXRadioButton(vfSI3, 'POWER LAW', form.fractureCriterionKw,
                                    POWER_LAW.getId(), RADIOBUTTON_NORMAL, 0,0,0,0, 4,4,4,4)
        self.radio3 = FXRadioButton(vfSI3, 'REEDER LAW', form.fractureCriterionKw,
                                    REEDER_LAW.getId(), RADIOBUTTON_NORMAL, 0,0,0,0, 4,4,4,4)
        vfSI4 = FXVerticalFrame(hfSI1, FRAME_NONE, 0,0,0,0, 0,0,0,0)
        self.texteta1 = AFXTextField(vfSI4, 6, 'eta: ', None, 0, AFXTEXTFIELD_FLOAT)
        hfSI2 = FXHorizontalFrame(vfSI4, FRAME_NONE, 0,0,0,0, 2,0,0,0)
        self.textm = AFXTextField(hfSI2, 6, '  m: ', form.mKw, 0, AFXTEXTFIELD_FLOAT)
        self.textn = AFXTextField(hfSI2, 6, 'n: ', form.nKw, 0, AFXTEXTFIELD_FLOAT)
        self.texto = AFXTextField(hfSI2, 6, 'o: ', form.oKw, 0, AFXTEXTFIELD_FLOAT)
        self.texteta2 = AFXTextField(vfSI4, 6, 'eta: ', None, 0, AFXTEXTFIELD_FLOAT)

        FXLabel(vfSI1, '')

        vaSI2 = AFXVerticalAligner(vfSI1, FRAME_NONE, 0,0,0,0, 0,0,0,0)
        self.textvisco = AFXTextField(vaSI2, textboxwidth, 'Viscosity coefficient: ', form.damvKw, 0, LAYOUT_FILL_X)
        self.texttol = AFXTextField(vaSI2, textboxwidth, 'Cutback tolerance: ', form.tolKw, 0, LAYOUT_FILL_X)

        # Set alias (1st tab)
        #
        getAFXAliasMap().setName(self.radio1, 'BK LAW radio')
        getAFXAliasMap().setName(self.radio2, 'POWER LAW radio')
        getAFXAliasMap().setName(self.radio3, 'REEDER LAW radio')
        getAFXAliasMap().setName(self.texteta1, 'eta1 text')
        getAFXAliasMap().setName(self.textm, 'm text')
        getAFXAliasMap().setName(self.textn, 'n text')
        getAFXAliasMap().setName(self.texto, 'o text')
        getAFXAliasMap().setName(self.texteta2, 'eta2 text')
        getAFXAliasMap().setName(self.textvisco, 'visco text')
        getAFXAliasMap().setName(self.texttol, 'tol text')

        # Properties Tab
        #
        hfSI3 = FXHorizontalFrame(vfSI2, FRAME_NONE|LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        gbSI2 = FXGroupBox(hfSI3, '', FRAME_NONE, 0,0,0,0, 0,0,0,0)

        self.spatiallySwitcher = FXSwitcher(vfSI2)
        self.radio4 = FXRadioButton(gbSI2, 'Spatially independent', self.spatiallySwitcher, FXSwitcher.ID_OPEN_FIRST)
        self.radio5 = FXRadioButton(gbSI2, 'Spatially dependent',   self.spatiallySwitcher, FXSwitcher.ID_OPEN_FIRST+1)

        gbSI5 = FXGroupBox(vfSI2, 'Critical Strain Energy Release Rate (GC)', LAYOUT_FILL_X|FRAME_GROOVE, 0,0,0,0)
        hfSI4 = FXHorizontalFrame(gbSI5, FRAME_NONE|LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        self.textGI = AFXTextField(hfSI4, textboxwidth, 'Mode I: ', form.G_ICKw, 0, AFXTEXTFIELD_FLOAT)
        self.textGII = AFXTextField(hfSI4, textboxwidth, 'Mode II: ', form.G_IICKw, 0, AFXTEXTFIELD_FLOAT)
        self.textGIII = AFXTextField(hfSI4, textboxwidth, 'Mode III: ', form.G_IIICKw, 0, AFXTEXTFIELD_FLOAT)

        # Empty frame using FXSwitcher
        #
        self.vfSI3 = FXVerticalFrame(self.spatiallySwitcher, FRAME_SUNKEN|FRAME_THICK, 0,0,0,0, 0,0,0,0)
        FXLabel(self.vfSI3, '')

        # Frame with AFXTable using FXSwitcher
        #
        self.vfSI3 = FXVerticalFrame(self.spatiallySwitcher, FRAME_SUNKEN|FRAME_THICK, 0,0,0,0, 0,0,0,0)

        self.tableSI1 = AFXTable(self.vfSI3, 4, 5, 2, 5, self.form.tblCriticalEnergySetsKw, 0,
                                 AFXTABLE_NO_COLUMN_SELECT|
                                 AFXTABLE_EDITABLE|
                                 LAYOUT_FILL_Y)
        self.tableSI1.setLeadingRows(1)
        self.tableSI1.showHorizontalGrid(TRUE)
        self.tableSI1.showVerticalGrid(TRUE)
        self.tableSI1.setLeadingRowLabels('\tNode ID\tMode I\tMode II\tMode III')
        self.tableSI1.setLeadingColumns(1)
        self.tableSI1.setColumnJustify( -1, AFXTable.CENTER)
        self.tableSI1.setColumnType( -1, AFXTable.FLOAT)
        self.tableSI1.setColumnType( 1, AFXTable.INT)
        self.tableSI1.setColumnWidth(-1, 70)
        self.tableSI1.setEmptyItemPolicy(AFXTable.DISALLOW_EMPTY)
        self.tableSI1.setPopupOptions(AFXTable.POPUP_CUT|
                                      AFXTable.POPUP_COPY|
                                      AFXTable.POPUP_PASTE|
                                      AFXTable.POPUP_INSERT_ROW|
                                      AFXTable.POPUP_DELETE_ROW|
                                      AFXTable.POPUP_CLEAR_CONTENTS|
                                      AFXTable.POPUP_READ_FROM_FILE)

        # Set alias (2nd tab)
        #
        getAFXAliasMap().setName(self.radio4, 'independent radio')
        getAFXAliasMap().setName(self.radio5, 'dependent radio')
        getAFXAliasMap().setName(self.textGI, 'GI text')
        getAFXAliasMap().setName(self.textGII, 'GII text')
        getAFXAliasMap().setName(self.textGIII, 'GIII text')
        getAFXAliasMap().setName(self.tableSI1, 'table')


        # Widget Properties
        #
        # Fracture Criterion Group
        #
        self.texteta1.disable()
        self.textm.disable()
        self.textn.disable()
        self.texto.disable()
        self.texteta2.disable()

        # Number Of Properties Radio and set Table
        #
        if self.form.mode == 'EDIT':
            vpName = session.currentViewportName
            modelName = session.sessionState[vpName]['modelName']
            model = mdb.models[modelName]
            vModel = mdb.customData.vcct.models[modelName]
            vInterfaceProperty = vModel.interfaceProperties[self.form.nameKw.getValue()]
            if vInterfaceProperty.fractureCriterion == BK_LAW:
                self.texteta1.setText(str(vInterfaceProperty.eta))
            elif vInterfaceProperty.fractureCriterion == REEDER_LAW:
                self.texteta2.setText(str(vInterfaceProperty.eta))
            if vInterfaceProperty.criticalEnergySets != NONE:
                self.spatiallySwitcher.setCurrent(1)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onShowWarning(self, sender, sel, ptr):

        self.showWarning = sender.getCheckButtonState()
        return True

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def processUpdates(self):

        # Fracture Criteria radio buttons
        #
        if self.form.fractureCriterionKw.getValue() == BK_LAW.getId():
            self.texteta1.enable()
            self.textm.disable()
            self.textn.disable()
            self.texto.disable()
            self.texteta2.disable()
            self.form.etaKw.setRequired(True)
            self.form.mKw.setRequired(False)
            self.form.nKw.setRequired(False)
            self.form.oKw.setRequired(False)
        elif self.form.fractureCriterionKw.getValue() == POWER_LAW.getId():
            self.texteta1.disable()
            self.textm.enable()
            self.textn.enable()
            self.texto.enable()
            self.texteta2.disable()
            self.form.etaKw.setRequired(False)
            self.form.mKw.setRequired(True)
            self.form.nKw.setRequired(True)
            self.form.oKw.setRequired(True)
        elif self.form.fractureCriterionKw.getValue() == REEDER_LAW.getId():
            self.texteta1.disable()
            self.textm.disable()
            self.textn.disable()
            self.texto.disable()
            self.texteta2.enable()
            self.form.etaKw.setRequired(True)
            self.form.mKw.setRequired(False)
            self.form.nKw.setRequired(False)
            self.form.oKw.setRequired(False)

        if self.radio5.getCheck():
            from kernelAccess import session, mdb
            vpName = session.currentViewportName
            modelName = session.sessionState[vpName]['modelName']
            model = mdb.models[modelName]
            if len(model.rootAssembly.instances) > 1:
                if self.showWarning:
                    msg = 'Spatially dependent option is not support for assemblies\n' \
                          'with more than one instance.'
                    showAFXDismissableWarningDialog(
                        getAFXApp().getAFXMainWindow(), msg, self, self.ID_SHOWWARNING)
                self.spatiallySwitcher.setCurrent(0)
