from abaqusGui import *
from abq_CaeManagerTemplate.createEditForm import CreateEditForm
from abq_Vcct.interfacePropertyDB import InterfacePropertyDB
from abq_Vcct.vcctConstants import *
from abaqusConstants import *

###########################################################################
# Class definition
###########################################################################

class InterfacePropertyForm(CreateEditForm):
                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, mode):

        self.mode = mode
        CreateEditForm.__init__(self, owner, InterfacePropertyDB, mode)

        if mode == 'CREATE':
            self.cmd = AFXGuiCommand(self, 'InterfaceProperty', 'mdb.customData.vcct.models[%s]')

        self.nameKw = AFXStringKeyword(self.cmd, 'name', self.required)

        self.G_ICKw = AFXFloatKeyword(self.cmd, 'G_IC', True, FLOAT_DEFAULT)
        self.G_IICKw = AFXFloatKeyword(self.cmd, 'G_IIC', True, FLOAT_DEFAULT)
        self.G_IIICKw = AFXFloatKeyword(self.cmd, 'G_IIIC', True, FLOAT_DEFAULT)
        self.fractureCriterionKw = AFXSymConstKeyword(self.cmd, 'fractureCriterion', True, BK_LAW.getId())
        self.etaKw = AFXFloatKeyword(self.cmd, 'eta', False, 0.0)
        self.mKw = AFXFloatKeyword(self.cmd, 'm', False)
        self.nKw = AFXFloatKeyword(self.cmd, 'n', False)
        self.oKw = AFXFloatKeyword(self.cmd, 'o', False)

        self.damvKw = AFXFloatKeyword(self.cmd, 'damv', False, 0.0)
        self.tolKw = AFXFloatKeyword(self.cmd, 'tol', False, 0.2)

        self.tblCriticalEnergySetsKw = AFXTableKeyword(self.cmd, 'criticalEnergySets', True, 0, -1)
        self.tblCriticalEnergySetsKw.setColumnType(0, AFXTABLE_TYPE_INT)
        self.tblCriticalEnergySetsKw.setColumnType(1, AFXTABLE_TYPE_FLOAT)
        self.tblCriticalEnergySetsKw.setColumnType(2, AFXTABLE_TYPE_FLOAT)
        self.tblCriticalEnergySetsKw.setColumnType(3, AFXTABLE_TYPE_FLOAT)

        self.tblCriticalEnergySetsKw.deactivate()
        self.symCriticalEnergySetsKw = AFXSymConstKeyword(self.cmd, 'criticalEnergySets', True, NONE.getId())



    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        db = self.getCurrentDialog()

        # Check the name.
        #
        if self.mode == 'CREATE':
            name = self.nameKw.getValue().strip()
            if not name:
                showAFXErrorDialog(db, 'A name must be specified.')
                db.nameText.setFocusToTextField()
                return False
            exec('existingNames = %s.keys()' % self.repository)
            existingNames = [ n.lower() for n in existingNames ]
            if name.lower() in existingNames:
                showAFXErrorDialog(self.getCurrentDialog(),
                    '"%s" already exists.\n\n' 
                    'Delete that object first, \n'
                    'or specify a different name.' % name
                )
                return False

        if db.radio1.getCheck():
            if db.texteta1.getText() == "":
                showAFXErrorDialog(db, 'A value for eta must be specified.')
                db.onKeywordError(self.etaKw)
                db.texteta1.setFocusToTextField()
                return False
            self.etaKw.setValue(float(db.texteta1.getText()))
        elif db.radio2.getCheck():
            if db.textm.getText() == "":
                showAFXErrorDialog(db, 'A value for m must be specified.')
                db.onKeywordError(self.mKw)
                db.textm.setFocusToTextField()
                return False
            elif db.textn.getText() == "":
                showAFXErrorDialog(db, 'A value for n must be specified.')
                db.onKeywordError(self.nKw)
                db.textn.setFocusToTextField()
                return False
            elif db.texto.getText() == "":
                showAFXErrorDialog(db, 'A value for o must be specified.')
                db.onKeywordError(self.oKw)
                db.texto.setFocusToTextField()
                return False
        elif db.radio3.getCheck():
            if db.texteta2.getText() == "":
                showAFXErrorDialog(db, 'A value for eta must be specified.')
                db.onKeywordError(self.etaKw)
                db.texteta2.setFocusToTextField()
                return False
            self.etaKw.setValue(float(db.texteta2.getText()))

        if db.textvisco.getText() == "":
            showAFXErrorDialog(db, 'A value for viscosity coefficient must be specified.')
            db.onKeywordError(self.damvKw)
            db.textvisco.setFocusToTextField()
            return False

        if db.texttol.getText() == "":
            showAFXErrorDialog(db, 'A value for cutback tolerance must be specified.')
            db.onKeywordError(self.tolKw)
            db.texttol.setFocusToTextField()
            return False

        if db.textGI.getText() == "" or self.G_ICKw.getValue() < 0:
            showAFXErrorDialog(db, 'A non-negative value for Mode I must be specified.')
            db.onKeywordError(self.G_ICKw)
            db.textGI.setFocusToTextField()
            return False

        if db.textGII.getText() == "" or self.G_IICKw.getValue() < 0:
            showAFXErrorDialog(db, 'A non-negative value for Mode II must be specified.')
            db.onKeywordError(self.G_IICKw)
            db.textGII.setFocusToTextField()
            return False

        if db.textGIII.getText() == "" or self.G_IIICKw.getValue() < 0:
            showAFXErrorDialog(db, 'A non-negative value for Mode III must be specified.')
            db.onKeywordError(self.G_IIICKw)
            db.textGIII.setFocusToTextField()
            return False

        if not db.radio4.getCheck():
            for i in range(self.tblCriticalEnergySetsKw.getNumRows()):
                for j in range(4):
                    if j == 0:
                        val = self.tblCriticalEnergySetsKw.getValueAsInt(i,j)[1]
                        if val <= 0 or val == "":
                            showAFXErrorDialog(db, 'A positive value must be specified.')
                            return False
                    else:
                        val = self.tblCriticalEnergySetsKw.getValueAsDouble(i,j)[1]
                        if val < 0 or val == "":
                            showAFXErrorDialog(db, 'A non-negative value must be specified.')
                            return False

        return AFXForm.verifyKeywordValues(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):

        db = self.getCurrentDialog()

        if db.radio5.getCheck():
            self.symCriticalEnergySetsKw.setValue(COLOR.getId()) # Activating a change in the widget
            self.symCriticalEnergySetsKw.deactivate()
            self.tblCriticalEnergySetsKw.activate()
        else:
            self.tblCriticalEnergySetsKw.deactivate()
            self.symCriticalEnergySetsKw.activate()
            self.symCriticalEnergySetsKw.setValue(NONE.getId())
            db.tableSI1.clearContents(1, 1, -1, -1)

        return True