# Input File Reader main

from abaqus import *
from abaqusConstants import NONE
from abq_Vcct.vcctConstants import *
import inpReader, methodCallback,sys

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createVcctModel(callingMethod, args, keywordArgs, userData ):

    print '------- Start of VCCT plug-in import -------'
    if not hasattr(mdb.customData,'vcct'):
        print 'Error: The VCCT class has not been created. Unable to '\
              'parse input file for interface and interface property objects.'
        return

    modelName = keywordArgs['name']
    if not mdb.customData.vcct.models.has_key(modelName):
        vcctModelObject = mdb.customData.vcct.Model(modelName)
        print 'The VCCT custom data model "%s" has been created.' % modelName
    else:
        del mdb.customData.vcct.models[modelName]
        vcctModelObject = mdb.customData.vcct.Model(modelName)
        print 'The VCCT custom data model "%s" has been created.' % modelName

    # Need to set object in viewport since object constructors expect 
    # model to be displayed in viewport
    #
    a = mdb.models[modelName].rootAssembly
    vp = session.currentViewportName
    session.viewports[vp].setValues(displayedObject=a)
    
    readInputFile(modelName, keywordArgs['inputFileName'])

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createFracCritAndNodalEnergyRate(mdb, kw, idx, nodalEnergyRates, modelName):

    iPSuffix = 'InterfaceProperty'
    criticalEnergySets = []
    eta, m, n, o = [0 for i in range(4)]
    iPName = ''
    if kw.suboption[idx].parameter.has_key('type'):
        if kw.suboption[idx].parameter['type'].lower() == 'vcct':

            # Set kw values
            #
            eta = kw.suboption[idx].data[0][3]
            if kw.suboption[idx].parameter.has_key('mixedmodebehavior'):
                if kw.suboption[idx].parameter['mixedmodebehavior'].lower() == 'bk':
                    fractureCriterion = BK_LAW
                elif kw.suboption[idx].parameter['mixedmodebehavior'].lower() == 'reeder':
                    fractureCriterion = REEDER_LAW
                else:
                    fractureCriterion = POWER_LAW
                    m = kw.suboption[idx].data[0][3]
                    n = kw.suboption[idx].data[0][4]
                    o = kw.suboption[idx].data[0][5]
            else:
                fractureCriterion = BK_LAW # BK_LAW by default
            if kw.parameter.has_key('viscosity'):
                damv = kw.parameter['viscosity']
            else:
                damv = 0.0 # Default
            if kw.parameter.has_key('tolerance'):
                tol = kw.parameter['tolerance']
            else:
                tol = 0.2 # Default

            # Nodal Energy Rate
            #
            if len(nodalEnergyRates):
                for d in nodalEnergyRates[-1].data: # Take the first one inserted
                    try:
                        first = float(d[0])
                    except:
                        first = None
                    if first:
                        criticalEnergySets.append((
                            float(d[0]), float(d[1]), float(d[2]), float(d[3])) )
                nodalEnergyRates.pop()
                G_IC, G_IIC, G_IIIC = 0, 0, 0
                criticalEnergySets = tuple(criticalEnergySets)
            else:
                G_IC = kw.suboption[idx].data[0][0]
                G_IIC = kw.suboption[idx].data[0][1]
                G_IIIC = kw.suboption[idx].data[0][2]
                if not G_IC:
                    G_IC, G_IIC, G_IIIC = 0, 0, 0
                criticalEnergySets=NONE


            # Set default InterfaceProperty name
            #
            i = 1 + len(mdb.customData.vcct.models[modelName].interfaceProperties)
            iPName = '%s-%d' % (iPSuffix, i)
            mdb.customData.vcct.models[modelName].InterfaceProperty(
                iPName, G_IC, G_IIC, G_IIIC, 
                fractureCriterion, eta, m, n, o,
                damv,                              # damv
                tol,                               # tol
                criticalEnergySets,
                )

    return criticalEnergySets, nodalEnergyRates, iPName


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def readInputFile(modelName, inputFileLocation):

        iPSuffix = 'InterfaceProperty'
    #try:
        inputFile = inpReader.InputFile(inputFileLocation)

        initConditions, interfaceProperties, nodalEnergyRates, contClearances, \
            surfInters, iPStandard =  [], [], [], [], [], []
        linearScaling = None

        # Parse input file and collect various keywords info
        #
        for kw in inputFile.keyword:
            if kw.name == 'initialconditions':
                initConditions.append(kw)
            if kw.name == 'part':
                nodalEnergyRates = [suboption for suboption in kw.suboption 
                                              if suboption.name == 'nodalenergyrate']
            if kw.name == 'contactclearance':
                contClearances.append(kw)
            if kw.name == 'surfaceinteraction':
                surfInters.append(kw)
            if kw.name == 'step':
                for suboption in kw.suboption:
                    if suboption.name == 'debond':
                        iPStandard.append((suboption,kw.parameter['name']))
                    if suboption.name == 'contact':
                        iPExplicit = [kw.parameter['name'] for subsuboption in suboption.suboption
                                                           if subsuboption.name == 'contactclearanceassignment']
                if len(iPStandard):
                    interfaceProperties.append(tuple(iPStandard)[0])
                for suboption in kw.suboption:
                    if suboption.name == 'controls':
                        if suboption.parameter.has_key('type'):
                            if suboption.parameter['type'].lower() == 'vcctlinearscaling':
                                linearScaling = suboption.data[0][0]

        counter = 1
        interfaceNameSuffix = 'Interface'
        criticalEnergySets = []
        nodalEnergyRates.reverse()

        # Remove duplicates from the interface property list
        #
        noDupes=[]
        [noDupes.append(i) for i in interfaceProperties if not noDupes.count(i)]
        interfaceProperties = noDupes
        #interfaceProperties = [x for x in interfaceProperties if x[0] not in [y[0] for y in locals()["_[1]"]] ]

        # Model objects
        #
        vModel = mdb.customData.vcct.models[modelName]
        model = mdb.models[modelName]

        # EXPLICIT
        #
        # Contact
        #
        # Due to limitation of Interacton API, we cannot uniquely identify the contact object
        # We take the first one (if any) and throw a warning
        #
        if len(model.interactions.keys()):
            contactPair = model.interactions.values()[0].name

            # Fracture criteria 
            #
            # If contactPropertyAssignments, excludedPairs, assume to be GENERAL CONTACT and
            # explicit analysis.
            #
            for surfInter in surfInters:
                for so in surfInter.suboption:
                    if so.name == 'fracturecriterion':
                        criticalEnergySets, nodalEnergyRates, interfacePropertyDefaultName = \
                            createFracCritAndNodalEnergyRate(mdb, surfInter, 1, nodalEnergyRates, modelName)

                        if len(contClearances) and len(iPExplicit):
                            # Set default Interface name
                            #
                            defaultName = '%s-%d' % (interfaceNameSuffix, 1) # Only one created since no iteration is done
                            vModel.Interface(
                                defaultName,
                                contactPair,                        # General contact interaction
                                True,                               # Analysis type, Explicit
                                interfacePropertyDefaultName,       # Interface Property
                                contClearances[0].parameter['name'],# Cannot check which one, so take first one
                                iPExplicit[0],                      # Step - Cannot check which one, so take first one
                                None,                               # Linear scaling
                                DAT,
                                1,)

        # Contact clearance objects
        #
        for contClearance in contClearances:

            # Default values
            #
            adjust = True
            clearance = 0.0
            if 'searchnset' in contClearance.parameter.keys():
                searchnset = contClearance.parameter['searchnset']
                defaultName = contClearance.parameter['name']
                if 'clearance' in contClearance.parameter.keys():
                    clearance = contClearance.parameter['clearance']
                if 'adjust' in contClearance.parameter.keys():
                    if contClearance.parameter['adjust'] == NO:
                        adjust = False
                vModel.ContactClearance(defaultName, searchnset, adjust, clearance, )
                
        # STANDARD
        #
        for interfacePropertyTuple in interfaceProperties:

            interfaceProperty = interfacePropertyTuple[0]

            # Check if interfaceProperty slave and master names are in any of contact pair objects
            #
            for contactPair in model.interactions.items():

                # Check if slave or master members exist.  If so, assume to be CONTACT PAIR and 
                # standard analysis.
                #
                if interfaceProperty.parameter['slave'] == contactPair[1].slave[0] and \
                   interfaceProperty.parameter['master'] == contactPair[1].master[0]:

                    # Check if interaction property exists in any surface interaction
                    #
                    for surfaceInteraction in model.interactionProperties.items():
                        for interactionState in model.steps[interfacePropertyTuple[1]].interactionStates.items():
                            if interactionState[1].interactionProperty == surfaceInteraction[0]:

                                # Check if Initial Condition matches
                                #
                                for initCondition in initConditions:
                                    if len(initCondition.data[0]) == 3:
                                        if interfaceProperty.parameter['slave'] == initCondition.data[0][0] and \
                                           interfaceProperty.parameter['master'] == initCondition.data[0][1]:

                                            # Fracture Criteria should be first option
                                            #
                                            if interfaceProperty.suboption[0].name.lower() == 'fracturecriterion':

                                                    # In standard, the suboption for fracture criteria is the first index
                                                    #
                                                    criticalEnergySets, nodalEnergyRates, interfacePropertyDefaultName = \
                                                        createFracCritAndNodalEnergyRate(mdb, interfaceProperty, 0,
                                                        nodalEnergyRates, modelName)

                                                    # Set default Interface name
                                                    #
                                                    defaultName = '%s-%d' % (interfaceNameSuffix, counter)
                                                    vModel.Interface(
                                                        defaultName,
                                                        contactPair[0],                 # Contact Pair interaction
                                                        False,                          # Analysis type - Standard
                                                        interfacePropertyDefaultName,   # Interface Property
                                                        initCondition.data[0][2],       # Initially Nset
                                                        interfacePropertyTuple[1],      # Step
                                                        linearScaling,                  # Linear scaling
                                                        DAT,
                                                        1,)
                                                    counter+=1
        if len(vModel.interfaceProperties):
            print '*FRACTURECRITERION keyword has been read by VCCT plug-in'
            if criticalEnergySets != NONE:
                print '*NODALENERGYRATE keyword has been read by VCCT plug-in'

        if len(vModel.interfaces):
            print '*DEBOND keyword has been read by VCCT plug-in'
            print '*INITIALCONDITIONS, TYPE=CONTACT keyword has been read by VCCT plug-in'

        if len(vModel.contactClearances):
            print '*CONTACT CLEARANCE keyword has been read by VCCT plug-in'
            print '*CONTACT CLEARANCE ASSIGNMENT keyword has been read by VCCT plug-in'
            print 'Please check attributes of contact clearance objects. Due to API limitations, '\
                  'objects may not have been properly created.'
        print '------- End of VCCT plug-in import -------'

"""    except AbaqusException, msg:
        if 'NODALENERGYRATE' in msg:
            print AbaqusException, 'Error reading Nodal Energy Rate.\n'\
                                   'The keyword is misplaced.\n'\
                                   'Please contact Support for details.'
            print '------- End of VCCT plug-in import -------'
        else:
            print AbaqusException, 'An unknown error has occured.\n'\
                                   'Please contact Support for details.'
    except:
        print AbaqusException, 'An unknown error has occured.\n'\
                               'Please contact Support for details.'"""

methodCallback.addCallback(caller=mdb, methodName='ModelFromInputFile', callback=createVcctModel, callAfter=True)


