from abaqusGui import *
from abaqusConstants import NODES
from abq_CaeManagerTemplate.createEditProcedure import CreateEditProcedure

###########################################################################
# Class definition
###########################################################################

class CreateSetProc(CreateEditProcedure):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, mode):

        CreateEditProcedure.__init__(self, owner, mode)

        # Create for nodes for default
        # in getNextStep, make modifications if element
        #
        if mode == 'CREATE':
            self.cmd = AFXGuiCommand(self, 'NodeSet',
            'session.odbs[%s].rootAssembly')

        self.nameKw = AFXStringKeyword(self.cmd, 'name', True)
        self.nodesKw = AFXObjectKeyword(self.cmd, 'nodes', True)
        self.elementsKw = AFXObjectKeyword(self.cmd, 'elements', True)
        self.elementsKw.deactivate()
        self.objsKw = self.nodesKw
        self.pickedType = 'pickedNodes ='

        # Keyword to control radio button and check pick type
        #
        self.typeKw = AFXSymConstKeyword(
            None, 'setType', True, NODES.getId())

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):

        from abq_ViewerSetSupport.createSetDB import CreateSetDB
        self.objectDB = CreateSetDB(self)
        CreateEditProcedure.activate(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getCommandString(self):
    
        cmds = AFXProcedure.getCommandString(self)
        
        # Because of limitation of AFXPickStep (only sends TUPLE or
        # ARRAY, need to edit the command string to put it in correct
        # syntax, then issue commands.
        # In find(), 10 is used arbitrarily, to make sure the second
        # session.odbs is found.
        #
        objsStr = cmds[cmds.rfind(
            self.pickedType) +
            len(self.pickedType) +1 : 
            cmds.find('session.odbs', 10)-1]
        
        cmds = cmds.replace(objsStr, '(' + objsStr.replace('+',',') + ',)')

        return cmds

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getDefaultName(self):
    
        # This method returns the next available default name for a
        # a repository, based on the baseName and number of objects
        # in the repository.
        #
        from kernelAccess import mdb, session
        keys = []
    
        repository = self.cmd.getExpandedObjectName()
        exec('keys += %s.nodeSets.keys()' % repository)
        exec('keys += %s.elementSets.keys()' % repository)
        index = len(keys) + 1
        defaultName = '%s-%d' % (self.baseName, index)
        while defaultName in keys:
            index += 1
            defaultName = '%s-%d' % (self.baseName, index)
        return defaultName

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstStep(self):

        db = self.objectDB
        self.step1 = AFXDialogStep(self, db)
        if self.mode == 'CREATE':
            self.cmd.setKeywordValuesToDefaults()
            self.nameKw.setValue( self.getDefaultName() )
            
        elif self.mode == 'EDIT':
            title = db.getTitle().replace('Create', 'Edit')
            db.setTitle(title)
            db.nameText.setReadOnlyState()
        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getNextStep(self, previousStep):

        # By default, pick step is set to Nodes    
        # Change to Elements if needed
        #
        if previousStep == self.step1:
            entitiesToPick = AFXPickStep.NODES
            prompt='Select nodes from viewport'

            # Set the proper values for nodes or elements
            #
            if self.typeKw.getValueAsString() == 'NODES':
                self.nodesKw.activate()
                self.elementsKw.deactivate()
                self.objsKw = self.nodesKw
                entitiesToPick = AFXPickStep.NODES
                self.cmd.setMethod('NodeSet')
                prompt='Select nodes from viewport'
                self.pickedType = 'pickedNodes ='
            else:
                self.nodesKw.deactivate()
                self.elementsKw.activate()
                self.objsKw = self.elementsKw
                entitiesToPick = AFXPickStep.ELEMENTS
                self.cmd.setMethod('ElementSet')
                prompt='Select elements from viewport'
                self.pickedType = 'pickedElements ='

            # Create the AFXPickStep command string
            #
            self.step2  = AFXPickStep(
                owner=self, keyword=self.objsKw, prompt=prompt,
                entitiesToPick=entitiesToPick, numberToPick=MANY,
                highlightLevel=1, sequenceStyle=ARRAY)
            return self.step2
        else:
            return None