from abaqus import *
from abaqusConstants import *
import visualization
import xyPlot
import time
import customKernel
from abq_contactForce.contactForceSymbConsts import *

#Color coding for each output variable
colorDict = {'CFN1':'Red',
             'CFN2':'Blue',
             'CFN3':'Cyan',
             'CFNM':'Brown',
             'CFT1':'Violet',
             'CFT2':'Green',
             'CFT3':'Orange',
             'CFTM':'Sky blue',
             'CFS1':'Aqua',
             'CFS2':'Spring green',
             'CFS3':'Grey',
             'CFSM':'Yellow'}

txt1 = 'Total force due to contact pressure: '
txt2 = 'Total force due to frictional stress: '
txt3 = 'Total force due to contact pressure and frictional stress: '
txt4 = 'Center of the total force due to contact pressure: '
txt5 = 'Center of the total force due to frictional stress: '
txt6 = 'Center of the total force due to contact pressure and frictional stress: '

collectCFN={}
collectCFS={}
collectCFT={}
dataCFN={}
dataCFS={}
dataCFT={}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Function to plot the arrow and text
def plotVector(startA,endA,color,o,frameNumber,vp,Force,hVar,name,stepName):

    arrow=o.userData.Arrow(color=color,name='contactAntnPlugin'+hVar+'-'+str(stepName)+str(frameNumber),startAnchor=startA,endAnchor=endA,startHeadStyle=FILLED_CIRCLE,lineThickness=MEDIUM)
    vp.plotAnnotation(annotation=arrow)
    t = o.userData.Text(name='contactAntnPlugin'+'Text-'+hVar+'-'+str(stepName)+str(frameNumber), text=str(name)+" "+str(Force),anchor=endA, color="Yellow")
    vp.plotAnnotation(annotation=t)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Function to close the animation incase of failures
def handleAnimation():
    m=session.imageAnimation()
    if m==None :
        pass
    else:
        m.close()

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Function that finds interpolated Y value of history data pair
def interpolate(val, frame):
    iterList = iter(val)
    a = iterList.next()
    b = iterList.next()
    index = 0
    while 1:
        if frame >= a[0] and frame <= b[0]:
            yVal=a[1]+(frame-a[0])*((b[1]-a[1])/(b[0]-a[0]))
            return (yVal, index)
        elif frame - a[0] < 1e-6:
            return (a[1], index)
        elif frame - b[0] < 1e-6:
            return (b[1], index)
        else:
            a=b
            b=iterList.next()
            index += 1

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Function that finds the contact force components 1, 2, and 3 when magnitude is selected
def forceOutputVarMag(o,xnStr,stepName,frameNumber,cfVar,xnVar,flagAnalysis,flagXpl,index):

    if not flagAnalysis:
        (comp1,comp2,comp3)=(cfVar+'1:'+" "+cfVar+'1'+"     "+str(xnStr),
                             cfVar+'2:'+" "+cfVar+'2'+"     "+str(xnStr),
                             cfVar+'3:'+" "+cfVar+'3'+"     "+str(xnStr))
    else:
        if flagXpl == False:
            if cfVar == 'CFN':
                comp1 = txt1 + cfVar+'1'+"     "+str(xnStr)
                comp2 = txt1 + cfVar+'2'+"     "+str(xnStr)
                comp3 = txt1 + cfVar+'3'+"     "+str(xnStr)
            if cfVar == 'CFS':
                comp1 = txt2 + cfVar+'1'+"     "+str(xnStr)
                comp2 = txt2 + cfVar+'2'+"     "+str(xnStr)
                comp3 = txt2 + cfVar+'3'+"     "+str(xnStr)
            if cfVar == 'CFT':
                comp1 = txt3 + cfVar+'1'+"     "+str(xnStr)
                comp2 = txt3 + cfVar+'2'+"     "+str(xnStr)
                comp3 = txt3 + cfVar+'3'+"     "+str(xnStr)
        else:
            if cfVar == 'CFN':
                comp1 = txt1 + cfVar+'1'+" "+str(xnStr)
                comp2 = txt1 + cfVar+'2'+" "+str(xnStr)
                comp3 = txt1 + cfVar+'3'+" "+str(xnStr)
            if cfVar == 'CFS':
                comp1 = txt2 + cfVar+'1'+" "+str(xnStr)
                comp2 = txt2 + cfVar+'2'+" "+str(xnStr)
                comp3 = txt2 + cfVar+'3'+" "+str(xnStr)
            if cfVar == 'CFT':
                comp1 = txt3 + cfVar+'1'+" "+str(xnStr)
                comp2 = txt3 + cfVar+'2'+" "+str(xnStr)
                comp3 = txt3 + cfVar+'3'+" "+str(xnStr)
    try:
        cfn1=xyPlot.XYDataFromHistory(odb=o,outputVariableName=comp1,steps=(stepName,))
        valComp1 = cfn1[index][1] + (frameNumber - cfn1[index][0])*(cfn1[index+1][1]-cfn1[index][1])/(cfn1[index+1][0]-cfn1[index][0])
        cfn2=xyPlot.XYDataFromHistory(odb=o,outputVariableName=comp2,steps=(stepName,))
        valComp2 = cfn2[index][1] + (frameNumber - cfn2[index][0])*(cfn2[index+1][1]-cfn2[index][1])/(cfn2[index+1][0]-cfn2[index][0])
        cfn3=xyPlot.XYDataFromHistory(odb=o,outputVariableName=comp3,steps=(stepName,))
        valComp3 = cfn3[index][1] + (frameNumber - cfn3[index][0])*(cfn3[index+1][1]-cfn3[index][1])/(cfn3[index+1][0]-cfn3[index][0])
    except:
        handleAnimation()
        raise ValueError,'Selected contact output variable is not available in the current ODB.\n'\
                         'Please request CFN/CFT/CFS and XN/XT/XS history output variables.'

    return (valComp1,valComp2,valComp3)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Function that finds magnitude of contact force and the components of center of force when components are selected
def forceOutputComp(o,xnStr,stepName,frameNumber,cfVar,xnVar,flagAnalysis,flagXpl):

    if not flagAnalysis:
        (compM,compXN1,compXN2,compXN3)=(cfVar+'M:'+" "+cfVar+'M'+"     "+str(xnStr),
                                         xnVar+'1:'+" "+xnVar+'1'+"      "+str(xnStr),
                                         xnVar+'2:'+" "+xnVar+'2'+"      "+str(xnStr),
                                         xnVar+'3:'+" "+xnVar+'3'+"      "+str(xnStr))
    else:
        if flagXpl == False:
            if cfVar == 'CFN':
                compM = txt1 + cfVar+'M'+"     "+str(xnStr)
                compXN1 = txt4 + xnVar+'1'+"      "+str(xnStr)
                compXN2 = txt4 + xnVar+'2'+"      "+str(xnStr)
                compXN3 = txt4 + xnVar+'3'+"      "+str(xnStr)
            if cfVar == 'CFS':
                compM = txt2 + cfVar+'M'+"     "+str(xnStr)
                compXN1 = txt5 + xnVar+'1'+"      "+str(xnStr)
                compXN2 = txt5 + xnVar+'2'+"      "+str(xnStr)
                compXN3 = txt5 + xnVar+'3'+"      "+str(xnStr)
            if cfVar == 'CFT':
                compM = txt3 + cfVar+'M'+"     "+str(xnStr)
                compXN1 = txt6 + xnVar+'1'+"      "+str(xnStr)
                compXN2 = txt6 + xnVar+'2'+"      "+str(xnStr)
                compXN3 = txt6 + xnVar+'3'+"      "+str(xnStr)
        else:
            if cfVar == 'CFN':
                compM = txt1 + cfVar+'M'+" "+str(xnStr)
                compXN1 = txt4 + xnVar+'1'+" "+str(xnStr)
                compXN2 = txt4 + xnVar+'2'+" "+str(xnStr)
                compXN3 = txt4 + xnVar+'3'+" "+str(xnStr)
            if cfVar == 'CFS':
                compM = txt2 + cfVar+'M'+" "+str(xnStr)
                compXN1 = txt5 + xnVar+'1'+" "+str(xnStr)
                compXN2 = txt5 + xnVar+'2'+" "+str(xnStr)
                compXN3 = txt5 + xnVar+'3'+" "+str(xnStr)
            if cfVar == 'CFT':
                compM = txt3 + cfVar+'M'+" "+str(xnStr)
                compXN1 = txt6 + xnVar+'1'+" "+str(xnStr)
                compXN2 = txt6 + xnVar+'2'+" "+str(xnStr)
                compXN3 = txt6 + xnVar+'3'+" "+str(xnStr)

    try:
        cfm = xyPlot.XYDataFromHistory(odb=o,outputVariableName=compM,steps=(stepName,))
        xn1 = xyPlot.XYDataFromHistory(odb=o,outputVariableName=compXN1,steps=(stepName,))
        xn2 = xyPlot.XYDataFromHistory(odb=o,outputVariableName=compXN2,steps=(stepName,))
        xn3 = xyPlot.XYDataFromHistory(odb=o,outputVariableName=compXN3,steps=(stepName,))
        (valMag, index) = interpolate(cfm, frameNumber)
        xnComp1 = xn1[index][1] + (frameNumber - xn1[index][0])*(xn1[index+1][1]-xn1[index][1])/(xn1[index+1][0]-xn1[index][0])
        xnComp2 = xn2[index][1] + (frameNumber - xn2[index][0])*(xn2[index+1][1]-xn2[index][1])/(xn2[index+1][0]-xn2[index][0])
        xnComp3 = xn3[index][1] + (frameNumber - xn3[index][0])*(xn3[index+1][1]-xn3[index][1])/(xn3[index+1][0]-xn3[index][0])
    except:
        handleAnimation()
        raise ValueError,'Selected contact output variable is not available in the current ODB.\n'\
                         'Please request CFN/CFT/CFS and XN/XT/XS history output variables.'

    return (xnComp1,xnComp2,xnComp3,valMag,index)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Function to determine the unit vector of force
def computeVector(o,stepName,hVar,frameNumber,vp,scale,forceComp, collectCFN, collectCFS, collectCFT,dataCFN, dataCFS, dataCFT):

    hisParam=hVar.split()
    xnStr = ''
    varList = ['CFN1', 'CFN2', 'CFN3', 'CFNM', 'CFS1', 'CFS2', 'CFS3', 'CFSM', 'CFT1', 'CFT2', 'CFT3', 'CFTM']
    for inc in range(len(hisParam)):
        for varType in varList:
            if varType == hisParam[inc]:
                place = inc
                break
    for inc1 in range(place+1, len(hisParam)):
        xnStr += hisParam[inc1]
        xnStr += " "
    xnStr = xnStr[0:-1]

    #Parsing string such as "Total" to see if it belongs to /Explicit analysis and string "surface" to
    #check for the spaces that differs between contact pair and general contact output
    if hisParam[0] == "Total":
        flagAnalysis = True
    else:
        flagAnalysis = False
    if "surface" in hisParam:
        flagXpl = True
    else:
        flagXpl = False

    if forceComp == RESULTANT or forceComp == 1:
        if 'CFNM' in hisParam:
            cfVar='CFN'
            xnVar='XN'
        elif 'CFSM' in hisParam:
            cfVar='CFS'
            xnVar='XS'
        elif 'CFTM' in hisParam:
            cfVar='CFT'
            xnVar='XT'
        (xnComp1,xnComp2,xnComp3,valMag,index)=forceOutputComp(o,xnStr,stepName,frameNumber,cfVar,xnVar,flagAnalysis,flagXpl)
        (valComp1,valComp2,valComp3)=forceOutputVarMag(o,xnStr,stepName,frameNumber,cfVar,xnVar,flagAnalysis,flagXpl,index)
    else:
        if 'CFN1' in hisParam or 'CFN2' in hisParam or 'CFN3' in hisParam:
            cfVar='CFN'
            xnVar='XN'
        elif 'CFS1' in hisParam or 'CFS2' in hisParam or 'CFS3' in hisParam:
            cfVar='CFS'
            xnVar='XS'
        elif 'CFT1' in hisParam or 'CFT2' in hisParam or 'CFT3' in hisParam:
            cfVar='CFT'
            xnVar='XT'
        (xnComp1,xnComp2,xnComp3,valMag,index)=forceOutputComp(o,xnStr,stepName,frameNumber,cfVar,xnVar,flagAnalysis,flagXpl)
        cf=xyPlot.XYDataFromHistory(odb=o,outputVariableName=hVar,steps=(stepName,))
        valComp = cf[index][1] + (frameNumber - cf[index][0])*(cf[index+1][1]-cf[index][1])/(cf[index+1][0]-cf[index][0])

    for var in ['CFN1', 'CFS1', 'CFT1', 'CFN2', 'CFS2', 'CFT2', 'CFN3', 'CFS3', 'CFT3']:
        if var in hisParam:
            try:
                Fi=valComp/valMag
            except:
                Fi=0
            sVar=hVar.split()[1]
            dictVar={}
            dictVar[sVar]=valComp 
            color=colorDict[var]
            if 'CFN1' in hVar or 'CFN2' in hVar or 'CFN3' in hVar:
                collectCFN.setdefault(hVar , []).append(dictVar)
                dataCFN[hVar]=[scale,o,color,frameNumber,vp,valComp,hVar,var,stepName,xnComp1,xnComp2,xnComp3, Fi]
            if 'CFS1' in hVar or 'CFS2' in hVar or 'CFS3' in hVar:
                collectCFS.setdefault(hVar , []).append(dictVar)
                dataCFS[hVar]=[scale, o,color,frameNumber,vp,valComp,hVar,var,stepName,xnComp1,xnComp2,xnComp3, Fi]
            if 'CFT1' in hVar or 'CFT2' in hVar or 'CFT3' in hVar:
                collectCFT.setdefault(hVar , []).append(dictVar)
                dataCFT[hVar]=[scale, o,color,frameNumber,vp,valComp,hVar,var,stepName,xnComp1,xnComp2,xnComp3, Fi]
            break

        
    for var in ['CFNM', 'CFSM', 'CFTM']:
        if var in hisParam:
            try:
                Fi=valComp1/valMag
            except:
                Fi=0

            try:
                Fj=valComp2/valMag
            except:
                Fj=0

            try:
                Fk=valComp3/valMag
            except:
                Fk=0
            startA=(xnComp1,xnComp2,xnComp3)
            endA=(xnComp1+scale*Fi,xnComp2+scale*Fj,xnComp3+scale*Fk)
            color=colorDict[var]
            plotVector(startA,endA,color,o,frameNumber,vp,valMag,hVar,var,stepName)
            break
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
##
def findMaxKey(collect):

    vals = []

    # find all the component values. Add to a list along with the name
    # of the surface and the type. We use the strings to find
    # the value in the data object
    # 
    for key in collect.keys():
        collectValue = collect[key][0]
        vals.append( (abs(collectValue.values()[0]), key ), )
    vals.sort()
      
    # Only need to return key. Last one is the highest value
    # entry
    #
    return vals[-1][1]

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Function to grab the current frame, step, history output variable, compute current step time
#and pass it to computeVector for computing the unit vector
def plotForceKernel(historyVar, scale, largCompScale, aviFileName, forceComp, step, anim,frate):

    vp=session.viewports[session.currentViewportName]
    odbName=vp.odbDisplay.name
    o=session.odbs[odbName]
    stepName1, frameNumber = session.viewports[session.currentViewportName].odbDisplay.fieldFrame
    if type(stepName1)!=type(""):
        stepName = o.steps.values()[stepName1].name
    j = 0
    if vp.annotationsToPlot:
        for i in vp.annotationsToPlot[:]:
            if i.name.startswith('contactAntnPlugin'):
                del vp.annotationsToPlot[j]
            else:
                j += 1
    prevStep = float(o.steps[stepName].totalTime)
    frameVal = prevStep + float(o.steps[stepName].frames[frameNumber].description.split()[-1])

    for hVar in historyVar:
        computeVector(o,stepName,hVar,frameVal,vp,scale,forceComp, collectCFN, collectCFS, collectCFT, dataCFN, dataCFS, dataCFT)

    # Get max for each and apply scale
    #
    if collectCFN.keys():
        key = findMaxKey(collectCFN)
        dataCFN[key][0] = largCompScale

    if collectCFT.keys():        
        key = findMaxKey(collectCFT)
        dataCFT[key][0] = largCompScale
        
    if collectCFS.keys():
        key = findMaxKey(collectCFS)
        dataCFS[key][0] = largCompScale
 
    dataUpdate=dataCFN.update(dataCFS)
    dataUpdate=dataCFN.update(dataCFT)
    combineData= dataCFN.keys()
    for var in combineData:
        for varCF in ['CFN1', 'CFS1', 'CFT1']:
            if varCF in var:
                data=dataCFN[var]
                startA=(data[9],data[10],data[11])
                endA=(data[9]+data[0]*data[12],data[10],data[11])
                plotVector(startA,endA,data[2],data[1],data[3],data[4],data[5],data[6],data[7],data[8])
    for var in combineData:
        for varCF in ['CFN2', 'CFS2', 'CFT2']:
            if varCF in var:
                data=dataCFN[var]
                startA=(data[9],data[10],data[11])
                endA=(data[9],data[10]+data[0]*data[12],data[11])
                plotVector(startA,endA,data[2],data[1],data[3],data[4],data[5],data[6],data[7],data[8])
    for var in combineData:
        for varCF in ['CFN3', 'CFS3', 'CFT3']:
            if varCF in var:
                data=dataCFN[var]
                startA=(data[9],data[10],data[11])
                endA=(data[9],data[10],data[11]+data[0]*data[12])
                plotVector(startA,endA,data[2],data[1],data[3],data[4],data[5],data[6],data[7],data[8])

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Function to loop through frames and compute unit vector of force for each frame to store animation
def saveAnimation(step,vp,historyVar,scale,forceComp,aviFileName,frate,anim,o,movie):
    tmp=o.steps[step]
    prevVal = float(o.steps[step].totalTime)
    frameCount=0
    i=0
    for f in tmp.frames:
        j = 0
        if vp.annotationsToPlot:
            for i in vp.annotationsToPlot[:]:
                if i.name.startswith('contactAntnPlugin'):
                    del vp.annotationsToPlot[j]
                else:
                    j += 1
        frameVal = prevVal + float(f.description.split()[-1])
        for hVar in historyVar:
            computeVector(o,step,hVar,frameVal,vp,int(scale),forceComp,
                collectCFN, collectCFS, collectCFT, dataCFN, dataCFS, dataCFT)
        frameCount+=1
        vp.odbDisplay.setFrame(frame=f)
        movie.writeFrame()
        if anim:
            vp.enableRefresh()
        else:
            pass
        time.sleep(abs(frate))
        flagMovie=True

    return flagMovie

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def closeMovie(flagMovie,movie,aviFileName):
    if flagMovie == True:
        print "Animation has been created successfully in the file "+str('"')+str(aviFileName)+str('"')
    if movie.closed() :
        pass
    else:
        handleAnimation()

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Function called from "Save Animation" button in GUI. Here we decide to loop through one step or all steps in the model
#and then compute vector for each frame for the animation
def generateAvi(historyVar,scale,aviFileName,forceComp,step,frate,anim):

    vp=session.viewports[session.currentViewportName]
    odbName=vp.odbDisplay.name
    o=session.odbs[odbName]
    movie = session.ImageAnimation(fileName=aviFileName, format=AVI)
    flagMovie=False

    if step == 1:
        stepName1, frameNumber = session.viewports[session.currentViewportName].odbDisplay.fieldFrame
        if type(stepName1)!=type(""):
            stepName = o.steps.values()[stepName1].name
        flagMovie=saveAnimation(stepName,vp,historyVar,scale,forceComp,aviFileName,frate,anim,o,movie)
        closeMovie(flagMovie,movie,aviFileName)
    else:
        for step in o.steps.keys():
            flagMovie=saveAnimation(step,vp,historyVar,scale,forceComp,aviFileName,frate,anim,o,movie)

        closeMovie(flagMovie,movie,aviFileName)