from abaqusGui import *
from kernelAccess import *
from abq_contactForce.contactForceSymbConsts import *

###########################################################################
# Class definition
###########################################################################

class ContactForceDB(AFXDataDialog):
    [ID_HisParam,ID_CREATE_AVI, ID_VECTOR_DISPLAY]= range(AFXDataDialog.ID_LAST,AFXDataDialog.ID_LAST+3)
    numInstance = 0 
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):
        ContactForceDB.numInstance += 1
        # Construct the base class.
        #
        self.form = form
        AFXDataDialog.__init__(self, form, 'Plot Contact Force Vector',
            0, opts=DECOR_RESIZE|DIALOG_NORMAL|DIALOG_ACTIONS_SEPARATOR)
        #GUILOG
        getAFXAliasMap().setPrefix(self, 'ContactForce-%d'
                                   % ContactForceDB.numInstance)
        #ENDGUILOG                       

        self.applyBtn = self.appendActionButton(text='Plot',tgt=self,sel=self.ID_CLICKED_APPLY)
        getAFXAliasMap().setName(self.applyBtn, 'plot')
        self.aviBtn = self.appendActionButton(text='Save Animation...',tgt=self,sel=self.ID_CREATE_AVI)
        getAFXAliasMap().setName(self.aviBtn, 'animation')
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CREATE_AVI, ContactForceDB.onGenerateAvi)
        self.dismissBtn = self.appendActionButton(text='Dismiss',tgt=self,sel=self.ID_CLICKED_DISMISS)
        getAFXAliasMap().setName(self.dismissBtn, 'dismiss1')

        FXMAPFUNC(self, SEL_COMMAND, self.ID_HisParam,
        ContactForceDB.onHisParamPicked)

        gb=FXGroupBox(p=self,text='Vector Display',opts=FRAME_GROOVE|LAYOUT_FILL_X)
        hf1 = FXHorizontalFrame(gb, LAYOUT_FILL_X,0, 0, 0, 0, 0, 0, 0, 0)
        self.resultant = FXRadioButton(p=hf1,text="Resultant",tgt=self.form.forceCompKw, sel=RESULTANT.getId())
        getAFXAliasMap().setName(self.resultant, 'reslutant')
        self.component = FXRadioButton(p=hf1,text="Component",tgt=self.form.forceCompKw, sel=COMPONENT.getId())
        getAFXAliasMap().setName(self.component, 'component')
        self.form.forceCompKw.setTarget(self)
        self.form.forceCompKw.setSelector(self.ID_VECTOR_DISPLAY)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_VECTOR_DISPLAY, ContactForceDB.onRadioChanged)

        hf1 = FXHorizontalFrame(gb, LAYOUT_FILL_X,0, 0, 0, 0, 0, 0, 0, 0)
        self.scale = AFXTextField(hf1, 8, 'Scale factor',form.scaleKw, 0,opts=AFXTEXTFIELD_INTEGER)
        getAFXAliasMap().setName(self.scale, 'scale factor')
        self.largCompScale = AFXTextField(hf1, 8, 'Largest component scale factor',form.largCompScaleKw, 0,opts=AFXTEXTFIELD_INTEGER)
        getAFXAliasMap().setName(self.largCompScale, 'large comp scale factor')

        gb1 = FXGroupBox(p=self, text='Force Output Variables', opts=FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        vf1 = FXVerticalFrame(gb1, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y,0, 0, 0, 0, 0, 0, 0, 0)

        self.list1 = AFXList(vf1, 6, self,self.ID_HisParam,LAYOUT_FILL_X|LAYOUT_FILL_Y|
                            LIST_EXTENDEDSELECT|HSCROLLING_OFF,w=50,h=200)
        getAFXAliasMap().setName(self.list1, 'List1')   

        self.onRadioChanged(0, 0 ,0)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onRadioChanged(self,sender,sel,ptr):
        mainWindow = getAFXApp().getAFXMainWindow()
        if self.list1.getNumItems() > 0:
            self.list1.clearItems()
        historyTypeRes=['CFNM','CFSM','CFTM']
        historyTypeComp=['CFT1','CFT2','CFT3','CFN1','CFN2','CFN3','CFS1','CFS2','CFS3']
        self.vp=session.viewports[session.currentViewportName]
        try:
            self.odbName=self.vp.odbDisplay.name
        except:
            msg='There is no output database (ODB) \n'\
                'associated with the current viewport.\n\n'\
                'Select an ODB from the context bar,\n'\
                'or use "File->Open" to open an ODB.'
            showAFXErrorDialog(mainWindow,msg)
            return
        if self.form.forceCompKw.getValue() == RESULTANT.getId():
            isResultant = 1
            self.largCompScale.disable()
        else:
            isResultant = 0
            self.largCompScale.enable()
        for hVar in session.odbData[self.odbName].historyVariables.keys():
            label=hVar.split()
            if isResultant:
                for i in range(3):
                    if historyTypeRes[i] in label:
                        self.list1.appendItem(hVar)
            else:
                for i in range(9):
                    if historyTypeComp[i] in label:
                        self.list1.appendItem(hVar)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onOdbChanged(self):
        self.onRadioChanged(0, 0 ,0)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onHisParamPicked(self,sender,sel,ptr):
        items = ''
        numItems = self.list1.getNumItems()
        for i in range(numItems):
            if self.list1.isItemSelected(i):
                items += self.list1.getItemText(i)
                items += ','
        self.form.historyVarKw.setValues(items[:-1])

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onGenerateAvi(self,sender,sel,ptr):
        from abq_contactForce.fileExportDB import FileExportDB
        self.exportDB = FileExportDB(self)
        self.exportDB.create()
        self.exportDB.showModal()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        unregisterCurrentContext(self.onOdbChanged)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):

        registerCurrentContext(self.onOdbChanged)
        AFXDataDialog.show(self)







