from abaqusGui import *
from kernelAccess import *
import abaqusConstants
from abq_contactForce.contactForceSymbConsts import *

###########################################################################
# Class definition
###########################################################################

class ContactForceForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        self.owner = owner
        AFXForm.__init__(self, owner=owner)
        self.cmd = AFXGuiCommand(self,'plotForceKernel','abq_contactForce.contactForce',registerQuery=False)
        self.historyVarKw =AFXTupleKeyword(command=self.cmd,name='historyVar',isRequired=True,opts=AFXTUPLE_TYPE_STRING)
        self.scaleKw = AFXIntKeyword(self.cmd, 'scale', True, 1)
        self.largCompScaleKw = AFXIntKeyword(self.cmd, 'largCompScale', True, 1)
        self.fileNameKw = AFXStringKeyword(self.cmd, 'aviFileName', True, 'animation')
        self.forceCompKw = AFXSymConstKeyword(command=self.cmd,name='forceComp',isRequired=True,defaultValue=RESULTANT.getId())
        self.stepKw = AFXSymConstKeyword(command=self.cmd,name='step',isRequired=True,defaultValue=CURSTEP.getId())
        self.animationKw = AFXBoolKeyword(command=self.cmd,name='anim',isRequired=True,defaultValue=False)
        self.sliderKw = AFXIntKeyword(self.cmd, 'frate', True,0)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        """Return the first dialog to be posted"""
        self.cmd.setKeywordValuesToDefaults()
        from abq_contactForce.contactForceDB import ContactForceDB
        self.db = ContactForceDB(self)
        return self.db

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):
        mainWindow = getAFXApp().getAFXMainWindow()
        vp=session.viewports[session.currentViewportName]
        anntn=vp.annotationsToPlot
        j = 0
        if vp.annotationsToPlot:
            for i in vp.annotationsToPlot[:]:
                if i.name.startswith('contactAntnPlugin'):
                    del vp.annotationsToPlot[j]
                else:
                    j += 1
        try:
            if vp.odbDisplay.name:
                AFXForm.activate(self)
        except:
            showAFXErrorDialog(mainWindow,'There is no output database (ODB) \n'
                                           'associated with the current viewport.\n\n'
                                           'Select an ODB from the context bar,\n'
                                            'or use "File->Open" to open an ODB.')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):
        mainWindow = getAFXApp().getAFXMainWindow()
        if self.scaleKw.getValue()<=0 or self.scaleKw.getValue() == None:
            showAFXErrorDialog(mainWindow,'Please specify scale factor greater than 0.')
            return 0
        numItems = self.db.list1.getNumItems()

        if numItems == 0:
            showAFXErrorDialog(mainWindow,'No contact output variable available in the current ODB\n'\
                                'Or there is no output database (ODB) associated with the \n'\
                                'current viewport.')
            return 0
        Flag=False
        for i in range(numItems):
            if self.db.list1.isItemSelected(i):
                Flag=True
                break
        if Flag == False:
            showAFXErrorDialog(mainWindow,'Please select at least one force output variable.')
            return 0
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def deactivate(self):
        AFXForm.deactivate(self)