from abaqusGui import *
from abq_contactForce.contactForceSymbConsts import *
import os
###########################################################################
# Class definition
###########################################################################

class FileExportDB(AFXDialog):
    ID_FILE_LOAD,ID_CHECK,ID_OVERWRITE,= range(AFXDialog.ID_LAST,AFXDialog.ID_LAST+3)
    numInstance = 0

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        FileExportDB.numInstance += 1

        # Construct the base class.
        #
        AFXDialog.__init__(self, title='Save Image Animation',opts=DIALOG_NORMAL|DIALOG_ACTIONS_SEPARATOR)

        #GUILOG
        getAFXAliasMap().setPrefix(self, 'FileExportDB-%d'
                                   % FileExportDB.numInstance)
        #ENDGUILOG

        self.form = form
        self.fileDb = None
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CLICKED_APPLY,
                    FileExportDB.onCmdSave)
        self.ok = self.appendActionButton('Save', self, self.ID_CLICKED_APPLY)
        getAFXAliasMap().setName(self.ok, 'save')
        self.dismiss = self.appendActionButton(self.DISMISS)
        getAFXAliasMap().setName(self.dismiss, 'dismiss')

        gb=FXGroupBox(p=self,text='Animation Options',opts=FRAME_GROOVE|LAYOUT_FILL_X)
        hf1 = FXHorizontalFrame(gb, LAYOUT_FILL_X,0, 0, 0, 0, 0, 0, 0, 0)
        self.aStep = FXRadioButton(p=hf1,text="Current Step",tgt=self.form.form.stepKw, sel=CURSTEP.getId())
        getAFXAliasMap().setName(self.aStep, 'current step')
        self.cStep = FXRadioButton(p=hf1,text="All Steps",tgt=self.form.form.stepKw, sel=ALLSTEP.getId())
        getAFXAliasMap().setName(self.cStep, 'All Steps')
        self.animation=FXCheckButton(p=gb,text="Animate in Viewport", tgt=self, sel=self.ID_CHECK)
        getAFXAliasMap().setName(self.animation, 'animation')
        FXMAPFUNC(self, SEL_COMMAND, self.ID_CHECK,FileExportDB.onCheck)
        self.form.form.animationKw.setValue(False)

        gb1=FXGroupBox(p=self,text='Frame Rate',opts=FRAME_GROOVE|LAYOUT_FILL_X)
        self.slider = AFXSlider(gb1, self.form.form.sliderKw, 0,AFXSLIDER_INSIDE_BAR|LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.slider, 'slider')
        self.slider.setMinLabelText('Slow')
        self.slider.setMaxLabelText('Fast')
        self.slider.setDecimalPlaces(0)
        self.slider.setRange(-5, 0)
        self.slider.setValue(0)
#        getAFXAliasMap().setName(self.slider, 'slider')
        FXMAPFUNC(self, SEL_COMMAND, self.ID_OVERWRITE, FileExportDB.onCmdOverwrite)

        va1 = AFXVerticalAligner(p=self, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
                    pl=0, pr=0, pt=0, pb=0)

        (head, tail) = os.path.split(__file__)
        fileLoadPath = os.path.join(head, "fileSelectSmall.png")

        FXMAPFUNC(self, SEL_COMMAND, self.ID_FILE_LOAD, FileExportDB.onLoadFile)
        fileTextHf = FXHorizontalFrame(va1, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)

        self.fileName = AFXTextField(p=fileTextHf, ncols=25, labelText='AVI file name:', tgt=self.form.form.fileNameKw, sel=0,
            opts=AFXTEXTFIELD_STRING|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        icon = afxCreatePNGIcon(fileLoadPath)
        self.fileSelect = FXButton(fileTextHf, None, icon, self, self.ID_FILE_LOAD, opts=BUTTON_TOOLBAR|FRAME_RAISED, pl=0, pr=0, pt=0, pb=0)
        getAFXAliasMap().setName(self.fileSelect, 'file button')
        self.msg='There is no output database (ODB) \n associated '\
                'with the current viewport.\n\nSelect an ODB from the '\
                'context bar,\nor use "File->Open" to open an ODB.'

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCheck(self,sender,sel,ptr):
        if self.animation.getCheck():
            self.form.form.animationKw.setValue(True)
        else:
            self.form.form.animationKw.setValue(False)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def itemsSelected(self):
        items = ''
        numItems = self.form.list1.getNumItems()
        if numItems == 0:
            return
        for i in range(numItems):
            if self.form.list1.isItemSelected(i):
                items += "'"+self.form.list1.getItemText(i)+"'"
                items += ','
        return items

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def writeAvi(self):
        mainWindow = getAFXApp().getAFXMainWindow()
        items=self.itemsSelected()
        if self.form.form.forceCompKw.getValue() == RESULTANT.getId():
            forceComp=1
        else:
            forceComp=2
        if self.form.form.stepKw.getValue() == CURSTEP.getId():
            step=1
        else:
            step=2
        try:
            file="'"+str(self.fileName.getText())+"'"
            sendCommand("import abq_contactForce.contactForce")
            sendCommand("abq_contactForce.contactForce.generateAvi(historyVar=("+items+\
                        "),scale="+str(self.form.form.scaleKw.getValue())+",aviFileName="+file+\
                        ",forceComp="+str(forceComp)+",step="+str(step)+",frate="+str(self.slider.getValue())+\
                        ",anim="+str(self.animation.getCheck())+")")
        except:
            showAFXErrorDialog(mainWindow,"Error occured while creating animation. Please contact Simulia support office to resolve this issue.")

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdSave(self, sender, sel, ptr):
        mainWindow = getAFXApp().getAFXMainWindow()
        if os.path.dirname(self.fileName.getText()) == '':
            file=os.path.join(os.getcwd(),self.fileName.getText())
        else:
            file=self.fileName.getText()
        if file.find('.avi') != -1:
            fileCheck=file
        else:
            fileCheck=str(file)+".avi"
        if os.path.exists(fileCheck):
            msg = 'File "%s" already exists.\n\nOK to overwrite?' % fileCheck
            showAFXWarningDialog(mainWindow, msg, AFXDialog.YES|AFXDialog.NO, self, self.ID_OVERWRITE)
        else:
            self.writeAvi()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdOverwrite(self,sender,sel,ptr):
        if sender.getPressedButtonId() == AFXDialog.ID_CLICKED_YES:
            self.writeAvi()
        else:
            pass

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onLoadFile(self, sender, sel, ptr):
        if not self.fileDb:
            fileName = 'AVI Files (*.avi)'
            self.fileDb = AFXFileSelectorDialog(self, 'Save File', self.form.form.fileNameKw, None, AFXSELECTFILE_ANY, fileName, None)
            self.fileDb.setReadOnlyPatterns('*.avi')
            self.fileDb.create()

        self.fileDb.showModal()
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def showModal(self):

        mainWindow = getAFXApp().getAFXMainWindow()

        numItems = self.form.list1.getNumItems()
        if numItems == 0:
            showAFXErrorDialog(mainWindow,'No contact output variable available in the current ODB\n'\
                                'Or there is no output database (ODB) associated with the \n'\
                                'current viewport.')
            return
        if self.form.form.scaleKw.getValue() <=0 or self.form.form.scaleKw.getValue() == None:
            showAFXErrorDialog(mainWindow,'Please specify scale factor greater than 0.')
            return

        items=self.itemsSelected()

        if items == '':
            showAFXErrorDialog(mainWindow,'Please select one contact output variable.')
            return

        if getDisplayedObjectType() == XY_PLOT:
            if vp.odbDisplay.name:

                AFXDialog.showModal(self)
            else:
                showAFXErrorDialog(mainWindow,self.msg)
        elif getDisplayedObjectType() == None: # No odb in viewport
            showAFXErrorDialog(mainWindow,self.msg)
        else:
            AFXDialog.showModal(self)
