from abaqusGui import *
from abq_hic.crashAttrSymbolicConstants import *

###########################################################################
# Class definition
###########################################################################

class HicResDB(AFXDataDialog):
    [
        ID_SAE,
        ID_LAST
        ] = range(AFXDataDialog.ID_LAST, AFXDataDialog.ID_LAST+2)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):
        
        AFXDataDialog.__init__(self, form, title='Compute Head Injury Criteria')
        getAFXAliasMap().setPrefix(self, 'HicResDB')
        self.form = form
        self.calculateBtn = self.appendActionButton(text='Compute',
                                                    tgt=self,
                                                    sel=self.ID_CLICKED_APPLY)
        getAFXAliasMap().setName(self.calculateBtn, 'CalBtn')
        self.cancelBtn = self.appendActionButton(text='Cancel',tgt=self,
                                sel=self.ID_CLICKED_CANCEL)
        getAFXAliasMap().setName(self.cancelBtn, 'CnlBtn')


        # XY Data Input
        #
        gb0 = FXGroupBox(self, 'XY Data Input',
                              FRAME_GROOVE|LAYOUT_FILL_X)
        self.radio01 = FXRadioButton(gb0, 'Raw components of Acceleration',
                                     form.resultTypeKw, COMP_TYPE.getId())
        getAFXAliasMap().setName(self.radio01, 'rdoBtn2')
  
        vf1 = FXVerticalFrame(gb0,LAYOUT_FILL_X, 0,0,0,0,20,0,0,0)
        self.component1CB = AFXComboBox(vf1, 0, 10, 'Component 1:',
                                         form.resultPlotKw, 1, LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.component1CB, 'comboBtn2')
        self.component1CB.setMaxVisible(10)
        self.component2CB = AFXComboBox(vf1, 0, 10, 'Component 2:',
                                        form.resultPlotKw, 2, LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.component2CB, 'comboBtn3')
        self.component2CB.setMaxVisible(10)
        self.component3CB = AFXComboBox(vf1, 0, 10, 'Component 3:',
                                        form.resultPlotKw, 3, LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.component3CB, 'comboBtn4')
        self.component3CB.setMaxVisible(10)
  

        self.radio00 = FXRadioButton(gb0, 'Raw Resultant Acceleration',
                                     form.resultTypeKw, RESULT_TYPE.getId())
        getAFXAliasMap().setName(self.radio00, 'rdoBtn1')
        hf0 = FXHorizontalFrame(gb0,LAYOUT_FILL_X, 0,0,0,0, 20,0,0,0)
        resRawAccelStr = 'Resultant acceleration:'
        self.resRawAccelCB = AFXComboBox(hf0, 0, 10, resRawAccelStr,
                                         form.resultPlotKw, 1, LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.resRawAccelCB, 'comboBtn1')
        self.resRawAccelCB.setMaxVisible(10)


        # Options
        #
        gb1 = FXGroupBox(self, 'Options', FRAME_GROOVE|LAYOUT_FILL_X)
        hf2 = FXHorizontalFrame(gb1,LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        vf2 = FXVerticalFrame(hf2,LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0)

        # Time Scale Factor
        #
        gb2 = FXGroupBox(vf2, 'Time Scale Factor', FRAME_GROOVE, LAYOUT_FILL_X)
        va1 = FXVerticalFrame(p=gb2, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y,x=0, y=0, w=0, h=0,pl=0, pr=0, pt=0, pb=0)
        self.radio1 = FXRadioButton(va1, 'Seconds', form.timeScaleFactorKw,
                                    TSF_SECOND.getId())
        getAFXAliasMap().setName(self.radio1, 'rdoBtn3')
        self.radio2 = FXRadioButton(va1, 'Milliseconds', form.timeScaleFactorKw,
                                    TSF_MILLISECOND.getId())
        getAFXAliasMap().setName(self.radio2, 'rdoBtn4')

        # HIC Window
        #
        gb4 = FXGroupBox(vf2, 'HIC Window', FRAME_GROOVE|LAYOUT_FILL_X)
        self.radio3 = FXRadioButton(gb4, '15', form.hicWindowKw,
                                    WIN_15.getId())
        getAFXAliasMap().setName(self.radio3, 'rdoBtn5', )
        self.radio4 = FXRadioButton(gb4, '36', form.hicWindowKw,
                                    WIN_36.getId())
        getAFXAliasMap().setName(self.radio4, 'rdoBtn6', )


        # Length Unit
        #
        gb3 = FXGroupBox(hf2, 'Length Unit', FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        va2 = FXVerticalFrame(p=gb3,opts=LAYOUT_FILL_X|LAYOUT_FILL_Y , x=0, y=0, w=0, h=0,pl=0, pr=0, pt=0, pb=0)
        self.radio5 = FXRadioButton(va2, 'Millimeter', form.lengthScaleFactorKw,
                                    LSF_MILLIMETER.getId())
        getAFXAliasMap().setName(self.radio5, 'rdoBtn7')
        self.radio6 = FXRadioButton(va2, 'Centimeter', form.lengthScaleFactorKw,
                                    LSF_CENTIMETER.getId())
        getAFXAliasMap().setName(self.radio6, 'rdoBtn8')
        self.radio7 = FXRadioButton(va2, 'Meter', form.lengthScaleFactorKw,
                                    LSF_METER.getId())
        getAFXAliasMap().setName(self.radio7, 'rdoBtn9')

        self.radio8 = FXRadioButton(va2, 'No conversion', form.lengthScaleFactorKw,
                                    LSF_NONE.getId())
        getAFXAliasMap().setName(self.radio8, 'rdoBtn10')

        # SAE Filter
        #
        vf5 = FXVerticalFrame(gb1,LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        self.checkButtonSAE = FXCheckButton(vf5, 'Apply SAE filter to input data',
                                            self, self.ID_SAE)
        getAFXAliasMap().setName(self.checkButtonSAE, 'chkBtn1')
        self.checkButtonSAE.setCheck(TRUE)

        # Filter class
        #
        self.saeFilterCB = AFXComboBox(vf5, 0, 6, 'SAE filter class',
                                       form.saeFilterKw, 0, LAYOUT_FILL_X)
        getAFXAliasMap().setName(self.saeFilterCB, 'comboBtn5')

        self.saeFilterCB.setMaxVisible(6)        

        saeList = ['60', '100', '180', '600', '1000']

        for item in saeList:
            self.saeFilterCB.appendItem('sae'+item+'Filter')
        self.form.saeFilterKw.setValue('sae1000Filter')
        #
        # Transition mapping
        #
        self.addTransition(self.form.resultTypeKw, AFXTransition.EQ,
                           COMP_TYPE.getId(), self.component1CB,
                           MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(self.form.resultTypeKw, AFXTransition.EQ,
                           RESULT_TYPE.getId(), self.resRawAccelCB,
                           MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(self.form.resultTypeKw, AFXTransition.EQ,
                           COMP_TYPE.getId(), self.component2CB,
                           MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(self.form.resultTypeKw, AFXTransition.EQ,
                           COMP_TYPE.getId(), self.component3CB,
                           MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        
        self.addTransition(self.form.resultTypeKw, AFXTransition.NE,
                           RESULT_TYPE.getId(), self.resRawAccelCB,
                           MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)
        self.addTransition(self.form.resultTypeKw, AFXTransition.NE,
                           COMP_TYPE.getId(), self.component1CB,
                           MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)
        self.addTransition(self.form.resultTypeKw, AFXTransition.NE,
                           COMP_TYPE.getId(), self.component2CB,
                           MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)
        self.addTransition(self.form.resultTypeKw, AFXTransition.NE,
                           COMP_TYPE.getId(), self.component3CB,
                           MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)

        # Message Map
        #
        FXMAPFUNC(self, SEL_COMMAND, self.ID_SAE, HicResDB.saeFilterFunc)

        self.populateXYData()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def show(self):

        AFXDataDialog.showModal(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def hide(self):

        AFXDataDialog.hide(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onUpdate(self):
        """
        Method to update combo box when xyDataObjects changes.
        """
        self.populateXYData()
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def populateXYData(self):
        """
        Method to populate the combo boxes.
        """

        from kernelAccess import session

        self.resRawAccelCB.clearItems()
        self.component1CB.clearItems()
        self.component2CB.clearItems()
        self.component3CB.clearItems()
        self.form.resultPlotKw.setValue(0,'')
        self.form.resultPlotKw.setValue(1,'')
        self.form.resultPlotKw.setValue(2,'')
        xyDataList = session.xyDataObjects.keys()
        firstItem = ''
        secondItem = ''
        thirdItem = ''
        firstFlag = 0
        secondFlag = 0
        thirdFlag = 0
        for item in xyDataList:
            if not item.startswith('HIC-Resultant') and \
               not item.startswith('HIC-Window:'):
                if secondFlag and  not thirdFlag:
                    thirdFlag = 1
                    thirdItem = item
                if firstFlag and not secondFlag:
                    secondFlag = 1
                    secondItem = item
                if not firstFlag:
                    firstItem = item
                    firstFlag = 1
                self.resRawAccelCB.appendItem(item)
                self.component1CB.appendItem(item)
                self.component2CB.appendItem(item)
                self.component3CB.appendItem(item)
        if firstFlag:
            self.form.resultPlotKw.setValue(0,firstItem)
        if secondFlag:
            self.form.resultPlotKw.setValue(1,secondItem)
        else:
            self.form.resultPlotKw.setValue(1,firstItem)
        if thirdFlag:
            self.form.resultPlotKw.setValue(2,thirdItem)
        else:
            self.form.resultPlotKw.setValue(2,firstItem)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def saeFilterFunc(self, sender, sel, ptr):
        """
        Method to enable or disable SAEFilter dialog box.
        """

        if ptr:
            self.saeFilterCB.enable()
        else:
            self.saeFilterCB.disable()
        
    
