"""
Kernel module for the HIC plug-in.
"""

from abaqus import *
from abaqusConstants import *
from viewerModules import *
try:
    from numpy.oldnumeric import *
except ImportError:
    from Numeric import *

from crashAttrSymbolicConstants import *
import time
fmax = 0
rmax = 0
jmax = 0
kmax = 0
kjl = 0
il = 0
hic = 0.0
t1 = 0.0
t2 = 0.0
ymax = -1.e12
ymin = 1.e12

scr = array((), Float)
jl = array((), Int)
kl = array((), Int)

def displayPlot(win):
    
    global hic, t1, t2
    global rmax, ymax, ymin
    import uti
    import visualization
    
    abqVersion = uti.getVersion()[0:3]
    
    vp = session.viewports[session.currentViewportName]

    newData = ((t1,ymin), (t1,ymax + (ymax/10)), (t2,ymax + (ymax/10)), (t2,ymin))
    xy2 = session.XYData(name='HIC-Window: ' + str(win), data=newData)

    if not session.xyPlots.has_key('hic'):
        session.XYPlot('hic')
    xyp = session.xyPlots['hic']
    xy1 = session.xyDataObjects['HIC-Resultant']

    titleText66 = 'Head Injury Criteria= ' + str(round(hic,3)) + '    T1 = ' + \
                  str(round(t1,4)) + ' sec , ' + 'T2 = ' + str(round(t2,4))+ ' sec'
    titleText = 'Head Injury Criteria= ' + str(round(hic,3)) + '   , ' + ' T1 = ' + \
                  str(round(t1,4)) + ' sec , ' + 'T2 = ' + str(round(t2,4))+ ' sec'    
    if abqVersion == '6.6':
        c1 = xyp.Curve('c1', xy1)
        c2 = xyp.Curve('c2', xy2)
        xyp.curves['c2'].curveOptions.setValues(lineStyle=DASHED)

        xyp.setValues(curvesToPlot=(c1,c2))
        xyp.setValues(xAxisTitleSource=USER_DEFINED,
                      yAxisTitleSource=USER_DEFINED, xAxisTitle='Time', 
                      yAxisTitle='Acceleration', plotTitle=titleText66)
        vp.setValues(displayedObject=xyp)
    else:
        xQuantity = visualization.QuantityType(type=TIME)
        yQuantity = visualization.QuantityType(type=ACCELERATION)
        xy1.setValues(axis1QuantityType=xQuantity, axis2QuantityType=yQuantity)
        xy2.setValues(axis1QuantityType=xQuantity, axis2QuantityType=yQuantity)
        c1 = session.Curve('c1', xy1)
        c2 = session.Curve('c2', xy2)
        session.curves['c2'].lineStyle.setValues(style=DASHED)

        chartName = xyp.charts.keys()[0]
        chart = xyp.charts[chartName]
        chart.setValues(curvesToPlot=(c1,c2))
        vp.setValues(displayedObject=xyp)
        session.xyPlots['hic'].title.setValues(text=titleText66)
        
            
def hic_partition(jli, jui, nj, njd, kli, kui, nk, nkd, inc):
    global jl, kl
    global il

    # Initializations
    j2m = 2 * int(fmod(nj, 2))
    nj2 = njd * j2m
    jmu = ( jui - int(nj)/2 ) * ( 1 - nj2 )

    k2m = 2 * int(fmod(nk, 2))
    nk2 = nkd * k2m
    kmu = ( kui - int(nk)/2 ) * ( 1 - nk2 )

    # First region
    il = il + inc
    jl[il] = jmu
    kl[il] = kmu

    # Second region if finite width along J axis
    if ( nj > 0 ):
        jli = jli * ( j2m - nj2 - 1)
        il = il + inc
        jl[il] = jli
        kl[il] = kmu

    #Third and fourth region
    if ( nk > 0 ):
        kli = kli * ( k2m - nk2 - 1 )

        # Fourth region if finite width along K axis
        il = il + inc
        jl[il] = jmu
        kl[il] = kli

        # Third region if finite width along J and K axes
        if ( nj != 0 ):
            jl[il] = jli

def hic_scan( ibr, ier, inc, kju, nwu ):
    global kjl
    global jl, kl
    global il
    global scr
    global fmax, rmax, jmax, kmax

    ir = ibr
    while ( 1 ):
        jli = jl[ir]
        if ( jli > 0 ):
            nj = nwu
        else:
            jli = -jli
            nj = nwu - 1
        jui = jli + nj

        kli = kl[ir]
        if ( kli > 0 ):
            nk = nwu
        else:
            kli = -kli
            nk = nwu - 1
        kui = kli + nk

        if ( ( kui-jli < kjl ) or ( kli-jui > kju ) ):
            # Infeasible region - Discard
            dummy = 1
        elif ( ( nj > 0 ) or ( nk > 0 ) ):
            # Active region with feasible points - Bound
            diff = kli - jui
            if ( kjl > diff ):
                diff = kjl

            f = ( scr[kui] - scr[jli] ) * pow( diff, -0.6 )
            if ( f > fmax ):
                # Region is still active - Evaluate and partition
                if ( kui-jui > kju ):
                    jlb = jui
                    kub = jui + kju
                else:
                    jlb = kui - kju
                    if ( jli > jlb ):
                        jlb = jli
                    kub = kui
                try:
                    f = ( scr[kub] - scr[jlb] ) * pow( kub-jlb, -0.6 )
                except:
                    import sys, string
                    errorMsg = '. Check if the selected "Time scale Factor" '\
                               'is consistent with the XY Data units.'
                    errorMsg = string.upper(str(sys.exc_info()[1])) + errorMsg
                    raise ValueError, errorMsg
                    
                if ( f > fmax ):
                    fmax = f
                    fr = pow( fmax/rmax, 2.5 )
                    ifr = int(fr+1)
                    comp = int(fr-ifr) + ifr
                    if ( comp > kjl ):
                        kjl = comp
                    jmax = jlb
                    kmax = kub

                if ( ( (il*inc + 4) > (ir*inc) ) or ( nwu <= 10 ) ):
                    # Complete with exhaustive method
                    itStart_1 = kli - jui
                    if ( kjl > itStart_1 ):
                        itStart_1 = kjl
                    itEnd_1 = kui - jli
                    if ( itEnd_1 > kju ):
                        itEnd_1 = kju

                    kj = itStart_1
                    while ( 1 ):
                        skjmax = 0.0
                        jkjmax = 0

                        itStart_2 = kli - kj
                        if ( jli > itStart_2 ):
                            itStart_2 = jli
                        itEnd_2 = kui - kj
                        if ( itEnd_2 > jui ):
                            itEnd_2 = jui

                        j = itStart_2
                        while ( 1 ):
                            skj = scr[kj+j] - scr[j]
                            if ( skj > skjmax ):
                                skjmax = skj
                                jkjmax = j
                            j = j + 1
                            if ( j > itEnd_2 ):
                                break                        
                        f = skjmax * pow(kj, -0.6)
                        if ( f > fmax ):
                            fmax = f
                            fr = pow( fmax/rmax, 2.5 )
                            ifr = int(fr+1)
                            comp = int(fr-ifr) + ifr
                            if ( comp > kjl ):
                                kjl = comp
                            jmax = jkjmax
                            kmax = kj + jmax

                        kj = kj + 1
                        if ( kj > itEnd_1 ):
                            break
                else:
                    # partition
                    hic_partition(jli, jui, nj, nwu-nj,
                                  kli, kui, nk, nwu-nk, inc)

        else:
            # Single point - Evaluate
            try:
                f = ( scr[kui] - scr[jli] ) * pow( (kui-jli), -0.6 )
            except:
                import sys, string
                errorMsg = '. Check if the selected "Time scale Factor" '\
                           'is consistent with the XY Data units.'
                errorMsg = string.upper(str(sys.exc_info()[1])) + errorMsg
                raise ValueError, errorMsg
                
            if ( f > fmax ):
                fmax = f
                fr = pow( fmax/rmax, 2.5 )
                ifr = int(fr+1)
                comp = int(fr-ifr) + ifr
                if ( comp > kjl ):
                    kjl = comp
                jmax = jli
                kmax = kui

        if ( ir == ier ):
            break;

        ir = ir + inc


def hic_s(data, win, timeArg):
    
    global scr
    global fmax, rmax, jmax, kmax
    global kjl
    global jl, kl
    global il
    global hic, t1, t2
    global ymin,ymax

    # initializations
    hic = 0.0
    t1 = 0.0
    t2 = 0.0
    ymax = -1.e12
    ymin = 1.e12

    length = len(data)
    dt = (data[1][0] - data[0][0]) * timeArg

    scr = array(zeros(length), Float)

    # compute integral array using trapezoid rule
    rm = abs( data[0][1] )
    rmax = rm    

    sm = 0
    for i in range(1,length):
        rc = abs( data[i][1] )
        sc = sm + ( rc + rm ) * 0.5
        scr[i] = sc
        if ( rc > rmax ):
            rmax = rc

        if data[i][1] > ymax:
            ymax = data[i][1]
        if data[i][1] < ymin:
            ymin = data[i][1]
            
        sm = sc
        rm = rc

    # initialize HIX computation values
    fmax = 0
    jmax = 0
    kmax = 0

    if (rmax > 0):
        nar = length * 2

        jl = array( zeros(nar+1), Int)
        kl = array( zeros(nar+1), Int)

        kl[0] = -1

        kju = length
        kjl = 1

        if (win > 0):
            kju = int( win / ( 1000 * dt ) * 1.0000001 )

        nwu = length - 1
        ibr = 1
        ier = 1
        inc = -1
        ibl = nar
        il = nar + 1

    # Scan list of active regions
    while (ibr*inc <= ier*inc):
        # Perform scan pass
        hic_scan(ibr, ier, inc, kju, nwu)

        # Set up for next scan pass
        itmp = ibl
        ibl = ier
        ier = itmp
        ibr = il
        inc = -inc
        il = ibl - inc
        nwu = nwu / 2

    hic = pow(fmax, 2.5) * dt
    t1 = jmax * dt
    t2 = kmax * dt

    # convert back to orig units
    t1 = t1 / timeArg
    t2 = t2 / timeArg

    displayPlot(win)

def hic_resul(resultPlot, resultType, timeScaleFactor, lengthScaleFactor, hicWindow,
              saeFilter=None):

    global hic, t1, t2
        
    if resultType == RESULT_TYPE:
        try:
            xy1 = session.xyDataObjects[resultPlot[0]]
        except:
            errorMsg = 'XY-Plot "%s" is not available in the session or odb.'
            raise ValueError, errorMsg %(resultPlot)
    elif resultType == COMP_TYPE:
        try:
            xy1 = session.xyDataObjects[resultPlot[0]]
            xy2 = session.xyDataObjects[resultPlot[1]]
            xy3 = session.xyDataObjects[resultPlot[2]]
        except:
            errorMsg = 'XY-Plot "%s" is not available in the session or odb.'
            raise ValueError, errorMsg %(resultPlot) 
    if timeScaleFactor == TSF_SECOND:
        timeArg = 1.0
    else:
        timeArg = .001

    if lengthScaleFactor == LSF_METER:
        lengthUnitScale = 9.81
    elif lengthScaleFactor == LSF_MILLIMETER:
        lengthUnitScale = 9810.0
    elif lengthScaleFactor == LSF_CENTIMETER:
        lengthUnitScale = 981.0
    elif lengthScaleFactor == LSF_NONE:
        lengthUnitScale = 1.0
    else:
        # to make adding units more simple
        print "No length unit selected, Please report this issue to Abaqus Suppor"
        sys.exit(1)
    if hicWindow == WIN_15:
        win = 15.0
    else:
        win = 36.0
    if resultType == RESULT_TYPE:
        if saeFilter:
            filtObj = saeFilter
            evalString = filtObj + '(' + 'xy1' + ',' + str(timeArg) + ')'
            resul = eval(evalString)
            resul = resul/lengthUnitScale
            data = resul.data
            session.XYData(name='HIC-Resultant', objectToCopy=resul)
        else:
            xy1=xy1/lengthUnitScale
            data = xy1.data
            session.XYData(name='HIC-Resultant', objectToCopy=xy1)    
        
    elif resultType == COMP_TYPE:
        if saeFilter:
            filtObj = saeFilter
            evalString1 = filtObj + '(' + 'xy1' + ',' + str(timeArg) + ')'
            evalString2 = filtObj + '(' + 'xy2' + ',' + str(timeArg) + ')'
            evalString3 = filtObj + '(' + 'xy3' + ',' + str(timeArg) + ')'
            filt1 = eval(evalString1)
            filt2 = eval(evalString2)
            filt3 = eval(evalString2)
            resul = srss((filt1, filt2, filt3))
            resul = resul/lengthUnitScale
            data = resul.data
            session.XYData(name='HIC-Resultant', objectToCopy=resul)
        else:
            resul = srss((xy1, xy2, xy3))
            resul = resul/lengthUnitScale
            data = resul.data
            session.XYData(name='HIC-Resultant', objectToCopy=resul)

    hic_s(data, win, timeArg)
