from abaqusGui import *
from abq_rMises.rMisesSymbConsts import *
import os
from kernelAccess import *
###########################################################################
# Class definition
###########################################################################

class RMisesDB(AFXDataDialog):
    
    ID_READ_ODB,ID_WRITE_ODB,ID_SEC_CAT,ID_SEC_PT,ID_SOLID_SHELL,ID_READ_SECTION= \
    range(AFXDataDialog.ID_LAST,AFXDataDialog.ID_LAST+6)
    numInstance = 0

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):

        RMisesDB.numInstance += 1

        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'Save RMISES results',
            0, opts=DECOR_RESIZE|DIALOG_NORMAL|DIALOG_ACTIONS_SEPARATOR)

        #GUILOG
        getAFXAliasMap().setPrefix(self, 'RMisesDB-%d'
                                   % RMisesDB.numInstance)
        #ENDGUILOG

        self.form = form
        self.readFileDb = None
        self.writeFileDb = None
        
        self.ok = self.appendActionButton('Save', self, self.ID_CLICKED_APPLY)
        getAFXAliasMap().setName(self.ok, 'Save')
        self.dismiss = self.appendActionButton(self.DISMISS)
        getAFXAliasMap().setName(self.dismiss, 'Dismiss')
        gb=FXGroupBox(p=self,text='Output Databases',opts=FRAME_GROOVE|LAYOUT_FILL_X)
        (head, tail) = os.path.split(__file__)
        fileLoadPath = os.path.join(head, "fileSelectSmall.png")
        sectionImage = os.path.join(head, "icoR_section.png")
        

        va1 = AFXVerticalAligner(p=gb, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
                    pl=0, pr=0, pt=0, pb=0)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_READ_ODB, RMisesDB.onLoadFile)
        fileTextHf = FXHorizontalFrame(va1, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)

        self.readOdbName = AFXTextField(p=fileTextHf, ncols=25, labelText='Read ODB:', 
                                        tgt=self.form.readOdbNameKw, sel=0,
                                        opts=AFXTEXTFIELD_STRING|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        getAFXAliasMap().setName(self.readOdbName, 'Read ODB Name')                                
        
        self.readOdbName.setEditable(False)
        self.readOdbName.setReadOnlyState(True)
        icon = afxCreatePNGIcon(fileLoadPath)
        
        self.readOdbSelect = FXButton(fileTextHf, None, icon, self, self.ID_READ_ODB, 
                                      opts=BUTTON_TOOLBAR|FRAME_RAISED, pl=0, pr=0, pt=0, pb=0)
    
        getAFXAliasMap().setName(self.readOdbSelect, 'Read ODB Select')
        FXMAPFUNC(self, SEL_COMMAND, self.ID_WRITE_ODB, RMisesDB.onLoadFile)
        fileTextHf = FXHorizontalFrame(va1, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
            pl=0, pr=0, pt=0, pb=0, hs=DEFAULT_SPACING, vs=DEFAULT_SPACING)

        self.writeOdbName = AFXTextField(p=fileTextHf, ncols=25, labelText='Write ODB:', 
                                        tgt=self.form.writeOdbNameKw, sel=0,
                                        opts=AFXTEXTFIELD_STRING|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        getAFXAliasMap().setName(self.writeOdbName, 'Write ODB Name')
        self.writeOdbName.setEditable(False)
        self.writeOdbName.setReadOnlyState(True)
        icon = afxCreatePNGIcon(fileLoadPath)
        sectionIcon = afxCreatePNGIcon(sectionImage)
        sectionIcon.scale(16,16)
        sectionIcon.create()
                
        self.writeOdbSelect = FXButton(fileTextHf, None, icon, self, self.ID_WRITE_ODB, 
                                       opts=BUTTON_TOOLBAR|FRAME_RAISED, pl=0, pr=0, pt=0, pb=0)
        
        getAFXAliasMap().setName(self.writeOdbSelect, 'Write ODB Select')
        gb1=FXGroupBox(p=self,text='Options',opts=FRAME_GROOVE|LAYOUT_FILL_X)
        vf1 = AFXVerticalAligner(p=gb1, opts=LAYOUT_FILL_X|LAYOUT_FILL_Y, x=0, y=0, w=0, h=0,
                    pl=0, pr=0, pt=0, pb=0)
        self.foName = AFXTextField(vf1, 24, 'Field Output Name:',self.form.foNameKw, 0,opts=AFXTEXTFIELD_STRING)
        getAFXAliasMap().setName(self.foName, 'field output name')
        self.nodeLimit = AFXTextField(vf1, 10, 'Node Limit:',self.form.nodeLimitKw, 0,opts=AFXTEXTFIELD_INTEGER)
        getAFXAliasMap().setName(self.nodeLimit, 'node limit')
        
        hf1 = FXHorizontalFrame(vf1, LAYOUT_FILL_X,0, 0, 0, 0, 0, 0, 0, 0)
        self.label2 = FXLabel(p=hf1 , text='Model type:', ic=None, opts=LAYOUT_CENTER_Y|JUSTIFY_LEFT)
        self.solid = FXRadioButton(p=hf1,text="Solid",tgt=self.form.modelTypeKw, sel=SOLID.getId())
        self.shell = FXRadioButton(p=hf1,text="Shell",tgt=self.form.modelTypeKw, sel=SHELL.getId())
        getAFXAliasMap().setName(self.solid, 'solid section')
        getAFXAliasMap().setName(self.shell, 'shell section')
        self.getSection = FXButton(hf1, "\tGet section categories", sectionIcon, self, self.ID_READ_SECTION, 
                                      opts=BUTTON_NORMAL, pl=0, pr=0, pt=0, pb=0)
        getAFXAliasMap().setName(self.getSection, 'get section')                              
        FXMAPFUNC(self, SEL_COMMAND, self.ID_READ_SECTION, RMisesDB.onReadSection)
        
        self.form.modelTypeKw.setTarget(self)
        self.form.modelTypeKw.setSelector(self.ID_SOLID_SHELL)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_SOLID_SHELL, RMisesDB.onModelChange)
        
        self.gb2=FXGroupBox(p=self,text='Section Categories and Points',opts=FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        self.secCatList = AFXList(self.gb2, 3, self,self.ID_SEC_CAT,LAYOUT_FILL_X|LIST_SINGLESELECT|HSCROLLING_OFF,w=50,h=200)
        getAFXAliasMap().setName(self.secCatList, 'section categories')                           
        self.secPtList = AFXList(self.gb2, 3, self,self.ID_SEC_PT,LAYOUT_FILL_X|LAYOUT_FILL_Y|
                            LIST_SINGLESELECT|HSCROLLING_OFF,w=50,h=200)
        getAFXAliasMap().setName(self.secPtList, 'section points')
        FXMAPFUNC(self, SEL_COMMAND, self.ID_SEC_CAT, RMisesDB.onSectionCat)
        self.mainWindow = getAFXApp().getAFXMainWindow()
        
        self.onModelChange(None,None,None)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def itemsSelected(self):
        items = ''
        numItems = self.secCatList.getNumItems()
        if numItems == 0:
            return ''
        for i in range(numItems):
            if self.secCatList.isItemSelected(i):
                items += self.secCatList.getItemText(i)
                return items
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onLoadFile(self, sender, sel, ptr):
        
        if SELID(sel) == self.ID_READ_ODB:
            if not self.readFileDb:
                fileName = 'Output databases (*.odb)'
                self.readFileDb = AFXFileSelectorDialog(self, 'Open ODB', self.form.readOdbNameKw, None, 
                                                        AFXSELECTFILE_ANY, fileName, None)
                self.readFileDb.setReadOnlyPatterns('*.odb')
                self.readFileDb.create()
                getAFXAliasMap().setPrefix(self.readFileDb, 'ReadFileDB')
            fileDb = self.readFileDb
        else:
            if not self.writeFileDb:
                fileName = 'Output databases (*.odb)'
                self.writeFileDb = AFXFileSelectorDialog(self, 'Open ODB', self.form.writeOdbNameKw, None, 
                                                         AFXSELECTFILE_ANY, fileName, None)
                self.writeFileDb.setReadOnlyPatterns('*.odb')
                self.writeFileDb.create()
                getAFXAliasMap().setPrefix(self.writeFileDb, 'WriteFileDB')
            fileDb = self.writeFileDb    

        fileDb.showModal()
        return 1
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onModelChange(self, sender, sel, ptr):
        
        if self.form.modelTypeKw.getValue() == SOLID.getId():
            self.gb2.hide()
            self.secCatList.hide()
            self.secPtList.hide()
            self.gb2.recalc()
            self.getSection.disable()
        else:
            self.secCatList.clearItems()
            self.gb2.show()
            self.secCatList.show()
            self.secPtList.show()
            self.gb2.recalc()
            self.getSection.enable()
            self.populateSecCat()
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def populateSecCat(self):
        
        odb = self.returnOdb()
        self.secCatList.clearItems()     
        if odb:
            for sc in odb.sectionCategories.keys():
                self.secCatList.appendItem(sc)
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onSectionCat(self, sender, sel, ptr):
        
        odb = self.returnOdb()
        if odb:
            sectionCats = odb.sectionCategories
            sectionCat = self.itemsSelected()
            self.secPtList.clearItems()
            if sectionCat:
                for sp in sectionCats[sectionCat].sectionPoints:
                    self.secPtList.appendItem(sp.description)
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    def returnOdb(self):
        
        if not self.writeOdbName.getText() in session.odbs.keys():
            if os.path.exists(self.writeOdbName.getText()):
                if session.isUpgradeRequiredForOdb(self.writeOdbName.getText()):
                    showAFXErrorDialog(self.mainWindow,'Output database "%s" should be upgraded.'%self.writeOdbName.getText())
                    odb = ''
                else:
                    odb = session.openOdb(name=self.writeOdbName.getText(),readOnly=True)
            else:
                print 'Output database "%s" could not be found.'%self.writeOdbName.getText()
                odb = ''
        else:
            odb = session.odbs[self.writeOdbName.getText()]
        return odb    
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onReadSection(self, sender, sel, ptr):
        
        self.populateSecCat()
