from abaqusGui import *
from kernelAccess import *
import abaqusConstants
import os
from abq_rMises.rMisesSymbConsts import *

###########################################################################
# Class definition
###########################################################################

class RMisesForm(AFXForm):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        self.owner = owner
        AFXForm.__init__(self, owner=owner)
        self.cmd = AFXGuiCommand(self,'saveRMises','abq_rMises.rMisesUtils',registerQuery=False)
        self.readOdbNameKw = AFXStringKeyword(self.cmd, 'readOdbName', True, '')
        self.writeOdbNameKw = AFXStringKeyword(self.cmd, 'writeOdbName', True, '')
        self.foNameKw = AFXStringKeyword(self.cmd, 'foName', True, '')
        self.nodeLimitKw = AFXIntKeyword(self.cmd, 'nodeLimit', True, 10000)
        self.modelTypeKw = AFXSymConstKeyword(command=self.cmd,name='modelType',isRequired=True,defaultValue=SHELL.getId())
        self.secCatKw =AFXStringKeyword(command=self.cmd,name='sectionCategory',isRequired=True)
        self.secPtKw =AFXStringKeyword(command=self.cmd,name='sectionPoint',isRequired=True)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        """Return the first dialog to be posted"""
        self.cmd.setKeywordValuesToDefaults()
        import rMisesDB
        reload(rMisesDB)
        self.db = rMisesDB.RMisesDB(self)
        return self.db

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def activate(self):
        mainWindow = getAFXApp().getAFXMainWindow()
        AFXForm.activate(self)
        """try:
            if vp.odbDisplay.name:
                AFXForm.activate(self)
        except:
            showAFXErrorDialog(mainWindow,'There is no output database (ODB) \n'
                                           'associated with the current viewport.\n\n'
                                           'Select an ODB from the context bar,\n'
                                            'or use "File->Open" to open an ODB.')"""

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):
        mainWindow = getAFXApp().getAFXMainWindow()
        
        secCatFlag= False
        secPtFlag = False
        secCatItems = self.db.secCatList.getNumItems()
        secPtItems = self.db.secPtList.getNumItems()
        
        
        if not self.foNameKw.getValue():
            showAFXErrorDialog(mainWindow,'Please specify the field output name.')
            return 0
        if not self.nodeLimitKw.getValue():
            showAFXErrorDialog(mainWindow,'Please specify the node limit.')
            return 0
        
        if self.modelTypeKw.getValue() == SHELL.getId():
            secCatFlag,secCatItems = self.returnListFlag(secCatFlag,secCatItems,self.db.secCatList)
            secPtFlag,secPtItems = self.returnListFlag(secPtFlag,secPtItems,self.db.secPtList)
        
        if not self.readOdbNameKw.getValue():
            showAFXErrorDialog(mainWindow,'Please specify the read output database.')
            return 0
        if not self.writeOdbNameKw.getValue():
            showAFXErrorDialog(mainWindow,'Please select the write output database.')
            return 0
            
        if not os.path.exists(self.readOdbNameKw.getValue()):
            showAFXErrorDialog(mainWindow,'Read output database does not exist.')
            return 0
        if not os.path.exists(self.writeOdbNameKw.getValue()):
            showAFXErrorDialog(mainWindow,'Write output database does not exist.')
            return 0
        
        if self.modelTypeKw.getValue() == SHELL.getId():
            if not secCatFlag:
                showAFXErrorDialog(mainWindow,'Please select a section category.')
                return 0
            if not secPtFlag:
                showAFXErrorDialog(mainWindow,'Please select a section point.')
                return 0
        self.secCatKw.setValue(str(secCatItems))
        self.secPtKw.setValue(str(secPtItems))
        
        return 1
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def deactivate(self):
        AFXForm.deactivate(self)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def returnListFlag(self,flag,items,list):
        
        if items:
            for i in range(items):
                if list.isItemSelected(i):
                    item = list.getItemText(i)
                    flag=True
                    break
                else:
                    item = ''
        else:
            item = ''
        return (flag,item)        
