from abaqus import *
from abaqusConstants import *
from abq_rMises.rMisesSymbConsts import * 
import visualization
import sys
import os
import timer

def saveRMises(readOdbName, writeOdbName, foName, nodeLimit, modelType, 
               sectionCategory='',sectionPoint=''):
    #nodeLimit = number of nodes per processing slice.  Must be integer.

    flag = False
    sectionPtDic = {}
    sViewport = session.viewports[session.currentViewportName]
    sViewport.makeCurrent()

    rOdb = session.openOdb(name=readOdbName)

    print "Reading " + readOdbName

    if writeOdbName in session.odbs.keys():
        session.odbs[writeOdbName].close()
    wOdb = session.openOdb(name=writeOdbName, readOnly=FALSE)

    delXYData = session.xyDataObjects.__delitem__

    #get rOdb steps and instances
    steps = [[sKey, step.procedure] for [sKey, step] in rOdb.steps.items()]
    instanceKeys = rOdb.rootAssembly.instances.keys()
    sNodes = {}
    #process rOdb steps
    for sKey, sProc in steps:
        print "Processing step " + sKey + "..."
        
        rStep = rOdb.steps[sKey]
        wStep = wOdb.steps[sKey]

        #get output nodes from *FREQUENCY steps
        if sProc == '*FREQUENCY':
            print "Collecting nodes for output..."
            #create dictionary of instance node sets
            sNodes = dict([(k, set()) for k in instanceKeys])
            
            #for each element stress output in last frame of rStep, 
            #determine instance nodes from element connectivity and 
            #add to sNodes
            if 'S' in rStep.frames[-1].fieldOutputs.keys():
                for val in rStep.frames[-1].fieldOutputs['S'].values:
                    sNodes[val.instance.name] |= set(
                        val.instance.getElementFromLabel(label=val.elementLabel)
                                                     .connectivity)
            else:
                raise ValueError,"Stress field output is not available in the frequency step."
            #convert values in sNodes to tuple so the order is fixed
            sNodes = dict([(key, sorted(values))
                           for (key, values) in sNodes.items()])


        #get rMises values from *RANDOM RESPONSE following *FREQUENCY step
        elif sProc == '*RANDOM RESPONSE':

            if not sNodes:
                raise ValueError,"The plug-in could not collect the nodes from the frequency step.\n"\
                                 "Please make sure Frequency step exists in the output database."
            #Must display rOdb in viewport to get RMISES
            sViewport.setValues(displayedObject=rOdb)

            #Make active step sKey
            session.odbData[readOdbName].setValues(activeFrames=((sKey,
                                                               ('0:-1', )), ))

            rMisesField = {}
            #create write frame in wStep given rStep frames with rMises info
            frameFlag = False
            for frame in [f for f in rStep.frames
                          if f.fieldOutputs.has_key('MATRIX RSqq')]:
                try:
                    wFrame = wStep.frames[frame.frameId-1]
                except:
                    wFrame = wStep.Frame(incrementNumber = frame.incrementNumber,
                                         frameValue = frame.frameValue,
                                         frequency = frame.frequency,
                                         description = frame.description)
                frameFlag = True
                #create rMises stress  field data
                if foName in wFrame.fieldOutputs.keys():
                    raise ValueError,"Field output '%s' exists in the output database."%foName
                else:    
                    rMisesField[frame.frequency] = wFrame.FieldOutput(name=foName,
                                                       description='Saved RMISES',
                                                       type=SCALAR)
            if not frameFlag:
                raise ValueError,"The plug-in could not create frames in the write output database.\n"\
                                 "Please make sure 'MATRIX RSqq' field output is available in the \n"\
                                 "read output database."
            #extract RMISES from instance nodes in sNodes
            for iName, nodes in sNodes.iteritems():
                print "Extracting RMISES from " + iName

                if len(nodes) == 0:
                    print "No data found"
                    continue

                instance = rOdb.rootAssembly.instances[iName]

                numTranche = len(nodes)/nodeLimit + 1

                #process nodes in tranches to preserve memory, enhance write, 
                #provide status
                for i in range(numTranche):
                    print "Processing tranche " + str(i+1) + " of " + \
                          str(numTranche)
                    
                    #get nodes to be processed
                    pNodes = nodes[i*nodeLimit:(i+1)*nodeLimit]

                    if modelType == SHELL:
                        sectionPtDic[sectionCategory] = sectionPoint
                        pRmises = session.xyDataListFromField(odb=rOdb,
                                  outputPosition=NODAL,
                                  variable=(('RMISES', ELEMENT_NODAL,(),sectionPtDic), ),
                                  nodeLabels=((iName, pNodes), ))
                    else:
                        pRmises = session.xyDataListFromField(odb=rOdb,
                                  outputPosition=NODAL,
                                  variable=(('RMISES', ELEMENT_NODAL), ),
                                  nodeLabels=((iName, pNodes), ))
                    #write rMises data for each frame
                    for frame in wStep.frames:
                        pRmisesData = []
                        freq = frame.frequency
                        
                        #assemble list of rMises data for frame frequency
                        #pRmises is organized by nodes, for each node get the 
                        #maximum rMises value for given frequency (just in 
                        #case duplicates exist) then add to pRmisesData 
                        #which is the node orderd list of rMises stress values
                        for pR in pRmises:
                            pRmisesData += [[max([r  for (f, r) in pR
                                                  if f == freq]),],]

                        #create new rMises field
                        if len(pNodes) != len(pRmisesData):
                            raise ValueError,"The number of nodes does not match the length of the field output data."\
                                            "\nPossible reasons of failure are:"\
                                            "\n  1. The model consists of shell elements and the user"\
                                            "has chosen solid section in the dialog."\
                                            "\n  2. The model consists of beams, couplings or constraints."
                        
                        try:
                            rMisesField[freq].addData(position=NODAL,
                                                      instance=instance,
                                                      labels=pNodes,
                                                      data=pRmisesData)
                        except:
                            raise ValueError,"An error occurred while adding the RMISES data."
                        flag = True                          

                    #Delete newly created xyData from session.xyDataObjects
                    [delXYData(data.name) for data in pRmises]

        else:
            
            raise ValueError,"Step type '%s' is not supported with this plug-in."%sProc
            
    wOdb.save()
    rOdb.close()
    wOdb.close()
    if flag:
        print "RMISES results have been stored as a nodal field output '%s' in the output database."%foName
    else:
        print "RMISES results were not saved in the output database."
    
