"Dialog box file for Auto mesher plug-in"

from abaqusGui import *
import os.path

###########################################################################
# Class definition
###########################################################################

class AutoDB(AFXDataDialog):
    "The class for definition of the dialog-box"

    [ID_FILE, ] = range(AFXForm.ID_LAST, AFXForm.ID_LAST + 1)
  
    # The constructor for the dialog box
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):
        "Constructor for the dialog box"
        AFXDataDialog.__init__(self, form, 'Auto Mesher', self.OK|self.APPLY|self.CANCEL)#, DIALOG_ACTIONS_SEPARATOR)
        self.fileDb = None
        self.form = form
        FXMAPFUNC(self, SEL_COMMAND, self.ID_FILE, AutoDB.onCmdFile)
        getAFXAliasMap().setPrefix(self, 'AutoDB')
        (head, tail) = os.path.split(__file__)

        filePath = os.path.join(head, "logo.bmp")
        logoIcon = afxCreateBMPIcon(filePath)
        logoLabel = FXLabel(self, None, logoIcon)

        options = FXGroupBox(self, "Options", opts=FRAME_GROOVE|GROUPBOX_NORMAL|LAYOUT_FILL_X)
        hf = FXHorizontalFrame(options, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)
        reportFile = AFXTextField(hf, ncols=20, labelText='Report file:', tgt=form.reportFileKw, sel=0, opts=AFXTEXTFIELD_STRING|LAYOUT_FILL_X)
        getAFXAliasMap().setName(reportFile, 'reportFile')
        filePath = os.path.join(head, "fileSave.png")
        fileSave = afxCreatePNGIcon(filePath)
        fileSelect = FXButton(hf, None, fileSave, self, self.ID_FILE, opts=BUTTON_TOOLBAR|FRAME_RAISED, pl=0, pr=0, pt=0, pb=0)
        getAFXAliasMap().setName(fileSelect, 'fileSelect')
        mapOn = FXCheckButton(options, "Use mapped meshing", tgt=form.mapOnKw)
        getAFXAliasMap().setName(mapOn, 'mapOn')
        incSize = FXCheckButton(options, "Increase size of interior elements", tgt=form.incSizeKw)
        getAFXAliasMap().setName(incSize, 'incSize')
        applyVT = FXCheckButton(options, "Apply virtual topology", tgt=form.applyVTKw)
        getAFXAliasMap().setName(applyVT, 'applyVT')
        showReport = FXCheckButton(options, "Display HTML report", tgt=form.showReportKw)
        getAFXAliasMap().setName(showReport, 'showReport')

    # The constructor for the file-open dialog box
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdFile(self, sender, sel, ptr):
        "This function opens up a dialog box to select file where report will be stored"
        if not self.fileDb:
            patterns = 'HTML Files (*.htm*)'
            self.fileDb = AFXFileSelectorDialog(self, 'Save Name', self.form.reportFileKw, None, AFXSELECTFILE_ANY, patterns, None)
            self.fileDb.setReadOnlyPatterns('*.html\n*.htm')
            self.fileDb.create()
            getAFXAliasMap().setPrefix(self.fileDb, 'FileDB')
        self.fileDb.showModal() 
        return 1
