"Form file for Auto mesher plug-in"

import autoDB
from abaqusGui import *

###########################################################################
# Class definition
###########################################################################

class autoForm(AFXForm):
    "The class for storing the form for Auto mesher"

    # The constructor for the form
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):
        "The constructor for the form"
        
        AFXForm.__init__(self, owner)
        cmd = AFXGuiCommand(mode=self, method='autoMesh', objectName='autoMesher.autoKernel')
        # Keywords
        self.reportFileKw = AFXStringKeyword(command=cmd, name='reportFile', isRequired=TRUE)
        self.reportFileKw.setValue('autoReport.html')
        self.mapOnKw = AFXBoolKeyword(command=cmd, name='mapOn', booleanType=AFXBoolKeyword.TRUE_FALSE, defaultValue=False, isRequired=TRUE)
        self.mapOnKw.setValue(False)
        self.showReportKw = AFXBoolKeyword(command=cmd, name='showReport', booleanType=AFXBoolKeyword.TRUE_FALSE, defaultValue=True, isRequired=TRUE)
        self.showReportKw.setValue(True)
        self.incSizeKw = AFXBoolKeyword(command=cmd, name='incSize', booleanType=AFXBoolKeyword.TRUE_FALSE, defaultValue=False, isRequired=TRUE)
        self.incSizeKw.setValue(False)
        self.applyVTKw = AFXBoolKeyword(command=cmd, name='applyVT', booleanType=AFXBoolKeyword.TRUE_FALSE, defaultValue=False, isRequired=TRUE)
        self.applyVTKw.setValue(False)

    # Getfirst Dialog function
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):
        "Getfirst dialog function"
        return autoDB.AutoDB(self)

    # This function validates the user input
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):
        "This function validates the user input"
        db = self.getCurrentDialog()
        reportFile = self.reportFileKw.getValue()
        showReport = self.showReportKw.getValue()
        # Whether or not the user sees the report, it is written and thus fileName is required
        if reportFile == '':
            showAFXErrorDialog(db, "Filename for the report must be entered.")
            return False
        # Valid Filename
        (head, tail) = os.path.split(reportFile)
        illegalFileChars = ['\\', '/', ':', '*', '?', '<', '>', '|']
        illegalDirChars = ['*', '?', '<', '>', '|']
        for ch in illegalFileChars:
            if tail.find(ch) != -1:
                showAFXErrorDialog(db, "Invalid filename")
                return False
        for ch in illegalDirChars:
            if head.find(ch) != -1:
                showAFXErrorDialog(db, "Invalid filename")
                return False
        # Valid File Extension
        (fname, suffix) = os.path.splitext(reportFile)
        suffix = suffix.lower()
        if suffix != '.htm' and suffix != '.html':
            newFileName = fname + '.html'
            self.reportFileKw.setValue(newFileName)
            #showAFXInformationDialog(db, "Report filename changed from " + reportFile + " to " + newFileName)
            reportFile = self.reportFileKw.getValue()
        return True
