"Kernel script for Auto mesher plug-in"

from abaqusConstants import *
from abaqus import mdb, session
import os, uti, time

refinePerLevel = 0.9
defaultTries = 5
maxVirtualTopologies = 10
devFactor = 0.1

# This function returns the filename for a given part/instance
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getValidImageFilename(entityName, modelName):
    "This function returns the filename for a given part/instance"
    tmpName = modelName + "_" + entityName
    newName = ""
    for c in tmpName:
        # Remove all charachters other than chars, numbers, - and _
        if c.isalnum():
            newName = newName + c
        if c == '-' or c == '_':
            newName = newName + c
    return newName

# This function writes the HTML code to write the part information to HTML
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def appendPartInformationToHTML(part, lines, addInfoList, good=True):
    "This function writes the HTML code to write the part information to HTML"
    lines.append('\t\t\t<tr>\n')
    lines.append('\t\t\t\t<td colspan="2"><br /><hr />\n')
    lines.append('\t\t\t\t</td>\n')
    lines.append('\t\t\t</tr>\n')
    lines.append('\t\t\t<tr>\n')
    lines.append('\t\t\t\t<td>\n')
    lines.append('\t\t\t\t\t<p><b>Part: </b>' + part.name + '</p>\n')
    lines.append('\t\t\t\t</td>\n')
    lines.append('\t\t\t\t<td>\n')
    lines.append('\t\t\t\t\t<p><b>Model: </b>' + part.modelName + '</p>\n')
    lines.append('\t\t\t\t</td>\n')
    lines.append('\t\t\t</tr>\n')
    lines.append('\t\t\t<tr>\n')
    lines.append('\t\t\t\t<td colspan="2">\n')
    lines.append('\t\t\t\t\t<img src="' + getValidImageFilename(part.name, part.modelName) + '.png" />\n')
    lines.append('\t\t\t\t</td>\n')
    lines.append('\t\t\t</tr>\n')
    if good:
        meshStats = part.getMeshStats()
        meshQuality = part.verifyMeshQuality(criterion=ANALYSIS_CHECKS)
        lines.append('\t\t\t<tr>\n')
        lines.append('\t\t\t\t<td>\n')
        lines.append('\t\t\t\t\t<p><b>Number of meshed regions: </b>' + str(meshStats.numMeshedRegions) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t\t<td>\n')
        lines.append('\t\t\t\t\t<p><b>Total number of regions: </b>' + str(len(part.cells)) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t</tr>\n')
        lines.append('\t\t\t<tr>\n')
        lines.append('\t\t\t\t<td>\n')
        lines.append('\t\t\t\t\t<p><b>Number of nodes: </b>' + str(meshStats.numNodes) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t\t<td>\n')
        lines.append('\t\t\t\t\t<p><b>Number of Tetrahedron elements: </b>' + str(meshStats.numTetElems) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t</tr>\n')
        lines.append('\t\t\t<tr>\n')
        lines.append('\t\t\t\t<td>\n')
        lines.append('\t\t\t\t\t<p><b>Number of failed elements: </b>' + str(len(meshQuality["failedElements"])) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t\t<td>\n')
        lines.append('\t\t\t\t\t<p><b>Number of warning elements: </b>' + str(len(meshQuality["warningElements"])) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t</tr>\n')
    partKey = part.modelName + '###Part###' + part.name
    if partKey in addInfoList.keys():
        num = addInfoList[partKey]
        lines.append('\t\t\t<tr>\n')
        lines.append('\t\t\t\t<td colspan="2">\n')
        lines.append('\t\t\t\t\t<p><b>Number of virtual topology features created: </b>' + str(num) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t</tr>\n')

# This function writes the HTML code to write the part information to HTML
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def appendInstanceInformationToHTML(instance, lines, addInfoList, good=True):
    "This function writes the HTML code to write the part information to HTML"
    lines.append('\t\t\t<tr>\n')
    lines.append('\t\t\t\t<td colspan="2"><br /><hr />\n')
    lines.append('\t\t\t\t</td>\n')
    lines.append('\t\t\t</tr>\n')
    lines.append('\t\t\t<tr>\n')
    lines.append('\t\t\t\t<td>\n')
    lines.append('\t\t\t\t\t<p><b>Instance: </b>' + instance.name + '</p>\n')
    lines.append('\t\t\t\t</td>\n')
    lines.append('\t\t\t\t<td>\n')
    lines.append('\t\t\t\t\t<p><b>Model: </b>' + instance.part.modelName + '</p>\n')
    lines.append('\t\t\t\t</td>\n')
    lines.append('\t\t\t</tr>\n')
    lines.append('\t\t\t<tr>\n')
    lines.append('\t\t\t\t<td colspan="2">\n')
    lines.append('\t\t\t\t\t<img src="' + getValidImageFilename(instance.name, instance.part.modelName) + '.png" />\n')
    lines.append('\t\t\t\t</td>\n')
    lines.append('\t\t\t</tr>\n')
    if good:
        ra = mdb.models[instance.part.modelName].rootAssembly
        meshStats = ra.getMeshStats(regions=(instance,))
        meshQuality = ra.verifyMeshQuality(regions=(instance,), criterion=ANALYSIS_CHECKS)
        lines.append('\t\t\t<tr>\n')
        lines.append('\t\t\t\t<td>\n')
        lines.append('\t\t\t\t\t<p><b>Number of meshed regions: </b>' + str(meshStats.numMeshedRegions) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t\t<td>\n')
        lines.append('\t\t\t\t\t<p><b>Total number of regions: </b>' + str(len(instance.cells)) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t</tr>\n')
        lines.append('\t\t\t<tr>\n')
        lines.append('\t\t\t\t<td>\n')
        lines.append('\t\t\t\t\t<p><b>Number of nodes: </b>' + str(meshStats.numNodes) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t\t<td>\n')
        lines.append('\t\t\t\t\t<p><b>Number of Tetrahedron elements: </b>' + str(meshStats.numTetElems) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t</tr>\n')
        lines.append('\t\t\t<tr>\n')
        lines.append('\t\t\t\t<td>\n')
        lines.append('\t\t\t\t\t<p><b>Number of failed elements: </b>' + str(len(meshQuality["failedElements"])) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t\t<td>\n')
        lines.append('\t\t\t\t\t<p><b>Number of warning elements: </b>' + str(len(meshQuality["warningElements"])) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t</tr>\n')
    instanceKey = instance.part.modelName + '###Instance###' + instance.name
    if instanceKey in addInfoList.keys():
        num = addInfoList[instanceKey]
        lines.append('\t\t\t<tr>\n')
        lines.append('\t\t\t\t<td colspan="2">\n')
        lines.append('\t\t\t\t\t<p><b>Number of virtual topology features created: </b>' + str(num) + '</p>\n')
        lines.append('\t\t\t\t</td>\n')
        lines.append('\t\t\t</tr>\n')

# This function checks to see if a part is meshed or not
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def doesPartHaveValidMesh(part):
    "This function checks to see if a part is meshed or not"
    if part.isLocked(): # Locked parts are not considered to be meshed
        return False
    meshStats = part.getMeshStats()
    if (meshStats.numTetElems + meshStats.numHexElems + meshStats.numWedgeElems) == 0: # If there are no 3D elements then the part is not meshed
        return False
    if meshStats.numMeshedRegions != len(part.cells): # Number of meshed regions must equal number of cells
        return False
    meshQuality = part.verifyMeshQuality(criterion=ANALYSIS_CHECKS)
    if len(meshQuality["failedElements"]) != 0: # If there are elements with failed mesh-quality then it is not considered meshed
        return False
    return True

# This function checks to see if an instance is meshed or not
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def doesInstanceHaveValidMesh(instance):
    "This function checks to see if an instance is meshed or not"
    ra = mdb.models[instance.part.modelName].rootAssembly
    if ra.isLocked:
        return False # Instances under locked assemblies are not considered to be meshed
    meshStats = ra.getMeshStats(regions=(instance,))
    if meshStats.numTetElems == 0: # If there are no tetrahedrons then the instance is not meshed
        return False
    if meshStats.numMeshedRegions != len(instance.cells): # Number of meshed regions must equal number of cells
        return False
    meshQuality = ra.verifyMeshQuality(regions=(instance,), criterion=ANALYSIS_CHECKS)
    if len(meshQuality["failedElements"]) != 0: # If there are elements with failed mesh-quality then it is not considered meshed
        return False
    return True

# Creates an HTML report for the automatic meshing process
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def htmlReport(myReport, goodParts, badParts, goodInstances, badInstances, meshedParts, meshedInstances, addInfoList):
    "Creates an HTML report for the automatic meshing process"
    # Create report of successful/failed parts
    lines = []
    lines.append('<html>\n')
    lines.append('\t<head>\n')
    lines.append('\t\t<!--Auto Mesher Report Generated by Auto Mesher -->\n')
    lines.append('\t\t<title>Auto Meshing Report</title>\n')
    lines.append('\t</head>\n')
    lines.append('\t<body>\n')
    lines.append('\t\t<h1>Auto Meshing Report</h1>\n')
    lines.append('\t\t<p>Report written at: %s</p>\n' %time.ctime(time.time())) 
    lines.append('\t\t<p>Meshing Summary:</p>\n')
    flagGood = False
    str1Good = ''
    str2Good = ''
    if (len(goodParts) + len(goodInstances)) > 0:
        flagGood = True
        str1Good = '<a href="#success">'
        str2Good = '</a>'
    flagBad = False
    str1Bad = ''
    str2Bad = ''
    if (len(badParts) + len(badInstances)) > 0:
        flagBad = True
        str1Bad = '<a href="#fail">'
        str2Bad = '</a>'
    flagMeshed = False
    str1Meshed = ''
    str2Meshed = ''
    if (len(meshedParts) + len(meshedInstances)) > 0:
        flagMeshed = True
        str1Meshed = '<a href="#meshed">'
        str2Meshed = '</a>'
    lines.append('\t\t<li>Number of ' + str1Good  + 'meshed parts and instances' + str2Good + ': ' + str(len(goodParts)+len(goodInstances)) + '</li>\n')
    lines.append('\t\t<li>Number of ' + str1Bad + 'un-meshed or partially meshed parts and instances' + str2Bad + ': ' + str(len(badParts)+len(badInstances)) + '</li>\n')
    lines.append('\t\t<li>Number of ' + str1Meshed + 'already meshed parts and instances' + str2Meshed + ': ' + str(len(meshedParts)+len(meshedInstances)) + '</li>\n')
    flag = False
    if (len(goodParts) + len(goodInstances) + len(badParts) + len(badInstances) + len(meshedParts) + len(meshedInstances)) > 0:
        flag = True
        lines.append('\t\t<table border="0">\n')
    if flagGood:
        lines.append('\t\t<tr><td colspan="2"><h3><br /><a name="success">Successfully meshed parts and instances:</a></h3></td></tr>\n')
    for part in goodParts:
        appendPartInformationToHTML(part, lines, addInfoList, True)
    for instance in goodInstances:
        appendInstanceInformationToHTML(instance, lines, addInfoList, True)
    if flagBad:
        lines.append('\t\t<tr><td colspan="2"><hr /><br /><br/><h3><a name="Fail">Un-meshed/Partially Meshed parts and instances:</a></h3></td></tr>\n')
    for part in badParts:
        appendPartInformationToHTML(part, lines, addInfoList, False)
    for instance in badInstances:
        appendInstanceInformationToHTML(instance, lines, addInfoList, False)
    if flagMeshed:
        lines.append('\t\t<tr><td colspan="2"><hr /><br /><br/><h3><a name="meshed">Already meshed parts and instances:</a></h3></td></tr>\n')
    for part in meshedParts:
        appendPartInformationToHTML(part, lines, addInfoList, False)
    for instance in meshedInstances:
        appendInstanceInformationToHTML(instance, lines, addInfoList, False)
    if flag:
        lines.append('\t\t</table>\n')
    lines.append('\t\t<br /><br />\n')
    lines.append('\t</body>\n')
    lines.append('</html>\n')
    # open file and write it out
    fname = myReport
    f = open(fname, 'w')
    f.writelines(lines)
    f.close()

# This function gets the path of the image file so that it is stored in the same directory as the html report
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getImageFileName(myReport, entityName, modelName):
    "This function gets the path of the image file so that it is stored in the same directory as the html report"
    reportPath, htmlPath = os.path.split(myReport)
    return os.path.join(reportPath, getValidImageFilename(entityName, modelName))

# Given a part finds out if can be meshed by this plug-in
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def is3DMeshableAndNonOrphan(part):
    "Given a part finds out if can be meshed by this plug-in"
    if part.space != THREE_D:
        print part.name + ": Part must be 3D Solid"
        return (False, True)
    if part.type != DEFORMABLE_BODY:
        print part.name + ": Part must be deformable"
        return (False, True)
    queryGeom = part.queryGeometry(printResults=FALSE, relativeAccuracy=1.0)
    if (queryGeom['category'] == 'OrphanMesh'):
        print part.name + ": Orphan mesh does not need meshing"
        return (False, False)
    attr = part.queryAttributes(printResults=False)
    if attr == None:
        print part.name + ": Does not have active features"
        return (False, True)
    if attr['numShellFaces'] != 0 or attr['numWireEdges'] != 0:
        print part.name + ": Cannot handle parts with shell and wire-features"
        return (False, True)
    if attr['numCells'] == 0:
        print part.name + ": Atleast one solid cell must exist"
        return (False, True)
    return (True, True)

# This function looks through all available parts and instances and finds out all objects that have to be meshed
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def getPartsAndInstancesToBeMeshed(meshedParts=[], meshedInstances=[], badParts=[], badInstances=[]):
    "This function looks through all available parts and instances and finds out all objects that have to be meshed"
    print "Obtaining a list of parts and instances that should be meshed"
    partRet = []
    instanceRet = []
    modelNames = mdb.models.keys()
    for modelName in modelNames:
        mdl = mdb.models[modelName]
        independentPartNames = []
        instances = mdl.rootAssembly.instances
        for instanceName in instances.keys():
            if instances[instanceName].dependent == 0:
                independentPartNames.append(instances[instanceName].part.name)
                (valid, native)= is3DMeshableAndNonOrphan(instances[instanceName].part)
                if valid:
                    if doesInstanceHaveValidMesh(instances[instanceName]):
                        meshedInstances.append(instances[instanceName])
                        print instanceName + ": This instance is already meshed"
                    else:
                        instanceRet.append(instances[instanceName])
                else:
                    if native:
                        badInstances.append(instances[instanceName])
                    else:
                        meshedInstances.append(instances[instanceName])
                        print instanceName + ": This instance is based on an orphan mesh"
        partNames = mdl.parts.keys()
        for partName in partNames:
            if partName in independentPartNames:
                continue
            if doesPartHaveValidMesh(mdl.parts[partName]):
                meshedParts.append(mdl.parts[partName])
                print partName + ": This part is already meshed"
                continue
            (valid, native) = is3DMeshableAndNonOrphan(mdl.parts[partName])
            if valid:
                partRet.append(mdl.parts[partName])
            else:
                if native:
                    badParts.append(mdl.parts[partName])
                else:
                    meshedParts.append(mdl.parts[partName])
                    print partName + ": This part has an orphan mesh"
    # Remove suppressed instances
    curI = 0
    for i in range(len(instanceRet)):
        instance = instanceRet[curI]
        ra = mdb.models[instance.part.modelName].rootAssembly
        if ra.features[instance.name].isSuppressed():
            badInstances.append(instance)
            del instanceRet[curI]
            curI -= 1
        curI += 1
    return (partRet, instanceRet)

# This function seeds a part
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def setPartMeshControls(part, mapOn, incSize, refineLevel):
    "This function seeds a part"
    if part.getPartSeeds(attribute=SIZE) == 0.0 or part.name == 'special part' or refineLevel > 0:
        seedSize = part.getPartSeeds(attribute=DEFAULT_SIZE)
        if (part.name == 'special part'):  # Special Case
            seedSize = 1.75 # TBD: Custom Code
        seedSize = seedSize * pow(refinePerLevel, refineLevel)
        try:
            part.seedPart(size=seedSize, deviationFactor=devFactor)
        except:
            print part.name, ": Could not assign seeds"
            return False
    # Force mesh controls even if some other controls already exist
    try:
        rg = tuple([c for c in part.cells])
        part.setMeshControls(regions=rg, elemShape=TET, technique=FREE, allowMapped=mapOn)
        if (incSize):
            rg = tuple([c for c in part.cells])
            part.setMeshControls(regions=rg, sizeGrowth=MODERATE)
    except:
        print part.name + ": Could not assign mesh controls"
        return False
    return True

# This function meshes a part
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def meshPart(part):
    "This function meshes a part"
    try:
        part.generateMesh()
    except:
        pass
    else:
        if doesPartHaveValidMesh(part):
            return True
    return False

# This function seeds an instance
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def setInstanceMeshControls(instance, mapOn, incSize, refineLevel):
    "This function seeds an instance"
    ra = mdb.models[instance.part.modelName].rootAssembly
    if ra.getPartSeeds(instance, attribute=SIZE) == 0.0 or instance.name == 'special part' or refineLevel > 0:
        seedSize = ra.getPartSeeds(instance, attribute=DEFAULT_SIZE)
        if (instance.name == 'special part'):  # Special Case
            seedSize = 1.75 # TBD: Custom Code
        seedSize = seedSize * pow(refinePerLevel, refineLevel)
        try:
            ra.seedPartInstance((instance, ), size=seedSize, deviationFactor=devFactor)
        except:
            print instance.name, ": Could not assign seeds"
            return False
    # Force mesh controls even if some other controls already exist
    try:
        rg = tuple([c for c in instance.cells])
        ra.setMeshControls(regions=rg, elemShape=TET, technique=FREE, allowMapped=mapOn)
        if (incSize):
            rg = tuple([c for c in instance.cells])
            ra.setMeshControls(regions=rg, sizeGrowth=MODERATE)
    except:
        print instance.name + ": Could not assign mesh controls"
        return False
    return True

# This function meshes an instance
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def meshInstance(instance):
    "This function meshes an instance"
    ra = mdb.models[instance.part.modelName].rootAssembly
    try:
        ra.generateMesh((instance,))
    except:
        pass
    else:
        if doesInstanceHaveValidMesh(instance):
            return True
    return False

# This function checks is a face can be meshed or not by meshing it
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def checkIfFaceIsMeshable(face, baseObj):
    "This function checks is a face can be meshed or not by meshing it"
    try:
        baseObj.generateMesh(regions=(face,), boundaryPreview=ON)
    except:
        return False
    else:
        meshStats = baseObj.getMeshStats(regions=(face,))
        if meshStats.numTetBoundaryElems != 0:
            baseObj.deleteMesh(regions=(face,))
            return True
        else:
            baseObj.deleteMesh(regions=(face,))
            return False

# This function uses virtual topology to ignore all edges of a face
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def IgnoreAllEdgesOfFace(face, baseObj, geom):
    "This function uses virtual topology to ignore all edges of a face"
    edgeIndexList = face.getEdges()
    edgeList = [geom.edges[i:i+1] for i in edgeIndexList]
    edgeTuple = tuple(edgeList)
    try:
        fe = baseObj.ignoreEntity(entities=edgeTuple)
    except:
        return False
    return True

# This function applies virtual topology to a part or an instance
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def ignoreEdges(geom, maxTry, baseObj):
    "This function applies virtual topology to a part or an instance"
    infLoop = maxVirtualTopologies # Maximum maxVirtualTopologies - 1 faces will be ignored
    for i in range(infLoop): # Prevent the program from going into infinite loop
        faceList = [face for face in geom.faces]
        ret = False
        for face in faceList:
            if not checkIfFaceIsMeshable(face, baseObj):
                ret = IgnoreAllEdgesOfFace(face, baseObj, geom)
                if not ret:
                    return False  # Something went wrong. Leave it to the user
                break  # Restart because faces have changed
        if not ret:
            return True
    return False # It came here, means it went into infinite loop

# This function does intelligent meshing of a part. i.e. It tries to re-mesh part if it could not mesh it earlier
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def intelligentMeshPart(part, mapOn, incSize, maxTry, applyVT):
    "This function does intelligent meshing of a part. i.e. It tries to re-mesh part if it could not mesh it earlier"
    if part.isLocked():
        try:
            part.Unlock(reportWarnings=False)
        except:
            print part.name, ": Failed to unlock. Skipping it"
            return (False, 0)
    for refineLevel in range(maxTry):
        if setPartMeshControls(part, mapOn, incSize, refineLevel):
            if meshPart(part):
                return (True, 0)
        else:
            return (False, 0)
    if applyVT:
        before = part.features.keys()
        if ignoreEdges(part, maxTry, part):
            if meshPart(part):
                after = part.features.keys()
                return (True, len(after)-len(before))
        after = part.features.keys()
        for afKey in after:
            if afKey not in before:
                del part.features[afKey]  # Delete New features
        after = part.features.keys()
        return (False, len(after)- len(before))
    return (False, 0)

# This function does intelligent meshing of an instance. i.e. It tries to re-mesh instance if it could not mesh it earlier
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def intelligentMeshInstance(instance, mapOn, incSize, maxTry, applyVT):
    "This function does intelligent meshing of an instance. i.e. It tries to re-mesh instance if it could not mesh it earlier"
    ra = mdb.models[instance.part.modelName].rootAssembly
    if ra.isLocked:
        print instance.name, ": Failed to unlock the assembly for this instance. Skipping it"
        return (False, 0)
    for refineLevel in range(maxTry):
        if setInstanceMeshControls(instance, mapOn, incSize, refineLevel):
            if meshInstance(instance):
                return (True, 0)
        else:
            return (False, 0)
    if applyVT:
        before = ra.features.keys()
        if ignoreEdges(instance, maxTry, ra):
            if meshInstance(instance):
                after = ra.features.keys()
                return (True, len(after)-len(before))
        after = ra.features.keys()
        for afKey in after:
            if afKey not in before:
                del ra.features[afKey] # Delete New Features
        after = ra.features.keys()
        return (False, len(after) - len(before))
    return (False, 0)

# This function tries to unlock all assemblies in the model
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def unlockAssemblies():
    "This function tries to unlock all assemblies in the model"
    modelNames = mdb.models.keys()
    for modelName in modelNames:
        ra = mdb.models[modelName].rootAssembly
        if ra.isLocked:
            try:
                print modelName, ": Unlocking the assembly for this model"
                ra.unlock()
            except:
                print modelName, ": Failed to unlock the assembly for this model"
            else:
                print modelName, ": Unlock successful"

# This function meshes all parts & instances
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def autoMesh(maxTry=defaultTries, reportFile='autoReport.html', mapOn=False, showReport=True, incSize=False, applyVT=True):
    "This function meshes all parts & instances"
    meshedParts =[]
    meshedInstances = []
    goodParts = []
    badParts = []
    goodInstances = []
    badInstances = []
    addInfoList = {}
    (partList, instanceList) = getPartsAndInstancesToBeMeshed(meshedParts, meshedInstances, badParts, badInstances)
    unlockAssemblies()
    imageOnlyParts = meshedParts + badParts
    imageOnlyInstances = meshedInstances + badInstances
    for part in imageOnlyParts:
        vp = session.viewports[session.currentViewportName]
        vp.setValues(displayedObject=part)
        vp.view.setValues(session.views['Iso'])
        vp.partDisplay.setValues(mesh=ON)
        session.printToFile(fileName=getImageFileName(reportFile, part.name, part.modelName), format=PNG, canvasObjects=(vp, ))
    for instance in imageOnlyInstances:
        vp = session.viewports[session.currentViewportName]
        ra = mdb.models[instance.part.modelName].rootAssembly
        vp.setValues(displayedObject=ra)
        vp.assemblyDisplay.setValues(visibleInstances=(instance.name, ))
        vp.view.setValues(session.views['Iso'])
        vp.view.fitView()
        if not ra.isLocked:
            ra.regenerate()
        vp.assemblyDisplay.setValues(mesh=ON)
        ra.regenerate()
        session.printToFile(fileName=getImageFileName(reportFile, instance.name, instance.part.modelName), format=PNG, canvasObjects=(vp, ))
    for part in partList:
        vp = session.viewports[session.currentViewportName]
        vp.setValues(displayedObject=part)
        vp.view.setValues(session.views['Iso'])
        success, addInfo = intelligentMeshPart(part, mapOn, incSize, maxTry, applyVT)
        addInfoList[part.modelName + '###Part###' + part.name] = addInfo
        if success:
            print part.name, ": MESHED!"
            goodParts.append(part)
            vp.partDisplay.setValues(mesh=ON)
        else:
            print part.name, ": NOT MESHED"
            badParts.append(part)
        session.printToFile(fileName=getImageFileName(reportFile, part.name, part.modelName), format=PNG, canvasObjects=(vp, ))
    for instance in instanceList:
        vp = session.viewports[session.currentViewportName]
        ra = mdb.models[instance.part.modelName].rootAssembly
        vp.setValues(displayedObject=ra)
        vp.assemblyDisplay.setValues(visibleInstances=(instance.name, ))
        vp.view.setValues(session.views['Iso'])
        vp.view.fitView()
        vp.assemblyDisplay.setValues(mesh=ON)
        if not ra.isLocked:
            ra.regenerate()
        success, addInfo = intelligentMeshInstance(instance, mapOn, incSize, maxTry, applyVT)
        addInfoList[instance.part.modelName + '###Instance###' + instance.name] = addInfo
        if success:
            print instance.name, ": MESHED!"
            goodInstances.append(instance)
            ra.regenerate()
        else:
            print instance.name, ": NOT MESHED"
            badInstances.append(instance)
            if not ra.isLocked:
                ra.regenerate()
        session.printToFile(fileName=getImageFileName(reportFile, instance.name, instance.part.modelName), format=PNG, canvasObjects=(vp, ))
        allInstances = mdb.models[instance.part.modelName].rootAssembly.instances.keys()
        vp.assemblyDisplay.setValues(visibleInstances=tuple(allInstances))
    htmlReport(reportFile, goodParts, badParts, goodInstances, badInstances, meshedParts, meshedInstances, addInfoList)
    for modelName in mdb.models.keys():
        try:
            mdb.models[modelName].rootAssembly.regenerate()
        except:
            print modelName, ": Root assembly for this model failed to regenerate. Please manually re-generate this assembly"
    if (showReport):
        currentDir = os.getcwd()
        uti.webBrowser.displayURL(os.path.join(currentDir, reportFile))
    return
