from abaqusGui import *
from abaqusConstants import TRUE, FALSE

###########################################################################
# Class definition
###########################################################################

class CdhFastenerDB(AFXDataDialog):

    [
        ID_OUTFILE_SELECT,
        ID_BULKDATA_SELECT,
        ID_LAST
    ] = range(AFXDialog.ID_LAST, AFXDialog.ID_LAST+3)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):
        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, 'CDH Fasteners', 
            self.OK|self.CANCEL,DECOR_RESIZE)

        self.form = form
        self.fileDbOut = self.fileDbBulk = None

        # MAP the select FX button to the method of CLASS FastenerDB.onCmdSelect
        #
        FXMAPFUNC(self, SEL_COMMAND, self.ID_OUTFILE_SELECT, CdhFastenerDB.onCmdOutFileSelect)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_BULKDATA_SELECT, CdhFastenerDB.onCmdBulkDataSelect)

        # Set up widgets for file input
        # 
        gb = FXGroupBox(self, 'NASTRAN BDF file name:', FRAME_GROOVE|LAYOUT_FILL_X)
        hf = FXHorizontalFrame(gb, LAYOUT_FILL_X,0,0,0,0,0,0,0,0)
        self.bdfText = AFXTextField(hf, 25, 'File name:', form.bulkDataFileKw, 0, LAYOUT_FILL_X)
        FXButton(hf, 'Select...', None, self, self.ID_BULKDATA_SELECT)

        # Output file data
        #
        gb = FXGroupBox(self, 'Output', FRAME_GROOVE|LAYOUT_FILL_X)
        hf = FXHorizontalFrame(gb, LAYOUT_FILL_X,0,0,0,0,0,0,0,0)
        self.inputText = AFXTextField(hf, 25, 'File name:', form.outputFileNameKw, 0, LAYOUT_FILL_X)
        FXButton(hf, 'Select...', None, self, self.ID_OUTFILE_SELECT)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdBulkDataSelect(self, sender, sel, ptr):

        # This dialog interacts with the AFXFileSelectorDialog
        # to enable the user to select a bulk data file.
        #
        if not self.fileDbBulk:            
            patterns = 'Bulk Data Files (*.bdf)\nDAT file (*.dat)\nAll files (*.*)'
            self.fileDbBulk = AFXFileSelectorDialog(self, 'Select a Nastran Bulk Data File',
                                                    self.form.bulkDataFileKw, None,
                                                    AFXSELECTFILE_ANY|AFXSELECTFILE_REMOTE_HOST, patterns, None)
            self.fileDbBulk.create()
        self.fileDbBulk.showModal()
        return 1
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdOutFileSelect(self, sender, sel, ptr):
        
        # This dialog interacts with the AFXFileSelectorDialog
        # to enable the user to select a file.
        #
        if not self.fileDbOut:
            patterns = 'Input files (*.inp)\nText Files (*.txt)\nAll files (*.*)'
            self.fileDbOut = AFXFileSelectorDialog(self, 'Select the Output File',
                                                    self.form.outputFileNameKw, None,
                                                    AFXSELECTFILE_ANY|AFXSELECTFILE_REMOTE_HOST, patterns, None)
            self.fileDbOut.create()
        self.fileDbOut.showModal()
        return 1

