from abaqusGui import *
from cDHSPT.cdhFastenerDB import CdhFastenerDB

###########################################################################
# Class definition
###########################################################################

class CdhFastenerForm(AFXForm):
    
    [
        ID_OVERWRITE,
        ID_OK,
        ID_LAST
    ] = range(AFXForm.ID_LAST, AFXForm.ID_LAST+3)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner):

        # Construct the base class.
        #
        AFXForm.__init__(self, owner)

        # Create the Map function to handle the overwrite request on the
        # output file.
        #
        FXMAPFUNC(self, SEL_COMMAND, self.ID_OVERWRITE, CdhFastenerForm.onCmdOverwrite)

        # Create the command and keywords.
        #
        self.cmd = AFXGuiCommand(self, 'cdhCreateFasteners', 'cDHSPT.cdhFastenerModule')

        # File I/O related Keywords
        #
        self.bulkDataFileKw = AFXStringKeyword(self.cmd, 'bulkDataFileName', FALSE, '')
        self.outputFileNameKw = AFXStringKeyword(self.cmd, 'outputFileName', TRUE, '')

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomTasks(self):

        # Report back to the user that output was successful
        #
        mw = getAFXApp().getAFXMainWindow()
        mw.writeToMessageArea('The output file has been written to %s' % \
                                 self.outputFileNameKw.getValue())

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstDialog(self):

        return CdhFastenerDB(self)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def verifyKeywordValues(self):

        # Method to check if the odbName is valid
        # 
        currentDb = self.getCurrentDialog()
        
        if self.bulkDataFileKw.getValue().strip() == '':
            showAFXErrorDialog(currentDb,'BDF file name must be specified.')
            currentDb.bdfText.setFocusToTextField()
            return FALSE
        if self.outputFileNameKw.getValue().strip() == '':
            showAFXErrorDialog(currentDb,'ODB file name must be specified.')
            currentDb.inputText.setFocusToTextField()
            return FALSE

        return TRUE

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):


        # Check to see if the output file already exists
        #
        import os
        if os.path.exists(self.outputFileNameKw.getValue()):
            db = self.getCurrentDialog()
            showAFXWarningDialog(db, 'Output file already exists.\n\nOK to overwrite?',
                AFXDialog.YES|AFXDialog.NO, self, self.ID_OVERWRITE)
            return 0
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdOverwrite(self, sender, sel, ptr):
 
        # If user allowed overwrite, then continue, otherwise just return.
        #
        if sender.getPressedButtonId() == AFXDialog.ID_CLICKED_YES:
            AFXForm.issueCommands(self)
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def handleException(self, exception):
    
        exceptionType = exception[0]
        exceptionValue = exception[1]

        # Post an error dialog
        #
        db = self.getCurrentDialog()
        showAFXErrorDialog(db, str(exceptionValue))