#=========================================================================
# Kernal Script to Generate an input deck from a BDF file
#=========================================================================
#

import posixpath
try:
    import Numeric
except:
    import numpy.oldnumeric as Numeric


###########################################################################
# Class definition
###########################################################################

class Node:
    def __init__(self, x, y, z):
        sa = myatof
        self.x=sa(x)
        self.y=sa(y)
        self.z=sa(z)

###########################################################################
# Class definition
###########################################################################

class NewNode:
    def __init__(self, nnum, nx, ny, nz):
        self.nnum=nnum
        self.node=Node(x=nx,y=ny,z=nz)

###########################################################################
# Class definition
###########################################################################

class Format:

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self):
        
        self.dataLines = None
        self.nodes = {}
        self.elementData = []
        self.newNodes = {}

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def splitLine(self, line):
        # Method to split the 80 char data line into a list of fields
        #

        return (line[0:8].strip(), line[8:16].strip(), line[16:24].strip(),
                line[24:32].strip(), line[32:40].strip(), line[40:48].strip(),
                line[48:56].strip(), line[56:64].strip(), line[64:72].strip(),
                line[72:80].strip())

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def processNodes(self):
        # Make sure all lines are at least 80 characters in length.
        # Process lines and add to node or element definitions
        
        print 'Reading raw data'
        getContinueLine = 0
        total = len(self.dataLines)

        for i in range(total):
            self.dataLines[i] = self.dataLines[i].strip()

            # Check line length. If less than 80 chars, bring
            # up to 80 chars. 
            lineLength = len(self.dataLines[i])
            
            # Add spaces
            #
            rawLine = self.dataLines[i] + ' '*(80-lineLength)
            
            if rawLine[:4].lower() == 'grid':
                line = self.splitLine(line=rawLine)
                self.nodes[line[1]] = Node(x=line[3],y=line[4],z=line[5])
            elif rawLine[:4].lower() == 'chex':
                self.elementData.append(rawLine)
                getContinueLine = 1
            elif rawLine[:1].lower() == '+' and getContinueLine:
                self.elementData.append(rawLine)
                getContinueLine = 0

            # Print progress
            #
            if i%1000 == 0:
                print 'Processed %i Lines of %i' % (i+1, total)
        print 'Processed %i Lines of %i' % (i+1, total)
        # Free up some memory
        #
        del self.dataLines
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def processElements(self):

        # Read element definitions and calculate output data
        #
        print '\n\nCalculating new nodes'
        nodeNumber = 10000000

        sr = Numeric.sqrt
        sp = Numeric.power

        searchRfactor = 0.3

        # Loop over elements and get node numbers.
        # g's refer to node numbers, d's are dummy variables
        #i = 0
        #
        for i in range(0,len(self.elementData),2):
            d1, d2, d3, g1, g2, g3, g4, g5, g6, d4 = \
                self.splitLine(line=self.elementData[i])
            d1, g7, g8, d2, d3, d4, d5, d6, d7, d8 = \
                self.splitLine(line=self.elementData[i+1])
            nx = (self.nodes[g1].x + self.nodes[g7].x)/2. 
            ny = (self.nodes[g1].y + self.nodes[g7].y)/2. 
            nz = (self.nodes[g1].z + self.nodes[g7].z)/2.
            
            searchR = sr(sp(self.nodes[g1].x - self.nodes[g5].x, 2.) +
                         sp(self.nodes[g1].y - self.nodes[g5].y, 2.) +
                         sp(self.nodes[g1].z - self.nodes[g5].z, 2.))/searchRfactor
            
            influenceR1 = sr(sp(self.nodes[g1].x - self.nodes[g2].x, 2.) +
                             sp(self.nodes[g1].y - self.nodes[g2].y, 2.) +
                             sp(self.nodes[g1].z - self.nodes[g2].z, 2.))
            influenceR2 = sr(sp(self.nodes[g1].x - self.nodes[g4].x, 2.) +
                             sp(self.nodes[g1].y - self.nodes[g4].y, 2.) +
                             sp(self.nodes[g1].z - self.nodes[g4].z, 2.))
            influenceR = sr((influenceR1*influenceR2)/(2.*Numeric.pi))
            try:
                self.newNodes[(round(searchR, 1), round(influenceR, 1))].append(
                                              NewNode(nnum=nodeNumber, nx=nx,
                                                      ny=ny, nz=nz))
            except:
                self.newNodes[(round(searchR, 1), round(influenceR, 1))] = \
                                              [NewNode(nnum=nodeNumber, nx=nx,
                                                       ny=ny, nz=nz),]
                                                      
            nodeNumber += 1
                             
            # Print progress
            #
            if i%2000 == 0:
                print 'Processed %i Nodes' % (i/2+1,)
        print 'Processed %i Nodes' % (i/2+1,)
        # Free up some memory
        #
        del self.nodes
        del self.elementData

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def printOutput(self):
        # Formats and prints output
        #
        out = open(self.fileName, 'w')
        
        for key in self.newNodes.keys():
            key0str = str(key[0]).replace('.', '-')
            key1str = str(key[1]).replace('.', '-')
            nsetName = 'Weld_nset_%s_%s' % (key0str,key1str)
            propName = 'Weld_prop_%s_%s' % (key0str,key1str)
            surfName = 'All_shells'
            
            out.write('**\n')
            out.write('** Nodes for search radius %s, influence radius %s\n'%\
                      (key[0], key[1]))
            out.write('*Node, Nset=%s\n' % nsetName)
            for nodeDef in self.newNodes[key]:
                out.write('%s, %s, %s, %s\n' % (nodeDef.nnum, nodeDef.node.x,
                                                nodeDef.node.y, nodeDef.node.z))
            # If model the fastener with an internally generated rigid beam MPC
            #
            out.write('*Fastener, interaction name=welds, property=%s, reference node set=%s,\n' \
                      'number of layers=1, search radius=%s, radius of influence=%s\n' \
                      '\n%s\n' % (propName, nsetName, key[0], key[1], surfName))
            # If model the fastener with an internally generated beam connector
            #
            out.write('*Fastener property, name=%s\n%s\n1, 6\n' % \
                      (propName, key[1]))
        out.write('*Surface, name=All_shells\nALL_SHELL_ISO, SPOS\n')
        out.close()

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def cdhCreateFasteners(bulkDataFileName, outputFileName):

    format = Format()
    format.dataLines = open(bulkDataFileName, 'r').readlines()
    format.processNodes()
    format.processElements()
    format.fileName = outputFileName
    format.printOutput()

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def myatof(x):
    x = str(x)

    if x.lower().find('e') < 0:
        if x.startswith('-'):
            newx = x[1:]
            minusIndex = newx.find('-') + 1
        else:
            minusIndex = x.find('-')
            
        if minusIndex > 0:
            newx = x[:minusIndex]+'e'+x[minusIndex:]
            x = newx
    import string
    return string.atof(x)