"""
This module provides symbolic constants for the CWELD of NVH suite
application.
"""
import os
from abaqusConstants import FALSE, TRUE
from symbolicConstants import SymbolicConstant

# CODE version related constants
#
CODE_VERSION= "1.1-1"
CODE_AUTHOR = "ABAQUS Inc."
CODE_TITLE="*FASTENER Generator"
CODE_DESCRIPTION ="Interface to translate master weld file data into equivalent *FASTENERS with either compliant or rigid behaviour."

# File I/O related default values
#
cwdPath = os.getcwd()

# Plug ins are launched from abaqus_plugins, but don't
# want cwd to be inside abaqus_plugins, so check for this.
#
if cwdPath.find('abaqus_plugins') > 0:
    cwdPath = cwdPath[:cwdPath.find('abaqus_plugins')]
    
DEFAULT_WELD_FILENAME = ''
DEFAULT_OUT_FILENAME = os.path.join(cwdPath, "weld_master.inp")
DEFAULT_BULKDATA_FILE = ''
absPath = os.path.abspath(__file__)
absDir  = os.path.dirname(absPath)

# Generic default constants
#
DEFAULT_RAD_INF = 2.5
DEFAULT_ECHO_OUTPUT = FALSE

# Delimiter related constants
#
COMMA = SymbolicConstant('COMMA')
SPACE = SymbolicConstant('SPACE')
TAB = SymbolicConstant('TAB')
DELIMITER_LIST = [COMMA, SPACE, TAB]
DEFAULT_DELIMITER = COMMA.getId()

# Numeric constants
#
EPS_VALUE = 1.0E-6
SURF_PER_LINE=8

# Constants related to weld format options.
#
EDIT_IMPORT = SymbolicConstant('EDIT_IMPORT')
DEFAULT_EDIT_IMPORT = EDIT_IMPORT.getId()
WELD_TABLE_HEADING = ['NODE ID', '# Sheets', 'Nx', 'Ny', 'Nz', '1st PSHELL']
DEFAULT_WELD_TABLE_IDS = '0,1,2,3,4,5'

# Contants and defaults relating to compliance
#
DATA_FROM_FILE = SymbolicConstant('DATA_FROM_FILE')
DATA_FROM_FORM = SymbolicConstant('DATA_FROM_FORM')

DEFAULT_USE_COMPLIANT = TRUE
DEFAULT_OFFSET_TYPE = DATA_FROM_FORM
DEFAULT_RADIUS_TYPE = DATA_FROM_FILE
DEFAULT_THICKNESS_TYPE = DATA_FROM_FILE
DEFAULT_MATERIAL_TYPE = DATA_FROM_FILE

DEFAULT_NODE_OFFSET = 0
DEFAULT_MODULUS = 200000.0
DEFAULT_POISSON = 0.3
DEFAULT_THICKNESS = 0.1

# NASTRAN card symbolic constants
VALID_NASTRAN_CARDS = 'PSHELL|MAT1|CWELD|PWELD'
