from abaqusGui import *
from abaqusConstants import TRUE, FALSE
from fastenerConst import *

###########################################################################
# Class definition
###########################################################################

class FastenerDB(AFXDataDialog):

    [
        ID_WELDFILE_SELECT,
        ID_OUTFILE_SELECT,
        ID_LOAD_DEFAULT,
        ID_BULKDATA_SELECT,
        ID_EDIT_WELD_FORMAT,
        ID_LAST
    ] = range(AFXDialog.ID_LAST, AFXDialog.ID_LAST+6)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form):
        # Construct the base class.
        #
        AFXDataDialog.__init__(self, form, CODE_TITLE, 
            self.OK|self.CANCEL,DECOR_RESIZE)
        
        self.form = form
        self.fileDbWeld = self.fileDbOut = self.fileDbBulk = None

        # MAP the select FX button to the method of CLASS FastenerDB.onCmdSelect
        #
        FXMAPFUNC(self, SEL_COMMAND, self.ID_WELDFILE_SELECT, FastenerDB.onCmdWeldFileSelect)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_OUTFILE_SELECT, FastenerDB.onCmdOutFileSelect)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_LOAD_DEFAULT, FastenerDB.onCmdLoadDefault)      
        FXMAPFUNC(self, SEL_COMMAND, self.ID_BULKDATA_SELECT, FastenerDB.onCmdBulkDataSelect)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_EDIT_WELD_FORMAT, FastenerDB.onCmdEditWeldFormat)
        
        # Master weld file widgits 
        #
        gb = FXGroupBox(self, 'Master Weld File', FRAME_GROOVE|LAYOUT_FILL_X)
        hf = FXHorizontalFrame(gb, LAYOUT_FILL_X, 0,0,0,0,0,0,0,0)
        self.weldText = AFXTextField(hf, 15, 'File name:', form.weldFileNameKw, 0, LAYOUT_FILL_X)

        FXButton(hf, 'Select...', None, self, self.ID_WELDFILE_SELECT)
                  
        vf = FXHorizontalFrame(gb, 0, 0,0,0,0,0,0,0,0)
        FXLabel(vf, 'File delimiter:')
        for i in range(len(DELIMITER_LIST)):
            FXRadioButton(vf, str(DELIMITER_LIST[i]).capitalize(),
                          form.delimiterKw, DELIMITER_LIST[i].getId())
            
        mid_hf = FXHorizontalFrame(self, LAYOUT_FILL_X, 0,0,0,0,0,0,0,0)

        # Define the tab book for t he WELD file format and Weld material
        # properties.
        #
        tabBook = FXTabBook(mid_hf, None, 0, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0)

        # Weld specific data
        #
        FXTabItem(tabBook, 'Weld Properties')
        hfTab = FXHorizontalFrame(tabBook, FRAME_THICK|FRAME_RAISED)
        
        vf = FXVerticalFrame(hfTab,LAYOUT_FILL_X|LAYOUT_LEFT,0,0,0,0,0,0,0)
        FXCheckButton(vf, 'Enable fastener compliance', form.compliantFastenerKw)

        hf = FXHorizontalFrame(vf,LAYOUT_LEFT,0,0,0,0,0,0,0)
        FXLabel(hf, 'Node ID offset:')
        self.offsetField = AFXTextField(hf, 10, ' ', form.nodeOffsetKw)

        hf = FXHorizontalFrame(vf,LAYOUT_LEFT,0,0,0,0,0,0,0)
        FXLabel(hf, 'Weld radius:')
        self.radiusRead = FXRadioButton(hf, 'Read', form.radiusTypeKw, DATA_FROM_FILE.getId())
        self.radiusSpecify = FXRadioButton(hf, 'Specify', form.radiusTypeKw, DATA_FROM_FORM.getId())
        self.radiusField = AFXTextField(hf, 10, ' ', form.radiusInfluenceKw)
        
        hf = FXHorizontalFrame(vf,LAYOUT_LEFT,0,0,0,0,0,0,0)
        FXLabel(hf, 'Material:')
        self.materialRead = FXRadioButton(hf, 'Read', form.materialTypeKw, DATA_FROM_FILE.getId())
        self.materialSpecify = FXRadioButton(hf, 'Specify', form.materialTypeKw, DATA_FROM_FORM.getId())

        va = AFXVerticalAligner(vf, LAYOUT_LEFT|LAYOUT_FILL_X, 0,0,0,0,20,0,0)

        self.elasticField = AFXTextField(va, 10, 'Elastic modulus:', form.elasticModulusKw)
        self.poissonField = AFXTextField(va, 10, 'Poisson ratio:', form.poissonRatioKw)

        hf = FXHorizontalFrame(vf,LAYOUT_LEFT,0,0,0,0,0,0,0)
        FXLabel(hf, 'Sheet thickness:')
        self.thicknessRead = FXRadioButton(hf, 'Read', form.thicknessTypeKw, DATA_FROM_FILE.getId())
        self.thicknessSpecify = FXRadioButton(hf, 'Specify', form.thicknessTypeKw, DATA_FROM_FORM.getId())
        self.thicknessField = AFXTextField(hf, 10, ' ', form.sheetThicknessKw)

        # Set up widgets for file input
        # 
        hf = FXHorizontalFrame(vf, LAYOUT_FILL_X,0,0,0,0,0,0,0,0)
        self.bulkText = AFXTextField(hf, 25, 'NASTRAN BDF file name:', form.bulkDataFileKw, 0, LAYOUT_FILL_X|AFXTEXTFIELD_VERTICAL)
        self.bulkButton = FXButton(hf, 'Select...', None, self, self.ID_BULKDATA_SELECT, LAYOUT_BOTTOM|BUTTON_NORMAL)

        # Weld table imported data to check format.
        #
        FXTabItem(tabBook, 'Weld Master File Format')
        hfTab = FXHorizontalFrame(tabBook, FRAME_THICK|FRAME_RAISED)
        
        vf = FXVerticalFrame(hfTab, LAYOUT_FILL_Y|LAYOUT_FILL_X, 0,0,0,0,0,0,0)
        tf = FXVerticalFrame(vf, FRAME_SUNKEN|FRAME_THICK, 0,0,0,0,0,0,0,0)

        nHeaders = len(WELD_TABLE_HEADING)
        headerList ='\t'.join(WELD_TABLE_HEADING)

        # Generate list for table selection
        # allow for additional 5 columns beyond default.
        #
        headerIds = '0'
        for i in range(nHeaders + 4):
            headerIds+='\t%s' % str(i+1)
   
        table = AFXTable(tf, nHeaders+1, 2, nHeaders+1, 2, None, self.ID_EDIT_WELD_FORMAT,
                         AFXTABLE_NORMAL|AFXTABLE_EDITABLE)

        table.setLeadingColumns(1)
        table.setLeadingRows(1)
        table.setLeadingRowLabels('Column #')
        table.setLeadingColumnLabels(headerList)
        table.showHorizontalGrid(TRUE) 
        table.showVerticalGrid(TRUE)
        
        listId = table.addList(headerIds)
        table.setColumnWidth(1, 80)
        table.setColumnJustify(0, AFXTable.LEFT)
        table.setColumnJustify(1, AFXTable.CENTER)
        table.setColumnType(1, AFXTable.LIST)
        table.setColumnListId(1, listId)
        table.setListMaxVisible(nHeaders)
        for i in range(nHeaders):
            table.setItemListIndex(i+1, 1, i)

        FXButton(vf, 'Reset Format', None, self, self.ID_LOAD_DEFAULT)
 
        # Store the table object for future reference
        #
        self.table = table
        
        # Output file data
        #
        gb = FXGroupBox(self, 'Output', FRAME_GROOVE|LAYOUT_FILL_X)
        hf = FXHorizontalFrame(gb, LAYOUT_FILL_X,0,0,0,0,0,0,0,0)
        self.outputText = AFXTextField(hf, 15, 'File name:', form.outputFileNameKw, 0, LAYOUT_FILL_X)
        FXButton(hf, 'Select...', None, self, self.ID_OUTFILE_SELECT)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdWeldFileSelect(self, sender, sel, ptr):
        
        # This dialog interacts with the AFXFileSelectorDialog
        # to enable the user to select a file.
        #
        if not self.fileDbWeld:          
            patterns = 'Weld Master Files (*.wmf)\nText Files (*.txt)\nAll files (*.*)'
            self.fileDbWeld = AFXFileSelectorDialog(self, 'Select the Master Weld File',
                                                    self.form.weldFileNameKw, None, 
                                                    AFXSELECTFILE_ANY|AFXSELECTFILE_REMOTE_HOST, patterns, None)
            self.fileDbWeld.create()
        self.fileDbWeld.showModal()
        return 1    

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdBulkDataSelect(self, sender, sel, ptr):

        # This dialog interacts with the AFXFileSelectorDialog
        # to enable the user to select a bulk data file.
        #
        if not self.fileDbBulk:            
            patterns = 'Bulk Data Files (*.bdf)\nDAT file (*.dat)\nAll files (*.*)'
            self.fileDbBulk = AFXFileSelectorDialog(self, 'Select a Nastran Bulk Data File',
                                                    self.form.bulkDataFileKw, None, AFXSELECTFILE_ANY, patterns, None)
            self.fileDbBulk.create()
        self.fileDbBulk.showModal()
        return 1
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdOutFileSelect(self, sender, sel, ptr):
        
        # This dialog interacts with the AFXFileSelectorDialog
        # to enable the user to select a file.
        #
        if not self.fileDbOut:
            patterns = 'Input files (*.inp)\nText Files (*.txt)\nAll files (*.*)'
            self.fileDbOut = AFXFileSelectorDialog(self, 'Select the Output File',
                                                    self.form.outputFileNameKw, None, AFXSELECTFILE_ANY, patterns, None)
            self.fileDbOut.create()
        self.fileDbOut.showModal()
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdLoadDefault(self, sender, sel, ptr):

        # this command re-loads the default node formats into the
        # self.tableObj
        
        nHeaders = len(WELD_TABLE_HEADING)
        # move through and reset index to i
        #
        for i in range(nHeaders):
            self.table.setItemListIndex(i+1, 1, i)
            if i!=0:
                weldTableStr += ',%s' % str(i+1)
            else:
                weldTableStr = str(i+1)

        self.form.weldTableFormatKw.setValue(weldTableStr)
            
        return 1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdEditWeldFormat(self, sender, sel, ptr):

        # This command stores changes into the weldTableFormatKw
        # object keyword.

         nHeaders = len(WELD_TABLE_HEADING)
         for i in range(nHeaders):
             if i!=0:
                 weldTableStr = weldTableStr + ',%s' % self.table.getItemListIndex(i+1,1)
             else:
                 weldTableStr = str(self.table.getItemListIndex(i+1,1))
         self.form.weldTableFormatKw.setValue(weldTableStr)

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def processUpdates(self):
                
        # This function re-formats the dialog and enabling and disabling
        # as required.
        #
        # Store changes in table to the weldTable format Kw.
        #
        import os
        
        nHeaders = len(WELD_TABLE_HEADING)
        for i in range(nHeaders):
            if i!=0:
                weldTableStr = weldTableStr + ',%s' % self.table.getItemListIndex(i+1,1)
            else:
                weldTableStr = str(self.table.getItemListIndex(i+1,1))
        self.form.weldTableFormatKw.setValue(weldTableStr)

        # Update form properties
        #
        if not self.form.compliantFastenerKw.getValue():
            self.thicknessRead.disable()
            self.thicknessSpecify.disable()
            self.materialRead.disable()
            self.materialSpecify.disable()
        else:
            self.thicknessRead.enable()
            self.thicknessSpecify.enable()
            self.materialRead.enable()
            self.materialSpecify.enable()
            
        if self.form.radiusTypeKw.getValue() == DATA_FROM_FILE.getId():
            self.radiusField.disable()           
        else:
            self.radiusField.enable()

        if self.form.materialTypeKw.getValue() == DATA_FROM_FILE.getId() or \
           not self.form.compliantFastenerKw.getValue() :
            self.elasticField.disable()
            self.poissonField.disable()
        else:
            self.elasticField.enable()
            self.poissonField.enable()
    
        if self.form.thicknessTypeKw.getValue() == DATA_FROM_FILE.getId() or \
           not self.form.compliantFastenerKw.getValue() :
            self.thicknessField.disable()
        else:
            self.thicknessField.enable()

        # Turn off the bulk file data if all requests are from the form
        #
        if (self.form.materialTypeKw.getValue() == DATA_FROM_FILE.getId() or \
            self.form.thicknessTypeKw.getValue() == DATA_FROM_FILE.getId()) and \
            self.form.compliantFastenerKw.getValue():
            self.bulkText.enable()
            self.bulkButton.enable()
        elif self.form.radiusTypeKw.getValue() == DATA_FROM_FILE.getId():
            self.bulkText.enable()
            self.bulkButton.enable()
        else:
            self.bulkText.disable()
            self.bulkButton.disable()