#========================================================================
# This module stores functions called by the main fastener generation
# program.
#=========================================================================
from fastenerConst import *
import os
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def importFileData(fileName):
    """ This function opens the file data and returns an simple array
    back as data """
    import os

    # read data into dataLines array from a valid filename.
    #
    dataLines=open(fileName,'r').readlines()
    return dataLines

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def interpretFileFormat(dataLines, delimiter, weldTableFormat):
    """ This function will strip out data from dataLines and determine
    the appropriate column format. This process determined by the weldTable
    format. Default columns ID's defined in fastenerConst module"""

    # initialise
    #
    foundRecord =0

    # define nodal data and sheet data as a dictionary
    # keyed to the nodal ID
    #
    nodes = {}
    sheets = {}
    uniqueSheet = []

    # determine the total number records in the inputFile
    #
    total = len(dataLines)
    print "\n** Importing a total of " + str(total) + " weld definitions"

    # Interpret the delimiter STRING.
    #
    if delimiter == COMMA:
        delStr = ","
    elif delimiter == SPACE:
        delStr = " "
    elif delimiter == TAB:
        delStr = chr(9)
        
    # Determine actual column ID's...
    # Current version of the code defaults will be assumed.
    # -1 is because arrays start from 0, ids start from 1.
    #
    # The weldTableFormat is comma delimited...
    # parse out the str.
    #
    weldTableFormat = weldTableFormat.split(',')
    print "** Master weld input file format \nNODE ID, NSHEETS, X, Y, Z, PSHELL_ID_1"
    print weldTableFormat

    # Get index values from format definition.
    #
    DATA_NODEID = int(weldTableFormat[0])
    DATA_NSHEETS = int(weldTableFormat[1])
    DATA_NODEX = int(weldTableFormat[2])
    DATA_NODEY = int(weldTableFormat[3])
    DATA_NODEZ = int(weldTableFormat[4])
    DATA_SHEET = int(weldTableFormat[5])

    # move through records parsing node data by delimiter
    #
    for i in range(total):
        data = dataLines[i].strip()
        data = data.split(delStr)
        entries = len(data)

        # If the particular row does not have enough entries skip
        #
        if entries > 5:
            # Use found record to determine if
            # at least one record was found with enough entries.
            #
            foundRecord+=1
            
            # store data into dictionaries using
            # default options
            #
            try:
                nodeID = int(data[DATA_NODEID])
                nodeX = float(data[DATA_NODEX])
                nodeY = float(data[DATA_NODEY])
                nodeZ = float(data[DATA_NODEZ])
                nodes[nodeID] = [nodeX,nodeY,nodeZ] 

                # The code will assume column 'DATA_NSHEETS' contains the number of
                # sheets, no limit to the number of sheets will be specifed.
                # 
                nSheets = int(data[DATA_NSHEETS])
                sheetArray = []
            except:
                raise TypeError, 'ERROR: Incorrect weld master file format, please refer to online help.'
    
            for j in range(nSheets):
                # assume that DATA_SHEET indexes the first occurance of
                # of a sheet in the data and that there will be (n=DATA_NSHEETS) in
                # the list.
                #
                try:
                    sheetStr = data[DATA_SHEET + j]
                    sheetArray.append(sheetStr)
    
                    # search uniqueSheet list and if not found
                    # store. will be used for generating surface defs
                    #
                    if sheetStr not in uniqueSheet:
                        uniqueSheet.append(sheetStr)
                    
                except:
                    raise TypeError, 'ERROR: Incorrect weld master file format, refer to online help.'
        
            # place sheetArray into Dictionary keyed to the nodedID
            #
            sheets[nodeID] = sheetArray

    # if foundRecord=0 then we do not even have one record
    # with enough entries, hence raise the file format error.
    #
    if foundRecord ==0:
        raise TypeError, 'ERROR: Incorrect weld master file format, refer to online help.'
        
    # the process operation has been successful return node and sheet.
    #
    return total, nodes, sheets, uniqueSheet

  
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def outputFileOpen(fileName, OP='CREATE'):
    """This function opens the file and returns the fileObject.
    ----------------------------------------------------------------------
    OP =
    CREATE (default) then a new file will be created
    APPEND then the existing file will be appended.
    ----------------------------------------------------------------------"""
    import os
    
    if OP == 'APPEND':
        fileObject = open(fileName, 'a')
    else:
        fileObject = open(fileName, 'w')
    return fileObject
        
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def writeDataFile(fileObject, echo, strEntry):
    """This function will append output data line by line to the file
    specifed file object.
    ----------------------------------------------------------------------
    fileObject = ouput file object
    strEntry = Text field to written to file
    echo = 1 if want to echo output to message area
    ----------------------------------------------------------------------"""

    # Commit strEntry to the file with fileObject
    #
    fileObject.write(strEntry)
    
    if echo:
        print strEntry

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   
def verifyInputValues(weldFileName, outputFileName, bulkDataFile, nodeOffset, radiusInfluence,
                      sheetThickness, echoOutput, delimiter, compliantFastener, elasticModulus,
                      poissonRatio, radiusType, thicknessType, materialType, weldTableFormat):
    
    """ This function checks all the input variables to make sure they are in the correct
    format and rasises appropriate exceptions"""
    
    # Verify that sensible keyword values have been entered.
    # 
    import types
    import os    

    # check that node offset is an integer
    #
    if type(nodeOffset) != types.IntType:
        raise TypeError, 'Node ID offset must be an integer.'

    # check radius of influence
    #
    if radiusInfluence  <= EPS_VALUE and radiusType != DATA_FROM_FILE:
        raise TypeError, 'Radius of influence must be greater than zero.'

    # check bulk data file if requested, material and thickness data is only relevant if
    # the welds are to be compliant.
    #
    if ((thicknessType==DATA_FROM_FILE or materialType == DATA_FROM_FILE) and compliantFastener) \
       or radiusType == DATA_FROM_FILE:

        # check to see if we have a case when the filename but no path is specified
        # default to cwd directory
        #
        tmpFile = os.path.split(bulkDataFile)
        if os.path.isfile(tmpFile[1]) and tmpFile[0]=='':
            bulkDataFile = os.path.join(cwdPath, tmpFile[1])
            
        if not os.path.exists(bulkDataFile):
            raise OSError, 'File not found %s' % bulkDataFile

    if compliantFastener:
        # Will be expecting sheet thickness or bulkDataFile.
        # but not both.
        #
        if thicknessType==DATA_FROM_FORM:
            if sheetThickness <= EPS_VALUE:
                raise TypeError, 'Thickness must be greater than zero.'

        # Will be expecting an valid bulkdata file ot valid form values.
        #
        if materialType==DATA_FROM_FORM:
            # Elastic modulus
            #
            if type(elasticModulus) != types.FloatType and type(elasticModulus) != types.IntType:
                raise TypeError, 'Elastic modulus must be a number greater than zero.'
        
            if float(elasticModulus) <=EPS_VALUE:
                raise TypeError, 'Elastic modulus must be a number greater than zero.'

            # Poissons Ratio
            #
            if type(poissonRatio) != types.FloatType and type(poissonRatio) != types.IntType:
                raise TypeError, 'Poisson ratio must be a number greater than zero and less than one.'
        
            if float(poissonRatio) <=EPS_VALUE or float(poissonRatio) >= 1.0:
                raise TypeError, 'Poisson ratio must be a number greater than zero and less than one.'

    # check to see if we have a case when the filename but no path is specified
    # default to cwd directory
    #
    tmpFile = os.path.split(weldFileName)
    if os.path.isfile(tmpFile[1]) and tmpFile[0]=='':
        weldFileName = os.path.join(cwdPath, tmpFile[1])
               
    # check to see if the weld file exists
    #    
    if not os.path.exists(weldFileName):
        raise OSError, 'File not found %s' % weldFileName
       
    # check to see if the output file path exits and that
    # it is not None.
    #
    if outputFileName != None:
        # check to see if we have a case when the filename but no path is specified
        # default to cwd directory
        #
        tmpFile = os.path.split(outputFileName)
        if tmpFile[0]=='' and tmpFile[1] != None:
            outputFileName = os.path.join(cwdPath, tmpFile[1])
            
        # Check to see if a valid path has been specified.
        #
        tmpFile= os.path.split(outputFileName)
        if not os.path.isdir(tmpFile[0]):
            raise OSError, 'Path not found %s' % tmpFile[0]
    else:
        raise OSError, 'No path entered for output file name'

    # check the weld table format is comma delimited and have
    # six entries
    if type(weldTableFormat)!=types.StringType or weldTableFormat.count(',')!=5:
        raise TypeError, 'Incorrect weld format definition.'

    return weldFileName, outputFileName, bulkDataFile
    
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def processBulkDataFile(bulkDataFileName, nodes, uniqueSheet):
    """ This function calls processes the bulkDataFile and returns the following
    dictionaries:
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    radiusDic: contains weld radii referenced by the wmf.
    thicknessDic : contains thicknesses of all PHSHELLS referenced in the wmf file.
    elasticDic : contains elastic modulus referenced by CWELDS
    poissonDic : contains all poisson ratio values referenced by CWELDS.
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ """

    # Transform the bulkDataFile into a space delimited
    # Using fromnastran translation
    #
    outFile = translateBulkDataFile(bulkDataFileName)

    # Open the nastran outfile and return data lines
    #
    datalines = importFileData(outFile)
    n = len(datalines)
    print '** [4] Scanning the bulk data file...'

    # Initalise dictionaries and found counters
    #
    radiusDic={}
    thicknessDic = {}
    elasticDic = {}
    poissonDic = {}
    pweld = []
    mat1Dic = {}
    foundShell =0
    foundMat = 0
    foundCWeld = 0
    foundPWeld = 0
    
    # First PASS
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # i ) store PSHELL and thickness into dictionaries, KEY = PSHELLID.
    # ii) build PWELD dictionary, KEY=reference node from CWELD entries
    #
    for i in range(n):
        if datalines[i].startswith('PSHELL'):
            # Parse data and return ID and Thickness for dataline i.
            #
            field, nField = parseNastranCard(datalines[i])
            pShellID = field[1]
            thickness = field[3]
        
            # Check to see if pShellID is in sheet list
            #
            if pShellID in uniqueSheet:
                # Store the pShellID, matID & thickness in the dictionary
                # keyed to the pShellID
                #
                foundShell = foundShell +1
                thicknessDic[pShellID] = thickness
           
        elif datalines[i].startswith('CWELD'):
            # Build Referenced PWELD dictionary. Will use the reference
            # grid point ID to link PWELD, CWELD and *FASTENER data together
            #
            field, nField = parseNastranCard(datalines[i])
            pweldId = field[2]
            gs = field[3]
            
            # The reference GRID point in the CWELD definition
            # is not guranteed to tie in with welf master file
            # reference node ids.
            #
            # Thus there is no hard link between the CWELD and WMF entries.
            # will assume that order of CWELD entries ties up with the
            # order of WMF file entires exactly.
            # 
            pweld.append(pweldId)
            foundCWeld+=1
                      
    # Second Pass
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # i) build foundMat dictionary
    # ii) store radius of influence for PWELDs in foundPWeld{}.
    #
    for i in range(n):
        if datalines[i].startswith('PWELD'):
            field, nField = parseNastranCard(datalines[i])
            pweldId = field[1]
            matId = field[2]
            diameter = field[3]
            foundPWeld+=1
            index=0
            for x in pweld:
                if x==pweldId:
                    # Store radius of influence indexed by ORDER.
                    #
                    radiusDic[index] = float(diameter)*0.5

                    # store referenced MatId in mat1Dic{}
                    mat1Dic[index] = matId
                index+=1

    # Third Pass
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # i) extract material data referenced relevnt PWELD entries
    #
    for i in range(n):
        if datalines[i].startswith('MAT1'):
            # Parse the material data from valid MAT1 Entries.
            #
            field, nField = parseNastranCard(datalines[i])
            matId = field[1]
            elasticModulus = field[2]
            poissonRatio = field[4]

            # Check to see if MAT1 entry is referenced by CWELD entry
            # if so then store data in elastic and poisson dictionaries.
            #
            if matId in mat1Dic.values():
                # store poisson and elastic modulus values in
                # dictionary.
                foundMat = foundMat +1

                # recursively update elastic and poisson dictionaries
                # with valid material properties.
                #
                for x in range(foundCWeld):
                    if mat1Dic.get(x)==matId:
                        elasticDic[x] = elasticModulus
                        poissonDic[x] = poissonRatio

    print '\n** %s referenced PSHELL entries found in NASTRAN BULK DATA file.' % str(foundShell)
    print '** %s CWELD entries found.' % str(foundCWeld)
    print '** %s PWELD entries found.' % str(foundPWeld)
    print '** %s referenced MAT1 entries found.\n' % str(foundMat)

    return radiusDic, thicknessDic, elasticDic, poissonDic
        
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def translateBulkDataFile(bulkDataFileName):
    """ This function uses fromnastran translator to cleanup an nastran
    bulk data file (bdf) into a space delimited format.
    Only data lines referenced by: <NASTRAN_VALID_CARDS> """
    
    import os, sys    
    
    # Split out the path and filename and extension
    #
    path, file = os.path.split(bulkDataFileName)
    job, extn = os.path.splitext(file)
    jobFile = os.path.join(path, job+'_clean')
    
    # Enforce valid extension
    #
    jobFileFull = jobFile + '.bdf'
    jobFileTmp = jobFile + '.tmp'
    
    # Echo to message window
    #
    print '\n** Processing the nastran bulk data file: %s%s' % (job, extn)

    # "Grep" out PSHELL ENTRIES to improve proccess speed and store in the request outfilename
    #
    print '** [1] Cleaning up bulk data file.'
    grepFile(bulkDataFileName, jobFileFull, VALID_NASTRAN_CARDS)

    print '** [2] Translating bulk data file to FIXED format.'
       
    # Call the fromnastran translator
    #
    try:
        abqCmd = os.environ['ABA_COMMAND']
    except KeyError:
        abqCmd = 'abq'
    
    command = '%s -fromnastran -job "%s" -pre_63_couplings reformat_only -verbose 2' % (abqCmd, jobFile)
    status = os.system(command)
    if status != 0:
        raise ValueError, 'Unable to run fromnastran\n\nSee command window for more details.'

    print '** [3] Cleaning up temporary files.'
    # Move the .tmp file to overwrite the ungreped/untranslated _out.bdf file.
    #
    os.remove(jobFileFull)
    os.rename(jobFileTmp, jobFileFull)

    # Clean up other temporary abaqus files
    #
    os.remove(jobFile + '.inp')
    os.remove(jobFile + '.log')
    return jobFileFull
  
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def grepFile(inputFile, outputFile, searchStr):
    """This function greps all lines in the inputfile and outputs
    them to the outputFile."""

    # Open the output fileobject
    #
    fileObj = outputFileOpen(outputFile, 'CREATE')
    
    # Read in the inputFile
    #
    input = open(inputFile)
    lines = input.readlines()

    searchString = searchStr.split('|')
    for line in lines:
        for search in searchString:
            if line.startswith(search):
                line+='\n'
                fileObj.write(line)

    # Close the file object
    #
    fileObj.close()

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def parseNastranCard(dataLine):
    """ This function takes a single line and parses it according to
    space delimited type format. Care must be taken beacuse there
    will be a non-uniform number of spaces between fields. Due to
    Nastran long data format"""

    # Check for Double or single precision fixed format.
    #
    field=[]
    if dataLine[72]=='*':
        # double precision format
        # 8 char / 16 char / 16 char / 16 char / 8 char
        i=0
        nField=4
        for x in range(nField):
            if i==0:
                field.append(dataLine[x*8:(x+1)*8].replace(' ',''))
            elif i > 0 and i < 4:
                field.append(dataLine[x*16:(x+1)*16].replace(' ',''))
            i+=1
    else:
        # single precision fixed field format.
        # 8 characters per field
        i=0
        nField=6
        for x in range(nField):
            field.append(dataLine[x*8:(x+1)*8].replace(' ',''))
            i+=1

    return field, nField

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def calculateWeldStiffness(radiusInfluence, elasticModulus, poissonRatio,
                           thicknessDic, sheets):
    """ This function will return the axial, shear, bending, tosion and
    moment stiffness of a weld between sheets[1 to n] based on:
    kaxial =AE/L.
    kbending = 3EI/L^3
    kshear = GA/L
    ktorsion = KG/L
    kmoment = EI/L

    Where:
    G = E/2(1+nu),
    L is defined from the thicknessSheet at weld definition or global value:
    I = pi*(r^4)/4 for solid circle
    K = 0.5*pi*(r^4) for solid circle
    returns: [kaxial, kbending, kshear, ktorsion, kmoment]
    Will return [0,0,0,0,0] if unable to compute hence WELD shoud be treated as rigid."""

    from abaqusConstants import TRUE, FALSE
    import types
    import math

    nSheets = len(sheets)
    length = 0.0

    if type(thicknessDic) == types.DictType:
        # We have a thickness dictionary
        # data obtained from bulk data file.
        #
        bulkData = TRUE
    else:
        # thickness dictionary will be a global value.
        #
        bulkData = FALSE

    # Calculate the average weld length
    # Updated - 7th July 2004 - P.Bosauder
    # Weld length now based on average not total length.
    #
    if bulkData:
        for i in range(nSheets):
            # find sheet ID in thickness dictionary
            # if it doesn't exist, return -1
            #
            try:
                # Calculate the total length
                #
                if i==0 or i==nSheets-1:
                    length+=float(thicknessDic.get(sheets[i]))*0.5
                else:
                    length+=float(thicknessDic.get(sheets[i]))
            except:
                return 0, 0, 0, 0, 0

        # Evaluate the average length
        #
        length = length/float((nSheets-1))

    else:
        length = float(thicknessDic)

    # Check for zero or near zero length
    #
    if length <= EPS_VALUE:
        return 0, 0, 0, 0, 0

    # Convert numbers float pt.
    #
    radiusInfluence = float(radiusInfluence)
    elasticModulus = float(elasticModulus)
    poissonRatio = float(poissonRatio)

    # Calculate the shear modulus.
    #
    try:
        G = float(elasticModulus)/(2.*(1.+float(poissonRatio)))
    except:
        return 0,0,0,0,0
    
    # Calculate the area and second moment of area
    #
    area=math.pi*radiusInfluence*radiusInfluence
    I = math.pi*radiusInfluence*radiusInfluence*radiusInfluence*radiusInfluence/4
    K = math.pi*radiusInfluence*radiusInfluence*radiusInfluence*radiusInfluence/2
    
    # Perform axial, bending and shear stiffness calculations
    #
    kaxial = area*elasticModulus/length
    kbending = 3*elasticModulus*I/(length*length*length)
    kshear = G*area/length
    ktorsion = K*G/length
    kmoment = elasticModulus*I/length
    return kaxial, kbending, kshear, ktorsion, kmoment
        
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def outputFastenerData(nodes, sheets, nodeOffset, uniqueSheet, thicknessDic, radiusDic,
                       elasticDic, poissonDic, echo, weldFileName, bulkDataFileName,
                       outFileName):
    """This function takes the nodal and sheet data and generates
    *FASTENER data storing it in the file defined by <outputFileName>"""
            
    import types
    from fastenerConst import CODE_VERSION
    from abaqusConstants import TRUE, FALSE

    # inialise
    #
    outputFail=1
    warningCount =0

    # Obtain the file object
    #
    fil = outputFileOpen(outFileName, OP='CREATE')

    if fil == None:
        # Error occured during file openeing
        #
        outputFail=1
        return  outputFail


    # Write header and open fileObject
    #   
    writeDataFile(fil,echo,'*********************************************************************\n')
    writeDataFile(fil,echo,'** Translated ABAQUS fastener representation from master weld file **\n')
    writeDataFile(fil,echo,'*********************************************************************\n')
    writeDataFile(fil,echo,'** Master Weld File:%s\n** Version: %s\n' % (weldFileName, CODE_VERSION))
    writeDataFile(fil,echo,'*********************************************************************\n**\n')

    # Check if compliance data is available
    #
    if thicknessDic == None:
        writeDataFile(fil,echo,'** Compliance not defined - FASTENER response will be [RIGID].\n')
        allCompliant = FALSE
        
        # Define read definitions
        #
        readMaterial = 0
        readThickness =0
        
        writeDataFile(fil,echo,'** Node ID offset: %s\n' % str(nodeOffset))        
        
        if type(radiusDic) != types.DictType:
            writeDataFile(fil,echo,'** Global weld radius of %s will be used.\n' % str(radiusDic))
            readRadius =0
        else:
            writeDataFile(fil,echo,'** Weld radius from Bulk data file:%s\n' % bulkDataFileName)
            readRadius = 1
    else:
        writeDataFile(fil,echo,'** Compliance activated - FASTENER response will be [ELASTIC].\n')
        allCompliant = TRUE

        writeDataFile(fil,echo,'** Node ID offset: %s\n' % str(nodeOffset))   

        if type(radiusDic) != types.DictType:
            writeDataFile(fil,echo,'** Global weld radius of %s will be used.\n' % str(radiusDic))
            readRadius =0
        else:
            writeDataFile(fil,echo,'** Weld radius from Bulk data file:%s\n' % bulkDataFileName)
            readRadius =1
        
        if type(thicknessDic) != types.DictType:
            writeDataFile(fil,echo,'** Global sheet thickness of %s will be used.\n' % str(thicknessDic))
            readThickness = 0
        else:
            writeDataFile(fil,echo,'** Shell Thickness from Bulk data file:%s\n' % bulkDataFileName)
            readThickness = 1 

        if type(elasticDic) != types.DictType:
            writeDataFile(fil,echo,'** Global elastic modulus of %s will be used.\n' % str(elasticDic))
            readMaterial = 0
        else:
            writeDataFile(fil,echo,'** Elastic modulus from Bulk data file:%s\n' % bulkDataFileName)
            readMaterial = 1

        if type(poissonDic) != types.DictType:
            writeDataFile(fil,echo,'** Global poisson ratio of %s will be used.\n' % str(poissonDic))
        else:
            writeDataFile(fil,echo,'** Poisson ratio from Bulk data file:%s\n' % bulkDataFileName)
            
        # Define global coordinate system for CARDAN connector elements
        #
        writeDataFile(fil,echo,'**\n** Global coordinate system for CARDAN connect elements.\n')
        writeDataFile(fil,echo,'*ORIENTATION, NAME=GLOBAL\n1,0,0,1,1,0,0,0,0\n')

    # store the node keys (node and sheet keys will be identical and based
    # the nodal ID.
    #
    nodeKeys = nodes.keys()
    nodeKeys.sort()
    
    # setup the unique surface ID array
    #
    uniqueSurfaceId =[]
    n = len(nodeKeys)
             
    for i in range(n):
        # WRITE NODAL DATA FOR EACH ENTRY IN NODES
        #
        # Determine NSET name
        #
        pShells = sheets.get(nodeKeys[i])
        m=len(pShells)
        nSetName = " ".join(pShells)
        nSetName = nSetName.replace(" ","_")
        nSetName = nSetName + '_REF' + str(nodeKeys[i]+nodeOffset)
           
        # get the nodal data and store in nodeStr
        #
        nodeXYZ = nodes.get(nodeKeys[i])
          
        writeDataFile(fil,echo,'*NODE, NSET=WELD_NSET_%s\n' % nSetName)
        writeDataFile(fil,echo,'%i, %f, %f, %f \n**\n' % (int(nodeKeys[i])+nodeOffset,
                                                              nodeXYZ[0], nodeXYZ[1], nodeXYZ[2]))
        
        # Check if values are to be read from file or specified
        # globally.
        #
        if readRadius:
            radiusVal = radiusDic.get(i)
        else:
            radiusVal = radiusDic
                
        if readMaterial:
            elasticVal = elasticDic.get(i)
            poissonVal = poissonDic.get(i)
        else:
            elasticVal = elasticDic
            poissonVal = poissonDic

        # Check to see if we have a 'RIGID' or compliant fastener.
        #            
        if not allCompliant:
            rigid = TRUE
               
        elif pShells[0] == pShells[1] and allCompliant:
            # if we have a duplicate PSHELL entry (=> 0 length hence will enforce rigid fastener)
            # Cannot have zero length weld.
            writeDataFile(fil,0,'** WARNING zero length weld detected for FASTENER:%s\n' %nSetName)
            writeDataFile(fil,0,'** This FASTENER will be treated as [RIGID]...\n')
            warningCount+=1
            rigid = TRUE             
        else:
            # We have a compliant fastener attempt to calculate the weld stiffness terms
            # Weld length L will be determined from sheet thicknesses or from global values
            #
            [kaxial, kbending, kshear, ktorsion, kmoment] = calculateWeldStiffness(radiusVal,
                                                 elasticVal, poissonVal, thicknessDic, pShells)

            # Stiffness in 1,2,3,4,5,6 components are determined from a combination of
            # axial, shear and bending stiffnesses. Because shear and bending sterms
            # are in series must be careful when adding.
            #
            # *FASTENER convention is to use 3 (or z) as the normal direction between surfaces.
            #
            if kshear==0 or kbending ==0 or kaxial==0 or kmoment ==0 or ktorsion==0:
                k1=-1
            else:
                k1 = k2 = kshear*kbending/(kshear+kbending)
                k3 = kaxial
                k4 = k5 = kmoment
                k6 = ktorsion
            if k1 ==-1:
                # Then no stiffeness could be calculated. Data not referenced in
                # thickness dictionary.
                #
                writeDataFile(fil,0,'** WARNING unable to calculate stiffness for FASTENER ID:%s\n' %nSetName) 
                writeDataFile(fil,0,'** Associated PSHELL may not be referenced in the bulk data file...\n')
                writeDataFile(fil,0,'** This FASTENER will be treated as [RIGID]...\n')
                warningCount+=1
                rigid = TRUE
            else:
                rigid = FALSE
    
        if rigid:
            # Write {RIGID} fastener data for each entry
            #
            writeDataFile(fil,echo,'** [RIGID] FASTENER  FOR WELD CONNECTING SHEETS: %s \n**\n' % nSetName)
            writeDataFile(fil,echo,'*FASTENER, INTERACTION NAME=SPOT WELDS, PROPERTY=WELDS,')
            writeDataFile(fil,echo,'REFERENCE NODE SET=WELD_NSET_%s' % nSetName)
            writeDataFile(fil,echo,", NUMBER OF LAYERS=%i" % (m-1))
            writeDataFile(fil,echo,", RADIUS OF INFLUENCE=%f\n" % float(radiusVal))
        else:
            # Write {ELASTIC} fastener data for each entry
            #
            writeDataFile(fil,echo,'** [ELASTIC] FASTENER  FOR WELD CONNECTING SHEETS: %s \n**\n' % nSetName)
            writeDataFile(fil,echo,'*FASTENER, INTERACTION NAME=SPOT WELDS, PROPERTY=WELDS, ELSET=EL_%s,' % nSetName)
            writeDataFile(fil,echo,'REFERENCE NODE SET=WELD_NSET_%s' % nSetName)
            writeDataFile(fil,echo,", NUMBER OF LAYERS=%i\n" % (m-1))

            # Radius of influence removed from compliant fasteners based on recommendation
            # from Charlie Chin. 06/22/2004
            #
            # writeDataFile(fil,echo,", RADIUS OF INFLUENCE=%f\n" % float(radiusInfluence))
           
        # write out *FASTENER surface definitions
        #
        for j in range(m):
            if j!=0:
                # Take into account the situation of a duplicate surface entry.
                # P10001, P10001 {10001 welded to itself}
                # only want to write one PSHELL entry
                #
                if pShells[j]!=pShells[j-1]:
                    writeDataFile(fil,echo,', SURF_PSHELL_%s' % pShells[j])

                # *FASTENER allows a max of SURF_PER_LINE=8 entries per line
                # after which they should be entereted on subsequent lines.
                #
                if abs(float(j)/SURF_PER_LINE - int(j)/SURF_PER_LINE) < EPS_VALUE:                   
                    writeDataFile(fil,echo,'\n')
            else:
                writeDataFile(fil,echo,'SURF_PSHELL_%s' % pShells[j])
        writeDataFile(fil,echo,'\n**\n')

        if not rigid:
            writeDataFile(fil,echo,"*CONNECTOR SECTION, ELSET=EL_%s, BEHAVIOR=BEHV_%s\n" % (nSetName, nSetName))
            writeDataFile(fil,echo,"CARTESIAN, CARDAN\nGLOBAL\n")
            writeDataFile(fil,echo,"*CONNECTOR BEHAVIOR, NAME=BEHV_%s\n" %nSetName)
            writeDataFile(fil,echo,"*CONNECTOR ELASTICITY, COMPONENT=1\n%f,\n"%float(k1))
            writeDataFile(fil,echo,"*CONNECTOR ELASTICITY, COMPONENT=2\n%f,\n"%float(k2))
            writeDataFile(fil,echo,"*CONNECTOR ELASTICITY, COMPONENT=3\n%f,\n"%float(k3))
            writeDataFile(fil,echo,"*CONNECTOR ELASTICITY, COMPONENT=4\n%f,\n"%float(k4))
            writeDataFile(fil,echo,"*CONNECTOR ELASTICITY, COMPONENT=5\n%f,\n"%float(k5))
            writeDataFile(fil,echo,"*CONNECTOR ELASTICITY, COMPONENT=6\n%f,\n"%float(k6))
                              
    # write weld property data
    #
    writeDataFile(fil,echo,'*FASTENER PROPERTY, NAME=WELDS\n%f\n**\n' % radiusVal) 
       
    # write surface definitions
    #
    writeDataFile(fil,echo,'** DEFINE SURFACES FOR SHELLS INVOLVED IN SPOTWELDS\n**\n')
    uniqueSheet.sort
    for i in range(len(uniqueSheet)):
        writeDataFile(fil,echo,'*SURFACE, NAME=SURF_PSHELL_%s\n' % uniqueSheet[i])
        writeDataFile(fil,echo,'PSHELL_%s, SPOS\n**\n' % uniqueSheet[i])
       
    writeDataFile(fil,echo,'** END OF DATA **\n')
       
    # close the output file and return
    #
    fil.close()
    outputFail =0

    # Echo warning count to message area.
    #
    print '** %s warnings were generated, please check the output (.inp) file for more information.' % str(warningCount)
    return outputFail
