#=========================================================================
# Kernal Script to Generate *FASTENER DATA from a Master Weld File
#=========================================================================
# 
from fastenerConst import *
from fastenerFunc import *

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def createFasteners(weldFileName, outputFileName,
                    nodeOffset = DEFAULT_NODE_OFFSET, radiusInfluence = DEFAULT_RAD_INF,
                    compliantFastener=DEFAULT_USE_COMPLIANT, elasticModulus=DEFAULT_MODULUS,
                    poissonRatio=DEFAULT_POISSON, delimiter=COMMA,
                    echoOutput=DEFAULT_ECHO_OUTPUT, sheetThickness=DEFAULT_THICKNESS,
                    bulkDataFileName=DEFAULT_BULKDATA_FILE, radiusType = DEFAULT_RADIUS_TYPE,
                    thicknessType=DEFAULT_THICKNESS_TYPE, materialType=DEFAULT_MATERIAL_TYPE,
                    weldTableFormat=DEFAULT_WELD_TABLE_IDS):
    """This is the main module used to create fastener data from the master
    weld file."""

    # Check data input data format.
    #
    weldFileName, outputFileName, bulkDataFile = verifyInputValues(weldFileName, outputFileName,
                      bulkDataFileName, nodeOffset, radiusInfluence, sheetThickness, echoOutput,
                      delimiter, compliantFastener, elasticModulus,  poissonRatio, radiusType,
                      thicknessType, materialType, weldTableFormat)
    
    # Open the weld file and return an array of values
    #
    dataLines = importFileData(weldFileName)   
       
    # Call generic interpretor to read data into suggested format.
    # 
    n, nodes, sheets, uniqueSheet = interpretFileFormat(dataLines, delimiter, weldTableFormat)
  
    # Determine radius, thickness and elastic material properties from the
    # Nastran bulk data file. If all valid data is obtain from the form
    # then do not call processBulkDataFile.
    #
    if ((thicknessType == DATA_FROM_FILE or materialType == DATA_FROM_FILE) and compliantFastener) or \
       radiusType == DATA_FROM_FILE:
        radiusDic, thicknessDic, elasticDic, poissonDic = processBulkDataFile(bulkDataFileName,
                                                                          nodes, uniqueSheet)

    # Dictionary values should be overwritten by global values when type = DATA_FROM_FORM
    #
    if radiusType == DATA_FROM_FORM:
        radiusDic = radiusInfluence
    if thicknessType == DATA_FROM_FORM:
        thicknessDic = sheetThickness
    if materialType == DATA_FROM_FORM:
        elasticDic = elasticModulus
        poissonDic = poissonRatio

    if not compliantFastener:
        # For completly rigid fasteners set elastic properties to none
        #
        thicknessDic = None
        elasticDic = None
        poissonDic = None        
    
    # Call the function to output data fastener data
    #
    outputFastenerData(nodes, sheets, nodeOffset, uniqueSheet, thicknessDic, radiusDic, elasticDic,
                       poissonDic, echoOutput, weldFileName, bulkDataFileName, outputFileName)
