"""
ABAQUS/CAE plug-in for exporting/importing CAE Data
"""

from abaqusGui import *
import uti, os.path
from kernelAccess import session
from abaqusConstants import *

folderName = "caeXML" # Folder in which plug-in files are stored
execfile(os.path.join(os.path.dirname(__file__), folderName, "commonExec.py")) # To load functions necessary for modifying module names

# Use package syntax to avoid possible name clashes with other plugins
# Use Modified module names to avoid clashing with other XML plug-ins
(treeForm, settings, defaultLoad) = importModifiedModules(folderName, ("caeXML.gui.treeForm", "caeXML.kernel.settings", "caeXML.gui.defaultLoad"))
(mainFile, ) = modifyName(folderName, ("caeXML.kernel.mainXML", ))

# Version check
majorNumber, minorNumber, updateNumber = getAFXApp().getBaseVersionNumbers()
verAbaqus = majorNumber * 1.0 + minorNumber * 0.1
version = settings.utilVersion
if verAbaqus < 6.7:
    status = "CAE Export/Import Plug-in version " + str(version) + " was not registered because "\
             "it is not supported in Abaqus version " + str(uti.getVersion())
elif verAbaqus == 6.7:
    status = 'OK'
else:
    import _abqPluginUtils
    status = _abqPluginUtils.checkCompatibility("caeXML", version)

def loadDefaults():
    try:
        if ('odbDisplay' in  session.viewports[session.currentViewportName].__members__) or \
           ('detailPlotOptions' in  session.viewports[session.currentViewportName].__members__):
            session.unregisterQuery(loadDefaults)
            defaultLoad.DefaultLoad(toolset, __file__, folderName)
        else:
            pass
    except:
        print "Error occured while importing default options. Please contact Simulia Support for assistance."

session.registerInclusive(loadDefaults)

# Register commands
if status == 'OK':
    toolset=getAFXApp().getAFXMainWindow().getPluginToolset()
    # Button for Export/Import Manager
    toolset.registerGuiMenuButton(
        object=treeForm.TreeForm(toolset, folderName),
        buttonText='Export/Import XML...',
        messageId=AFXMode.ID_ACTIVATE,
        kernelInitString= "import " + mainFile,
        applicableModules=["Sketch", "Part", "Property", "Assembly", "Step", "Interaction", "Load", "Mesh", "Job", "Visualization",],
        version=version,
        author="Dassault Systemes Simulia Corp.",
        description='This menu-button starts the CAE Export/Import Plugin',
        helpUrl="http://abaqus.custhelp.com/cgi-bin/abaqus.cfg/php/enduser/std_adp.php?p_faqid=3625",
        )
else:
    getAFXApp().getAFXMainWindow().writeToMessageArea(status)
    getAFXApp().beep()
