"""
This script will create the Campbell Plug-In
"""

from abaqusGui import getAFXApp

# Register commands
#
version = '1.3-1'

try:
    import _abqPluginUtils
    status = _abqPluginUtils.checkCompatibility('CampbellDiagram', version)
except:    
    status = 'OK'


msg = 'Unable to load Campbell Diagram without MAC plug-in. Please ensure MAC '\
      'plug-in is properly installed. For details, see '\
      'http://abaqus.custhelp.com/cgi-bin/abaqus.cfg'\
                '/php/enduser/std_adp.php?p_faqid=3146'
try:
    from mac_plugin import version as macVersion
    if macVersion > 2.1:
        status = 'OK'
    else:
        status = msg
except:    
    status = msg

if status == 'OK':
    author = 'Dassault Systemes Simulia Corp.'
    kernelInitString = 'import abq_CampbellDiagram'
    helpUrl = r'http://abaqus.custhelp.com/cgi-bin/abaqus.cfg/php/' \
               'enduser/std_adp.php?p_faqid=4721'
    description = 'This plug-in displays Campbell-Diagrams \n'+\
                  'based on the calculation of the (complex)'+\
                  'Modal Assurance Criteria (MAC).'

    # Use package syntax to avoid possible name clashes with other plugins
    #
    toolset = getAFXApp().getAFXMainWindow().getPluginToolset()
    from abq_CampbellDiagram.campbellForm import CampbellForm
    toolset.registerGuiMenuButton(object=CampbellForm(toolset),
                                  buttonText=r'Tools|Plot Campbell Diagram...',
                                  kernelInitString=kernelInitString,
                                  version=version,
                                  author=author,
                                  applicableModules=('Visualization',),
                                  description=description,
                                  helpUrl=helpUrl
                                  )
else:
    getAFXApp().getAFXMainWindow().writeToMessageArea(status)
    getAFXApp().beep()

