from abaqus import *
from abaqusConstants import *
from connVisConstants import *
import displayGroupOdbToolset as dgo

def connVis(stepName, stepTime, sel_button, damageVariable=None,
            damageFlag=None, firstValue=None, secondValue=None,
            damEvolVariable=None, damInitVariable=None):
    """
    Method to plot connectors in the viewport. The connectors
    are plotted for the ODB displayed in the viewport.

    Required arguments:
    
    damageVariable: A string specifying the connector damage variable
    stepName: A string specifying the step name
    stepTime: A float specifying the step time
    sel_button: Flag to indicate if selection is query or color code
    damageFlag : A flag specifying the operator (<=, >=, =, etc.)
    firstValue: A float specifying the value in the first text field
    secondValue: A float specifying the value in the second text field
    damEvolVariable: Damage evolution variable for color coding
    damInitVariable: Damage initiation variable for color coding
    """
    
    tolerance = 1.e-05
    vp = session.viewports[session.currentViewportName]
    odb = session.odbs[vp.odbDisplay.name]
    #Plotting all the connectors
    vp = session.viewports[session.currentViewportName]
    stepObj = odb.steps[stepName]
    connDict = {}
    connEvolDict = {}
    connInitDict = {}
    initCount = 0
    evolCount = 0 
    for historyRegion in stepObj.historyRegions.keys():
        if historyRegion.startswith('Element'):
            historyRegionObj = stepObj.historyRegions[historyRegion]
            if sel_button == FIRST_BUTTON:
                if historyRegionObj.historyOutputs.has_key(damageVariable):
                    damageData = historyRegionObj.\
                                 historyOutputs[damageVariable].data
                    for data in damageData:
                        if abs(data[0] - stepTime) <= tolerance:               
                            partInstance = historyRegionObj.point.instance.name
                            elementNum = historyRegionObj.point.element.label
                            tempList = [partInstance, elementNum]
                            if damageFlag == GREATER_THAN:
                                if data[1] >= firstValue:
                                    updateDict(connDict, tempList)
                            elif damageFlag == LESSER_THAN:
                                if data[1] <= firstValue:
                                    updateDict(connDict, tempList)
                            elif damageFlag == EQUAL_TO:
                                if abs(data[1] - firstValue) <= tolerance:
                                    updateDict(connDict, tempList)
                            elif damageFlag == WITHIN_RANGE:
                                if firstValue > secondValue:
                                    tempValue = firstValue
                                    firstValue = secondValue
                                    secondValue = tempValue
                                    if data[1] >=firstValue and \
                                       data[1] <= secondValue:
                                        updateDict(connDict, tempList)
                                elif abs(firstValue - secondValue) <= tolerance:
                                    if abs(data[1] - firstValue) <= tolerance:
                                        updateDict(connDict, tempList)
                                else:
                                    if data[1] >=firstValue and \
                                       data[1] <= secondValue:
                                        updateDict(connDict, tempList)
                            break                    
            elif sel_button == SECOND_BUTTON:
                if historyRegionObj.historyOutputs.has_key(damInitVariable):
                    damageData = historyRegionObj.\
                                 historyOutputs[damInitVariable].data
                    for data in damageData:
                        if abs(data[0] - stepTime) <= tolerance:
                            partInstance = historyRegionObj.point.instance.name
                            elementNum = historyRegionObj.point.element.label
                            initCount += 1
                            connInitDict[initCount]= [data[1], partInstance,
                                                      elementNum]
                            break
                if damEvolVariable and historyRegionObj.historyOutputs.\
                   has_key(damEvolVariable):
                    damageData = historyRegionObj.\
                                 historyOutputs[damEvolVariable].data
                    for data in damageData:
                        if abs(data[0] - stepTime) <= tolerance:
                            if data[1] > 0.0:
                                partInstance = historyRegionObj.point.\
                                               instance.name
                                elementNum = historyRegionObj.point.\
                                             element.label
                                evolCount += 1
                                connEvolDict[evolCount] = [data[1],
                                                           partInstance,
                                                           elementNum]
                                break
                            
    if sel_button == FIRST_BUTTON:
        if not connDict:
            errMsg = 'There are no connectors available for display with the '\
                     'specified arguments.'
            reset()
            raise ValueError, errMsg

    elif sel_button == SECOND_BUTTON:
        initFlag = 0
        evolFlag = 0
        if not connInitDict and not connEvolDict:
            errMsg = 'There are no connectors available for display with the '\
                 'specified arguments.'
            reset()
            raise ValueError, errMsg
        elif connInitDict and not connEvolDict:
            if damEvolVariable:
                print 'Warning: There is no damage evolution occuring in the '\
                      'model for the selected step and time. The color coding'\
                      ' will be based on damage initiation.'
            initFlag = 1
        elif connEvolDict and not connInitDict:
            print 'Warning: Damage initiation data is not available in the '\
                  'odb for the selected step and time.'
            evolFlag = 1
        else:
            initFlag = 1
            evolFlag = 1        
    
    connAxesFlag = OFF
    connTypeFlag = OFF
    
    try:
        plotState = vp.odbDisplay.display.plotState    
        vp.odbDisplay.display.setValues(plotState=plotState)
        if vp.odbDisplay.basicOptions.connectorDisplay:
            connAxesFlag = vp.odbDisplay.basicOptions.showConnectorAxes
            connTypeFlag = vp.odbDisplay.basicOptions.showConnectorType
    except:
        vp.odbDisplay.display.setValues(plotState=(DEFORMED,))
        
    vp.odbDisplay.basicOptions.setValues(connectorDisplay=ON,
                                         highlightConnectorPts=ON,
                                         showConnectorAxes=connAxesFlag,
                                         showConnectorType=connTypeFlag)
    vp.odbDisplay.basicOptions.setValues(otherSymbolSize=10)
    vp.odbDisplay.commonOptions.setValues(translucencyFactor=0.85)
    vp.odbDisplay.commonOptions.setValues(translucency=ON)
    vp.enableMultipleColors()
    vp.setColor(initialColor='#BDBDBD')
    cmap = vp.colorMappings['Default']
    vp.setColor(colorMapping=cmap)
    vp.disableMultipleColors()

    #Removing any existing connectors displayed in viewport 
    try:
        leaf = dgo.LeafFromOdbElementTypes(elementTypes=('CONN3D2', ))
        vp.odbDisplay.displayGroup.remove(leaf=leaf)
    except:        
        pass

    try:
        leaf = dgo.LeafFromOdbElementTypes(elementTypes=('CONN2D2', ))
        vp.odbDisplay.displayGroup.remove(leaf=leaf)
    except:
        pass    

    if sel_button == FIRST_BUTTON:
        for key in connDict.keys():
            instanceName = key
            elementLabelSeq = connDict[instanceName]
            leaf = dgo.LeafFromElementLabels(partInstanceName=instanceName,
                                             elementLabels=elementLabelSeq)
            vp.odbDisplay.displayGroup.add(leaf=leaf)

        totElem = 0
        for value in connDict.values():
            totElem += len(value)

        print '%s connectors match the specified arguments.\n' %(totElem)
        
    elif sel_button == SECOND_BUTTON:
        redDict = {}
        orangeDict = {}
        yellowDict = {}
        greenDict = {}
        defaultMaxDam = 1.0
        defaultMaxInit = 0.8
        defaultMaxInitOnly = 1.0
        if evolFlag:
            for evolData in connEvolDict.values():
                if evolData[0] >= defaultMaxDam:
                    tempList = [evolData[1], evolData[2]]
                    updateDict(redDict, tempList)
                else:
                    tempList = [evolData[1], evolData[2]]
                    updateDict(orangeDict, tempList)
            if initFlag:
                for initData in connInitDict.values():
                    if initData[0] >= defaultMaxInit and \
                    initData[0] < defaultMaxInitOnly:
                        tempList = [initData[1], initData[2]]
                        updateDict(yellowDict, tempList)
                    elif initData[0] < defaultMaxInit:
                        tempList = [initData[1], initData[2]]
                        updateDict(greenDict, tempList)
        elif initFlag and not evolFlag:
            for initData in connInitDict.values():
                if initData[0] >= defaultMaxInitOnly:
                    tempList = [initData[1], initData[2]]
                    updateDict(redDict, tempList)
                elif initData[0] >= defaultMaxInit and \
                     initData[0] < defaultMaxInitOnly:
                    tempList = [initData[1], initData[2]]
                    updateDict(yellowDict, tempList)
                else:
                    tempList = [initData[1], initData[2]]
                    updateDict(greenDict, tempList)
        
        vp.enableMultipleColors()
        vp.setColor(initialColor='Default')
        setColor(redDict, '#FF0000')
        setColor(orangeDict, '#FF7F00')
        setColor(yellowDict, '#FFFF00')
        setColor(greenDict, '#00FF00')
        vp.disableMultipleColors()

        printCount(redDict, 'RED')
        printCount(orangeDict, 'ORANGE')
        printCount(yellowDict, 'YELLOW')
        printCount(greenDict, 'GREEN')

def setColor(dictName, color):
    """
    Method to set color coding to connectors.
    """
    vp = session.viewports[session.currentViewportName]    
    for key in dictName.keys():
        instanceName = key
        elementLabelSeq = dictName[instanceName]
        leaf = dgo.LeafFromElementLabels(partInstanceName=instanceName,
                                         elementLabels=elementLabelSeq)
        vp.setColor(leaf=leaf, edgeColorWireHide=color,
                    edgeColorFillShade=color,
                    fillColor=color)
        vp.odbDisplay.displayGroup.add(leaf=leaf)

def printCount(dictName, colorName):
    """
    Method to print out the elements in the dictionary.
    """
    totElem = 0
    for value in dictName.values():
        totElem += len(value)

    print 'No. of elements with color code %s = ' %(colorName), totElem
    
def updateDict(dictName, tempList):
    """
    Method to sort part instances and elements into a dictionary.
    """
    if dictName.has_key(tempList[0]):
        dictName[tempList[0]].append(tempList[1])        
    else:
        dictName[tempList[0]] = [tempList[1]]        
        
def reset():
    """
    Method to reset.
    """
    
    vp = session.viewports[session.currentViewportName]
    vp.odbDisplay.commonOptions.setValues(translucency=OFF)
    vp.enableMultipleColors()
    vp.setColor(initialColor='#BDBDBD')
    cmap = vp.colorMappings['Default']
    vp.setColor(colorMapping=cmap)
    vp.disableMultipleColors()
    leaf = dgo.Leaf(leafType=DEFAULT_MODEL)
    try:
        vp.odbDisplay.displayGroup.replace(leaf=leaf)
    except VisError:
        return
    
    vp.odbDisplay.basicOptions.setValues(connectorDisplay=OFF,
                                     highlightConnectorPts=OFF)
    try:
        plotState = vp.odbDisplay.display.plotState
        vp.odbDisplay.display.setValues(plotState=plotState)
    except:        
        vp.odbDisplay.display.setValues(plotState=(UNDEFORMED,))
    
def resetTrans():
    """
    Method to reset translucency to OFF.
    """

    vp = session.viewports[session.currentViewportName]
    vp.odbDisplay.commonOptions.setValues(translucency=OFF)
