"Custom Views XML derived class"

from abaqus import *
from abaqusConstants import *

import os.path

execfile(os.path.join(os.path.dirname(os.path.dirname(__file__)), "commonExec.py")) # Initializes the importFile function
(abqXML, xmlUtils) = importFile(__file__, ("caeXML.kernel.abqXML", "caeXML.kernel.xmlUtils")) # Allows base-package directory to be changed
(settingsModule, ) = importFile(__file__, ("caeXML.customViews.customViewSettings", )) # Settings Module

################################################################################

def readView(rootEle, funcList, baseObj, parameterDict):
    "This function reads object of type view of the base viewport"
    objType = xmlUtils.getObjectTypeFromXmlElement(rootEle)
    if objType != "View":
        return (None, None, False)
    (memberDict, tag, success) = xmlUtils.readObjectValueAsDictionary(rootEle, funcList, baseObj, parameterDict)
    if not success:
        return (None, None, False)
    baseStr = xmlUtils.getStringRepresentationOfObject(baseObj[-1], "baseObj[-1]")
    keyList = ['cameraPosition', 'cameraTarget', 'cameraUpVector', 'farPlane', 'farPlaneMode', 'height', 'movieMode', 'nearPlane', 'projection', 'viewOffsetX', 'viewOffsetY', 'width']
    cmd = baseStr + ".setValues("
    flag = False
    for key in keyList:
        if key not in memberDict.keys():
            return (None, None, False)
        cmd = cmd + key + "=" + str(memberDict[key]) + ", "
        flag = True
    if not flag:
        return (None, None, False)
    cmd = cmd + "drawImmediately=True)"
    try:
        exec(xmlUtils.upgradeText(cmd, parameterDict))
    except:
        return (None, None, False)
    return (baseObj[-1], tag, True)

################################################################################

class CustomViewsXML(abqXML.AbqXML):
    "This class specializes the abqXML class for custom views export"

    ################################################################################

    def __init__(self):
        self.m_settingsModule = settingsModule
        abqXML.AbqXML.__init__(self)
        self.m_str = ["session.viewports[" + session.currentViewportName.__repr__() + "].view"]
        self.m_objLoadFuncList = [readView] + self.m_objLoadFuncList
        self.m_overwrite = True
        self.m_disableKeyCheck = True # For being able to import options across viewports

    ################################################################################
    
    def saveInfo(self):
        "This function saves the information about various views into xmldoc"
        return self.parseAndAttachMultiple()

    ################################################################################

    def loadInfo(self, saveEle):
        "This function loads the view information from the xmldoc"

        results = self.detachAndReturn(self.m_str[0], [session.viewports[session.currentViewportName].view])
        return self.numResults(results)
    
################################################################################
