"Display Groups XML derived class"

from abaqus import *
from abaqusConstants import *

currentModeViewer = False
# If model database is not available then do not do anything. The plug-in may have been initialized in Abaqus/Viewer
try:
    import displayGroupMdbToolset
    from caeModules import *
except:
    currentModeViewer = True
    from viewerModules import *

import displayGroupOdbToolset

#from xml.dom.ext import * #Commented out, since we do not support xml.dom.ext anymore
from xml.dom.minidom import *

import os.path

execfile(os.path.join(os.path.dirname(os.path.dirname(__file__)), "commonExec.py")) # Initializes the importFile function
(abqXML, xmlUtils) = importFile(__file__, ("caeXML.kernel.abqXML", "caeXML.kernel.xmlUtils")) # Allows base-package directory to be changed
(settingsModule, ) = importFile(__file__, ("caeXML.displayGroups.dgSettings", )) # Settings Module

################################################################################

def readSymbolicConstant(rootEle, funcList, baseObj, parameterDict):
    "This function reads the symbolic constants from the XML file"
    flag = False
    tag = rootEle.tagName
    objType = xmlUtils.getObjectTypeFromXmlElement(rootEle)
    parentObjType = xmlUtils.getObjectTypeFromXmlElement(rootEle.parentNode)
    if objType == None and (tag == 'module' or tag == 'action' or tag == 'validModules'):
        flag = True
    if parentObjType == "EnumerationLeaf":
        flag = True
    if not flag:
        return (None, None, False)
    val = None
    childList = rootEle.childNodes
    ## If two text nodes are present for an element, then only the first one is read
    for child in childList:
        if child.nodeType == child.TEXT_NODE:
            val = SymbolicConstant(child.nodeValue)
            break
    ##
    return (val, tag, True)

################################################################################

def customProcessLeafFromDisplayGroup(objDict):
    "This function will custom process objDict to give out the constructor string for LeafFromDisplayGroup"
    dgList = []
    for dgName in objDict[u'displayGroup']:
        if dgName not in session.displayGroups.keys():
            print "Cannot load the leaf as this display group was created with the help of non-existant display group " + dgName
            return None
        dgList.append(session.displayGroups[dgName])
    return {u'displayGroup':tuple(dgList)}

################################################################################

def readModelElementNodeLabelLeaf(rootEle, funcList, baseObj, parameterDict):
    "This function reads ODB element label leaf"
    objType = xmlUtils.getObjectTypeFromXmlElement(rootEle)
    supportedLeaves = {'ModelElementLabelLeaf':('elementLabels', 'LeafFromModelElemLabels'), 'ModelNodeLabelLeaf':('nodeLabels', 'LeafFromModelNodeLabels')}
    if objType not in supportedLeaves.keys():
        return (None, None, False)
    (objDict, tag, success) = xmlUtils.readObjectValueAsDictionary(rootEle, funcList, baseObj, parameterDict)
    elemList = []
    for val in objDict[supportedLeaves[objType][0]]:
        labels = []
        for label in val[1]:
            if type(label) != type(""):
                labels.append(str(label))
            else:
                labels.append(label)
        elemList.append((val[0], tuple(labels)))
    elemLabels = tuple(elemList)
    constr = "dgo." + supportedLeaves[objType][1] + "(" + supportedLeaves[objType][0] + "=elemLabels)"
    try:
        retVal = eval(xmlUtils.upgradeText(constr, parameterDict))
    except:
        return (None, None, False)  # Wrong parameters for constructor
    return (retVal, tag, True)

################################################################################

def readMdbLeaf(rootEle, funcList, baseObj, parameterDict):
    "This function reads the display group leaves"
    if currentModeViewer:
        return (None, None, False)
    tag = rootEle.tagName
    if tag != 'leaf':
        return (None, None, False)
    (objDict, tag, success) = xmlUtils.readObjectValueAsDictionary(rootEle, funcList, baseObj, parameterDict)
    if not success:
        return (None, None, False)
    objType = xmlUtils.getObjectTypeFromXmlElement(rootEle)
    constr = ""
    if objType == "GeometryLeaf":
        constr = "dgm.LeafFromGeometry"
    elif objType == "ElementLabelLeaf":
        constr = "dgm.LeafFromMeshElementLabels"
    elif objType == "PartElemLabelsLeaf":
        constr = "dgm.LeafFromPartElementLabels"
    elif objType == "NodeLabelLeaf":
        constr = "dgm.LeafFromMeshNodeLabels"
    elif objType == "PartNodeLabelsLeaf":
        constr = "dgm.LeafFromPartNodeLabels"
    elif objType == "DatumsLeaf":
        constr = "dgm.LeafFromDatums"
    elif objType == "RefPtLeaf":
        constr = "dgm.LeafFromReferencePoint"
    elif objType == "SetsLeaf":
        constr = "dgm.LeafFromSets"
    elif objType == "SurfaceSetsLeaf":
        constr = "dgm.LeafFromMeshSurfaceSets"
    elif objType == "InstanceElemLabelsLeaf":
        constr = "dgm.LeafFromInstanceElementLabels"
    elif objType == "InstanceNodeLabelsLeaf":
        constr = "dgm.LeafFromInstanceNodeLabels"
    elif objType == "InstanceLeaf":
        constr = "dgm.LeafFromInstance"
    else:
        return (None, None, False)
    constr += "("
    flag = False
    for key in objDict.keys():
        if key != u'validModules' and objDict[key]:
            ## To avoid a bug
            key2 = key
            if objType == "PartNodeLabelsLeaf" and key == "elementLabels":
                key2 = "nodeLabels"
            ##
            constr = constr + key2 + " = " + "objDict[" + key.__repr__() + "]" + ", "
            flag = True
    if flag:
        constr = constr[:-2]
    constr += ")"
    try:
        val = eval(xmlUtils.upgradeText(constr, parameterDict))
    except:
        return (None, None, False)  # Wrong parameters for constructor
    return (val, tag, True)

################################################################################

def readLeaf(rootEle, funcList, baseObj, parameterDict):
    "This function reads the display group leaves"
    tag = rootEle.tagName
    if tag != 'leaf':
        return (None, None, False)
    (objDict, tag, success) = xmlUtils.readObjectValueAsDictionary(rootEle, funcList, baseObj, parameterDict)
    if not success:
        return (None, None, False)
    objType = xmlUtils.getObjectTypeFromXmlElement(rootEle)
    constr = ""
    if objType == "EnumerationLeaf":
        constr = "dgo.Leaf"
    elif objType == "PartElementSetLeaf":
        constr = "dgo.LeafFromElementSets"
    elif objType == "DisplayGroupLeaf":
        constr = "dgo.LeafFromDisplayGroup"
        objDict = customProcessLeafFromDisplayGroup(objDict)
        if not objDict:
            return (None, None, False)
    elif objType == "PartInstanceLeaf":
        constr = "dgo.LeafFromPartInstance"
    elif objType == "ElementVariableRangeLeaf":
        constr = "dgo.LeafFromElementVarRange"
    elif objType == "PartNodeSetLeaf":
        constr = "dgo.LeafFromNodeSets"
    elif objType == "NodeVarRangeLeaf":
        constr = "dgo.LeafFromNodeVarRange"
    elif objType == "SurfVariableRangeLeaf":
        constr = "dgo.LeafFromSurfaceVarRange"
    elif objType == "PartSurfaceSetLeaf":
        constr = "dgo.LeafFromSurfaceSets"
    elif objType == "CsysLeaf":
        if u'isUserCsys' in objDict.keys():
            if objDict[u'isUserCsys'] == OFF:
                constr = "dgo.LeafFromOdbCoordSystem"
            else:
                constr = "dgo.LeafFromUserCoordSystem"
        else:
            raise ValueError, "Illegal coordinate system specification for display group leaf"
    elif objType == "ElementSectionLeaf":
        constr = "dgo.LeafFromOdbElementSections"
    elif objType == "ElementMaterialLeaf":
        constr = "dgo.LeafFromOdbElementMaterials"
    elif objType == "ElementLayupLeaf":
        constr = "dgo.LeafFromOdbElementLayups"
    elif objType == "ElementPlyLeaf":
        constr = "dgo.LeafFromOdbElementPlies"
    elif objType == "ElementPickLeaf":
        constr = "dgo.LeafFromOdbElementPick"
    elif objType == "ElementTypeLeaf":
        constr = "dgo.LeafFromOdbElementTypes"
    elif objType == "NodePickLeaf":
        constr = "dgo.LeafFromOdbNodePick"
    elif objType == "EdgePickLeaf":
        constr = "dgo.LeafFromOdbEdgePick"
    elif objType == "SurfacePickLeaf":
        constr = "dgo.LeafFromOdbSurfacePick"
    else:
        return readMdbLeaf(rootEle, funcList, baseObj, parameterDict)
    constr += "("
    flag = False
    for key in objDict.keys():
        if key != u'validModules' and key != u'isUserCsys':
            constr = constr + key + " = " + "objDict[" + key.__repr__() + "]" + ", "
            flag = True
    if flag:
        constr = constr[:-2]
    constr += ")"
    try:
        val = eval(xmlUtils.upgradeText(constr, parameterDict))
    except:
        return (None, None, False)  # Wrong parameters for constructor
    return (val, tag, True)

################################################################################

def readLeafDummy(rootEle, funcList, baseObj, parameterDict):
    "Dummy function This function reads the display group leaves"
    tag = rootEle.tagName
    if tag != 'leaf':
        return (None, None, False)
    (objDict, tag, success) = xmlUtils.readObjectValueAsDictionary(rootEle, funcList, baseObj, parameterDict)
    if not success:
        return (None, None, False)
    return (None, tag, True)

################################################################################

def readDisplaygroupKnot(rootEle, funcList, baseObj, parameterDict):
    "This function reads the display group knots"
    objType = xmlUtils.getObjectTypeFromXmlElement(rootEle)
    if objType != 'DisplayGroupKnot':
        return (None, None, False)
    (objDict, tag, success) = xmlUtils.readObjectValueAsDictionary(rootEle, funcList, baseObj, parameterDict)
    if not success:
        return (None, None, False)
    return (objDict, tag, True)

################################################################################

def setObjectInCurrentViewport(objDict,newName):
    "This function sets the module to the one mentioned in objDict"
    module = objDict[u'module']
    modelName = objDict[u'modelName']    
    if module == ASSEMBLY or module == PART_ASSEMBLY:
        if "rootAssembly" not in mdb.models[modelName].__members__:
            raise ValueError, "The Display group " + newName + " was created in Abaqus/CAE and cannot be imported into current session."
        if modelName not in mdb.models.keys():
            # If model name is renamed, take the first model name
            a = mdb.models.values()[0].rootAssembly
        else:
            a = mdb.models[modelName].rootAssembly
        session.viewports[session.currentViewportName].setValues(displayedObject=a)
        return True
    elif module == ODB:
        allOdbs = session.odbs.values()
        if len(allOdbs) == 0:
            return False
        odb = allOdbs[0]
        session.viewports[session.currentViewportName].setValues(displayedObject=odb)
        return True
    elif module == PART:
        if "parts" not in mdb.models[modelName].__members__:
            raise ValueError, "The Display group " + newName + " was created in Abaqus/CAE and cannot be imported into current session."
        if modelName in mdb.models.keys():
            model = mdb.models[modelName]
            partName = objDict[u'partName']
            if partName in model.parts.keys():
                p = model.parts[partName]
                session.viewports[session.currentViewportName].setValues(displayedObject=p)
                return True
            elif len(model.parts.keys()) != 0:
                p = model.parts.values()[0]
                session.viewports[session.currentViewportName].setValues(displayedObject=p)
                return True
        # If model name is renamed, take the first model name with a part
        for model in mdb.models.values():
            if len(model.parts.keys()) != 0:
                p = model.parts.values()[0]
                session.viewports[session.currentViewportName].setValues(displayedObject=p)
                return True
        return False
    return True

################################################################################

def getCurrentDisplayGroup(objDict):
    "This function gets the current display group object"
    vp = session.viewports[session.currentViewportName]
    module = objDict[u'module']
    if module == ASSEMBLY or module == PART_ASSEMBLY:
        return vp.assemblyDisplay.displayGroup
    elif module == ODB:
        return vp.odbDisplay.displayGroup
    elif module == PART:
        return vp.partDisplay.displayGroup
    dType = str(type(vp.displayedObject)).split("'")[1]
    if dType == "NoneType":
        return None
    elif dType == "Odb":
        return vp.odbDisplay.displayGroup
    elif dType == "Part":
        return vp.partDisplay.displayGroup
    elif dType == "Assembly":
        return vp.assemblyDisplay.displayGroup
    else:
        return None


################################################################################

def readDisplayGroup(rootEle, funcList, baseObj, parameterDict):
    "Function to load object of the type display group"
    objType = xmlUtils.getObjectTypeFromXmlElement(rootEle)
    if objType != 'DisplayGroup':
        return (None, None, False)
    (objDict, tag, success) = xmlUtils.readObjectValueAsDictionary(rootEle, funcList, baseObj, parameterDict)
    if not success:
        return (None, None, False)
    newName = xmlUtils.checkForNameMember('name', objDict[u'name'], 'DisplayGroup', parameterDict)
    do = session.viewports[session.currentViewportName].displayedObject
    if not setObjectInCurrentViewport(objDict,newName):
        return (None, None, False)
    retVal = getCurrentDisplayGroup(objDict)
    if not retVal:
        return (None, None, False)
    newFuncList = [readModelElementNodeLabelLeaf, readLeaf] + funcList
    (objDict, tag, success) = xmlUtils.readObjectValueAsDictionary(rootEle, newFuncList, baseObj, parameterDict)
    if not success:
        return (None, None, False)
    leafDict= objDict[u'root']
    if newName == "All":
        raise NameError, 'The display group "All" cannot be modified.'
    constr = "session.DisplayGroup(name=" + newName.__repr__() + ", objectToCopy=retVal)"
    try:
        retVal = eval(xmlUtils.upgradeText(constr, parameterDict))
    except:
        return (None, None, False)
    while leafDict != None:
        if u'leaf' not in leafDict.keys(): # Something is awfully wrong, Still the display group exists.....
            return (retVal, tag, True)
        if leafDict[u'leaf'] == None:  # Leaf was not loaded. Still the display group exists....
            return (retVal, tag, True)
        if leafDict[u'action'] == SymbolicConstant('ADD'):
            retVal.add(leaf=leafDict[u'leaf'])
        elif leafDict[u'action'] == SymbolicConstant('EITHER'):
            retVal.either(leaf=leafDict[u'leaf'])
        elif leafDict[u'action'] == SymbolicConstant('INTERSECTION'):
            retVal.intersect(leaf=leafDict[u'leaf'])
        elif leafDict[u'action'] == SymbolicConstant('REPLACE'):
            retVal.replace(leaf=leafDict[u'leaf'])
        elif leafDict[u'action'] == SymbolicConstant('REMOVE'):
            retVal.remove(leaf=leafDict[u'leaf'])
        leafDict = leafDict[u'next']
    session.viewports[session.currentViewportName].setValues(displayedObject=do)
    return (retVal, tag, True)

################################################################################

def storeDisplaygroupKnots(xmlDoc, rootEle, value, funcList):
    "This function stores display group knots into xmlDoc"
    if str(type(value)).split("'")[1] != 'DisplayGroupKnot':
        return False
    try:
        memberList = value.__members__
    except:
        return False
    objType = str(type(value)).split("'")[1]
    if objType == "AbaqusObject" or rootEle.tagName == "AbaqusObject":   # Special case. These objects should not be read back
        print "Warning: Type information missing for the object:", value.__repr__()
    rootEle.setAttribute("Type", "Object")
    rootEle.setAttribute("ObjectType", objType)
    for member in memberList:
        if member == 'previous':
            continue
        val = None
        try:
            val = getattr(value, member)
            str(val)   # Check to avoid cases like session.viewports['...'].odbDisplay
        except: #except AttributeError:
            pass
        else:
            newEle = xmlDoc.createElement(member)
            rootEle.appendChild(newEle)
            xmlUtils.storeValue(xmlDoc, newEle, val, funcList)
    return True

################################################################################

def isObjOfTypeDisplayGroup(objType, memberName):
    "This function checks if the given object is of type display group"
    if objType == 'DisplayGroup':
        if memberName == 'name':
            return True
    return False

################################################################################

class DGXML(abqXML.AbqXML):
    "This class specializes the abqXML class for display groups export"

    ################################################################################

    def __init__(self):
        self.m_settingsModule = settingsModule
        abqXML.AbqXML.__init__(self)
        self.m_str = ["session.displayGroups[]"]
        self.m_objSaveFuncList = [storeDisplaygroupKnots] + self.m_objSaveFuncList
        self.m_objLoadFuncList = [readLeafDummy, readSymbolicConstant, readDisplaygroupKnot, readDisplayGroup] + self.m_objLoadFuncList
        self.m_nameCheckFuncs = [isObjOfTypeDisplayGroup]

    ################################################################################

    def saveInfo(self):
        "This function saves the display groups objects into xml doc"
        return self.parseAndAttachMultiple()

    ################################################################################

    def loadInfo(self, saveEle):
        "This function loads display groups from xmldoc"

        results = self.detachAndReturn(self.m_str[0], [session])
        return self.numResults(results)

################################################################################
