"Materials XML derived class"

from abaqus import *
from abaqusConstants import *

currentModeViewer = False
# The plug-in may have been initialized in Abaqus/Viewer
try:
    import material
except:
    currentModeViewer = True
    raise AssertionError, "Cannot access materials. This error is usually shown when the plug-in is used in Abaqus/Viewer instead of Abaqus/CAE."

import os.path

execfile(os.path.join(os.path.dirname(os.path.dirname(__file__)), "commonExec.py")) # Initializes the importFile function
(abqXML, xmlUtils) = importFile(__file__, ("caeXML.kernel.abqXML", "caeXML.kernel.xmlUtils")) # Allows base-package directory to be changed
(settingsModule, ) = importFile(__file__, ("caeXML.materials.materialsSettings", )) # Settings Module for views

################################################################################

def readMullinsTestData(rootEle, funcList, baseObj, parameterDict):
    "This function will read the test data for mullins effect"
    objType = xmlUtils.getObjectTypeFromXmlElement(rootEle)
    if objType != 'BiaxialTestData' and objType != 'PlanarTestData' and objType != 'UniaxialTestData':
        return (None, None, False)
    return xmlUtils.readAnyObject(rootEle, funcList, [material], parameterDict)

################################################################################

def readMullinsEffect(rootEle, funcList, baseObj, parameterDict):
    "This function reads mullins effect data from XML file"
    objType = xmlUtils.getObjectTypeFromXmlElement(rootEle)
    if objType != "MullinsEffect":
        return (None, None, False)
    tag = rootEle.tagName
    childList = rootEle.childNodes
    for child in childList:
        if child.tagName == 'definition':
            if child.childNodes[0].nodeValue != 'TEST_DATA':
                return xmlUtils.readAnyObject(rootEle, funcList, baseObj, parameterDict)
    constructorFunction = "baseObj[-1].MullinsEffect("
    md = {}
    for child in childList:
        cTag = child.tagName
        if cTag == 'biaxialTests' or cTag == 'planarTests' or cTag == 'uniaxialTests':
            (cVal, childTag, cSuccess) = xmlUtils.readArrayValue(child, [readMullinsTestData] + funcList, baseObj, parameterDict)
            md[cTag] = tuple(cVal)
        else:
            (childTag, md[cTag]) = xmlUtils.readTagAndValue(child, funcList, baseObj, parameterDict)
        constructorFunction = constructorFunction + cTag + " = " + xmlUtils.getStringRepresentationOfObject(md[cTag], "md[" + cTag.__repr__() + "]") + ", "
    constructorFunction = constructorFunction[:-2] + ")"
    try:
        val = eval(xmlUtils.upgradeText(constructorFunction, parameterDict))
    except:
        return (None, None, False)  # Wrong parameters for constructor
    return (val, tag, True)

################################################################################

def storeUserDefinedField(xmlDoc, rootEle, value, funcList):
    "This function stores user-defined field into xmlDoc"
    if str(type(value)).split("'")[1] != 'UserDefinedField':
        return False
    rootEle.setAttribute("Type", "Object")
    rootEle.setAttribute("ObjectType", 'UserDefinedField')
    return True  # No need of anything else

################################################################################

def isObjOfTypeMaterial(objType, memberName):
    "This function checks if the given object is of type material"
    if objType == 'Material':
        if memberName == 'name':
            return True
    return False

################################################################################

class MaterialsXML(abqXML.AbqXML):
    "This class specializes the abqXML class for materials export"

    ################################################################################

    def __init__(self):
        self.m_settingsModule = settingsModule
        abqXML.AbqXML.__init__(self)
        self.m_str = ["mdb.models[].materials[]"]
        self.m_objLoadFuncList = [readMullinsEffect] + self.m_objLoadFuncList
        self.m_objSaveFuncList = [storeUserDefinedField] + self.m_objSaveFuncList
        self.m_overwrite = True # To avoid fake warning messages
        self.m_nameCheckFuncs = [isObjOfTypeMaterial]
        self.m_useMemberNameForConstructor =["DamageInitiation"]

    ################################################################################
    
    def saveInfo(self):
        "This function saves the materials objects into xml doc"
        return self.parseAndAttachMultiple()

    ################################################################################

    def loadInfo(self, saveEle):
        "This function loads the material information from the xmldoc"
        count = 0
        results = self.getMatchingKeys(self.m_str[0])
        for result in results:
            modelName = result[0][0]
            materialName = result[0][1]
            if self.m_params:
                if type(self.m_params) == type(()):
                    if len(self.m_params) == 2:
                        modelName = self.m_params[0]
                        materialName = self.m_params[1]
            if modelName in mdb.models.keys():
                if self.objOverwriteCheck(self.m_str[0], (modelName, materialName)):
                    results2 = self.detachAndReturn(result[1], [mdb.models[modelName]])
                    count += self.numResults(results2)
        return count

################################################################################
