from calibrationBehavior import *
from abaqusConstants import *
from mesh import *
import visualization


###########################################################################
# Class definition
###########################################################################

class NitinolSuperelasticPlasticBehavior( CalibrationBehavior ):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__( self, name, typeName, repository ):
            
        CalibrationBehavior.__init__(self, name, typeName, repository)
        self.materialName = ''
        self.ds1Name = ''
        self.anaType = 'std'
        self.sigmasl = 'equalsigmasl'
        self.volStrain = 0.0
        self.dimension = '3D'
        self.numAnneals = 1.0
        self.numSteps = '2'
        self.sigmaSCl = 0.0
        self.sTGradL = 5.0
        self.sTGradU = 5.0
        self.eL = 0.0
        self.refTemp = 22.0
        self.aPoisRat = 0.33
        self.mPoisRat = 0.33
        self.xPt1 = 1.0
        self.xPt2 = 1.0
        self.xPt3 = 1.0
        self.xPt4 = 1.0
        self.xPt5 = 1.0
        self.yPt1 = 1.0
        self.yPt2 = 1.0
        self.yPt3 = 1.0
        self.yPt4 = 1.0
        self.yPt5 = 1.0
        self.plasticPoints=''
        self.density=''

         

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #Below function is required for mapping the material behavior for each
    #and every calibration plug-in kernel
    #
    def mapToMaterial( self, materialName ):
               
        self.materialName = materialName
        vp = session.currentViewportName
        modelName = session.sessionState[vp]['modelName']
        origModelName = modelName
        model = mdb.models[modelName]
        materials = model.materials

        #Checking for the material name to match the UMAT naming convention
        #
        
        if not self.materialName.startswith('ABQ_SUPER_ELASTIC') or \
                   not self.materialName.startswith('ABQ_SUPER_ELASTIC_N3D') and \
                   not self.materialName.startswith('ABQ_SUPER_ELASTIC_N2D') and \
                   not self.materialName.startswith('ABQ_SUPER_ELASTIC_N1D'):

            if self.anaType == 'std':               
                i = 1
                mat = 'ABQ_SUPER_ELASTIC'+'_'+self.materialName

                while mat in materials.keys():
                   
                    mat = 'ABQ_SUPER_ELASTIC' + '_'+ str(i)
                    i = i+1
                try:
                    materials.changeKey(fromName=self.materialName, toName=mat)
                except: #except does not work
                    msg = "Error occured while renaming the material.\n"\
                                     "Make sure that given material name does not exceed 20 characters."
                    raise ValueError, msg
                self.materialName = mat

            elif self.anaType == 'expl':
                i = 1
                mat = 'ABQ_SUPER_ELASTIC_N'+self.dimension+'_'+self.materialName
                while mat in materials.keys():
                    mat = 'ABQ_SUPER_ELASTIC_N'+self.dimension+ '_'+ str(i)
                    i = i +1
                try:                                
                    materials.changeKey(fromName=self.materialName, 
                    toName= mat)
                except: #except does not work
                    msg = "Error occured while renaming the material.\n"\
                          "Make sure that given material name does not exceed 20 characters."
                    raise ValueError,msg
                    
                self.materialName = mat
                                    
            else:
                print 'Analysis type not defined.'
        else:
            self.materialName=materialName
        
        mat1 = materials[self.materialName]

        #Only in the case of a plstic behavior
        #
        if self.plasticPoints:
            #checking (0,0) and removing it if it is there
            #
            self.plasticPoints = list(self.plasticPoints)
            for i in range(len(self.plasticPoints)-1):
                if self.plasticPoints[i]==(0,0):
                    self.plasticPoints.remove(self.plasticPoints[i])
            
            #Removes the duplicate plastic points entered in the table
            #
            self.plasticPoints = tuple(set(self.plasticPoints)) #Removes the duplicate plastic points entered in the table
  
            self.numPlasticPts= len(self.plasticPoints)
            
            #Swaping (strain, stress) to (stress, strain) for the plastic points
            #
            swapPlasticPoints1=[(b,a) for (a,b) in list(self.plasticPoints)]

            
            #The plastic points should be arragned such that the strains are in the ascending order
            #Here in the (stress, stress) pair the second member is strain so only that needs to be rearranged
            #
            swapPlasticPoints = sorted(swapPlasticPoints1, key = lambda test:test[1])

        
        #Generation of mechanical constants for the mapped material
        #for standard and explicit analysis types and with/without the plastic points
        #
        tup = [float(st) for st in self.numSteps.split(',')]
        self.austEa, self.martEm, self.tStrainL, self.vStrain, self.flag1 = self.calcConstants()


        if self.anaType == 'std':
            stdMechanicalConstants1= (self.austEa, self.aPoisRat, self.martEm,
            self.mPoisRat,self.tStrainL, self.sTGradL, self.startTransLoad, 
            self.endTransLoad, self.refTemp, self.sTGradU, self.startTransUnLoad, 
            self.endTransUnLoad, float(self.sigmascl), float(self.vStrain), float(self.numAnneals))

            if self.plasticPoints:
                stdMechanicalConstants2=tuple(list(stdMechanicalConstants1)+tup) + (self.numPlasticPts,)

                for i in tuple(swapPlasticPoints):
                    stdMechanicalConstants2=stdMechanicalConstants2+i

                stdMechanicalConstants=stdMechanicalConstants2

        
            else:
                stdMechanicalConstants=tuple(list(stdMechanicalConstants1)+tup)
     

        elif self.anaType == 'expl':
            if self.plasticPoints:
                explMechanicalConstants1 = (self.austEa, self.aPoisRat, self.martEm,
                 self.mPoisRat,self.tStrainL, self.sTGradL, self.startTransLoad, 
                 self.endTransLoad, self.refTemp, self.sTGradU, self.startTransUnLoad, 
                 self.endTransUnLoad, float(self.sigmascl), float(self.vStrain), self.numPlasticPts)
                for i in tuple(swapPlasticPoints):
                    explMechanicalConstants1=explMechanicalConstants1 + i
                explMechanicalConstants= explMechanicalConstants1    
                    
                                                            
            else:
                explMechanicalConstants = (self.austEa, self.aPoisRat, self.martEm,
                 self.mPoisRat,self.tStrainL, self.sTGradL, self.startTransLoad, 
                 self.endTransLoad, self.refTemp, self.sTGradU, self.startTransUnLoad, 
                 self.endTransUnLoad, float(self.sigmascl), float(self.vStrain))                
                                           
        
        #Density is required for the explicit analysis
        #
        if self.anaType == 'expl':
           model.materials[self.materialName].Density(table=((self.density,),))
           
        #Check for unsym flag
        #
        if self.flag1 == 'entered':
            has_unsym = True
            unsymFlag = ON
        else:
            unsymFlag = OFF  
       
       
        #Check if User material is definied  for the material to be mapped or not
        #
        has_userMat = False
        
        for attr in dir(mat1):
            if attr == 'userMaterial':
                has_userMat = True
                break
        if has_userMat:
            if self.anaType == 'std':
                mat1.userMaterial.setValues(unsymm=unsymFlag, mechanicalConstants=stdMechanicalConstants)

            elif self.anaType == 'expl':
                mat1.userMaterial.setValues(unsymm=unsymFlag, mechanicalConstants=explMechanicalConstants)    
        else:
            if self.anaType == 'std':
                mat1.UserMaterial(unsymm=unsymFlag, mechanicalConstants=stdMechanicalConstants)
            elif self.anaType == 'expl':
                mat1.UserMaterial(unsymm=unsymFlag, mechanicalConstants=explMechanicalConstants)

        #Check if depvar is definied  for the material to be mapped or not
        #
        has_depvar = False
        
        for attr in dir(mat1):
            if attr == 'depvar':
                has_depvar = True                
                break
        
        #Check and set different depvar values for superelastic and superelasticPlastic 
        #material behaviors
        #
        if self.plasticPoints:
            if has_depvar:
                mat1.depvar.setValues(n=31)
            else:
                mat1.Depvar(n=31)
        else:
            if has_depvar:
                mat1.depvar.setValues(n=24)
            else:
                mat1.Depvar(n=24)

        #Add depvar description using keywordblock object
        #
        self.modelName = model.name
        self.searchString = '*depvar'

        index = self.getIndex(self.anaType, model, self.materialName)

        depvarDataLines = '1, SDV1_EE11, "Linear elastic strain component"\n'\
        '2, SDV2_EE22, "Linear elastic strain component"\n'\
        '3, SDV3_EE33, "Linear elastic strain component"\n'\
        '4, SDV4_EE12, "Linear elastic strain component"\n'\
        '5, SDV5_EE13, "Linear elastic strain component"\n'\
        '6, SDV6_EE23, "Linear elastic strain component"\n'\
        '7, SDV7_TE11, "Transformation strain component"\n'\
        '8, SDV8_TE22, "Transformation strain component"\n'\
        '9, SDV9_TE33, "Transformation strain component"\n'\
        '10, SDV10_TE12, "Transformation strain component"\n'\
        '11, SDV11_TE13, "Transformation strain component"\n'\
        '12, SDV12_TE23, "Transformation strain component"\n'\
        '19, SDV19_ETE, "Equivalent transformation strain"\n'\
        '20, SDV20_VTE, "Volumetric transformation strain"\n'\
        '21, SDV21_FM, "Fraction of martensite"\n'\
        '22, SDV22_EUTS, "Equivalent uniaxial tensile stress"\n'\
        '23, SDV23_EUTTE, "Equivalent uniaxial tensile transformation strain"\n'\
        '24, SDV24_EUTE, "Equivalent uniaxial tensile total strain"\n'\
  
        #For superelastic-plastic behavior there are additional description lines
        #
        if self.plasticPoints:
            plasticLines = '26, SDV26_PE11, "Plastic strain component"\n'\
                           '27, SDV27_PE22, "Plastic strain component"\n'\
                           '28, SDV28_PE33, "Plastic strain component"\n'\
                           '29, SDV29_PE12, "Plastic strain component"\n'\
                           '30, SDV30_PE13, "Plastic strain component"\n'\
                           '31, SDV31_PE23, "Plastic strain component"'
            depvarDataLines = depvarDataLines + plasticLines       
	    
	         
        model.keywordBlock.insert(index,depvarDataLines)

        #Check to see if the calibrated material evaluation with one element model
        #
        if self.eval1:
            self.evalMat(origModelName,depvarDataLines, self.materialName, self.anaType)
        else:
            print "Material evaluation with one element material model will not be performed."
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #Function to evaluate the calibrated material with a one element model
    #
    def evalMat(self, origModelName,depvarDataLines,origMatName, anaType):
        
        #Create and set up the one element model with the calibrated material
        #for uniaxial tension analysis
        #
        modelName= self.generateName('Model_mat_eval')

        mdb.Model(name=modelName)
        model=mdb.models[modelName]
        origModel=mdb.models[origModelName]
        
        #Create a sketch and the 3D deformatble part
        #
        model.ConstrainedSketch(name='__profile__', sheetSize=10.0)
        model.sketches['__profile__'].rectangle(point1=(-0.5, 0.5),
            point2=(0.5, -0.5))
        partName= self.generateName('Part_mat_eval', modelName)  

        model.Part(dimensionality=THREE_D, name=partName, type=
            DEFORMABLE_BODY)
        part=model.parts[partName]
        part.BaseSolidExtrude(depth=1.0, sketch=
            model.sketches['__profile__'])
        del model.sketches['__profile__']
        
        #Seed and mesh the part
        part.seedPart(deviationFactor=0.1, size=1.0)
        part.generateMesh()
        part.setElementType(elemTypes=(ElemType(
            elemCode=C3D8, elemLibrary=STANDARD, secondOrderAccuracy=OFF,
            distortionControl=DEFAULT), ElemType(elemCode=C3D6, elemLibrary=STANDARD),
            ElemType(elemCode=C3D4, elemLibrary=STANDARD)), regions=(
            part.cells.getSequenceFromMask(('[#1 ]',
            ), ), ))
        #Create an instance
        #
        assembly=model.rootAssembly
        assembly.DatumCsysByDefault(CARTESIAN)
        instanceName=self.generateName('Instance_mat_eval', modelName)
        assembly.Instance(dependent=ON, name=instanceName,
            part=part)
        instance=assembly.instances[instanceName]           
        
        #Create BCs
        #
        model.EncastreBC(createStepName='Initial', localCsys=None,
            name='encaster', region=Region(
            vertices=instance.vertices.getSequenceFromMask(
            mask=('[#20 ]', ), )))
        model.DisplacementBC(amplitude=UNSET, createStepName='Initial',
            distributionType=UNIFORM, fieldName='', localCsys=None, name='Fix-XY',
            region=Region(
            vertices=instance.vertices.getSequenceFromMask(
            mask=('[#10 ]', ), )), u1=SET, u2=SET, u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=
            UNSET)
        model.DisplacementBC(amplitude=UNSET, createStepName='Initial',
            distributionType=UNIFORM, fieldName='', localCsys=None, name='Fix-XZ',
            region=Region(
            vertices=instance.vertices.getSequenceFromMask(
            mask=('[#80 ]', ), )), u1=SET, u2=UNSET, u3=SET, ur1=UNSET, ur2=UNSET, ur3=
            UNSET)
        model.DisplacementBC(amplitude=UNSET, createStepName='Initial',
            distributionType=UNIFORM, fieldName='', localCsys=None, name='Fix-X',
            region=Region(
            vertices=instance.vertices.getSequenceFromMask(
            mask=('[#40 ]', ), )), u1=SET, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=UNSET,
            ur3=UNSET)
        
        
        #Create a static step if the analysis type is Standard and dynamic explicit if the analysis
        #type is explicit
        #
        stepName=self.generateName('Step_mat_eval', modelName, partName)

        if anaType == 'std':
            model.StaticStep(name=stepName, nlgeom=ON, timePeriod=0.75, previous='Initial',initialInc=0.001, maxInc=0.001,
                             maxNumInc=1000)
        elif anaType == 'expl':                     	
            model.ExplicitDynamicsStep(name=stepName, previous='Initial')
        
        #Find max strain from the data
	#
	data= origModel.calibrations[self.calibName].dataSets[self.ds1Name].data
	maxStrain=data[0][0]

	for i in range(len(data)):
	    if data[i][0] > maxStrain:
	        maxStrain = data[i][0]
	maxStrain=exp(maxStrain)-1	

        
	#Set up BC attributes with the tabular amplitude for the step
	#
	model.TabularAmplitude(data=((0.0, 0.0), (0.25, self.xPt2), (0.5, 0.0), (0.75, maxStrain)), name='Amp-1', smooth=SOLVER_DEFAULT, timeSpan=STEP)
        model.DisplacementBC(amplitude='Amp-1', createStepName=stepName,distributionType=UNIFORM, fieldName='', fixed=OFF, localCsys=None, name='Load', region=Region(faces=instance.faces.getSequenceFromMask(mask=('[#1 ]', ), )), u1=1.0, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=UNSET,ur3=UNSET)

        #Use the material calibrated material for the one element model
        #and also add *depvar keywords and description
        #
        model.Material(origMatName, origModel.materials[origMatName])
        self.searchString = '*depvar'
        #sIndex=self.Index(modelName, self.searchString)
        index = self.getIndex(anaType, model, origMatName)
	
        model.keywordBlock.insert(index,depvarDataLines)       
        if anaType == 'expl':
            
            model.materials[self.materialName].Density(table=((self.density,),))

        #Assign a solid section using the calibrated material
        #
        model.HomogeneousSolidSection(material=origMatName, name='Section-1', thickness=None)
        part.SectionAssignment(offset=0.0,
            offsetField='', offsetType=MIDDLE_SURFACE, region=Region(
            cells=part.cells.getSequenceFromMask(
            mask=('[#1 ]', ), )), sectionName='Section-1', thicknessAssignment=
            FROM_SECTION)
        assembly.regenerate()
        
        #Create a temperature distribution
        #
        model.Temperature(createStepName='Initial',
            crossSectionDistribution=CONSTANT_THROUGH_THICKNESS, distributionType=
            UNIFORM, magnitudes=(float(self.refTemp), ), name='Predefined Field-1', region=Region(
            cells=instance.cells.getSequenceFromMask(
            mask=('[#1 ]', ), ),
            faces=instance.faces.getSequenceFromMask(
            mask=('[#3f ]', ), ),
            edges=instance.edges.getSequenceFromMask(
            mask=('[#fff ]', ), ),
            vertices=instance.vertices.getSequenceFromMask(
            mask=('[#ff ]', ), )))
        
        #Create a field output request
        #
        model.fieldOutputRequests['F-Output-1'].setValues(variables=('S', 'PE', 'PEEQ', 'LE', 'U', 'RF', 'CF', 'SDV'))
            
        #Create a job and submit it
        #
        
        mdb.Job(atTime=None, contactPrint=OFF, description='', echoPrint=OFF,
            explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF,
            memory=50, memoryUnits=PERCENTAGE, model=modelName, modelPrint=OFF,
            multiprocessingMode=DEFAULT, name='one-element-eval', nodalOutputPrecision=
            SINGLE, numCpus=1, queue=None, scratch='', type=ANALYSIS, userSubroutine=''
            , waitHours=0, waitMinutes=0)
        
        job=mdb.jobs['one-element-eval']
        
        #To avoid the round off errors for Explicit analysis double precision is activated
        #
        if self.anaType=='expl':
            job.setValues(explicitPrecision=DOUBLE_PLUS_PACK, nodalOutputPrecision=FULL)
   
        job.submit(consistencyChecking=OFF)
        job.waitForCompletion()
        
        #Calling the post-processing function
        #
        self.postProcess(instanceName, maxStrain)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
    #Function to return index location to add lines to the keywordblock
    #
    def getIndex(self, anaType, model, origMatName):    

        keywordBlock = model.keywordBlock
        keywordBlock.synchVersions(storeNodesAndElements = FALSE)
        sieBlocksObj = keywordBlock.sieBlocks
	flag='False'
	for i in range(len(sieBlocksObj)):
	    if origMatName in sieBlocksObj[i]:
		j=i
		while not '*Depvar' in sieBlocksObj[j]:
		    j=j+1
		break
   	index=j
        return index
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    #Function to post-process one element model
    #
    def postProcess(self,instanceName, maxStrain):

      
	#Open and ODB and display in the current viewport
	#
        odb1 = session.openOdb(name='one-element-eval.odb')
	cvp1 = self.getCurrentVp()
	cvp= session.viewports[cvp1]       
	cvp.setValues(displayedObject=odb1)
	odb = session.odbs['one-element-eval.odb']
	
	#Extracting the required field output data from ODB
	#
	session.xyDataListFromField(odb=odb, outputPosition=INTEGRATION_POINT, variable=(('SDV_SDV22_EUTS', INTEGRATION_POINT), ('SDV_SDV24_EUTE',INTEGRATION_POINT), ), elementLabels=((instanceName.upper(), (1)), ),)
	xyDataObjects=session.xyDataObjects
	xy1 = xyDataObjects['SDV_SDV24_EUTE PI: '+instanceName.upper()+' E: 1 IP: 1']
	xy2 = xyDataObjects['SDV_SDV22_EUTS PI: '+instanceName.upper()+' E: 1 IP: 1']
	xy3 = combine(xy1, xy2)
	xy3.setValues(
	sourceDescription='combine ( "SDV_SDV24_EUTE PI: "+instanceName.upper()+" E: 1 IP: 1", "SDV_SDV22_EUTS PI: "+instanceName.uppper()+" E: 1 IP: 1" )')
	
	#Setting a plot for material evaluation results data
	#
	plotName=self.generatePlotName('matEvalPlot')
	    
	if not session.xyPlots.has_key(plotName):
	    xyp = session.XYPlot(plotName)
	    xyPlots = session.xyPlots
	else:
	    xyp = session.xyPlots[plotName]
	    xyPlots = session.xyPlots
 	tmpName = xy3.name
	matEvalXYObj=self.generatePlotName('Evaluation')
	xyDataObjects.changeKey(tmpName, matEvalXYObj)


	#Setting X and Y axis data for the XY plots
	#
	datasetChart = xyPlots[self.ds1Name].charts.keys()[0]
	dataSetCh = session.charts[datasetChart]
	dataSetCh.axes1[0].axisData.setValues(minValue=0,maxValue=.08,  minAutoCompute=False)
	dataSetCh.axes2[0].axisData.setValues(minValue=0, maxValue=1000, minAutoCompute=False)

	
	xQuantity = visualization.QuantityType(type=NONE)
	yQuantity = visualization.QuantityType(type=NONE)
	xyDataObjects[self.ds1Name].setValues(axis1QuantityType=xQuantity,
		axis2QuantityType=yQuantity, )
	xyDataObjects[matEvalXYObj].setValues(axis1QuantityType=xQuantity,
		axis2QuantityType=yQuantity, )

	#Plotting the dataset and material evaluation stress strain results on top
	#of each other for comparision purposes
	#
	chartName = xyp.charts.keys()[0]
	chart = xyp.charts[chartName]
	xy1 = xyDataObjects[self.ds1Name]
	c1 = session.Curve(xyData=xy1)
	xy2 = xyDataObjects[matEvalXYObj]
	c2 = session.Curve(xyData=xy2)
	chart.setValues(curvesToPlot=(c1, c2, ), )

	#Setting X and Y titles for the XY plots
	#
	chartCh = session.charts[chartName]
	chartCh.axes1[0].axisData.setValues(useSystemTitle=False,
			title='SDV_SDV24_EUTE')	
	chartCh.axes2[0].axisData.setValues(useSystemTitle=False,
			title='SDV_SDV24_EUTE')
	chartCh.axes2[0].axisData.setValues(useSystemTitle=False, 
	title='SDV_SDV22_EUTS')         
	
	#Plotting on the current viewport
	#                                   
	x = xyPlots[plotName]
	cvp.setValues(displayedObject=x)

	# Delete unnecessary xyDataObjects
	#
	for i in xyDataObjects.keys():
	    if i.startswith('SDV_SDV22_EUTS PI: INSTANCE_MAT_EVAL_1') or \
			i.startswith('SDV_SDV24_EUTE PI: INSTANCE_MAT_EVAL_1'):
	        del xyDataObjects[i]	


   
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #Below function is required for mapping the material behavior for each
    #and every calibration plug-in kernel
    #
    def setValues(self, ds1Name=None, anaType=None, sigmasl=None, materialName=None, volStrain=None,
         dimension=None, numAnneals=None, numSteps=None, sigmaSCl=None, eL=None, refTemp=None,aPoisRat=None, 
         mPoisRat=None, sTGradL=None, sTGradU=None, xPt1=None, xPt2=None, xPt3=None, xPt4=None, xPt5=None, 
         yPt1=None, yPt2=None, yPt3=None, yPt4=None, yPt5=None, eval1=None, calibName=None, behavName=None, name=None, reposPath=None, plasticPoints=None, density=None):
    
        if not calibName == None:
            self.calibName = calibName
        if not behavName == None:
            self.behavName = behavName
        if not ds1Name == None:
            self.ds1Name = ds1Name
        if not anaType == None:
            self.anaType = anaType
        
        if not xPt1 == None:
            self.xPt1 = xPt1
        if not xPt2 == None:
            self.xPt2 = xPt2
        if not xPt3 == None:
            self.xPt3 = xPt3
        if not xPt4 == None:
            self.xPt4 = xPt4
        if not xPt5 == None:
            self.xPt5 = xPt5
        if not yPt1 == None:
            self.yPt1 = yPt1
        if not yPt2 == None:
            self.yPt2 = yPt2
        if not yPt3 == None:
            self.yPt3 = yPt3
        if not yPt4 == None:
            self.yPt4 = yPt4
        if not yPt5 == None:
            self.yPt5 = yPt5
        if not eval1==None:            
            self.eval1=eval1

        if not eL == None:
            self.eL = eL
        if not volStrain == None:
            self.volStrain = volStrain            
        if not plasticPoints == None:
            self.plasticPoints = plasticPoints
  
        self.startTransLoad = self.yPt1
        self.endTransLoad = self.yPt2
        self.startTransUnLoad = self.yPt4
        self.endTransUnLoad = self.yPt5    

        if not name == None:
            self.name = name
        if not reposPath==None:
            self.reposPath=reposPath
        if sigmasl == 'entersigmasl':

            self.sigmascl = sigmaSCl
        elif sigmasl == 'equalsigmasl':

            self.sigmascl = self.startTransLoad
        elif sigmasl == 'timessigmasl':

            self.sigmascl = 1.25 * self.startTransLoad

        if self.volStrain == 'enterVol':
            self.flag1 = 'entered'
        elif self.volStrain == 'eqVol':
            self.flag1 = 'notEntered'
        if not dimension == None:
            self.dimension = dimension
        if not numAnneals == None:
            self.numAnneals = numAnneals
        if not numSteps == None:
            self.numSteps = numSteps
        if sigmaSCl == None:
            self.sigmaSCl = sigmaSCl
        if not sTGradL == None:
            self.sTGradL = sTGradL
        if not sTGradU == None:
            self.sTGradU = sTGradU
        
        if not refTemp == None:
            self.refTemp = refTemp
        if not aPoisRat == None:
            self.aPoisRat = aPoisRat
        if not mPoisRat == None:
            self.mPoisRat = mPoisRat
        if not density == None:
            self.density = density
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #Function to calculate some of the mechanical constants using the picked
    #points
    #
    def calcConstants(self):
       try:
           self.austEa = self.yPt1/self.xPt1
       except:
           msg = "X coordinate of point 1 cannot be zero."
           raise ValueError, msg
       
       try:
           self.martEm = (self.yPt3 - self.yPt2)/(self.xPt3 - self.xPt2)
       except:
           msg = "X coordinates of point 2 and point 3 cannot be identical."
           raise ValueError, msg
       try:
           self.tStrainL = self.xPt3 -  (self.xPt3-self.xPt2)*self.yPt3/(self.yPt3 - self.yPt2)
       except:
           msg = "Y coordinates of point 2 and point 3 cannot be identical."
           raise ValueError, msg
       if self.flag1 == "entered":
           self.vStrain = self.eL
       elif self.flag1 == "notEntered":
           self.vStrain = self.tStrainL
       return self.austEa, self.martEm, self.tStrainL, self.vStrain, self.flag1
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #Function to generate a unique names for a one element model, part,
    #instance, material and step
    #
    def generateName(self, nameType=None, modelName=None, partName=None):

        if nameType == 'Model_mat_eval':
            existingNames= mdb.models.keys()
        elif nameType == 'Part_mat_eval':
            existingNames= mdb.models[modelName].parts.keys()
        elif nameType == 'Step_mat_eval':
            existingNames = mdb.models[modelName].steps.keys()
        elif nameType == 'Instance_mat_eval':
            existingNames= mdb.models[modelName].rootAssembly.instances.keys()
        elif nameType == 'MatName_mat_eval':
            existingNames=mdb.models[modelName].materials.keys()
        count=1        
        while 1:
            if nameType+'_'+str(count) in existingNames:
                count+=1
                continue
            else:
                newName=nameType+'_'+str(count)
            break
        return newName   
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
   #Function to generate a unique plot name
   #
    def generatePlotName(self, nameType):
        count=1       

	if nameType== 'matEvalPlot':

	    existingNames=session.xyPlots.keys()
	else:

	    existingNames=session.xyDataObjects.keys()
	while 1:
	    if nameType+'_'+str(count) in existingNames:
                count+=1
	        continue
	    else:
		newName=nameType+'_'+str(count)
		break
	return newName         
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
    #Function to return a current viewport name
    #
    def getCurrentVp(self):
        vp = session.currentViewportName
        return vp        