from abaqusGui import *
from symbolicConstants import SymbolicConstant
from kernelAccess import session, mdb

###########################################################################
# Class definition
###########################################################################

class NitinolSuperelasticPlasticBehaviorForm( MCBehaviorEditForm ):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def createKeywords( self ):
    
       
        self.ds1NameKw = AFXStringKeyword( self.cmd, 'ds1Name', False)

        self.anaTypeKw = AFXStringKeyword(self.cmd, 'anaType', TRUE, 'std')
        self.sigmaslKw = AFXStringKeyword(self.cmd, 'sigmasl', TRUE, 'equalsigmasl')
        self.volStrainKw = AFXStringKeyword(self.cmd, 'volStrain', TRUE, 'enterVol')
        self.dimensionKw = AFXStringKeyword(self.cmd, 'dimension', TRUE, '3D')
        self.nAnnealsKw = AFXIntKeyword(self.cmd, 'numAnneals', True, 1)
        self.nStepsKw = AFXStringKeyword(self.cmd, 'numSteps', True, '2')
        self.sigSClKw = AFXStringKeyword(self.cmd,'sigmaSCl',True,'1')
        self.elKw = AFXStringKeyword(self.cmd,'eL',True,'0')
        self.rTempKw=AFXFloatKeyword(self.cmd,'refTemp', True, 22.0)
        self.aPoisKw=AFXFloatKeyword(self.cmd,'aPoisRat', True, 0.33)
        self.mPoisKw=AFXFloatKeyword(self.cmd,'mPoisRat', True, 0.33)
        self.tGradLKw=AFXFloatKeyword(self.cmd,'sTGradL', True, 5.)
        self.tGradUKw=AFXFloatKeyword(self.cmd,'sTGradU', True, 5.)
        self.xPt1Kw=AFXFloatKeyword(self.cmd,'xPt1', True, 1.)
        self.YPt1Kw=AFXFloatKeyword(self.cmd,'yPt1', True, 1.)
        self.xPt2Kw=AFXFloatKeyword(self.cmd,'xPt2', True, 1.)
        self.YPt2Kw=AFXFloatKeyword(self.cmd,'yPt2', True, 1.)
        self.xPt3Kw=AFXFloatKeyword(self.cmd,'xPt3', True, 1.)
        self.YPt3Kw=AFXFloatKeyword(self.cmd,'yPt3', True, 1.)
        self.xPt4Kw=AFXFloatKeyword(self.cmd,'xPt4', True, 1.)
        self.YPt4Kw=AFXFloatKeyword(self.cmd,'yPt4', True, 1.)
        self.xPt5Kw=AFXFloatKeyword(self.cmd,'xPt5', True, 1.)
        self.YPt5Kw=AFXFloatKeyword(self.cmd,'yPt5', True, 1.)
        self.plasticPointsKw = AFXTableKeyword(command=self.cmd,
                                                       name='plasticPoints', 
                                                       isRequired=True,
                                                       minLength=0,
                                                       maxLength=-1,
                                                       opts=AFXTABLE_TYPE_FLOAT)
        self.eval1Kw = AFXBoolKeyword(command=self.cmd,name='eval1',isRequired=True,defaultValue=False)       

        self.calibNameKw=AFXStringKeyword(self.cmd, 'calibName', isRequired=True, defaultValue='')
        self.behavNameKw=AFXStringKeyword(self.cmd, 'behavName', isRequired=True, defaultValue='')
        self.densKw = AFXFloatKeyword(self.cmd, 'density', isRequired=True, defaultValue=1e-2)
        self.tgtPt1 = Point1PickProcedure1(self.getOwner(),self)
        self.tgtPt2 = Point2PickProcedure1(self.getOwner(),self)
        self.tgtPt3 = Point3PickProcedure1(self.getOwner(),self)
        self.tgtPt4 = Point4PickProcedure1(self.getOwner(),self)
        self.tgtPt5 = Point5PickProcedure1(self.getOwner(),self)
        self.tgtPPP = PlasticPointPickProcedure1(self.getOwner(),self)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def doCustomChecks(self):
        
        if len(self.materialNameKw.getValue()) > 38:
            msg = "Make sure that given material name does not exceed 38 characters."
            showAFXErrorDialog( getAFXApp().getAFXMainWindow(), msg )
            return 0

        self.calibNameKw.setValue(self.calibrationName)
        self.behavNameKw.setValue(self.behaviorName)
        if self.nAnnealsKw.getValue() <0:
            msg = "Number of annealings to be performed cannot be negative."
            showAFXErrorDialog( getAFXApp().getAFXMainWindow(), msg )
            return 0
        if self.nAnnealsKw.getValue() and self.nStepsKw.getValue() =='':
            msg = "Make sure number of annealings match with annealing step numbers."
            showAFXErrorDialog( getAFXApp().getAFXMainWindow(), msg )
            return 0

        if self.nStepsKw.getValue()!= '':
            nSteps=(self.nStepsKw.getValue()).split(',')
            nAnneals = self.nAnnealsKw.getValue()
            for i in nSteps:

                if not i.isdigit():
                    msg= "Annealing step numbers must be correctly specified."
                    showAFXErrorDialog( getAFXApp().getAFXMainWindow(), msg )
                    return 0

        if self.nStepsKw.getValue()!='':
            nSteps=(self.nStepsKw.getValue()).split(',')

            for i in nSteps:

                if i == '0':
                    msg ="Annealing step number cannot be zero."
                    showAFXErrorDialog( getAFXApp().getAFXMainWindow(), msg )
                    return 0
                       
            
        
        if self.nAnnealsKw.getValue()!=0:
            if self.nAnnealsKw.getValue()!=len((self.nStepsKw.getValue()).split(',')):
                msg = "Make sure number of annealings match with annealing step numbers."
                showAFXErrorDialog( getAFXApp().getAFXMainWindow(), msg )
                return 0   
   
        if self.densKw.getValue()<0 or self.densKw.getValue()==0:

            msg = "For Abaqus/Expliict analysis type positive density must be given."
            showAFXErrorDialog( getAFXApp().getAFXMainWindow(), msg )
            return 0
        if self.xPt1Kw.getValue()==0:
            msg= "X coordinate of point 1 cannot be zero."
            showAFXErrorDialog( getAFXApp().getAFXMainWindow(), msg )
            return 0        
        
        if (self.xPt3Kw.getValue() - self.xPt2Kw.getValue())==0:
            msg= "X coordinates of point 2 and point 3 cannot be identical."
            showAFXErrorDialog( getAFXApp().getAFXMainWindow(), msg )
            return 0   
        
        if (self.YPt2Kw.getValue() - self.YPt4Kw.getValue())==0:
            msg= "Y coordinates of point 2 and point 4 cannot be identical."
            showAFXErrorDialog( getAFXApp().getAFXMainWindow(), msg )
            return 0                

        return 1

        
        
###########################################################################
# Class definition
###########################################################################

class Point1PickProcedure1(AFXProcedure):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, form):
    
        AFXProcedure.__init__(self, owner, AFXProcedure.NORMAL)
        self.form = form
        self.cmd = AFXGuiCommand(mode=self, method='myMethod', objectName='myObject', registerQuery=False)
        self.dataKw = AFXObjectKeyword(command=self.cmd, name='data', isRequired=True,defaultValue='()')
        self.cmd.deactivate()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstStep(self):
    
        self.form.getCurrentDialog().pt1PickBtn.setState(STATE_DOWN)
        self.step1 = AFXPickStep(owner=self, keyword=self.dataKw, prompt='', entitiesToPick=XY_POINT,
                                 numberToPick=ONE, highlightLevel=4, sequenceStyle=TUPLE)
        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def deactivate(self, force=False):
        if force:
            AFXProcedure.deactivate(self)
            return None

        if session.viewports[session.currentViewportName].displayedObject == None:
            AFXProcedure.deactivate(self)
            return None
        # if there is something in the viewport, make sure that it is an XY Plot of the current dataset.
        #
        dsName = self.form.ds1NameKw.getValue()
        if not session.viewports[session.currentViewportName].displayedObject.name == dsName:
            showAFXInformationDialog(owner=self.form.getCurrentDialog(), 
                                     message="Warning: No data set selected", 
                                     tgt=None, 
                                     sel=0)
            AFXProcedure.deactivate(self)
            return None
        
        try:
            # get the picked x value
            #
            cmd_list = self.cmd.getCommandString().splitlines()
            pickedData_str0 = cmd_list[0]
            pickedData_str1 = pickedData_str0.split('=')[1]
            pickedData_str2 = pickedData_str1.replace('(', '').replace(')', '').strip().rstrip(',').split(',')

            xval_str = pickedData_str2[0].strip()
            yval_str = pickedData_str2[1].strip()

            point=(eval(xval_str),eval(yval_str))

            self.form.getCurrentDialog().setPoint1FromXY(point)
            AFXProcedure.deactivate(self)
            return None
        except:
            self.form.getCurrentDialog().pt1PickBtn.setState(STATE_UP)
            AFXProcedure.deactivate(self)
            return None               

        

###########################################################################
# Class definition
###########################################################################

class Point2PickProcedure1(AFXProcedure):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, form):
    
        AFXProcedure.__init__(self, owner, AFXProcedure.NORMAL)
        self.form = form
        self.cmd = AFXGuiCommand(mode=self, method='myMethod', objectName='myObject', registerQuery=False)
        self.dataKw = AFXObjectKeyword(command=self.cmd, name='data', isRequired=True,defaultValue='()')
        self.cmd.deactivate()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstStep(self):
    
        self.form.getCurrentDialog().pt2PickBtn.setState(STATE_DOWN)
        self.step1 = AFXPickStep(owner=self, keyword=self.dataKw, prompt='', entitiesToPick=XY_POINT,
                                 numberToPick=ONE, highlightLevel=4, sequenceStyle=TUPLE)
        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def deactivate(self, force=False):
        if force:
            AFXProcedure.deactivate(self)
            return None

        if session.viewports[session.currentViewportName].displayedObject == None:
            AFXProcedure.deactivate(self)
            return None
        # if there is something in the viewport, make sure that it is an XY Plot of the current dataset.
        #
        dsName = self.form.ds1NameKw.getValue()
        if not session.viewports[session.currentViewportName].displayedObject.name == dsName:
            showAFXInformationDialog(owner=self.form.getCurrentDialog(), 
                                     message="Warning: No data set selected", 
                                     tgt=None, 
                                     sel=0)
            AFXProcedure.deactivate(self)
            return None
            
        try:
            # Get the picked x & y values
            #
            cmd_list = self.cmd.getCommandString().splitlines()
            pickedData_str0 = cmd_list[0]
            pickedData_str1 = pickedData_str0.split('=')[1]
            pickedData_str2 = pickedData_str1.replace('(', '').replace(')', '').strip().rstrip(',').split(',')

            xval_str = pickedData_str2[0].strip()
            yval_str = pickedData_str2[1].strip()

            point=(eval(xval_str),eval(yval_str))

            self.form.getCurrentDialog().setPoint2FromXY(point)
            AFXProcedure.deactivate(self)
            return None            
        except:      
            self.form.getCurrentDialog().pt2PickBtn.setState(STATE_UP)
            AFXProcedure.deactivate(self)
            return None
        
###########################################################################
# Class definition
###########################################################################

class Point3PickProcedure1(AFXProcedure):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, form):
    
        AFXProcedure.__init__(self, owner, AFXProcedure.NORMAL)
        self.form = form
        self.cmd = AFXGuiCommand(mode=self, method='myMethod', objectName='myObject', registerQuery=False)
        self.dataKw = AFXObjectKeyword(command=self.cmd, name='data', isRequired=True,defaultValue='()')
        self.cmd.deactivate()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstStep(self):
    
        self.form.getCurrentDialog().pt3PickBtn.setState(STATE_DOWN)
        self.step1 = AFXPickStep(owner=self, keyword=self.dataKw, prompt='', entitiesToPick=XY_POINT,
                                 numberToPick=ONE, highlightLevel=4, sequenceStyle=TUPLE)
        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def deactivate(self, force=False):
        if force:
            AFXProcedure.deactivate(self)
            return None

        if session.viewports[session.currentViewportName].displayedObject == None:
            AFXProcedure.deactivate(self)
            return None
        # if there is something in the viewport, make sure that it is an XY Plot of the current dataset.
        #
        dsName = self.form.ds1NameKw.getValue()
        if not session.viewports[session.currentViewportName].displayedObject.name == dsName:
            showAFXInformationDialog(owner=self.form.getCurrentDialog(), 
                                     message="Warning: No data set selected", 
                                     tgt=None, 
                                     sel=0)
            AFXProcedure.deactivate(self)
            return None
            
        try:
            # get the picked x value
            #
            cmd_list = self.cmd.getCommandString().splitlines()
            pickedData_str0 = cmd_list[0]
            pickedData_str1 = pickedData_str0.split('=')[1]
            pickedData_str2 = pickedData_str1.replace('(', '').replace(')', '').strip().rstrip(',').split(',')

            xval_str = pickedData_str2[0].strip()
            yval_str = pickedData_str2[1].strip()

            point=(eval(xval_str),eval(yval_str))

            self.form.getCurrentDialog().setPoint3FromXY(point)
            AFXProcedure.deactivate(self)
            return None            
        except:
            self.form.getCurrentDialog().pt3PickBtn.setState(STATE_UP)
            AFXProcedure.deactivate(self)
            return None

        
###########################################################################
# Class definition
###########################################################################

class Point4PickProcedure1(AFXProcedure):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, form):
    
        AFXProcedure.__init__(self, owner, AFXProcedure.NORMAL)
        self.form = form
        self.cmd = AFXGuiCommand(mode=self, method='myMethod', objectName='myObject', registerQuery=False)
        self.dataKw = AFXObjectKeyword(command=self.cmd, name='data', isRequired=True,defaultValue='()')
        self.cmd.deactivate()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstStep(self):
    
        self.form.getCurrentDialog().pt4PickBtn.setState(STATE_DOWN)
        self.step1 = AFXPickStep(owner=self, keyword=self.dataKw, prompt='', entitiesToPick=XY_POINT,
                                 numberToPick=ONE, highlightLevel=4, sequenceStyle=TUPLE)
        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def deactivate(self, force=False):
        if force:
            AFXProcedure.deactivate(self)
            return None

        if session.viewports[session.currentViewportName].displayedObject == None:
            AFXProcedure.deactivate(self)
            return None
        # if there is something in the viewport, make sure that it is an XY Plot of the current dataset.
        #
        dsName = self.form.ds1NameKw.getValue()
        if not session.viewports[session.currentViewportName].displayedObject.name == dsName:
            showAFXInformationDialog(owner=self.form.getCurrentDialog(), 
                                     message="Warning: No data set selected", 
                                     tgt=None, 
                                     sel=0)
            AFXProcedure.deactivate(self)
            return None
            
        try:
            # get the picked x value
            cmd_list = self.cmd.getCommandString().splitlines()
            pickedData_str0 = cmd_list[0]
            pickedData_str1 = pickedData_str0.split('=')[1]
            pickedData_str2 = pickedData_str1.replace('(', '').replace(')', '').strip().rstrip(',').split(',')

            xval_str = pickedData_str2[0].strip()
            yval_str = pickedData_str2[1].strip()

            point=(eval(xval_str),eval(yval_str))

            self.form.getCurrentDialog().setPoint4FromXY(point)
            AFXProcedure.deactivate(self)
            return None
        except:
            self.form.getCurrentDialog().pt4PickBtn.setState(STATE_UP)
            AFXProcedure.deactivate(self)
            return None
               
       

###########################################################################
# Class definition
###########################################################################

class Point5PickProcedure1(AFXProcedure):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, form):
    
        AFXProcedure.__init__(self, owner, AFXProcedure.NORMAL)
        self.form = form
        self.cmd = AFXGuiCommand(mode=self, method='myMethod', objectName='myObject', registerQuery=False)
        self.dataKw = AFXObjectKeyword(command=self.cmd, name='data', isRequired=True,defaultValue='()')
        self.cmd.deactivate()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstStep(self):
    
        self.form.getCurrentDialog().pt5PickBtn.setState(STATE_DOWN)
        self.step1 = AFXPickStep(owner=self, keyword=self.dataKw, prompt='', entitiesToPick=XY_POINT,
                                 numberToPick=ONE, highlightLevel=4, sequenceStyle=TUPLE)
        return self.step1

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def deactivate(self, force=False):
        if force:
            AFXProcedure.deactivate(self)
            return None

        if session.viewports[session.currentViewportName].displayedObject == None:
            AFXProcedure.deactivate(self)
            return None
        # if there is something in the viewport, make sure that it is an XY Plot of the current dataset.
        #
        dsName = self.form.ds1NameKw.getValue()
        if not session.viewports[session.currentViewportName].displayedObject.name == dsName:
            showAFXInformationDialog(owner=self.form.getCurrentDialog(), 
                                     message="Warning: No data set selected", 
                                     tgt=None, 
                                     sel=0)
            AFXProcedure.deactivate(self)
            return None
            
        try:
            # get the picked x value
            #
            cmd_list = self.cmd.getCommandString().splitlines()
            pickedData_str0 = cmd_list[0]
            pickedData_str1 = pickedData_str0.split('=')[1]
            pickedData_str2 = pickedData_str1.replace('(', '').replace(')', '').strip().rstrip(',').split(',')

            xval_str = pickedData_str2[0].strip()
            yval_str = pickedData_str2[1].strip()

            point=(eval(xval_str),eval(yval_str))

            self.form.getCurrentDialog().setPoint5FromXY(point)
        except:
            self.form.getCurrentDialog().pt4PickBtn.setState(STATE_UP)
            AFXProcedure.deactivate(self)
            return None
############################################################################
## Class definition
############################################################################

class PlasticPointPickProcedure1(AFXProcedure):

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, owner, form):
    
        AFXProcedure.__init__(self, owner, AFXProcedure.SUBPROCEDURE)
        self.form = form
        self.cmd = AFXGuiCommand(mode=self, method='myMethod', objectName='myObject', registerQuery=False)
        self.dataKw = AFXObjectKeyword(command=self.cmd, name='data', isRequired=True,defaultValue='()')
        self.cmd.deactivate()

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getFirstStep(self):
    
        self.form.getCurrentDialog().elasticPlasticP11PickPlasticBTN.setState(STATE_DOWN)
        self.step1 = AFXPickStep(owner=self, 
                                 keyword=self.dataKw,
                                 prompt='Pick XY points from the plot, click cancel when done.',
                                 entitiesToPick=XY_POINT,
                                 numberToPick=ONE,
                                 highlightLevel=1,
                                 sequenceStyle=TUPLE
                                 )
        return self.step1
    
    def getLoopStep(self):
    	return self.step1
    
    def issueCommands(self, writeToReplay=False, writeToJournal=False):

        # get the picked x and y values
        #
        cmd_list = self.cmd.getCommandString()

        pickedData_str0 = cmd_list
        pickedData_str1 = pickedData_str0.split('=')[1]
        pickedData_str2 = pickedData_str1.replace('(', '').replace(')', '').strip().rstrip(',').split(',')
        try:
            xval_str = pickedData_str2[0].strip()
            yval_str = pickedData_str2[1].strip()
        except:
            print 'An error occurred when processing the picked plastic point.'
        
        #test whether picked x value is in range
        #

        modelName = self.form.modelName
        calibrationName = self.form.calibrationName
        behaviorName = self.form.behaviorName
        dsName = self.form.ds1NameKw.getValue()

        
        try:

            picked_xval = eval(xval_str)
            picked_yval = eval(yval_str)
            
            xval = xval_str
            yval = yval_str


            picked_pt = picked_xval, picked_yval
 
            if self.form.plasticPointsKw.getValues()=="": # No plastic points

                plastic_points = (picked_pt,)
            else:

                try:
                    plastic_points = eval(self.form.plasticPointsKw.getValues()) # eval to turn string into tuple
                    plastic_points[0][0]
                except:
                    plastic_points = (eval(self.form.plasticPointsKw.getValues()),)

            index = 0
            duplicate_pt = False

            if len(plastic_points) > 1:

                try:
                    for pt in plastic_points:
                        if round(picked_pt[0],5) == round(pt[0],5):
                            duplicate_pt = True
                        if round(picked_pt[0],5) > round(pt[0],5) \
                            and round(picked_pt[0],5) < round(plastic_points[index+1][0],5):
                            break
                        else:
                            index = index+1
                except:
                    index = len(plastic_points)-1
            else: ## len(plastic_points) == 1

                if round(picked_pt[0],5) > round(plastic_points[0][0],5):
                    index = 0
                else:
                    index = -1
  
            tablePlasticPoints = ()

            if not duplicate_pt:
                try:
                    if self.form.plasticPointsKw.getValues()=="": # No plastic points
                        self.form.plasticPointsKw.insertRows(0, 1)
                        rowString = str(xval) + "," + str(yval)
                        self.form.plasticPointsKw.setRow(0, rowString)
                        tablePlasticPoints = (eval(self.form.plasticPointsKw.getValues()),)
                    else:    
                        self.form.plasticPointsKw.insertRows(index+1, 1)
                        rowString = str(xval) + "," + str(yval)
                        self.form.plasticPointsKw.setRow(index+1, rowString)
                        tablePlasticPoints = eval(self.form.plasticPointsKw.getValues())
  
                    self.form.getCurrentDialog().plotPickedPoint(plasticPoints=tablePlasticPoints)
                except:
                    self.form.getCurrentDialog().elasticPlasticP11PickPlasticBTN.setState(STATE_UP)
            #set button to up state
            #
            self.form.getCurrentDialog().elasticPlasticP11PickPlasticBTN.setState(STATE_UP)
            
        except:
            self.form.getCurrentDialog().elasticPlasticP11PickPlasticBTN.setState(STATE_UP)
        
        self.form.getCurrentDialog().elasticPlasticP11PickPlasticBTN.setState(STATE_UP)
        return True
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def deactivate(self, force=False):
        
        if force:
            AFXProcedure.deactivate(self)
            return None

        if session.viewports[session.currentViewportName].displayedObject == None:
            AFXProcedure.deactivate(self)
            return None
        # if there is something in the viewport, make sure that it is an XY Plot of the current dataset.
        #
        dsName = self.form.ds1NameKw.getValue()
        if not session.viewports[session.currentViewportName].displayedObject.name == dsName:
            showAFXInformationDialog(owner=self.form.getCurrentDialog(), 
                                     message="Warning: No data set selected", 
                                     tgt=None, 
                                     sel=0)
            AFXProcedure.deactivate(self)
            return None

        AFXProcedure.deactivate(self)                