from abaqusGui import *
from nom2trueConstants import *

class Nom2trueDB(AFXDataDialog):

    [
        ID_FILE
    ] = range(AFXToolsetGui.ID_LAST, AFXToolsetGui.ID_LAST+1)


    def __init__(self, form):

        self.form = form
        self.fileDb = None
    
        # Construct the base class dialog
        #
        AFXDataDialog.__init__(self, form, 
                               title='Nominal Stress-Strain Processing')

        FXMAPFUNC(self, SEL_COMMAND, self.ID_FILE, Nom2trueDB.onCmdFile)

        # File name
        #
        gb = FXGroupBox(self, 'Raw Data File', FRAME_GROOVE|LAYOUT_FILL_X)
        va = AFXVerticalAligner(gb, LAYOUT_FILL_X)
        hf = FXHorizontalFrame(va, LAYOUT_FILL_X)
        AFXTextField(hf, 25, 'File name:', form.dataFileNameKw, 0, LAYOUT_FILL_X)
        FXButton(hf, 'Select...', None, self, self.ID_FILE)

        # Strain, Stress column input
        #
        va = AFXVerticalAligner(gb, LAYOUT_FILL_X)
        AFXTextField(va, ncols=6, 
                     labelText='Read nominal strain values from column:', 
	             tgt=form.eColumnKw, sel=0)
        AFXTextField(va, ncols=6, 
                     labelText='Read nominal stress values from column:', 
	             tgt=form.sColumnKw, sel=0)

        FXLabel(gb, 'Field delimeter: spaces, tabs or commas', None,
                LAYOUT_CENTER_X)
        FXLabel(gb, 'Comment lines beginning with "**" are allowed.', None,
                LAYOUT_CENTER_X)

        # Skip Frequency Radio Buttons
        #
        gb = FXGroupBox(self, 'Data Filter', FRAME_GROOVE|LAYOUT_FILL_X)
        FXRadioButton(gb, 'Read all rows', form.dataFilterKw, NOFILTER.getId())
        hf = FXHorizontalFrame(gb, pl=0, pr=0, pt=0, pb=0)
        FXRadioButton(hf, 'Skip', form.dataFilterKw, FILTER.getId())
        tf = AFXTextField(hf, ncols=6, labelText=None, tgt=form.skipKw, sel=0)
        FXLabel(hf, 'rows between reads', None)

        # Transition for radio button text box

        self.addTransition(form.dataFilterKw, AFXTransition.EQ, NOFILTER.getId(),
            tf, MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)
        self.addTransition(form.dataFilterKw, AFXTransition.EQ, FILTER.getId(),
            tf, MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)

        # Manipulate Data
        #
        gb = FXGroupBox(self, 'Manipulate Raw Data', FRAME_GROOVE|LAYOUT_FILL_X)
        m  = FXMatrix(gb, 4)
        # Row 1, Col 1
	FXCheckButton(m, 'Shift all points by (dX,dY):', 
                      tgt=form.shiftValuesKw, sel=0)
        # Row 2, Col 1
	FXCheckButton(m, 'Scale all points by (X,Y):', 
                      tgt=form.scaleValuesKw, sel=0)
        # Row 3, Col 1
	FXCheckButton(m, 'Extrapolate to max. strain of:', 
                      tgt=form.extrapolateValuesKw, sel=0)
        # Row 4, Col 1
        FXCheckButton(m, 'Allow decreasing stress values', 
                      tgt=form.allowDecreasingValuesKw, sel=0)
        # Row 1, Col 2
	tf1 = AFXTextField(m, ncols=15, labelText=None, 
                           tgt=form.shiftKw, sel=0)
        # Row 2, Col 2
	tf2 = AFXTextField(m, ncols=15, labelText=None, 
                           tgt=form.scaleKw, sel=0)
        # Row 3, Col 2
	tf3 = AFXTextField(m, ncols=15, labelText=None, 
                           tgt=form.extrapolateStrainKw, sel=0)

        self.addTransition(form.shiftValuesKw, AFXTransition.EQ, TRUE, tf1,
            MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(form.shiftValuesKw, AFXTransition.EQ, FALSE, tf1,
            MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)
        self.addTransition(form.scaleValuesKw, AFXTransition.EQ, TRUE, tf2,
            MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(form.scaleValuesKw, AFXTransition.EQ, FALSE, tf2,
            MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)
        self.addTransition(form.extrapolateValuesKw, AFXTransition.EQ, TRUE, tf3,
            MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(form.extrapolateValuesKw, AFXTransition.EQ, FALSE, tf3,
            MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)


        # Elastic modulus & Proportional limit
        #
        gb = FXGroupBox(self, 'Elastic Modulus & Proportional Limit', 
             FRAME_GROOVE|LAYOUT_FILL_X)
        va = AFXVerticalAligner(gb, pl=0, pr=0, pt=0, pb=0)
        FXRadioButton(va, 'Specify elastic modulus and proportional limit', 
                      form.modulusAndLimitKw, MANUAL.getId())
        hf = FXHorizontalFrame(gb, pl=18, pr=0, pt=0, pb=0)
	tf1 = AFXTextField(hf, ncols=10, labelText='E:', tgt=form.EKw, 
              sel=0)
	tf2 = AFXTextField(hf, ncols=10, labelText='     Prop. Limit:', 
              tgt=form.proportionalLimitKw, sel=0)

        FXRadioButton(gb, 'Auto estimate elastic modulus and proportional limit', 
                      form.modulusAndLimitKw, AUTO.getId())
	lb = FXLabel(gb, 'Goodness of fit, R-squared, for elastic modulus:', 
                     None, LAYOUT_CENTER_X)
        slider = AFXSlider(gb, form.fitKw, 0, 
            AFXSLIDER_INSIDE_BAR|AFXSLIDER_SHOW_VALUE|LAYOUT_FILL_X|LAYOUT_CENTER_X)
        slider.setDecimalPlaces(2)
        slider.setRange(0, 100)
        slider.setValue(95)

        # Transition for elastic modulus radio button

        self.addTransition(form.modulusAndLimitKw, AFXTransition.EQ, 
            MANUAL.getId(), lb, MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)
        self.addTransition(form.modulusAndLimitKw, AFXTransition.EQ, 
            MANUAL.getId(), slider, MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), 
            None)
        self.addTransition(form.modulusAndLimitKw, AFXTransition.EQ, 
            MANUAL.getId(), tf1, MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(form.modulusAndLimitKw, AFXTransition.EQ, 
            MANUAL.getId(), tf2, MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(form.modulusAndLimitKw, AFXTransition.EQ, AUTO.getId(),
            lb, MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(form.modulusAndLimitKw, AFXTransition.EQ, AUTO.getId(),
            slider, MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(form.modulusAndLimitKw, AFXTransition.EQ, AUTO.getId(),
            tf1, MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)
        self.addTransition(form.modulusAndLimitKw, AFXTransition.EQ, AUTO.getId(),
            tf2, MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)

        # Check button & Save file name
        #
        gb = FXGroupBox(self, 'Save Input File Snippet', 
                        FRAME_GROOVE|LAYOUT_FILL_X)
        m  = FXMatrix(gb, 2)

        # Row 1, Col 1
        FXCheckButton(m, 'Save to file:', tgt=form.saveFileKw, sel=0)

        # Row 2, Col 1
	FXLabel(m, "Poisson's Ratio:", None)

        # Row 1, Col 2
	tf1 = AFXTextField(m, ncols=30, labelText=None, 
                     tgt=form.saveNameKw, sel=0)
        # Row 2, Col 2
	tf2 = AFXTextField(m, ncols=5, labelText=None, 
                     tgt=form.poissonsKw, sel=0)


        self.addTransition(form.saveFileKw, AFXTransition.EQ, TRUE, tf1,
            MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(form.saveFileKw, AFXTransition.EQ, FALSE, tf1,
            MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)
        self.addTransition(form.saveFileKw, AFXTransition.EQ, TRUE, tf2,
            MKUINT(FXWindow.ID_ENABLE, SEL_COMMAND), None)
        self.addTransition(form.saveFileKw, AFXTransition.EQ, FALSE, tf2,
            MKUINT(FXWindow.ID_DISABLE, SEL_COMMAND), None)

        self.appendActionButton('Plot', self, self.ID_CLICKED_APPLY)
        self.appendActionButton('Cancel', self, self.ID_CLICKED_CANCEL)


    def onCmdFile(self, sender, sel, ptr):
        if not self.fileDb:
            patterns = 'All files (*.*)'
            self.fileDb = AFXFileSelectorDialog(self, 'Select a File', 
                self.form.dataFileNameKw, None, 
                AFXSELECTFILE_ANY, patterns, self.form.filePatternIndexTgt)
            self.fileDb.setReadOnlyPatterns('*.odb')
            self.fileDb.create()
            
        self.fileDb.showModal()
        return 1

