"Path XML derived class"

from abaqus import *
from abaqusConstants import *

import os.path

execfile(os.path.join(os.path.dirname(os.path.dirname(__file__)), "commonExec.py")) # Initializes the importFile function
(abqXML, xmlUtils) = importFile(__file__, ("caeXML.kernel.abqXML", "caeXML.kernel.xmlUtils")) # Allows base-package directory to be changed
(settingsModule, ) = importFile(__file__, ("caeXML.paths.pathSettings", )) # Settings Module

################################################################################

def readPath(rootEle, funcList, baseObj, parameterDict):
    "This function reads object of type view of the base viewport"
    objType = xmlUtils.getObjectTypeFromXmlElement(rootEle)
    if objType != "Path":
        return (None, None, False)
    (memberDict, tag, success) = xmlUtils.readObjectValueAsDictionary(rootEle, funcList, baseObj, parameterDict)
    if not success:
        return (None, None, False)
    if 'type' not in memberDict.keys():
        return (None, None, False)
    paramArr = parameterDict['curParams']
    if paramArr:
        if type(paramArr) == type(()):
            if len(paramArr) == 1:
                if type(paramArr[0]) == type(""):
                    memberDict['name'] = paramArr[0]
    typ = memberDict['type']
    cmd = xmlUtils.getStringRepresentationOfObject(baseObj[-1], "baseObj[-1]") + ".Path("
    for key in memberDict.keys():
        addKey = False
        if typ == CIRCUMFERENTIAL or typ == RADIAL:
            if key == 'circleDefinition' or key == 'numSegments':
                addKey = True
        if typ == CIRCUMFERENTIAL:
            if key == 'startAngle' or key == 'endAngle' or key == 'radius':
                addKey = True
        if typ == RADIAL:
            if key == 'radialAngle' or key == 'startRadius' or key == 'endRadius':
                addKey = True
        if key == 'name' or key == 'type' or key == 'expression':
            addKey = True
        if addKey:
            cmd = cmd + key + '=' + xmlUtils.getStringRepresentationOfObject(memberDict[key], "memberDict[" + key.__repr__() + "]") + ', '
    cmd = cmd[:-2] + ")"
    try:
        val = eval(xmlUtils.upgradeText(cmd, parameterDict))
    except:
        return (None, None, False)  # Wrong parameters for constructor
    return (val, tag, True)

################################################################################

class PathXML(abqXML.AbqXML):
    "This class specializes the abqXML class for session path export"

    ################################################################################

    def __init__(self):
        self.m_settingsModule = settingsModule
        abqXML.AbqXML.__init__(self)
        self.m_str = ["session.paths[]"]
        self.m_objLoadFuncList = [readPath] + self.m_objLoadFuncList

    ################################################################################
    
    def saveInfo(self):
        "This function saves the information about various paths into xmldoc"
        return self.parseAndAttachMultiple()

    ################################################################################

    def loadInfo(self, saveEle):
        "This function loads the path information from the xmldoc"

        results = self.detachAndReturn(self.m_str[0], [session])
        return self.numResults(results)
    
################################################################################
