"Profiles XML derived class"

from abaqus import *
from abaqusConstants import *

currentModeViewer = False
# The plug-in may have been initialized in Abaqus/Viewer
try:
    import material  # Intentional. Profile library is available even in Abaqus/Viewer even though mdb.models[modelName].profiles does not exist!!!
except:
    currentModeViewer = True
    raise AssertionError, "Cannot access profiles. This error is usually shown when the plug-in is used in Abaqus/Viewer instead of Abaqus/CAE."

import os.path

execfile(os.path.join(os.path.dirname(os.path.dirname(__file__)), "commonExec.py")) # Initializes the importFile function
(abqXML, xmlUtils) = importFile(__file__, ("caeXML.kernel.abqXML", "caeXML.kernel.xmlUtils")) # Allows base-package directory to be changed
(settingsModule, ) = importFile(__file__, ("caeXML.profiles.profileSettings", )) # Settings Module

################################################################################

def isObjOfTypeProfile(objType, memberName):
    "This function checks if the given object is of type profile"
    if objType.endswith('Profile'):
        if memberName == 'name':
            return True
    return False

################################################################################

class ProfilesXML(abqXML.AbqXML):
    "This class specializes the abqXML class for profiles export"

    ################################################################################

    def __init__(self):
        self.m_settingsModule = settingsModule
        abqXML.AbqXML.__init__(self)
        self.m_str = ["mdb.models[].profiles[]"]
        self.m_overwrite = True # To avoid fake warning messages
        self.m_nameCheckFuncs = [isObjOfTypeProfile]
        #self.m_readObjectsCreatedFromLibrary = {'SectionLayer':'section', 'LayerProperties':'section'}

    ################################################################################
    
    def saveInfo(self):
        "This function saves the profiles objects into xml doc"
        return self.parseAndAttachMultiple()

    ################################################################################

    def loadInfo(self, saveEle):
        "This function loads the profiles information from the xmldoc"

        count = 0
        results = self.getMatchingKeys(self.m_str[0])
        for result in results:
            modelName = result[0][0]
            profileName = result[0][1]
            if self.m_params:
                if type(self.m_params) == type(()):
                    if len(self.m_params) == 2:
                        modelName = self.m_params[0]
                        profileName = self.m_params[1]
            if modelName in mdb.models.keys():
                if self.objOverwriteCheck(self.m_str[0], (modelName, profileName)):
                    results2 = self.detachAndReturn(result[1], [mdb.models[modelName]])
                    count += self.numResults(results2)
        return count

################################################################################
