from abaqus import *
from abaqusConstants import *
import visualization, uti
import math
from odbAccess import *

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class StlKernel:
    
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self):
        """Constructor method"""
        pass

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def calculateStl(self, elset1=None, elset2=None):
        """Method to calculate the STL between two elsets"""

        # get current odb
        vp = session.viewports[session.currentViewportName]
        odb = vp.displayedObject
        if type(odb) != OdbType:
            raise TypeError, 'ODB must be displayed in active viewport.'

        if elset1 == None:
            raise TypeError, 'elset1 must be specified.'
        if elset2 == None:
            raise TypeError, 'elset2 must be specified.'
            
        # do calculation
        data = []
        es = elset1.split('.')
        if len(es) == 1:
            # must be an assembly set
            elset1set = odb.rootAssembly.elementSets[es[0]]
            lenElset1 = len(elset1set.elements[0])
        else: 
            # must be an instance set
            elset1set = odb.rootAssembly.instances[es[0]].elementSets[es[1]]
            lenElset1 = len(elset1set.elements)
        es = elset2.split('.')
        if len(es) == 1:
            # must be an assembly set
            elset2set = odb.rootAssembly.elementSets[es[0]]
            lenElset2 = len(elset2set.elements[0])
        else:
            # must be an instance set
            elset2set = odb.rootAssembly.instances[es[0]].elementSets[es[1]]
            lenElset2 = len(elset2set.elements)
            
        for step in odb.steps.values():
            print 'Working on step %s' % step.name
            isFirst = 1
            for frame in step.frames:
                if isFirst:
                    isFirst = 0
                    continue
                if not frame.fieldOutputs.has_key('INTEN'):
                    continue
                freq = frame.frameValue
                print '  Working on frequency %s' % freq
                incident = frame.fieldOutputs['INTEN'].getSubset(position=CENTROID).getSubset(region = elset1set)
                incident_sum = 0
                if len(incident.values) == 0:
                    print '    No INTEN data found for set %s. Ignoring frame.' % elset1
                    continue
                elif len(incident.values) != lenElset1:
                    print '    INTEN data not available for all elements in set %s. Ignoring frame.' % elset1
                    continue                    
                for value in incident.values:
                    temp1 = value.data
                    temp2 = value.conjugateData
                    mag = 0.0
                    for i in range(len(temp1)):
                        mag += abs((temp1[i]**2) + (temp2[i]**2))
                    incident_sum += mag**0.5

                transmitted = frame.fieldOutputs['INTEN'].getSubset(position=CENTROID).getSubset(region = elset2set)
                transmitted_sum = 0
                if len(transmitted.values) == 0:
                    print '    No INTEN data found for set %s. Ignoring frame.' % elset2
                    continue
                elif len(transmitted.values) != lenElset2:
                    print '    INTEN data not available for all elements in set %s. Ignoring frame.' % elset2
                    continue                    
                for value in transmitted.values:
                    temp1 = value.data
                    temp2 = value.conjugateData
                    mag = 0.0
                    for i in range(len(temp1)):
                        mag += abs((temp1[i]**2) + (temp2[i]**2))
                    transmitted_sum += mag**0.5

                stl = 10.0*math.log10(incident_sum/transmitted_sum)
                data.append((freq,stl))

        # create and plot data
        if len(data) == 0:
            raise ValueError, 'No data could be calculated\nbetween given sets.'
        curveName = 'STL-%s-%s' % (elset1, elset2)
        xy1 = session.XYData(name=curveName, data=data)
        try:
            xyp = session.xyPlots['XYPlot-1']
        except:
            xyp = session.XYPlot('XYPlot-1')

       
        chartName = xyp.charts.keys()[0]
        chart = xyp.charts[chartName]
        c1 = session.Curve(xyData=xy1)
        chart.setValues(curvesToPlot=(c1, ), )
        chart.axes1[0].axisData.setValues(useSystemTitle=False, title='Frequency',)
        chart.axes2[0].axisData.setValues(useSystemTitle=False, title='STL (dB)',)

        vp.setValues(displayedObject=xyp)





