from abaqusGui import *
from abq_CaeManagerTemplate.managerDB import ManagerDB


###########################################################################
# Class definition
###########################################################################

class TriangleManagerDB(ManagerDB):

    [
        ID_PRINT,
        ID_LAST
    ] = range(ManagerDB.ID_LAST, ManagerDB.ID_LAST+2)
    

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def __init__(self, form, repository, title, leadingRowLabels='Name'):
            
        ManagerDB.__init__(self, form, repository, title, leadingRowLabels)
                                
        # Append a custom button to the action area
        #
        self.printBtn = self.appendActionButton('Print...', self, self.ID_PRINT)
        FXMAPFUNC(self, SEL_COMMAND, self.ID_PRINT, TriangleManagerDB.onCmdPrint)

        # Move the custom button before the delete button.
        #
        deleteBtn = self.getActionButton(self.ID_DELETE)
        self.printBtn.linkBefore(deleteBtn)
        
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getDefaultWidth(self):
    
        # Set an initial width to the dialog, otherwise the default will
        # be too narrow.
        #
        return 350

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def onCmdPrint(self, sender, sel, ptr):
        
        # Print information about the selected item to the Message Area.
        
        name = self.getSelections()[0]
        exec("base = mdb.customData.triangles['%s'].base" % name)
        exec("height = mdb.customData.triangles['%s'].height" % name)
        getAFXApp().getAFXMainWindow().writeToMessageArea('* Triangle: %s' % name)
        getAFXApp().getAFXMainWindow().writeToMessageArea('  Base  : %s' % base)
        getAFXApp().getAFXMainWindow().writeToMessageArea('  Height: %s' % height)
            
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    def setButtonStates(self, numSelections):
    
        # Set the print button state based on
        # whether one or more items are selected.
        #
        if numSelections == 0:
            self.printBtn.disable()
            
        elif numSelections == 1:
            self.printBtn.enable()           
        
        elif numSelections > 1:
            self.printBtn.disable()    
            
        # Have the base class handle the states of the rest
        # of the action area buttons.
        #
        ManagerDB.setButtonStates(self, numSelections)
                
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    def getSummary(self):
    
        # Get the name and hypotenuse for each triangle.
        
        from kernelAccess import mdb
        exec('keys = %s.keys()' % self.repository)
        keys.sort()
        summary = []
        for key in keys:
            hypotenuse = str(mdb.customData.triangles[key].hypotenuse)
            summary.append((key, hypotenuse))
            
        return summary
