"Sketch Background XML derived class"

from abaqus import *

import os.path

execfile(os.path.join(os.path.dirname(os.path.dirname(__file__)), "commonExec.py")) # Initializes the importFile function
(abqXML, xmlUtils) = importFile(__file__, ("caeXML.kernel.abqXML", "caeXML.kernel.xmlUtils")) # Allows base-package directory to be changed
(settingsModule, ) = importFile(__file__, ("caeXML.sketchBackground.sketchSettings", )) # Settings Module

################################################################################

def readImageOption(rootEle, funcList, baseObj, parameterDict):
    "Function to load object of the type Image"
    if rootEle.tagName != 'imageOptions':
        return (None, None, False)
    (objDict, tag, success) = xmlUtils.readObjectValueAsDictionary(rootEle, funcList, baseObj, parameterDict)
    if not success:
        return (None, None, False)
    if objDict["imageName"] not in session.images.keys() and objDict["imageName"] != "":
        return (None, None, False)
    cmd = xmlUtils.getStringRepresentationOfObject(baseObj[-1], "baseObj[-1]")
    cmd = cmd + ".imageOptions.setValues("
    imageKw = ["imageName", "origin", "showImage", "translucency", "xScale", "yScale"]
    for kw in imageKw:
        if type(objDict[kw]) == type(""):
            cmd = cmd + kw + "=" + objDict[kw].__repr__() + ", "
        else:
            cmd = cmd + kw + "=" + str(objDict[kw]) + ", "
    cmd = cmd[:-2] + ")"
    retVal = eval(xmlUtils.upgradeText(cmd, parameterDict))
    return (retVal, tag, True)

################################################################################

def readImage(rootEle, funcList, baseObj, parameterDict):
    "Function to load object of the type Image"
    objType = xmlUtils.getObjectTypeFromXmlElement(rootEle)
    if objType != 'Image':
        return (None, None, False)
    (objDict, tag, success) = xmlUtils.readObjectValueAsDictionary(rootEle, funcList, baseObj, parameterDict)
    if not success:
        return (None, None, False)
    if not os.path.exists(objDict["fileName"]):
        getWarningReply("Could not find image '" + objDict["fileName"] + "'\nPlease place the image file in appropriate folder and try again", (CANCEL,))
        return (None, None, False)
    cmd = "session.Image("
    imageKw = ["name", "fileName"]
    for kw in imageKw:
        cmd = cmd + kw + "=" + objDict[kw].__repr__() + ", "
    cmd = cmd[:-2] + ")"
    retVal = eval(xmlUtils.upgradeText(cmd, parameterDict))
    return (retVal, tag, True)

################################################################################

class SketchXML(abqXML.AbqXML):
    "This class specializes the abqXML class for sketch background export"

    ################################################################################

    def __init__(self):
        self.m_settingsModule = settingsModule
        abqXML.AbqXML.__init__(self)
        self.m_str = ["session.images[]", "mdb.models[].sketches[].imageOptions"]
        self.m_objLoadFuncList = [readImageOption, readImage] + self.m_objLoadFuncList
        self.m_overwrite = True

    ################################################################################
    
    def saveInfo(self):
        "This function saves the information about sketch background into xmldoc"
        if "models" in mdb.__members__:
            if "sketches" in mdb.models.values()[0].__members__:   # This check will fail if used in Abaqus/Viewer
                return self.parseAndAttachMultiple()
        raise AssertionError, "Cannot save the required information as the plug-in cannot navigate to mdb.models[modelName].sketches[partName]. This is usually observed when the information is exported in Abaqus/Viewer instead of Abaqus/CAE."
        return None

    ################################################################################

    def loadInfo(self, saveEle):
        "This function loads the sketch background information from the xmldoc"
        results = self.detachAndReturn(self.m_str[0], [session])
        count = 0
        results = self.getMatchingKeys(self.m_str[1])
        for result in results:
            modelName = result[0][0]
            sketchName = result[0][1]
            if self.m_params:
                if type(self.m_params) == type(()):
                    if len(self.m_params) == 2:
                        modelName = self.m_params[0]
                        sketchName = self.m_params[1]
            if modelName in mdb.models.keys():
                if sketchName in mdb.models[modelName].sketches.keys():
                    results2 = self.detachAndReturn(result[1], [mdb.models[modelName].sketches[sketchName]])
                    count += self.numResults(results2)
        return count
    
################################################################################
