from abaqusGui import *
import os, uti

majorNumber, minorNumber, updateNumber = getAFXApp().getBaseVersionNumbers()
verAbaqus = float(majorNumber) + minorNumber * 0.1
if verAbaqus > 6.6:
    import _abqPluginUtils
    
thisPath = os.path.abspath(__file__)
thisDir = os.path.dirname(thisPath)

# Use package syntax to avoid possible name clashes with other plugins
#
from abq_SlideLine.slnSlideLineForm import SlnSlideLineForm

# Register commands
#
pgVersion = '1.3-5'
if verAbaqus < 6.5:
    status = "Plug-in SlideLine " + str(pgVersion) + " version was not registered because "\
             "it is not supported in Abaqus version " + str(uti.getVersion())
elif verAbaqus == 6.5 or verAbaqus == 6.6:
    status = 'OK'
else:
    status = _abqPluginUtils.checkCompatibility('SlideLine', pgVersion)
if status == 'OK':
    toolset=getAFXApp().getAFXMainWindow().getPluginToolset()
    helpUrlLocation = os.path.join(os.getcwd(), 'SlideLine_help.html')
    toolset.registerGuiMenuButton(
        object=SlnSlideLineForm(toolset),
        buttonText='Tools|Slideline Creation...',
        messageId=AFXMode.ID_ACTIVATE,
        icon=None,
        kernelInitString='import abq_SlideLine.slnModule',
        applicableModules=['Assembly','Interaction','Load',],
        version=pgVersion,
        author='Dassault Systemes',
        description='This plug-in generates slide line definitions based on defined surfaces in Abaqus/CAE',
        helpUrl='http://abaqus.custhelp.com/cgi-bin/abaqus.cfg'
                '/php/enduser/std_adp.php?p_faqid=3077')
else:
    getAFXApp().getAFXMainWindow().writeToMessageArea(status)
    getAFXApp().beep()
