from abaqusGui import *

# Use package syntax to avoid possible name clashes with other plugins
#
from sTL.stlForm import StlForm

majorNumber, minorNumber, updateNumber = getAFXApp().getBaseVersionNumbers()
verAbaqus = float(majorNumber) + minorNumber * 0.1
if verAbaqus > 6.6:
    import _abqPluginUtils

# Register commands
#
version = '1.1-4'
if verAbaqus < 6.5:
    status = "Plug-in STL " + str(version) + " version was not registered because "\
             "it is not supported in Abaqus version " + str(uti.getVersion())
elif verAbaqus == 6.6 or verAbaqus == 6.5:
    status = 'OK'
else:
    status = _abqPluginUtils.checkCompatibility('STL', version)
if status:
    toolset=getAFXApp().getAFXMainWindow().getPluginToolset()
    toolset.registerGuiMenuButton(
        object=StlForm(toolset),
        buttonText='NVH|Acoustics Toolset|STL...',
        messageId=AFXMode.ID_ACTIVATE,
        icon=None,
        kernelInitString='import sTL.stlKernel; stlKernel=sTL.stlKernel.StlKernel()',
        applicableModules=["Visualization",],
        version=version,
        author='Dassault Systemes',
        description='This plug-in allows to create sound transmission loss (STL) output in acoustics analyses.',
        helpUrl='http://abaqus.custhelp.com/cgi-bin/abaqus.cfg/php/enduser/std_adp.php?p_faqid=3439',
        )
else:
    getAFXApp().getAFXMainWindow().writeToMessageArea(status)
    getAFXApp().beep()