from abaqusGui import *
import _abqPluginUtils, os

# Use package syntax to avoid possible name clashes with other plugins
#
from abq_Vcct.interfacePropertyManForm import InterfacePropertyManForm
from abq_Vcct.interfaceManForm import InterfaceManForm
from abq_Vcct.contactClearanceManForm import ContactClearanceManForm
from abq_Vcct.vcctConstants import pgVersion

# Register commands
#
vcctVersion = pgVersion # remember to update on vcctConstants
status = _abqPluginUtils.checkCompatibility('VCCT', vcctVersion)
if status:
    toolset=getAFXApp().getAFXMainWindow().getPluginToolset()
    description = 'VCCT'
    author = 'Dassault Systemes Simulia Corp.'

    # Need to import vcctKernelInit through the kernel
    #
    thisPath = os.path.abspath(__file__)
    thisDir = os.path.dirname(thisPath)
    sendCommand('import sys')
    sendCommand("sys.path.append(r'%s')" % thisDir)
    sendCommand("sys.path.append(r'%s')" % os.path.join(thisDir,'abq_Vcct'))
    kernelInitString='import vcctKernelInit\nimport vcctInputFileWriter\nimport vcctInputFileReader'
    sendCommand(kernelInitString)

    kernelInitString = 'from abq_Vcct.vcctConstants import *'
    toolset.registerGuiMenuButton(
        object=InterfaceManForm(toolset),
        buttonText='VCCT|Interface Manager...',
        messageId=AFXMode.ID_ACTIVATE,
        icon=None,
        kernelInitString=kernelInitString,
        applicableModules=['Interaction',],
        version=vcctVersion,
        author=author,
        description=description,
        helpUrl='http://abaqus.custhelp.com/cgi-bin/abaqus.cfg'
                '/php/enduser/std_adp.php?p_faqid=3235'
        )        
    toolset.registerGuiMenuButton(
        object=InterfacePropertyManForm(toolset),
        buttonText='VCCT|Interface Property Manager...',
        messageId=AFXMode.ID_ACTIVATE,
        icon=None,
        kernelInitString=kernelInitString,
        applicableModules=['Interaction',],
        version=vcctVersion,
        author=author,
        description=description,
        helpUrl='http://abaqus.custhelp.com/cgi-bin/abaqus.cfg'
                '/php/enduser/std_adp.php?p_faqid=3235'
        )
    toolset.registerGuiMenuButton(
        object=ContactClearanceManForm(toolset),
        buttonText='VCCT|Contact Clearance Manager...',
        messageId=AFXMode.ID_ACTIVATE,
        icon=None,
        kernelInitString=kernelInitString,
        applicableModules=['Interaction',],
        version=vcctVersion,
        author=author,
        description=description,
        helpUrl='http://abaqus.custhelp.com/cgi-bin/abaqus.cfg'
                '/php/enduser/std_adp.php?p_faqid=3235'
        )
else:
    getAFXApp().getAFXMainWindow().writeToMessageArea(status)
    getAFXApp().beep()