"XY Data XML derived class"

from abaqus import *

import os.path

execfile(os.path.join(os.path.dirname(os.path.dirname(__file__)), "commonExec.py")) # Initializes the importFile function
(abqXML, xmlUtils) = importFile(__file__, ("caeXML.kernel.abqXML", "caeXML.kernel.xmlUtils")) # Allows base-package directory to be changed
(settingsModule, ) = importFile(__file__, ("caeXML.xyData.xydataSettings", )) # Settings Module

################################################################################

def readXYData(rootEle, funcList, baseObj, parameterDict):
    "Function to load object of the type XY Data"
    objType = xmlUtils.getObjectTypeFromXmlElement(rootEle)
    if objType != 'xyData':
        return (None, None, False)
    (objDict, tag, success) = xmlUtils.readObjectValueAsDictionary(rootEle, funcList, baseObj, parameterDict)
    if not success:
        return (None, None, False)
    paramArr = parameterDict['curParams']
    if paramArr:
        if type(paramArr) == type(()):
            if len(paramArr) == 1:
                if type(paramArr[0]) == type(""):
                    objDict['name'] = paramArr[0]
    constrStr = "session.XYData("
    for key in objDict.keys():
        if key == "description" or key == "fileName" or key == "sourceType":
            continue
        constrStr = constrStr + key + " = objDict[" + key.__repr__() + "], "
    constrStr = constrStr[:-2] + ")"
    retVal = eval(xmlUtils.upgradeText(constrStr, parameterDict))
    return (retVal, tag, True)

################################################################################

class XYDataXML(abqXML.AbqXML):
    "This class specializes the abqXML class for session xyData export"

    ################################################################################

    def __init__(self):
        self.m_settingsModule = settingsModule
        abqXML.AbqXML.__init__(self)
        self.m_str = ["session.xyDataObjects[]"]
        self.m_objLoadFuncList = [readXYData] + self.m_objLoadFuncList
        self.m_readObjectsCreatedFromLibrary = {'QuantityType':'xyPlot'}

    ################################################################################
    
    def saveInfo(self):
        "This function saves the xyData objects into xml doc"
        return self.parseAndAttachMultiple()

    ################################################################################

    def loadInfo(self, saveEle):
        "This function loads xyData objects from xmldoc"

        results = self.detachAndReturn(self.m_str[0], [session])
        return self.numResults(results)

################################################################################
