from abaqus import *
from abaqusConstants import *
import os
from caeModules import connectorBehavior

def createLGSModel(modelName,
    pitch_deg, initial_vel_y, initial_vel_z, gravity_acc, CG_x, CG_y, CG_z, time_constant,
    delay_time_constant, time_step, end_time, field_output_numbers,
    RP_NLG1_x, RP_NLG1_y, RP_NLG1_z, RP_NLG2_x, RP_NLG2_y, RP_NLG2_z,
    RP_NLG3_x, RP_NLG3_y, RP_NLG3_z, RP_NLG4_x, RP_NLG4_y, RP_NLG4_z,
    alpha_nose, SB_NLG, MF_NLG, Piston_NLG, WA_NLG, Tire_NLG,
    SA_NLG_Stroke, nosetire_damping_coeff, spring_nose_filename, damper_nose_filename, tire_nose_filename,
    RP_MLG1_x, RP_MLG1_y, RP_MLG1_z, RP_MLG2_x, RP_MLG2_y, RP_MLG2_z,
    RP_MLG3_x, RP_MLG3_y, RP_MLG3_z, 
    alpha_mf, alpha_shock, SB_MLG, MF_MLG, TA_MLG, SA_MLG, WA_MLG, Tire_MLG,
    Offset_SA_MLG, Offset_MF_MLG, SA_MLG_Stroke, 
    maintire_damping_coeff, spring_main_filename, damper_main_filename, tire_main_filename,
    airplane_mass, airplane_inertia_x, airplane_inertia_y, airplane_inertia_z,
    nose_FB_m, nose_MF_m, nose_piston_m, nose_TP_m, nose_TB_m, nose_WA_m, nose_Wheel_m,
    main_FB_m, main_MF_m, main_TA_m, main_SAT_m, main_SAB_m, main_WA_m, main_Wheel_m,
    nose_FB_m_x, nose_FB_m_y, nose_FB_m_z, nose_MF_m_x, nose_MF_m_y, nose_MF_m_z,
    nose_piston_m_x, nose_piston_m_y, nose_piston_m_z, nose_TP_m_x, nose_TP_m_y, nose_TP_m_z,
    nose_TB_m_x, nose_TB_m_y, nose_TB_m_z, nose_WA_m_x, nose_WA_m_y, nose_WA_m_z,
    nose_Wheel_m_x, nose_Wheel_m_y, nose_Wheel_m_z,
    main_FB_m_x, main_FB_m_y, main_FB_m_z, main_MF_m_x, main_MF_m_y, main_MF_m_z,
    main_TA_m_x, main_TA_m_y, main_TA_m_z, main_SAT_m_x, main_SAT_m_y, main_SAT_m_z,
    main_SAB_m_x, main_SAB_m_y, main_SAB_m_z, main_WA_m_x, main_WA_m_y, main_WA_m_z,
    main_Wheel_m_x, main_Wheel_m_y, main_Wheel_m_z,
    vis=True):

    ###################################################################
    ###################################################################

    dirrr = os.path.expanduser('~')
    #:  
    if vis:
    
        fileName = 'Airplane.inp'
        if not os.path.isfile(fileName):
            
            raise ValueError, 'Unable to find "%s" in current directory.' % fileName
            return
        else:
            if mdb.models.has_key(modelName):
                raise ValueError, 'Model "%s" already exists.' % modelName
                return
            mdb.ModelFromInputFile(name=modelName, inputFileName=fileName)
            model = mdb.models[modelName]
    else:
        return
            
    #: The model "Landing_Gear" has been created.
    #: The part "PART-1" has been imported from the input file.
    #: The model "Landing_Gear" has been imported from an input file. 
    #: Please scroll up to check for error and warning messages.

    a = model.rootAssembly
    p1 = model.parts['PART-1']
    vpName = session.currentViewportName
    vp = session.viewports[vpName]
    vp.setValues(displayedObject=p1)
    model.parts.changeKey(fromName='PART-1', toName='AIRPLANE')
    #del a.features['PART-1-1']

    ################################################################### Variable Set-up & Auto-Calculation
    #******* Positive Pitch Angle(degree) = Negative X-axis Rotation
    #pitch_deg = 10.
    #******* Positive Pitch Angle(rad) = Negative X-axis Rotation
    pitch = -pitch_deg*3.14159/180. 
    #******* Initial Velocity: Vertical Direction (Positive: Upward, Negative: Downward) 
    #initial_vel_y = -2.44e3
    #******* Initial Velocity: Longitudinal Direction (Positive: Forward, Negative: Backward) 
    #initial_vel_z =  2.00e4 
    #******* Initial Position between Tire Bottom and ground 
    #time_constant = 10.
    #delay_time_constant = 0.4
    Dist_to_ground = -initial_vel_y/time_constant
    #******* Gravity Acceleration 
    #gravity_acc =    9800.
    #******* Start Time for removing lift
    start_time = 1./time_constant
    #******* Time Step 
    #time_step = 2.e-5
    #******* End Time
    #end_time = 1.0
    #******* Lift Force Removal Time Set-up 
    Lift_delay_time = start_time + delay_time_constant 
    #******* Field Output Interval Numbers
    #field_output_numbers = 50
    #******* Aircraft Center of Gravity Coordinate 
    #CG_x = 0.
    #CG_y = -8.3
    #CG_z = -11.233e3
    #**     Calculated Coordinates
    CGb_x = CG_x + 1.
    CGb_y = CG_y
    CGb_z = CG_z
    #*******************    Nose Gear
    #******* Reference Attachement Point NLG: RP-NLG1
    #RP_NLG1_x =  222.75
    #RP_NLG1_y = -756.
    #RP_NLG1_z = -2.552e3
    #******* Reference Attachement Point NLG: RP-NLG2 (Assumed it is symmetric to RP-NLG1)
    #RP_NLG2_x = -RP_NLG1_x 
    #RP_NLG2_y =  RP_NLG1_y 
    #RP_NLG2_z =  RP_NLG1_z 
    #******* Reference Attachement Point NLG: RP-NLG3
    #RP_NLG3_x =  230.5
    #RP_NLG3_y = -1.115e3
    #RP_NLG3_z = -3.145e3
    #******* Reference Attachement Point NLG: RP-NLG4 (Assumed it is symmetric to RP-NLG3)
    #RP_NLG4_x = -RP_NLG3_x 
    #RP_NLG4_y =  RP_NLG3_y 
    #RP_NLG4_z =  RP_NLG3_z 
    #******* Alpha_nose Angle Parameter
    #alpha_nose = 7.5
    #******* SB_NLG Parameter
    #SB_NLG = 600. 
    #******* MF_NLG Parameter
    #MF_NLG = 690. 
    #******* Piston_NLG Parameter
    #Piston_NLG = 550
    #******* WA_NLG Parameter
    #WA_NLG = 240
    #******* Tire_NLG Parameter
    #Tire_NLG = 212 
    #******* SA_NLG Stroke Parameter
    #SA_NLG_Stroke = 300.
    #******* NLG TIRE Damping Parameter
    #nosetire_damping_coeff = 10.
    #**     Calculated Coordinates in NLG
    RP_NLGp_C1_x = RP_NLG1_x
    RP_NLGp_C1_y = RP_NLG1_y
    RP_NLGp_C1_z = RP_NLG1_z
    RP_NLGp_C2_x = RP_NLG2_x
    RP_NLGp_C2_y = RP_NLG2_y
    RP_NLGp_C2_z = RP_NLG2_z
    RP_NLGp_C5_x = RP_NLG3_x
    RP_NLGp_C5_y = RP_NLG3_y
    RP_NLGp_C5_z = RP_NLG3_z
    RP_NLGp_C6_x = RP_NLG4_x
    RP_NLGp_C6_y = RP_NLG4_y
    RP_NLGp_C6_z = RP_NLG4_z
    RP_NLGp_C20_x = 0.
    RP_NLGp_C20_y = RP_NLG3_y
    RP_NLGp_C20_z = RP_NLG3_z
    RP_NLGp_C4_x = 0.
    RP_NLGp_C4_y = RP_NLG3_y - SB_NLG*cos(alpha_nose*3.14159/180.) 
    RP_NLGp_C4_z = RP_NLG3_z + SB_NLG*sin(alpha_nose*3.14159/180.) 
    RP_NLGp_C10_x = 0.
    RP_NLGp_C10_y = RP_NLG3_y - MF_NLG*cos(alpha_nose*3.14159/180.) 
    RP_NLGp_C10_z = RP_NLG3_z + MF_NLG*sin(alpha_nose*3.14159/180.) 
    RP_NLGp_C7_x = 0.
    RP_NLGp_C7_y = RP_NLG3_y - 0.9*SB_NLG*cos(alpha_nose*3.14159/180.) 
    RP_NLGp_C7_z = RP_NLG3_z + 0.7*SB_NLG*sin(alpha_nose*3.14159/180.) 
    RP_NLGp_C8_x = 0.
    RP_NLGp_C8_y = RP_NLG3_y - MF_NLG*cos(alpha_nose*3.14159/180.) 
    RP_NLGp_C8_z = RP_NLG3_z - 4.0*MF_NLG*sin(alpha_nose*3.14159/180.) 
    RP_NLGp_C9_x = 0.
    RP_NLGp_C9_y = RP_NLG3_y - 1.5*MF_NLG*cos(alpha_nose*3.14159/180.) 
    RP_NLGp_C9_z = RP_NLG3_z + 0.7*MF_NLG*sin(alpha_nose*3.14159/180.) 
    RP_NLGp_C11_x = 0.
    RP_NLGp_C11_y = RP_NLG3_y - (MF_NLG+Piston_NLG)*cos(alpha_nose*3.14159/180.) 
    RP_NLGp_C11_z = RP_NLG3_z + (MF_NLG+Piston_NLG)*sin(alpha_nose*3.14159/180.) 
    RP_NLGp_C12_x = RP_NLGp_C11_x - WA_NLG/2. 
    RP_NLGp_C12_y = RP_NLGp_C11_y 
    RP_NLGp_C12_z = RP_NLGp_C11_z 
    RP_NLGp_C13_x = RP_NLGp_C11_x + WA_NLG/2. 
    RP_NLGp_C13_y = RP_NLGp_C11_y 
    RP_NLGp_C13_z = RP_NLGp_C11_z 
    RP_NLGp_C14_x = RP_NLGp_C11_x - WA_NLG/2. 
    RP_NLGp_C14_y = RP_NLGp_C11_y - Tire_NLG 
    RP_NLGp_C14_z = RP_NLGp_C11_z 
    RP_NLGp_C15_x = RP_NLGp_C11_x + WA_NLG/2. 
    RP_NLGp_C15_y = RP_NLGp_C11_y - Tire_NLG 
    RP_NLGp_C15_z = RP_NLGp_C11_z 
    RP_NLGp_C1000_x = (RP_NLGp_C1_x + RP_NLGp_C2_x + RP_NLGp_C4_x) / 3. 
    RP_NLGp_C1000_y = (RP_NLGp_C1_y + RP_NLGp_C2_y + RP_NLGp_C4_y) / 3. 
    RP_NLGp_C1000_z = (RP_NLGp_C1_z + RP_NLGp_C2_z + RP_NLGp_C4_z) / 3. 
    RP_NLGp_C1200_x = (RP_NLGp_C10_x + RP_NLGp_C20_x) / 2. 
    RP_NLGp_C1200_y = (RP_NLGp_C10_y + RP_NLGp_C20_y) / 2. 
    RP_NLGp_C1200_z = (RP_NLGp_C10_z + RP_NLGp_C20_z) / 2. 
    RP_NLGp_C1300_x = (RP_NLGp_C10_x + RP_NLGp_C11_x) / 2. 
    RP_NLGp_C1300_y = (RP_NLGp_C10_y + RP_NLGp_C11_y) / 2. 
    RP_NLGp_C1300_z = (RP_NLGp_C10_z + RP_NLGp_C11_z) / 2. 
    RP_NLGp_C1400_x = (RP_NLGp_C7_x + RP_NLGp_C8_x) / 2. 
    RP_NLGp_C1400_y = (RP_NLGp_C7_y + RP_NLGp_C8_y) / 2. 
    RP_NLGp_C1400_z = (RP_NLGp_C7_z + RP_NLGp_C8_z) / 2. 
    RP_NLGp_C1500_x = (RP_NLGp_C9_x + RP_NLGp_C8_x) / 2. 
    RP_NLGp_C1500_y = (RP_NLGp_C9_y + RP_NLGp_C8_y) / 2. 
    RP_NLGp_C1500_z = (RP_NLGp_C9_z + RP_NLGp_C8_z) / 2. 
    #********************
    #**     Calculated Pitched Coordinates in NLG
    RP_NLG_C1_x = RP_NLGp_C1_x
    RP_NLG_C1_y = (RP_NLGp_C1_y-CG_y)*cos(pitch)-(RP_NLGp_C1_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C1_z = (RP_NLGp_C1_y-CG_y)*sin(pitch)+(RP_NLGp_C1_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C2_x = RP_NLGp_C2_x
    RP_NLG_C2_y = (RP_NLGp_C2_y-CG_y)*cos(pitch)-(RP_NLGp_C2_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C2_z = (RP_NLGp_C2_y-CG_y)*sin(pitch)+(RP_NLGp_C2_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C5_x = RP_NLGp_C5_x
    RP_NLG_C5_y = (RP_NLGp_C5_y-CG_y)*cos(pitch)-(RP_NLGp_C5_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C5_z = (RP_NLGp_C5_y-CG_y)*sin(pitch)+(RP_NLGp_C5_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C6_x = RP_NLGp_C6_x
    RP_NLG_C6_y = (RP_NLGp_C6_y-CG_y)*cos(pitch)-(RP_NLGp_C6_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C6_z = (RP_NLGp_C6_y-CG_y)*sin(pitch)+(RP_NLGp_C6_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C20_x = RP_NLGp_C20_x
    RP_NLG_C20_y = (RP_NLGp_C20_y-CG_y)*cos(pitch)-(RP_NLGp_C20_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C20_z = (RP_NLGp_C20_y-CG_y)*sin(pitch)+(RP_NLGp_C20_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C4_x = RP_NLGp_C4_x
    RP_NLG_C4_y = (RP_NLGp_C4_y-CG_y)*cos(pitch)-(RP_NLGp_C4_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C4_z = (RP_NLGp_C4_y-CG_y)*sin(pitch)+(RP_NLGp_C4_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C10_x = RP_NLGp_C10_x
    RP_NLG_C10_y = (RP_NLGp_C10_y-CG_y)*cos(pitch)-(RP_NLGp_C10_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C10_z = (RP_NLGp_C10_y-CG_y)*sin(pitch)+(RP_NLGp_C10_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C7_x = RP_NLGp_C7_x
    RP_NLG_C7_y = (RP_NLGp_C7_y-CG_y)*cos(pitch)-(RP_NLGp_C7_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C7_z = (RP_NLGp_C7_y-CG_y)*sin(pitch)+(RP_NLGp_C7_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C8_x = RP_NLGp_C8_x
    RP_NLG_C8_y = (RP_NLGp_C8_y-CG_y)*cos(pitch)-(RP_NLGp_C8_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C8_z = (RP_NLGp_C8_y-CG_y)*sin(pitch)+(RP_NLGp_C8_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C9_x = RP_NLGp_C9_x
    RP_NLG_C9_y = (RP_NLGp_C9_y-CG_y)*cos(pitch)-(RP_NLGp_C9_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C9_z = (RP_NLGp_C9_y-CG_y)*sin(pitch)+(RP_NLGp_C9_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C11_x = RP_NLGp_C11_x
    RP_NLG_C11_y = (RP_NLGp_C11_y-CG_y)*cos(pitch)-(RP_NLGp_C11_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C11_z = (RP_NLGp_C11_y-CG_y)*sin(pitch)+(RP_NLGp_C11_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C12_x = RP_NLGp_C12_x
    RP_NLG_C12_y = (RP_NLGp_C12_y-CG_y)*cos(pitch)-(RP_NLGp_C12_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C12_z = (RP_NLGp_C12_y-CG_y)*sin(pitch)+(RP_NLGp_C12_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C13_x = RP_NLGp_C13_x
    RP_NLG_C13_y = (RP_NLGp_C13_y-CG_y)*cos(pitch)-(RP_NLGp_C13_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C13_z = (RP_NLGp_C13_y-CG_y)*sin(pitch)+(RP_NLGp_C13_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C14_x = RP_NLG_C11_x - WA_NLG/2. 
    RP_NLG_C14_y = RP_NLG_C11_y - Tire_NLG 
    RP_NLG_C14_z = RP_NLG_C11_z 
    RP_NLG_C15_x = RP_NLG_C11_x + WA_NLG/2. 
    RP_NLG_C15_y = RP_NLG_C11_y - Tire_NLG 
    RP_NLG_C15_z = RP_NLG_C11_z 
    RP_NLG_C1000_x = RP_NLGp_C1000_x
    RP_NLG_C1000_y = (RP_NLGp_C1000_y-CG_y)*cos(pitch)-(RP_NLGp_C1000_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C1000_z = (RP_NLGp_C1000_y-CG_y)*sin(pitch)+(RP_NLGp_C1000_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C1200_x = RP_NLGp_C1200_x
    RP_NLG_C1200_y = (RP_NLGp_C1200_y-CG_y)*cos(pitch)-(RP_NLGp_C1200_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C1200_z = (RP_NLGp_C1200_y-CG_y)*sin(pitch)+(RP_NLGp_C1200_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C1300_x = RP_NLGp_C1300_x
    RP_NLG_C1300_y = (RP_NLGp_C1300_y-CG_y)*cos(pitch)-(RP_NLGp_C1300_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C1300_z = (RP_NLGp_C1300_y-CG_y)*sin(pitch)+(RP_NLGp_C1300_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C1400_x = RP_NLGp_C1400_x
    RP_NLG_C1400_y = (RP_NLGp_C1400_y-CG_y)*cos(pitch)-(RP_NLGp_C1400_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C1400_z = (RP_NLGp_C1400_y-CG_y)*sin(pitch)+(RP_NLGp_C1400_z-CG_z)*cos(pitch) + CG_z
    RP_NLG_C1500_x = RP_NLGp_C1500_x
    RP_NLG_C1500_y = (RP_NLGp_C1500_y-CG_y)*cos(pitch)-(RP_NLGp_C1500_z-CG_z)*sin(pitch) + CG_y
    RP_NLG_C1500_z = (RP_NLGp_C1500_y-CG_y)*sin(pitch)+(RP_NLGp_C1500_z-CG_z)*cos(pitch) + CG_z
    #***********************
    #**** Relative position of Absorber in NLG
    Rel_Pos_Abs_min_NLG = -sqrt((RP_NLG_C20_x-RP_NLG_C11_x)**2+(RP_NLG_C20_y-RP_NLG_C11_y)**2+(RP_NLG_C20_z-RP_NLG_C11_z)**2) 
    Rel_Pos_Abs_max_NLG = Rel_Pos_Abs_min_NLG + 300.
    #*******************    Main Gear
    #******* Reference Attachement Point MLG: RP-NLG2
    RP_MLG_left2_x =  RP_MLG1_x   #2.169e3
    RP_MLG_left2_y =  RP_MLG1_y   #-1.074e3
    RP_MLG_left2_z =  RP_MLG1_z   #-11.567e3
    #******* Reference Attachement Point MLG: RP-NLG1
    RP_MLG_left1_x =  RP_MLG2_x   #2.233e3
    RP_MLG_left1_y =  RP_MLG2_y   #-1.136e3
    RP_MLG_left1_z =  RP_MLG2_z   #-11.065e3
    #******* Reference Attachement Point MLG: RP-NLG3
    RP_MLG_left3_x =  RP_MLG3_x   #1.429e3
    RP_MLG_left3_y =  RP_MLG3_y   #-1.136e3 
    RP_MLG_left3_z =  RP_MLG3_z   #-10.806e3
    #******* alpha_mf parameter
    #alpha_mf = 4. 
    #******* alpha_shock parameter
    #alpha_shock = 9. 
    #******* SB_MLG parameter
    #SB_MLG = 880 
    #******* MF_MLG parameter
    #MF_MLG = 950 
    #******* TA_MLG parameter
    #TA_MLG = 460 
    #******* WA_MLG parameter
    #WA_MLG = 480 
    #******* Tire_MLG parameter
    #Tire_MLG = 370 
    #******* SA_MLG parameter
    #SA_MLG = 1330 
    #******* Offset_SA_MLG parameter
    #Offset_SA_MLG = 146 
    #******* Offset_MF_MLG parameter
    #Offset_MF_MLG = 270 
    #******* SA_MLG Stroke Parameter
    #SA_MLG_Stroke = 360.
    #******* MLG TIRE Damping Parameter
    #maintire_damping_coeff = 10.
    #** Calculated Length between RP-MLG1 and RP-MLG2
    v_dist = sqrt((RP_MLG_left2_x - RP_MLG_left1_x)**2+(RP_MLG_left2_y - RP_MLG_left1_y)**2+(RP_MLG_left2_z - RP_MLG_left1_z)**2)
    length_Offset_x = (RP_MLG_left1_x - RP_MLG_left2_x)
    length_Offset_y = (RP_MLG_left1_y - RP_MLG_left2_y)
    length_Offset_z = (RP_MLG_left1_z - RP_MLG_left2_z)
    #** Calculated Coordinates in MLG Left
    RP_MLGp_L1_x = RP_MLG_left3_x
    RP_MLGp_L1_y = RP_MLG_left3_y
    RP_MLGp_L1_z = RP_MLG_left3_z
    RP_MLGp_L5_x = RP_MLG_left1_x
    RP_MLGp_L5_y = RP_MLG_left1_y
    RP_MLGp_L5_z = RP_MLG_left1_z
    RP_MLGp_L6_x = RP_MLG_left2_x
    RP_MLGp_L6_y = RP_MLG_left2_y
    RP_MLGp_L6_z = RP_MLG_left2_z
    RP_MLGp_L3_x = RP_MLG_left2_x + length_Offset_x*(Offset_MF_MLG/v_dist)
    RP_MLGp_L3_y = RP_MLG_left2_y + length_Offset_y*(Offset_MF_MLG/v_dist)
    RP_MLGp_L3_z = RP_MLG_left2_z + length_Offset_z*(Offset_MF_MLG/v_dist) 
    RP_MLGp_L8_x = RP_MLG_left2_x + length_Offset_x*(Offset_SA_MLG/v_dist)
    RP_MLGp_L8_y = RP_MLG_left2_y + length_Offset_y*(Offset_SA_MLG/v_dist)
    RP_MLGp_L8_z = RP_MLG_left2_z + length_Offset_z*(Offset_SA_MLG/v_dist)
    RP_MLGp_L4_x = RP_MLGp_L3_x
    RP_MLGp_L4_y = RP_MLGp_L3_y - SB_MLG*cos(alpha_mf*3.14159/180)
    RP_MLGp_L4_z = RP_MLGp_L3_z + SB_MLG*sin(alpha_mf*3.14159/180)
    RP_MLGp_L7_x = RP_MLGp_L3_x
    RP_MLGp_L7_y = RP_MLGp_L3_y - MF_MLG*cos(alpha_mf*3.14159/180)
    RP_MLGp_L7_z = RP_MLGp_L3_z + MF_MLG*sin(alpha_mf*3.14159/180)
    RP_MLGp_L9_x = RP_MLGp_L8_x
    RP_MLGp_L9_y = RP_MLGp_L8_y - 0.5*SA_MLG*cos(alpha_shock*3.14159/180)
    RP_MLGp_L9_z = RP_MLGp_L8_z - 0.5*SA_MLG*sin(alpha_shock*3.14159/180)
    RP_MLGp_L11_x = RP_MLGp_L8_x
    RP_MLGp_L11_y = RP_MLGp_L8_y - SA_MLG*cos(alpha_shock*3.14159/180)
    RP_MLGp_L11_z = RP_MLGp_L8_z - SA_MLG*sin(alpha_shock*3.14159/180)
    RP_MLGp_L12_x = RP_MLGp_L11_x - WA_MLG/2.
    RP_MLGp_L12_y = RP_MLGp_L11_y  
    RP_MLGp_L12_z = RP_MLGp_L11_z  
    RP_MLGp_L13_x = RP_MLGp_L11_x + WA_MLG/2.
    RP_MLGp_L13_y = RP_MLGp_L11_y  
    RP_MLGp_L13_z = RP_MLGp_L11_z  
    RP_MLGp_L14_x = RP_MLGp_L11_x - WA_MLG/2.
    RP_MLGp_L14_y = RP_MLGp_L11_y - Tire_MLG
    RP_MLGp_L14_z = RP_MLGp_L11_z  
    RP_MLGp_L15_x = RP_MLGp_L11_x + WA_MLG/2.
    RP_MLGp_L15_y = RP_MLGp_L11_y - Tire_MLG
    RP_MLGp_L15_z = RP_MLGp_L11_z 
    RP_MLGp_L1000_x = (RP_MLGp_L1_x + RP_MLGp_L4_x) / 2. 
    RP_MLGp_L1000_y = (RP_MLGp_L1_y + RP_MLGp_L4_y) / 2. 
    RP_MLGp_L1000_z = (RP_MLGp_L1_z + RP_MLGp_L4_z) / 2. 
    RP_MLGp_L1200_x = (RP_MLGp_L8_x + RP_MLGp_L9_x) / 2. 
    RP_MLGp_L1200_y = (RP_MLGp_L8_y + RP_MLGp_L9_y) / 2. 
    RP_MLGp_L1200_z = (RP_MLGp_L8_z + RP_MLGp_L9_z) / 2. 
    RP_MLGp_L1300_x = (RP_MLGp_L11_x + RP_MLGp_L9_x) / 2. 
    RP_MLGp_L1300_y = (RP_MLGp_L11_y + RP_MLGp_L9_y) / 2. 
    RP_MLGp_L1300_z = (RP_MLGp_L11_z + RP_MLGp_L9_z) / 2. 
    RP_MLGp_L1400_x = (RP_MLGp_L3_x + RP_MLGp_L7_x) / 2. 
    RP_MLGp_L1400_y = (RP_MLGp_L3_y + RP_MLGp_L7_y) / 2. 
    RP_MLGp_L1400_z = (RP_MLGp_L3_z + RP_MLGp_L7_z) / 2. 
    RP_MLGp_L1500_x = (RP_MLGp_L11_x + RP_MLGp_L7_x) / 2. 
    RP_MLGp_L1500_y = (RP_MLGp_L11_y + RP_MLGp_L7_y) / 2. 
    RP_MLGp_L1500_z = (RP_MLGp_L11_z + RP_MLGp_L7_z) / 2. 
    #***********************
    #** Calculated Pitched Coordinates in MLG Left
    RP_MLG_L1_x =  RP_MLGp_L1_x
    RP_MLG_L1_y = (RP_MLGp_L1_y-CG_y)*cos(pitch) - (RP_MLGp_L1_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L1_z = (RP_MLGp_L1_y-CG_y)*sin(pitch) + (RP_MLGp_L1_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L5_x =  RP_MLGp_L5_x
    RP_MLG_L5_y = (RP_MLGp_L5_y-CG_y)*cos(pitch) - (RP_MLGp_L5_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L5_z = (RP_MLGp_L5_y-CG_y)*sin(pitch) + (RP_MLGp_L5_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L6_x =  RP_MLGp_L6_x
    RP_MLG_L6_y = (RP_MLGp_L6_y-CG_y)*cos(pitch) - (RP_MLGp_L6_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L6_z = (RP_MLGp_L6_y-CG_y)*sin(pitch) + (RP_MLGp_L6_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L3_x =  RP_MLGp_L3_x
    RP_MLG_L3_y = (RP_MLGp_L3_y-CG_y)*cos(pitch) - (RP_MLGp_L3_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L3_z = (RP_MLGp_L3_y-CG_y)*sin(pitch) + (RP_MLGp_L3_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L8_x =  RP_MLGp_L8_x
    RP_MLG_L8_y = (RP_MLGp_L8_y-CG_y)*cos(pitch) - (RP_MLGp_L8_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L8_z = (RP_MLGp_L8_y-CG_y)*sin(pitch) + (RP_MLGp_L8_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L4_x =  RP_MLGp_L4_x
    RP_MLG_L4_y = (RP_MLGp_L4_y-CG_y)*cos(pitch) - (RP_MLGp_L4_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L4_z = (RP_MLGp_L4_y-CG_y)*sin(pitch) + (RP_MLGp_L4_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L7_x =  RP_MLGp_L7_x
    RP_MLG_L7_y = (RP_MLGp_L7_y-CG_y)*cos(pitch) - (RP_MLGp_L7_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L7_z = (RP_MLGp_L7_y-CG_y)*sin(pitch) + (RP_MLGp_L7_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L9_x =  RP_MLGp_L9_x
    RP_MLG_L9_y = (RP_MLGp_L9_y-CG_y)*cos(pitch) - (RP_MLGp_L9_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L9_z = (RP_MLGp_L9_y-CG_y)*sin(pitch) + (RP_MLGp_L9_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L11_x =  RP_MLGp_L11_x
    RP_MLG_L11_y = (RP_MLGp_L11_y-CG_y)*cos(pitch) - (RP_MLGp_L11_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L11_z = (RP_MLGp_L11_y-CG_y)*sin(pitch) + (RP_MLGp_L11_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L12_x =  RP_MLGp_L12_x
    RP_MLG_L12_y = (RP_MLGp_L12_y-CG_y)*cos(pitch) - (RP_MLGp_L12_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L12_z = (RP_MLGp_L12_y-CG_y)*sin(pitch) + (RP_MLGp_L12_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L13_x =  RP_MLGp_L13_x
    RP_MLG_L13_y = (RP_MLGp_L13_y-CG_y)*cos(pitch) - (RP_MLGp_L13_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L13_z = (RP_MLGp_L13_y-CG_y)*sin(pitch) + (RP_MLGp_L13_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L14_x = RP_MLG_L11_x - WA_MLG/2.
    RP_MLG_L14_y = RP_MLG_L11_y - Tire_MLG
    RP_MLG_L14_z = RP_MLG_L11_z  
    RP_MLG_L15_x = RP_MLG_L11_x + WA_MLG/2.
    RP_MLG_L15_y = RP_MLG_L11_y - Tire_MLG
    RP_MLG_L15_z = RP_MLG_L11_z  
    RP_MLG_L1000_x =  RP_MLGp_L1000_x
    RP_MLG_L1000_y = (RP_MLGp_L1000_y-CG_y)*cos(pitch) - (RP_MLGp_L1000_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L1000_z = (RP_MLGp_L1000_y-CG_y)*sin(pitch) + (RP_MLGp_L1000_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L1200_x =  RP_MLGp_L1200_x
    RP_MLG_L1200_y = (RP_MLGp_L1200_y-CG_y)*cos(pitch) - (RP_MLGp_L1200_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L1200_z = (RP_MLGp_L1200_y-CG_y)*sin(pitch) + (RP_MLGp_L1200_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L1300_x =  RP_MLGp_L1300_x
    RP_MLG_L1300_y = (RP_MLGp_L1300_y-CG_y)*cos(pitch) - (RP_MLGp_L1300_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L1300_z = (RP_MLGp_L1300_y-CG_y)*sin(pitch) + (RP_MLGp_L1300_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L1400_x =  RP_MLGp_L1400_x
    RP_MLG_L1400_y = (RP_MLGp_L1400_y-CG_y)*cos(pitch) - (RP_MLGp_L1400_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L1400_z = (RP_MLGp_L1400_y-CG_y)*sin(pitch) + (RP_MLGp_L1400_z-CG_z)*cos(pitch) + CG_z
    RP_MLG_L1500_x =  RP_MLGp_L1500_x
    RP_MLG_L1500_y = (RP_MLGp_L1500_y-CG_y)*cos(pitch) - (RP_MLGp_L1500_z-CG_z)*sin(pitch) + CG_y
    RP_MLG_L1500_z = (RP_MLGp_L1500_y-CG_y)*sin(pitch) + (RP_MLGp_L1500_z-CG_z)*cos(pitch) + CG_z
    #****** Right Side of MLG (Assumed it is symmetric)
    RP_MLG_R1_x = -RP_MLG_L1_x
    RP_MLG_R5_x = -RP_MLG_L5_x
    RP_MLG_R6_x = -RP_MLG_L6_x
    RP_MLG_R3_x = -RP_MLG_L3_x
    RP_MLG_R8_x = -RP_MLG_L8_x
    RP_MLG_R4_x = -RP_MLG_L4_x
    RP_MLG_R7_x = -RP_MLG_L7_x
    RP_MLG_R9_x = -RP_MLG_L9_x
    RP_MLG_R11_x = -RP_MLG_L11_x
    RP_MLG_R12_x = -RP_MLG_L12_x 
    RP_MLG_R13_x = -RP_MLG_L13_x 
    RP_MLG_R14_x = -RP_MLG_L14_x 
    RP_MLG_R15_x = -RP_MLG_L15_x 
    RP_MLG_R1000_x = -RP_MLG_L1000_x 
    RP_MLG_R1200_x = -RP_MLG_L1200_x 
    RP_MLG_R1300_x = -RP_MLG_L1300_x 
    RP_MLG_R1400_x = -RP_MLG_L1400_x 
    RP_MLG_R1500_x = -RP_MLG_L1500_x 
    #****** Ground Display Coordinate 
    grounddisplay_x = RP_MLG_L15_x + 500.
    grounddisplay_z1 = RP_MLG_L15_z - 1000.
    grounddisplay_z2 = RP_MLG_L15_z + initial_vel_z*end_time + 9000.
    #************** Axis Coordinate Calculation for RP-MLG3 point joint 
    #************** Based on Left Main Gear
    #******************** Assumption: Axis is in the XZ plane.
    L1 = sqrt((RP_MLG_L1_x-RP_MLG_L4_x)**2+(RP_MLG_L1_y-RP_MLG_L4_y)**2+(RP_MLG_L1_z-RP_MLG_L4_z)**2)
    Rev4_xz_x=RP_MLG_L4_x
    Rev4_xz_y=RP_MLG_L1_y
    Rev4_xz_z=RP_MLG_L4_z
    omega = atan((RP_MLG_L1_x-Rev4_xz_x)/(-RP_MLG_L1_z+Rev4_xz_z))
    mu=(3.14159/2)-omega
    L2=L1*tan(30.*3.14159/180.)
    #**
    MLG_FB_Axis_L_x = RP_MLG_L1_x + L2*sin(mu) 
    MLG_FB_Axis_L_y = RP_MLG_L1_y 
    MLG_FB_Axis_L_z = RP_MLG_L1_z + L2*cos(mu)
    MLG_FB_Axis_R_x = -MLG_FB_Axis_L_x 
    #**** Relative position of Absorber in MLG
    Rel_Pos_Abs_min_MLG = -sqrt((RP_MLG_L8_x-RP_MLG_L11_x)**2+(RP_MLG_L8_y-RP_MLG_L11_y)**2+(RP_MLG_L8_z-RP_MLG_L11_z)**2) 
    Rel_Pos_Abs_max_MLG = Rel_Pos_Abs_min_MLG + 360.
    #****************************************
    #****** Ground Reference Coordinate 
    ground_x = 0.
    ground_y = RP_MLG_L11_y-Tire_MLG-Dist_to_ground
    ground_z = 0.
    dist2 = RP_NLG_C11_y - RP_MLG_L11_y
    #****************************************
    #**** Airplane Mass (Ton), landing gear mass = .65 ton and aircraft body = 22. ton
    #airplane_mass = 22.0
    #**** Airplane Inertia (Ton*mm^2)
    #airplane_inertia_x = 65.e6
    #airplane_inertia_y = 100.e6
    #airplane_inertia_z = 100.e6
    #**** Mass (Ton) 
    #nose_FB_m = 0.001
    #nose_MF_m = 0.003
    #nose_piston_m = 0.003
    #nose_TP_m = 0.001
    #nose_TB_m = 0.001
    #nose_WA_m = 0.001
    #nose_Wheel_m = 0.1
    #main_FB_m = 0.006
    #main_MF_m = 0.006
    #main_TA_m = 0.005
    #main_SAT_m = 0.001
    #main_SAB_m = 0.001
    #main_WA_m = 0.001
    #main_Wheel_m = 0.1
    #**** Rotary Inertia (Ton*mm^2)
    #nose_FB_m_x = 1.e4
    #nose_FB_m_y = 1.e4
    #nose_FB_m_z = 2.e3
    #nose_MF_m_x = 6.e2
    #nose_MF_m_y = 6.e2
    #nose_MF_m_z = 8.e1
    #nose_piston_m_x = 4.e1
    #nose_piston_m_y = 4.e1
    #nose_piston_m_z = 1.e-1
    #nose_TP_m_x = 1.e-1
    #nose_TP_m_y = 1.e-1
    #nose_TP_m_z = 1.e-3
    #nose_TB_m_x = 1.e-1
    #nose_TB_m_y = 1.e-1
    #nose_TB_m_z = 1.e-3
    #nose_WA_m_x = 2.e0
    #nose_WA_m_y = 2.e0
    #nose_WA_m_z = 2.e-2
    #nose_Wheel_m_x = 3.e3
    #nose_Wheel_m_y = 3.e3
    #nose_Wheel_m_z = 1.e3
    #main_FB_m_x = 1.e3
    #main_FB_m_y = 1.e3
    #main_FB_m_z = 2.e0
    #main_MF_m_x = 5.e3
    #main_MF_m_y = 5.e3
    #main_MF_m_z = 1.e2
    #main_TA_m_x = 7.e0
    #main_TA_m_y = 7.e0
    #main_TA_m_z = 4.e-2
    #main_SAT_m_x = 2.e2
    #main_SAT_m_y = 2.e2
    #main_SAT_m_z = 1.e0
    #main_SAB_m_x = 4.e1
    #main_SAB_m_y = 4.e1
    #main_SAB_m_z = 7.e-2
    #main_WA_m_x = 2.e1
    #main_WA_m_y = 2.e1
    #main_WA_m_z = 5.e-2
    #main_Wheel_m_x = 8.e4
    #main_Wheel_m_y = 8.e4
    #main_Wheel_m_z = 4.e4

    # Grab model objects
    #
    model = model
    a = model.rootAssembly
    a.DatumCsysByDefault(CARTESIAN)
    p = model.parts['AIRPLANE']
    
    a.Instance(name='AIRPLANE-1', part=p, dependent=ON)

    vp.view.setValues(nearPlane=56055.7, 
    farPlane=107644, width=27644.2, height=31703.3, cameraPosition=(50136, 
    41432.7, 38525.2), cameraUpVector=(-0.555088, 0.645758, -0.524285))

    a.translate(instanceList=('AIRPLANE-1', ), vector=(0.0, 0.0, 0.0))
    #: The instance AIRPLANE-1 was translated by 0., 0., 0. with respect to the assembly coordinate system

    a.rotate(instanceList=('AIRPLANE-1', ), axisPoint=(CG_x, CG_y, CG_z), 
    axisDirection=(1.0, 0.0, 0.0), angle=-pitch_deg)

    #: The instance AIRPLANE-1 was rotated by -10. degrees about the axis defined by the point 0., -8.3, -11.233E+03 and the vector 1., 0., 0.

    model 
    # List of parts to be created
    #
    partsNameList = ['GROUND', 'NLG_1000', 'NLG_1200', 'NLG_1300', 'NLG_1400',
                     'NLG_1500' , 'NLG_1600', 'NLG_1700','NLG_1800','MLG_L1000',
                     'MLG_L1400','MLG_L1500','MLG_L1200','MLG_L1300','MLG_L1600',
                     'MLG_L1700','MLG_L1800','MLG_R1000','MLG_R1400','MLG_R1500',
                     'MLG_R1200','MLG_R1300','MLG_R1600','MLG_R1700','MLG_R1800']

    coordsSeqList = [ ( (grounddisplay_x, ground_y, grounddisplay_z1),  
                        (-grounddisplay_x, ground_y, grounddisplay_z1),
                        (-grounddisplay_x, ground_y, grounddisplay_z2),
                        (grounddisplay_x, ground_y, grounddisplay_z2), ),

                      # NLG_1000 Display Body Construction
                      ( (RP_NLG_C1_x, RP_NLG_C1_y, RP_NLG_C1_z),
                        (RP_NLG_C2_x, RP_NLG_C2_y, RP_NLG_C2_z),
                        (RP_NLG_C4_x, RP_NLG_C4_y, RP_NLG_C4_z),       ),

                      # NLG_1200 Display Body Construction
                      ( (RP_NLG_C5_x, RP_NLG_C5_y, RP_NLG_C5_z), 
                        (RP_NLG_C6_x, RP_NLG_C6_y, RP_NLG_C6_z),
                        (RP_NLG_C20_x, RP_NLG_C20_y, RP_NLG_C20_z),
                        (RP_NLG_C10_x, RP_NLG_C10_y, RP_NLG_C10_z),    ),

                      # NLG_1300 Display Body Constructio
                      ( (RP_NLG_C10_x, RP_NLG_C10_y, RP_NLG_C10_z),
                        (RP_NLG_C11_x, RP_NLG_C11_y, RP_NLG_C11_z),    ),
                        
                      # NLG_1400 Display Body Construction
                      ( (RP_NLG_C7_x, RP_NLG_C7_y, RP_NLG_C7_z),
                        (RP_NLG_C8_x, RP_NLG_C8_y, RP_NLG_C8_z),       ),
                        
                      # NLG_1500 Display Body Construction
                      ( (RP_NLG_C8_x, RP_NLG_C8_y, RP_NLG_C8_z),
                        (RP_NLG_C9_x, RP_NLG_C9_y, RP_NLG_C9_z),       ),
                        
                      # NLG_1600 Display Body Construction
                      ( (RP_NLG_C12_x, RP_NLG_C12_y, RP_NLG_C12_z),
                        (RP_NLG_C13_x, RP_NLG_C13_y, RP_NLG_C13_z),    ),
                        
                      # NLG_1700 Display Body Construction
                      ( (RP_NLG_C12_x, RP_NLG_C12_y, RP_NLG_C12_z),
                        (RP_NLG_C14_x, RP_NLG_C14_y, RP_NLG_C14_z),    ),

                      # NLG_1800 Display Body Construction
                      ( (RP_NLG_C13_x, RP_NLG_C13_y, RP_NLG_C13_z),
                        (RP_NLG_C15_x, RP_NLG_C15_y, RP_NLG_C15_z),    ), 
       
                      # MLG_L1000 Display Body Construction
                      ( (RP_MLG_L1_x, RP_MLG_L1_y, RP_MLG_L1_z),
                        (RP_MLG_L4_x, RP_MLG_L4_y, RP_MLG_L4_z),       ),

                      # MLG_L1400 Display Body Construction
                      ( (RP_MLG_L3_x, RP_MLG_L3_y, RP_MLG_L3_z),
                        (RP_MLG_L7_x, RP_MLG_L7_y, RP_MLG_L7_z),
                        (RP_MLG_L5_x, RP_MLG_L5_y, RP_MLG_L5_z),
                        (RP_MLG_L6_x, RP_MLG_L6_y, RP_MLG_L6_z),       ),
       
                      # MLG_L1500 Display Body Construction 
                      ( (RP_MLG_L7_x, RP_MLG_L7_y, RP_MLG_L7_z),
                        (RP_MLG_L11_x, RP_MLG_L11_y, RP_MLG_L11_z),    ),
                
                      # MLG_L1200 Display Body Construction
                      ( (RP_MLG_L8_x, RP_MLG_L8_y, RP_MLG_L8_z),
                        (RP_MLG_L9_x, RP_MLG_L9_y, RP_MLG_L9_z),       ),

                      # MLG_L1300 Display Body Construction  
                      ( (RP_MLG_L9_x, RP_MLG_L9_y, RP_MLG_L9_z),
                        (RP_MLG_L11_x, RP_MLG_L11_y, RP_MLG_L11_z),    ),


                      # MLG_L1600 Display Body Construction
                      ( (RP_MLG_L12_x, RP_MLG_L12_y, RP_MLG_L12_z),
                        (RP_MLG_L13_x, RP_MLG_L13_y, RP_MLG_L13_z),    ),
  
                      # MLG_L1700 Display Body Construction
                      ( (RP_MLG_L12_x, RP_MLG_L12_y, RP_MLG_L12_z),
                        (RP_MLG_L14_x, RP_MLG_L14_y, RP_MLG_L14_z),    ),

                      # MLG_L1800 Display Body Construction
                      ( (RP_MLG_L13_x, RP_MLG_L13_y, RP_MLG_L13_z),
                        (RP_MLG_L15_x, RP_MLG_L15_y, RP_MLG_L15_z),    ),

                      # MLG_R1000 Display Body Construction
                      ( (-RP_MLG_L1_x, RP_MLG_L1_y, RP_MLG_L1_z),
                        (-RP_MLG_L4_x, RP_MLG_L4_y, RP_MLG_L4_z),      ),

                      # MLG_R1400 Display Body Construction
                      ( (-RP_MLG_L3_x, RP_MLG_L3_y, RP_MLG_L3_z),
                        (-RP_MLG_L7_x, RP_MLG_L7_y, RP_MLG_L7_z),
                        (-RP_MLG_L5_x, RP_MLG_L5_y, RP_MLG_L5_z),
                        (-RP_MLG_L6_x, RP_MLG_L6_y, RP_MLG_L6_z),      ),

                      # MLG_R1500 Display Body Construction
                      ( (-RP_MLG_L7_x, RP_MLG_L7_y, RP_MLG_L7_z),
                        (-RP_MLG_L11_x, RP_MLG_L11_y, RP_MLG_L11_z),   ),

                      # MLG_R1200 Display Body Construction
                      ( (-RP_MLG_L8_x, RP_MLG_L8_y, RP_MLG_L8_z),
                        (-RP_MLG_L9_x, RP_MLG_L9_y, RP_MLG_L9_z),      ),

                      # MLG_R1300 Display Body Construction
                      ( (-RP_MLG_L9_x, RP_MLG_L9_y, RP_MLG_L9_z),
                        (-RP_MLG_L11_x, RP_MLG_L11_y, RP_MLG_L11_z),   ),

                      # MLG_R1600 Display Body Construction
                      ( (-RP_MLG_L12_x, RP_MLG_L12_y, RP_MLG_L12_z),
                        (-RP_MLG_L13_x, RP_MLG_L13_y, RP_MLG_L13_z),   ),

                      # MLG_R1700 Display Body Construction
                      ( (-RP_MLG_L12_x, RP_MLG_L12_y, RP_MLG_L12_z),
                        (-RP_MLG_L14_x, RP_MLG_L14_y, RP_MLG_L14_z),   ),

                      # MLG_R1800 Display Body Construction
                      ( (-RP_MLG_L13_x, RP_MLG_L13_y, RP_MLG_L13_z),
                        (-RP_MLG_L15_x, RP_MLG_L15_y, RP_MLG_L15_z),   ),
                    ]



                    
    pointsSeqList = [ ( '(d[5], d[2]), (d[2], d[3]), (d[3], d[4]), (d[4], d[5]), '),
                      ( '(d[3], d[2]), (d[2], d[4]), (d[4], d[3]),               '),
                      ( '(d[3], d[2]), (d[4], d[5]),                             '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '), 
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),(d[4], d[5]),                              '), 
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),(d[4], d[5]),                              '), 
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),
                      ( '(d[2], d[3]),                                           '),

                    ]
    



    
    for partName, coordsSeq, pointsSeq in zip( partsNameList, coordsSeqList, pointsSeqList):
        p = model.Part(name=partName, dimensionality=THREE_D, type=DEFORMABLE_BODY)
        p.ReferencePoint(point=(0, 0, 0))               # point 1
        for coords in coordsSeq:
            p.DatumPointByCoordinate(coords=coords) # point 4
        d = p.datums
        exec('p.WirePolyLine(points=(%s),mergeType=SEPARATE, meshable=ON)') % pointsSeq
        p.seedPart(size=1000.0, deviationFactor=0.1)
        e = p.edges
        edges = e[:] 
        pickedRegions =(edges, )
        p.generateMesh()    
        a.Instance(name=partName+'-1', part=p, dependent=ON)

    vp.view.setValues(session.views['Right'])
    vp.view.setValues(nearPlane=64057.6, 
    farPlane=93440.1, width=3099.88, height=3555.04, viewOffsetX=-7899.7, 
    viewOffsetY=-648.919)
    vp.view.fitView()



    ###################################################################
    import regionToolset

    #################################################################### Variables for abq_SetNodeCoordinates
    #############                                       Node ID
    hp1111111 = a.ReferencePoint(point=(CG_x, CG_y, CG_z))                  # 1111111
    hp1000 = a.ReferencePoint(point=(RP_NLG_C1000_x, RP_NLG_C1000_y, RP_NLG_C1000_z))   # 1000
    hp1200 = a.ReferencePoint(point=(RP_NLG_C1200_x, RP_NLG_C1200_y, RP_NLG_C1200_z))   # 1200
    hp1300 = a.ReferencePoint(point=(RP_NLG_C1300_x, RP_NLG_C1300_y, RP_NLG_C1300_z))   # 1300
    hp1400 = a.ReferencePoint(point=(RP_NLG_C1400_x, RP_NLG_C1400_y, RP_NLG_C1400_z))   # 1400
    hp1500 = a.ReferencePoint(point=(RP_NLG_C1500_x, RP_NLG_C1500_y, RP_NLG_C1500_z))   # 1500
    hp1600 = a.ReferencePoint(point=(RP_NLG_C11_x, RP_NLG_C11_y, RP_NLG_C11_z))     # 1600
    hp1700 = a.ReferencePoint(point=(RP_NLG_C12_x, RP_NLG_C12_y, RP_NLG_C12_z))     # 1700
    hp1800 = a.ReferencePoint(point=(RP_NLG_C13_x, RP_NLG_C13_y, RP_NLG_C13_z))     # 1800
    #####
    hp11000 = a.ReferencePoint(point=(RP_MLG_L1000_x, RP_MLG_L1000_y, RP_MLG_L1000_z))  # 11000
    hp11400 = a.ReferencePoint(point=(RP_MLG_L1400_x, RP_MLG_L1400_y, RP_MLG_L1400_z))  # 11400
    hp11500 = a.ReferencePoint(point=(RP_MLG_L1500_x, RP_MLG_L1500_y, RP_MLG_L1500_z))  # 11500
    hp11200 = a.ReferencePoint(point=(RP_MLG_L1200_x, RP_MLG_L1200_y, RP_MLG_L1200_z))  # 11200
    hp11300 = a.ReferencePoint(point=(RP_MLG_L1300_x, RP_MLG_L1300_y, RP_MLG_L1300_z))  # 11300
    hp11600 = a.ReferencePoint(point=(RP_MLG_L11_x, RP_MLG_L11_y, RP_MLG_L11_z))        # 11600
    hp11700 = a.ReferencePoint(point=(RP_MLG_L12_x, RP_MLG_L12_y, RP_MLG_L12_z))        # 11700
    hp11800 = a.ReferencePoint(point=(RP_MLG_L13_x, RP_MLG_L13_y, RP_MLG_L13_z))        # 11800
    #####
    hp21000 = a.ReferencePoint(point=(RP_MLG_R1000_x, RP_MLG_L1000_y, RP_MLG_L1000_z))  # 21000
    hp21400 = a.ReferencePoint(point=(RP_MLG_R1400_x, RP_MLG_L1400_y, RP_MLG_L1400_z))  # 21400
    hp21500 = a.ReferencePoint(point=(RP_MLG_R1500_x, RP_MLG_L1500_y, RP_MLG_L1500_z))  # 21500
    hp21200 = a.ReferencePoint(point=(RP_MLG_R1200_x, RP_MLG_L1200_y, RP_MLG_L1200_z))  # 21200
    hp21300 = a.ReferencePoint(point=(RP_MLG_R1300_x, RP_MLG_L1300_y, RP_MLG_L1300_z))  # 21300
    hp21600 = a.ReferencePoint(point=(RP_MLG_R11_x, RP_MLG_L11_y, RP_MLG_L11_z))        # 21600
    hp21700 = a.ReferencePoint(point=(RP_MLG_R12_x, RP_MLG_L12_y, RP_MLG_L12_z))        # 21700
    hp21800 = a.ReferencePoint(point=(RP_MLG_R13_x, RP_MLG_L13_y, RP_MLG_L13_z))        # 21800
    #####################   Nose Gear Joints

    hp1001 = a.ReferencePoint(point=(RP_NLG_C1_x, RP_NLG_C1_y, RP_NLG_C1_z))        # 1001
    hp1002 = a.ReferencePoint(point=(RP_NLG_C2_x, RP_NLG_C2_y, RP_NLG_C2_z))        # 1002
    hp9901001 = a.ReferencePoint(point=(RP_NLG_C1_x, RP_NLG_C1_y, RP_NLG_C1_z))     # 9901001
    hp9901002 = a.ReferencePoint(point=(RP_NLG_C2_x, RP_NLG_C2_y, RP_NLG_C2_z))     # 9901002
    hp1004 = a.ReferencePoint(point=(RP_NLG_C4_x, RP_NLG_C4_y, RP_NLG_C4_z))        # 1004
    hp1205 = a.ReferencePoint(point=(RP_NLG_C5_x, RP_NLG_C5_y, RP_NLG_C5_z))        # 1205
    hp1206 = a.ReferencePoint(point=(RP_NLG_C6_x, RP_NLG_C6_y, RP_NLG_C6_z))        # 1206
    hp9901205 = a.ReferencePoint(point=(RP_NLG_C5_x, RP_NLG_C5_y, RP_NLG_C5_z))     # 9901205
    hp9901206 = a.ReferencePoint(point=(RP_NLG_C6_x, RP_NLG_C6_y, RP_NLG_C6_z))     # 9901206
    hp1204 = a.ReferencePoint(point=(RP_NLG_C4_x, RP_NLG_C4_y, RP_NLG_C4_z))        # 1204
    hp1207 = a.ReferencePoint(point=(RP_NLG_C7_x, RP_NLG_C7_y, RP_NLG_C7_z))        # 1207
    hp1210 = a.ReferencePoint(point=(RP_NLG_C10_x, RP_NLG_C10_y, RP_NLG_C10_z))     # 1210
    hp1220 = a.ReferencePoint(point=(RP_NLG_C20_x, RP_NLG_C20_y, RP_NLG_C20_z))     # 1220
    hp1310 = a.ReferencePoint(point=(RP_NLG_C10_x, RP_NLG_C10_y, RP_NLG_C10_z))     # 1310
    hp1311 = a.ReferencePoint(point=(RP_NLG_C11_x, RP_NLG_C11_y, RP_NLG_C11_z))     # 1311
    hp1309 = a.ReferencePoint(point=(RP_NLG_C9_x, RP_NLG_C9_y, RP_NLG_C9_z))        # 1309
    hp1407 = a.ReferencePoint(point=(RP_NLG_C7_x, RP_NLG_C7_y, RP_NLG_C7_z))        # 1407
    hp1408 = a.ReferencePoint(point=(RP_NLG_C8_x, RP_NLG_C8_y, RP_NLG_C8_z))        # 1408
    hp1508 = a.ReferencePoint(point=(RP_NLG_C8_x, RP_NLG_C8_y, RP_NLG_C8_z))        # 1508
    hp1509 = a.ReferencePoint(point=(RP_NLG_C9_x, RP_NLG_C9_y, RP_NLG_C9_z))        # 1509
    hp1612 = a.ReferencePoint(point=(RP_NLG_C12_x, RP_NLG_C12_y, RP_NLG_C12_z))     # 1612
    hp1613 = a.ReferencePoint(point=(RP_NLG_C13_x, RP_NLG_C13_y, RP_NLG_C13_z))     # 1613
    hp1712 = a.ReferencePoint(point=(RP_NLG_C12_x, RP_NLG_C12_y, RP_NLG_C12_z))     # 1712
    hp1813 = a.ReferencePoint(point=(RP_NLG_C13_x, RP_NLG_C13_y, RP_NLG_C13_z))     # 1813
    hp1714 = a.ReferencePoint(point=(RP_NLG_C14_x, RP_NLG_C14_y, RP_NLG_C14_z))     # 1714
    hp1815 = a.ReferencePoint(point=(RP_NLG_C15_x, RP_NLG_C15_y, RP_NLG_C15_z))     # 1815
    #####################   Main Gear Left Joints
    hp11001 = a.ReferencePoint(point=(RP_MLG_L1_x, RP_MLG_L1_y, RP_MLG_L1_z))       # 11001
    hp11101 = a.ReferencePoint(point=(MLG_FB_Axis_L_x, MLG_FB_Axis_L_y, MLG_FB_Axis_L_z))   # 11101
    hp9911001 = a.ReferencePoint(point=(RP_MLG_L1_x, RP_MLG_L1_y, RP_MLG_L1_z))     # 9911001
    hp11004 = a.ReferencePoint(point=(RP_MLG_L4_x, RP_MLG_L4_y, RP_MLG_L4_z))       # 11004
    hp11403 = a.ReferencePoint(point=(RP_MLG_L3_x, RP_MLG_L3_y, RP_MLG_L3_z))       # 11403
    hp11404 = a.ReferencePoint(point=(RP_MLG_L4_x, RP_MLG_L4_y, RP_MLG_L4_z))       # 11404
    hp11405 = a.ReferencePoint(point=(RP_MLG_L5_x, RP_MLG_L5_y, RP_MLG_L5_z))       # 11405
    hp11406 = a.ReferencePoint(point=(RP_MLG_L6_x, RP_MLG_L6_y, RP_MLG_L6_z))       # 11406
    hp11408 = a.ReferencePoint(point=(RP_MLG_L8_x, RP_MLG_L8_y, RP_MLG_L8_z))       # 11408
    hp9911405 = a.ReferencePoint(point=(RP_MLG_L5_x, RP_MLG_L5_y, RP_MLG_L5_z))     # 9911405
    hp9911406 = a.ReferencePoint(point=(RP_MLG_L6_x, RP_MLG_L6_y, RP_MLG_L6_z))     # 9911406
    hp11407 = a.ReferencePoint(point=(RP_MLG_L7_x, RP_MLG_L7_y, RP_MLG_L7_z))       # 11407
    hp11507 = a.ReferencePoint(point=(RP_MLG_L7_x, RP_MLG_L7_y, RP_MLG_L7_z))       # 11507
    hp11511 = a.ReferencePoint(point=(RP_MLG_L11_x, RP_MLG_L11_y, RP_MLG_L11_z))        # 11511
    hp11208 = a.ReferencePoint(point=(RP_MLG_L8_x, RP_MLG_L8_y, RP_MLG_L8_z))       # 11208
    hp11209 = a.ReferencePoint(point=(RP_MLG_L9_x, RP_MLG_L9_y, RP_MLG_L9_z))       # 11209
    hp11309 = a.ReferencePoint(point=(RP_MLG_L9_x, RP_MLG_L9_y, RP_MLG_L9_z))       # 11309
    hp11311 = a.ReferencePoint(point=(RP_MLG_L11_x, RP_MLG_L11_y, RP_MLG_L11_z))        # 11311
    hp11612 = a.ReferencePoint(point=(RP_MLG_L12_x, RP_MLG_L12_y, RP_MLG_L12_z))        # 11612
    hp11613 = a.ReferencePoint(point=(RP_MLG_L13_x, RP_MLG_L13_y, RP_MLG_L13_z))        # 11613
    hp11714 = a.ReferencePoint(point=(RP_MLG_L14_x, RP_MLG_L14_y, RP_MLG_L14_z))        # 11714
    hp11815 = a.ReferencePoint(point=(RP_MLG_L15_x, RP_MLG_L15_y, RP_MLG_L15_z))        # 11815
    hp11712 = a.ReferencePoint(point=(RP_MLG_L12_x, RP_MLG_L12_y, RP_MLG_L12_z))        # 11712
    hp11813 = a.ReferencePoint(point=(RP_MLG_L13_x, RP_MLG_L13_y, RP_MLG_L13_z))        # 11813
    #####################   Main Gear Right Joints
    hp21001 = a.ReferencePoint(point=(RP_MLG_R1_x, RP_MLG_L1_y, RP_MLG_L1_z))       # 21001
    hp21101 = a.ReferencePoint(point=(MLG_FB_Axis_R_x, MLG_FB_Axis_L_y, MLG_FB_Axis_L_z))   # 21101
    hp9921001 = a.ReferencePoint(point=(RP_MLG_R1_x, RP_MLG_L1_y, RP_MLG_L1_z))     # 9921001
    hp21004 = a.ReferencePoint(point=(RP_MLG_R4_x, RP_MLG_L4_y, RP_MLG_L4_z))       # 21004
    hp21403 = a.ReferencePoint(point=(RP_MLG_R3_x, RP_MLG_L3_y, RP_MLG_L3_z))       # 21403
    hp21404 = a.ReferencePoint(point=(RP_MLG_R4_x, RP_MLG_L4_y, RP_MLG_L4_z))       # 21404
    hp21405 = a.ReferencePoint(point=(RP_MLG_R5_x, RP_MLG_L5_y, RP_MLG_L5_z))       # 21405
    hp21406 = a.ReferencePoint(point=(RP_MLG_R6_x, RP_MLG_L6_y, RP_MLG_L6_z))       # 21406
    hp21408 = a.ReferencePoint(point=(RP_MLG_R8_x, RP_MLG_L8_y, RP_MLG_L8_z))       # 21408
    hp9921405 = a.ReferencePoint(point=(RP_MLG_R5_x, RP_MLG_L5_y, RP_MLG_L5_z))     # 9921405
    hp9921406 = a.ReferencePoint(point=(RP_MLG_R6_x, RP_MLG_L6_y, RP_MLG_L6_z))     # 9921406
    hp21407 = a.ReferencePoint(point=(RP_MLG_R7_x, RP_MLG_L7_y, RP_MLG_L7_z))       # 21407
    hp21507 = a.ReferencePoint(point=(RP_MLG_R7_x, RP_MLG_L7_y, RP_MLG_L7_z))       # 21507
    hp21511 = a.ReferencePoint(point=(RP_MLG_R11_x, RP_MLG_L11_y, RP_MLG_L11_z))        # 21511
    hp21208 = a.ReferencePoint(point=(RP_MLG_R8_x, RP_MLG_L8_y, RP_MLG_L8_z))       # 21208
    hp21209 = a.ReferencePoint(point=(RP_MLG_R9_x, RP_MLG_L9_y, RP_MLG_L9_z))       # 21209
    hp21309 = a.ReferencePoint(point=(RP_MLG_R9_x, RP_MLG_L9_y, RP_MLG_L9_z))       # 21309
    hp21311 = a.ReferencePoint(point=(RP_MLG_R11_x, RP_MLG_L11_y, RP_MLG_L11_z))        # 21311
    hp21612 = a.ReferencePoint(point=(RP_MLG_R12_x, RP_MLG_L12_y, RP_MLG_L12_z))        # 21612
    hp21613 = a.ReferencePoint(point=(RP_MLG_R13_x, RP_MLG_L13_y, RP_MLG_L13_z))        # 21613
    hp21714 = a.ReferencePoint(point=(RP_MLG_R14_x, RP_MLG_L14_y, RP_MLG_L14_z))        # 21714
    hp21815 = a.ReferencePoint(point=(RP_MLG_R15_x, RP_MLG_L15_y, RP_MLG_L15_z))        # 21815
    hp21712 = a.ReferencePoint(point=(RP_MLG_R12_x, RP_MLG_L12_y, RP_MLG_L12_z))        # 21712
    hp21813 = a.ReferencePoint(point=(RP_MLG_R13_x, RP_MLG_L13_y, RP_MLG_L13_z))        # 21813
    #####################   
    hp5501714 = a.ReferencePoint(point=(RP_NLG_C14_x, ground_y, RP_NLG_C14_z))          # 5501714
    hp5501815 = a.ReferencePoint(point=(RP_NLG_C15_x, ground_y, RP_NLG_C15_z))      # 5501815
    hp5511714 = a.ReferencePoint(point=(RP_MLG_L14_x, ground_y, RP_MLG_L14_z))      # 5511714
    hp5511815 = a.ReferencePoint(point=(RP_MLG_L15_x, ground_y, RP_MLG_L15_z))      # 5511815
    hp5521714 = a.ReferencePoint(point=(RP_MLG_R14_x, ground_y, RP_MLG_L14_z))      # 5521714
    hp5521815 = a.ReferencePoint(point=(RP_MLG_R15_x, ground_y, RP_MLG_L15_z))      # 5521815
    #####################
    hp5500000 = a.ReferencePoint(point=(ground_x, ground_y, ground_z))          # 5500000
    #####################   Reference Name Change   

    a.features.changeKey(fromName='RP-1', toName='HP_airplane_CG')
    a.features.changeKey(fromName='RP-2', toName='HP1000')
    a.features.changeKey(fromName='RP-3', toName='HP1200')
    a.features.changeKey(fromName='RP-4', toName='HP1300')
    a.features.changeKey(fromName='RP-5', toName='HP1400')
    a.features.changeKey(fromName='RP-6', toName='HP1500')
    a.features.changeKey(fromName='RP-7', toName='HP1600')
    a.features.changeKey(fromName='RP-8', toName='HP1700')
    a.features.changeKey(fromName='RP-9', toName='HP1800')
    a.features.changeKey(fromName='RP-10', toName='HP11000')
    a.features.changeKey(fromName='RP-11', toName='HP11400')
    a.features.changeKey(fromName='RP-12', toName='HP11500')
    a.features.changeKey(fromName='RP-13', toName='HP11200')
    a.features.changeKey(fromName='RP-14', toName='HP11300')
    a.features.changeKey(fromName='RP-15', toName='HP11600')
    a.features.changeKey(fromName='RP-16', toName='HP11700')
    a.features.changeKey(fromName='RP-17', toName='HP11800')
    a.features.changeKey(fromName='RP-18', toName='HP21000')
    a.features.changeKey(fromName='RP-19', toName='HP21400')
    a.features.changeKey(fromName='RP-20', toName='HP21500')
    a.features.changeKey(fromName='RP-21', toName='HP21200')
    a.features.changeKey(fromName='RP-22', toName='HP21300')
    a.features.changeKey(fromName='RP-23', toName='HP21600')
    a.features.changeKey(fromName='RP-24', toName='HP21700')
    a.features.changeKey(fromName='RP-25', toName='HP21800')
    a.features.changeKey(fromName='RP-26', toName='HP1001')
    a.features.changeKey(fromName='RP-27', toName='HP1002')
    a.features.changeKey(fromName='RP-28', toName='HP9901001')
    a.features.changeKey(fromName='RP-29', toName='HP9901002')
    a.features.changeKey(fromName='RP-30', toName='HP1004')
    a.features.changeKey(fromName='RP-31', toName='HP1205')
    a.features.changeKey(fromName='RP-32', toName='HP1206')
    a.features.changeKey(fromName='RP-33', toName='HP9901205')
    a.features.changeKey(fromName='RP-34', toName='HP9901206')
    a.features.changeKey(fromName='RP-35', toName='HP1204')
    a.features.changeKey(fromName='RP-36', toName='HP1207')
    a.features.changeKey(fromName='RP-37', toName='HP1210')
    a.features.changeKey(fromName='RP-38', toName='HP1220')
    a.features.changeKey(fromName='RP-39', toName='HP1310')
    a.features.changeKey(fromName='RP-40', toName='HP1311')
    a.features.changeKey(fromName='RP-41', toName='HP1309')
    a.features.changeKey(fromName='RP-42', toName='HP1407')
    a.features.changeKey(fromName='RP-43', toName='HP1408')
    a.features.changeKey(fromName='RP-44', toName='HP1508')
    a.features.changeKey(fromName='RP-45', toName='HP1509')
    a.features.changeKey(fromName='RP-46', toName='HP1612')
    a.features.changeKey(fromName='RP-47', toName='HP1613')
    a.features.changeKey(fromName='RP-48', toName='HP1712')
    a.features.changeKey(fromName='RP-49', toName='HP1813')
    a.features.changeKey(fromName='RP-50', toName='HP1714')
    a.features.changeKey(fromName='RP-51', toName='HP1815')
    a.features.changeKey(fromName='RP-52', toName='HP11001')
    a.features.changeKey(fromName='RP-53', toName='HP11101')
    a.features.changeKey(fromName='RP-54', toName='HP9911001')
    a.features.changeKey(fromName='RP-55', toName='HP11004')
    a.features.changeKey(fromName='RP-56', toName='HP11403')
    a.features.changeKey(fromName='RP-57', toName='HP11404')
    a.features.changeKey(fromName='RP-58', toName='HP11405')
    a.features.changeKey(fromName='RP-59', toName='HP11406')
    a.features.changeKey(fromName='RP-60', toName='HP11408')
    a.features.changeKey(fromName='RP-61', toName='HP9911405')
    a.features.changeKey(fromName='RP-62', toName='HP9911406')
    a.features.changeKey(fromName='RP-63', toName='HP11407')
    a.features.changeKey(fromName='RP-64', toName='HP11507')
    a.features.changeKey(fromName='RP-65', toName='HP11511')
    a.features.changeKey(fromName='RP-66', toName='HP11208')
    a.features.changeKey(fromName='RP-67', toName='HP11209')
    a.features.changeKey(fromName='RP-68', toName='HP11309')
    a.features.changeKey(fromName='RP-69', toName='HP11311')
    a.features.changeKey(fromName='RP-70', toName='HP11612')
    a.features.changeKey(fromName='RP-71', toName='HP11613')
    a.features.changeKey(fromName='RP-72', toName='HP11714')
    a.features.changeKey(fromName='RP-73', toName='HP11815')
    a.features.changeKey(fromName='RP-74', toName='HP11712')
    a.features.changeKey(fromName='RP-75', toName='HP11713')
    a.features.changeKey(fromName='RP-76', toName='HP21001')
    a.features.changeKey(fromName='RP-77', toName='HP21101')
    a.features.changeKey(fromName='RP-78', toName='HP9921001')
    a.features.changeKey(fromName='RP-79', toName='HP21004')
    a.features.changeKey(fromName='RP-80', toName='HP21403')
    a.features.changeKey(fromName='RP-81', toName='HP21404')
    a.features.changeKey(fromName='RP-82', toName='HP21405')
    a.features.changeKey(fromName='RP-83', toName='HP21406')
    a.features.changeKey(fromName='RP-84', toName='HP21408')
    a.features.changeKey(fromName='RP-85', toName='HP9921405')
    a.features.changeKey(fromName='RP-86', toName='HP9921406')
    a.features.changeKey(fromName='RP-87', toName='HP21407')
    a.features.changeKey(fromName='RP-88', toName='HP21507')
    a.features.changeKey(fromName='RP-89', toName='HP21511')
    a.features.changeKey(fromName='RP-90', toName='HP21208')
    a.features.changeKey(fromName='RP-91', toName='HP21209')
    a.features.changeKey(fromName='RP-92', toName='HP21309')
    a.features.changeKey(fromName='RP-93', toName='HP21311')
    a.features.changeKey(fromName='RP-94', toName='HP21612')
    a.features.changeKey(fromName='RP-95', toName='HP21613')
    a.features.changeKey(fromName='RP-96', toName='HP21714')
    a.features.changeKey(fromName='RP-97', toName='HP21815')
    a.features.changeKey(fromName='RP-98', toName='HP21712')
    a.features.changeKey(fromName='RP-99', toName='HP21713')
    a.features.changeKey(fromName='RP-100', toName='HP5501714')
    a.features.changeKey(fromName='RP-101', toName='HP5501815')
    a.features.changeKey(fromName='RP-102', toName='HP5511714')
    a.features.changeKey(fromName='RP-103', toName='HP5511815')
    a.features.changeKey(fromName='RP-104', toName='HP5521714')
    a.features.changeKey(fromName='RP-105', toName='HP5521815')
    a.features.changeKey(fromName='RP-106', toName='HP5500000')
    #####################   

    r1 = a.referencePoints
    refPoints1=(r1[hp1111111.id], r1[hp1000.id], r1[hp1200.id], r1[hp1300.id], r1[hp1400.id],
            r1[hp1500.id], r1[hp1600.id], r1[hp1700.id], r1[hp1800.id],
            r1[hp11000.id], r1[hp11400.id], r1[hp11500.id], r1[hp11200.id], r1[hp11300.id],
            r1[hp11600.id], r1[hp11700.id], r1[hp11800.id], 
            r1[hp21000.id], r1[hp21400.id], r1[hp21500.id], r1[hp21200.id], r1[hp21300.id],
            r1[hp21600.id], r1[hp21700.id], r1[hp21800.id], 
            r1[hp1001.id], r1[hp1002.id], r1[hp9901001.id], r1[hp9901002.id], r1[hp1004.id], 
            r1[hp1205.id], r1[hp1206.id], r1[hp9901205.id], r1[hp9901206.id], r1[hp1204.id], 
            r1[hp1207.id], r1[hp1210.id], r1[hp1220.id], r1[hp1310.id], r1[hp1311.id], 
            r1[hp1309.id], r1[hp1407.id], r1[hp1408.id], r1[hp1508.id], r1[hp1509.id], 
            r1[hp1612.id], r1[hp1613.id], r1[hp1712.id], r1[hp1813.id], r1[hp1714.id], r1[hp1815.id], 
            r1[hp11001.id], r1[hp11101.id], r1[hp9911001.id], r1[hp11004.id], 
            r1[hp11403.id], r1[hp11404.id], r1[hp11405.id], r1[hp9911406.id], r1[hp11408.id], 
            r1[hp9911405.id], r1[hp9911406.id], r1[hp11407.id], r1[hp11507.id], r1[hp11511.id], 
            r1[hp11208.id], r1[hp11209.id], r1[hp11309.id], r1[hp11311.id], 
            r1[hp11612.id], r1[hp11613.id], r1[hp11714.id], r1[hp11815.id], r1[hp11712.id], r1[hp11813.id],
            r1[hp21001.id], r1[hp21101.id], r1[hp9921001.id], r1[hp21004.id], 
            r1[hp21403.id], r1[hp21404.id], r1[hp21405.id], r1[hp9921406.id], r1[hp21408.id], 
            r1[hp9921405.id], r1[hp9921406.id], r1[hp21407.id], r1[hp21507.id], r1[hp21511.id], 
            r1[hp21208.id], r1[hp21209.id], r1[hp21309.id], r1[hp21311.id], 
            r1[hp21612.id], r1[hp21613.id], r1[hp21714.id], r1[hp21815.id], r1[hp21712.id], r1[hp21813.id],) 
    a.Set(referencePoints=refPoints1, name='nall')

    r1 = a.referencePoints
    refPoints1=(r1[hp5500000.id], ) 
    a.Set(referencePoints=refPoints1, name='nground')
    r1 = a.referencePoints
    refPoints1=(r1[hp1111111.id], ) 
    a.Set(referencePoints=refPoints1, name='nCG')
    #################################################################### for abq_SetNodeCoordinates

    #vp.assemblyDisplay.setValues(interactions=ON, 
    #    constraints=ON, connectors=ON, engineeringFeatures=ON)
    r1 = a.referencePoints
    refPoints1=(r1[hp1111111.id],)
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='CG_mass', region=region, mass=airplane_mass, i11=airplane_inertia_x, i22=airplane_inertia_y, 
    i33=airplane_inertia_z, alpha=0.0, composite=0.0)
    ###
    refPoints1=(r1[hp1000.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='Nose_FB_m', region=region, mass=nose_FB_m, i11=nose_FB_m_x, i22=nose_FB_m_y, 
    i33=nose_FB_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp1200.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='Nose_MF_m', region=region, mass=nose_MF_m, i11=nose_MF_m_x, i22=nose_MF_m_y, 
    i33=nose_MF_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp1300.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='Nose_piston_m', region=region, mass=nose_piston_m, i11=nose_piston_m_x, i22=nose_piston_m_y, 
    i33=nose_piston_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp1400.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='Nose_TP_m', region=region, mass=nose_TP_m, i11=nose_TP_m_x, i22=nose_TP_m_y, 
    i33=nose_TP_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp1500.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='Nose_TB_m', region=region, mass=nose_TB_m, i11=nose_TB_m_x, i22=nose_TB_m_y, 
    i33=nose_TB_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp1600.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='Nose_WA_m', region=region, mass=nose_WA_m, i11=nose_WA_m_x, i22=nose_WA_m_y, 
    i33=nose_WA_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp1700.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='Nose_Wheel1_m', region=region, mass=nose_Wheel_m, i11=nose_Wheel_m_x, i22=nose_Wheel_m_y, 
    i33=nose_Wheel_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp1800.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='Nose_Wheel2_m', region=region, mass=nose_Wheel_m, i11=nose_Wheel_m_x, i22=nose_Wheel_m_y, 
    i33=nose_Wheel_m_z, alpha=0.0, composite=0.0)
    ####
    refPoints1=(r1[hp11000.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainL_FB_m', region=region, mass=main_FB_m, i11=main_FB_m_x, i22=main_FB_m_y, 
    i33=main_FB_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp11400.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainL_MF_m', region=region, mass=main_MF_m, i11=main_MF_m_x, i22=main_MF_m_y, 
    i33=main_MF_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp11500.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainL_TA_m', region=region, mass=main_TA_m, i11=main_TA_m_x, i22=main_TA_m_y, 
    i33=main_TA_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp11200.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainL_SAT_m', region=region, mass=main_SAT_m, i11=main_SAT_m_x, i22=main_SAT_m_y, 
    i33=main_SAT_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp11300.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainL_SAB_m', region=region, mass=main_SAB_m, i11=main_SAB_m_x, i22=main_SAB_m_y, 
    i33=main_SAB_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp11600.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainL_WA_m', region=region, mass=main_WA_m, i11=main_WA_m_x, i22=main_WA_m_y, 
    i33=main_WA_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp11700.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainL1_Wheel_m', region=region, mass=main_Wheel_m, i11=main_Wheel_m_x, i22=main_Wheel_m_y, 
    i33=main_Wheel_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp11800.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainL2_Wheel_m', region=region, mass=main_Wheel_m, i11=main_Wheel_m_x, i22=main_Wheel_m_y, 
    i33=main_Wheel_m_z, alpha=0.0, composite=0.0)
    ####
    refPoints1=(r1[hp21000.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainR_FB_m', region=region, mass=main_FB_m, i11=main_FB_m_x, i22=main_FB_m_y, 
    i33=main_FB_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp21400.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainR_MF_m', region=region, mass=main_MF_m, i11=main_MF_m_x, i22=main_MF_m_y, 
    i33=main_MF_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp21500.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainR_TA_m', region=region, mass=main_TA_m, i11=main_TA_m_x, i22=main_TA_m_y, 
    i33=main_TA_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp21200.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainR_SAT_m', region=region, mass=main_SAT_m, i11=main_SAT_m_x, i22=main_SAT_m_y, 
    i33=main_SAT_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp21300.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainR_SAB_m', region=region, mass=main_SAB_m, i11=main_SAB_m_x, i22=main_SAB_m_y, 
    i33=main_SAB_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp21600.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainR_WA_m', region=region, mass=main_WA_m, i11=main_WA_m_x, i22=main_WA_m_y, 
    i33=main_WA_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp21700.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainR1_Wheel_m', region=region, mass=main_Wheel_m, i11=main_Wheel_m_x, i22=main_Wheel_m_y, 
    i33=main_Wheel_m_z, alpha=0.0, composite=0.0)
    refPoints1=(r1[hp21800.id], )
    region=regionToolset.Region(referencePoints=refPoints1)
    model.rootAssembly.engineeringFeatures.PointMassInertia(
    name='mainR2_Wheel_m', region=region, mass=main_Wheel_m, i11=main_Wheel_m_x, i22=main_Wheel_m_y, 
    i33=main_Wheel_m_z, alpha=0.0, composite=0.0)
    #############################################
    b = model.rootAssembly.engineeringFeatures
    b.PointMassInertia(name='m1', region=regionToolset.Region(referencePoints=(r1[hp1001.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m2', region=regionToolset.Region(referencePoints=(r1[hp1002.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m3', region=regionToolset.Region(referencePoints=(r1[hp9901001.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m4', region=regionToolset.Region(referencePoints=(r1[hp9901002.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m5', region=regionToolset.Region(referencePoints=(r1[hp1004.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m6', region=regionToolset.Region(referencePoints=(r1[hp1205.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m7', region=regionToolset.Region(referencePoints=(r1[hp1206.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m8', region=regionToolset.Region(referencePoints=(r1[hp9901205.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m9', region=regionToolset.Region(referencePoints=(r1[hp9901206.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m10', region=regionToolset.Region(referencePoints=(r1[hp1204.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m11', region=regionToolset.Region(referencePoints=(r1[hp1207.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m12', region=regionToolset.Region(referencePoints=(r1[hp1210.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m13', region=regionToolset.Region(referencePoints=(r1[hp1220.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m14', region=regionToolset.Region(referencePoints=(r1[hp1310.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m15', region=regionToolset.Region(referencePoints=(r1[hp1311.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m16', region=regionToolset.Region(referencePoints=(r1[hp1309.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m17', region=regionToolset.Region(referencePoints=(r1[hp1407.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m18', region=regionToolset.Region(referencePoints=(r1[hp1408.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m19', region=regionToolset.Region(referencePoints=(r1[hp1508.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m20', region=regionToolset.Region(referencePoints=(r1[hp1509.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m21', region=regionToolset.Region(referencePoints=(r1[hp1612.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m22', region=regionToolset.Region(referencePoints=(r1[hp1613.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m23', region=regionToolset.Region(referencePoints=(r1[hp1712.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m24', region=regionToolset.Region(referencePoints=(r1[hp1813.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m25', region=regionToolset.Region(referencePoints=(r1[hp1714.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='m26', region=regionToolset.Region(referencePoints=(r1[hp1815.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    #
    b.PointMassInertia(name='1m1', region=regionToolset.Region(referencePoints=(r1[hp11001.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m2', region=regionToolset.Region(referencePoints=(r1[hp11101.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m3', region=regionToolset.Region(referencePoints=(r1[hp9911001.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m4', region=regionToolset.Region(referencePoints=(r1[hp11004.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m5', region=regionToolset.Region(referencePoints=(r1[hp11403.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m6', region=regionToolset.Region(referencePoints=(r1[hp11404.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m7', region=regionToolset.Region(referencePoints=(r1[hp11405.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m8', region=regionToolset.Region(referencePoints=(r1[hp11406.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m9', region=regionToolset.Region(referencePoints=(r1[hp11408.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m10', region=regionToolset.Region(referencePoints=(r1[hp9911405.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m11', region=regionToolset.Region(referencePoints=(r1[hp9911406.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m12', region=regionToolset.Region(referencePoints=(r1[hp11407.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m13', region=regionToolset.Region(referencePoints=(r1[hp11507.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m14', region=regionToolset.Region(referencePoints=(r1[hp11511.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m15', region=regionToolset.Region(referencePoints=(r1[hp11208.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m16', region=regionToolset.Region(referencePoints=(r1[hp11209.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m17', region=regionToolset.Region(referencePoints=(r1[hp11309.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m18', region=regionToolset.Region(referencePoints=(r1[hp11311.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m19', region=regionToolset.Region(referencePoints=(r1[hp11612.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m20', region=regionToolset.Region(referencePoints=(r1[hp11613.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m21', region=regionToolset.Region(referencePoints=(r1[hp11714.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m22', region=regionToolset.Region(referencePoints=(r1[hp11815.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m23', region=regionToolset.Region(referencePoints=(r1[hp11712.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='1m24', region=regionToolset.Region(referencePoints=(r1[hp11813.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    #
    b.PointMassInertia(name='2m1', region=regionToolset.Region(referencePoints=(r1[hp21001.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m2', region=regionToolset.Region(referencePoints=(r1[hp21101.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m3', region=regionToolset.Region(referencePoints=(r1[hp9921001.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m4', region=regionToolset.Region(referencePoints=(r1[hp21004.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m5', region=regionToolset.Region(referencePoints=(r1[hp21403.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m6', region=regionToolset.Region(referencePoints=(r1[hp21404.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m7', region=regionToolset.Region(referencePoints=(r1[hp21405.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m8', region=regionToolset.Region(referencePoints=(r1[hp21406.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m9', region=regionToolset.Region(referencePoints=(r1[hp21408.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m10', region=regionToolset.Region(referencePoints=(r1[hp9921405.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m11', region=regionToolset.Region(referencePoints=(r1[hp9921406.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m12', region=regionToolset.Region(referencePoints=(r1[hp21407.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m13', region=regionToolset.Region(referencePoints=(r1[hp21507.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m14', region=regionToolset.Region(referencePoints=(r1[hp21511.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m15', region=regionToolset.Region(referencePoints=(r1[hp21208.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m16', region=regionToolset.Region(referencePoints=(r1[hp21209.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m17', region=regionToolset.Region(referencePoints=(r1[hp21309.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m18', region=regionToolset.Region(referencePoints=(r1[hp21311.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m19', region=regionToolset.Region(referencePoints=(r1[hp21612.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m20', region=regionToolset.Region(referencePoints=(r1[hp21613.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m21', region=regionToolset.Region(referencePoints=(r1[hp21714.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m22', region=regionToolset.Region(referencePoints=(r1[hp21815.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m23', region=regionToolset.Region(referencePoints=(r1[hp21712.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='2m24', region=regionToolset.Region(referencePoints=(r1[hp21813.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    #
    b.PointMassInertia(name='3m1', region=regionToolset.Region(referencePoints=(r1[hp5501714.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='3m2', region=regionToolset.Region(referencePoints=(r1[hp5501815.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='3m3', region=regionToolset.Region(referencePoints=(r1[hp5511714.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='3m4', region=regionToolset.Region(referencePoints=(r1[hp5511815.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='3m5', region=regionToolset.Region(referencePoints=(r1[hp5521714.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='3m6', region=regionToolset.Region(referencePoints=(r1[hp5521815.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    b.PointMassInertia(name='3m7', region=regionToolset.Region(referencePoints=(r1[hp5500000.id], )), mass=1.e-7, i11=1.e-7, i22=1.e-7, i33=1.e-7, alpha=0.0, composite=0.0)
    #############################################

    model.DisplayBody(name='DISPLAY-AIRPLANE', 
    instance=a.instances['AIRPLANE-1'], controlPoints=(r1[hp1111111.id], ))
    refPoints1=(r1[hp9901001.id], r1[hp9901002.id], r1[hp9901205.id], r1[hp9901206.id], r1[hp9911001.id], r1[hp9911405.id], r1[hp9911406.id], r1[hp9921001.id], r1[hp9921405.id], r1[hp9921406.id] )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp1111111.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-AIRPLANE', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-ground', 
    instance=a.instances['GROUND-1'], controlPoints=(r1[hp5500000.id], ))
    ##
    model.DisplayBody(name='DISPLAY-NLG_1000', 
    instance=a.instances['NLG_1000-1'], controlPoints=(r1[hp1000.id], ))
    refPoints1=(r1[hp1001.id], r1[hp1002.id], r1[hp1004.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp1000.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-NLG_1000', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-NLG_1200', 
    instance=a.instances['NLG_1200-1'], controlPoints=(r1[hp1200.id], ))
    refPoints1=(r1[hp1205.id], r1[hp1206.id], r1[hp1204.id], r1[hp1207.id], r1[hp1210.id], r1[hp1220.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp1200.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-NLG_1200', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-NLG_1300', 
    instance=a.instances['NLG_1300-1'], controlPoints=(r1[hp1300.id], ))
    refPoints1=(r1[hp1310.id], r1[hp1311.id], r1[hp1309.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp1300.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-NLG_1300', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-NLG_1400', 
    instance=a.instances['NLG_1400-1'], controlPoints=(r1[hp1400.id], ))
    refPoints1=(r1[hp1407.id], r1[hp1408.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp1400.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-NLG_1400', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-NLG_1500', 
    instance=a.instances['NLG_1500-1'], controlPoints=(r1[hp1500.id], ))
    refPoints1=(r1[hp1509.id], r1[hp1508.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp1500.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-NLG_1500', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-NLG_1600', 
    instance=a.instances['NLG_1600-1'], controlPoints=(r1[hp1600.id], ))
    refPoints1=(r1[hp1612.id], r1[hp1613.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp1600.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-NLG_1600', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-NLG_1700', 
    instance=a.instances['NLG_1700-1'], controlPoints=(r1[hp1700.id], ))
    refPoints1=(r1[hp1712.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp1700.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-NLG_1700', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-NLG_1800', 
    instance=a.instances['NLG_1800-1'], controlPoints=(r1[hp1800.id], ))
    refPoints1=(r1[hp1813.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp1800.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-NLG_1800', 
    refPointRegion=region1, tieRegion=region4)
    ######
    ##
    model.DisplayBody(name='DISPLAY-MLG_L1000', 
    instance=a.instances['MLG_L1000-1'], controlPoints=(r1[hp11000.id], ))
    refPoints1=(r1[hp11001.id], r1[hp11004.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp11000.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_L1000', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_L1400', 
    instance=a.instances['MLG_L1400-1'], controlPoints=(r1[hp11400.id], ))
    refPoints1=(r1[hp11405.id], r1[hp11406.id], r1[hp11408.id], r1[hp11404.id], r1[hp11407.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp11400.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_L1400', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_L1500', 
    instance=a.instances['MLG_L1500-1'], controlPoints=(r1[hp11500.id], ))
    refPoints1=(r1[hp11507.id], r1[hp11511.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp11500.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_L1500', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_L1200', 
    instance=a.instances['MLG_L1200-1'], controlPoints=(r1[hp11200.id], ))
    refPoints1=(r1[hp11208.id], r1[hp11209.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp11200.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_L1200', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_L1300', 
    instance=a.instances['MLG_L1300-1'], controlPoints=(r1[hp11300.id], ))
    refPoints1=(r1[hp11311.id], r1[hp11309.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp11300.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_L1300', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_L1600', 
    instance=a.instances['MLG_L1600-1'], controlPoints=(r1[hp11600.id], ))
    refPoints1=(r1[hp11612.id], r1[hp11613.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp11600.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_L1600', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_L1700', 
    instance=a.instances['MLG_L1700-1'], controlPoints=(r1[hp11700.id], ))
    refPoints1=(r1[hp11712.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp11700.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_L1700', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_L1800', 
    instance=a.instances['MLG_L1800-1'], controlPoints=(r1[hp11800.id], ))
    refPoints1=(r1[hp11813.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp11800.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_L1800', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_R1000', 
    instance=a.instances['MLG_R1000-1'], controlPoints=(r1[hp21000.id], ))
    refPoints1=(r1[hp21001.id], r1[hp21004.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp21000.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_R1000', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_R1400', 
    instance=a.instances['MLG_R1400-1'], controlPoints=(r1[hp21400.id], ))
    refPoints1=(r1[hp21405.id], r1[hp21406.id], r1[hp21408.id], r1[hp21404.id], r1[hp21407.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp21400.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_R1400', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_R1500', 
    instance=a.instances['MLG_R1500-1'], controlPoints=(r1[hp21500.id], ))
    refPoints1=(r1[hp21507.id], r1[hp21511.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp21500.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_R1500', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_R1200', 
    instance=a.instances['MLG_R1200-1'], controlPoints=(r1[hp21200.id], ))
    refPoints1=(r1[hp21208.id], r1[hp21209.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp21200.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_R1200', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_R1300', 
    instance=a.instances['MLG_R1300-1'], controlPoints=(r1[hp21300.id], ))
    refPoints1=(r1[hp21311.id], r1[hp21309.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp21300.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_R1300', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_R1600', 
    instance=a.instances['MLG_R1600-1'], controlPoints=(r1[hp21600.id], ))
    refPoints1=(r1[hp21612.id], r1[hp21613.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp21600.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_R1600', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_R1700', 
    instance=a.instances['MLG_R1700-1'], controlPoints=(r1[hp21700.id], ))
    refPoints1=(r1[hp21712.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp21700.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_R1700', 
    refPointRegion=region1, tieRegion=region4)
    ##
    model.DisplayBody(name='DISPLAY-MLG_R1800', 
    instance=a.instances['MLG_R1800-1'], controlPoints=(r1[hp21800.id], ))
    refPoints1=(r1[hp21813.id], )
    region4=regionToolset.Region(referencePoints=refPoints1)
    refPoints1=(r1[hp21800.id], )
    region1=regionToolset.Region(referencePoints=refPoints1)
    model.RigidBody(name='RIGID-MLG_R1800', 
    refPointRegion=region1, tieRegion=region4)
    ################################
    ################################
    ################################
    ################################    CONNECTOR SECTION TYPE AND BEHAVIOR DEFINITION
    import section
    ####    NOSE SPRING PROPERTY DEFINITION
    ####        READ spring_node.property file.
    ####            - no tab, ',' is right after value
    ####            ex)  -10.e3, 0.1    : OK
    ####            ex)  -10.e3 , 0.1   : NG
    f_spn=open(spring_nose_filename, 'r')
    FL_spn=f_spn.readlines()
    spring_nose_data = [] 
    for line in FL_spn:
        dummy =  []
        #FL_spn1 = line.rstrip()
        #FL_spn2 = FL_spn1.split(',')
        FL_spn2 = line.split(',')
        FL1 = FL_spn2[0].strip(' ')
        FL2 = FL_spn2[1].strip(' ')
        dummy.append(float(FL1))
        dummy.append(float(FL2))
        spring_nose_data.append(dummy)
    ####    NOSE DAMPER PROPERTY DEFINITION
    f_dpn=open(damper_nose_filename, 'r')
    FL_dpn=f_dpn.readlines()
    damper_nose_data = [] 
    for line in FL_dpn:
        dummy =  []
        #FL_dpn1 = line.rstrip()
        #FL_dpn2 = FL_dpn1.split(',')
        FL_dpn2 = line.split(',')
        FL1 = FL_dpn2[0].strip(' ')
        FL2 = FL_dpn2[1].strip(' ')
        FL3 = FL_dpn2[2].strip(' ')
        dummy.append(float(FL1))
        dummy.append(float(FL2))
        dummy.append(float(FL3))
        damper_nose_data.append(dummy)
    ####
    f_spm=open(spring_main_filename, 'r')
    FL_spm=f_spm.readlines()
    spring_main_data = [] 
    for line in FL_spm:
        dummy =  []
        #FL_spm1 = line.rstrip()
        #FL_spm2 = FL_spm1.split(',')
        FL_spm2 = line.split(',')
        FL1 = FL_spm2[0].strip(' ')
        FL2 = FL_spm2[1].strip(' ')
        dummy.append(float(FL1))
        dummy.append(float(FL2))
        spring_main_data.append(dummy)
    ####    NOSE DAMPER PROPERTY DEFINITION
    f_dpm=open(damper_main_filename, 'r')
    FL_dpm=f_dpm.readlines()
    damper_main_data = [] 
    for line in FL_dpm:
        dummy =  []
        #FL_dpm1 = line.rstrip()
        #FL_dpm2 = FL_dpm1.split(',')
        FL_dpm2 = line.split(',')
        FL1 = FL_dpm2[0].strip(' ')
        FL2 = FL_dpm2[1].strip(' ')
        FL3 = FL_dpm2[2].strip(' ')
        dummy.append(float(FL1))
        dummy.append(float(FL2))
        dummy.append(float(FL3))
        damper_main_data.append(dummy)
    ####    TIRE NOSE ELASTIC PROPERTY
    f_tin=open(tire_nose_filename, 'r')
    FL_tin=f_tin.readlines()
    tire_nose_data = [] 
    for line in FL_tin:
        dummy =  []
        #FL_tin1 = line.rstrip()
        #FL_tin2 = FL_tin1.split(',')
        FL_tin2 = line.split(',')
        FL1 = FL_tin2[0].strip(' ')
        FL2 = FL_tin2[1].strip(' ')
        dummy.append(float(FL1))
        dummy.append(float(FL2))
        tire_nose_data.append(dummy)
    ####    TIRE MAIN ELASTIC PROPERTY
    f_tim=open(tire_main_filename, 'r')
    FL_tim=f_tim.readlines()
    tire_main_data = [] 
    for line in FL_tim:
        dummy =  []
        #FL_tim1 = line.rstrip()
        #FL_tim2 = FL_tim1.split(',')
        FL_tim2 = line.split(',')
        FL1 = FL_tim2[0].strip(' ')
        FL2 = FL_tim2[1].strip(' ')
        dummy.append(float(FL1))
        dummy.append(float(FL2))
        tire_main_data.append(dummy)
    ####    NOSE SHOCK ABSORBER SECTION DEFINITION
    model.ConnectorSection(name='SA-NOSE', assembledType=CYLINDRICAL)
    elastic_nose_0 = connectorBehavior.ConnectorElasticity(components=(1, ), 
    behavior=NONLINEAR, table=spring_nose_data)
    elastic_nose_0.ConnectorOptions()
    damping_nose_1 = connectorBehavior.ConnectorDamping(components=(1, ), 
    behavior=NONLINEAR, coupling=COUPLED_MOTION, independentComponents=(1, ), table=damper_nose_data )
    damping_nose_1.ConnectorOptions(useBehRegSettings=OFF, regularize=OFF)
    stop_nose_2 = connectorBehavior.ConnectorStop(components=(1, ), minMotion=0.0, 
    maxMotion=SA_NLG_Stroke)
    model.sections['SA-NOSE'].setValues(
    behaviorOptions = (elastic_nose_0, damping_nose_1, stop_nose_2, ) )
    ####    NOSE TIRE SECTION DEFINITION
    model.ConnectorSection(name='TIRE-NOSE', assembledType=TRANSLATOR)
    elastic_nosetire_0 = connectorBehavior.ConnectorElasticity(components=(1, ), 
    behavior=NONLINEAR, table=tire_nose_data)
    elastic_nosetire_0.ConnectorOptions()
    damping_nosetire_1 = connectorBehavior.ConnectorDamping(components=(1, ), table=((nosetire_damping_coeff, 
    ), ))
    damping_nosetire_1.ConnectorOptions()
    stop_nosetire_2 = connectorBehavior.ConnectorStop(components=(1, ), minMotion=-Tire_NLG, 
    maxMotion=0.)
    model.sections['TIRE-NOSE'].setValues(
    behaviorOptions = (elastic_nosetire_0, damping_nosetire_1, stop_nosetire_2, ) )
    ####    MAIN SHOCK ABSORBER SECTION DEFINITION
    model.ConnectorSection(name='SA-MAIN', assembledType=CYLINDRICAL)
    elastic_main_0 = connectorBehavior.ConnectorElasticity(components=(1, ), 
    behavior=NONLINEAR, table=spring_main_data)
    elastic_main_0.ConnectorOptions()
    damping_main_1 = connectorBehavior.ConnectorDamping(components=(1, ), 
    behavior=NONLINEAR, coupling=COUPLED_MOTION, independentComponents=(1, ), table=damper_main_data )
    damping_main_1.ConnectorOptions(useBehRegSettings=OFF, regularize=OFF)
    stop_main_2 = connectorBehavior.ConnectorStop(components=(1, ), minMotion=0.0, 
    maxMotion=SA_MLG_Stroke)
    model.sections['SA-MAIN'].setValues(
    behaviorOptions = (elastic_main_0, damping_main_1, stop_main_2, ) )
    ####    MAIN TIRE SECTION DEFINITION
    model.ConnectorSection(name='TIRE-MAIN', assembledType=TRANSLATOR)
    elastic_maintire_0 = connectorBehavior.ConnectorElasticity(components=(1, ), 
    behavior=NONLINEAR, table=tire_main_data)
    elastic_maintire_0.ConnectorOptions()
    damping_maintire_1 = connectorBehavior.ConnectorDamping(components=(1, ), table=((maintire_damping_coeff, 
    ), ))
    damping_maintire_1.ConnectorOptions()
    stop_maintire_2 = connectorBehavior.ConnectorStop(components=(1, ), minMotion=-Tire_MLG, 
    maxMotion=0.)
    model.sections['TIRE-MAIN'].setValues(
    behaviorOptions = (elastic_maintire_0, damping_maintire_1, stop_maintire_2, ))
    ####    STOP SECTION DEFINITION
    model.ConnectorSection(name='STOP', assembledType=TRANSLATOR)
    stop_0 = connectorBehavior.ConnectorStop(components=(1, ), maxMotion=0.)
    model.sections['STOP'].setValues(
    behaviorOptions = (stop_0, ))
    ################################################## NOSE GEAR JOINTS
    ####    NOSE SHOCK ABSORBER DEFINITION

    r1 = a.referencePoints
    csys_nose_cyl=a.DatumCsysByThreePoints(origin=r1[hp1311.id], point1=r1[hp1220.id], point2=r1[hp1001.id], 
    name='NOSE_CYL_CSYS1', coordSysType=CARTESIAN)
    a.WirePolyLine(points=((r1[hp1210.id], r1[hp1310.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp1210.xValue, hp1210.yValue, hp1210.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_CYL')
    region = model.rootAssembly.sets['Wire-NOSE_CYL']
    csa = a.SectionAssignment(sectionName='SA-NOSE', region=region)
    #: The section "SA-NOSE" has been assigned to the selected regions.
    dtmid1 = model.rootAssembly.datums[csys_nose_cyl.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ####    NOSE HINGE JOINT DEFINITION

    r1 = a.referencePoints
    #csys_nose_hingeY=a.DatumCsysByThreePoints(origin=r1[hp1612.id], point1=r1[hp1613.id], point2=r1[hp1001.id], 
    #name='NOSE_HINGE_CSYS', coordSysType=CARTESIAN)
    #csys_nose_hingeV=a.DatumCsysByThreePoints(origin=r1[hp1714.id], point1=r1[hp1712.id], point2=r1[hp1001.id], 
    #name='NOSE_VERTICAL_CSYS', coordSysType=CARTESIAN)
    csys_global_X=a.DatumCsysByThreePoints(name='global X', coordSysType=CARTESIAN, origin=(
        0.0, 0.0, 0.0), point1=(1.0, 0.0, 0.0), point2=(0.0, 1.0, 0.0))
    csys_global_Y=a.DatumCsysByThreePoints(name='global Y', coordSysType=CARTESIAN, origin=(
        0.0, 0.0, 0.0), point1=(0.0, 1.0, 0.0), point2=(0.0, 0.0, 1.0))
    ##
    model.ConnectorSection(name='HINGE1-NOSE', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp1004.id], r1[hp1204.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp1004.xValue, hp1004.yValue, hp1004.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_HINGE1')
    region = model.rootAssembly.sets['Wire-NOSE_HINGE1']
    csa = a.SectionAssignment(sectionName='HINGE1-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='HINGE2-NOSE', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp1207.id], r1[hp1407.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp1207.xValue, hp1207.yValue, hp1207.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_HINGE2')
    region = model.rootAssembly.sets['Wire-NOSE_HINGE2']
    csa = a.SectionAssignment(sectionName='HINGE2-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='HINGE3-NOSE', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp1408.id], r1[hp1508.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp1408.xValue, hp1408.yValue, hp1408.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_HINGE3')
    region = model.rootAssembly.sets['Wire-NOSE_HINGE3']
    csa = a.SectionAssignment(sectionName='HINGE3-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='HINGE4-NOSE', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp1309.id], r1[hp1509.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp1309.xValue, hp1309.yValue, hp1309.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_HINGE4')
    region = model.rootAssembly.sets['Wire-NOSE_HINGE4']
    csa = a.SectionAssignment(sectionName='HINGE4-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='BODY_ATTACHED1-NOSE', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp9901001.id], r1[hp1001.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp9901001.xValue, hp9901001.yValue, hp9901001.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_BODY_ATTACHED1')
    region = model.rootAssembly.sets['Wire-NOSE_BODY_ATTACHED1']
    csa = a.SectionAssignment(sectionName='BODY_ATTACHED1-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='BODY_ATTACHED2-NOSE', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp9901002.id], r1[hp1002.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp9901002.xValue, hp9901002.yValue, hp9901002.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_BODY_ATTACHED2')
    region = model.rootAssembly.sets['Wire-NOSE_BODY_ATTACHED2']
    csa = a.SectionAssignment(sectionName='BODY_ATTACHED2-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='BODY_ATTACHED3-NOSE', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp9901205.id], r1[hp1205.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp9901205.xValue, hp9901205.yValue, hp9901205.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_BODY_ATTACHED3')
    region = model.rootAssembly.sets['Wire-NOSE_BODY_ATTACHED3']
    csa = a.SectionAssignment(sectionName='BODY_ATTACHED3-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='BODY_ATTACHED4-NOSE', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp9901206.id], r1[hp1206.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp9901206.xValue, hp9901206.yValue, hp9901206.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_BODY_ATTACHED4')
    region = model.rootAssembly.sets['Wire-NOSE_BODY_ATTACHED4']
    csa = a.SectionAssignment(sectionName='BODY_ATTACHED4-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='AXLE-NOSE', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp1311.id], r1[hp1600.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp1311.xValue, hp1311.yValue, hp1311.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_AXLE')
    region = model.rootAssembly.sets['Wire-NOSE_AXLE']
    csa = a.SectionAssignment(sectionName='AXLE-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='TIREL-REV-NOSE', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp1612.id], r1[hp1712.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp1612.xValue, hp1612.yValue, hp1612.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_TIREL-REV')
    region = model.rootAssembly.sets['Wire-NOSE_TIREL-REV']
    csa = a.SectionAssignment(sectionName='TIREL-REV-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='TIRER-REV-NOSE', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp1613.id], r1[hp1813.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp1613.xValue, hp1613.yValue, hp1613.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_TIRER-REV')
    region = model.rootAssembly.sets['Wire-NOSE_TIRER-REV']
    csa = a.SectionAssignment(sectionName='TIRER-REV-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    a.WirePolyLine(points=((r1[hp1712.id], r1[hp1714.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp1714.xValue, hp1714.yValue, hp1714.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_TIREL')
    region = model.rootAssembly.sets['Wire-NOSE_TIREL']
    csa = a.SectionAssignment(sectionName='TIRE-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    a.WirePolyLine(points=((r1[hp1813.id], r1[hp1815.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp1815.xValue, hp1815.yValue, hp1815.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_TIRER')
    region = model.rootAssembly.sets['Wire-NOSE_TIRER']
    csa = a.SectionAssignment(sectionName='TIRE-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    a.WirePolyLine(points=((r1[hp1714.id], r1[hp5501714.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp5501714.xValue, hp5501714.yValue, hp5501714.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_TRL')
    region = model.rootAssembly.sets['Wire-NOSE_TRL']
    csa = a.SectionAssignment(sectionName='STOP', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    a.WirePolyLine(points=((r1[hp1815.id], r1[hp5501815.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp5501815.xValue, hp5501815.yValue, hp5501815.zValue), ))
    a.Set(edges=edges1, name='Wire-NOSE_TRR')
    region = model.rootAssembly.sets['Wire-NOSE_TRR']
    csa = a.SectionAssignment(sectionName='STOP', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='PLL-NOSE', translationalType=SLIDE_PLANE, rotationalType=CARDAN)

    a.features[hp5500000.name].setValues(xValue=0.0, yValue=0.0, zValue=100.0)
    a.WirePolyLine(points=((r1[hp5500000.id], r1[hp5501714.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    k_x = e1[0].pointOn[0][0]
    k_y = e1[0].pointOn[0][1]
    k_z = e1[0].pointOn[0][2]
    edges1 = e1.findAt(((k_x, k_y, k_z), ))
    a.Set(edges=edges1, name='Wire-NOSE_PLL')
    region = model.rootAssembly.sets['Wire-NOSE_PLL']
    csa = a.SectionAssignment(sectionName='PLL-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    a.features[hp5500000.name].setValues(xValue=ground_x, yValue=ground_y, zValue=ground_z)
    ##
    model.ConnectorSection(name='PLR-NOSE', translationalType=SLIDE_PLANE, rotationalType=CARDAN)

    a.features[hp5500000.name].setValues(xValue=0.0, yValue=0.0, zValue=100.0)
    a.WirePolyLine(points=((r1[hp5500000.id], r1[hp5501815.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    k_x = e1[0].pointOn[0][0]
    k_y = e1[0].pointOn[0][1]
    k_z = e1[0].pointOn[0][2]
    edges1 = e1.findAt(((k_x, k_y, k_z), ))
    a.Set(edges=edges1, name='Wire-NOSE_PLR')
    region = model.rootAssembly.sets['Wire-NOSE_PLR']
    csa = a.SectionAssignment(sectionName='PLR-NOSE', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    a.features[hp5500000.name].setValues(xValue=ground_x, yValue=ground_y, zValue=ground_z)
    ################################################## 
    ################################################## 
    ################################################## MAIN GEAR JOINTS
    ####    MAIN SHOCK ABSORBER DEFINITION
    ##      LEFT

    r1 = a.referencePoints
    csys_mainL_cyl=a.DatumCsysByThreePoints(origin=r1[hp11311.id], point1=r1[hp11208.id], point2=r1[hp11001.id], 
    name='MAINL_CYL_CSYS1', coordSysType=CARTESIAN)
    a.WirePolyLine(points=((r1[hp11209.id], r1[hp11309.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp11209.xValue, hp11209.yValue, hp11209.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_CYL')
    region = model.rootAssembly.sets['Wire-MAINL_CYL']
    csa = a.SectionAssignment(sectionName='SA-MAIN', region=region)
    dtmid1 = model.rootAssembly.datums[csys_mainL_cyl.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##      RIGHT

    r1 = a.referencePoints
    csys_mainR_cyl=a.DatumCsysByThreePoints(origin=r1[hp21311.id], point1=r1[hp21208.id], point2=r1[hp21001.id], 
    name='MAINR_CYL_CSYS1', coordSysType=CARTESIAN)
    a.WirePolyLine(points=((r1[hp21209.id], r1[hp21309.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp21209.xValue, hp21209.yValue, hp21209.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_CYL')
    region = model.rootAssembly.sets['Wire-MAINR_CYL']
    csa = a.SectionAssignment(sectionName='SA-MAIN', region=region)
    dtmid1 = model.rootAssembly.datums[csys_mainR_cyl.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ####    MAIN HINGE JOINT DEFINITION
    ##      LEFT

    r1 = a.referencePoints
    csys_mainL_hingeTop=a.DatumCsysByThreePoints(origin=r1[hp11406.id], point1=r1[hp11405.id], point2=r1[hp11001.id], 
    name='MAINL_HINGE_CSYS1', coordSysType=CARTESIAN)
    csys_mainL_hingeFB=a.DatumCsysByThreePoints(origin=r1[hp11001.id], point1=r1[hp11101.id], point2=r1[hp11405.id], 
    name='MAINL_HINGE_CSYS2', coordSysType=CARTESIAN)
    csys_mainL_hinge4=a.DatumCsysByThreePoints(origin=r1[hp11403.id], point1=r1[hp11407.id], point2=r1[hp11001.id], 
    name='MAINL_HINGE_CSYS3', coordSysType=CARTESIAN)
    #csys_mainL_hingeY=a.DatumCsysByThreePoints(origin=r1[hp11612.id], point1=r1[hp11613.id], point2=r1[hp11001.id], 
    #name='MAINL_HINGE_CSYS', coordSysType=CARTESIAN)
    #csys_mainL_hingeV=a.DatumCsysByThreePoints(origin=r1[hp11714.id], point1=r1[hp11712.id], point2=r1[hp11001.id], 
    #name='MAINL_VERTICAL_CSYS', coordSysType=CARTESIAN)
    ##
    model.ConnectorSection(name='TOPL1-MAINL', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp9911405.id], r1[hp11405.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp9911405.xValue, hp9911405.yValue, hp9911405.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_HINGE_TOPL1-BODY_ATTACHED1')
    region = model.rootAssembly.sets['Wire-MAINL_HINGE_TOPL1-BODY_ATTACHED1']
    csa = a.SectionAssignment(sectionName='TOPL1-MAINL', region=region)
    dtmid1 = model.rootAssembly.datums[csys_mainL_hingeTop.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='TOPL2-MAINL', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp9911406.id], r1[hp11406.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp9911406.xValue, hp9911406.yValue, hp9911406.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_HINGE_TOPL2-BODY_ATTACHED2')
    region = model.rootAssembly.sets['Wire-MAINL_HINGE_TOPL2-BODY_ATTACHED2']
    csa = a.SectionAssignment(sectionName='TOPL2-MAINL', region=region)
    dtmid1 = model.rootAssembly.datums[csys_mainL_hingeTop.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='HINGE1-MAINL', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp11208.id], r1[hp11408.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp11208.xValue, hp11208.yValue, hp11208.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_HINGE1')
    region = model.rootAssembly.sets['Wire-MAINL_HINGE1']
    csa = a.SectionAssignment(sectionName='HINGE1-MAINL', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='HINGE2-MAINL', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp11407.id], r1[hp11507.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp11407.xValue, hp11407.yValue, hp11407.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_HINGE2')
    region = model.rootAssembly.sets['Wire-MAINL_HINGE2']
    csa = a.SectionAssignment(sectionName='HINGE2-MAINL', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='HINGE3-MAINL', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp11511.id], r1[hp11311.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp11311.xValue, hp11311.yValue, hp11311.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_HINGE3')
    region = model.rootAssembly.sets['Wire-MAINL_HINGE3']
    csa = a.SectionAssignment(sectionName='HINGE3-MAINL', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='AXLE-MAINL', assembledType=HINGE)

    a.features[hp11600.name].setValues(xValue=0.0, yValue=0.0, zValue=0.0)
    a.WirePolyLine(points=((r1[hp11600.id], r1[hp11511.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    k_x = e1[0].pointOn[0][0]
    k_y = e1[0].pointOn[0][1]
    k_z = e1[0].pointOn[0][2]
    edges1 = e1.findAt(((k_x, k_y, k_z), ))
    a.Set(edges=edges1, name='Wire-MAINL_AXLE')
    region = model.rootAssembly.sets['Wire-MAINL_AXLE']
    csa = a.SectionAssignment(sectionName='AXLE-MAINL', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    a.features[hp11600.name].setValues(xValue=RP_MLG_L11_x, yValue=RP_MLG_L11_y, zValue=RP_MLG_L11_z)
    ##
    model.ConnectorSection(name='FBL-MAINL', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp9911001.id], r1[hp11001.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp9911001.xValue, hp9911001.yValue, hp9911001.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_FBL-BODY_ATTACHED3')
    region = model.rootAssembly.sets['Wire-MAINL_FBL-BODY_ATTACHED3']
    csa = a.SectionAssignment(sectionName='FBL-MAINL', region=region)
    dtmid1 = model.rootAssembly.datums[csys_mainL_hingeFB.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='MLH4-MAINL', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp11004.id], r1[hp11404.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp11004.xValue, hp11004.yValue, hp11004.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_MLH4')
    region = model.rootAssembly.sets['Wire-MAINL_MLH4']
    csa = a.SectionAssignment(sectionName='MLH4-MAINL', region=region)
    dtmid1 = model.rootAssembly.datums[csys_mainL_hinge4.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='TIREL-REV-MAINL', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp11612.id], r1[hp11712.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp11612.xValue, hp11612.yValue, hp11612.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_TIREL-REV')
    region = model.rootAssembly.sets['Wire-MAINL_TIREL-REV']
    csa = a.SectionAssignment(sectionName='TIREL-REV-MAINL', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='TIRER-REV-MAINL', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp11613.id], r1[hp11813.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp11613.xValue, hp11613.yValue, hp11613.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_TIRER-REV')
    region = model.rootAssembly.sets['Wire-MAINL_TIRER-REV']
    csa = a.SectionAssignment(sectionName='TIRER-REV-MAINL', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    a.WirePolyLine(points=((r1[hp11712.id], r1[hp11714.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp11714.xValue, hp11714.yValue, hp11714.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_TIREL')
    region = model.rootAssembly.sets['Wire-MAINL_TIREL']
    csa = a.SectionAssignment(sectionName='TIRE-MAIN', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    a.WirePolyLine(points=((r1[hp11813.id], r1[hp11815.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp11815.xValue, hp11815.yValue, hp11815.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_TIRER')
    region = model.rootAssembly.sets['Wire-MAINL_TIRER']
    csa = a.SectionAssignment(sectionName='TIRE-MAIN', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    a.WirePolyLine(points=((r1[hp11714.id], r1[hp5511714.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp5511714.xValue, hp5511714.yValue, hp5511714.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_TRL')
    region = model.rootAssembly.sets['Wire-MAINL_TRL']
    csa = a.SectionAssignment(sectionName='STOP', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    a.WirePolyLine(points=((r1[hp11815.id], r1[hp5511815.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp5511815.xValue, hp5511815.yValue, hp5511815.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINL_TRR')
    region = model.rootAssembly.sets['Wire-MAINL_TRR']
    csa = a.SectionAssignment(sectionName='STOP', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='PLL-MAINL', translationalType=SLIDE_PLANE, rotationalType=CARDAN)

    a.features[hp5500000.name].setValues(xValue=0.0, yValue=0.0, zValue=100.0)
    a.WirePolyLine(points=((r1[hp5500000.id], r1[hp5511714.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    k_x = e1[0].pointOn[0][0]
    k_y = e1[0].pointOn[0][1]
    k_z = e1[0].pointOn[0][2]
    edges1 = e1.findAt(((k_x, k_y, k_z), ))
    a.Set(edges=edges1, name='Wire-MAINL_PLL')
    region = model.rootAssembly.sets['Wire-MAINL_PLL']
    csa = a.SectionAssignment(sectionName='PLL-MAINL', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    a.features[hp5500000.name].setValues(xValue=ground_x, yValue=ground_y, zValue=ground_z)
    ##
    model.ConnectorSection(name='PLR-MAINL', translationalType=SLIDE_PLANE, rotationalType=CARDAN)

    a.features[hp5500000.name].setValues(xValue=0.0, yValue=0.0, zValue=100.0)
    a.WirePolyLine(points=((r1[hp5500000.id], r1[hp5511815.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    k_x = e1[0].pointOn[0][0]
    k_y = e1[0].pointOn[0][1]
    k_z = e1[0].pointOn[0][2]
    edges1 = e1.findAt(((k_x, k_y, k_z), ))
    a.Set(edges=edges1, name='Wire-MAINL_PLR')
    region = model.rootAssembly.sets['Wire-MAINL_PLR']
    csa = a.SectionAssignment(sectionName='PLR-MAINL', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    a.features[hp5500000.name].setValues(xValue=ground_x, yValue=ground_y, zValue=ground_z)
    ##      RIGHT

    r1 = a.referencePoints
    csys_mainR_hingeTop=a.DatumCsysByThreePoints(origin=r1[hp21406.id], point1=r1[hp21405.id], point2=r1[hp21001.id], 
    name='MAINR_HINGE_CSYS1', coordSysType=CARTESIAN)
    csys_mainR_hingeFB=a.DatumCsysByThreePoints(origin=r1[hp21001.id], point1=r1[hp21101.id], point2=r1[hp21405.id], 
    name='MAINR_HINGE_CSYS2', coordSysType=CARTESIAN)
    csys_mainR_hinge4=a.DatumCsysByThreePoints(origin=r1[hp21403.id], point1=r1[hp21407.id], point2=r1[hp21001.id], 
    name='MAINR_HINGE_CSYS3', coordSysType=CARTESIAN)
    #csys_mainR_hingeY=a.DatumCsysByThreePoints(origin=r1[hp21612.id], point1=r1[hp21613.id], point2=r1[hp21001.id], 
    #name='MAINR_HINGE_CSYS', coordSysType=CARTESIAN)
    #csys_mainR_hingeV=a.DatumCsysByThreePoints(origin=r1[hp21714.id], point1=r1[hp21712.id], point2=r1[hp21001.id], 
    #name='MAINR_VERTICAL_CSYS', coordSysType=CARTESIAN)
    ##
    model.ConnectorSection(name='TOPR1-MAINR', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp9921405.id], r1[hp21405.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp9921405.xValue, hp9921405.yValue, hp9921405.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_HINGE_TOPR1-BODY_ATTACHED1')
    region = model.rootAssembly.sets['Wire-MAINR_HINGE_TOPR1-BODY_ATTACHED1']
    csa = a.SectionAssignment(sectionName='TOPR1-MAINR', region=region)
    dtmid1 = model.rootAssembly.datums[csys_mainR_hingeTop.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='TOPR2-MAINR', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp9921406.id], r1[hp21406.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp9921406.xValue, hp9921406.yValue, hp9921406.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_HINGE_TOPR2-BODY_ATTACHED2')
    region = model.rootAssembly.sets['Wire-MAINR_HINGE_TOPR2-BODY_ATTACHED2']
    csa = a.SectionAssignment(sectionName='TOPR2-MAINR', region=region)
    dtmid1 = model.rootAssembly.datums[csys_mainR_hingeTop.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='HINGE1-MAINR', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp21208.id], r1[hp21408.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp21208.xValue, hp21208.yValue, hp21208.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_HINGE1')
    region = model.rootAssembly.sets['Wire-MAINR_HINGE1']
    csa = a.SectionAssignment(sectionName='HINGE1-MAINR', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='HINGE2-MAINR', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp21407.id], r1[hp21507.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp21407.xValue, hp21407.yValue, hp21407.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_HINGE2')
    region = model.rootAssembly.sets['Wire-MAINR_HINGE2']
    csa = a.SectionAssignment(sectionName='HINGE2-MAINR', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='HINGE3-MAINR', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp21511.id], r1[hp21311.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp21511.xValue, hp21511.yValue, hp21511.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_HINGE3')
    region = model.rootAssembly.sets['Wire-MAINR_HINGE3']
    csa = a.SectionAssignment(sectionName='HINGE3-MAINR', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='AXLE-MAINR', assembledType=HINGE)
    a.features[hp21600.name].setValues(xValue=0.0, yValue=0.0, zValue=0.0)
    a.WirePolyLine(points=((r1[hp21600.id], r1[hp21511.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    k_x = e1[0].pointOn[0][0]
    k_y = e1[0].pointOn[0][1]
    k_z = e1[0].pointOn[0][2]
    edges1 = e1.findAt(((k_x, k_y, k_z), ))
    a.Set(edges=edges1, name='Wire-MAINR_AXLE')
    region = model.rootAssembly.sets['Wire-MAINR_AXLE']
    csa = a.SectionAssignment(sectionName='AXLE-MAINR', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    a.features[hp21600.name].setValues(xValue=RP_MLG_R11_x, yValue=RP_MLG_L11_y, zValue=RP_MLG_L11_z)
    ##
    model.ConnectorSection(name='FBR-MAINR', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp9921001.id], r1[hp21001.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp9921001.xValue, hp9921001.yValue, hp9921001.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_FBR-BODY_ATTACHED3')
    region = model.rootAssembly.sets['Wire-MAINR_FBR-BODY_ATTACHED3']
    csa = a.SectionAssignment(sectionName='FBR-MAINR', region=region)
    dtmid1 = model.rootAssembly.datums[csys_mainR_hingeFB.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='MRH4-MAINR', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp21004.id], r1[hp21404.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp21004.xValue, hp21004.yValue, hp21004.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_MRH4')
    region = model.rootAssembly.sets['Wire-MAINR_MRH4']
    csa = a.SectionAssignment(sectionName='MRH4-MAINR', region=region)
    dtmid1 = model.rootAssembly.datums[csys_mainR_hinge4.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='TIREL-REV-MAINR', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp21612.id], r1[hp21712.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp21612.xValue, hp21612.yValue, hp21612.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_TIREL-REV')
    region = model.rootAssembly.sets['Wire-MAINR_TIREL-REV']
    csa = a.SectionAssignment(sectionName='TIREL-REV-MAINR', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='TIRER-REV-MAINR', assembledType=HINGE)
    a.WirePolyLine(points=((r1[hp21613.id], r1[hp21813.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp21613.xValue, hp21613.yValue, hp21613.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_TIRER-REV')
    region = model.rootAssembly.sets['Wire-MAINR_TIRER-REV']
    csa = a.SectionAssignment(sectionName='TIRER-REV-MAINR', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_X.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    a.WirePolyLine(points=((r1[hp21712.id], r1[hp21714.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp21714.xValue, hp21714.yValue, hp21714.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_TIREL')
    region = model.rootAssembly.sets['Wire-MAINR_TIREL']
    csa = a.SectionAssignment(sectionName='TIRE-MAIN', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    a.WirePolyLine(points=((r1[hp21813.id], r1[hp21815.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp21815.xValue, hp21815.yValue, hp21815.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_TIRER')
    region = model.rootAssembly.sets['Wire-MAINR_TIRER']
    csa = a.SectionAssignment(sectionName='TIRE-MAIN', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    a.WirePolyLine(points=((r1[hp21714.id], r1[hp5521714.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp5521714.xValue, hp5521714.yValue, hp5521714.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_TRL')
    region = model.rootAssembly.sets['Wire-MAINR_TRL']
    csa = a.SectionAssignment(sectionName='STOP', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    a.WirePolyLine(points=((r1[hp21815.id], r1[hp5521815.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    edges1 = e1.findAt(((hp5521815.xValue, hp5521815.yValue, hp5521815.zValue), ))
    a.Set(edges=edges1, name='Wire-MAINR_TRR')
    region = model.rootAssembly.sets['Wire-MAINR_TRR']
    csa = a.SectionAssignment(sectionName='STOP', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    ##
    model.ConnectorSection(name='PLL-MAINR', translationalType=SLIDE_PLANE, rotationalType=CARDAN)

    a.features[hp5500000.name].setValues(xValue=0.0, yValue=0.0, zValue=100.0)
    a.WirePolyLine(points=((r1[hp5500000.id], r1[hp5521714.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    k_x = e1[0].pointOn[0][0]
    k_y = e1[0].pointOn[0][1]
    k_z = e1[0].pointOn[0][2]
    edges1 = e1.findAt(((k_x, k_y, k_z), ))
    a.Set(edges=edges1, name='Wire-MAINR_PLL')
    region = model.rootAssembly.sets['Wire-MAINR_PLL']
    csa = a.SectionAssignment(sectionName='PLL-MAINR', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    a.features[hp5500000.name].setValues(xValue=ground_x, yValue=ground_y, zValue=ground_z)
    ##
    model.ConnectorSection(name='PLR-MAINR', translationalType=SLIDE_PLANE, rotationalType=CARDAN)

    a.features[hp5500000.name].setValues(xValue=0.0, yValue=0.0, zValue=100.0)
    a.WirePolyLine(points=((r1[hp5500000.id], r1[hp5521815.id]), ), mergeType=SEPARATE, meshable=False)
    e1 = a.edges
    k_x = e1[0].pointOn[0][0]
    k_y = e1[0].pointOn[0][1]
    k_z = e1[0].pointOn[0][2]
    edges1 = e1.findAt(((k_x, k_y, k_z), ))
    a.Set(edges=edges1, name='Wire-MAINR_PLR')
    region = model.rootAssembly.sets['Wire-MAINR_PLR']
    csa = a.SectionAssignment(sectionName='PLR-MAINR', region=region)
    dtmid1 = model.rootAssembly.datums[csys_global_Y.id]
    a.ConnectorOrientation(region=csa.getSet(), localCsys1=dtmid1)
    a.features[hp5500000.name].setValues(xValue=ground_x, yValue=ground_y, zValue=ground_z)
    #######################################################################################################################
    #############################   Initial Velocity

    region = a.sets['nall']
    model.Velocity(name='Initial Velocity', 
    region=region, field='', distributionType=MAGNITUDE, velocity1=0.0, 
    velocity2=initial_vel_y, velocity3=initial_vel_z, omega=0.0)
    #############################   Ground Fix
    region = a.sets['nground']
    model.EncastreBC(name='Ground_Fix', 
    createStepName='Initial', region=region)
    #############################   Lift Body Force Amplitude Definition
    model.TabularAmplitude(name='Lift_removal_ampl', 
    timeSpan=STEP, smooth=SOLVER_DEFAULT, data=((0.0, 1.0), (start_time, 1.0), (
    Lift_delay_time, 0.0), (end_time, 0.0)))
    #######################################################################################################################
    #############################   Step Definition
    model.ExplicitDynamicsStep(name='Landing Simulation', 
    previous='Initial', description='Landing Simulation', timePeriod=end_time,  
    timeIncrementationMethod=FIXED_USER_DEFINED_INC, userDefinedInc=time_step)
    vp.assemblyDisplay.setValues(step='Landing Simulation')
    model.steps['Landing Simulation'].setValues(
        linearBulkViscosity=0.0, quadBulkViscosity=0.0)
    #############################   Planar Conector Rotational DOF Constraint 
    region = model.rootAssembly.sets['Wire-NOSE_PLL']
    model.ConnDisplacementBC(name='Connector ROT DOF Constraint1', 
    createStepName='Landing Simulation', region=region, u1=UNSET, u2=UNSET, 
    u3=UNSET, ur1=0.0, ur2=0.0, ur3=0.0, amplitude=UNSET, fixed=OFF, 
    distributionType=UNIFORM)
    region = model.rootAssembly.sets['Wire-NOSE_PLR']
    model.ConnDisplacementBC(name='Connector ROT DOF Constraint2', 
    createStepName='Landing Simulation', region=region, u1=UNSET, u2=UNSET, 
    u3=UNSET, ur1=0.0, ur2=0.0, ur3=0.0, amplitude=UNSET, fixed=OFF, 
    distributionType=UNIFORM)
    region = model.rootAssembly.sets['Wire-MAINL_PLL']
    model.ConnDisplacementBC(name='Connector ROT DOF Constraint3', 
    createStepName='Landing Simulation', region=region, u1=UNSET, u2=UNSET, 
    u3=UNSET, ur1=0.0, ur2=0.0, ur3=0.0, amplitude=UNSET, fixed=OFF, 
    distributionType=UNIFORM)
    region = model.rootAssembly.sets['Wire-MAINL_PLR']
    model.ConnDisplacementBC(name='Connector ROT DOF Constraint4', 
    createStepName='Landing Simulation', region=region, u1=UNSET, u2=UNSET, 
    u3=UNSET, ur1=0.0, ur2=0.0, ur3=0.0, amplitude=UNSET, fixed=OFF, 
    distributionType=UNIFORM)
    region = model.rootAssembly.sets['Wire-MAINR_PLL']
    model.ConnDisplacementBC(name='Connector ROT DOF Constraint5', 
    createStepName='Landing Simulation', region=region, u1=UNSET, u2=UNSET, 
    u3=UNSET, ur1=0.0, ur2=0.0, ur3=0.0, amplitude=UNSET, fixed=OFF, 
    distributionType=UNIFORM)
    region = model.rootAssembly.sets['Wire-MAINR_PLR']
    model.ConnDisplacementBC(name='Connector ROT DOF Constraint6', 
    createStepName='Landing Simulation', region=region, u1=UNSET, u2=UNSET, 
    u3=UNSET, ur1=0.0, ur2=0.0, ur3=0.0, amplitude=UNSET, fixed=OFF, 
    distributionType=UNIFORM)


    #############################   Gravity Definition  
    model.Gravity(name='Gravity', 
    createStepName='Landing Simulation', comp2=-gravity_acc, 
    distributionType=UNIFORM, field='')
    #############################   Lift Body Force Definition  
    model.Gravity(name='Lift', 
    createStepName='Landing Simulation', comp2=gravity_acc, 
    amplitude='Lift_removal_ampl', distributionType=UNIFORM, field='')
    #############################   Field Output Definition
    model.FieldOutputRequest(name='F-Output-1', 
            createStepName='Landing Simulation', variables=PRESELECT, 
            numIntervals=field_output_numbers)
    #############################   History Output Definition
    model.HistoryOutputRequest(name='H-Output-1', 
            createStepName='Landing Simulation', variables=PRESELECT)
    regionDef=model.rootAssembly.sets['nCG']
    model.HistoryOutputRequest(name='History_Output_1111', 
            createStepName='Landing Simulation', variables=('U', 'V', 'A'),
        frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-NOSE_BODY_ATTACHED1']
    model.HistoryOutputRequest(name='History_Output_11', 
            createStepName='Landing Simulation', variables=('U', 'CTF', 'CRF', 'CCF'),
        frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-NOSE_BODY_ATTACHED2']
    model.HistoryOutputRequest(name='History_Output_12', 
            createStepName='Landing Simulation', variables=('U', 'CTF', 'CRF', 'CCF'),
        frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-NOSE_BODY_ATTACHED3']
    model.HistoryOutputRequest(name='History_Output_13', 
            createStepName='Landing Simulation', variables=('U', 'CTF', 'CRF', 'CCF'),
        frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-NOSE_BODY_ATTACHED4']
    model.HistoryOutputRequest(name='History_Output_14', 
            createStepName='Landing Simulation', variables=('U', 'CTF', 'CRF', 'CCF'),
        frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINL_HINGE_TOPL1-BODY_ATTACHED1']
    model.HistoryOutputRequest(name='History_Output_15', 
            createStepName='Landing Simulation', variables=('U', 'CTF', 'CRF', 'CCF'),
        frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINL_HINGE_TOPL2-BODY_ATTACHED2']
    model.HistoryOutputRequest(name='History_Output_16', 
            createStepName='Landing Simulation', variables=('U', 'CTF', 'CRF', 'CCF'),
        frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINL_FBL-BODY_ATTACHED3']
    model.HistoryOutputRequest(name='History_Output_17', 
            createStepName='Landing Simulation', variables=('U', 'CTF', 'CRF', 'CCF'),
        frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINR_HINGE_TOPR1-BODY_ATTACHED1']
    model.HistoryOutputRequest(name='History_Output_18', 
            createStepName='Landing Simulation', variables=('U', 'CTF', 'CRF', 'CCF'),
        frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINR_HINGE_TOPR2-BODY_ATTACHED2']
    model.HistoryOutputRequest(name='History_Output_19', 
            createStepName='Landing Simulation', variables=('U', 'CTF', 'CRF', 'CCF'),
        frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINR_FBR-BODY_ATTACHED3']
    model.HistoryOutputRequest(name='History_Output_20', 
            createStepName='Landing Simulation', variables=('U', 'CTF', 'CRF', 'CCF'),
        frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-NOSE_CYL']
    model.HistoryOutputRequest(name='History_Output_30', 
            createStepName='Landing Simulation', variables=('U', 'CU1', 'CV1', 'CA1', 'CTF1',
        'CEF1', 'CVF1', 'CRF1', 'CCF1', 'CTF'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINL_CYL']
    model.HistoryOutputRequest(name='History_Output_31', 
            createStepName='Landing Simulation', variables=('U', 'CU1', 'CV1', 'CA1', 'CTF1',
        'CEF1', 'CVF1', 'CRF1', 'CCF1', 'CTF'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINR_CYL']
    model.HistoryOutputRequest(name='History_Output_32', 
            createStepName='Landing Simulation', variables=('U', 'CU1', 'CV1', 'CA1', 'CTF1',
        'CEF1', 'CVF1', 'CRF1', 'CCF1', 'CTF'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-NOSE_TIREL']
    model.HistoryOutputRequest(name='History_Output_33', 
            createStepName='Landing Simulation', variables=('CU1', 'CV1', 'CA1', 'CTF1',
        'CEF1', 'CVF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-NOSE_TIRER']
    model.HistoryOutputRequest(name='History_Output_34', 
            createStepName='Landing Simulation', variables=('CU1', 'CV1', 'CA1', 'CTF1',
        'CEF1', 'CVF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINL_TIREL']
    model.HistoryOutputRequest(name='History_Output_35', 
            createStepName='Landing Simulation', variables=('CU1', 'CV1', 'CA1', 'CTF1',
        'CEF1', 'CVF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINL_TIRER']
    model.HistoryOutputRequest(name='History_Output_36', 
            createStepName='Landing Simulation', variables=('CU1', 'CV1', 'CA1', 'CTF1',
        'CEF1', 'CVF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINR_TIREL']
    model.HistoryOutputRequest(name='History_Output_37', 
            createStepName='Landing Simulation', variables=('CU1', 'CV1', 'CA1', 'CTF1',
        'CEF1', 'CVF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINR_TIRER']
    model.HistoryOutputRequest(name='History_Output_38', 
            createStepName='Landing Simulation', variables=('CU1', 'CV1', 'CA1', 'CTF1',
        'CEF1', 'CVF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-NOSE_TRL']
    model.HistoryOutputRequest(name='History_Output_40', 
            createStepName='Landing Simulation', variables=('CTF1', 'CCF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-NOSE_TRR']
    model.HistoryOutputRequest(name='History_Output_41', 
            createStepName='Landing Simulation', variables=('CTF1', 'CCF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-NOSE_PLL']
    model.HistoryOutputRequest(name='History_Output_42', 
            createStepName='Landing Simulation', variables=('CTF1', 'CEF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-NOSE_PLR']
    model.HistoryOutputRequest(name='History_Output_43', 
            createStepName='Landing Simulation', variables=('CTF1', 'CEF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINL_TRL']
    model.HistoryOutputRequest(name='History_Output_44', 
            createStepName='Landing Simulation', variables=('CTF1', 'CCF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINL_TRR']
    model.HistoryOutputRequest(name='History_Output_45', 
            createStepName='Landing Simulation', variables=('CTF1', 'CCF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINL_PLL']
    model.HistoryOutputRequest(name='History_Output_46', 
            createStepName='Landing Simulation', variables=('CTF1', 'CEF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINL_PLR']
    model.HistoryOutputRequest(name='History_Output_47', 
            createStepName='Landing Simulation', variables=('CTF1', 'CEF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINR_TRL']
    model.HistoryOutputRequest(name='History_Output_48', 
            createStepName='Landing Simulation', variables=('CTF1', 'CCF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINR_TRR']
    model.HistoryOutputRequest(name='History_Output_49', 
            createStepName='Landing Simulation', variables=('CTF1', 'CCF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINR_PLL']
    model.HistoryOutputRequest(name='History_Output_50', 
            createStepName='Landing Simulation', variables=('CTF1', 'CEF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
    regionDef=model.rootAssembly.sets['Wire-MAINR_PLR']
    model.HistoryOutputRequest(name='History_Output_51', 
            createStepName='Landing Simulation', variables=('CTF1', 'CEF1'),
            frequency=1, region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)

    a = model.rootAssembly
    vp.setValues(displayedObject=a)